
//
// @(#)btyaccpar, based on byacc 1.8 (Berkeley)
// Parser skeleton modified for use in the Firebird project by Nickolay Samofatov
//
#define YYBTYACC 1

#include "firebird.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../dsql/Nodes.h"
#include "../dsql/AggNodes.h"
#include "../dsql/DdlNodes.h"
#include "../dsql/BoolNodes.h"
#include "../dsql/ExprNodes.h"
#include "../dsql/PackageNodes.h"
#include "../dsql/StmtNodes.h"
#include "../dsql/WinNodes.h"
#include "../jrd/RecordSourceNodes.h"
#include "../common/classes/Nullable.h"
#include "gen/parse.h"
#include "../dsql/Parser.h"

/*
*	PROGRAM:	Dynamic SQL runtime support
*	MODULE:		parse.y
*	DESCRIPTION:	Dynamic SQL parser
*
* The contents of this file are subject to the Interbase Public
* License Version 1.0 (the "License"); you may not use this file
* except in compliance with the License. You may obtain a copy
* of the License at http://www.Inprise.com/IPL.html
*
* Software distributed under the License is distributed on an
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
* or implied. See the License for the specific language governing
* rights and limitations under the License.
*
* The Original Code was created by Inprise Corporation
* and its predecessors. Portions created by Inprise Corporation are
* Copyright (C) Inprise Corporation.
*
* All Rights Reserved.
* Contributor(s): ______________________________________.
*
* 2002-02-24 Sean Leyne - Code Cleanup of old Win 3.1 port (WINDOWS_ONLY)
* 2001.05.20 Neil McCalden: Allow a udf to be used in a 'group by' clause.
* 2001.05.30 Claudio Valderrama: DROP TABLE and DROP VIEW lead now to two
*   different node types so DDL can tell which is which.
* 2001.06.13 Claudio Valderrama: SUBSTRING is being surfaced.
* 2001.06.30 Claudio valderrama: Feed (line,column) for each node. See node.h.
* 2001.07.10 Claudio Valderrama: Better (line,column) report and "--" for comments.
* 2001.07.28 John Bellardo: Changes to support parsing LIMIT and FIRST
* 2001.08.03 John Bellardo: Finalized syntax for LIMIT, change LIMIT to SKIP
* 2001.08.05 Claudio Valderrama: closed Bug #448062 and other spaces that appear
*   in rdb$*_source fields when altering domains plus one unexpected null pointer.
* 2001.08.12 Claudio Valderrama: adjust SUBSTRING's starting pos argument here
*   and not in gen.c; this closes Bug #450301.
* 2001.10.01 Claudio Valderrama: enable explicit GRANT...to ROLE role_name.
* 2001.10.06 Claudio Valderrama: Honor explicit USER keyword in GRANTs and REVOKEs.
* 2002.07.05 Mark O'Donohue: change keyword DEBUG to KW_DEBUG to avoid
*			clashes with normal DEBUG macro.
* 2002.07.30 Arno Brinkman:
* 2002.07.30 	Let IN predicate handle value_expressions
* 2002.07.30 	tokens CASE, NULLIF, COALESCE added
* 2002.07.30 	See block < CASE expression > what is added to value as case_expression
* 2002.07.30 	function is split up into aggregate_function, numeric_value_function, string_value_function, generate_value_function
* 2002.07.30 	new group_by_function and added to grp_column_elem
* 2002.07.30 	cast removed from function and added as cast_specification to value
* 2002.08.04 Claudio Valderrama: allow declaring and defining variables at the same time
* 2002.08.04 Dmitry Yemanov: ALTER VIEW
* 2002.08.06 Arno Brinkman: ordinal added to grp_column_elem for using positions in group by
* 2002.08.07 Dmitry Yemanov: INT64/LARGEINT are replaced with BIGINT and available in dialect 3 only
* 2002.08.31 Dmitry Yemanov: allowed user-defined index names for PK/FK/UK constraints
* 2002.09.01 Dmitry Yemanov: RECREATE VIEW
* 2002.09.28 Dmitry Yemanov: Reworked internal_info stuff, enhanced
*							exception handling in SPs/triggers,
*							implemented ROWS_AFFECTED system variable
* 2002.10.21 Nickolay Samofatov: Added support for explicit pessimistic locks
* 2002.10.29 Nickolay Samofatov: Added support for savepoints
* 2002.12.03 Dmitry Yemanov: Implemented ORDER BY clause in subqueries.
* 2002.12.18 Dmitry Yemanov: Added support for SQL-compliant labels and LEAVE statement
* 2002.12.28 Dmitry Yemanov: Added support for parametrized events.
* 2003.01.14 Dmitry Yemanov: Fixed bug with cursors in triggers.
* 2003.01.15 Dmitry Yemanov: Added support for runtime trigger action checks.
* 2003.02.10 Mike Nordell  : Undefined Microsoft introduced macros to get a clean compile.
* 2003.05.24 Nickolay Samofatov: Make SKIP and FIRST non-reserved keywords
* 2003.06.13 Nickolay Samofatov: Make INSERTING/UPDATING/DELETING non-reserved keywords
* 2003.07.01 Blas Rodriguez Somoza: Change DEBUG and IN to avoid conflicts in win32 build/bison
* 2003.08.11 Arno Brinkman: Changed GROUP BY to support all expressions and added "AS" support
*						   with table alias. Also removed group_by_function and ordinal.
* 2003.08.14 Arno Brinkman: Added support for derived tables.
* 2003.10.05 Dmitry Yemanov: Added support for explicit cursors in PSQL.
* 2004.01.16 Vlad Horsun: added support for default parameters and
*   EXECUTE BLOCK statement
* Adriano dos Santos Fernandes
*/

#include "firebird.h"
#include "dyn_consts.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "iberror.h"
#include "../dsql/dsql.h"
#include "ibase.h"
#include "../jrd/flags.h"
#include "../jrd/jrd.h"
#include "../jrd/DataTypeUtil.h"
#include "../dsql/errd_proto.h"
#include "../dsql/make_proto.h"
#include "../yvalve/gds_proto.h"
#include "../jrd/err_proto.h"
#include "../common/intlobj_new.h"
#include "../jrd/Attachment.h"
#include "../common/StatusArg.h"

/* This is needed here to provide backward compatibility when working with SSPI plugin*/
#include "../auth/trusted/AuthSspi.h"

/* since UNIX isn't standard, we have to define*/
/* stuff which is in <limits.h> (which isn't available on all UNIXes...*/

const long SHRT_POS_MAX			= 32767;
const long SHRT_UNSIGNED_MAX	= 65535;
const long SHRT_NEG_MAX			= 32768;
const int POSITIVE	= 0;
const int NEGATIVE	= 1;
const int UNSIGNED	= 2;

/*const int MIN_CACHE_BUFFERS	= 250;*/
/*const int DEF_CACHE_BUFFERS	= 1000;*/

#define YYSTYPE YYSTYPE
#if defined(DEBUG) || defined(DEV_BUILD)
#define YYDEBUG		1
#endif

#define YYREDUCEPOSNFUNC yyReducePosn
#define YYREDUCEPOSNFUNCARG NULL


/* ASF: Inherited attributes (aka rule parameters) are executed even when in trial mode, but action*/
/* rules ({}) are executed only when in full parse mode. NOTRIAL should be used to avoid segfaults*/
/* due to accessing invalid pointers in parameters (not yet returned from action rules).*/
#define NOTRIAL(x) (yytrial ? NULL : (x))


inline unsigned trigger_type_suffix(const unsigned slot1, const unsigned slot2, const unsigned slot3)
{
	return ((slot1 << 1) | (slot2 << 3) | (slot3 << 5));
}


#include "../dsql/chars.h"

using namespace Jrd;
using namespace Firebird;

static int yylhs[] = {                                        -1,
    0,    0,    1,    1,    1,    1,    2,    2,    2,    2,
    2,    2,    2,    2,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    4,    4,    4,    4,    5,    5,
    5,    5,    5,    5,    5,    5,    5,  659,  660,    6,
  661,  663,    7,    7,    7,    7,    7,    7,    7,  664,
    7,    7,  665,  666,    7,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,  662,  662,
    9,    9,  667,    9,   10,  668,   10,   11,   12,   13,
   13,   13,   13,   13,   14,   14,  669,  669,   15,   15,
   16,   16,   16,   17,   17,   18,   18,   19,   19,   19,
   20,   20,   21,   21,   22,   22,  670,  670,   23,   23,
  671,   24,  672,  673,   25,   25,   25,   25,   25,   25,
   25,  674,   25,   25,  675,   25,   25,   26,   26,   27,
   27,  676,   28,   28,  677,   28,   28,   29,   29,   29,
   29,   29,   29,   29,   30,   30,   30,   31,  678,   31,
   32,   32,   33,   33,   34,   34,   34,   35,   36,   36,
  679,  680,  681,   37,   38,   38,   38,   39,  682,   39,
  683,   39,   40,  684,   40,   41,   42,   42,   42,   42,
  685,   43,  686,   43,   44,   44,   45,   45,   45,   45,
   45,   46,   47,   47,   48,   49,  687,  688,   49,   49,
   49,   49,   49,   49,   49,   49,   49,   49,   49,   49,
   49,   49,   49,   49,   49,  689,   49,  690,   49,   50,
   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
   51,   51,   52,   53,   53,   53,   53,   53,   53,   53,
   53,   53,   53,   53,   54,   55,   56,   57,   57,  691,
   58,   59,   59,   59,   60,   60,  693,  694,   61,   62,
   62,   62,   63,   63,   64,   64,   65,  697,   65,   66,
   66,  699,  700,   67,   68,   68,   69,  701,   69,   70,
   70,  698,  698,   71,   72,   72,  702,   73,  703,  704,
   74,   75,  705,   75,   76,   76,   77,   78,  706,  706,
  707,   79,   80,   80,   81,   81,   81,  708,   82,   83,
   83,   84,   84,   85,   86,   86,   87,   88,   88,   88,
   88,  709,  710,   89,   90,   90,   90,  711,   91,   92,
   93,  712,   93,   94,  713,  714,  715,  716,   95,   96,
   96,   96,   97,   97,   98,  717,   98,   99,   99,   99,
  100,  100,  101,  101,  102,  102,  103,  103,  104,  105,
  105,  105,  105,  105,  106,  106,  106,  718,  719,  720,
  107,  695,  695,  108,  109,  109,  110,  721,  110,  111,
  111,  111,  111,  111,  111,  111,  111,  112,  112,  113,
  113,  114,  114,  114,  114,  722,  723,  115,  116,  116,
  117,  724,  117,  118,  118,  725,  725,  725,  696,  696,
  696,  726,  727,  728,  119,  120,  729,  730,  120,  121,
  121,  122,  122,  123,  123,  124,  124,  731,  732,  125,
  733,  126,  734,  126,  127,  127,  127,  127,  128,  128,
  128,  129,  129,  735,  130,  130,  736,  737,  131,  738,
  131,  131,  131,  739,  740,  132,  133,  133,  134,  741,
  134,  135,  135,  136,  136,  137,  742,  742,  743,  692,
  692,  138,  138,  139,  139,  140,  140,  140,  140,  141,
  141,  141,  141,  141,  141,  142,  744,  142,  143,  745,
  143,  144,  145,  145,  145,  145,  145,  746,  146,  147,
  147,  148,  148,  148,  148,  149,  149,  150,  150,  150,
  150,  150,  151,  152,  153,  153,  153,  153,  154,  154,
  155,  156,  747,  748,  749,  157,  158,  159,  160,  160,
  160,  161,  161,  162,  162,  163,  164,  164,  165,  166,
  167,  167,  167,  168,  168,  169,  170,  750,  171,  172,
  172,  172,  173,  173,  174,  174,  175,  176,  177,  178,
  178,  179,  179,  180,  180,  181,  182,  183,  183,  184,
  184,  185,  185,  186,  186,  186,  186,  187,  188,  188,
  189,  189,  190,  190,  191,  191,  192,  192,  193,  193,
  193,  752,  753,  754,  194,  755,  756,  195,  196,  196,
  197,  751,  751,  198,  199,  199,  199,  200,  200,  201,
  202,  202,  202,  203,  203,  204,  204,  205,  205,  205,
  205,  205,  205,  205,  205,  205,  205,  205,  205,  205,
  205,  205,  205,  205,  205,  205,  205,  205,  206,  206,
  207,  207,  207,  207,  207,  208,  209,  209,  209,  210,
  757,  758,  211,  212,  212,  213,  213,  214,  759,  760,
  761,  215,  216,  217,  218,  762,  218,  218,  219,  219,
  220,  220,  221,  221,  222,  223,  223,  763,  224,  224,
  225,  225,  225,  225,  225,  225,  225,  225,  226,  226,
  227,  228,  229,  230,  231,  231,  231,  231,  232,  233,
  233,  234,  234,  235,  236,  236,  237,  237,  238,  239,
  239,  764,  765,  240,  766,  241,  767,  241,  242,  242,
  242,  242,  242,  243,  243,  243,  244,  245,  768,  769,
  246,  246,  247,  247,  247,  247,  247,  247,  248,  248,
  249,  249,  249,  250,  250,  250,  770,  771,  772,  251,
  252,  252,  253,  253,  254,  255,  256,  257,  258,  258,
  259,  259,  773,  774,  260,  775,  261,  261,  262,  263,
  263,  263,  264,  264,  264,  265,  266,  266,  266,  266,
  266,  267,  267,  268,  268,  268,  268,  268,  268,  268,
  268,  268,  268,  268,  268,  268,  268,  268,  268,  268,
  268,  268,  268,  268,  268,  268,  268,  268,  268,  268,
  268,  268,  268,  268,  268,  268,  268,  268,  268,  268,
  268,  268,  268,  268,  268,  268,  268,  268,  269,  269,
  270,  270,  270,  270,  270,  270,  270,  270,  270,  270,
  270,  270,  270,  270,  270,  271,  271,  272,  272,  273,
  776,  273,  273,  273,  273,  273,  777,  273,  273,  273,
  273,  273,  273,  273,  273,  273,  273,  273,  273,  273,
  273,  778,  779,  274,  275,  780,  275,  276,  276,  276,
  276,  276,  276,  276,  276,  276,  277,  277,  278,  278,
  278,  278,  278,  278,  278,  278,  278,  278,  278,  278,
  278,  782,  278,  278,  278,  278,  278,  278,  278,  279,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  280,  280,  280,  280,  280,  280,
  280,  280,  280,  280,  781,  781,  281,  281,  783,  282,
  282,  283,  283,  284,  284,  285,  285,  286,  286,  287,
  287,  287,  288,  288,  289,  290,  290,  291,  291,  292,
  293,  293,  294,  294,  295,  295,  784,  296,  296,  296,
  296,  296,  296,  785,  296,  296,  296,  296,  296,  296,
  296,  296,  296,  297,  297,  298,  298,  299,  299,  300,
  301,  301,  301,  302,  302,  303,  303,  303,  303,  304,
  305,  305,  305,  305,  305,  305,  305,  305,  305,  305,
  305,  305,  305,  305,  305,  305,  305,  305,  305,  305,
  305,  306,  306,  306,  307,  307,  308,  308,  309,  309,
  310,  310,  310,  311,  311,  312,  312,  313,  313,  314,
  314,  315,  315,  316,  316,  316,  316,  316,  316,  316,
  316,  316,  316,  316,  316,  316,  316,  316,  786,  786,
  787,  787,  788,  789,  317,  317,  317,  317,  318,  318,
  319,  319,  319,  320,  320,  321,  321,  321,  322,  322,
  322,  323,  323,  323,  794,  794,  794,  793,  793,  790,
  790,  790,  791,  792,  792,  324,  325,  325,  326,  326,
  326,  795,  795,  327,  327,  327,  327,  328,  328,  329,
  329,  329,  330,  331,  331,  332,  797,  797,  333,  334,
  796,  796,  335,  335,  798,  798,  799,  800,  336,  337,
  338,  338,  339,  340,  801,  802,  341,  342,  343,  343,
  344,  344,  345,  345,  345,  345,  345,  345,  346,  346,
  347,  803,  347,  348,  349,  349,  350,  350,  351,  351,
  351,  351,  352,  352,  352,  352,  352,  353,  354,  354,
  355,  355,  356,  804,  356,  357,  357,  357,  357,  357,
  357,  357,  357,  357,  357,  805,  806,  357,  358,  358,
  359,  359,  359,  360,  360,  360,  360,  361,  361,  362,
  362,  362,  362,  363,  363,  363,  364,  364,  365,  365,
  366,  367,  367,  368,  368,  369,  370,  370,  370,  370,
  370,  370,  370,  807,  808,  371,  372,  372,  373,  374,
  374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
  374,  374,  375,  376,  376,  376,  377,  377,  377,  378,
  379,  379,  380,  380,  381,  382,  382,  383,  383,  384,
  384,  385,  385,  386,  386,  387,  387,  388,  388,  389,
  389,  389,  390,  390,  391,  392,  393,  394,  394,  394,
  395,  396,  396,  397,  398,  398,  398,  398,  399,  399,
  399,  400,  400,  400,  401,  401,  402,  402,  403,  403,
  404,  404,  405,  405,  811,  811,  406,  407,  407,  408,
  408,  409,  409,  409,  409,  410,  410,  411,  412,  413,
  413,  413,  414,  414,  415,  415,  416,  416,  416,  417,
  418,  419,  419,  420,  421,  422,  422,  422,  422,  423,
  423,  424,  424,  425,  426,  426,  426,  426,  426,  812,
  812,  427,  427,  428,  428,  429,  430,  430,  431,  431,
  432,  432,  433,  433,  434,  435,  435,  436,  436,  814,
  815,  437,  813,  813,  813,  813,  813,  813,  438,  438,
  439,  439,  440,  440,  441,  816,  817,  441,  818,  441,
  442,  442,  443,  819,  443,  444,  444,  445,  446,  446,
  447,  448,  448,  448,  449,  449,  450,  450,  451,  451,
  452,  452,  820,  820,  453,  453,  821,  821,  454,  454,
  454,  822,  455,  455,  455,  456,  457,  457,  457,  458,
  458,  459,  459,  823,  824,  460,  461,  461,  825,  461,
  461,  826,  827,  828,  462,  830,  831,  463,  832,  833,
  463,  829,  829,  834,  464,  835,  464,  464,  464,  836,
  465,  837,  465,  466,  466,  467,  468,  469,  469,  838,
  470,  471,  839,  840,  841,  472,  473,  842,  473,  474,
  474,  474,  475,  476,  843,  477,  844,  477,  478,  478,
  479,  479,  480,  480,  481,  482,  482,  483,  845,  483,
  484,  485,  485,  486,  486,  486,  487,  488,  488,  489,
  490,  490,  490,  490,  490,  490,  490,  491,  491,  491,
  491,  491,  491,  491,  491,  491,  491,  491,  491,  492,
  493,  493,  493,  493,  493,  493,  493,  493,  494,  495,
  495,  495,  496,  496,  497,  497,  498,  498,  499,  499,
  499,  500,  500,  500,  500,  500,  500,  500,  500,  501,
  501,  502,  503,  504,  504,  504,  505,  506,  506,  506,
  506,  507,  507,  508,  846,  509,  848,  510,  849,  511,
  850,  512,  847,  847,  513,  513,  514,  851,  514,  515,
  515,  515,  515,  515,  515,  515,  515,  515,  852,  515,
  516,  517,  517,  518,  518,  853,  854,  855,  519,  856,
  857,  520,  858,  859,  521,  522,  523,  523,  524,  860,
  861,  862,  525,  526,  527,  527,  528,  528,  528,  529,
  529,  530,  530,  530,  530,  530,  531,  531,  532,  532,
  533,  533,  534,  534,  535,  535,  536,  536,  537,  537,
  537,  538,  538,  538,  538,  538,  538,  538,  538,  538,
  538,  538,  538,  538,  538,  538,  538,  538,  538,  538,
  538,  538,  538,  538,  538,  538,  538,  538,  538,  538,
  538,  539,  539,  540,  540,  540,  540,  540,  541,  541,
  542,  542,  543,  544,  544,  545,  545,  546,  546,  546,
  546,  546,  546,  547,  547,  547,  547,  548,  549,  549,
  549,  549,  549,  550,  550,  550,  550,  550,  551,  551,
  552,  553,  553,  554,  555,  555,  555,  555,  555,  555,
  555,  556,  556,  556,  556,  556,  557,  557,  558,  559,
  559,  560,  561,  562,  563,  564,  564,  565,  566,  566,
  566,  567,  567,  567,  568,  568,  569,  569,  569,  569,
  569,  569,  569,  569,  569,  569,  569,  569,  569,  569,
  569,  569,  569,  569,  569,  569,  569,  569,  569,  569,
  569,  569,  569,  569,  569,  570,  570,  570,  570,  570,
  570,  570,  570,  570,  570,  570,  570,  570,  570,  570,
  571,  571,  571,  572,  572,  572,  573,  573,  574,  574,
  575,  576,  576,  577,  863,  864,  577,  865,  577,  578,
  578,  579,  579,  579,  580,  580,  580,  580,  581,  581,
  581,  581,  582,  582,  582,  582,  582,  583,  583,  584,
  584,  585,  586,  586,  586,  587,  588,  588,  589,  590,
  590,  591,  591,  591,  591,  591,  591,  591,  591,  591,
  591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
  591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
  591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
  591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
  591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
  591,  591,  591,  591,  591,  591,  591,  592,  592,  592,
  592,  592,  592,  592,  592,  592,  592,  592,  592,  592,
  592,  592,  592,  592,  593,  593,  594,  594,  595,  595,
  596,  596,  597,  597,  598,  598,  599,  599,  600,  600,
  601,  601,  602,  602,  603,  603,  604,  604,  605,  605,
  606,  606,  606,  606,  607,  607,  607,  607,  608,  608,
  609,  609,  610,  610,  610,  610,  611,  611,  611,  612,
  612,  612,  612,  613,  614,  614,  615,  615,  615,  615,
  615,  616,  616,  866,  867,  868,  617,  618,  618,  619,
  619,  620,  620,  621,  621,  622,  623,  624,  625,  625,
  626,  626,  627,  627,  627,  627,  627,  627,  627,  627,
  627,  627,  627,  627,  627,  627,  810,  810,  809,  809,
  628,  628,  629,  630,  631,  631,  632,  632,  633,  634,
  635,  636,  637,  638,  639,  640,  641,  642,  643,  644,
  645,  646,  647,  648,  649,  650,  651,  652,  653,  654,
  655,  656,  657,  657,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
  658,  658,  658,  658,  658,  658,
};
static int yylen[] = {                                         2,
    1,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    0,    0,    4,
    0,    0,   10,   10,   10,   10,   10,   10,   10,    0,
    8,    8,    0,    0,    7,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    2,    1,    1,    0,    1,
    1,    2,    0,    2,    2,    0,    4,    1,    1,    1,
    1,    1,    2,    2,    2,    2,    0,    1,    2,    4,
    1,    2,    2,    2,    2,    2,    4,    1,    1,    1,
    0,    3,    0,    3,    0,    2,    2,    1,    1,    2,
    0,    4,    0,    0,   10,   10,   10,   10,   10,   10,
   10,    0,    8,    8,    0,    7,    6,    0,    3,    0,
    3,    0,    2,    2,    0,    4,    4,    2,    2,    2,
    2,    2,    2,    3,    1,    2,    2,    2,    0,    4,
    1,    2,    2,    4,    1,    2,    2,    2,    2,    3,
    0,    0,    0,   11,    1,    1,    5,    0,    0,    2,
    0,    4,    2,    0,    4,    2,    0,    2,    2,    1,
    0,    2,    0,    4,    2,    2,    0,    2,    2,    1,
    3,    9,    1,    1,    2,    2,    0,    0,    9,    2,
    2,    2,    4,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    3,    0,    3,    0,    4,    2,
    2,    2,    2,    4,    2,    2,    2,    3,    2,    2,
    2,    2,    4,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    3,    4,    2,    2,    2,    0,    1,    0,
    3,    1,    1,    4,    0,    2,    0,    0,    8,    0,
    1,    1,    0,    1,    0,    4,    0,    0,    2,    1,
    2,    0,    0,    8,    0,    2,    2,    0,    3,    1,
    1,    0,    1,    2,    0,    1,    2,    4,    0,    0,
    4,    0,    0,    4,    2,    2,    3,    3,    0,    1,
    0,    4,    0,    4,    1,    2,    2,    0,    4,    0,
    4,    2,    2,    2,    0,    2,    5,    1,    1,    2,
    2,    0,    0,    4,    0,    2,    2,    0,    6,    3,
    2,    0,    4,    1,    0,    0,    0,    0,   10,    0,
    2,    5,    0,    2,    2,    0,    3,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    0,    1,    5,    3,
    4,    2,    2,    2,    1,    1,    1,    0,    0,    0,
    6,    0,    1,    1,    0,    2,    2,    0,    3,    3,
    2,    2,    2,    2,    2,    3,    4,    0,    2,    2,
    3,    1,    4,    6,    3,    0,    0,    5,    0,    2,
    2,    0,    3,    3,    4,    0,    1,    2,    0,    1,
    1,    0,    0,    0,    9,    0,    0,    0,    4,    1,
    1,    3,    3,    0,    1,    2,    2,    0,    0,    8,
    0,    2,    0,    4,    0,    1,    4,    4,    0,    3,
    2,    2,    4,    0,    2,    2,    0,    0,    7,    0,
    7,    3,    2,    0,    0,    7,    2,    1,    0,    0,
    4,    3,    2,    3,    3,    4,    1,    1,    3,    2,
    1,    1,    1,    0,    2,    1,    3,    6,    1,    1,
    1,    1,    1,    1,    1,    0,    0,    2,    2,    0,
    3,    3,    1,    1,    5,    2,    3,    0,    3,    0,
    2,    3,    4,    8,    1,    0,    4,    0,    1,    1,
    2,    2,    3,    3,    1,    2,    2,    2,    1,    1,
    5,    3,    0,    0,    0,    6,    1,    1,    0,    2,
    4,    0,    5,    2,    4,    3,    1,    4,    2,    2,
    0,    2,    2,    1,    1,    5,    3,    0,    8,    0,
    2,    1,    5,    3,    0,    2,    1,    1,    6,    1,
    0,    1,    2,    3,    3,    1,    1,    6,    5,    0,
    1,    1,    2,    2,    3,    2,    3,    2,    0,    1,
    1,    2,    2,    2,    0,    1,    1,    2,    4,    4,
    4,    0,    0,    0,    8,    0,    0,   10,    1,    1,
    3,    0,    1,    7,    0,    2,    1,    1,    1,    3,
    0,    1,    2,    1,    2,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    2,    1,    1,    2,
    1,    1,    1,    1,    1,    5,    2,    3,    6,    1,
    0,    0,    8,    2,    2,    0,    1,    2,    0,    0,
    0,    7,    3,    5,    1,    0,    7,    7,    2,    4,
    3,    4,    2,    4,    1,    0,    2,    0,    3,    2,
    5,    3,    3,    2,    2,    3,    3,    3,    8,    6,
    3,    0,    3,    2,    1,    1,    3,    3,    7,    0,
    2,    1,    2,    2,    0,    1,    0,    1,    3,    1,
    2,    0,    0,    6,    0,    2,    0,    4,    2,    2,
    2,    2,    1,    1,    1,    1,    2,    2,    0,    0,
    7,    3,    1,    1,    1,    1,    2,    2,    5,    7,
    0,    1,    3,    0,    2,    4,    0,    0,    0,   10,
    0,    4,    2,    4,    4,    5,    1,    1,    0,    3,
    5,    3,    0,    0,    4,    0,    4,    5,    1,    1,
    1,    0,    4,    2,    2,    2,    1,    1,    2,    2,
    2,    1,    3,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    3,    2,
    2,    2,    3,    3,    2,    2,    2,    3,    1,    1,
    1,    1,    1,    3,    3,    3,    3,    3,    3,    5,
    5,    5,    5,    5,    5,    0,    2,    2,    1,    2,
    0,    5,    2,    2,    2,    2,    0,    4,    2,    2,
    3,    2,    2,    2,    3,    3,    2,    2,    3,    4,
    4,    0,    0,    4,    2,    0,    3,    2,    3,    2,
    2,    2,    3,    3,    2,    2,    2,    4,    3,    3,
    3,    3,    4,    4,    5,    5,    4,    5,    3,    4,
    4,    0,    5,    4,    4,    4,    3,    2,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    2,    1,    1,    0,    4,
    2,    3,    4,    0,    2,    3,    2,    2,    4,    0,
    1,    1,    2,    2,    3,    0,    2,    0,    2,    5,
    1,    1,    1,    1,    2,    3,    0,    3,    4,    3,
    2,    2,    5,    0,    5,    1,    4,    2,    3,    2,
    2,    5,    5,    0,    2,    1,    3,    2,    4,    1,
    8,    6,    5,    2,    0,    0,    3,    3,    3,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    3,    2,
    3,    2,    2,    2,    2,    5,    2,    2,    3,    3,
    4,    0,    2,    2,    1,    1,    1,    3,    1,    1,
    3,    6,    1,    1,    1,    4,    5,    1,    3,    1,
    3,    1,    2,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    2,    4,    2,    4,    1,    1,    1,
    0,    3,    0,    0,    6,    4,    6,    5,    0,    3,
    0,    2,    2,    0,    3,    4,    1,    5,    4,    1,
    4,    4,    1,    4,    1,    2,    2,    1,    1,    1,
    2,    2,    1,    1,    2,    2,    2,    2,    0,    3,
    5,    1,    1,    2,    3,    1,    2,    0,    3,    1,
    1,    1,    2,    3,    4,    5,    0,    1,    3,    3,
    0,    1,    0,    2,    0,    1,    0,    0,    5,    3,
    3,    3,    6,    4,    0,    0,    7,    6,    1,    2,
    1,    1,    1,    1,    1,    1,    5,    5,    0,    2,
    2,    0,    4,    1,    3,    4,    6,    5,    0,    1,
    1,    1,    0,    1,    1,    1,    1,    4,    1,    1,
    0,    2,    2,    0,    3,    2,    2,    1,    2,    2,
    3,    2,    2,    2,    3,    0,    0,    4,    4,    1,
    2,    3,    3,    1,    4,    2,    3,    1,    1,    0,
    1,    2,    2,    0,    1,    1,    1,    1,    1,    3,
    2,    0,    3,    1,    3,    4,    5,    6,    7,    7,
    6,    1,    1,    0,    0,    9,    0,    2,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
    1,    1,    1,    1,    2,    2,    1,    2,    1,    2,
    1,    3,    1,    1,    4,    0,    3,    0,    2,    0,
    3,    0,    2,    2,    0,    3,    3,    4,    5,    0,
    3,    2,    1,    3,    6,    1,    1,    1,    4,    4,
    1,    1,    6,   10,    0,    2,    1,    1,    2,    4,
    2,    2,    4,    2,    1,    1,    1,    1,    1,    3,
    1,    3,    1,    3,    0,    1,    2,    1,    3,    1,
    1,    1,    1,    1,    1,    3,    3,    5,    2,    0,
    1,    2,    0,    3,    1,    3,    1,    1,    1,    4,
    5,    5,    5,    2,    4,    4,    2,    6,    4,    0,
    3,    1,    3,    1,    0,    1,    2,    2,    2,    0,
    1,    0,    3,    1,    3,    1,    0,    2,    0,    2,
    0,    2,    1,    3,    5,    0,    1,    0,    2,    0,
    0,    6,    0,    1,    2,    1,    1,    1,    1,    3,
    2,    1,    1,    2,    1,    0,    0,    6,    0,    5,
    1,    3,    0,    0,    5,    0,    1,    3,    1,    3,
    3,    0,    1,    1,    0,    2,    1,    1,    2,    4,
    0,    1,    1,    1,    0,    3,    1,    1,    0,    5,
    4,    0,    9,    6,    4,    3,    0,    3,    3,    1,
    3,    1,    1,    0,    0,   13,    2,    2,    0,    3,
    3,    0,    0,    0,    6,    0,    0,    7,    0,    0,
    9,    0,    2,    0,    5,    0,    7,    2,    4,    0,
    9,    0,   11,    1,    1,    9,    5,    1,    1,    0,
   11,    7,    0,    0,    0,   19,    0,    0,    3,    0,
    2,    4,    4,    3,    0,    2,    0,    4,    3,    3,
    1,    1,    0,    1,    3,    1,    3,    0,    0,    2,
    4,    1,    3,    1,    3,    4,    1,    1,    3,    1,
    1,    3,    3,    2,    3,    3,    4,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    3,
    1,    1,    1,    1,    1,    1,    1,    1,    6,    1,
    1,    1,    5,    6,    5,    6,    3,    4,    1,    1,
    2,    3,    5,    4,    6,    4,    6,    5,    7,    3,
    4,    4,    4,    1,    1,    1,    1,    3,    3,    4,
    4,    1,    3,    3,    0,    4,    0,    5,    0,    4,
    0,    5,    0,    1,    0,    2,    2,    0,    3,    2,
    2,    2,    2,    3,    3,    1,    1,    2,    0,    5,
    3,    2,    4,    3,    2,    0,    0,    0,    5,    0,
    0,    5,    0,    0,    5,    1,    5,    6,    3,    0,
    0,    0,    8,    1,    2,    2,    1,    1,    1,    1,
    1,    3,    3,    3,    2,    2,    0,    2,    3,    2,
    1,    1,    1,    1,    1,    3,    1,    3,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    2,    2,    3,    3,    3,    3,    5,    3,
    3,    3,    3,    1,    1,    1,    1,    3,    1,    1,
    1,    1,    1,    1,    2,    2,    2,    2,    0,    3,
    0,    3,    4,    0,    1,    1,    3,    1,    2,    2,
    2,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    2,    2,    2,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    4,    1,    1,    1,    1,    1,    1,    2,    1,    1,
    2,    1,    1,    1,    1,    1,    2,    1,    1,    1,
    1,    1,    1,    1,    1,    6,    4,    5,    5,    5,
    5,    5,    5,    5,    5,    5,    5,    6,    6,    4,
    4,    4,    4,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    3,    3,    3,    3,    3,
    4,    4,    7,    8,    6,    4,    8,    6,    4,    4,
    0,    2,    2,    1,    1,    1,    1,    1,    3,    5,
    5,    0,    3,    0,    0,    0,    4,    0,    4,    1,
    4,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    0,    3,    3,    2,    2,    0,    2,    1,
    1,    6,    1,    1,    1,    4,    4,    4,    4,    4,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    7,    8,    8,
    8,   12,    6,    4,    6,    6,   10,    8,    6,    4,
    9,    9,   11,    4,    1,    1,    1,    1,    0,    2,
    0,    1,    0,    2,    0,    2,    0,    2,    0,    2,
    0,    1,    1,    1,    0,    2,    1,    1,    1,    1,
    2,    2,    2,    2,    1,    1,    4,    4,    7,    8,
    0,    2,    7,    6,    6,    4,    1,    1,    1,    4,
    3,    6,    5,    6,    1,    1,    6,    8,    6,    6,
    8,    1,    1,    0,    0,    0,    8,    4,    5,    0,
    2,    3,    5,    4,    5,    1,    1,    1,    3,    5,
    4,    6,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    0,    1,    0,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,
};
static int yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,   38,    0,    0,
  111,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    3,    4,    5,    6,   21,   23,   19,   17,   22,
   18,  849,   10,   11,   15,   20,   26, 1180, 1181, 1182,
   27,   28,   25,   34,   33,   29,   30,   31,   36,   37,
   32,   35,   24,   16, 1292,   12,    0,    0,    8,    0,
    9,    7, 1534, 1535,   13, 1538, 1539,   14, 1293,    0,
    0,  857,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  216,    0,    0,  848,
 1192,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  249,    0,    0,    0,    0,    0,    0,  216,
    0,  195,    0,    0,    0,  158,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  216,    0, 1070,    0,  747,   39,
    0,    0,   39,    0,    0,    0, 1197,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2242, 2243, 2244, 2245,
 2246, 2247, 2248, 2249, 2250, 2251, 2252, 2253, 2254, 2255,
 2256, 2257, 2258, 2260, 2261, 2262, 2263, 2264, 2265, 2266,
 2259, 2267, 2268, 2269, 2270, 2271, 2272,    0, 2273, 2274,
 2275, 2276, 2277, 2278, 2279, 2280, 2281, 2282, 2283, 2284,
 2285, 2286, 2287, 2288, 2289, 2290, 2291, 2292, 2293, 2294,
 2295, 2296, 2297, 2298, 2300, 2301, 2133, 2135, 2136, 2137,
 2138, 2139, 2140, 2158, 2159, 2141, 2143, 2142, 2151, 2152,
 2148, 2144, 2149, 2145, 2147, 2150, 2146, 2154, 2155, 2153,
 2156, 2157, 2161, 2160, 2162, 2163, 2164, 2165, 2166, 2167,
 2168, 2169, 2170, 2171, 2172, 2173, 2174, 2175, 2176, 2177,
 2178, 2179, 2180, 2181, 2182, 2183, 2184, 2185, 2186, 2187,
 2188, 2189, 2190, 2191, 2192, 2193, 2194, 2195, 2196, 2197,
 2198, 2199, 2200, 2201, 2202, 2203, 2204, 2205, 2206, 2207,
 2208, 2209, 2210, 2211, 2212, 2213, 2214, 2215, 2216, 2217,
 2218, 2219, 2220, 2221, 2222, 2223, 2224, 2225, 2226, 2299,
 2227, 2228, 2229, 2235, 2231, 2230, 2232, 2233, 2234, 2236,
 2237, 2238, 2239, 2240, 2241, 2306, 2307, 2308, 2309, 2310,
 2311, 2313, 2314, 2315, 2316, 2302, 2317, 2303, 2304, 2305,
 2312, 2318, 2319, 2320, 2321, 2322, 2323, 2324, 2325, 2326,
 2327, 2328, 2329, 2330, 2331, 2332, 2333, 2334, 2335, 2338,
 2340, 2337, 2341, 2342, 2343, 2344, 2345, 2346, 2347, 2348,
 2349, 2350, 2351, 2352, 2353, 2354, 2355, 2356, 2357, 2358,
 2359, 2360, 2361, 2363, 2364, 2365, 2366, 2367, 2368, 2369,
 2370, 2372, 2373, 2374, 2376, 2377, 2378, 2379, 2380, 2381,
 2382, 2383, 2384, 2385, 2386, 2387, 2388, 2389, 2390, 2391,
 2392, 2393, 2394, 2395, 2396, 2339, 2375, 2362, 2336, 2371,
 2397, 2398, 2399, 2400, 2401, 2402, 2403, 2404, 2405, 2406,
    0, 1412,    0, 2125, 2134,    0,    0, 1343,    0, 2124,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  220, 1183, 2130,    0,    0,    2,    0,    0,    0,
    0,    0, 1348, 1351, 1352,    0,    0,    0,    0,  369,
  912,  913,  914,  915,  916,  917,  918,  919,  920,  921,
  922,  923,  924,  925,  926,  927,  928,  929,  930,  931,
  932,  940,  934,  935,  933,  967,  987,  964,  936,  937,
  938,  939,  941,  943,  946,  942,  944,  945,  947,  948,
  949,  950,  951,  952,  953,  954,  955,  956,  957,  958,
  959,  969,  981,  979,  965,  982,  960,  966,  985,  986,
  980,  968,  983,  984,  988,  989,  963,  962,  961,  970,
  971,  972,  973,  974,  975,  976,  977,  978,  990,  991,
  992,  993,  994,  995,  996,  997,  998,  999, 1000, 1001,
 1002, 1003, 1004,  859,  872,  911,  850,    0, 2114,    0,
    0,  557,  544,  545,    0,  862,  548, 2104,  867,  308,
 2117,  860,    0, 2118,  527,  519,  520,    0,  855,  523,
 2122,  851, 1414,  854,    0, 2126,  864,    0, 2127,  758,
  853,    0, 1031, 1032,  863,    0, 2123,  868,  218, 1680,
  566,  856,    0, 2131, 1200,    0, 1189, 1807,    0,  208,
  368, 1809,  374,  209,    0, 2110,  196,    0,  200,  206,
  289,    0,  201, 1812,  210, 1814,    0,  202,    0,  204,
    0,  763,  213, 1645,  205,  211,  322,  207,  212,    0,
 2109,    0,  218, 1676,    0,  214, 1473, 1474,    0,    0,
  159,    0, 2115,    0, 1078, 2113, 1071,    0, 1077, 1080,
 1083, 1072, 1073,    0, 1074, 1075,    0, 1076, 2129, 1082,
 1084, 1085,  218,    0,    0, 1088,    0,    0,    0,  748,
    0, 1496,    0,    0,    0, 1190,    0,    0, 1198, 1201,
    0,    0, 1202,    0,    0,    0,    0,    0,    0,    0,
 1225,    0, 1540, 1376,    0,    0,    0,    0,    0,  229,
  222,  230,  221,  223,  226,  232,  225,  231,    0,    0,
  227,    0,    0, 1299, 1300, 1305,    0, 1304,    0, 1306,
 1308, 1314,    0, 1301, 1303,    0, 1302, 1309, 1313, 1307,
 1311,    0,  216, 1312,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1358,    0,    0,    0,
    0, 1467,    0,    0,    0, 2108,  866,    0, 2107, 1654,
  865, 1649,  369,  873,  247,  861,    0,    0,    0,    0,
  425,    0,    0,  162,  290, 1023, 1024,    0,    0,  524,
  417,  770,  771,    0, 1647,    0,    0, 1564, 1034, 1033,
    0,  323, 1680,  869,    0,    0, 1196, 1194, 1808,  369,
  283,    0,  245,  290,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  216,    0,  233,  262,  261,    0,    0,
  412,    0,    0,    0,    0,  764, 1295,    0,    0, 1676,
  217,    0,  215,    0,    0,  160,  161,    0,    0,    0,
    0, 1079,    0,    0, 1081,    0,    0, 1090, 1686, 1694,
 1089,    0,    0,    0,    0, 1752,    0, 1797,    0, 2101,
    0, 1749, 1951,    0,    0,    0,    0, 1969,    0, 1798,
    0, 1792, 1780, 1781, 1775, 1776, 1778,    0, 1779,    0,
    0,    0, 1754,    0,    0, 1782, 1783, 1793, 1794,    0,
 1795, 1796,    0,    0,    0, 1800, 1634, 1635, 1636,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1963, 1966,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1799, 1789, 1790, 2102, 1753,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1637,    0,
 1964, 1965,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1791, 1728,    0,
    0, 1574, 1714, 1581, 1588, 1595, 1589, 1590, 1591, 1592,
 1593, 1596, 1597, 1598, 1599, 1594,    0,    0,    0, 1717,
 1747, 1750, 1723,    0, 1784, 1777, 1774, 1725, 1726, 1727,
 1744, 1745, 1746, 1785, 1724, 1820,    0,    0, 1878,    0,
 1821, 1822, 1910, 1911, 1913, 1914, 1915, 1824,    0, 1921,
    0,    0,    0, 1823, 2035, 2036, 1732, 1729, 1730, 2055,
 2056, 2062, 2063, 1731, 1751,    0, 1577,    0,    0,    0,
    0,    0,   40,    0,   39,    0,    0,    0,    0,  112,
  113,  125, 1188,    0,    0, 1286,    0, 1240, 1238, 1239,
 1818,    0,    0,    0, 1123, 1172, 1129,    0, 1173,    0,
    0, 1130,    0,    0,    0, 1160, 1176, 1122, 1155,    0,
    0, 1119, 1128,    0,    0, 1120, 1164, 1211, 1104, 1112,
 1105, 1114, 1115,    0, 1118, 1116, 1117,    0, 1209, 1121,
    0,    0,    0,    0,    0,    0, 1204, 1205,    0,    0,
    0,    0, 1226,    0, 1555, 1413, 1344,    0, 1395, 2119,
    0,    0,  228, 1185, 1310, 1318, 1316, 1315, 1294,  218,
    0,    0,    0, 2121,    0,    0,    0,    0,    0, 1327,
    0,    0, 1325,    0, 1361, 1359,    0, 1364, 1362, 2098,
 1365,    0, 1366, 1356,    0,    0,    0, 2100,    0,    0,
    0, 1338,    0,    0, 1495,    0,    0, 1548, 1570,    0,
 1295,    0,    0,  873,    0,    0,    0,    0,  871,    0,
    0,    0,    0,    0,  547,    0,    0,    0,  522,    0,
  417,  830,  829,    0,    0, 1295,    0, 1566,    0,    0,
    0,  870, 1681, 1690,    0,    0,    0,    0, 1095, 1096,
    0,    0,    0,    0,  239,    0,  558,  235,  240,  301,
  528,  234,  769,  236,  242,    0,  757,  238,  241,  218,
 1683,  567,  237,  264,    0,    0,  441,    0,    0,    0,
  762,    0,    0,    0,  335,  203,  428,  219, 1677,    0,
    0,  162, 2106,    0,    0,  194, 1810,  193, 2105,    0,
    0, 1430,    0,    0,    0, 1537, 1094, 1093,    0, 1091,
    0,    0,    0, 1786,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1787, 1788,    0, 1756,    0, 1758,
    0,    0,    0,    0, 2064,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1755, 1757,
    0,    0,    0,    0,    0,    0,    0,    0, 1715,    0,
 1346, 1347,    0,    0,    0,    0,    0,  694,    0,    0,
  739,    0,    0,    0, 1619, 1604,    0,    0, 1605,    0,
 1608,    0, 1606, 1607,    0,    0,    0, 1601, 1602, 1603,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  171,  749,    0,   78,   79,    0,    0,    0,    0,    0,
   73,    0,    0,    0,    0,    0, 1381, 1385, 1383, 1384,
 1380, 1397, 1398, 1399, 1382,    0,    0,    0,    0,    0,
    0,   39, 1186,  319,    0,  317,  318, 1816, 1199, 1198,
 1234, 1235, 1236, 1237, 1228,    0, 1134, 1156, 1157, 1177,
    0, 1174,    0,    0, 1167, 1161, 1162,    0,    0, 1210,
 1124,    0, 1126, 1165, 1166,    0, 1113,    0,    0,    0,
    0,    0,    0,    0, 1168, 1206,    0,    0, 1337, 1336,
 1543,    0,    0,    0, 1394,    0,  224, 1295,    0, 1687,
    0, 1324, 1287, 1323,    0,    0,    0,    0,    0,    0,
    0,    0, 1331, 1334,    0,    0, 1368,    0,    0, 1369,
 1371,    0,    0,    0,    0, 1469,    0, 1350, 1349,    0,
    0,    0, 1720,    0, 1719, 1768, 1773, 1721,    0, 1339,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1046,    0,    0,    0,  378,  378,
 1035,    0,    0, 1027,    0, 1025,    0,    0,  363,  364,
  362,  554,    0,  422,  423,    0,    0,    0,    0,  581,
    0,    0,  556,    0,    0,  309,  290,    0,  525,    0,
    0,    0,    0,    0, 1064,    0,    0,    0,    0, 1565,
    0,    0,  324,    0, 1678,    0,    0,  370,  369,    0,
  286,  272,    0,    0,  291,  290,  246,  290, 1651, 1683,
  243,    0,    0,  440,  413, 1069, 1067, 1068,    0,    0,
  765,  766, 1646, 1295,  336,    0, 1678,    0,    0,    0,
 1813, 1811,    0,    0,    0, 1444, 1446,    0, 1447, 1439,
 1440,    0, 1086,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2083, 2085, 2086, 2087, 2088, 2089, 2095, 2096,
 2090, 2094, 2091, 2092, 2084, 2093,    0,    0,    0,    0,
    0,    0, 2072,    0, 2065,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2047, 2049, 2048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1856,    0,    0,    0,    0,    0, 1857,
 1860,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1859,    0,    0,
    0, 1875, 1874, 1876,    0, 1858, 1805, 1802, 1803, 1804,
 1806,    0,    0,    0,    0,    0, 1743,    0, 1718,  743,
    0,    0,  695,    0,  696,    0,    0,    0,    0,    0,
    0, 1630, 1642,    0, 1638,    0, 1639, 1586,    0,    0,
    0,    0,    0,    0,    0, 1621,    0, 1610, 1612, 1611,
    0,    0,    0,    0, 1738, 1737,    0,    0,    0,    0,
    0,    0,    0,    0, 1879, 2132,    0,    0,    0,    0,
 2051,    0, 1748, 1575,    0,    0,  169,    0,    0,    0,
    0,    0,    0,    0,   63,   64,   68,   59,   61,   58,
   56,   57,   65,   62,   67,   60,    0,    0,    0,    0,
    0,    0,   54,  149,    0,  148,  151,    0, 1389,    0,
    0,    0,    0,    0, 1416,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  129,  131,    0,    0,    0,
    0,    0,    0,  321,  320, 1817,    0,    0, 1815,    0,
    0,    0,    0, 1175,    0,    0,    0,    0,    0,    0,
    0, 1214, 1215, 1213, 1208,    0,    0,    0,    0,    0,
    0,    0, 1230, 1231, 1232, 1227, 1203, 1544, 1557,    0,
    0, 1556, 1578,    0,    0, 1396,    0,    0, 1688,    0,
 1288, 1320,    0,    0, 1322, 1291, 1333, 1360, 1363,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1771, 1770,
 1772, 1769, 2128, 1484, 1483, 1486, 1487, 1488,    0,    0,
 1498, 1499,    0, 1494,    0, 1572,    0, 1650, 1036,    0,
    0, 1041,    0, 1048, 1042,    0,    0,    0, 1051, 1050,
    0,    0,    0,    0,    0,    0,    0,    0,  875, 1029,
  360,    0,    0,    0,    0,    0,  546,    0,  578,    0,
  587,    0,    0,  582,  583,  584,  530,  169,    0,  293,
    0,  521,    0,  418,  498,    0,    0,    0,    0,  887,
    0,    0,    0,    0,  847,    0,    0, 1648, 1567,    0,
  756, 1030,    0,    0,  326,  327,    0, 1691,    0,    0,
    0,    0,  562,    0,    0,    0,  284,  485,  480,  481,
  482,  483,  484,  273,    0, 1108,    0,    0,  293,    0,
    0, 1295,  244, 1684,    0,  257,  431,  761,    0,    0,
    0,    0,    0,  413,    0,    0,    0,    0,    0,    0,
  572,  197,  171,    0,  169,    0, 1553, 2112, 1445,    0,
 1482,    0,    0,    0,    0,    0, 1827,    0, 1632,    0,
 2037,    0,    0,    0,    0,    0, 2000, 1633,    0,    0,
    0, 1760, 1762,    0,    0,    0,    0,    0,    0, 2066,
    0,    0,    0, 2081, 1916, 1917, 1918, 2038, 1919,    0,
 2046,    0,    0,    0,    0,    0,    0,    0, 1994,    0,
    0,    0, 1861,    0, 1862,    0, 1866,    0, 1869, 1840,
 1841, 1842, 1843,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2031, 2033, 2034, 2032,    0,
    0, 1870, 1801, 2004,    0,    0, 1716, 1576,    0,    0,
    0, 1763,    0,    0,    0,    0,    0, 1640, 1641, 1587,
    0,    0, 1631,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1437, 1920,    0,    0,    0, 2050,    0,  740,
    0,    0,    0,    0,   70,    0,    0,    0,    0,    0,
    0,    0,   66,   50,   50,   76,   76,   76,    0,    0,
   82,    0,   81,    0,   80,    0,   75,   89,   96,  132,
    0,  152,    0, 1386, 1387, 1421, 1419, 1417,    0, 1504,
 1418,    0,    0,    0,    0,    0,   50,    0,    0,    0,
    0,    0,   50,    0,    0,    0,    0, 1256, 1248,    0,
    0,    0, 1243,    0,    0,    0, 1136,    0,    0, 1179,
    0, 1170, 1125, 1132, 1127, 1145,    0,    0,    0, 1146,
 1149, 1151, 1152, 1154, 1207, 1206,    0,    0,    0, 1542,
    0,    0, 1345,    0,    0, 1689, 1289, 1290,    0,    0,
    0,    0, 1370, 1374, 1372, 1353, 1470,    0, 1471,    0,
    0,    0,    0, 1503,    0, 1500,    0,    0, 1571,  359,
    0,    0,    0,  270, 1040,    0, 1049,    0, 1044, 1056,
  378,    0,    0,  877,    0,    0,  880,  882,  881,    0,
    0,  878,  885,  886,  365,  366,  367,  361,  553,  596,
  592,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  638,  614,
  609,    0,    0,  608,    0,  618,  617,  641,  629,  630,
  644,    0,  627,  645,  642,  631,  624,  643,    0,  633,
  634,  632,  724,  726,  725,  625,    0,  619,  620,  623,
  621,  622,  639,  628,    0, 1562, 1561,    0,    0,  603,
    0,  588,    0,    0,    0,    0,    0,    0,    0, 1100,
 1099, 1103,    0,    0,    0,  313,  312,  526,    0,    0,
 1006,    0,    0,    0,    0,  909,  908,    0,  910,    0,
    0,    0,    0,    0,    0,    0,    0, 1062,    0,    0,
    0,    0,    0, 1682,    0,    0,    0,  559,  563,  371,
  369,    0,    0,    0,    0,    0,    0,    0,  377,    0,
    0, 1106,    0,    0,    0,    0,  295,  296,  302,    0,
    0, 1678,  373,    0,  258,    0,  444,    0,    0,    0,
    0,    0,    0, 1666,    0, 1667,    0,    0,    0,    0,
    0,    0, 1657,    0,    0,  337,  431, 1679,  574,    0,
    0,  576,    0,    0,    0,  569,    0,    0,  573,  198,
  169,  163,    0,    0,    0, 1441,    0, 1833, 1832,    0,
    0,  476,  479, 1829, 1828,    0, 1837, 1836, 1835, 1834,
    0,    0, 1831, 1830,    0,    0,    0, 2074, 2073,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  746,  698,  697,    0,
 1644, 1643, 1613,    0,    0,    0,    0,    0,    0,    0,
 1739,    0,    0,    0, 1880,    0,    0,    0, 2053,    0,
  174,  752,    0,  753,    0,  174,    0,    0,    0,    0,
    0,    0,    0,    0,  132,  132,    0,    0,    0,   92,
   93,   84,   83,    0,    0,  150,    0,    0, 1391,    0,
 1505, 1400,    0, 1411, 1406,    0,  132,    0,    0,    0,
    0,    0,    0,    0,  122,  122,  132, 1245, 1254,    0,
 1249, 1246, 1247, 1257,    0, 1253, 1252,    0,    0, 1250,
 1260,    0, 1138,    0, 1142, 1143,    0,    0,    0,    0,
    0, 1148,    0,    0,    0, 1558,    0, 1560,    0, 1579,
    0,    0, 1298,    0,    0,    0, 1477, 1478, 1476,    0,
 1491,    0,    0, 1573, 1039,  396,  271,    0, 1047,    0,
  369,    0,    0,    0,  879,  883,  884,  597,  593,    0,
  659,    0,    0,    0,  703,  706, 2120,  727,  728,    0,
  730,  704,    0,  640,  610,  712,  615,    0,  710,  616,
    0,    0,    0,    0,    0,  701,    0,    0,  599,  600,
    0,    0,    0,    0,  174,  531,  534,    0,    0,    0,
    0,    0,  311,    0,  314,  300,    0,  888,    0,  891,
  892,  498,    0,    0,  890, 2111,  907, 1022, 1021,  889,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  899,
  467,    0,  468,  455,    0,    0,    0,    0,    0,    0,
    0,  760,  330,    0, 1712, 1711,    0,    0,    0,    0,
    0,    0,  564,  565,    0,    0,  379,    0,    0,  385,
    0,  382,  381,  384,  383,    0,  273, 1109, 1111, 1107,
  294,    0,  305,  293,    0, 1652,    0,    0,    0,  433,
  414,  442,    0,    0,    0,    0,  777,  778,  774,  767,
    0,    0,    0,    0,    0,    0,  782,  776, 1659,    0,
 1663,    0, 1660, 1662, 1661, 1669, 1668,    0,  341,    0,
    0,  575,  577,  568,  250,    0,    0,  174,  166,    0,
    0,  173,  165,    0,    0, 1536,    0, 2054, 2082,    0,
 1999, 1912,    0,    0,    0, 2075,    0,    0, 2057, 2059,
    0, 2044, 2045,    0,    0,    0,    0,    0,    0,    0,
 2060,    0, 1839, 1838,    0,    0,    0, 1865,    0, 1868,
 1844, 1845, 1846, 1847, 1848, 1849, 1850, 1851, 1852, 1853,
 1854, 1855, 1993, 1996,    0,    0,    0, 1614,    0,    0,
    0,    0, 1609, 1826,    0,    0, 1995,    0,    0,    0,
    0, 2052,    0,    0,  540,    0,    0,   42,   42,   42,
   42,   42,   42,   42,    0,    0,    0,   77,    0,   91,
    0,   90,   99,   98,  100,   97,    0,  135,    0,    0,
    0,  153,  155, 1392, 1388, 1401, 1505,    0,    0, 1402,
 1403,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  132,  132,    0, 1251,    0, 1255, 1257,    0,    0,    0,
 1261, 1137,    0, 1135, 1171,    0,    0, 1222, 1221, 1224,
    0,    0, 1296,    0,    0,    0,    0,    0, 1490, 1501,
 1493,  397, 1043,    0, 1045,    0,    0, 1053, 1052,    0,
    0,    0,    0,    0,  660,    0,    0,  691,    0,  732,
  657,    0,  713,  711,    0,    0,    0,    0,  651,    0,
    0,    0,    0,  589,    0,  607,    0,  590,  591,    0,
    0,    0, 1101,  475,    0,  552,  549, 1098,  316,  298,
  501,    0,    0,    0,  453,    0,  472,    0,  473,  905,
  906,    0,  893,    0,  894,  470,  901,  904,  900,  897,
    0, 1008,    0,    0,  455,  844,  845,  841,  840,  843,
  842, 1061,  328, 1709,    0,    0, 1705,    0, 1706, 1692,
    0,    0,    0,  390,  392,    0,  380,  386,  274,    0,
    0,  297,    0,  306,  307, 1685,  410,  411,  266,  259,
    0,  444,    0,  445,  446,  768,  780,  781,  779,    0,
    0,  803,  797,  791,  815,  788,  785,  794,  812,  800,
  806,  809,  826,  821,    0,  802,  796,  790,  814,  787,
  784,  793,  811,  799,  805,  808,  817,  825,    0,  804,
  798,  792,  816,  789,  786,  795,  813,  801,  807,  810,
  818,  827,    0,    0, 1664, 1665, 1295,    0,  338,  336,
  429,  199,    0,  172,  183,    0,    0,    0,    0,    0,
  180,  176,    0,    0,    0, 1452,    0, 1449,    0, 1453,
    0,  477,    0, 2039,    0,    0,    0,    0,    0,    0,
    0, 2043, 1988,    0,    0,    0,    0,    0,    0,    0,
 1863,    0,    0, 2014,    0,    0,    0,    0,    0, 1888,
 1885,    0,    0,    0, 2018,    0,  754,    0,    0,  537,
    0,  750,  132,  132,  132,  132,  132,  132,  132,    0,
  135,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  133,  134,  145,    0,    0,    0,  108,    0,   55,
    0,  156,  157,    0,    0, 1404,    0, 1408,  114,  114,
  114,  114,  114,  114,  114,    0,    0,  126,    0, 1279,
    0, 1284,    0,    0,    0, 1271, 1263, 1262,    0,    0,
 1140, 1217, 1218,    0,    0,    0, 1671,    0, 1424,    0,
 1428,    0,    0,    0, 1055,  370, 1058, 1060,  288,    0,
  594,    0,    0,    0,    0,    0,  733,  734,  736,  735,
    0,    0,    0,  715,    0,  652,    0,    0,    0,  601,
  606,    0,  535,  536,    0,  551,    0,    0,    0,  505,
  499,    0,  447,  450,  452,  895,  896,  898,  469,    0,
  903,    0,  323, 1704, 1703, 1708, 1702,    0,  391,    0,
    0,  387,    0,    0,  277,  281,  280,  304,    0,  443,
  415,    0,  773,  819,  783,  823,  824,  828,    0,    0,
    0,    0,    0,  336,  431,    0,  253,    0,    0,    0,
    0,    0,    0,  182,  175,    0,  178,  179,  192,    0,
 1442, 1456, 1455, 1459, 1451, 1454,    0, 2040,    0, 2071,
    0, 2067, 2058, 1989, 1990, 1991,    0, 2061,    0, 1998,
 1872, 1873, 1864, 1867,    0, 2012,    0,    0, 1886, 1886,
    0, 1881,    0,    0,  755,  174,  533,  539,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   51,  139,  147,
  138,  141,  146,  142,  143,  140,    0,   52,  104,  154,
  107,    0,  106,  109,    0,    0,    0, 1507, 1508,    0,
  132,  132,  132,  132,  132,  132,  132,  123,  124,    0,
    0,    0, 1281, 1259, 1272, 1273,    0, 1267, 1223,    0,
 1541,    0,    0, 1433,    0, 1354,  398,  397,    0,    0,
    0,    0,    0,  661,    0,    0,  646,    0,    0,    0,
    0,    0,  664,    0,    0,  542,  543,    0,    0,    0,
    0,    0,    0,  458,    0,  448,  448,  466,    0, 1009,
    0,    0,    0,    0,    0, 1699, 1698, 1693,    0, 1697,
    0,  395,  279,  287,    0,    0,  418,  420,  421, 1669,
 1670,    0, 1672,    0,  342,  339,  358,  336,  345,    0,
    0,    0,    0,  251,    0,    0,    0,  186,    0,  190,
  185,    0, 1450,    0,    0,    0, 2068,    0,    0,    0,
 2002,    0,    0,    0,    0, 1906,    0, 1907, 2001,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  102,
  136,  137,  144,  110,    0, 1510, 1511,    0, 1512, 1405,
    0,    0,    0,    0,    0,    0,    0, 1280, 1276, 1275,
    0, 1285, 1268, 1269, 1265,    0, 1425,    0,    0,    0,
    0, 1059,    0,  595,  649,    0,    0,    0,    0,    0,
  717,  723,    0,    0,    0,    0,  716,    0,    0,    0,
    0,    0, 1102,    0,    0,    0,  502,  457,    0,    0,
    0,    0,    0,    0,    0,    0, 1017,  332,  331,  334,
 1696, 1695, 1700, 1701,    0,  427,  426,    0,    0, 1675,
    0,  347,    0,    0,    0,    0,  348,  349,  350,  433,
    0,  436,  432,  256,    0,  184,    0,  188,    0,  167,
 1457, 1457,  478, 2069, 1997,    0,    0,    0,    0,    0,
 1889, 1890, 1887, 1905, 1904,    0, 2023, 2024,    0, 2022,
  538,   43,   45,   44,   46,   47,   48,   49, 1506,    0,
 1513,  115,  117,  116,  118,  119,  120,  121, 1277, 1278,
 1283, 1545, 1434,    0,    0,    0,    0,  401,    0,    0,
  662,  660,    0,  731,  714,    0,  722,  721, 2116,  719,
  720,    0,    0,  655,  654,  708,  699,    0,    0,  503,
    0,  454,    0,  487,    0,    0, 1012, 1010,  455, 1016,
    0, 1018,    0,    0,  419, 1673, 1674,  351,  354,  353,
  356,  355,  352,    0,    0,  254,  164,  191,    0,    0,
 2003,    0,    0,    0,    0, 1893, 1892, 1894, 2027, 2028,
    0,    0,    0, 1516, 1514,    0,    0,  403,    0,    0,
    0,  598,  666,    0,    0,  689,  718,  653,    0,  604,
    0,    0,  455,  449,    0,    0,  451, 1013,    0,    0,
  333,  394,  434,    0,    0,    0, 1461,    0, 1460, 1896,
 1895, 1898, 1897,    0, 1992,    0, 1519, 1523, 1517,    0,
 1548,    0, 1435,    0,  408,  404,  660,    0,    0,    0,
    0,    0,    0,  680,  709,    0,  507,    0,    0,  489,
  490, 1019,  437,  438,    0, 1458, 1464,    0,    0,    0,
 1891, 1513,    0,    0,    0, 1515,    0,    0,  405,    0,
    0,    0,  679,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  460,  456,  491,    0, 1462,    0,
 1900, 1899, 1902, 1901, 1520,    0,    0, 1518,    0, 1528,
    0, 1549,    0,  661,  667,  661,  668,    0,  669,  673,
    0,    0,    0,    0,    0,    0,  686,  688,  687,    0,
    0,  504,    0,  512,    0,  511,  455,    0,    0,    0,
  494,  492,  493,    0,    0,    0, 1530,    0, 1524,    0,
    0,    0,    0,    0,    0,    0,    0,  515,  514,  513,
    0,    0,    0,    0,  496, 1465, 1521,    0,    0, 1529,
    0, 1555, 1546,  670,  674,    0,    0,    0,  518,  516,
  517,  461,    0,    0,    0,  463,  497,    0, 1532,    0,
 1526,    0,    0,  462,    0,    0,    0,    0,    0, 1555,
  464,  465,  495,    0,    0,    0,    0,    0,    0,    0,
    0, 1531,    0, 1533,
};
static int yydgoto[] = {                                      20,
   21,   22,   23,   24, 2449,   26, 1123, 1917, 1486, 1919,
 1487, 1488, 2307, 1489, 1920, 2308, 1490, 1921, 2309, 3352,
 3059, 3370, 3573,   27, 1130, 1131, 1132, 3045, 3362, 3363,
 1492, 1926, 2714, 3062,   28,  116,  856, 2961, 2154, 2603,
 2962, 3292, 3286, 3504, 3681,  661, 1335,   29,  112,   30,
  452,   31,  836,  627, 1295,  567,  113, 3282, 3496, 3674,
  635,  839, 1315, 2136, 3220, 2403,  624, 2906, 3210, 3475,
 1681, 1682, 3476,  630, 1685, 2129, 2557, 2506, 1299, 2559,
 2914,  579, 1656, 2080, 2507, 2845,   32, 1516,  603, 1673,
 2105, 2106, 3643, 3779,  649, 2586, 3279, 3493, 3669, 3797,
 3798, 3799, 3666,  777, 1259, 2428,  620,  621, 1678, 2115,
 2549, 2540, 2895, 3204, 2404, 3607, 3740, 3858,  638, 3481,
 3657,  791,  792, 3659, 1326, 3671, 3803,  841, 2566, 2922,
 2850, 3454, 3635, 4026, 4091, 4116, 2870, 3166, 2841, 2611,
 2117, 3883, 3945, 3990, 4072, 2851, 2852, 3451, 3767, 4022,
 4023, 4024, 4089,  585,  586,  587,  588,  589, 1302, 1655,
 2284, 2495, 2837, 3339, 3340, 2693, 3440,  572,  573,  574,
  575, 3157,  793, 1265,  576, 1298,  611, 2111, 2112, 2113,
  612, 1313,  853, 2149, 2150, 2151, 1647, 1648, 1649, 1650,
 2069, 2070, 2071, 1651, 1652, 2828, 2829, 2830, 3147, 2450,
 2451, 2452, 2453, 2454, 2455, 2456, 2457, 2458, 2459, 2460,
 2461, 3758, 3130, 3131, 2462, 2463, 2464, 3614, 4010, 4049,
 4011, 4050, 3861, 3934, 3984, 2465, 2466, 3128, 2467, 1049,
 1844, 2468, 2469, 2470, 2471, 2805, 3875, 3874, 2818, 2819,
 3621, 3757, 2472, 2473, 2474, 2475, 3433, 2476, 1050, 1441,
   34, 1482, 2281, 2694,  600, 1308,  601, 2101,  640,  641,
 1701, 1304,  804, 2570, 2571, 2929, 2936, 2937, 1274, 2938,
 1667,   35,   90,  564, 1632, 2059, 1660, 2090, 2518,  565,
 3180, 3461, 3640, 3888, 3641, 3777, 2860,  582,  786, 1256,
 1636,  604,  605,  811, 1252, 1631, 3115, 2412, 3116, 3417,
  594, 1275,  845,   36,  137,  865, 1288, 2498, 2499, 2500,
 2501, 1290, 2125, 2126, 1169, 1170, 1171, 2758, 2349, 1547,
 1172, 1173, 1174, 1175, 1176, 1535, 1177, 1532,   37,   38,
   39,   40,   41,   42,  617,   43,   44,   45,   46,   47,
   48,   49, 1178, 1179, 1975, 2365, 2763, 3099,   50,   51,
 1986, 1525,   52, 1139, 1519, 1957, 2343, 2750, 2751, 3091,
 3735, 3397, 3731, 3851, 3389, 3390, 3593, 3391,   53,   54,
   55, 2374,  755,  756,  757,  758,  759, 1577, 1216, 1573,
 2477,  459, 1220,  762, 1583, 1223,  711,   57,   58,  437,
  438, 1432, 1433,  462,  463,  464,  465,  765,  766,  767,
 1232, 1588, 1589, 1590, 1591, 2011, 2379, 1496, 1497, 1498,
 1499, 1500, 2718,  719, 1198, 1501, 1502, 1503, 1504, 3070,
 3071, 1505, 1943,  431,  602, 1941, 2776, 3408, 3409, 3108,
  860, 3413, 3603, 3604, 2271, 1345, 3296, 3297, 3298, 3299,
 3515, 3956, 3959,  771,  772, 1595, 1596,  659, 2389, 2779,
 2161, 2162, 1243, 1610, 2478,   60, 1247, 2395, 2396, 2479,
 3374, 3578, 3579, 4006, 4038, 2480,   63,   64, 2481,   66,
   67, 2482, 3972, 1245,  861, 2483, 1562, 1992, 2484,  807,
  808, 1277,  774, 1249, 2035, 1051, 1052, 2485, 1342, 1053,
 1054, 1055, 1461, 1056, 1871, 1057, 1058, 1059, 1462, 1060,
 1061, 1062, 1063, 1064, 1065, 1066, 1852, 1853,  643,  597,
  781, 1305, 1703, 2141, 2583, 2773, 3489, 3663,  851,  814,
 1691,  868,   69, 1570, 1283, 1284, 3648, 3649, 3782, 2892,
 3197, 2534, 2887, 1343, 1068, 1069, 1604, 1070, 1071, 1072,
 1368, 1370, 1073, 1738, 1074, 2119, 1075, 1076, 1077, 1078,
 1079, 1080, 1081, 1082, 1083, 1832, 1084, 1574, 1336,  636,
 1712,  637, 1961, 1517, 1518, 1085, 1086, 1087, 1088, 1089,
 3321, 1825, 1090, 1091, 2272, 2684, 3332, 3821, 3822, 3915,
 4001, 3542, 2645, 1092, 1093, 1094, 1095, 1096, 1097, 1098,
 1099, 1100, 1101, 1102, 3029, 3537, 3016, 3326, 3031, 3701,
 3829, 3830, 3921, 3922, 1103, 1820, 1104, 1105, 2974, 1106,
 1779, 1107, 1108, 1109, 1110, 1111, 1112, 2978, 3310, 1113,
 1374, 3307, 1375, 2188, 2643, 1114, 1757, 1115, 1116,  577,
 1338,  778,  650, 1117, 2855, 2157,  665,  628,  662, 3868,
  631, 3957, 1199, 2488, 1217,  590, 1927, 1118,  593,  642,
  644, 1438,  678,  453, 1119, 2273, 1120,  435,  140, 1493,
 1124, 2286, 3343, 2705, 1125, 2310, 1491, 2707, 1900, 3371,
  143, 1511, 3581, 3081, 1512, 3046, 3367, 2311, 1332, 1266,
 2957, 2604, 1897, 3033, 3287, 3500, 2600, 2955,  610,  813,
 3283, 2871, 2565, 2919, 2564, 3219, 3221,  822, 2124, 2550,
 3473, 3477,  824, 1657, 2503, 2847, 1688,  795,  812, 1281,
 3463, 3893, 1705, 3670, 2950, 3492, 3668,  820,  783, 2114,
 1619, 3112, 3414, 3855, 3931, 1318, 2137, 3223, 3482, 2509,
 1706, 3495, 2567, 3222, 2923, 3636, 3770, 3637, 3943, 3642,
 3774, 2872, 2873, 3884, 3989, 2510,  800, 1270, 2083,  794,
 2491, 2799, 3122, 3611, 2798, 3121, 3436, 3624, 3125, 3423,
 3747, 3977, 3978, 3133, 3434, 3622, 3866, 2811, 3132,  690,
 1122, 1898,  846, 1323, 2140,  801,  470,  784, 1254, 2053,
 2091, 2874, 3773, 2041, 2790, 1180, 1541, 1527, 1962, 1181,
 1182, 1183, 1184, 1293, 1186,   92, 1134,  818,  699, 2344,
 1556, 1982, 3404, 2334, 2744, 3085, 1568, 1704, 1239, 1233,
  715, 2317, 1721, 2160, 2965, 3684, 3909, 3685, 4030, 2026,
 2029,  467, 2721, 3067, 3576, 3841, 3925, 3970, 3924, 3969,
 4003, 4002, 4075, 4102, 4130, 4099, 4128, 1195, 1988, 2367,
 3926, 1616, 1563, 2368,  775,  847,  782, 1276, 1251, 2132,
 2573, 3277,  852, 1707, 2107,  815, 1675, 1692, 2562, 1676,
 2535, 3468, 3540, 3693, 3539, 1765, 2190, 2631,
};
static int yysindex[] = {                                   9782,
 8321,   17, 5929, 2339,  901, 7612,  736,    0, 1152, 1291,
    0,   17, 2188,138769,139277, 2705,144865, 1183, 1315,    0,
 1748,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1539,  281,    0, 1671,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1578,
 1658,    0,130133,144865,  -62,144865,144865,144865,144865,144865,
144865,144865,144865,144865,144865, 1492,    0,144865, 1455,    0,
    0, 1806, 2283,144865,144865,144865,144865, 2019,144865, 1855,
144865,144865,    0,144865,144865,144865,144865,144865, 2206,    0,
145373,    0, 1816, 2005,144865,    0,144865,144865,144865, 2017,
144865,144865,144865,144865,144865, 1855,144865,144865,144865,144865,
144865,144865,144865, 1752,    0,145881,    0,144865,    0,    0,
144865,144865,    0, 1734,144865, 2037,    0,144865, 1624, 1997,
 1947, 2225,  520, 1844, 2173,  892,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2277,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2224,    0, 2414,    0,    0,144865, 2650,    0, 2684,    0,
144865,144865,144865,144865,144865,144865,144865,144865,144865, 2184,
146389,    0,    0,    0,144865, 6888,    0, 2322, 2367,    0,
  281, 1655,    0,    0,    0, 2396,    0,139785, 2505,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2283,    0,144865,
 2115,    0,    0,    0,  -32,    0,    0,    0,    0,    0,
    0,    0, 1377,    0,    0,    0,    0,  -32,    0,    0,
    0,    0,    0,    0, 1634,    0,    0, 2505,    0,    0,
    0, 2846,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2109,    0,    0, 2524,    0,    0, 2544,    0,
    0,    0,    0,    0, 2742,    0,    0, 2283,    0,    0,
    0, 4256,    0,    0,    0,    0,  860,    0, 2698,    0,
  139,    0,    0,    0,    0,    0,    0,    0,    0, 2701,
    0, 2600,    0,    0,150453,    0,    0,    0, 2711,144865,
    0, 2721,    0, 2620,    0,    0,    0,144865,    0,    0,
    0,    0,    0,   51,    0,    0, 2572,    0,    0,    0,
    0,    0,    0,144865,144865,    0,115709, 3030,    0,    0,
    0,    0, 2595, 1237, 2592,    0, 2687,144865,    0,    0,
 1079, 2662,    0,153521,144865, 2694, 2566,144865,  853, 2828,
    0, 2792,    0,    0,144865, 2650,144865,144865, 2864,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2717,150453,
    0, 2791, 2753,    0,    0,    0, 2853,    0, 2802,    0,
    0,    0, 2825,    0,    0,144865,    0,    0,    0,    0,
    0, 2571,    0,    0, 2837,144865,144865,144865,144865, 2819,
 2703, 2450,  404,  537, 2096,    0,    0, 1655, 2932, 2125,
 -120,    0, 2699, 2495, 3184,    0,    0, 2829,    0,    0,
    0,    0,    0,    0,    0,    0, 2925,  575, 2216, 2500,
    0, 2977, 2985,    0,    0,    0,    0, 2994, 2985,    0,
    0,    0,    0, 2398,    0,144865, 2998,    0,    0,    0,
 3005,    0,    0,    0,144865, 3015,    0,    0,    0,    0,
    0,153595,    0,    0,144865,144865,144865,144865,144865,144865,
144865,144865, 2692,    0,144865,    0,    0,    0, 3017, 1378,
    0, 2564, 2569, 2985, 3042,    0,    0,139785,144865,    0,
    0,144865,    0, 3046,144865,    0,    0,127536,107009, 2940,
 2699,    0, 2983, 2986,    0, 2678,144865,    0,    0,    0,
    0, 3285, 3291, 3292, 2882,    0, 3294,    0, 3296,    0,
 3297,    0,    0, 3298, 3309,116434, 3310,    0, 3311,    0,
 3312,    0,    0,    0,    0,    0,    0,    0,    0, 3315,
 2912, 2914,    0, 3318, 3320,    0,    0,    0,    0, 3324,
    0,    0,107734, 3326, 3328,    0,    0,    0,    0, 3333,
 3027, 3338, 3341, 3356, 3357, 3359, 3361,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 3362, 3364, 3366,    0,    0,    0,    0,
 3367,    0,    0,    0,    0,    0,    0,    0, 3375,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 3379, 3380, 3383, 3385, 3388, 3390, 3392,
 3396,    0,    0,    0,    0,    0, 3398, 3400, 3403, 3406,
 3408, 3415, 3420, 3423, 3425, 3429, 3433, 3440, 3441, 3442,
 3445, 3449,    0,    0,    0,    0, 3452, 3453,    0,    0,
 3454, 3318, 3320,    0,    0, 3455, 3460,    0,    0, 3462,
    0,    0, 3465,    0,    0,    0,    0, 3469, 3473,    0,
    0,    0,    0,121509,121509,126415,144865,    0,    0, 3040,
 3354,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,80883, 3114,    0,    0,
    0,    0,    0, 3470,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 3204,    0, 2898,
    0,    0,    0,    0,    0,    0,    0,    0, 3477,    0,
 3479, 3481, 3483,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3485,    0, 3480, 3484,    0,
144865, 3487,    0,  743,    0,123703, 3161, 3165, 3167,    0,
    0,    0,    0,144865,  903,    0,    0,    0,    0,    0,
    0, 1908, 3498, 3115,    0,    0,    0, 3121,    0, 3155,
 3510,    0, 3211, 3514,  164,    0,    0,    0,    0,  738,
  592,    0,    0, 3125, 3510,    0,    0,    0,    0,    0,
    0,    0,    0, 3282,    0,    0,    0, 3145,    0,    0,
  253, 3523, 3527, 3528, 3529, 3514,    0,    0, 2662, 3509,
 3067, 3070,    0,144865,    0,    0,    0,  412,    0,    0,
 3537,144865,    0,    0,    0,    0,    0,    0,    0,    0,
144865, 1715, 3249,    0, 3538, 3538,    0, 3252,144865,    0,
 3108, 3280,    0,116434,    0,    0,116434,    0,    0,    0,
    0,108459,    0,    0, 2938,116434,  281,    0,  281,116434,
 1698,    0, 3091,108459,    0,  -60, 2263,    0,    0, 3306,
    0,    0, 4158,    0, 2283, 3240,  547,  744,    0,144865,
 2283, 1838,    0, 2283,    0, 3563,    0,    0,    0, 3563,
    0,    0,    0, 1732, 3222,    0, 2406,    0, 3168, 3158,
    0,    0,    0,    0, 3337,    0,  732, 3316,    0,    0,
 3522, 1385, 3529,    0,    0, 2283,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2505,    0,    0,    0,    0,
    0,    0,    0,    0, 2283, 2283,    0, 3574, 2886, 1925,
    0,    0,  153,    0,    0,    0,    0,    0,    0, 3346,
 3256,    0,    0, 3174, 3251,    0,    0,    0,    0, 3259,
126415,    0,80883, 1604, 3255,    0,    0,    0,144865,    0,
 2210,116434, 1482,    0, 3168,144865,116434, 2212, 2244, 2679,
116434, 3168, 2465, 5317,    0,    0, 1855,    0, 1855,    0,
116434,116434,80883,   85,    0,116434,116434,116434, 3317,116434,
116434,116434,116434,116434,102659,101934, 5317,116434,116434, 2475,
116434, 3587,116434,116434,116434,116434,116434, 3592, 3593,116434,
116434,116434,116434,116434,116434,116434,116434,116434,116434,116434,
116434,116434,116434,116434,116434, 3595, 3271, 3271,    0,    0,
 3319, 3599,  724,144865,116434,116434,126415, 2512,    0, 2512,
    0,    0, 3600,    0,    0, 2799, 3596,    0,    0,124286,
    0,116434,116434,121509,    0,    0, 3603, 1584,    0,116434,
    0, 1679,    0,    0,116434, 3208, 3230,    0,    0,    0,
103384,116434, -170,144865,121509,121509,121509,121509,121509,116434,
 3606,124797,116434,116434,116434,116434,109184,136229,144865,115709,
    0,    0, 3266,    0,    0, 3286, 3293, 3295, 5745, 3369,
    0,  569,140293, 3624,136737, 2361,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3625, 3622, 3413, 3365, 3370,
  743,    0,    0,    0, 2353,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  244,    0,    0,    0,    0,
 1855,    0, 3510,  507,    0,    0,    0, 3210, 3212,    0,
    0, 3219,    0,    0,    0, 3288,    0,126967, 3641, 1855,
 1855, 1855, 1855, 1855,    0,    0, 2350, 4639,    0,    0,
    0,  360,144865,144865,    0, 3168,    0,    0,144865,    0,
 3353,    0,    0,    0, 1715,144865, 3360, 3363,144865, 1715,
 3646, 2934,    0,    0, 9143, 9868,    0, 3391, 3653,    0,
    0, 5105, 3656, 3091, 3655,    0,51910,    0,    0,51921,
 3302,144865,    0,  622,    0,    0,    0,    0, 2010,    0,
 3376, 3368, 3371, 3669, 2699,144865, 3198,    0, 4158, 3214,
 3218, -137, 3220,  367,    0, 3284, 3003, 3006,    0,    0,
    0,    0, 1846,    0, 2283,    0, 3281, 3281,    0,    0,
    0,    0, 3110,    0,    0,  103, 3459, 3436,    0,    0,
 3672, 3674,    0, 3693, 3342,    0,    0, 3459,    0, 3694,
 1059, 3378, 3381, 3382,    0, 1855,  139,    0,144865,    0,
 3304, 3151,    0,   94,    0, 3267, 1484,    0,    0, 5823,
    0,    0,  507,  507,    0,    0,    0,    0,    0,    0,
    0,144865, 3409,    0,    0,    0,    0,    0, 3459,144865,
    0,    0,    0,    0,    0, 3724,    0, 1763,144865, 3728,
    0,    0,127536,144865,80883,    0,    0, 3424,    0,    0,
    0, 3274,    0,116434,116434,52635,116434, 3743,116434, 3745,
 3748,10593,116434,116434,116434,116434,81607, 3746, 3470, 3753,
116434,116434,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3486, 3754, 3757,53360,
 3386,116434,    0,116434,    0,59887,60611, 3755,144865,11318,
12043,12768,13493,14218,    0,    0,    0, 7693,109909,    0,
    0,    0,    0,    0,    0,129654, 3759,  973,61335,14943,
116434,116434,52646,    0,15668,62059,16393,56267,56991,    0,
    0,17118,17843,18568,19293,62783,63507,64231,64955,65679,
66403,67127,67851,68575,69299,70023,70747,    0,  813, 3492,
 3494,    0,    0,    0, 3760,    0,    0,    0,    0,    0,
    0, 3768, 3769,53371, 4280, 3771,    0,    0,    0,    0,
144865,129093,    0, 3770,    0, 3772,    0, 1168, 5672, 2773,
126415,    0,    0, 3502,    0, 1104,    0,    0, 3488,121509,
 3603,116434, 3412,116434, 5353,    0,116434,    0,    0,    0,
 3777, 3488, 3488, 3076,    0,    0, 2095, 1764, 1764, 2512,
 2512,80883, 3384,152485,    0,    0, 3787,54094,54810,55544,
    0, 2801,    0,    0, 3793, 3040,    0, 3439,    0,    0,
 3418, 1197, 1512, 3435,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3426, 3427, 3806, 3807,
 3808, 3156,    0,    0,144865,    0,    0, 3168,    0, 3796,
 2654, 3812, 3814, 3495,    0, 3495, 1942,116434, 3495, 3521,
 3532,116434,    0,144865, 3556,    0,    0, 3503, 3505, 3507,
 5745, 3586, 1070,    0,    0,    0,    0, 1826,    0,  608,
 2815, 3463, 3842,    0, 2662, 2820, 3137, 3139, 3140,139785,
 2408,    0,    0,    0,    0, 1855, 3849, 3850, 3851, 3852,
 3855,    0,    0,    0,    0,    0,    0,    0,    0, 2940,
 2699,    0,    0, 3836, 3857,    0, 3860,    0,    0, 1715,
    0,    0, 1715, 1715,    0,    0,    0,    0,    0,123703,
 3471,116434,144865,125817, 3866,116434, 3410,116434,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 4253,129093,
    0,    0,110634,    0, 2840,    0,144865,    0,    0, 3568,
 3569,    0, 3572,    0,    0,  -23, 3496,144865,    0,    0,
  746,  746, 1846, 1892, 1820, 1572,144865,153595,    0,    0,
    0, 2565,144865,144865,144865,128574,    0, 1772,    0, 3436,
    0, 3649, 3652,    0,    0,    0,    0,    0,131657,    0,
  911,    0, 3439,    0,    0, 3466,140801, 3199, 3201,    0,
130133,   67, 1298, 2048,    0, 2985, 3658,    0,    0, 3647,
    0,    0, 3178, 3188,    0,    0, 3512,    0,144865,144865,
 3623, 1484,    0,    0,    0, 2176,    0,    0,    0,    0,
    0,    0,    0,    0, 2129,    0, 3882, 2156,    0,  929,
    0,    0,    0,    0, 3890,    0,    0,    0, 1634,  148,
    0, 2238, 3629,    0, 3512,144865,144865, 1763, 3666, 1791,
    0,    0,    0, 3557,    0, 3667,    0,    0,    0, 3930,
    0, 3108,20018,20743,132165,21468,    0,22193,    0,116434,
    0,22918,23643,24368,25093,117159,    0,    0,25818,26543,
116434,    0,    0,116434,111359,116434,80883, 3670, 3554,    0,
116434,116434,116434,    0,    0,    0,    0,    0,    0,116434,
    0,116434,75815, 3555,116434,116434,116434,116434,    0,71471,
71471,116434,    0,116434,    0,116434,    0,116434,    0,    0,
    0,    0,    0,116434,116434,116434,116434,116434,116434,116434,
116434,116434,116434,116434,116434,    0,    0,    0,    0,116434,
116434,    0,    0,    0,116434,116434,    0,    0, 2851,129093,
144865,    0,121509, 3934, 2870,116434, 3662,    0,    0,    0,
116434, 2806,    0, 3488,116434, 3488, 3488, 3168,121509,116434,
 3372, 3936,    0,    0,144865,116434,144865,    0,112084,    0,
 2871,144865, 3942, 3719,    0,144865,144865,144865,144865,144865,
144865,144865,    0,    0,    0,    0,    0,    0, 3721,    0,
    0, 3722,    0, 2846,    0, 2846,    0,    0,    0,    0,
140293,    0,151469,    0,    0,    0,    0,    0, 3664,    0,
    0,123703,123703, 2876,144865, 3625,    0, 3418, 1256, 1720,
 3673, 3677,    0, 1826, 3710,   81, 1429,    0,    0, 3472,
 3474, 3468,    0,   79, 3956,  608,    0,127536, 3608,    0,
  507,    0,    0,    0,    0,    0, 3565, 3573, 3964,    0,
    0,    0,    0,    0,    0,    0,    0,144865, 3255,    0,
112809,144865,    0, 3676, 3524,    0,    0,    0, 3968, 2654,
116434, 3695,    0,    0,    0,    0,    0, 2272,    0,80883,
  622, 3650, 3770,    0, 2877,    0,80883,144865,    0,    0,
 2283, 2283, 3569,    0,    0, 3617,    0, 2662,    0,    0,
    0, 3706, 3602,    0, 3982, 3749,    0,    0,    0, 3680,
 3687,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3299,144865, 1102,    0, 3992, 3443,116434, 2824,    0,
    0,144865,144865,144865,152993,144865,116434,144865,    0,    0,
    0, 3733,128055,    0, 3979,    0,    0,    0,    0,    0,
    0, 3707,    0,    0,    0,    0,    0,    0,  844,    0,
    0,    0,    0,    0,    0,    0, 3708,    0,    0,    0,
    0,    0,    0,    0, 3984,    0,    0, 3985,    0,    0,
144865,    0,    0,    0, 2891,144865, 3679, 3773,    0,    0,
    0,    0,    0, 3611, 3776,    0,    0,    0, 1059,141309,
    0, 3322,144865, 3327, 2303,    0,    0, 1686,    0, 1045,
    0, 3681, 3682, 3683, 3690, 3697, 3698,    0,    0, 3700,
 3665, 3686, 1631,    0,  230, 4014, 4015,    0,    0,    0,
    0, 2176, 3890, 3890, 2283, 3717,137245,137245,    0,    0,
  507,    0,  507, 3282,    0, 3639,    0,    0,    0, 3570,
    0,    0,    0, 2662,    0, 2897,    0, 2398, 1426, 3222,
 3222, 2313, 2238,    0, 3630,    0, 2283, 3642, 2283, 2283,
 2283, 4051,    0, 3524,141817,    0,    0,    0,    0, 4035,
  166,    0, 4036,  283, 3794,    0,144865,144865,    0,    0,
    0,    0, 4054,153495, 2283,    0, 2699,    0,    0, 3735,
 4059,    0,    0,    0,    0,27268,    0,    0,    0,    0,
126415,27993,    0,    0,28718,76539,77263,    0,    0,116434,
 3675,29443, 2904,72195,30168,30893,116434,116434,72919,77987,
73643,74367, 2905,116434, 4063, 4065,78711,31618,57715,58439,
32343,33068,33793,34518,35243,35968,36693,37418,38143,38868,
39593,40318,41043,41768,79435,80159,    0,    0,    0, 3114,
    0,    0,    0,116434, 3488,121509,116434, 3488,116434, 4066,
    0, 4069, 3838, 3255,    0, 4071, 6126, 3404,    0, 2911,
    0,    0, 3773,    0,131657,    0,    0, 3696, 3703, 3709,
 3711, 3712, 3713, 3714,    0,    0, 1825, 2252, 2501,    0,
    0,    0,    0,  226,142325,    0,144865, 2684,    0,123703,
    0,    0, 2783,    0,    0,    0,    0,144865,144865,144865,
144865,144865,144865,144865,    0,    0,    0,    0,    0, 3609,
    0,    0,    0,    0, 1855,    0,    0, 3774,  879,    0,
    0, 3727,    0, 4085,    0,    0, 3734, 3282, 4088, 3678,
 3688,    0, 4097,144865, 2495,    0, 3274,    0,80883,    0,
 1715, 3493,    0,123703, 3873, 3825,    0,    0,    0, 3780,
    0,110634, 2699,    0,    0,    0,    0,139785,    0, 3819,
    0, 3437, 3438,116434,    0,    0,    0,    0,    0,113534,
    0,116434, 3738,80883,    0,    0,    0,    0,    0, 3827,
    0,    0,80883,    0,    0,    0,    0, 3726,    0,    0,
129093, 1403, 4121,129093,116434,    0, 3773, 4103,    0,    0,
  -38,    0, 3459, 3459,    0,    0,    0, 3773,142833,144865,
  -33, 3820,    0,  903,    0,    0,  507,    0,144865,    0,
    0,    0,132673, 2004,    0,    0,    0,    0,    0,    0,
 3799, 1855, 3803,144865, 3896,  729, 3316,133181, 3644,    0,
    0, 4131,    0,    0, 3758, 3859, 3765, 3884, 3862, 3886,
 3459,    0,    0, 3778,    0,    0,144865, 3543, 3870,144865,
 3870, 3885,    0,    0,    0, 1462,    0, 2662, 3762,    0,
 2283,    0,    0,    0,    0, 3773,    0,    0,    0,    0,
    0,  903,    0,    0,  929,    0, 3512, 2503,    0,    0,
    0,    0,141309, 3222, 1732,  828,    0,    0,    0,    0,
 3832, 3394, 3588, 6099, 9043, 3835,    0,    0,    0, 3756,
    0, 3761,    0,    0,    0,    0,    0, 4171,    0,    0,
 2913,    0,    0,    0,    0, 2915, 4174,    0,    0, 4177,
 1898,    0,    0, 3861,137753,    0,143341,    0,    0,    0,
    0,    0,116434, 4178,116434,    0,116434, 1281,    0,    0,
116434,    0,    0,42493,43218,116434,116434,116434,116434,116434,
    0,80883,    0,    0,116434, 3909,116434,    0,116434,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,144865, 3490,116434,    0, 3114, 3488,
116434, 3488,    0,    0,116434,  811,    0,116434, 3684,144865,
 3891,    0,144865, 4168,    0,144865, 3459,    0,    0,    0,
    0,    0,    0,    0,  460,134705,  460,    0, 3721,    0,
 3722,    0,    0,    0,    0,    0, 3784,    0,   28,144865,
150961,    0,    0,    0,    0,    0,    0,116434, 4190,    0,
    0,144865, 4194, 3931, 3933, 3935, 3937, 3938, 3939, 3940,
    0,    0,  161,    0,144865,    0,    0, 3817, 3817, 1241,
    0,    0, 3281,    0,    0, 3501, 3508,    0,    0,    0,
 3829, 3108,    0,144865, 2654,116434,116434, 3497,    0,    0,
    0,    0,    0,144865,    0, 4214,144865,    0,    0, 4218,
 3563, 3563, 4221,80883,    0, 4223, 3967,    0,129093,    0,
    0, 1627,    0,    0, 3770, 3775, 3707, 3969,    0,116434,
 3770,80883, 1390,    0, 3648,    0, 3978,    0,    0,144865,
 1390,144865,    0,    0, 3643,    0,    0,    0,    0,    0,
    0, 1354, 3896, 3644,    0,  824,    0, 3990,    0,    0,
    0, 3928,    0, 3929,    0,    0,    0,    0,    0,    0,
  704,    0, 4012,116434,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  601,144865,    0, 3870,    0,    0,
 1462, 4004, 3941,    0,    0, 2503,    0,    0,    0,    0,
  135,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 3569,    0,    0,    0,    0,    0,    0,    0,    0,144865,
 3883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 3789,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 3685,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 3689, 2504,    0,    0,    0, 2283,    0,    0,
    0,    0,125308,    0,    0, 3981,126992,153495, 1855,  624,
    0,    0, 2283, 3424,    0,    0, 2920,    0,135213,    0,
144865,    0,80883,    0,43943,80883, 3872,116434,116434, 3988,
44668,    0,    0,45393,46118,46843,80883,59163, 8418, 2291,
    0,47568,48293,    0,116434, 3545,79435, 3488, 3470,    0,
    0, 3610,80883, 3545,    0,116434,    0, 4239, 2930,    0,
 3773,    0,    0,    0,    0,    0,    0,    0,    0, 3983,
    0,   28,144865,144865,144865,144865,144865,144865,150961,150961,
 3585,    0,    0,    0,   28, 3943,142325,    0, 4032,    0,
143849,    0,    0, 2940, 3874,    0,144865,    0,    0,    0,
    0,    0,    0,    0,    0,  202,  202,    0, 4262,    0,
 3998,    0, 3923, 3892, 3637,    0,    0,    0, 3604, 3911,
    0,    0,    0,144865, 4284, 2699,    0, 4282,    0,80883,
    0,144865, 2940,    0,    0,    0,    0,    0,    0, 3952,
    0,116434,117884, 3910,128574, 3770,    0,    0,    0,    0,
116434,116434, 4017,    0,128574,    0, 4292, 5823, 5823,    0,
    0, 4022,    0,    0, 4289,    0, 4013, 4018, 2846,    0,
    0,  798,    0,    0,    0,    0,    0,    0,    0,49018,
    0,    0,    0,    0,    0,    0,    0,135721,    0, 3947,
 2283,    0,  135, 4007,    0,    0,    0,    0, 3569,    0,
    0, 1875,    0,    0,    0,    0,    0,    0, 2936,144357,
 4312, 2283,    0,    0,    0, 3922,    0, 3646, 4318,126992,
 2283, 1855,   77,    0,    0, 4319,    0,    0,    0,137753,
    0,    0,    0,    0,    0,    0, 4315,    0,116434,    0,
 3948,    0,    0,    0,    0,    0,116434,    0,116434,    0,
    0,    0,    0,    0,80883,    0, 4322, 3490,    0,    0,
   63,    0, 4324, 4830,    0,    0,    0,    0,  460,  460,
  460,  460,  460,  460,  460, 3997,134705,    0,    0,    0,
    0,    0,    0,    0,    0,    0,144865,    0,    0,    0,
    0,144865,    0,    0, 3255, 3874,  -21,    0,    0, 2958,
    0,    0,    0,    0,    0,    0,    0,    0,    0,144865,
 1588,144865,    0,    0,    0,    0, 1473,    0,    0,110634,
    0,116434, 4326,    0, 4107,    0,    0,    0,144865,131657,
 3439, 2966,126415,    0,80883,128574,    0,80883,80883,144865,
  291, 1247,    0, 3827, 4073,    0,    0, 4333,144865, 2846,
 2846, 3888, 4081,    0, 4113,    0,    0,    0, 3823,    0,
    0,  -11, 4335,144865,138261,    0,    0,    0,137245,    0,
139785,    0,    0,    0, 3663, 3668,    0,    0,    0,    0,
    0,144865,    0, 4320,    0,    0,    0,    0,    0, 1411,
 4340,  149,116434,    0,116434, 4345, 4033,    0,   46,    0,
    0, 3282,    0, 4352,144865,144865,    0,116434,80883,49743,
    0, 3545,104109,104109, 3739,    0, 3691,    0,    0,116434,
 2118,144865,   28,   28,   28,   28,   28,   28,   28,    0,
    0,    0,    0,    0, 2699,    0,    0, 3833,    0,    0,
  202,  202,  202,  202,  202,  202,  202,    0,    0,    0,
   25,    0,    0,    0,    0, 3016,    0,144865, 4358,    0,
 1431,    0, 3773,    0,    0,50468,    0, 4098, 3827,128574,
    0,    0,144865,144865,  608, 3955,    0, 4100, 4139,    0,
128574, 3168,    0, 4019, 3888, 1816,    0,    0, 3779,    0,
    0,  959,    0,  -11, 3766, 3611,    0,    0,    0,    0,
    0,    0,    0,    0, 3899,    0,    0,    0,144357,    0,
 2283,    0, 3848, 1148, 1731, 3839,    0,    0,    0,    0,
 4135,    0,    0,    0,51193,    0, 2283,    0, 3970,    0,
    0,    0,    0,    0,    0, 4380,104834, 3785, 3720, 3085,
    0,    0,    0,    0,    0,80883,    0,    0, 2384,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4151,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,152485, 1431, 3890, 4160,    0,  -33, 4391,
    0,    0,128574,    0,    0, 1247,    0,    0,    0,    0,
    0,128574, 4145,    0,    0,    0,    0, 4400,144865,    0,
 4115,    0, 3773,    0, 3773, 4147,    0,    0,    0,    0,
 3776,    0,144865,144865,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  149,  887,    0,    0,    0,144865, 4117,
    0, 3797, 3736, 5379, 4188,    0,    0,    0,    0,    0,
 4413,116434, -230,    0,    0, 3893, 4415,    0, 2662, 4084,
 2662,    0,    0,    0, 1755,    0,    0,    0,144865,    0,
 2846,144865,    0,    0,    0, 4179,    0,    0,    0,  507,
    0,    0,    0, 3953, 3959, 3022,    0, 4425,    0,    0,
    0,    0,    0,105559,    0,80883,    0,    0,    0,   95,
    0, 2940,    0, 2503,    0,    0,    0, 1755, 4043, 4161,
116434, 2139,116434,    0,    0, 4110,    0, 4435, 4179,    0,
    0,    0,    0,    0,144865,    0,    0, 3830, 3791,129674,
    0,    0,  842,116434, 1383,    0, 3184, 3255,    0, 3033,
 3045,118609,    0,116434,119334,80883, 4061, 4093, 4067,80883,
 1497, 3888, 4118, 4120,    0,    0,    0, 1921,    0,144865,
    0,    0,    0,    0,    0,116434, 4159,    0, 4072,    0,
 4090,    0, 3274,    0,    0,    0,    0,151977,    0,    0,
80883, 3887,    0,80883, 3894,80883,    0,    0,    0, 1847,
 1847,    0, 4196,    0, 4068,    0,    0, 4163,144865, 3888,
    0,    0,    0, 3060,   95, 4094,    0, 1498,    0, 2699,
146897,116434, 3902,116434,116434, 4048, 1948,    0,    0,    0,
 4461, 1606, 3888, 2846,    0,    0,    0, 4183,    0,    0,
 4114,    0,    0,    0,    0,116434,80883,80883,    0,    0,
    0,    0, 1606, 4077, 3776,    0,    0, 4110,    0, 2495,
    0, 4473,80883,    0,  903,  507, 3888,    0, 4091,    0,
    0,    0,    0, 2495, 4479, 4473, 4105,110634, 4480, 3089,
110634,    0, 3100,    0,
};
static int yyrindex[] = {                                    572,
    0, 1019, 2082,    0,    0,    0,    0,    0,    0,    0,
    0,  609,    0,    0,    0,    0,    0,    0,    0,    0,
 4530,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  214,    0,    0,  221,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  403,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4204,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  403,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,100154,    0,    0,    0,  593,    0, 4270,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  222,114259,
    0,  287,    0,    0,    0,    0,  793,    0, 1164,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 4273,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 4273,    0,    0,
    0,    0,    0,    0,  317,    0,    0, 1164,    0,    0,
    0, 4275,    0,    0,    0, 2063,    0,    0,    0,    0,
    0,    0, 4273,    0,    0, 2181,    0,    0,    0,    0,
    0,    0,    0,    0,153653,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1453,    0, 4500,    0,
 4277,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 4279,    0,    0,    0,    0,    0,
    0,    0,    0, 1460,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2340,    0,    0, 2343,    0,    0,    0,
    0,    0,    0,    0, 2381,    0, 1127,    0, 6349,    0,
133689,    0,    0,134197,147405,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  927,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4279,
    0, 2383,    0,    0,    0,    0,    0,    0,147913,    0,
    0,    0,148421,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  239,
    0, 1776,    0,    0,114984,106284,    0,  204,    0,  976,
  365,    0, 2390,  955,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1111,    0,    0,    0,
    0,    0, 2417,    0,    0,    0,    0,    0, 2417,    0,
    0,    0,    0,  330,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2449,    0,
    0,    0,    0, 2417,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1014,
 2390,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,62904,    0,
    0,    0,    0,    0,    0,    0,    0,    0,63628,    0,
    0,    0,    0,    0,    0,    0,    0, 8539,    0,    0,
    0,    0,    0, 7090, 7814,    0,    0,    0,    0,56388,
    0,    0,    0,64352,65076,    0,    0,    0,    0,65800,
66524,    0,    0,    0,    0,    0,    0, 9264, 9989,10714,
11439,12164,12889,13614,14339,15064,15789,16514,17239,17964,
18689,19414,20139,57112,57836,67248,20864,21589,22314,58560,
67972,23039,23764,24489,25214,25939,26664,27389,59284,28114,
28839,29564,30289,31014,31739,32464,33189,33914,34639,35364,
36089,36814,37539,38264,38989,39714,40439,41164,    0,    0,
41889,42614,43339,68696,69420,70144,70868,71592,72316,73040,
73764,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,44064,44789,45514,46239,74488,60008,46964,47689,
60732, 7090, 7814,48414,49139,75212,75936,49864,    0,    0,
    0,    0,    0,50589,51314,52039,52764,61456,62180,53489,
54214,54939,55664,    0,    0, 4146,    0,    0,    0, 2421,
79556,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  692,    0,80280,    0,
    0,    0,    0, 1181,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,76660,77384,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,78108,
    0, 1282,    0, 3405,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1973,    0,    0,    0,
    0, 2424, 2624, 3783,    0,    0,    0, 3841,    0,    0,
 4558,    0,    0, 4594,    0,    0,    0,    0,    0, 4126,
 4671,    0,    0, 4079, 4558,    0,    0,    0,    0,    0,
    0,    0,    0, 5030,    0,    0,    0,    0,    0,    0,
 4820, 4960,    0, 4172,  890, 4594,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 4215,    0,    0,    0,
  227,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  263,    0,    0,    0,    0,    0,
    0,    0,  241,    0,    0,    0,  572,    0,    0,    0,
    0, 1423,    0,    0,    0, 2446,    0,    0,    0,    0,
    0,    0, 4283,    0,    0, 4152,  294, 4283,    0,  461,
    0,    0,    0,    0,  337,    0,    0,    0,  572,    0,
 1098,    0,    0,    0,    0, 1567, 4671, 1316,    0,    0,
 3227, 5289,    0,  308,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1164,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 4283, 1665, 1288,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4146,    0, 2885, 4510,  259,    0,    0,    0,    0,    0,
120059,    0,120059,    0,  572,    0,    0,120059,120059,96208,
 4511,  572,120059,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4122,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,120059,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 4146,82452,    0,83176,
    0,    0,    0,75091,129968,    0,    0,    0,78832,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,120784,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4511,    0,    0,    0,    0,    0,    0, 1127,
    0,    0, 3875,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 4146, 4219,    0,    0,    0,    0,
    0,    0,    0,    0,    0,98316,    0,    0,    0,    0,
 3405,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4558,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2456,    0,    0,    0,
    0, 1460,    0,    0,    0,  572,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  248,    0,    0,    0,    0,    0,    0,    0, 1617,    0,
    0,100814,    0, 4514,  915,    0, 1566,    0,    0,  320,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2485,    0,    0,    0, 2390,    0,    0, 1288,    0, 4220,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1404,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 4291, 4296,    0,
    0,    0,    0,148929,    0,    0,    0,    0,    0, 2486,
    0,  220,  314,  359,    0,    0, 1314, 1288,    0,    0,
 2488,    0,    0,    0,    0,    0, 4265,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1356,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 4267,    0,127001,
    0,    0,    0,    0,    0,    0,    0, 4534,    0,    0,
    0,  251,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3109,    0, 4536,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 5359,
 5476, 5742, 5860,127138,127161,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,82331,    0,81004,    0,    0,
    0,    0,    0, 2541,    0,    0, 2392,    0,96932,    0,
 4146,    0,    0,    0,    0,    0,    0,    0,93312,    0,
    0,    0,    0,    0,97656,    0,    0,    0,    0,    0,
    0,88244,88968,    0,    0,    0,81728,85348,86072,83900,
84624,  950,    0,  382,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2421,    0, 4314, 4979, 4609,
149437,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 4212, 5827,
 4293,    0,    0,    0,    0,    0,    0,  572,    0,    0,
 4219, 2040, 2422, 4246,    0, 4246, 4219,    0, 4246,    0,
    0,    0,98976,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2722,   78,    0,    0,
    0, 4998,    0,    0,    0,    0,    0,    0,    0,    0,
 2568,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,101332,    0,    0,    0,    0,    0,    0,    0, 1014,
 2390,    0,    0,    0,    0,    0,    0,  970,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1208,    0,    0,    0,    0,    0,  206,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1097,    0,149945,    0, 4317,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  224,    0,    0,130641,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2417, 2570,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 4281,    0,  536, 2131,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2239,    0,    0,    0,
  264,    0,    0,    0, 4136,    0,    0,    0, 2532,    0,
 1799, 4101,  718,    0,    0,    0,    0, 4267,    0, 4286,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  275,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1438,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4544,
 4544,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,94036,    0,89692,94760,  572,    0,    0,
  233,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  994, 5537,
    0, 1036,    0, 1028,    0, 1028,    0,    0,    0,    0,
    0,    0, 3335,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,98316,    0,149437,    0,    0,
    0,    0,    0,   78,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 4185,    0,    0,    0,    0, 3945,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  209,    0,  259,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  285, 1557,
    0, 1912,    0,    0,    0,    0,    0,    0,    0,  693,
    0,    0, 2605,    0,    0,    0, 3123,    0,    0,    0,
    0,    0, 2090,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2844,    0, 2881, 2612,    0,    0,    0, 2937,
 3081, 3117,    0,    0,    0, 3206,    0,    0,    0,    0,
    0,    0, 1213,    0,    0,    0,    0,    0,    0,    0,
    0, 4531,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2807,    0,
    0,    0, 4283, 4283,    0,    0,  998,  687,  339,    0,
    0,    0,  294,  316,  630,    0,    0,    0,    0, 1509,
    0,    0,    0,  804, 1673,    0,    0,    0,    0, 4082,
 1735,  483,  496,  532,  628,  773,  990,    0, 4283,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4136, 4136,    0,    0,    0,    0,    0, 1475,
    0,    0,    0, 3195,  308,    0,    0,    0,    0, 1618,
 1288,    0,    0,    0,    0,    0,    0,    0,    0,   12,
 4227,    0, 4101,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4273,    0,    0, 4273,    0,    0,    0,    0,    0,    0,
    0,    0, 4202,    0,    0,    0, 2390,    0,    0, 4559,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4146,    0,    0,    0,    0, 4560,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  169,    0,    0,    0,    0,86796,
    0,    0,    0,    0,90416,    0,    0,95484,    0,    0,
    0,    0,    0,  196,    0,    0,  178, 4268,    0,    0,
    0,    0, 4542,    0,    0,    0, 4283,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  340,    0,    0,    0, 3152,    0,    0,
    0,    0, 4219,    0,    0,99636,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 5204,    0,    0,
    0,    0, 2615,    0, 4180,    0,  251,    0, 1694,    0,
    0,    0,    0,    0,    0, 1265,    0,    0,    0,    0,
    0,    0, 2390,    0,    0,    0,    0,    0,    0, 2279,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4547,
    0,    0,    0, 4549,    0,    0,    0,    0,    0, 4550,
    0,    0, 4551,    0,    0,    0,    0, 4308,    0,    0,
    0,  572,    0,    0,    0,    0, 1477,    0,    0,    0,
 4323,131149,    0,    0,    0,    0,    0, 1693,    0,    0,
  310,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 4574,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2633,    0, 4301,    0,
 4301,    0,    0,    0, 1103,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2636,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1380, 1424,    0,
    0,    0, 1509,   12,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  923,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1243,
    0,    0,    0,    0,    0,    0,153486,    0,    0,    0,
 1233,    0,    0,    0, 4510,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 4313,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 4577,    0,    0,    0, 4002,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  177,    0,    0,87520,91140,
    0,91864,    0,    0,    0,  185,    0,    0,  187,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  392,    0,  392,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2639,    0,
  438,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2640,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2639,    0,    0,    0,    0, 1063, 1063, 2785,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  275,    0,    0, 1663,    0,    0, 1210,    0,    0,
    0,    0,    0,    0,    0, 1562,    0,    0,    0,    0,
 4152, 1275,    0, 4562,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1058,    0,    0,    0,    0,    0,
 4563, 4564, 4566,    0,    0,    0,    0,    0,    0,    0,
 3124,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2368, 3813,    0, 4350,    0, 4044,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2055,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 4301,    0,    0, 4301,    0,    0,
    0,    0,    0,    0,    0, 2172,    0,    0,    0, 1561,
    0,    0,  264,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  207,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   38,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  897,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,122681,123192,    0,    0,    0,    0,    0,
    0,    0, 4588,    0,    0, 4213,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3144,    0,    0,    0,
    0,    0,    0,    0,    0, 4595,  169,92588,  345,    0,
    0, 4598,  183, 4595,    0,    0,    0,    0,    0,    0,
 3146,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2639,    0,    0,    0,    0,    0,    0,  438, 1040,
  301,    0,    0,    0, 2639,    0,    0,    0,    0,    0,
    0,    0,    0, 1101,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2639, 2639,    0, 3020,    0,
 3037,    0, 4185,    0,    0,    0,    0,    0,    0, 3099,
    0,    0,    0,    0,    0, 2390,    0, 2305,    0, 2223,
    0,    0, 2093, 2588,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  907, 1597,    0,    0,    0,    0,
    0,    0,    0,    0,  907,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 4129,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2648,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2652, 1319,    0,    0, 2675,    0,  402,    0,    0,
    0,    0,  292,    0,    0,    0,    0,    0,    0, 4510,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  188,    0,    0,  177,    0,    0,
    0,    0,    0,  380,    0,    0,    0,    0,  392,  392,
  392,  392,  392,  392,  392,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  297,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  750,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1873,    0,    0,    0,    0,    0,    0,    0,
 1589,    0, 4146,    0,  953,  907,    0, 4327, 4334,    0,
    0,    0,    0, 4383,    0,    0,    0,    0,    0,    0,
    0, 2462,    0,    0,    0,    0,    0,    0,    0,    0,
 1094,    0, 2693,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2696,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1577,    0,    0,    0,    0,    0,    0, 3157,    0,
    0, 4595,    0,    0,    0,    0,    0,    0,    0,    0,
  191,    0, 2639, 2639, 2639, 2639, 2639, 2639, 2639,    0,
    0,    0,    0,    0, 2390,    0,    0,    0,    0,    0,
 2639, 2639, 2639, 2639, 2639, 2639, 2639,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2772,
    0,    0,  208,    0,    0,    0, 1610,122170, 4550,  907,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4351,
  907,  572,    0,    0, 2462, 4204,    0,    0,    0, 4748,
 4748,    0, 1108,    0,    0, 1214,    0,    0,    0,    0,
    0,    0,    0,    0, 1172,    0,    0,  207,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  331,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  409,    0,    0, 4611,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  983,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  382,    0, 4136, 4208,    0, 1600,    0,
    0,    0,  907,    0,    0,    0,    0,    0,    0,    0,
    0,  907,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2522,    0, 2522,    0,    0,    0,    0,    0,
  630,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3182,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  606,    0,    0,    0, 4210,
    0,    0,    0, 1750,    0,    0,    0,    0,    0,    0,
 1222,    0,    0,    0, 5088, 2187,    0,    0, 1239,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 4616,    0,    0,    0,    0,
    0,  655,    0, 4137,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1092,    0, 5472, 2187,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  332,    0,    0,
    0,    0,    0,    0,    0, 2149,    0,    0,    0, 2329,
    0, 2462, 3289, 3788,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  303,    0,    0,    0,    0,127189,    0,    0,
 3187,    0,129817, 2530,129860, 2729,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 5603,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2390,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1619,    0, 5603, 2594,    0,    0,    0,    0,  209,    0,
    0,    0,    0,    0,    0,    0, 3207, 2857,    0,    0,
    0,    0,    0,    0,  630,    0,    0, 4042,    0, 4180,
    0,  822, 3215,    0,    0,    0, 5603,  209,    0,    0,
    0,    0,    0, 4180,    0, 1142,    0,    0,    0,    0,
    0,    0,    0,    0,
};
static int yycindex[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  -64,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2554,
    0,    0,    0,    0,    0,    0, 4618,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1343,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 3921,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4016,    0,    0,   49,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3925,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 4181,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4620,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 4624,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4619,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 4621,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4372,    0,    0,    0,    0, 4374,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 4375,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4630,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 4632,    0,    0,    0,
    0,    0,    0, 4633, 4634,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 4636,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1481,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 4379,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4638,    0,    0,    0,    0,    0,    0,    0, 4376,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1263,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  -29,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 4290,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3949,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 4381,    0,    0, 4384,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4646,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  107,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 4302,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1704,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 4652,    0,
    0,    0, 4386, 4408,    0,    0,    0,    0, 4339,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4303,
 1428,    0,    0,    0,    0,    0,    0,    0, 4411,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4666,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 4402,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 4414,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  114,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4668,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4116,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2089, 2151,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 4156,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 4119, 4124,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4673,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 4321,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4678,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2359,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 4462,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1422,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2531,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1531,    0,    0,
    0,    0, 4682,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,
};
static int yygindex[] = {                                      0,
    0,    0,    0,    0, 4731,    0,    0, 2782, 3225,    0,
 3229, 3231, 2036, 3236,    0, 2041, 3239,    0, 2045,-1544,
    0,  801,    0,    0,    0,    0,    0,-2437, 1199, 1206,
 3258, 2453, 2031, 1408,    0,    0,    0,-2205,    0, 2175,
 1502,    0,    0, 1295,    0,    0, 3078,    0,    0,    0,
    0,    0,    0, 4355,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1576,    0,    0,    0, 1327,
-1866, 1635,-2008,  962, 2247,    0, 1889,-1935, 3973, 1593,
    0, 4723, 2307,    0,    0, 1035,    0,-2696, 4707,    0,
    0,    0,    0,  922,    0,    0,    0,    0, 1151,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2285,    0,    0, 1620,-2235,    0,    0,  975, 4392, 1037,
    0,-1914, -577,    0, 3626,    0,  932,    0, 2253, 1621,
 1916,    0,    0,    0,    0,  733,-1653,    0,-2532,    0,
 -369, 1071,    0,  859,    0, 1926,-2028,    0,-1969,  737,
  826,  834,  790,  131,-1885,-1870, -145,    0,    0,-1217,
-1984,    0, 1710,    0, 1161,-2331, 1714,  102,-1835,-1819,
  157, 1007, -546, -652,    0,    0,   97,    0, 3164,  295,
    0,    0, 4143, 2728,    0, 2719,-1126,    0,    0, 3234,
    0,    0, 2808,-1390,-1284,    0,    0,    0,    0,-2320,
-1496,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,-2531, 1261, 2058, 2064,    0,    0,    0,  806,
    0,  812,    0,    0,  910,    0,    0,    0,    0,-1183,
-1689,    0,    0,    0,    0, 2447,    0, 1140,    0, 2083,
    0, 1039,    0,    0,    0,    0,    0, 4903, 3428, 3010,
    0,    0,    0, 1879,   99,    0,    0,    0, -155,    0,
    0,    0,-1140,    0, 2345,    0, 1640,    0,-1785, 3645,
-1475,    0,    0,    0,    0, 2863,    0, 2409,    0, 2826,
    0,    0,    0,    0, 1032, 1149,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3307,    0, 2875,    0, 1322,
    0,    0, 3265,    0,    0,    0,-1948,-2525,    0,    0,
 -665,    0, 3250, 2382,-2430, -782,    0,    0,    0,-2390,
    0,    0, -777,    0,    0, 3750,    0, -860,    0,    0,
    0,    0,    0,    0, 4793,    0,    0,    0,    0,    0,
    0,    0,    0, 3393,    0,    0,    0, 1535,    0,    0,
    0,    0,    0,    0,    0,    0, 2606,    0, 1549,    0,
    0, 1856,    0,    0,    0, 1360,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 3730, 4193,-1421,
    7,    0,    0,    0,-1993,    0, 3740,-1026,    0, 4519,
 4240,-1657,    0, 4495, 1739,    0,    0,    0,    0, 4195,
    0, 3723,    0, 2951,    0,    0,    0,-1346,-1952, 3475,
 3482,    0,    0, 2246,    0, 3476,    0,    0,    0,    0,
    0,    0, 2642,  329,   92, 3028,    0,    0, 1367,    0,
-1263,    0,    0, 1235,    0,-1947, 3634,    0, 1464,    0,
    0,  954,    0, -758,    0,    0, 2964,-1564,    0,    0,
 4216,-2612, 3751, 3389, 4988,    0,-2642,-3369, 2208, 4992,
    0, 1417, 1418,  924,    0, 4997,    0,    0, 5002,    0,
    0, 5003,    0, -848, 3444, 2557,-1842, 2641,    0,-2252,
-1841,-1181,-2316, 1000,    0,-1319, -756,-1376,-1349,-1022,
    0,    0,    0,    0,    0,    0,    0,    0, 3559,    0,
    0,    0,    0,    0,    0,    0, 3147,    0, 4565,    0,
    0,    0,-1445,    0, 2443, 2436,    0, 1234, 4182, 4209,
 3339, 4166,    0, 3458, -666, -513,    0, 1389,    0,    0,
-1497,-1946,    0, 2553, -899, -905, 2999,    0,-1234,-1619,
 4021, 4024,    0, 3564, -967,-1050, 3615, 3447, 3448,-1200,
-1206, -732,-1605,-1594,-1584,    0,  -87,  -79,-1903, -842,
    0, -117,-1508,-1506, -687,    0,-1966,    0,    0,    0,
    0,    0,    0,    0, 1184,    0,    0, 1346,    0,    0,
    0,    0, 2833,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,-3050,-2717, 1507,    0,    0,
    0,    0,    0,    0,    0, 3635,    0,    0,    0,    0,
    0,-1964,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1743,    0,-2102,    0,    0,-1005,-1582,    0, -110,
 2706, -830, -128,   69, 2209,-2307,    0,  -70, 4934,    0,
   35,  -76,-1226, -445, -568, -108,  -59,    6,    8,  -55,
  -74,-1786, 1702, -243,   64,-1438,  -14,    0,    0, 3120,
    0, 2733,  273,   87,    0,    0, 3576,  820,    0,    0,
    0,    0,  479, 2328,    0,-1858, 1717,    0,    0, 3737,
    0,-1439,-1394,-2511,    0,    0,    0,    0,  101, -440,
    0, 1782,    0,    0,-2400,-3005,    0,    0,    0,-1530,
    0, 1042,    0, -768,-1932,-2707,    0,    0,    0,-1244,
    0,    0,    0,-1627,    0,    0,    0,    0, -694, 1657,
-1448,    0,-2326,    0,    0,    0, 2931,    0, -545, 1419,
    0,    0, 1582, 1294,    0,    0, 1461,    0,    0,-2708,
-2140,    0,    0, 1215, 1113, 2250,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,-2196,
-1189,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 3853,    0,
    0,    0,    0,    0,    0,    0, 3957,    0,    0,    0,
    0,    0,    0, -525,    0, 5096,    0,    0,    0, -680,
    0,-1484,    0,    0,    0, 2022,    0, -803,    0, 2183,
-1436, 1359,    0,    0,    0,    0, 1304,    0,    0,-1472,
    0,    0,    0, 2050,    0,    0, 1118,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1154,    0,    0,    0,    0, -383,    0,    0,    0,
    0, 1466,    0,    0,-1531,    0,    0,    0,    0,    0,
    0,    0,    0, 1591,    0,    0,    0,    0,
};
#define YYTABLESIZE 154399
static int yytable[] = {                                     434,
  440,  583,  454,  568,  682,  622,   56,  598,  674, 1235,
  798,  670, 1346,  623, 1142, 1337,  673, 1325, 1137, 1431,
  439,  433, 1761, 1434,  606,  595, 1267, 1966, 1768,  687,
 1225, 1228, 2017, 1885, 1607,  816, 1674, 1581, 1168, 1291,
 1606,  799, 2369, 1324, 1292, 2417,  647,  672,  667, 1278,
 2765, 2712, 1659, 2713,  677, 1294, 2345, 1603,  566,  569,
 2118,  578,  581,  584,  591,  434,  596,  599,  434,  607,
  581,  680,  676,  614, 2120, 1226, 1229, 2143, 1436,  626,
  569,  578,  581, 2628,  591, 2121,  434,  596, 1253,  599,
  434,  607,  581,  651,  844, 2122,  614, 2123, 2508, 2486,
  663, 2487,  434,  666,  569,  432,  663,  578,  581,  584,
  591,  580,  434,  596,  599,  679,  607,  581,  651,  580,
 1845,  614, 3101,  689,  433, 1286,  434,  434, 2061, 2062,
  581, 2407, 2817,  700,  675, 2808, 2809, 2810, 1429, 1429,
 1435, 1658, 2898, 2899, 1428, 1430, 1269, 3159, 1931,  433,
 2067,  613, 2249, 2001, 3102, 2013, 1289,  671, 2006, 2827,
 3034, 2082,  625, 2910, 2838, 3185,  681, 2787, 2607, 3035,
  869,  592, 2038, 2963,  613, 2145, 2127, 2127, 1185,  697,
 2051, 2052, 1702, 2831, 3036, 1329, 1994, 1213, 1215, 2422,
 1605, 1321,  639, 2254, 2558, 1699, 2555,  629, 2588,  686,
 3472,  688, 2138,  645, 3058, 1475,  416,  656,  432, 2013,
  654,  732,  850, 1326,  805, 3212, 2612, 2015, 2009,  833,
 1615, 1330, 2098, 2010, 2893, 1884, 1332, 2013, 2016,  633,
 3736, 2021,  692,  432,   79,  684, 1466, 1822, 1328, 2036,
 1489, 1858,  867, 1893, 1466, 3351, 1475, 1329, 1569, 1475,
 1481, 2406, 1671,  570, 1326, 1271, 1843, 2072, 1466, 2078,
 1492, 2592, 1330,  303, 1475,  416,  474, 1332, 3047, 3058,
 2155, 2891, 1326, 1882, 1332,  846, 2593,  789,  833, 1328,
 1330, 1489,  532, 3543, 1377, 1332, 1466, 1960, 1329, 3073,
  725, 3368, 1550,  310, 3143, 1874, 1466, 1328, 1990, 1489,
 2392,  820, 1481, 1485, 1545, 3151, 1329,  292,   25, 1481,
 2589, 3334,  846,  831, 1431,  315,  772, 1466, 1434, 1479,
  461,  846,  303, 3150, 3145, 1377, 2590, 1466, 1730, 1065,
 3155, 1466,  187, 1332, 3751, 1740,  846, 1996,  820,  103,
 2393, 2894, 3718, 1377, 2392, 1466,  863,  820, 2740, 3679,
 3695, 1607,  310,  869, 2572, 1466, 4004, 1606,  832, 2392,
 1479, 1481, 2043, 2073, 1485, 1612,  292, 3094,  550, 2722,
  100,  189,  831, 3209,  315,  772, 3967,  139, 1479, 1097,
 1787, 1788, 1097, 1240, 3696, 1883, 1762, 1763, 1065, 3775,
 1466,  101, 3808, 1739, 2103,  846, 2522, 1097,  103, 1097,
 1431,  252, 1193, 1989, 1836, 1485, 1272, 2795,  820,  802,
 2415, 1198, 2748, 3849, 2189,    0, 1273,  832, 2064,  820,
 2019,  440, 1436, 1485, 3368, 3697,  569,  578,  581,  591,
  434,  596,  599,  434,  581, 1435,  614, 2139, 1536,  842,
  454,  439, 2754, 1224, 2755,  664, 3288, 1618,  789, 2020,
  101, 2715, 1565,  779,   91, 1564, 1520, 2282,    0,  787,
  252, 1193, 1278, 3850, 1700, 3368, 1048, 2749, 1930, 1475,
  693,  474, 1668, 1607, 1848,  789, 3462, 1326,  803, 1606,
  785, 2139,  838,  833, 1198, 1330, 2486,  532, 2487, 2065,
 1332, 2523, 2888,   81, 2104,  839, 2139, 2366, 3474, 3776,
 1537, 1557, 1328, 3351, 1489, 1739, 2325, 1987, 1608, 1892,
 2044, 1329, 4005, 2741,  613, 2569, 3801, 1475, 1764,  529,
  833, 1435, 1845, 3809, 1758, 1686, 1759, 2976, 3680,  833,
 3968,  835, 1875,  532, 1241,  388,  639, 1475, 1429, 1997,
  823,  838, 3719,  721, 1850, 1326,  727,  731, 1377,  857,
 1466, 1965, 1489, 1330,  839,  578, 2392,  862, 1332, 1429,
 1429, 1429, 1429, 1429, 2392, 1877, 1878, 1879, 1880, 1881,
 1328,  474, 1489,  550,  723,  772, 1227,  831, 2376, 1329,
  772, 2377, 2378, 1479, 3146,  772, 1722,  833, 2320, 3750,
  835, 1679,  789, 1065,  388,  187, 1377, 3156, 1466, 1048,
  846,  833, 1097,  103, 1882, 1547, 1377, 1377, 1191, 3538,
 2680, 1340, 1924, 1140,  831, 1097, 1377,  772, 1466,  550,
 1492, 1136,  832,  831, 3369,  864,  772,  834, 1485,  103,
 1065, 1479, 1342, 1097,  189, 1568,  141,  846, 2496, 1065,
  614, 3816, 1475, 3386, 3387,  578,  846, 1492, 1097, 2260,
 1326, 1479, 1334,  578, 1438,  101, 1377, 1492, 1843,  832,
 3205, 2046, 3057, 2380, 1547, 1377, 2542, 1191,  832,  870,
  614, 1209, 1964, 1303,  299, 1328, 1485, 1489, 1613, 2072,
  460,  831, 1549,  584, 1329, 1475,  834,  571, 1824, 1959,
 1187, 1766, 1480, 1190,   27,  831, 1485, 1571,  772, 1466,
  440, 2139,  440, 1200, 3341,  790, 1466, 1377, 1475, 1475,
 1845, 1065, 2592, 1438,  790,  614, 1883,  340,  854, 1377,
 1196, 1377,  439, 1466,  529, 1661,  832, 2593, 1475,   81,
 1481,  599, 1766, 1480, 2009, 1766, 1882, 2392, 1466, 2010,
  832, 1214, 1214, 1214, 1214,  474,  838, 2382,  871,  846,
 1766, 1480,  622, 1436, 1296, 1306, 1479, 3369, 2601,  839,
 1317, 2589, 1466, 1489, 1998, 1485, 2034, 3066, 4140, 1569,
  529, 4143,  837, 1481,  290,  820,  340, 2590, 1331, 1893,
 2204, 1466, 2925,  838, 1766, 2073, 2384, 1377, 1377, 1466,
 1466,  626,  838,  854,  859,  835,  839, 1332, 3369, 1479,
  870, 1485, 3698, 2391,  290,  839, 1993, 1377, 3548, 1466,
  569,  578,  581,  591,  596,  599,  434,  581, 2838, 1466,
  614, 1525, 1607, 2095, 1431, 1481,  315, 1956, 1606, 2889,
  369,  837,  835,  779,  434, 1466,  252,  870,  474, 1958,
  584,  835, 1479, 1339, 2759, 1603, 1843, 2391, 1883, 1141,
  838,  290,  870, 1211, 1466,  529, 2963, 2963, 1466, 1993,
 1475, 1300, 2391,  839,  838,  106, 1300, 1485, 2021, 1690,
 1337, 2021,  141, 1884, 2139,  369,  843,  839, 2715, 2890,
 1525, 2764, 1168, 2255, 1466, 1436,  790, 1485, 2081, 1212,
 1513,  834,  417,  417, 1475,  290, 3350, 2392,  613,  835,
 2142, 1930, 2013,  790, 3167, 3549, 3550, 3551, 3552, 3553,
 3554, 3555, 2099,  835, 1468,  290, 1466, 2130, 2780, 2131,
 2682, 1882, 1689, 1568, 2013, 3196, 1475, 1297,  834, 1307,
 2669, 1312, 2015, 2009, 1311, 1492,  103,  834, 2010, 1377,
 1327, 1466, 2013, 2016,  417,  290, 1340, 1515, 1212, 1767,
  474, 1637, 1141, 1882, 1429, 1468, 1480,  833, 1301,  290,
 2262,  532, 2791, 2970, 1475, 3205, 1341,  315, 4009, 1097,
 3216, 1340, 1326, 1468, 2324, 1257, 2723, 1547, 1605, 3215,
 1330, 3213, 1141, 2865, 2116, 1332, 1547, 1923,  101,  836,
 1767, 1994, 1342, 1767, 1497,  834,  474, 1328, 2568, 1489,
 3179, 1191, 1483, 1639, 1480, 2410, 1329, 2002, 1767,  834,
 2005,  665, 1480, 1438, 2047, 2099, 2207, 1506, 1191, 2907,
  790, 2784, 1185, 3177, 1480, 2134, 1438, 1191, 1542, 1827,
 2917, 1377, 1439, 1883, 3702,  369,  837,   99, 2140, 2314,
 1828, 2140, 1767, 1377, 2127, 1466, 2909,  550,  836, 1484,
  474,  831, 1437,  634,  772, 1571, 2140, 2019, 2140, 2019,
 2019, 2019, 1270, 1766, 3889, 1883, 2668, 1065,  648, 2963,
 3633, 1563, 1438,  837,  846,  299, 1097, 1191, 1479,  100,
  340, 3503,  837, 2314, 3743, 1766, 2020, 3608, 2020, 2020,
 2020,  508, 3759, 1011, 2930, 2931,  832,  325, 2314,  705,
 1438, 3507,  389, 4036, 3617, 3227,  591, 1014, 3406, 1547,
 1026,  689, 1191, 1924, 3623, 2916,  663, 1337, 1680,  454,
 2096, 1270,  138, 1485,  622, 2024,  741,  837, 1547, 1480,
 3508, 3135,  508, 1829, 3141,  508, 1340, 1011, 1274, 2356,
  837, 1527, 2370, 3721, 3722, 3723, 3724, 3725, 3726, 3727,
  508, 1014, 1011, 3450,  837, 2822,  325, 3088, 1438, 1438,
  822,  389, 2411, 1653,  340,  340, 1014,  622, 1538, 1026,
 1641,  393, 1480,  622, 1538, 1634,  622, 1438, 1468,  434,
  742, 1643, 3954, 3950, 1653,  741,  775,  434, 1274, 1507,
  618,  619, 1568, 1525, 2375, 1766,  870,  822,  529, 3165,
 1527,  340, 1525,  709,  626, 3520,  822, 1429, 1687, 1431,
 3859, 1470,  838,  315,  117, 1480,  665, 3864,  700, 1568,
  838, 1563, 1653,  775, 2633, 3228, 1468,  622,  622, 1568,
  393, 3886,  775,  839, 3988, 1693, 1694, 1593, 1015,  742,
  139, 1431,  343, 2787, 1522, 1642, 1468,  340, 1429, 1593,
 1431, 3464, 1340,  836, 2391, 1525, 2391,  315, 2869, 4037,
 2252, 2140, 1563,  706, 1427, 1563, 1429,  822, 1431,  835,
 2236, 2237,  315,  177, 2140, 2025,  177, 1956,  822, 2823,
 1563, 3741, 1015,   99,  665, 1561, 2206, 1655,  340, 3072,
  836, 3810, 2140, 1327, 3503, 3748, 1608, 1015, 1638,  836,
 3089,  343, 1297, 2314,  775, 1427, 1340, 2140, 1212, 1993,
  710, 2690, 3749, 1066, 3330,  285, 2085,  474,  344, 2086,
  665, 1767, 3280, 1427, 2865,  100, 1341, 1258, 2561, 3634,
 1270, 1192,  665,  529, 1723, 1876, 3148, 3149, 2584, 1539,
 3160,  581,  700, 1767, 1525, 2866, 1655, 1429, 1141, 3103,
 3944, 1468, 3947, 2670, 1497,  265,  663,  836, 4092, 2087,
 3178, 2140, 2560, 1429, 1830,  834, 2833, 2834, 1895, 2681,
 3341,  836, 1066, 1514,  285, 2099, 3211,  344, 3452,  409,
 1766, 1497, 1291, 2333, 3192, 1438, 1506, 1292, 1485,  665,
 1731, 1497, 2424, 3199, 1468, 1563, 1270,  378,  323,  611,
 1522, 2257, 2881,  874,  722, 2238,  665, 4126,  700, 1833,
  728, 1191, 1766, 1963,  265, 3229, 3687, 1468, 1468, 2541,
 1653, 2504,  858,  267,  474, 1847, 3575, 3105, 4131, 3865,
 4080, 1831, 1977, 1978, 1979, 1980, 1981, 1468,  409, 3426,
 3877, 3497,  378, 2528, 3120, 1846, 3201, 2258, 3226,  651,
 3439, 1480, 3126, 1767, 1270, 3929, 1295, 1886, 2486, 1429,
 2487, 3955,  874,  626,  578, 3606,  393, 1026, 2486, 1270,
 2487, 1429, 1438, 1431,  275, 1684, 2314, 4129,  607, 1539,
  689,  858,  267,  141, 1653, 1539, 1540,  622,  138, 1653,
 3887, 4137,  622, 1438,  460, 1270, 1340, 1640,  323, 1289,
 2868, 1845, 3365, 1337, 1845, 1337, 3399, 1568,  455, 3752,
  837,  393, 2287, 1527, 3455,  612, 2556, 1438, 1429, 1429,
  741, 1431, 1527, 1728,  700,  618,  619, 3458, 1427, 1429,
 1429, 3331,  700,  275, 1509,  474, 1438,  474,  529, 1429,
 3342, 1431, 3936, 1653, 1295,  751, 1894,  622, 1847, 1200,
 3862, 3938, 3753, 1270,  870, 2060, 1378, 1128, 1507, 1653,
  276, 1214, 2505, 3754, 1214, 1472,  375, 3658, 1995, 1468,
 3037, 2729, 2239, 1270,  742, 1527, 1427, 1066,  508, 1429,
 2505, 1431, 3308, 2288, 3401, 3814, 1270, 2023, 1429, 1563,
 1431,  285,  285, 2524, 1431, 2801, 1427, 1378, 1434, 2869,
 1378, 1847,  378, 1468,  460,  336, 1472, 3632, 1295, 1472,
 2767, 1993, 1340, 1295,  315, 1378, 1367, 1144,  455,  276,
 1144, 1995,  142, 1438, 1472,  375, 3744,  177, 2140, 2415,
 1429, 3985, 1431,  796,  822, 1468, 1427, 1843, 1767,  460,
 1843, 1993, 2730,   13, 1429, 1427, 1431,  532,  700, 2486,
 1653, 2487, 3494, 2255,  626,  658, 3755, 3400,  550, 1546,
  775,  876, 1379, 2752, 1527, 3935, 3447, 1295,  676,  529,
 1767,  393, 1020, 1468,  455, 1367,  751,  870, 4040,  285,
  378,  346,  456, 1295, 3438,  434, 1129, 1427, 3986,  343,
  343, 1993,  378, 1559,  434,  265, 1993, 3465, 1339, 2158,
 3467, 1427,  797, 1379,  876, 3899, 1379, 2139, 1563, 3136,
 1429, 1429, 1431, 1431, 3309, 1435, 1020, 1316, 3376,  409,
 2789, 1379, 2525,  378,  315,  378,  336,  836,  132, 3900,
 1429, 1020, 1431,  474,  260, 2140,  474, 1559, 3448, 2078,
 2078, 1230, 1601,   16, 2088, 2089,  993,  994, 2259,  460,
  273,  275, 1559,  474,  581, 1602, 3202, 3411, 2966, 1653,
 1048, 1653, 1653,  268,   58,  344,  344, 1427, 1427, 2584,
 1429,  474,  336, 3793, 1295, 1335, 3019, 3449, 3856, 1727,
 4012, 1138, 4120, 2486,  500, 2487, 1686, 1427, 3764, 3765,
 3437, 2915, 4060, 4100, 2486, 3880, 2487, 2742, 1656, 2109,
 2152, 2402,  346, 2194,  876, 1468,  457, 4041,  677, 1845,
 1469, 4134, 1653,  336,  175, 2289, 1335, 2290, 2118, 2118,
 1378,  500,  876,  378,  618,  619,  626, 1847,  132, 1472,
 2291, 1429, 2120, 2120, 1335, 2326,  278,  276,  273,   15,
 1429, 4118, 3857, 2121, 2121, 2926, 2896, 1846,  346, 1144,
  458, 3870,  532, 2122, 2122, 2123, 2123, 1656, 2359,  876,
  658,  375, 1429,  550, 1431, 2312, 1680, 2743, 1378, 1886,
 2110, 2078, 1432,  660, 2731, 1057, 2918, 1472, 1378, 1378,
 1144, 1854,   70, 1295, 4067, 1295, 1295, 3756, 1378,  346,
  802, 1415,   61,  500, 3794,  658, 2486, 1472, 2487,  260,
  260, 1506, 3086, 3420, 3421, 2486,  375, 2487, 2400, 2248,
  607, 1422,  622, 1432,  369,  622,  622, 3991, 3733, 1427,
 1144, 4061, 4101,  772,  278, 3026, 1379, 1855, 1378,  591,
 1367, 1432,  500,  772, 3111, 2421, 1295, 1378, 1716, 1429,
 3795,  676,  178, 3734,  369, 1843,  369, 1856, 1367, 1860,
 4113,   73, 1422, 2430, 1429,  779, 2431, 3113, 1717,  803,
 3991, 3203, 1445, 1429, 2537,  466, 1429,  369, 1431, 2927,
 1422, 1144, 1241, 3605, 1379, 3729, 2928,  660,  468, 1378,
 1057, 3796, 1429,   76, 1379, 1379, 2861,  474,  875,  660,
 3730, 1378,  369, 1378, 1379,  689, 2806, 1415, 1200,  626,
 2806, 2594, 1472, 1861, 3703, 3704, 3705, 3706, 3707, 3708,
 3709, 1427, 1718,  678, 2110, 1847,  798, 1593, 3979, 4071,
  369, 2292,  651, 1427, 4008, 2732,  769, 1662, 1144, 1463,
 1862, 1241,  772, 2409, 1379, 1846,  378,  378, 2733, 1335,
 1464,   70,  626, 1379, 1465, 1472,  660,  799, 2429,  578,
  591, 2489, 4062, 1472, 1007, 1658, 2885, 3329, 1572, 1378,
 1378, 1663,  322,  660,  626, 1559,  375, 2862, 1472, 1472,
 1291, 1995,  626, 1507, 1559, 1292, 2521,  770, 2146, 1378,
  657,  677, 1894,  469, 2886, 1379, 2863, 2064, 1472, 1038,
 1456,  609, 1438, 2335,  578,  591, 3117, 1379, 1007, 1379,
 4095, 3498, 2418, 2054, 2864, 3427, 2597, 1335,  378,  378,
   73,  378,  658, 1007, 2419, 3428,  911,  678,  378, 1658,
 2301,  322, 3980, 4117, 1658, 2423,  616, 1559, 1559,  678,
  376,  578,  591, 1438, 3981, 3429, 1432,  893,  894,  895,
  896,  897,   76,  899,  618,  619, 2055, 2502, 1038, 2147,
  626, 1438,  695,  911, 2303, 2515, 1664, 4133, 2065, 2520,
 1719,  618,  619,  901,  902, 1379, 1379, 2415,  906,  907,
 3290,  409, 2551, 1559, 2416, 1422, 2699, 2598, 1658, 2700,
 1195,  615,  109, 2420, 1432, 1379,  678, 3168, 2336,  376,
 2110, 3982,  112, 4114, 1658, 3580, 2415, 1559,  616, 2551,
 2490, 3881, 3181,  678, 1432,  290,  460,  684, 3983, 4086,
 3206, 1378,  115, 1506, 1506, 2304, 1559, 2338, 3644, 2702,
 1472, 2552, 1426, 1422, 2305, 4052,  993,  994, 1857, 2734,
  409, 1607, 1607, 2613, 1422, 1847,  626, 1606, 1606, 1195,
 1198, 3291, 4110, 1422, 1432, 2110, 2056, 4087, 2554, 2306,
 3430, 3431, 3432, 1432, 1472, 1846, 1934, 4115, 2339, 4122,
 2686, 4083, 2688, 1426, 2057, 2536, 1426,  626, 1935,  290,
  460,  434,  578,  591,  614,  569,  581,  581, 1054,  632,
 1863, 1426, 1110, 1422, 3474, 1658, 1472, 4136, 3901, 2110,
 1936, 4111, 1422, 2698, 4052, 1432,  607, 4088,  440, 3605,
  634, 2058, 2591, 1378, 1423, 4068, 1198,  689,  689, 1432,
  440, 4069, 3902,  622,  622, 1378, 2340, 1379, 2719, 1894,
  660, 2785, 2786, 3804, 1472, 2703, 2704,  118,  121,  124,
 2725, 1110,  668, 1339, 1422, 1198, 2341, 1054, 1939, 1092,
  993,  994, 1087, 2526, 4070, 1423,  248, 1038, 1422, 2342,
 2695,  683, 2701, 1847, 1385, 1230, 1438,  626,  168, 1038,
 1463, 1198, 2800, 1423, 1198,  698, 1385, 2248, 1521, 1522,
 1523, 1464,  701, 1995, 1385, 1432, 1432, 2527,  248, 1198,
 2306, 2706, 1184, 1847, 1237, 1507, 1507,  685, 1385, 1550,
 1038, 2110, 1038, 1231, 1658, 1432, 1658, 1658, 1092, 1385,
  703, 1087,  168, 1995, 1438, 1198, 2539,  471, 2248, 1379,
  248, 2248,  684, 2727, 1422, 1422,  555,  168,  569, 2737,
  744, 1379, 1238, 1233, 1438,  376, 2248, 2807, 2158, 2158,
 2158, 2807, 2110, 1847, 1422, 2110, 1385, 2124, 2489, 2306,
 2770, 1184, 2539, 3992, 2560, 1028, 1670, 1658, 1550, 1669,
 2110,  151, 2110, 1995, 3612, 1229, 2949,  622, 1995,  622,
  622,  506,  500, 1198, 1438, 2900,  409, 2902, 2904, 1230,
  376, 1230, 2903, 3490, 1524,  555, 2832,  154,  378,  744,
  684,  626, 1233, 1198, 1551,  852, 1426,  759, 2027,  622,
 1038,  622,  622,  622, 2574,  626, 1198, 2941, 2856, 2943,
 2944, 2945,  506, 1230, 1028,  506,  145, 1724,  378, 1733,
  378,  409, 3049,  151, 1229, 1438,  684,  622, 2028,  409,
  506,  474, 1644, 2543,  702, 2964, 1278, 1432,  684, 1438,
  168,  378,  599, 2905, 1426, 1645, 1054, 3050,  707,  154,
  745, 1735,  708, 1551,  852, 1426,  759, 1385, 1054,  409,
  500,  409, 3051, 2544, 1426, 2545,  378, 3601, 2575, 2695,
 3655, 3656,  474, 2576, 2695,  474, 1422, 1216, 1423, 1063,
  651,  500,  409, 2932,  168, 2933, 2546,  500, 2853, 1054,
  474, 1054,  578,  591,  378,  684, 1431,  399,  683,  168,
 1434,  704,  685, 1563, 1426, 1438, 1438,  409, 2934,  745,
  146, 2547,  684, 1426, 1552, 3545,  712,  147, 1662, 1697,
  500, 1506,  790, 2935, 1220, 1438, 1423, 2577, 3075, 4132,
 1278, 3076, 1698, 1133,  713,  409, 1216, 1423, 1063, 2548,
  376, 1432, 1710, 2578, 1563,  474, 1423, 1563,  105,  127,
 3063,  148, 1663, 2248, 2248, 1426,  399,  269,  114,  115,
 2248,  357, 1563,  149, 4039, 2110, 1272, 2262, 1422, 1426,
  685, 3078, 2248, 1552, 1133, 1506, 1273, 1133, 1993, 2262,
 1422,  409, 2262, 1220,  255, 1934, 1423,  714, 1993, 1054,
  626,  150, 1133,  622, 1133, 1423, 4076, 1935, 3170, 1614,
 2110, 1710,  329,  717,  474,  430,  685,  105,  127,   15,
  599, 3171,  440, 1038, 1038,  689,  269, 1435,  685, 1936,
  357, 3154, 2752,  434,  578,  591,  614,  569,  581,  581,
 1937, 1242, 3064,  718, 1230, 1426, 1426, 1423, 1938,  618,
  619, 2248, 4017,  255, 1230, 3074, 1380, 1664, 1459, 1458,
 1460, 1423, 4018, 4019, 3173, 1426,  760, 1438, 1380, 3100,
 2777,  329, 2248, 2858,  430, 2859, 1380, 1939, 2248,  689,
 2778, 3053, 1741, 2502, 2932,  685, 2933, 3079, 3080, 3531,
 1380,  400, 1791,  779,  729, 1038, 1038, 1463, 1038, 3532,
 1242, 1380,  685, 1507, 1264, 1038, 3054,  682, 1464, 2934,
  772, 2248, 1465,  683, 3077, 1954,  652, 3827, 1141, 3828,
  772, 3055, 2248,  761, 2935,  653, 1847, 1423, 1423, 1847,
 1983, 1984, 1985,  622, 1468, 1466, 3715, 1467, 1380, 1469,
 3785, 3208,  773, 1955,  626,  651, 1846, 1423, 3139, 1846,
  400, 1260, 1261, 2579, 2856, 2580, 2581, 1507,  626, 1840,
  151, 2278, 1470, 1264, 1470,  399,  790, 1468, 1466,  626,
 1467, 1438, 1469,  626, 4081, 2347, 4082,  399, 2346, 1993,
 2352,  683,  152, 2351, 2120,  788, 3839, 2110, 1940, 1563,
 1563,  153, 3194, 2357, 2358, 3198, 1563, 1426, 3217, 3218,
 2399, 1133,  399, 2398, 1580,  806, 2582, 1133,  399, 2257,
  399, 2667, 1054, 1054, 2250,  263,  263,  683, 1133, 1133,
 1133, 2257,  650, 1133, 2257,  780, 1133, 3153,  626,  683,
 2672, 2692, 1337, 1470, 2691,  681, 2724, 2783, 1133, 1470,
 2782, 3169, 2248,  154, 1133, 1580, 2258,  399, 1580, 1380,
  817, 2836, 3175, 1133, 2835,  397, 3182, 2921, 2258,  636,
 2920, 2258,  155, 1580, 2980, 2991,  156, 1470, 2990, 1444,
  434, 3032,  626, 3281, 1470, 3284, 2920, 1563, 2958, 1423,
 3511, 1563, 1445, 3510, 1054, 1054,  683, 1054, 1934, 1426,
 3547, 3364, 3300, 3546, 1054, 1598, 3661, 1599, 1563, 3660,
 1935, 1426, 2295,  683, 1563, 3372, 3373, 1133,  399, 1244,
  819, 2853,  682, 1993, 2295,  635, 1446, 2295, 3720,  397,
 3324, 1669, 1936, 1447, 1993,  821, 3745,  840, 1133, 1470,
  441, 1448,  848, 1937, 1133, 3335,  849, 1563,  626, 1258,
  442,  626, 1133,  443, 1133, 2425, 2426, 2427, 1449,  400,
 1450,  599,    4,    7, 2253, 3302, 1282, 1133, 1463,  855,
 1451,  400, 1133, 1453, 1454,  599,  599, 1133,  858, 1464,
 1939, 1423, 1264, 1465,  859, 1244, 3852,  440,  866, 2782,
  682, 3919, 3996, 1423, 3920, 3995,  400, 2676, 3626, 3627,
  434, 1463,  400, 4045,  400, 1121, 4044, 3378, 1258, 1133,
 1563, 1126, 1464, 1133, 1244, 4047, 1465,  399, 4046, 3407,
 1456,  444, 3392, 1189, 3802, 1282,  682, 1934, 1266, 3415,
 4096, 2695,  434, 3995, 2695, 1135, 1993, 1141,  682, 1935,
 1244,  400, 1188, 1244, 1847, 1993, 2708, 2709, 1264,  402,
  681,  445, 1193, 1194, 3418,  446, 2141, 1201, 1244, 4142,
  447, 1936, 2782, 1202, 1846,  626,  448,  626, 2141,  702,
 4144, 2141, 1937, 2782, 1459, 1458, 1460, 1264, 1580, 1766,
 3068, 1393, 1766, 1205, 1244, 1468, 1466, 1266, 1467, 1204,
 1469, 1940, 2146, 1502,  541,  682, 1502,  541, 1206, 1212,
 1210, 3506,  400, 1264, 2146,  705, 1264, 2146, 1133, 1939,
 1207, 3466,  682,  402, 2079, 1219,  474, 2079,  681,  474,
  622, 1264, 1393, 1221, 1144, 1393, 1580, 2080, 3491, 1580,
 2080,  399,  399, 1208, 1236,  622, 1580, 1580, 1222, 1246,
 1393, 1580, 1244, 3509,  449,  434, 1580, 1264, 2695, 1580,
 3445, 1244, 1463, 1248,  681, 1463, 1112,  675, 1255, 1250,
  675, 4103, 1244, 1580, 1262, 1144,  681, 3483, 1144, 1067,
 1263, 3974, 3559, 3976, 1580, 1244, 3561,  671, 1264, 4043,
  671, 2307, 1580, 1144, 1133,  672, 1580, 1268,  672,  691,
  450, 1279,  694, 2307,  705, 1580, 2307, 1112,  626, 3069,
 1112,  400, 1280,  399,  399, 1264,  399,  148, 1285, 3560,
 1457, 1580, 3563,  399,  434, 1112,  626, 1258,  509,  149,
 1940, 1310, 3063,  681, 2318, 1264, 3574, 2321, 1314, 3565,
 3562, 1319, 1580, 1320, 1282, 1322, 3516, 1580, 1264, 1330,
  681, 3344, 3345, 3346, 3347, 3348, 3349,  150, 1580, 1580,
 1344, 1580, 1347,  451, 1351, 1348, 3802, 1349, 1354,  509,
 1352, 1353,  509, 1355, 1390, 1356, 1357, 1358,  578,  599,
  591,  596,  599,  679,  607,  614, 1443,  509, 1359, 1361,
 1362, 1363,  599, 1258, 1364, 1444,  599, 1367, 1365, 1369,
 1366, 1133,  626, 1371, 1580, 1376, 1266, 1377, 1445, 3517,
 1282, 1580, 1378, 1379, 1390, 1390, 1602, 1380, 1390, 1463,
 1381, 1048, 1258,  622, 3678,  400,  400, 1580, 1580, 3100,
 1464, 3652, 1580, 1390, 1465, 1382, 1383, 1886, 1384, 1282,
 1385, 1386, 1446, 1387,  622, 1388, 1390, 1580, 1258, 1447,
 2489, 1258, 3667,  622, 1391, 1393, 2299, 1448, 1392, 1393,
 2489, 3677, 1394, 3566, 1395, 1282, 1258, 1396, 1282, 1397,
 1995, 1398, 1266, 1440, 1449, 1399, 1450, 1400, 1360, 1401,
 1995, 2300, 1402, 1282, 1442, 1403, 1451, 1404, 1452, 1453,
 1454, 2301, 1258, 3650, 1405, 1455, 2302,  400,  400, 1406,
  400, 1266, 1407, 1393, 1408, 1373, 1393,  400, 1409, 1282,
 1144, 1144, 1410, 1393, 1393, 3664,  151, 1144, 1393, 1411,
 1412, 1413, 3364, 1393, 1414, 2303, 1393, 1266, 1415, 1144,
 1266, 1416, 1417, 1418, 1421,  434, 1456, 3714,  152, 1422,
 1393, 1423, 1112, 1112, 1424, 1266, 1112,  153, 1425, 1112,
 1258, 1393, 1426, 1470, 1471, 1472, 1473, 3300, 1474, 1393,
 1475, 1112, 1476, 1393, 1477, 1478, 1481, 1282, 1508, 1479,
 1258, 1266, 1393, 1725, 1509, 1729, 1510, 1526, 1530, 1580,
 1734, 1736,  599, 1258, 1528, 1742, 2304, 1282, 1393, 1531,
 1529, 1533, 3713, 1534, 1544, 2305, 1546,  599, 1144,  154,
 1282, 3783, 1551, 1548,  509,  509, 1552, 1553, 1554, 1558,
 1559,  509, 1792, 1560, 1393,  434, 1566,  434,  155, 1144,
 2306, 1575,  156, 1576, 1580, 1144, 1393, 1582, 1393, 1266,
 1112,  709, 1241, 1609,  434,  626, 1635, 3392, 1067, 3732,
 1617, 2489, 1654, 1666,   15, 2158, 1677, 1672, 3812, 1266,
 1680, 1112, 1683, 1695,  626, 1708, 3418, 1112, 1144, 1711,
 1696, 1995, 1266, 1709, 1713, 1714,  769, 1794, 1769, 3780,
 3650, 1580, 1800, 1801, 3784, 1818,  779, 1819, 1393, 1826,
 1837, 1841, 1851, 1580, 1866, 1883, 1390, 3790, 1867, 1390,
 1112, 1899,  509, 1901, 1393, 1393, 1390, 1390, 1918, 1393,
 1902, 1390, 1903, 1928, 1942,   73, 1390, 1944, 3231, 1390,
  584,  626, 1945,  509, 1393, 1967, 1946, 1968, 2502,  509,
 1976, 1947, 3867, 1390, 1969, 2000, 1457,  626, 1970, 1669,
   73, 2007, 2003, 3232, 1390, 2004, 2012, 3763, 2016, 3233,
   73, 2014, 1390,  622, 2010,   73, 1390, 2030, 2033, 3234,
 2037, 3897,  509, 2040, 2031, 1390, 2042, 2032, 2045,  622,
 2048, 2049, 3235, 1886, 2050, 2063, 1959, 3907, 2066, 2068,
 2075, 1390, 2076, 2077,   73, 2489, 2079, 2084,  569, 3869,
 2100,  893,  894, 2019, 2020,  897, 2489,  899, 2092, 1144,
 2102, 2093, 2094, 2108, 3813, 1995, 2135, 1390,  893,  894,
  895,  896,  897, 2144,  899, 3952, 1995, 2153, 3878, 1390,
 2695, 1390,  906,  907, 3664,  509, 1585, 1240, 2159, 1586,
 3236, 1112, 1158, 2167, 1592, 2169, 2177,  510, 1597,  906,
  907, 2170, 1600, 2178, 2182,   73, 1592, 2183, 2193, 2181,
 2242, 3918, 2205, 2186,   73, 2240, 1393, 2241, 2243, 2244,
 3237, 2247, 2194, 2250, 3238, 2256, 2268, 2251, 2270, 3239,
 1448, 1390, 1158, 1158, 2269, 3240, 1158, 2274,  510,   73,
 2265,  510, 2279, 2283, 2285, 2293, 2313, 1390, 1390, 1886,
 1159, 1158, 1390, 1158, 2294, 2295,  510, 3241, 2489, 2296,
 2297, 2298, 2314, 2194, 2315, 2322, 2194, 2489, 3582, 3583,
 3584, 3585, 3586, 3587,  434, 3987, 2323, 2316, 1995, 2327,
 2328, 2194, 2329, 1158, 2330, 2332, 2348, 1995, 3780,  651,
 1159, 1159, 2350, 3388, 1159, 2353, 3941, 2354, 2355, 2360,
 2361, 2362, 2363, 1715,  584, 2364, 2371, 2388, 1393, 1159,
 2373, 1159, 2372, 3242, 1726, 2381, 2386, 2401, 2402, 1732,
 1393, 2405, 2493, 1737, 2408, 2494, 2531, 2516, 4029, 2517,
 2511, 2529, 2530, 1760, 2158, 2538, 2532,  584, 1766, 1767,
 2533, 1159, 1770, 1771, 1772, 1773, 1774, 1778, 1786, 2553,
 1789, 1790, 2585, 1793, 1139, 1795, 1796, 1797, 1798, 1799,
 2563, 2602, 1802, 1803, 1804, 1805, 1806, 1807, 1808, 1809,
 1810, 1811, 1812, 1813, 1814, 1815, 1816, 1817, 2596, 2606,
 2605, 2630, 2629, 2638, 2671, 2674, 2685, 1834, 1835, 1715,
  584, 2696, 2697, 2710, 2711, 1139, 2735, 2739, 1139, 1390,
 2736, 2683, 2747, 3243, 1067, 1849, 2753, 4053, 2720, 2745,
 2746, 2760, 1859, 1139, 2762, 1139, 2757, 1865, 2771, 2761,
 2772, 2774, 3244, 1872, 1873,  584, 2775, 2788, 2781, 2792,
 2793, 2794, 1882, 2796, 2415, 1067, 1888, 1889, 1890, 1067,
 2797, 2802, 1067, 2023,   89, 2815, 2803, 2820, 2821, 2824,
 1158,  508, 2826, 1095, 2825, 2839, 1158, 2844, 2846, 2840,
 2883, 2875, 2876, 2877,  434, 1158, 2854, 1158, 1158, 1158,
 2878, 2857, 1158,  510,  510, 1158, 2023, 2879, 2880, 2882,
  510, 2884, 2893, 2894, 2901, 2912, 4094, 1158, 1163, 2940,
 2913, 1390,  508, 1158, 1095,  508, 1158, 1095, 2194, 2194,
 2946, 2942, 1158, 2952, 2953, 2194, 2954, 2958, 1159, 2968,
  508, 2967, 1095, 2993, 1159, 2994, 3023, 2194, 2977, 3024,
 3025, 3027, 3030, 1159, 3038, 1159, 1159, 1159, 1163, 1163,
 1159, 3039, 1163, 1159, 3087, 3092, 1158, 3040, 3095, 3041,
 3042, 3043, 3044, 3090, 3084, 1159,  409, 1163, 3093, 1163,
 3098, 1159, 3104, 3096, 1159, 3106, 1158, 3107, 3109,   87,
 1159,  510, 3558, 3097, 3114, 3118, 3119, 3127, 3129, 2816,
 3140, 3144, 3172, 3158,   87, 3568, 3174, 1158, 3176, 1163,
 3184, 1153,  510, 1158,   87, 3183, 2194, 1158,  510, 3189,
   87, 3191, 3186, 1158, 1159,   87, 3588, 3589, 3187, 3188,
   87, 3190, 3195,   87, 3196,  409, 3193, 2194, 3200, 3230,
 3245, 1158, 1139, 2194, 1159, 3274, 1158, 3207, 1139, 3275,
 3278,  510, 1153, 3285, 3276, 1153, 3289, 3293, 3304, 1139,
 1139, 1139, 3320, 3325, 1139, 1159, 3336, 1139, 3338, 3377,
 1153, 1159, 1153, 3366, 1158, 1159, 2194, 3351, 1158, 1139,
 3015, 1159,   71, 3412, 3379, 1139, 3380, 3396, 3381, 3402,
 3382, 3383, 3384, 3385, 1139, 3405, 3403, 3416, 3419, 1159,
 3422,   87, 1153, 3424, 1159, 3425, 3442, 3435, 2801, 2865,
 3441, 3456, 3457, 3446,  510, 3459, 2163, 2164, 3470, 2166,
 3471, 2168, 3485, 3484, 3501, 2172, 2173, 2174, 2175, 3519,
 3522,   87, 1159, 2179, 2180, 3486, 1159, 1601, 3541, 3487,
 3536, 1048, 3567, 3556, 3571, 3590,   87, 3577, 1139, 3591,
 1602, 2749, 3569, 3596, 2187, 1048, 3597,  508,  508, 1095,
 1095, 3598, 3595, 3600,  508, 3602, 1095, 3616,   87, 1139,
 3620, 2203, 3625, 3628, 3629, 1139, 1163, 1158, 1095, 1459,
 1458, 1460, 1163, 2210, 2211, 1139, 3610, 3651, 3630,  285,
 3654, 1163, 3665, 3631, 1163, 1163, 3673, 3675, 1163, 3682,
 3686, 1163, 3691, 1139, 3699, 3688, 3710, 2194, 1139, 3738,
 3739, 3761, 3762, 1163, 3766, 3768, 3769, 3772, 3778, 1163,
 3791, 3786, 1163, 3800,   87, 3806, 3787,   87, 1163, 3807,
  285, 3811, 3824,  285,  409, 1159, 3840, 3854, 3872, 3863,
 1139, 3871, 3873, 1067, 3825,  508,  409, 1095,  285, 3879,
 3882, 3894, 3905, 1158, 2264, 1620, 2266, 3891, 3898, 2267,
 3911, 3908, 1163, 3923, 3903, 3930,  508, 1621, 1095, 1153,
 3933,  409,  508, 3939, 1095, 1153, 3917,  409, 3916,  409,
 3940, 3948, 1163, 3942, 1153, 3958, 1153, 1153, 1153, 3964,
 3960, 1153, 3961, 3965, 1153, 3973, 3993, 3971, 1622, 3975,
 1623, 2849, 3994, 1163, 3997,  508, 1153, 1095, 4014, 1163,
 4015, 1159, 1153, 1163, 4025, 1153,  409, 4021, 4058, 1163,
 4057, 1153, 4032, 4031,  409, 4063, 4059, 4065, 4077, 4078,
 4079, 4060, 4061,   87, 1067, 4084, 4109, 1163, 4093, 1139,
 4085, 4112, 1163, 3832, 3833, 3834, 3835, 3836, 3837, 3838,
 4106, 4098, 4119, 4125, 4121, 1153, 1989, 4135, 4138, 4141,
 1158, 3842, 3843, 3844, 3845, 3846, 3847, 3848,  508,    1,
 1163, 4139, 1472, 1393, 1163, 1153,  424,  409, 1563,  439,
 1066, 1443, 2306,  875, 1131, 1340,  529, 1321,  409, 1443,
 1444, 1764,  579, 1415, 1489, 2077, 1153, 1178, 1624, 1037,
  585,  825, 1153, 1445, 1592,  580, 1153,  561, 1597,  570,
 2390,  826, 1153, 1448,  827, 1139, 1765,  532, 1159,   74,
 1420,  372,   95,  560, 1908, 2397,  586, 1295,  571,  626,
 1153, 1198,  902, 1169,  846, 1153,  170, 1446, 1178, 2140,
 2041, 1178,  474, 2017, 1447,  647, 1497,  692,  656,  637,
  613,  605, 1448,  471, 1707, 2070, 1178, 1909, 1178, 1871,
  648,  693, 1554, 1153,  541,  285,  285, 1153, 2042, 1449,
 2076, 1450,  285, 1163, 1169, 2011,  409, 1169, 1903,  455,
  737, 1451,  828, 1452, 1453, 1454,  656,  738, 1178,  707,
 1455, 2025, 1169,  406, 1169,  407, 2026,    9,   19,   34,
   31,   21,   19,   37,   39,   46,   43,   46,   46,   37,
 1131,   37,   64,   49,   52,   55,  829,   37,   85,   46,
   67,  830, 1139, 1601, 1169,   91,   88,  831,   94,   96,
   85, 1456,  893,  894,  895,  896,  897,   39,  899,  618,
  619,   46,  127,  130,   46,   37,  136,   46,  145, 1163,
  157, 1131,  148,  285, 1131,  148,  163,   37,  901,  902,
  160,  166, 2616,  906,  907,  172, 1153,   39, 2622, 1131,
   25, 1131, 2331, 2625,  285, 1948, 2626, 2627, 2187, 1949,
  285, 1950, 3048, 2632, 1067, 2634, 1951,  486, 3052, 1952,
  409,  409, 2635, 3056, 2636, 3711, 2024, 2639, 2640, 2641,
 2642, 1131, 3712, 2716, 2647,  832, 2648, 3083, 2649, 1953,
 2650, 1625, 1626,  285, 3570, 2956, 2651, 2652, 2653, 2654,
 2655, 2656, 2657, 2658, 2659, 2660, 2661, 2662,  486, 3505,
 2156,  486, 2663, 2664, 3676,  720, 3479, 2665, 2666, 3653,
 3453, 2911, 1153, 3214, 1309, 3478,  486,  608, 2673, 2843,
 3892,  833,  646, 2675, 3951, 1178, 1163, 2678, 3792, 1147,
 3469, 1178,  409,  409, 3895,  409, 2897, 1567, 2687, 3928,
 1178, 1067,  409, 1178, 1178, 3953,  724, 1178, 3224, 2951,
 1178, 3885, 3480, 1627, 1628, 4124, 1629, 4027, 3225, 4066,
 4090, 1169, 1178, 1630, 4127,  834, 4064, 1169, 1178, 3443,
 1147, 1178, 3831, 1147, 3444, 3932, 1169, 1178, 2599, 1169,
 1169, 2148, 1203, 1169,  835, 2595, 1169, 2492, 1147, 3137,
 1147, 1457, 2074,   85, 3760, 3138, 4104, 4013, 1169, 1459,
 1458, 1460, 2812, 4105, 1169,  993,  994, 1169,   94, 3876,
 3134, 1178,   33, 1169, 3937, 2280, 2025, 1896,   85, 1153,
 1147, 3337, 2924, 3488,   85, 2414, 2519, 2848, 1665,   85,
 3949, 1178, 3890, 2769,   85, 2039, 2413,   85, 1131,  875,
 3742, 2097, 2908, 2128, 1131, 1555,  696, 1169, 3599, 2738,
 1974, 3594, 1178, 1131, 3398, 1578, 1131, 1131, 1178, 3728,
 1131, 1218, 1178, 1131,  716,  768, 1197, 1169, 1178, 1150,
 1234, 1584, 2383, 3065, 2319, 1131, 1611, 2726, 3737, 1932,
 1933, 1131, 3853, 3683, 1131, 1929, 1178, 1720, 1169, 2387,
 1131, 1178, 2015, 4074, 1169, 1594, 1242,   59, 1169, 3110,
 2804,   61, 3716, 3717, 1169,   85,   62, 1141, 4097, 2813,
 1150,   65,   68, 1150, 2814, 1991, 4042, 2263, 2766, 1178,
 1864,  726, 1169, 1178, 1131, 2939, 2246, 1169, 1150, 2947,
 1150, 1282, 3896,  487,  486,   85, 1999, 2391, 2133, 1144,
  487, 1328, 1350, 3781, 1131, 1823, 1887, 3927, 1141, 3823,
   85, 1141, 1419, 2646, 3692, 1169, 1420, 2021, 2022, 1169,
 1150, 3521, 1821, 2756,  669, 1131, 1141, 3161, 1141, 3564,
 2728, 1131,   85, 3082, 3499, 1131, 1922, 3557, 1710, 4073,
 1144, 1131, 3609, 1144, 2587, 3788, 3672, 1147,  893,  894,
  895,  896,  897, 1147,  899,  618,  619,  488, 1144, 1131,
 1144, 1443, 1147, 3904, 1131, 1147, 1147, 3771, 3946, 1147,
 1444, 3162, 1147, 4028,  901,  902, 1633,  144, 3393,  906,
  907,  487, 1178, 1445, 1147, 3910, 3375, 1543,   85, 4035,
 1147,   85, 1131, 1147, 4007, 3789, 1131,    0,  488, 1147,
 3694,  488,  487,    0,    0,    0,    0,    0,  487,    0,
    0,    0,    0,    0,    0,    0,  488, 1446, 1169,    0,
    0,    0,    0,    0, 1447,    0,    0,    0,    0,    0,
    0,    0, 1448, 1147, 1459, 1458, 1460,    0,    0,    0,
    0,  487,    0, 1067,    0,    0,    0,    0,    0, 1449,
    0, 1450, 2187, 1147,    0,    0,    0,    0, 1178, 2984,
 2985, 1451,    0, 1452, 1453, 1454, 2992,    0,    0,    0,
 1455,    0,    0, 1144, 1147,    0,    0,    0,    0,    0,
 1147,    0,    0,    0, 1147,    0,    0, 1150,    0,    0,
 1147,    0,    0, 1150, 1169, 1131, 3018,   85,    0, 3020,
    0, 3022, 1150,    0,    0, 1150, 1150,    0, 1147, 1150,
    0, 1456, 1150, 1147, 1144,    0,    0, 1144,    0,    0,
    0,    0,    0,   88, 1150, 1141,    0,    0,    0,    0,
 1150, 1141, 1144, 1150, 1144,    0,    0,    0,   88, 1150,
    0, 1147, 1141, 1141, 1141, 1147,    0, 1141,   88,    0,
 1141,  993,  994,    0,   88,    0,    0, 1144, 1144,   88,
    0,    0, 1141, 1144,   88, 1178,    0,   88, 1141,    0,
    0, 1131, 1144, 1150,    0,    0, 1144, 1141,    0, 1144,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1150, 1144,    0,    0,    0,    0, 1144,
 1144, 1169, 1144, 1144, 2397,    0,    0,    0,    0, 1144,
    0,    0,    0,    0, 1150,    0,   72, 1144,    0,    0,
 1150,    0, 3124,    0, 1150,    0,    0,    0,    0,    0,
 1150, 1141,    0,  490,  488,   88, 1443,    0,  714,    0,
  490,    0,    0, 1144, 1147, 1444,    0, 3142, 1150,    0,
    0,    0, 1141, 1150,    0,    0,    0,    0, 1141,    0,
    0,    0,    0, 1144,    0,   88, 1141,    0, 1141,    0,
 2158, 2158, 2095, 2158, 2158, 2158,    0,    0, 1131,    0,
   88, 1150, 1459, 1458, 1460, 1150, 1141,    0, 2158, 2158,
 2158, 1141, 1446,    0, 1144,    0,    0,    0,    0, 1447,
 1144, 1457,   88,    0,    0,    0,    0, 1448, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0,    0, 1144, 2158,
 1147,  490,    0, 1141, 1449,    0, 1450,    0,    0,    0,
    0, 1144,    0,    0,    0,    0, 1451, 1144, 1452, 1453,
 1454,  459,  490,    0,    0, 1455,    0,    0,  490, 1144,
 1144, 1144,    0, 1144,    0, 1144, 1144,    0,   88,    0,
    0,   88,    0,    0,    0,    0,    0,    0, 1144,    0,
    0,    0,    0,    0, 1144,    0,    0,    0,    0,    0,
    0,  490,  459, 1144, 1150,  459,    0, 2159, 2159, 2096,
 2159, 2159, 2159,    0,    0, 3303,    0, 3305, 3700, 3306,
  459,    0,    0, 3311,    0, 2159, 2159, 2159, 3314, 3315,
 3316, 3317, 3318,    0,    0,    0,    0, 3319,    0, 3322,
    0, 3323, 1141,    0,    0,    0,    0, 1147,    0,    0,
    0,    0,    0,    0, 1144, 1144, 2159, 1144, 1144, 3327,
    0, 1144,    0, 3328,    0,    0,    0, 1067,    0,    0,
 3333,    0,    0, 1144, 1144,    0,    0,    0, 1144,    0,
 1150,    0,    0,    0, 1144,    0,    0,   88,    0,    0,
    0,    0,  506,    0, 1144,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
 2158,    0, 1144, 1444, 2158,    0,    0, 1144, 1141, 2158,
    0,    0,    0,    0,    0, 2158, 1445,    0,    0, 2158,
 1443,    0, 2158,  506,    0,    0,  506,    0,    0, 1444,
    0,    0, 1144,    0,    0,    0,    0,    0, 3410, 1144,
 1144,  506, 1445,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0, 1144,    0,    0, 2158, 1447,    0, 1144,
    0,    0,    0, 2158,    0, 1448,    0,    0,    0,    0,
    0, 2158,    0,    0,    0,    0, 1446, 1150,    0,    0,
    0,    0, 1449, 1447, 1450,    0, 1457,    0, 2158,    0,
 2158, 1448, 1144,    0, 1451,    0, 1452, 1453, 1454,    0,
 2158,    0, 2158, 2158, 2158,    0,    0,    0, 1449, 2158,
 1450, 1459, 1458, 1460,    0, 1141, 3460, 2159,    0,    0,
 1451, 2159, 1452, 1453, 1454,    0, 2159,  459,  459, 1455,
    0,    0, 2159,    0,  459,    0, 2159,    0, 1144, 2159,
    0,    0,    0,    0, 1456,    0,    0, 1144,    0,    0,
 2158,    0,    0,    0, 1743, 1744, 1745, 1746, 1747, 1748,
 1749, 1750,    0, 2204, 2204, 2090, 2204, 2204, 2204,    0,
 1456,    0,    0, 2159,    0,    0,    0,    0,    0,    0,
 2159, 2204, 2204, 2204,    0,    0,    0,    0, 2159,    0,
    0,   91,    0,    0,    0,    0, 2158, 2158, 2158, 2158,
 2158, 2158, 2158, 2158,    0, 2159,   98, 2159,    0,    0,
    0,    0, 2204,    0, 1144,  459,   91, 2159,    0, 2159,
 2159, 2159,   91, 1144,    0,    0, 2159,   91,    0,    0,
    0,    0,   91,    0,    0,   91,  459,    0,    0,    0,
 2187, 3306,  459,    0,    0,    0,    0, 1601,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 3535,  506,  506,
    0,    0,    0, 1751,    0,  506,    0, 2159, 3544,    0,
    0,    0,    0,    0,    0,  459,    0,    0,    0,    0,
    0, 2299, 2299, 2094, 2299, 2299, 2299,    0,    0, 1752,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2299,
 2299, 2299,    0,   91,    0, 2158,    0,    0,    0,    0,
    0,    0,    0, 2159, 2159, 2159, 2159, 2159, 2159, 2159,
 2159, 1144, 1444,    0,    0,    0,    0,    0,    0,    0,
 2299, 2158,    0,   91, 1457, 1445,    0,    0,    0,    0,
 2158,    0,    0,    0,    0,    0,  506,    0,   91,    0,
    0,    0,    0,    0, 1067, 3615,    0,    0,    0,    0,
 1457,    0,    0, 3618, 3619,    0,    0,  506,    0, 1446,
   91,    0,    0,  506,    0,    0, 1447,    0,    0,    0,
    0,    0,    0, 2204, 1448,    0,    0, 2204,    0,    0,
    0,    0, 2204,    0,    0,    0,    0,    0, 2204, 1904,
    0, 1449, 2204, 1450,    0, 2204,  506,    0,    0,    0,
    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,    0,
    0,    0, 2159,    0, 1905,    0,   91,    0,    0,   91,
 1906,    0,    0,    0,    0, 1907,    0, 1753, 1754, 2204,
 1908,    0,    0, 1909,    0,    0, 2204,    0, 2159,    0,
 3962, 2187,    0,    0, 2204,    0, 1755, 2159, 1756, 3689,
    0, 3690,    0, 1456,    0,    0,    0,    0,    0,    0,
    0, 2204,    0, 2204,    0, 3963,    0,    0,    0, 2158,
 2158,   86,    0, 2204,    0, 2204, 2204, 2204,    0,    0,
    0,    0, 2204,  875,    0,    0,    0,    0, 2158,    0,
 2158, 2299,    0,    0,    0, 2299,   86,    0,    0,    0,
 2299, 1910,   86,    0,    0,    0, 2299,   86,    0,  878,
 2299,    0,   86, 2299,    0,   86,    0,    0,    0,    0,
    0,    0, 2397, 2204, 3410,   91,    0,    0,    0,    0,
    0, 1911,    0,    0,    0, 3746,  880,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1912, 2299,    0,    0,
    0,    0,    0,    0, 2299, 1459, 1458, 1460,    0,    0,
    0,    0, 2299,    0,    0,    0,    0,    0, 1913, 2204,
 2204, 2204, 2204, 2204, 2204, 2204, 2204,    0,    0, 2299,
    0, 2299,    0,   86,    0,    0, 2159, 2159,   93,    0,
    0, 2299,    0, 2299, 2299, 2299,    0, 3805,   94,    0,
 2299,    0,  890,    0,   95, 2159,    0, 2159,    0,    0,
 2187,    0,    0,   86,   96, 3820, 3820,   97,  892,    0,
    0,    0, 3826,    0, 1914,    0,    0, 1915,   86,    0,
    0,    0,  893,  894,  895,  896,  897,    0,  899,  618,
  619, 2299,    0, 1457,    0,    0,    0,    0,    0,    0,
   86,    0,    0,    0,    0,    0,    0,    0,  901,  902,
  903,  904,  905,  906,  907,  908,  909,    0,    0,   98,
    0,    0,    0,  911,  912,    0,    0,    0, 2204,    0,
    0,    0,  916,    0,    0,   99,    0, 2299, 2299, 2299,
 2299, 2299, 2299, 2299, 2299,    0,    0,    0,    0,    0,
  100,    0,    0,    0, 2204,    0,   86,    0,    0,   86,
    0,    0,    0, 2204,    0,  101,    0,    0, 2122,  102,
    0,    0,  103,    0,  104,    0,    0,    0,    0,    0,
  105,    0,    0, 1916,    0,    0,    0,    0,    0, 3914,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  106,    0,    0,    0,    0, 1443, 2122, 2122,
    0, 2122, 2122, 2122, 2131,    0, 1444,    0, 3246,    0,
    0,    0,    0,    0, 3247,    0, 2122, 2122,    0, 1445,
    0, 2122,    0,    0, 3248,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2299, 3249,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  107,    0,
    0,  108,    0, 1446,    0,   86,    0,    0,    0,    0,
 1447,    0, 2299,    0,    0,    0,    0,    0, 1448,    0,
    0, 2299,    0,  992,    0,  993,  994,  995,    0,    0,
    0,    0,    0,    0, 3966, 1449,    0, 1450,    0,    0,
    0,    0, 2204, 2204,  109, 3250,    0, 1451,    0, 1452,
 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,    0,
    0, 2204,    0, 2204,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3251, 4000,    0,    0, 3252,
    0,    0,    0,    0, 3253,    0, 1022, 1023,  110,    0,
 3254,    0,    0, 4016,    0, 4020,    0, 1456,    0,    0,
    0,    0,    0,    0,    0, 1030,    0,  111,    0,    0,
    0,    0, 3255,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 4051,    0, 4054, 4056,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2299, 2299,    0,    0,    0, 2122,    0, 2122, 3256,    0,
    0, 3257, 2122, 2122,    0, 2122, 2122, 2122,    0, 2299,
    0, 2299, 2122,    0,    0,    0,    0, 2122, 2122,    0,
 2122,    0, 2122, 2122, 4051, 2122, 4107, 4108, 2122, 2122,
 2122,    0,    0,    0,    0, 2122,    0, 2122, 2122,    0,
    0,    0, 2122,    0, 2122,    0, 2122, 2122, 4123,    0,
 2122,    0,    0, 2122,    0, 2122,    0, 2122, 2122,    0,
 2122, 2122, 2122,    0, 2122, 2122, 2122,    0,    0,    0,
 2122,    0, 2122, 2122, 2122,    0,    0,    0, 2122,    0,
 2397,    0, 2122, 2397, 2122, 2122, 2122, 2122, 3258, 2122,
    0, 2122, 2122,    0, 2122, 2122, 2122,    0, 2122,    0,
    0,    0, 2122,    0,    0,    0, 2122, 3259, 2122,    0,
 2122,    0, 2122, 2122, 2122, 2122, 2122, 1457, 2122, 2122,
 2122,    0,    0,    0, 2122,    0, 2122, 2122,    0,    0,
 2122, 2122, 2122,    0,    0, 2122,    0, 2122,    0,    0,
 2122, 2122, 2122, 2122,    0, 2122,    0, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122,    0,    0,    0, 2122,    0,
 2122, 2122,    0,    0, 2122,    0,    0,    0,    0,    0,
    0, 2122,    0, 2122,    0, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,    0, 2122,
 2122, 2122, 2122,    0, 2122, 2122,    0,    0,    0,    0,
    0,    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122,    0, 2122, 2122, 2122, 2122,
 2122, 3028, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
    0,    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122,    0,
    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
    0, 2122,    0, 2122, 2122, 2122,    0, 2122, 2122,    0,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122,    0, 2122, 2122, 2122, 2122, 2122, 2122,    0,
 2122, 2122, 2122, 2122,    0, 2122,    0, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122,    0, 2122, 2122, 2122, 2122,
    0, 2122, 2122, 2122, 2122, 2122,    0, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
    0, 2122, 2122, 2122, 2122, 2122, 2122,    0, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122,    0, 2122, 2122, 2122,
 2122,    0, 2122, 2122, 2122, 2122, 2122, 2122,    0,    0,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122,    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122,    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122,    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122,    0, 2122, 2122, 2122,    0,
 2122,    0, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122,    0, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 1759,
    0, 2122, 2122,    0,    0, 2122,    0, 2122, 2122, 2122,
 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122, 2122,
 2122, 2122, 2122,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1759, 1759, 1759, 1759, 1759,    0, 1759,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1759, 1759,
 1759, 1759,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  733,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  734,    0,    0,
    0,    0, 1759,    0,    0,    0,    0,  735,    0,    0,
    0,    0,    0,  736,    0,    0,    0,  737,  738,    0,
    0,    0,    0,  739,    0,    0,  740,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  741,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  742,    0,    0,    0,
    0,    0,    0,    0,  743,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  744,    0,    0,    0,  745,    0,
    0,    0,    0,  746,    0,    0,    0,    0,    0,  747,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  748,  749,    0,    0,    0, 1759, 1759, 1759,    0,
    0, 1759,    0, 1759, 1759, 1759,    0, 1759, 1759,    0,
 1759,    0,    0,    0,    0, 1759, 1759,    0, 1759, 1759,
 1759, 1759, 1759, 1759,    0,    0, 1759,    0,    0, 1759,
    0,    0,    0,    0,    0,    0, 1759,    0, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759,    0,  750, 1759,    0,
  751, 1759,    0, 1759, 1759,    0,    0, 1759, 1759, 1759,
    0, 1759, 1759, 1759, 1759, 1759, 1759, 1759,    0,    0,
    0, 1759, 1759, 1759, 1759, 1759,    0,    0,    0, 1759,
    0,    0,    0,    0,    0,    0,    0, 1759, 1759, 1759,
 1759, 1759, 1759,  752,    0,    0, 1759, 1759,    0, 1759,
    0, 1759,    0, 1759, 1759, 1759,    0, 1759,    0, 1759,
 1759, 1759,    0, 1759, 1759, 1759, 1759, 1759,    0, 1759,
 1759,    0,    0,    0, 1759, 1759,    0, 1759, 1759,    0,
 1759, 1759, 1759, 1759,    0,    0, 1759,  753, 1759,    0,
 1759, 1759, 1759, 1759, 1759,    0, 1759,    0, 1759,    0,
 1759, 1759, 1759, 1759, 1759,    0,  754, 1759, 1759, 1759,
    0, 1759, 1759, 1759,    0,    0,    0,    0,    0,    0,
    0,    0, 1759, 1759, 1759,    0, 1759, 1759, 1759,    0,
    0,    0,    0,    0, 1759,    0,    0,    0, 1759,    0,
 1759, 1759, 1759, 1759,    0, 1759,    0, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1759,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1759, 1759,    0, 1759, 1759, 1759,
    0,    0,    0,    0, 1759, 1759, 1759, 1759, 1759,    0,
 1759,    0,    0, 1759, 1759,    0,    0,    0, 1759, 1759,
    0,    0, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
 1759,    0, 1759,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759,    0, 1759, 1759, 1759, 1759, 1759, 1759,
    0, 1759, 1759, 1759, 1759,    0, 1759,    0, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759,    0, 1759, 1759, 1759,
 1759,    0, 1759, 1759, 1759, 1759, 1759,    0, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,    0, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759,    0, 1759, 1759,
    0,    0,    0, 1759, 1759, 1759, 1759, 1759, 1759,    0,
    0, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
    0,    0,    0, 2201,    0, 1759,    0, 1759, 1759, 1759,
 1759, 1759, 1759,    0, 1759,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 1759, 1759,    0, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759,    0, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759,    0, 1759, 1759, 1759,
    0, 1759,    0,    0,    0, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,    0, 1759,
 1759,    0,    0, 1761,    0,    0, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759,    0,    0, 1759,    0, 1759, 1759,
 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759, 1759,
 1759, 1759, 1759, 1759, 1761, 1761, 1761, 1761, 1761,    0,
 1761,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1761, 1761, 1761, 1761,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1761,    0,    0,    0,
    0,  118,    0,    0,    0,    0,    0,  119,    0,    0,
    0,  120,  121,    0,    0,    0,    0,  122,    0,    0,
  123,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  124,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  125,    0,
    0,    0,    0,    0,    0,    0, 2200,    0,    0,    0,
 1446,    0,    0,  126,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,  127,    0,
    0,    0,  128,    0,    0,    0,    0,  129,    0,    0,
    0,    0, 1449,  130, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,  131,    0,    0,    0,    0,
 1761, 1761, 1761,    0,    0, 1761,    0, 1761, 1761, 1761,
    0, 1761, 1761,    0, 1761,    0,    0,    0,    0, 1761,
 1761,    0, 1761, 1761, 1761, 1761, 1761, 1761,    0,    0,
 1761,    0,    0, 1761, 1456,    0,    0,    0,    0,    0,
 1761,    0, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
    0,  132, 1761,    0,  133, 1761,    0, 1761, 1761,    0,
    0, 1761, 1761, 1761,    0, 1761, 1761, 1761, 1761, 1761,
 1761, 1761,    0,    0,    0, 1761, 1761, 1761, 1761, 1761,
    0,    0,    0, 1761,    0,    0,    0,    0,    0,    0,
    0, 1761, 1761, 1761, 1761, 1761, 1761,  134,    0,    0,
 1761, 1761,    0, 1761,    0, 1761,    0, 1761, 1761, 1761,
    0, 1761,    0, 1761, 1761, 1761,    0, 1761, 1761, 1761,
 1761, 1761,    0, 1761, 1761,    0,    0,    0, 1761, 1761,
    0, 1761, 1761,    0, 1761, 1761, 1761, 1761,    0,    0,
 1761,  135, 1761,    0, 1761, 1761, 1761, 1761, 1761,    0,
 1761,    0, 1761,    0, 1761, 1761, 1761, 1761, 1761,    0,
  136, 1761, 1761, 1761,    0, 1761, 1761, 1761,    0,    0,
    0,    0,    0,    0,    0,    0, 1761, 1761, 1761,    0,
 1761, 1761, 1761,    0,    0,    0,    0,    0, 1761,    0,
    0,    0, 1761,    0, 1761, 1761, 1761, 1761,    0, 1761,
    0, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1761, 1761,
    0, 1761, 1761, 1761, 1457,    0,    0,    0, 1761, 1761,
 1761, 1761, 1761,    0, 1761,    0,    0, 1761, 1761,    0,
    0,    0, 1761, 1761,    0,    0, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761, 1761,    0, 1761,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761,    0, 1761, 1761,
 1761, 1761, 1761, 1761,    0, 1761, 1761, 1761, 1761,    0,
 1761,    0, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
    0, 1761, 1761, 1761, 1761,    0, 1761, 1761, 1761, 1761,
 1761,    0, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761, 1761,    0, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761,    0, 1761, 1761,    0,    0,    0, 1761, 1761, 1761,
 1761, 1761, 1761,    0,    0, 1761, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761,    0,    0,    0,    0, 3530, 1761,
    0, 1761, 1761, 1761, 1761, 1761, 1761,    0, 1761,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 1761, 1761,    0, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761,    0, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
    0, 1761, 1761, 1761,    0, 1761,    0,    0,    0, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761, 1761,    0, 1761, 1761,    0,    0,    0, 2133,    0,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,    0,    0,
 1761,    0, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761,
 1761, 1761, 1761, 1761, 1761, 1761, 1761, 1761, 2103, 2133,
 2133, 2133, 2133, 2133, 2133, 2133,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   70, 2133, 2133, 2133, 2133,
 2133,    0,    0,    0,    0,    0,    0,    0,   71,    0,
   72,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   73,    0,    0,    0,    0,    0,   74,    0,    0, 2133,
   75, 2133,    0,    0,    0,    0,   76,    0,    0,   77,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   78,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,   79,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3529,
    0,    0,    0,    0,    0, 1446,    0,   80,    0,    0,
    0,   81, 1447,    0,    0,    0,   82,    0,    0,    0,
 1448,    0,   83,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,   84,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2133,    0, 2133,    0,    0,
 2133,    0, 2133, 2133, 2133,    0, 2133, 2133,    0, 2133,
    0,    0,    0,    0,    0, 2133,    0, 2133, 2133, 2133,
 2133,    0, 2133,    0,    0, 2133,    0,    0, 2133, 1456,
   85,    0,    0,    0,    0, 2133,    0, 2133, 2133,    0,
 2133, 2133, 2133, 2133, 2133,    0,    0, 2133,    0,    0,
 2133,    0, 2133, 2133,    0,    0, 2133, 2133, 2133,    0,
 2133, 2133, 2133, 2133, 2133, 2133, 2133,    0,    0,    0,
 2133, 2133, 2133, 2133, 2133,    0,   86,    0, 2133,    0,
    0,    0,    0,    0,    0,    0, 2133, 2133, 2133, 2133,
 2133, 2133,    0,    0,    0, 2133, 2133,    0, 2133,    0,
 2133,    0, 2133, 2133, 2133,    0, 2133,    0, 2133, 2133,
 2133,    0, 2133, 2133, 2133, 2133, 2133,    0, 2133, 2133,
   87,    0,    0,    0, 2133,    0, 2133, 2133,    0,    0,
 2133, 2133, 2133,    0,    0, 2133,    0, 2133,    0,   88,
 2133, 2133, 2133, 2133,    0, 2133,    0, 2133,    0, 2133,
 2133, 2133, 2133, 2133,    0,    0, 2133, 2133, 2133,    0,
 2133, 2133,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2133, 2133, 2133,    0, 2133, 2133, 2133,    0,    0,
    0,    0,    0, 2133,    0,    0,    0, 2133,    0, 2133,
 2133, 2133, 2133,    0, 2133,    0, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2133, 2133,    0, 2133, 2133, 2133, 1457,
    0,    0,    0, 2133, 2133, 2133, 2133, 2133,    0, 2133,
    0,    0, 2133, 2133,    0,    0,    0, 2133, 2133,    0,
    0, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
    0, 2133,    0,    0,    0,    0,   89,    0,    0,    0,
    0, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133,    0, 2133, 2133, 2133, 2133, 2133, 2133,    0,
 2133, 2133, 2133, 2133,    0, 2133,    0, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133,    0, 2133, 2133, 2133, 2133,
    0, 2133, 2133, 2133, 2133, 2133,    0, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133,    0, 2133, 2133,    0,
    0,    0, 2133, 2133, 2133, 2133, 2133, 2133,    0,    0,
 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,    0,
    0,    0,    0, 2008, 2133,    0, 2133, 2133, 2133, 2133,
 2133, 2133,    0, 2133,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2133, 2133,    0, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133,    0, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133,    0, 2133, 2133, 2133,    0,
 2133,    0,    0,    0, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133, 2133,    0, 2133, 2133,
    0,    0,    0, 2170,    0, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133,    0,    0, 2133,    0, 2133, 2133, 2133,
 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133, 2133,
 2133, 2133, 2133, 1922, 2170, 2170, 2170, 2170, 2170, 2170,
 2170,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2170, 2170, 2170, 2170, 2170,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 3260,    0,    0,    0,    0,    0, 3261,    0,
    0,    0,    0,    0, 2170,    0, 2170,    0, 3262,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3263,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0, 3264,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3265,
 1446,    0,    0, 3266,    0,    0,    0, 1447, 3267,    0,
    0,    0,    0,    0, 3268, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0, 3269,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2170,    0, 2170,    0,    0, 2170,    0, 2170, 2170, 2170,
    0, 2170, 2170,    0, 2170,    0,    0,    0,    0,    0,
 2170,    0, 2170, 2170, 2170, 2170,    0, 2170,    0,    0,
 2170,    0, 3270, 2170, 1456, 3271,    0,    0,    0,    0,
 2170,    0, 2170, 2170,    0, 2170, 2170, 2170, 2170, 2170,
    0,    0, 2170,    0,    0, 2170,    0, 2170, 2170,    0,
    0, 2170, 2170, 2170,    0, 2170, 2170, 2170, 2170, 2170,
 2170, 2170,    0,    0,    0, 2170, 2170, 2170, 2170, 2170,
    0,    0,    0, 2170,    0,    0,    0,    0,    0,    0,
    0, 2170, 2170, 2170, 2170, 2170, 2170,    0,    0,    0,
 2170, 2170,    0, 2170,    0, 2170,    0, 2170, 2170, 2170,
    0, 2170,    0, 2170, 2170, 2170,    0, 2170, 2170, 2170,
 2170, 2170, 3272, 2170, 2170,    0,    0,    0,    0, 2170,
    0, 2170, 2170,    0,    0, 2170, 2170, 2170,    0,    0,
 2170, 3273, 2170,    0,    0, 2170, 2170, 2170, 2170,    0,
 2170,    0, 2170,    0, 2170, 2170, 2170, 2170, 2170,    0,
    0, 2170, 2170, 2170,    0, 2170, 2170,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2170, 2170, 2170,    0,
 2170, 2170, 2170,    0,    0,    0,    0,    0, 2170,    0,
    0,    0, 2170,    0, 2170, 2170, 2170, 2170,    0, 2170,
    0, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2170, 2170,
    0, 2170, 2170, 2170, 1457,    0,    0,    0, 2170, 2170,
 2170, 2170, 2170,    0, 2170,    0,    0, 2170, 2170,    0,
    0,    0, 2170, 2170,    0,    0, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170,    0, 2170,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170,    0, 2170, 2170,
 2170, 2170, 2170, 2170,    0, 2170, 2170, 2170, 2170,    0,
 2170,    0, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
    0, 2170, 2170, 2170, 2170,    0, 2170, 2170, 2170, 2170,
 2170,    0, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170,    0, 2170, 2170,    0,    0,    0, 2170, 2170, 2170,
 2170, 2170, 2170,    0,    0, 2170, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170,    0,    0,    0,    0, 2009, 2170,
    0, 2170, 2170, 2170, 2170, 2170, 2170,    0, 2170,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2170, 2170,    0, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170,    0, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
    0, 2170, 2170, 2170,    0, 2170,    0,    0,    0, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170, 2170,    0, 2170, 2170,    0,    0,    0, 2172,    0,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,    0,    0,
 2170,    0, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170,
 2170, 2170, 2170, 2170, 2170, 2170, 2170, 2170, 1923, 2172,
 2172, 2172, 2172, 2172, 2172, 2172,    0,    0,    0,    0,
    0,    0,    1,    0,    0,    0, 2172, 2172, 2172, 2172,
 2172,    0,    0,    0,    0,    0,    0,    0,    0,    2,
    0,    0,    0,    0,    0,    0,    0,    3,    0,    0,
    0,    0,    0,    0,    0,    4,    0,    5,    0, 2172,
    0, 2172,    6,    0,    0,    0,    0,    0,    7,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    8,    0,    0,    0,    0,    0,    0,    0,
    0,    9,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,   10,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   11,    0,   12,
    0,    0,   13,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,   14,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,   15, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2172,    0, 2172,    0,    0,
 2172,    0, 2172, 2172, 2172,    0, 2172, 2172,   16, 2172,
    0,    0,    0,    0,    0, 2172,    0, 2172, 2172, 2172,
 2172,    0, 2172,   17,   18, 2172,    0,    0, 2172, 1456,
    0,    0,    0,    0,    0, 2172,    0, 2172, 2172,    0,
 2172, 2172, 2172, 2172, 2172,   19,    0, 2172,    0,    0,
 2172,    0, 2172, 2172,    0,    0, 2172, 2172, 2172,    0,
 2172, 2172, 2172, 2172, 2172, 2172, 2172,    0,    0,    0,
 2172, 2172, 2172, 2172, 2172,    0,    0,    0, 2172,    0,
    0,    0,    0,    0,    0,    0, 2172, 2172, 2172, 2172,
 2172, 2172,    0,    0,    0, 2172, 2172,    0, 2172,    0,
 2172,    0, 2172, 2172, 2172,    0, 2172,    0, 2172, 2172,
 2172,    0, 2172, 2172, 2172, 2172, 2172,    0, 2172, 2172,
    0,    0,    0,    0, 2172,    0, 2172, 2172,    0,    0,
 2172, 2172, 2172,    0,    0, 2172,    0, 2172,    0,    0,
 2172, 2172, 2172, 2172,    0, 2172,    0, 2172,    0, 2172,
 2172, 2172, 2172, 2172,    0,    0, 2172, 2172, 2172,    0,
 2172, 2172,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2172, 2172, 2172,    0, 2172, 2172, 2172,    0,    0,
    0,    0,    0, 2172,    0,    0,    0, 2172,    0, 2172,
 2172, 2172, 2172,    0, 2172,    0, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2172, 2172,    0, 2172, 2172, 2172, 1457,
    0,    0,    0, 2172, 2172, 2172, 2172, 2172,    0, 2172,
    0,    0, 2172, 2172,    0,    0,    0, 2172, 2172,    0,
    0, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
    0, 2172,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172,    0, 2172, 2172, 2172, 2172, 2172, 2172,    0,
 2172, 2172, 2172, 2172,    0, 2172,    0, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172,    0, 2172, 2172, 2172, 2172,
    0, 2172, 2172, 2172, 2172, 2172,    0, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172,    0, 2172, 2172,    0,
    0,    0, 2172, 2172, 2172, 2172, 2172, 2172,    0,    0,
 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,    0,
    0,    0,    0, 2171, 2172,    0, 2172, 2172, 2172, 2172,
 2172, 2172,    0, 2172,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2172, 2172,    0, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172,    0, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172,    0, 2172, 2172, 2172,    0,
 2172,    0,    0,    0, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172, 2172,    0, 2172, 2172,
    0,    0,    0, 2174,    0, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172,    0,    0, 2172,    0, 2172, 2172, 2172,
 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172, 2172,
 2172, 2172, 2172, 1925, 2174, 2174, 2174, 2174, 2174, 2174,
 2174,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2174, 2174, 2174, 2174, 2174,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2174,    0, 2174,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2174,    0, 2174,    0,    0, 2174,    0, 2174, 2174, 2174,
    0, 2174, 2174,    0, 2174,    0,    0,    0,    0,    0,
 2174,    0, 2174, 2174, 2174, 2174,    0, 2174,    0,    0,
 2174,    0,    0, 2174, 1456,    0,    0,    0,    0,    0,
 2174,    0, 2174, 2174,    0, 2174, 2174, 2174, 2174, 2174,
    0,    0, 2174,    0,    0, 2174,    0, 2174, 2174,    0,
    0, 2174, 2174, 2174,    0, 2174, 2174, 2174, 2174, 2174,
 2174, 2174,    0,    0,    0, 2174, 2174, 2174, 2174, 2174,
    0,    0,    0, 2174,    0,    0,    0,    0,    0,    0,
    0, 2174, 2174, 2174, 2174, 2174, 2174,    0,    0,    0,
 2174, 2174,    0, 2174,    0, 2174,    0, 2174, 2174, 2174,
    0, 2174,    0, 2174, 2174, 2174,    0, 2174, 2174, 2174,
 2174, 2174,    0, 2174, 2174,    0,    0,    0,    0, 2174,
    0, 2174, 2174,    0,    0, 2174, 2174, 2174,    0,    0,
 2174,    0, 2174,    0,    0, 2174, 2174, 2174, 2174,    0,
 2174,    0, 2174,    0, 2174, 2174, 2174, 2174, 2174,    0,
    0, 2174, 2174, 2174,    0, 2174, 2174,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2174, 2174, 2174,    0,
 2174, 2174, 2174,    0,    0,    0,    0,    0, 2174,    0,
    0,    0, 2174,    0, 2174, 2174, 2174, 2174,    0, 2174,
    0, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2174, 2174,
    0, 2174, 2174, 2174, 1457,    0,    0,    0, 2174, 2174,
 2174, 2174, 2174,    0, 2174,    0,    0, 2174, 2174,    0,
    0,    0, 2174, 2174,    0,    0, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174,    0, 2174,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174,    0, 2174, 2174,
 2174, 2174, 2174, 2174,    0, 2174, 2174, 2174, 2174,    0,
 2174,    0, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
    0, 2174, 2174, 2174, 2174,    0, 2174, 2174, 2174, 2174,
 2174,    0, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174,    0, 2174, 2174,    0,    0,    0, 2174, 2174, 2174,
 2174, 2174, 2174,    0,    0, 2174, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174,    0,    0,    0,    0, 2195, 2174,
    0, 2174, 2174, 2174, 2174, 2174, 2174,    0, 2174,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2174, 2174,    0, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174,    0, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
    0, 2174, 2174, 2174,    0, 2174,    0,    0,    0, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174, 2174,    0, 2174, 2174,    0,    0,    0, 2175,    0,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,    0,    0,
 2174,    0, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174,
 2174, 2174, 2174, 2174, 2174, 2174, 2174, 2174, 1926, 2175,
 2175, 2175, 2175, 2175, 2175, 2175,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2175, 2175, 2175, 2175,
 2175,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2175,
    0, 2175,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2175,    0, 2175,    0,    0,
 2175,    0, 2175, 2175, 2175,    0, 2175, 2175,    0, 2175,
    0,    0,    0,    0,    0, 2175,    0, 2175, 2175, 2175,
 2175,    0, 2175,    0,    0, 2175,    0,    0, 2175, 1456,
    0,    0,    0,    0,    0, 2175,    0, 2175, 2175,    0,
 2175, 2175, 2175, 2175, 2175,    0,    0, 2175,    0,    0,
 2175,    0, 2175, 2175,    0,    0, 2175, 2175, 2175,    0,
 2175, 2175, 2175, 2175, 2175, 2175, 2175,    0,    0,    0,
 2175, 2175, 2175, 2175, 2175,    0,    0,    0, 2175,    0,
    0,    0,    0,    0,    0,    0, 2175, 2175, 2175, 2175,
 2175, 2175,    0,    0,    0, 2175, 2175,    0, 2175,    0,
 2175,    0, 2175, 2175, 2175,    0, 2175,    0, 2175, 2175,
 2175,    0, 2175, 2175, 2175, 2175, 2175,    0, 2175, 2175,
    0,    0,    0,    0, 2175,    0, 2175, 2175,    0,    0,
 2175, 2175, 2175,    0,    0, 2175,    0, 2175,    0,    0,
 2175, 2175, 2175, 2175,    0, 2175,    0, 2175,    0, 2175,
 2175, 2175, 2175, 2175,    0,    0, 2175, 2175, 2175,    0,
 2175, 2175,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2175, 2175, 2175,    0, 2175, 2175, 2175,    0,    0,
    0,    0,    0, 2175,    0,    0,    0, 2175,    0, 2175,
 2175, 2175, 2175,    0, 2175,    0, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2175, 2175,    0, 2175, 2175, 2175, 1457,
    0,    0,    0, 2175, 2175, 2175, 2175, 2175,    0, 2175,
    0,    0, 2175, 2175,    0,    0,    0, 2175, 2175,    0,
    0, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
    0, 2175,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175,    0, 2175, 2175, 2175, 2175, 2175, 2175,    0,
 2175, 2175, 2175, 2175,    0, 2175,    0, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175,    0, 2175, 2175, 2175, 2175,
    0, 2175, 2175, 2175, 2175, 2175,    0, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175,    0, 2175, 2175,    0,
    0,    0, 2175, 2175, 2175, 2175, 2175, 2175,    0,    0,
 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,    0,
    0,    0,    0, 2196, 2175,    0, 2175, 2175, 2175, 2175,
 2175, 2175,    0, 2175,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2175, 2175,    0, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175,    0, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175,    0, 2175, 2175, 2175,    0,
 2175,    0,    0,    0, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175, 2175,    0, 2175, 2175,
    0,    0,    0, 2176,    0, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175,    0,    0, 2175,    0, 2175, 2175, 2175,
 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175, 2175,
 2175, 2175, 2175, 1927, 2176, 2176, 2176, 2176, 2176, 2176,
 2176,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2176, 2176, 2176, 2176, 2176,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2176,    0, 2176,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2176,    0, 2176,    0,    0, 2176,    0, 2176, 2176, 2176,
    0, 2176, 2176,    0, 2176,    0,    0,    0,    0,    0,
 2176,    0, 2176, 2176, 2176, 2176,    0, 2176,    0,    0,
 2176,    0,    0, 2176, 1456,    0,    0,    0,    0,    0,
 2176,    0, 2176, 2176,    0, 2176, 2176, 2176, 2176, 2176,
    0,    0, 2176,    0,    0, 2176,    0, 2176, 2176,    0,
    0, 2176, 2176, 2176,    0, 2176, 2176, 2176, 2176, 2176,
 2176, 2176,    0,    0,    0, 2176, 2176, 2176, 2176, 2176,
    0,    0,    0, 2176,    0,    0,    0,    0,    0,    0,
    0, 2176, 2176, 2176, 2176, 2176, 2176,    0,    0,    0,
 2176, 2176,    0, 2176,    0, 2176,    0, 2176, 2176, 2176,
    0, 2176,    0, 2176, 2176, 2176,    0, 2176, 2176, 2176,
 2176, 2176,    0, 2176, 2176,    0,    0,    0,    0, 2176,
    0, 2176, 2176,    0,    0, 2176, 2176, 2176,    0,    0,
 2176,    0, 2176,    0,    0, 2176, 2176, 2176, 2176,    0,
 2176,    0, 2176,    0, 2176, 2176, 2176, 2176, 2176,    0,
    0, 2176, 2176, 2176,    0, 2176, 2176,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2176, 2176, 2176,    0,
 2176, 2176, 2176,    0,    0,    0,    0,    0, 2176,    0,
    0,    0, 2176,    0, 2176, 2176, 2176, 2176,    0, 2176,
    0, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2176, 2176,
    0, 2176, 2176, 2176, 1457,    0,    0,    0, 2176, 2176,
 2176, 2176, 2176,    0, 2176,    0,    0, 2176, 2176,    0,
    0,    0, 2176, 2176,    0,    0, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176,    0, 2176,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176,    0, 2176, 2176,
 2176, 2176, 2176, 2176,    0, 2176, 2176, 2176, 2176,    0,
 2176,    0, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
    0, 2176, 2176, 2176, 2176,    0, 2176, 2176, 2176, 2176,
 2176,    0, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176,    0, 2176, 2176,    0,    0,    0, 2176, 2176, 2176,
 2176, 2176, 2176,    0,    0, 2176, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176,    0,    0,    0,    0, 2197, 2176,
    0, 2176, 2176, 2176, 2176, 2176, 2176,    0, 2176,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2176, 2176,    0, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176,    0, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
    0, 2176, 2176, 2176,    0, 2176,    0,    0,    0, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176, 2176,    0, 2176, 2176,    0,    0,    0, 2177,    0,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,    0,    0,
 2176,    0, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176,
 2176, 2176, 2176, 2176, 2176, 2176, 2176, 2176, 1929, 2177,
 2177, 2177, 2177, 2177, 2177, 2177,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2177, 2177, 2177, 2177,
 2177,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2177,
    0, 2177,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2177,    0, 2177,    0,    0,
 2177,    0, 2177, 2177, 2177,    0, 2177, 2177,    0, 2177,
    0,    0,    0,    0,    0, 2177,    0, 2177, 2177, 2177,
 2177,    0, 2177,    0,    0, 2177,    0,    0, 2177, 1456,
    0,    0,    0,    0,    0, 2177,    0, 2177, 2177,    0,
 2177, 2177, 2177, 2177, 2177,    0,    0, 2177,    0,    0,
 2177,    0, 2177, 2177,    0,    0, 2177, 2177, 2177,    0,
 2177, 2177, 2177, 2177, 2177, 2177, 2177,    0,    0,    0,
 2177, 2177, 2177, 2177, 2177,    0,    0,    0, 2177,    0,
    0,    0,    0,    0,    0,    0, 2177, 2177, 2177, 2177,
 2177, 2177,    0,    0,    0, 2177, 2177,    0, 2177,    0,
 2177,    0, 2177, 2177, 2177,    0, 2177,    0, 2177, 2177,
 2177,    0, 2177, 2177, 2177, 2177, 2177,    0, 2177, 2177,
    0,    0,    0,    0, 2177,    0, 2177, 2177,    0,    0,
 2177, 2177, 2177,    0,    0, 2177,    0, 2177,    0,    0,
 2177, 2177, 2177, 2177,    0, 2177,    0, 2177,    0, 2177,
 2177, 2177, 2177, 2177,    0,    0, 2177, 2177, 2177,    0,
 2177, 2177,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2177, 2177, 2177,    0, 2177, 2177, 2177,    0,    0,
    0,    0,    0, 2177,    0,    0,    0, 2177,    0, 2177,
 2177, 2177, 2177,    0, 2177,    0, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2177, 2177,    0, 2177, 2177, 2177, 1457,
    0,    0,    0, 2177, 2177, 2177, 2177, 2177,    0, 2177,
    0,    0, 2177, 2177,    0,    0,    0, 2177, 2177,    0,
    0, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
    0, 2177,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177,    0, 2177, 2177, 2177, 2177, 2177, 2177,    0,
 2177, 2177, 2177, 2177,    0, 2177,    0, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177,    0, 2177, 2177, 2177, 2177,
    0, 2177, 2177, 2177, 2177, 2177,    0, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177,    0, 2177, 2177,    0,
    0,    0, 2177, 2177, 2177, 2177, 2177, 2177,    0,    0,
 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,    0,
    0,    0,    0, 2198, 2177,    0, 2177, 2177, 2177, 2177,
 2177, 2177,    0, 2177,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2177, 2177,    0, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177,    0, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177,    0, 2177, 2177, 2177,    0,
 2177,    0,    0,    0, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177, 2177,    0, 2177, 2177,
    0,    0,    0, 2178,    0, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177,    0,    0, 2177,    0, 2177, 2177, 2177,
 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177, 2177,
 2177, 2177, 2177, 1930, 2178, 2178, 2178, 2178, 2178, 2178,
 2178,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2178, 2178, 2178, 2178, 2178,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2178,    0, 2178,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2178,    0, 2178,    0,    0, 2178,    0, 2178, 2178, 2178,
    0, 2178, 2178,    0, 2178,    0,    0,    0,    0,    0,
 2178,    0, 2178, 2178, 2178, 2178,    0, 2178,    0,    0,
 2178,    0,    0, 2178, 1456,    0,    0,    0,    0,    0,
 2178,    0, 2178, 2178,    0, 2178, 2178, 2178, 2178, 2178,
    0,    0, 2178,    0,    0, 2178,    0, 2178, 2178,    0,
    0, 2178, 2178, 2178,    0, 2178, 2178, 2178, 2178, 2178,
 2178, 2178,    0,    0,    0, 2178, 2178, 2178, 2178, 2178,
    0,    0,    0, 2178,    0,    0,    0,    0,    0,    0,
    0, 2178, 2178, 2178, 2178, 2178, 2178,    0,    0,    0,
 2178, 2178,    0, 2178,    0, 2178,    0, 2178, 2178, 2178,
    0, 2178,    0, 2178, 2178, 2178,    0, 2178, 2178, 2178,
 2178, 2178,    0, 2178, 2178,    0,    0,    0,    0, 2178,
    0, 2178, 2178,    0,    0, 2178, 2178, 2178,    0,    0,
 2178,    0, 2178,    0,    0, 2178, 2178, 2178, 2178,    0,
 2178,    0, 2178,    0, 2178, 2178, 2178, 2178, 2178,    0,
    0, 2178, 2178, 2178,    0, 2178, 2178,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2178, 2178, 2178,    0,
 2178, 2178, 2178,    0,    0,    0,    0,    0, 2178,    0,
    0,    0, 2178,    0, 2178, 2178, 2178, 2178,    0, 2178,
    0, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2178, 2178,
    0, 2178, 2178, 2178, 1457,    0,    0,    0, 2178, 2178,
 2178, 2178, 2178,    0, 2178,    0,    0, 2178, 2178,    0,
    0,    0, 2178, 2178,    0,    0, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178,    0, 2178,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178,    0, 2178, 2178,
 2178, 2178, 2178, 2178,    0, 2178, 2178, 2178, 2178,    0,
 2178,    0, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
    0, 2178, 2178, 2178, 2178,    0, 2178, 2178, 2178, 2178,
 2178,    0, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178,    0, 2178, 2178,    0,    0,    0, 2178, 2178, 2178,
 2178, 2178, 2178,    0,    0, 2178, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178,    0,    0,    0,    0, 2199, 2178,
    0, 2178, 2178, 2178, 2178, 2178, 2178,    0, 2178,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2178, 2178,    0, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178,    0, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
    0, 2178, 2178, 2178,    0, 2178,    0,    0,    0, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178, 2178,    0, 2178, 2178,    0,    0,    0, 2179,    0,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,    0,    0,
 2178,    0, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178,
 2178, 2178, 2178, 2178, 2178, 2178, 2178, 2178, 1934, 2179,
 2179, 2179, 2179, 2179, 2179, 2179,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2179, 2179, 2179, 2179,
 2179,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2179,
    0, 2179,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2179,    0, 2179,    0,    0,
 2179,    0, 2179, 2179, 2179,    0, 2179, 2179,    0, 2179,
    0,    0,    0,    0,    0, 2179,    0, 2179, 2179, 2179,
 2179,    0, 2179,    0,    0, 2179,    0,    0, 2179, 1456,
    0,    0,    0,    0,    0, 2179,    0, 2179, 2179,    0,
 2179, 2179, 2179, 2179, 2179,    0,    0, 2179,    0,    0,
 2179,    0, 2179, 2179,    0,    0, 2179, 2179, 2179,    0,
 2179, 2179, 2179, 2179, 2179, 2179, 2179,    0,    0,    0,
 2179, 2179, 2179, 2179, 2179,    0,    0,    0, 2179,    0,
    0,    0,    0,    0,    0,    0, 2179, 2179, 2179, 2179,
 2179, 2179,    0,    0,    0, 2179, 2179,    0, 2179,    0,
 2179,    0, 2179, 2179, 2179,    0, 2179,    0, 2179, 2179,
 2179,    0, 2179, 2179, 2179, 2179, 2179,    0, 2179, 2179,
    0,    0,    0,    0, 2179,    0, 2179, 2179,    0,    0,
 2179, 2179, 2179,    0,    0, 2179,    0, 2179,    0,    0,
 2179, 2179, 2179, 2179,    0, 2179,    0, 2179,    0, 2179,
 2179, 2179, 2179, 2179,    0,    0, 2179, 2179, 2179,    0,
 2179, 2179,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2179, 2179, 2179,    0, 2179, 2179, 2179,    0,    0,
    0,    0,    0, 2179,    0,    0,    0, 2179,    0, 2179,
 2179, 2179, 2179,    0, 2179,    0, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2179, 2179,    0, 2179, 2179, 2179, 1457,
    0,    0,    0, 2179, 2179, 2179, 2179, 2179,    0, 2179,
    0,    0, 2179, 2179,    0,    0,    0, 2179, 2179,    0,
    0, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
    0, 2179,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179,    0, 2179, 2179, 2179, 2179, 2179, 2179,    0,
 2179, 2179, 2179, 2179,    0, 2179,    0, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179,    0, 2179, 2179, 2179, 2179,
    0, 2179, 2179, 2179, 2179, 2179,    0, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179,    0, 2179, 2179,    0,
    0,    0, 2179, 2179, 2179, 2179, 2179, 2179,    0,    0,
 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,    0,
    0,    0,    0, 2209, 2179,    0, 2179, 2179, 2179, 2179,
 2179, 2179,    0, 2179,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2179, 2179,    0, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179,    0, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179,    0, 2179, 2179, 2179,    0,
 2179,    0,    0,    0, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179, 2179,    0, 2179, 2179,
    0,    0,    0, 2180,    0, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179,    0,    0, 2179,    0, 2179, 2179, 2179,
 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179, 2179,
 2179, 2179, 2179, 1936, 2180, 2180, 2180, 2180, 2180, 2180,
 2180,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2180, 2180, 2180, 2180, 2180,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2180,    0, 2180,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2180,    0, 2180,    0,    0, 2180,    0, 2180, 2180, 2180,
    0, 2180, 2180,    0, 2180,    0,    0,    0,    0,    0,
 2180,    0, 2180, 2180, 2180, 2180,    0, 2180,    0,    0,
 2180,    0,    0, 2180, 1456,    0,    0,    0,    0,    0,
 2180,    0, 2180, 2180,    0, 2180, 2180, 2180, 2180, 2180,
    0,    0, 2180,    0,    0, 2180,    0, 2180, 2180,    0,
    0, 2180, 2180, 2180,    0, 2180, 2180, 2180, 2180, 2180,
 2180, 2180,    0,    0,    0, 2180, 2180, 2180, 2180, 2180,
    0,    0,    0, 2180,    0,    0,    0,    0,    0,    0,
    0, 2180, 2180, 2180, 2180, 2180, 2180,    0,    0,    0,
 2180, 2180,    0, 2180,    0, 2180,    0, 2180, 2180, 2180,
    0, 2180,    0, 2180, 2180, 2180,    0, 2180, 2180, 2180,
 2180, 2180,    0, 2180, 2180,    0,    0,    0,    0, 2180,
    0, 2180, 2180,    0,    0, 2180, 2180, 2180,    0,    0,
 2180,    0, 2180,    0,    0, 2180, 2180, 2180, 2180,    0,
 2180,    0, 2180,    0, 2180, 2180, 2180, 2180, 2180,    0,
    0, 2180, 2180, 2180,    0, 2180, 2180,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2180, 2180, 2180,    0,
 2180, 2180, 2180,    0,    0,    0,    0,    0, 2180,    0,
    0,    0, 2180,    0, 2180, 2180, 2180, 2180,    0, 2180,
    0, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2180, 2180,
    0, 2180, 2180, 2180, 1457,    0,    0,    0, 2180, 2180,
 2180, 2180, 2180,    0, 2180,    0,    0, 2180, 2180,    0,
    0,    0, 2180, 2180,    0,    0, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180,    0, 2180,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180,    0, 2180, 2180,
 2180, 2180, 2180, 2180,    0, 2180, 2180, 2180, 2180,    0,
 2180,    0, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
    0, 2180, 2180, 2180, 2180,    0, 2180, 2180, 2180, 2180,
 2180,    0, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180,    0, 2180, 2180,    0,    0,    0, 2180, 2180, 2180,
 2180, 2180, 2180,    0,    0, 2180, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180,    0,    0,    0,    0, 2213, 2180,
    0, 2180, 2180, 2180, 2180, 2180, 2180,    0, 2180,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2180, 2180,    0, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180,    0, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
    0, 2180, 2180, 2180,    0, 2180,    0,    0,    0, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180, 2180,    0, 2180, 2180,    0,    0,    0, 2181,    0,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,    0,    0,
 2180,    0, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180,
 2180, 2180, 2180, 2180, 2180, 2180, 2180, 2180, 1937, 2181,
 2181, 2181, 2181, 2181, 2181, 2181,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2181, 2181, 2181, 2181,
 2181,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2181,
    0, 2181,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2181,    0, 2181,    0,    0,
 2181,    0, 2181, 2181, 2181,    0, 2181, 2181,    0, 2181,
    0,    0,    0,    0,    0, 2181,    0, 2181, 2181, 2181,
 2181,    0, 2181,    0,    0, 2181,    0,    0, 2181, 1456,
    0,    0,    0,    0,    0, 2181,    0, 2181, 2181,    0,
 2181, 2181, 2181, 2181, 2181,    0,    0, 2181,    0,    0,
 2181,    0, 2181, 2181,    0,    0, 2181, 2181, 2181,    0,
 2181, 2181, 2181, 2181, 2181, 2181, 2181,    0,    0,    0,
 2181, 2181, 2181, 2181, 2181,    0,    0,    0, 2181,    0,
    0,    0,    0,    0,    0,    0, 2181, 2181, 2181, 2181,
 2181, 2181,    0,    0,    0, 2181, 2181,    0, 2181,    0,
 2181,    0, 2181, 2181, 2181,    0, 2181,    0, 2181, 2181,
 2181,    0, 2181, 2181, 2181, 2181, 2181,    0, 2181, 2181,
    0,    0,    0,    0, 2181,    0, 2181, 2181,    0,    0,
 2181, 2181, 2181,    0,    0, 2181,    0, 2181,    0,    0,
 2181, 2181, 2181, 2181,    0, 2181,    0, 2181,    0, 2181,
 2181, 2181, 2181, 2181,    0,    0, 2181, 2181, 2181,    0,
 2181, 2181,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2181, 2181, 2181,    0, 2181, 2181, 2181,    0,    0,
    0,    0,    0, 2181,    0,    0,    0, 2181,    0, 2181,
 2181, 2181, 2181,    0, 2181,    0, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2181, 2181,    0, 2181, 2181, 2181, 1457,
    0,    0,    0, 2181, 2181, 2181, 2181, 2181,    0, 2181,
    0,    0, 2181, 2181,    0,    0,    0, 2181, 2181,    0,
    0, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
    0, 2181,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181,    0, 2181, 2181, 2181, 2181, 2181, 2181,    0,
 2181, 2181, 2181, 2181,    0, 2181,    0, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181,    0, 2181, 2181, 2181, 2181,
    0, 2181, 2181, 2181, 2181, 2181,    0, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181,    0, 2181, 2181,    0,
    0,    0, 2181, 2181, 2181, 2181, 2181, 2181,    0,    0,
 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,    0,
    0,    0,    0, 2215, 2181,    0, 2181, 2181, 2181, 2181,
 2181, 2181,    0, 2181,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2181, 2181,    0, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181,    0, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181,    0, 2181, 2181, 2181,    0,
 2181,    0,    0,    0, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181, 2181,    0, 2181, 2181,
    0,    0,    0, 2182,    0, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181,    0,    0, 2181,    0, 2181, 2181, 2181,
 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181, 2181,
 2181, 2181, 2181, 1938, 2182, 2182, 2182, 2182, 2182, 2182,
 2182,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2182, 2182, 2182, 2182, 2182,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2182,    0, 2182,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2182,    0, 2182,    0,    0, 2182,    0, 2182, 2182, 2182,
    0, 2182, 2182,    0, 2182,    0,    0,    0,    0,    0,
 2182,    0, 2182, 2182, 2182, 2182,    0, 2182,    0,    0,
 2182,    0,    0, 2182, 1456,    0,    0,    0,    0,    0,
 2182,    0, 2182, 2182,    0, 2182, 2182, 2182, 2182, 2182,
    0,    0, 2182,    0,    0, 2182,    0, 2182, 2182,    0,
    0, 2182, 2182, 2182,    0, 2182, 2182, 2182, 2182, 2182,
 2182, 2182,    0,    0,    0, 2182, 2182, 2182, 2182, 2182,
    0,    0,    0, 2182,    0,    0,    0,    0,    0,    0,
    0, 2182, 2182, 2182, 2182, 2182, 2182,    0,    0,    0,
 2182, 2182,    0, 2182,    0, 2182,    0, 2182, 2182, 2182,
    0, 2182,    0, 2182, 2182, 2182,    0, 2182, 2182, 2182,
 2182, 2182,    0, 2182, 2182,    0,    0,    0,    0, 2182,
    0, 2182, 2182,    0,    0, 2182, 2182, 2182,    0,    0,
 2182,    0, 2182,    0,    0, 2182, 2182, 2182, 2182,    0,
 2182,    0, 2182,    0, 2182, 2182, 2182, 2182, 2182,    0,
    0, 2182, 2182, 2182,    0, 2182, 2182,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2182, 2182, 2182,    0,
 2182, 2182, 2182,    0,    0,    0,    0,    0, 2182,    0,
    0,    0, 2182,    0, 2182, 2182, 2182, 2182,    0, 2182,
    0, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2182, 2182,
    0, 2182, 2182, 2182, 1457,    0,    0,    0, 2182, 2182,
 2182, 2182, 2182,    0, 2182,    0,    0, 2182, 2182,    0,
    0,    0, 2182, 2182,    0,    0, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182,    0, 2182,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182,    0, 2182, 2182,
 2182, 2182, 2182, 2182,    0, 2182, 2182, 2182, 2182,    0,
 2182,    0, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
    0, 2182, 2182, 2182, 2182,    0, 2182, 2182, 2182, 2182,
 2182,    0, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182,    0, 2182, 2182,    0,    0,    0, 2182, 2182, 2182,
 2182, 2182, 2182,    0,    0, 2182, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182,    0,    0,    0,    0, 2220, 2182,
    0, 2182, 2182, 2182, 2182, 2182, 2182,    0, 2182,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2182, 2182,    0, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182,    0, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
    0, 2182, 2182, 2182,    0, 2182,    0,    0,    0, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182, 2182,    0, 2182, 2182,    0,    0,    0, 2183,    0,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,    0,    0,
 2182,    0, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182,
 2182, 2182, 2182, 2182, 2182, 2182, 2182, 2182, 1939, 2183,
 2183, 2183, 2183, 2183, 2183, 2183,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2183, 2183, 2183, 2183,
 2183,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2183,
    0, 2183,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2183,    0, 2183,    0,    0,
 2183,    0, 2183, 2183, 2183,    0, 2183, 2183,    0, 2183,
    0,    0,    0,    0,    0, 2183,    0, 2183, 2183, 2183,
 2183,    0, 2183,    0,    0, 2183,    0,    0, 2183, 1456,
    0,    0,    0,    0,    0, 2183,    0, 2183, 2183,    0,
 2183, 2183, 2183, 2183, 2183,    0,    0, 2183,    0,    0,
 2183,    0, 2183, 2183,    0,    0, 2183, 2183, 2183,    0,
 2183, 2183, 2183, 2183, 2183, 2183, 2183,    0,    0,    0,
 2183, 2183, 2183, 2183, 2183,    0,    0,    0, 2183,    0,
    0,    0,    0,    0,    0,    0, 2183, 2183, 2183, 2183,
 2183, 2183,    0,    0,    0, 2183, 2183,    0, 2183,    0,
 2183,    0, 2183, 2183, 2183,    0, 2183,    0, 2183, 2183,
 2183,    0, 2183, 2183, 2183, 2183, 2183,    0, 2183, 2183,
    0,    0,    0,    0, 2183,    0, 2183, 2183,    0,    0,
 2183, 2183, 2183,    0,    0, 2183,    0, 2183,    0,    0,
 2183, 2183, 2183, 2183,    0, 2183,    0, 2183,    0, 2183,
 2183, 2183, 2183, 2183,    0,    0, 2183, 2183, 2183,    0,
 2183, 2183,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2183, 2183, 2183,    0, 2183, 2183, 2183,    0,    0,
    0,    0,    0, 2183,    0,    0,    0, 2183,    0, 2183,
 2183, 2183, 2183,    0, 2183,    0, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2183, 2183,    0, 2183, 2183, 2183, 1457,
    0,    0,    0, 2183, 2183, 2183, 2183, 2183,    0, 2183,
    0,    0, 2183, 2183,    0,    0,    0, 2183, 2183,    0,
    0, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
    0, 2183,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183,    0, 2183, 2183, 2183, 2183, 2183, 2183,    0,
 2183, 2183, 2183, 2183,    0, 2183,    0, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183,    0, 2183, 2183, 2183, 2183,
    0, 2183, 2183, 2183, 2183, 2183,    0, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183,    0, 2183, 2183,    0,
    0,    0, 2183, 2183, 2183, 2183, 2183, 2183,    0,    0,
 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,    0,
    0,    0,    0, 2221, 2183,    0, 2183, 2183, 2183, 2183,
 2183, 2183,    0, 2183,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2183, 2183,    0, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183,    0, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183,    0, 2183, 2183, 2183,    0,
 2183,    0,    0,    0, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183, 2183,    0, 2183, 2183,
    0,    0,    0, 2184,    0, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183,    0,    0, 2183,    0, 2183, 2183, 2183,
 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183, 2183,
 2183, 2183, 2183, 1941, 2184, 2184, 2184, 2184, 2184, 2184,
 2184,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2184, 2184, 2184, 2184, 2184,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2184,    0, 2184,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2184,    0, 2184,    0,    0, 2184,    0, 2184, 2184, 2184,
    0, 2184, 2184,    0, 2184,    0,    0,    0,    0,    0,
 2184,    0, 2184, 2184, 2184, 2184,    0, 2184,    0,    0,
 2184,    0,    0, 2184, 1456,    0,    0,    0,    0,    0,
 2184,    0, 2184, 2184,    0, 2184, 2184, 2184, 2184, 2184,
    0,    0, 2184,    0,    0, 2184,    0, 2184, 2184,    0,
    0, 2184, 2184, 2184,    0, 2184, 2184, 2184, 2184, 2184,
 2184, 2184,    0,    0,    0, 2184, 2184, 2184, 2184, 2184,
    0,    0,    0, 2184,    0,    0,    0,    0,    0,    0,
    0, 2184, 2184, 2184, 2184, 2184, 2184,    0,    0,    0,
 2184, 2184,    0, 2184,    0, 2184,    0, 2184, 2184, 2184,
    0, 2184,    0, 2184, 2184, 2184,    0, 2184, 2184, 2184,
 2184, 2184,    0, 2184, 2184,    0,    0,    0,    0, 2184,
    0, 2184, 2184,    0,    0, 2184, 2184, 2184,    0,    0,
 2184,    0, 2184,    0,    0, 2184, 2184, 2184, 2184,    0,
 2184,    0, 2184,    0, 2184, 2184, 2184, 2184, 2184,    0,
    0, 2184, 2184, 2184,    0, 2184, 2184,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2184, 2184, 2184,    0,
 2184, 2184, 2184,    0,    0,    0,    0,    0, 2184,    0,
    0,    0, 2184,    0, 2184, 2184, 2184, 2184,    0, 2184,
    0, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2184, 2184,
    0, 2184, 2184, 2184, 1457,    0,    0,    0, 2184, 2184,
 2184, 2184, 2184,    0, 2184,    0,    0, 2184, 2184,    0,
    0,    0, 2184, 2184,    0,    0, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184,    0, 2184,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184,    0, 2184, 2184,
 2184, 2184, 2184, 2184,    0, 2184, 2184, 2184, 2184,    0,
 2184,    0, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
    0, 2184, 2184, 2184, 2184,    0, 2184, 2184, 2184, 2184,
 2184,    0, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184,    0, 2184, 2184,    0,    0,    0, 2184, 2184, 2184,
 2184, 2184, 2184,    0,    0, 2184, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184,    0,    0,    0,    0, 2222, 2184,
    0, 2184, 2184, 2184, 2184, 2184, 2184,    0, 2184,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2184, 2184,    0, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184,    0, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
    0, 2184, 2184, 2184,    0, 2184,    0,    0,    0, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184, 2184,    0, 2184, 2184,    0,    0,    0, 2185,    0,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,    0,    0,
 2184,    0, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184,
 2184, 2184, 2184, 2184, 2184, 2184, 2184, 2184, 1943, 2185,
 2185, 2185, 2185, 2185, 2185, 2185,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2185, 2185, 2185, 2185,
 2185,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2185,
    0, 2185,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2185,    0, 2185,    0,    0,
 2185,    0, 2185, 2185, 2185,    0, 2185, 2185,    0, 2185,
    0,    0,    0,    0,    0, 2185,    0, 2185, 2185, 2185,
 2185,    0, 2185,    0,    0, 2185,    0,    0, 2185, 1456,
    0,    0,    0,    0,    0, 2185,    0, 2185, 2185,    0,
 2185, 2185, 2185, 2185, 2185,    0,    0, 2185,    0,    0,
 2185,    0, 2185, 2185,    0,    0, 2185, 2185, 2185,    0,
 2185, 2185, 2185, 2185, 2185, 2185, 2185,    0,    0,    0,
 2185, 2185, 2185, 2185, 2185,    0,    0,    0, 2185,    0,
    0,    0,    0,    0,    0,    0, 2185, 2185, 2185, 2185,
 2185, 2185,    0,    0,    0, 2185, 2185,    0, 2185,    0,
 2185,    0, 2185, 2185, 2185,    0, 2185,    0, 2185, 2185,
 2185,    0, 2185, 2185, 2185, 2185, 2185,    0, 2185, 2185,
    0,    0,    0,    0, 2185,    0, 2185, 2185,    0,    0,
 2185, 2185, 2185,    0,    0, 2185,    0, 2185,    0,    0,
 2185, 2185, 2185, 2185,    0, 2185,    0, 2185,    0, 2185,
 2185, 2185, 2185, 2185,    0,    0, 2185, 2185, 2185,    0,
 2185, 2185,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2185, 2185, 2185,    0, 2185, 2185, 2185,    0,    0,
    0,    0,    0, 2185,    0,    0,    0, 2185,    0, 2185,
 2185, 2185, 2185,    0, 2185,    0, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2185, 2185,    0, 2185, 2185, 2185, 1457,
    0,    0,    0, 2185, 2185, 2185, 2185, 2185,    0, 2185,
    0,    0, 2185, 2185,    0,    0,    0, 2185, 2185,    0,
    0, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
    0, 2185,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185,    0, 2185, 2185, 2185, 2185, 2185, 2185,    0,
 2185, 2185, 2185, 2185,    0, 2185,    0, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185,    0, 2185, 2185, 2185, 2185,
    0, 2185, 2185, 2185, 2185, 2185,    0, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185,    0, 2185, 2185,    0,
    0,    0, 2185, 2185, 2185, 2185, 2185, 2185,    0,    0,
 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,    0,
    0,    0,    0, 2223, 2185,    0, 2185, 2185, 2185, 2185,
 2185, 2185,    0, 2185,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2185, 2185,    0, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185,    0, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185,    0, 2185, 2185, 2185,    0,
 2185,    0,    0,    0, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185, 2185,    0, 2185, 2185,
    0,    0,    0, 2186,    0, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185,    0,    0, 2185,    0, 2185, 2185, 2185,
 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185, 2185,
 2185, 2185, 2185, 1944, 2186, 2186, 2186, 2186, 2186, 2186,
 2186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2186, 2186, 2186, 2186, 2186,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2186,    0, 2186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2186,    0, 2186,    0,    0, 2186,    0, 2186, 2186, 2186,
    0, 2186, 2186,    0, 2186,    0,    0,    0,    0,    0,
 2186,    0, 2186, 2186, 2186, 2186,    0, 2186,    0,    0,
 2186,    0,    0, 2186, 1456,    0,    0,    0,    0,    0,
 2186,    0, 2186, 2186,    0, 2186, 2186, 2186, 2186, 2186,
    0,    0, 2186,    0,    0, 2186,    0, 2186, 2186,    0,
    0, 2186, 2186, 2186,    0, 2186, 2186, 2186, 2186, 2186,
 2186, 2186,    0,    0,    0, 2186, 2186, 2186, 2186, 2186,
    0,    0,    0, 2186,    0,    0,    0,    0,    0,    0,
    0, 2186, 2186, 2186, 2186, 2186, 2186,    0,    0,    0,
 2186, 2186,    0, 2186,    0, 2186,    0, 2186, 2186, 2186,
    0, 2186,    0, 2186, 2186, 2186,    0, 2186, 2186, 2186,
 2186, 2186,    0, 2186, 2186,    0,    0,    0,    0, 2186,
    0, 2186, 2186,    0,    0, 2186, 2186, 2186,    0,    0,
 2186,    0, 2186,    0,    0, 2186, 2186, 2186, 2186,    0,
 2186,    0, 2186,    0, 2186, 2186, 2186, 2186, 2186,    0,
    0, 2186, 2186, 2186,    0, 2186, 2186,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2186, 2186, 2186,    0,
 2186, 2186, 2186,    0,    0,    0,    0,    0, 2186,    0,
    0,    0, 2186,    0, 2186, 2186, 2186, 2186,    0, 2186,
    0, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2186, 2186,
    0, 2186, 2186, 2186, 1457,    0,    0,    0, 2186, 2186,
 2186, 2186, 2186,    0, 2186,    0,    0, 2186, 2186,    0,
    0,    0, 2186, 2186,    0,    0, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186,    0, 2186,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186,    0, 2186, 2186,
 2186, 2186, 2186, 2186,    0, 2186, 2186, 2186, 2186,    0,
 2186,    0, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
    0, 2186, 2186, 2186, 2186,    0, 2186, 2186, 2186, 2186,
 2186,    0, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186,    0, 2186, 2186,    0,    0,    0, 2186, 2186, 2186,
 2186, 2186, 2186,    0,    0, 2186, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186,    0,    0,    0,    0, 2608, 2186,
    0, 2186, 2186, 2186, 2186, 2186, 2186,    0, 2186,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2186, 2186,    0, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186,    0, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
    0, 2186, 2186, 2186,    0, 2186,    0,    0,    0, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186, 2186,    0, 2186, 2186,    0,    0,    0, 2187,    0,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,    0,    0,
 2186,    0, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186,
 2186, 2186, 2186, 2186, 2186, 2186, 2186, 2186, 1945, 2187,
 2187, 2187, 2187, 2187, 2187, 2187,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2187, 2187, 2187, 2187,
 2187,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2187,
    0, 2187,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2187,    0, 2187,    0,    0,
 2187,    0, 2187, 2187, 2187,    0, 2187, 2187,    0, 2187,
    0,    0,    0,    0,    0, 2187,    0, 2187, 2187, 2187,
 2187,    0, 2187,    0,    0, 2187,    0,    0, 2187, 1456,
    0,    0,    0,    0,    0, 2187,    0, 2187, 2187,    0,
 2187, 2187, 2187, 2187, 2187,    0,    0, 2187,    0,    0,
 2187,    0, 2187, 2187,    0,    0, 2187, 2187, 2187,    0,
 2187, 2187, 2187, 2187, 2187, 2187, 2187,    0,    0,    0,
 2187, 2187, 2187, 2187, 2187,    0,    0,    0, 2187,    0,
    0,    0,    0,    0,    0,    0, 2187, 2187, 2187, 2187,
 2187, 2187,    0,    0,    0, 2187, 2187,    0, 2187,    0,
 2187,    0, 2187, 2187, 2187,    0, 2187,    0, 2187, 2187,
 2187,    0, 2187, 2187, 2187, 2187, 2187,    0, 2187, 2187,
    0,    0,    0,    0, 2187,    0, 2187, 2187,    0,    0,
 2187, 2187, 2187,    0,    0, 2187,    0, 2187,    0,    0,
 2187, 2187, 2187, 2187,    0, 2187,    0, 2187,    0, 2187,
 2187, 2187, 2187, 2187,    0,    0, 2187, 2187, 2187,    0,
 2187, 2187,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2187, 2187, 2187,    0, 2187, 2187, 2187,    0,    0,
    0,    0,    0, 2187,    0,    0,    0, 2187,    0, 2187,
 2187, 2187, 2187,    0, 2187,    0, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2187, 2187,    0, 2187, 2187, 2187, 1457,
    0,    0,    0, 2187, 2187, 2187, 2187, 2187,    0, 2187,
    0,    0, 2187, 2187,    0,    0,    0, 2187, 2187,    0,
    0, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
    0, 2187,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187,    0, 2187, 2187, 2187, 2187, 2187, 2187,    0,
 2187, 2187, 2187, 2187,    0, 2187,    0, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187,    0, 2187, 2187, 2187, 2187,
    0, 2187, 2187, 2187, 2187, 2187,    0, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187,    0, 2187, 2187,    0,
    0,    0, 2187, 2187, 2187, 2187, 2187, 2187,    0,    0,
 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,    0,
    0,    0,    0, 2609, 2187,    0, 2187, 2187, 2187, 2187,
 2187, 2187,    0, 2187,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2187, 2187,    0, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187,    0, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187,    0, 2187, 2187, 2187,    0,
 2187,    0,    0,    0, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187, 2187,    0, 2187, 2187,
    0,    0,    0, 2191,    0, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187,    0,    0, 2187,    0, 2187, 2187, 2187,
 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187, 2187,
 2187, 2187, 2187, 1946, 2191, 2191, 2191, 2191, 2191, 2191,
 2191,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2191, 2191, 2191, 2191, 2191,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2191,    0, 2191,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2191,    0, 2191,    0,    0, 2191,    0, 2191, 2191, 2191,
    0, 2191, 2191,    0, 2191,    0,    0,    0,    0,    0,
 2191,    0, 2191, 2191, 2191, 2191,    0, 2191,    0,    0,
 2191,    0,    0, 2191, 1456,    0,    0,    0,    0,    0,
 2191,    0, 2191, 2191,    0, 2191, 2191, 2191, 2191, 2191,
    0,    0, 2191,    0,    0, 2191,    0, 2191, 2191,    0,
    0, 2191, 2191, 2191,    0, 2191, 2191, 2191, 2191, 2191,
 2191, 2191,    0,    0,    0, 2191, 2191, 2191, 2191, 2191,
    0,    0,    0, 2191,    0,    0,    0,    0,    0,    0,
    0, 2191, 2191, 2191, 2191, 2191, 2191,    0,    0,    0,
 2191, 2191,    0, 2191,    0, 2191,    0, 2191, 2191, 2191,
    0, 2191,    0, 2191, 2191, 2191,    0, 2191, 2191, 2191,
 2191, 2191,    0, 2191, 2191,    0,    0,    0,    0, 2191,
    0, 2191, 2191,    0,    0, 2191, 2191, 2191,    0,    0,
 2191,    0, 2191,    0,    0, 2191, 2191, 2191, 2191,    0,
 2191,    0, 2191,    0, 2191, 2191, 2191, 2191, 2191,    0,
    0, 2191, 2191, 2191,    0, 2191, 2191,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2191, 2191, 2191,    0,
 2191, 2191, 2191,    0,    0,    0,    0,    0, 2191,    0,
    0,    0, 2191,    0, 2191, 2191, 2191, 2191,    0, 2191,
    0, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2191, 2191,
    0, 2191, 2191, 2191, 1457,    0,    0,    0, 2191, 2191,
 2191, 2191, 2191,    0, 2191,    0,    0, 2191, 2191,    0,
    0,    0, 2191, 2191,    0,    0, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191,    0, 2191,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191,    0, 2191, 2191,
 2191, 2191, 2191, 2191,    0, 2191, 2191, 2191, 2191,    0,
 2191,    0, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
    0, 2191, 2191, 2191, 2191,    0, 2191, 2191, 2191, 2191,
 2191,    0, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191,    0, 2191, 2191,    0,    0,    0, 2191, 2191, 2191,
 2191, 2191, 2191,    0,    0, 2191, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191,    0,    0,    0,    0, 2614, 2191,
    0, 2191, 2191, 2191, 2191, 2191, 2191,    0, 2191,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2191, 2191,    0, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191,    0, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
    0, 2191, 2191, 2191,    0, 2191,    0,    0,    0, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191, 2191,    0, 2191, 2191,    0,    0,    0, 2192,    0,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,    0,    0,
 2191,    0, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191,
 2191, 2191, 2191, 2191, 2191, 2191, 2191, 2191, 1947, 2192,
 2192, 2192, 2192, 2192, 2192, 2192,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2192, 2192, 2192, 2192,
 2192,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2192,
    0, 2192,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2192,    0, 2192,    0,    0,
 2192,    0, 2192, 2192, 2192,    0, 2192, 2192,    0, 2192,
    0,    0,    0,    0,    0, 2192,    0, 2192, 2192, 2192,
 2192,    0, 2192,    0,    0, 2192,    0,    0, 2192, 1456,
    0,    0,    0,    0,    0, 2192,    0, 2192, 2192,    0,
 2192, 2192, 2192, 2192, 2192,    0,    0, 2192,    0,    0,
 2192,    0, 2192, 2192,    0,    0, 2192, 2192, 2192,    0,
 2192, 2192, 2192, 2192, 2192, 2192, 2192,    0,    0,    0,
 2192, 2192, 2192, 2192, 2192,    0,    0,    0, 2192,    0,
    0,    0,    0,    0,    0,    0, 2192, 2192, 2192, 2192,
 2192, 2192,    0,    0,    0, 2192, 2192,    0, 2192,    0,
 2192,    0, 2192, 2192, 2192,    0, 2192,    0, 2192, 2192,
 2192,    0, 2192, 2192, 2192, 2192, 2192,    0, 2192, 2192,
    0,    0,    0,    0, 2192,    0, 2192, 2192,    0,    0,
 2192, 2192, 2192,    0,    0, 2192,    0, 2192,    0,    0,
 2192, 2192, 2192, 2192,    0, 2192,    0, 2192,    0, 2192,
 2192, 2192, 2192, 2192,    0,    0, 2192, 2192, 2192,    0,
 2192, 2192,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2192, 2192, 2192,    0, 2192, 2192, 2192,    0,    0,
    0,    0,    0, 2192,    0,    0,    0, 2192,    0, 2192,
 2192, 2192, 2192,    0, 2192,    0, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2192, 2192,    0, 2192, 2192, 2192, 1457,
    0,    0,    0, 2192, 2192, 2192, 2192, 2192,    0, 2192,
    0,    0, 2192, 2192,    0,    0,    0, 2192, 2192,    0,
    0, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
    0, 2192,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192,    0, 2192, 2192, 2192, 2192, 2192, 2192,    0,
 2192, 2192, 2192, 2192,    0, 2192,    0, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192,    0, 2192, 2192, 2192, 2192,
    0, 2192, 2192, 2192, 2192, 2192,    0, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192,    0, 2192, 2192,    0,
    0,    0, 2192, 2192, 2192, 2192, 2192, 2192,    0,    0,
 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,    0,
    0,    0,    0, 2615, 2192,    0, 2192, 2192, 2192, 2192,
 2192, 2192,    0, 2192,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2192, 2192,    0, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192,    0, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192,    0, 2192, 2192, 2192,    0,
 2192,    0,    0,    0, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192, 2192,    0, 2192, 2192,
    0,    0,    0, 2193,    0, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192,    0,    0, 2192,    0, 2192, 2192, 2192,
 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192, 2192,
 2192, 2192, 2192, 1948, 2193, 2193, 2193, 2193, 2193, 2193,
 2193,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2193, 2193, 2193, 2193, 2193,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2193,    0, 2193,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2193,    0, 2193,    0,    0, 2193,    0, 2193, 2193, 2193,
    0, 2193, 2193,    0, 2193,    0,    0,    0,    0,    0,
 2193,    0, 2193, 2193, 2193, 2193,    0, 2193,    0,    0,
 2193,    0,    0, 2193, 1456,    0,    0,    0,    0,    0,
 2193,    0, 2193, 2193,    0, 2193, 2193, 2193, 2193, 2193,
    0,    0, 2193,    0,    0, 2193,    0, 2193, 2193,    0,
    0, 2193, 2193, 2193,    0, 2193, 2193, 2193, 2193, 2193,
 2193, 2193,    0,    0,    0, 2193, 2193, 2193, 2193, 2193,
    0,    0,    0, 2193,    0,    0,    0,    0,    0,    0,
    0, 2193, 2193, 2193, 2193, 2193, 2193,    0,    0,    0,
 2193, 2193,    0, 2193,    0, 2193,    0, 2193, 2193, 2193,
    0, 2193,    0, 2193, 2193, 2193,    0, 2193, 2193, 2193,
 2193, 2193,    0, 2193, 2193,    0,    0,    0,    0, 2193,
    0, 2193, 2193,    0,    0, 2193, 2193, 2193,    0,    0,
 2193,    0, 2193,    0,    0, 2193, 2193, 2193, 2193,    0,
 2193,    0, 2193,    0, 2193, 2193, 2193, 2193, 2193,    0,
    0, 2193, 2193, 2193,    0, 2193, 2193,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2193, 2193, 2193,    0,
 2193, 2193, 2193,    0,    0,    0,    0,    0, 2193,    0,
    0,    0, 2193,    0, 2193, 2193, 2193, 2193,    0, 2193,
    0, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2193, 2193,
    0, 2193, 2193, 2193, 1457,    0,    0,    0, 2193, 2193,
 2193, 2193, 2193,    0, 2193,    0,    0, 2193, 2193,    0,
    0,    0, 2193, 2193,    0,    0, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193,    0, 2193,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193,    0, 2193, 2193,
 2193, 2193, 2193, 2193,    0, 2193, 2193, 2193, 2193,    0,
 2193,    0, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
    0, 2193, 2193, 2193, 2193,    0, 2193, 2193, 2193, 2193,
 2193,    0, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193,    0, 2193, 2193,    0,    0,    0, 2193, 2193, 2193,
 2193, 2193, 2193,    0,    0, 2193, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193,    0,    0,    0,    0, 2617, 2193,
    0, 2193, 2193, 2193, 2193, 2193, 2193,    0, 2193,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2193, 2193,    0, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193,    0, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
    0, 2193, 2193, 2193,    0, 2193,    0,    0,    0, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193, 2193,    0, 2193, 2193,    0,    0,    0, 2197,    0,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,    0,    0,
 2193,    0, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193,
 2193, 2193, 2193, 2193, 2193, 2193, 2193, 2193, 1952, 2197,
 2197, 2197, 2197, 2197, 2197, 2197,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2197, 2197, 2197, 2197,
 2197,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2197,
    0, 2197,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2197,    0, 2197,    0,    0,
 2197,    0, 2197, 2197, 2197,    0, 2197, 2197,    0, 2197,
    0,    0,    0,    0,    0, 2197,    0, 2197, 2197, 2197,
 2197,    0, 2197,    0,    0, 2197,    0,    0, 2197, 1456,
    0,    0,    0,    0,    0, 2197,    0, 2197, 2197,    0,
 2197, 2197, 2197, 2197, 2197,    0,    0, 2197,    0,    0,
 2197,    0, 2197, 2197,    0,    0, 2197, 2197, 2197,    0,
 2197, 2197, 2197, 2197, 2197, 2197, 2197,    0,    0,    0,
 2197, 2197, 2197, 2197, 2197,    0,    0,    0, 2197,    0,
    0,    0,    0,    0,    0,    0, 2197, 2197, 2197, 2197,
 2197, 2197,    0,    0,    0, 2197, 2197,    0, 2197,    0,
 2197,    0, 2197, 2197, 2197,    0, 2197,    0, 2197, 2197,
 2197,    0, 2197, 2197, 2197, 2197, 2197,    0, 2197, 2197,
    0,    0,    0,    0, 2197,    0, 2197, 2197,    0,    0,
 2197, 2197, 2197,    0,    0, 2197,    0, 2197,    0,    0,
 2197, 2197, 2197, 2197,    0, 2197,    0, 2197,    0, 2197,
 2197, 2197, 2197, 2197,    0,    0, 2197, 2197, 2197,    0,
 2197, 2197,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2197, 2197, 2197,    0, 2197, 2197, 2197,    0,    0,
    0,    0,    0, 2197,    0,    0,    0, 2197,    0, 2197,
 2197, 2197, 2197,    0, 2197,    0, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2197, 2197,    0, 2197, 2197, 2197, 1457,
    0,    0,    0, 2197, 2197, 2197, 2197, 2197,    0, 2197,
    0,    0, 2197, 2197,    0,    0,    0, 2197, 2197,    0,
    0, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
    0, 2197,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197,    0, 2197, 2197, 2197, 2197, 2197, 2197,    0,
 2197, 2197, 2197, 2197,    0, 2197,    0, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197,    0, 2197, 2197, 2197, 2197,
    0, 2197, 2197, 2197, 2197, 2197,    0, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197,    0, 2197, 2197,    0,
    0,    0, 2197, 2197, 2197, 2197, 2197, 2197,    0,    0,
 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,    0,
    0,    0,    0, 2618, 2197,    0, 2197, 2197, 2197, 2197,
 2197, 2197,    0, 2197,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2197, 2197,    0, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197,    0, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197,    0, 2197, 2197, 2197,    0,
 2197,    0,    0,    0, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197, 2197,    0, 2197, 2197,
    0,    0,    0, 2198,    0, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197,    0,    0, 2197,    0, 2197, 2197, 2197,
 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197, 2197,
 2197, 2197, 2197, 1953, 2198, 2198, 2198, 2198, 2198, 2198,
 2198,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2198, 2198, 2198, 2198, 2198,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2198,    0, 2198,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2198,    0, 2198,    0,    0, 2198,    0, 2198, 2198, 2198,
    0, 2198, 2198,    0, 2198,    0,    0,    0,    0,    0,
 2198,    0, 2198, 2198, 2198, 2198,    0, 2198,    0,    0,
 2198,    0,    0, 2198, 1456,    0,    0,    0,    0,    0,
 2198,    0, 2198, 2198,    0, 2198, 2198, 2198, 2198, 2198,
    0,    0, 2198,    0,    0, 2198,    0, 2198, 2198,    0,
    0, 2198, 2198, 2198,    0, 2198, 2198, 2198, 2198, 2198,
 2198, 2198,    0,    0,    0, 2198, 2198, 2198, 2198, 2198,
    0,    0,    0, 2198,    0,    0,    0,    0,    0,    0,
    0, 2198, 2198, 2198, 2198, 2198, 2198,    0,    0,    0,
 2198, 2198,    0, 2198,    0, 2198,    0, 2198, 2198, 2198,
    0, 2198,    0, 2198, 2198, 2198,    0, 2198, 2198, 2198,
 2198, 2198,    0, 2198, 2198,    0,    0,    0,    0, 2198,
    0, 2198, 2198,    0,    0, 2198, 2198, 2198,    0,    0,
 2198,    0, 2198,    0,    0, 2198, 2198, 2198, 2198,    0,
 2198,    0, 2198,    0, 2198, 2198, 2198, 2198, 2198,    0,
    0, 2198, 2198, 2198,    0, 2198, 2198,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2198, 2198, 2198,    0,
 2198, 2198, 2198,    0,    0,    0,    0,    0, 2198,    0,
    0,    0, 2198,    0, 2198, 2198, 2198, 2198,    0, 2198,
    0, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2198, 2198,
    0, 2198, 2198, 2198, 1457,    0,    0,    0, 2198, 2198,
 2198, 2198, 2198,    0, 2198,    0,    0, 2198, 2198,    0,
    0,    0, 2198, 2198,    0,    0, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198,    0, 2198,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198,    0, 2198, 2198,
 2198, 2198, 2198, 2198,    0, 2198, 2198, 2198, 2198,    0,
 2198,    0, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
    0, 2198, 2198, 2198, 2198,    0, 2198, 2198, 2198, 2198,
 2198,    0, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198,    0, 2198, 2198,    0,    0,    0, 2198, 2198, 2198,
 2198, 2198, 2198,    0,    0, 2198, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198,    0,    0,    0,    0, 2619, 2198,
    0, 2198, 2198, 2198, 2198, 2198, 2198,    0, 2198,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2198, 2198,    0, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198,    0, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
    0, 2198, 2198, 2198,    0, 2198,    0,    0,    0, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198, 2198,    0, 2198, 2198,    0,    0,    0, 2199,    0,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,    0,    0,
 2198,    0, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198,
 2198, 2198, 2198, 2198, 2198, 2198, 2198, 2198, 1954, 2199,
 2199, 2199, 2199, 2199, 2199, 2199,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2199, 2199, 2199, 2199,
 2199,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2199,
    0, 2199,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2199,    0, 2199,    0,    0,
 2199,    0, 2199, 2199, 2199,    0, 2199, 2199,    0, 2199,
    0,    0,    0,    0,    0, 2199,    0, 2199, 2199, 2199,
 2199,    0, 2199,    0,    0, 2199,    0,    0, 2199, 1456,
    0,    0,    0,    0,    0, 2199,    0, 2199, 2199,    0,
 2199, 2199, 2199, 2199, 2199,    0,    0, 2199,    0,    0,
 2199,    0, 2199, 2199,    0,    0, 2199, 2199, 2199,    0,
 2199, 2199, 2199, 2199, 2199, 2199, 2199,    0,    0,    0,
 2199, 2199, 2199, 2199, 2199,    0,    0,    0, 2199,    0,
    0,    0,    0,    0,    0,    0, 2199, 2199, 2199, 2199,
 2199, 2199,    0,    0,    0, 2199, 2199,    0, 2199,    0,
 2199,    0, 2199, 2199, 2199,    0, 2199,    0, 2199, 2199,
 2199,    0, 2199, 2199, 2199, 2199, 2199,    0, 2199, 2199,
    0,    0,    0,    0, 2199,    0, 2199, 2199,    0,    0,
 2199, 2199, 2199,    0,    0, 2199,    0, 2199,    0,    0,
 2199, 2199, 2199, 2199,    0, 2199,    0, 2199,    0, 2199,
 2199, 2199, 2199, 2199,    0,    0, 2199, 2199, 2199,    0,
 2199, 2199,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2199, 2199, 2199,    0, 2199, 2199, 2199,    0,    0,
    0,    0,    0, 2199,    0,    0,    0, 2199,    0, 2199,
 2199, 2199, 2199,    0, 2199,    0, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2199, 2199,    0, 2199, 2199, 2199, 1457,
    0,    0,    0, 2199, 2199, 2199, 2199, 2199,    0, 2199,
    0,    0, 2199, 2199,    0,    0,    0, 2199, 2199,    0,
    0, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
    0, 2199,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199,    0, 2199, 2199, 2199, 2199, 2199, 2199,    0,
 2199, 2199, 2199, 2199,    0, 2199,    0, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199,    0, 2199, 2199, 2199, 2199,
    0, 2199, 2199, 2199, 2199, 2199,    0, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199,    0, 2199, 2199,    0,
    0,    0, 2199, 2199, 2199, 2199, 2199, 2199,    0,    0,
 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,    0,
    0,    0,    0, 2620, 2199,    0, 2199, 2199, 2199, 2199,
 2199, 2199,    0, 2199,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2199, 2199,    0, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199,    0, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199,    0, 2199, 2199, 2199,    0,
 2199,    0,    0,    0, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199, 2199,    0, 2199, 2199,
    0,    0,    0, 2200,    0, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199,    0,    0, 2199,    0, 2199, 2199, 2199,
 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199, 2199,
 2199, 2199, 2199, 1955, 2200, 2200, 2200, 2200, 2200, 2200,
 2200,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2200, 2200, 2200, 2200, 2200,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2200,    0, 2200,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2200,    0, 2200,    0,    0, 2200,    0, 2200, 2200, 2200,
    0, 2200, 2200,    0, 2200,    0,    0,    0,    0,    0,
 2200,    0, 2200, 2200, 2200, 2200,    0, 2200,    0,    0,
 2200,    0,    0, 2200, 1456,    0,    0,    0,    0,    0,
 2200,    0, 2200, 2200,    0, 2200, 2200, 2200, 2200, 2200,
    0,    0, 2200,    0,    0, 2200,    0, 2200, 2200,    0,
    0, 2200, 2200, 2200,    0, 2200, 2200, 2200, 2200, 2200,
 2200, 2200,    0,    0,    0, 2200, 2200, 2200, 2200, 2200,
    0,    0,    0, 2200,    0,    0,    0,    0,    0,    0,
    0, 2200, 2200, 2200, 2200, 2200, 2200,    0,    0,    0,
 2200, 2200,    0, 2200,    0, 2200,    0, 2200, 2200, 2200,
    0, 2200,    0, 2200, 2200, 2200,    0, 2200, 2200, 2200,
 2200, 2200,    0, 2200, 2200,    0,    0,    0,    0, 2200,
    0, 2200, 2200,    0,    0, 2200, 2200, 2200,    0,    0,
 2200,    0, 2200,    0,    0, 2200, 2200, 2200, 2200,    0,
 2200,    0, 2200,    0, 2200, 2200, 2200, 2200, 2200,    0,
    0, 2200, 2200, 2200,    0, 2200, 2200,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2200, 2200, 2200,    0,
 2200, 2200, 2200,    0,    0,    0,    0,    0, 2200,    0,
    0,    0, 2200,    0, 2200, 2200, 2200, 2200,    0, 2200,
    0, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2200, 2200,
    0, 2200, 2200, 2200, 1457,    0,    0,    0, 2200, 2200,
 2200, 2200, 2200,    0, 2200,    0,    0, 2200, 2200,    0,
    0,    0, 2200, 2200,    0,    0, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200,    0, 2200,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200,    0, 2200, 2200,
 2200, 2200, 2200, 2200,    0, 2200, 2200, 2200, 2200,    0,
 2200,    0, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
    0, 2200, 2200, 2200, 2200,    0, 2200, 2200, 2200, 2200,
 2200,    0, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200,    0, 2200, 2200,    0,    0,    0, 2200, 2200, 2200,
 2200, 2200, 2200,    0,    0, 2200, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200,    0,    0,    0,    0, 2623, 2200,
    0, 2200, 2200, 2200, 2200, 2200, 2200,    0, 2200,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2200, 2200,    0, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200,    0, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
    0, 2200, 2200, 2200,    0, 2200,    0,    0,    0, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200, 2200,    0, 2200, 2200,    0,    0,    0, 2203,    0,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,    0,    0,
 2200,    0, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200,
 2200, 2200, 2200, 2200, 2200, 2200, 2200, 2200, 1957, 2203,
 2203, 2203, 2203, 2203, 2203, 2203,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2203, 2203, 2203, 2203,
 2203,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2203,
    0, 2203,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2203,    0, 2203,    0,    0,
 2203,    0, 2203, 2203, 2203,    0, 2203, 2203,    0, 2203,
    0,    0,    0,    0,    0, 2203,    0, 2203, 2203, 2203,
 2203,    0, 2203,    0,    0, 2203,    0,    0, 2203, 1456,
    0,    0,    0,    0,    0, 2203,    0, 2203, 2203,    0,
 2203, 2203, 2203, 2203, 2203,    0,    0, 2203,    0,    0,
 2203,    0, 2203, 2203,    0,    0, 2203, 2203, 2203,    0,
 2203, 2203, 2203, 2203, 2203, 2203, 2203,    0,    0,    0,
 2203, 2203, 2203, 2203, 2203,    0,    0,    0, 2203,    0,
    0,    0,    0,    0,    0,    0, 2203, 2203, 2203, 2203,
 2203, 2203,    0,    0,    0, 2203, 2203,    0, 2203,    0,
 2203,    0, 2203, 2203, 2203,    0, 2203,    0, 2203, 2203,
 2203,    0, 2203, 2203, 2203, 2203, 2203,    0, 2203, 2203,
    0,    0,    0,    0, 2203,    0, 2203, 2203,    0,    0,
 2203, 2203, 2203,    0,    0, 2203,    0, 2203,    0,    0,
 2203, 2203, 2203, 2203,    0, 2203,    0, 2203,    0, 2203,
 2203, 2203, 2203, 2203,    0,    0, 2203, 2203, 2203,    0,
 2203, 2203,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2203, 2203, 2203,    0, 2203, 2203, 2203,    0,    0,
    0,    0,    0, 2203,    0,    0,    0, 2203,    0, 2203,
 2203, 2203, 2203,    0, 2203,    0, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2203, 2203,    0, 2203, 2203, 2203, 1457,
    0,    0,    0, 2203, 2203, 2203, 2203, 2203,    0, 2203,
    0,    0, 2203, 2203,    0,    0,    0, 2203, 2203,    0,
    0, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
    0, 2203,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203,    0, 2203, 2203, 2203, 2203, 2203, 2203,    0,
 2203, 2203, 2203, 2203,    0, 2203,    0, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203,    0, 2203, 2203, 2203, 2203,
    0, 2203, 2203, 2203, 2203, 2203,    0, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203,    0, 2203, 2203,    0,
    0,    0, 2203, 2203, 2203, 2203, 2203, 2203,    0,    0,
 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,    0,
    0,    0,    0, 2624, 2203,    0, 2203, 2203, 2203, 2203,
 2203, 2203,    0, 2203,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2203, 2203,    0, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203,    0, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203,    0, 2203, 2203, 2203,    0,
 2203,    0,    0,    0, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203, 2203,    0, 2203, 2203,
    0,    0,    0, 2205,    0, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203,    0,    0, 2203,    0, 2203, 2203, 2203,
 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203, 2203,
 2203, 2203, 2203, 1958, 2205, 2205, 2205, 2205, 2205, 2205,
 2205,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2205, 2205, 2205, 2205, 2205,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2205,    0, 2205,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2205,    0, 2205,    0,    0, 2205,    0, 2205, 2205, 2205,
    0, 2205, 2205,    0, 2205,    0,    0,    0,    0,    0,
 2205,    0, 2205, 2205, 2205, 2205,    0, 2205,    0,    0,
 2205,    0,    0, 2205, 1456,    0,    0,    0,    0,    0,
 2205,    0, 2205, 2205,    0, 2205, 2205, 2205, 2205, 2205,
    0,    0, 2205,    0,    0, 2205,    0, 2205, 2205,    0,
    0, 2205, 2205, 2205,    0, 2205, 2205, 2205, 2205, 2205,
 2205, 2205,    0,    0,    0, 2205, 2205, 2205, 2205, 2205,
    0,    0,    0, 2205,    0,    0,    0,    0,    0,    0,
    0, 2205, 2205, 2205, 2205, 2205, 2205,    0,    0,    0,
 2205, 2205,    0, 2205,    0, 2205,    0, 2205, 2205, 2205,
    0, 2205,    0, 2205, 2205, 2205,    0, 2205, 2205, 2205,
 2205, 2205,    0, 2205, 2205,    0,    0,    0,    0, 2205,
    0, 2205, 2205,    0,    0, 2205, 2205, 2205,    0,    0,
 2205,    0, 2205,    0,    0, 2205, 2205, 2205, 2205,    0,
 2205,    0, 2205,    0, 2205, 2205, 2205, 2205, 2205,    0,
    0, 2205, 2205, 2205,    0, 2205, 2205,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2205, 2205, 2205,    0,
 2205, 2205, 2205,    0,    0,    0,    0,    0, 2205,    0,
    0,    0, 2205,    0, 2205, 2205, 2205, 2205,    0, 2205,
    0, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2205, 2205,
    0, 2205, 2205, 2205, 1457,    0,    0,    0, 2205, 2205,
 2205, 2205, 2205,    0, 2205,    0,    0, 2205, 2205,    0,
    0,    0, 2205, 2205,    0,    0, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205,    0, 2205,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205,    0, 2205, 2205,
 2205, 2205, 2205, 2205,    0, 2205, 2205, 2205, 2205,    0,
 2205,    0, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
    0, 2205, 2205, 2205, 2205,    0, 2205, 2205, 2205, 2205,
 2205,    0, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205,    0, 2205, 2205,    0,    0,    0, 2205, 2205, 2205,
 2205, 2205, 2205,    0,    0, 2205, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205,    0,    0,    0,    0, 2969, 2205,
    0, 2205, 2205, 2205, 2205, 2205, 2205,    0, 2205,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2205, 2205,    0, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205,    0, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
    0, 2205, 2205, 2205,    0, 2205,    0,    0,    0, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205, 2205,    0, 2205, 2205,    0,    0,    0, 2206,    0,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,    0,    0,
 2205,    0, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205,
 2205, 2205, 2205, 2205, 2205, 2205, 2205, 2205, 1959, 2206,
 2206, 2206, 2206, 2206, 2206, 2206,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2206, 2206, 2206, 2206,
 2206,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2206,
    0, 2206,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2206,    0, 2206,    0,    0,
 2206,    0, 2206, 2206, 2206,    0, 2206, 2206,    0, 2206,
    0,    0,    0,    0,    0, 2206,    0, 2206, 2206, 2206,
 2206,    0, 2206,    0,    0, 2206,    0,    0, 2206, 1456,
    0,    0,    0,    0,    0, 2206,    0, 2206, 2206,    0,
 2206, 2206, 2206, 2206, 2206,    0,    0, 2206,    0,    0,
 2206,    0, 2206, 2206,    0,    0, 2206, 2206, 2206,    0,
 2206, 2206, 2206, 2206, 2206, 2206, 2206,    0,    0,    0,
 2206, 2206, 2206, 2206, 2206,    0,    0,    0, 2206,    0,
    0,    0,    0,    0,    0,    0, 2206, 2206, 2206, 2206,
 2206, 2206,    0,    0,    0, 2206, 2206,    0, 2206,    0,
 2206,    0, 2206, 2206, 2206,    0, 2206,    0, 2206, 2206,
 2206,    0, 2206, 2206, 2206, 2206, 2206,    0, 2206, 2206,
    0,    0,    0,    0, 2206,    0, 2206, 2206,    0,    0,
 2206, 2206, 2206,    0,    0, 2206,    0, 2206,    0,    0,
 2206, 2206, 2206, 2206,    0, 2206,    0, 2206,    0, 2206,
 2206, 2206, 2206, 2206,    0,    0, 2206, 2206, 2206,    0,
 2206, 2206,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2206, 2206, 2206,    0, 2206, 2206, 2206,    0,    0,
    0,    0,    0, 2206,    0,    0,    0, 2206,    0, 2206,
 2206, 2206, 2206,    0, 2206,    0, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2206, 2206,    0, 2206, 2206, 2206, 1457,
    0,    0,    0, 2206, 2206, 2206, 2206, 2206,    0, 2206,
    0,    0, 2206, 2206,    0,    0,    0, 2206, 2206,    0,
    0, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
    0, 2206,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206,    0, 2206, 2206, 2206, 2206, 2206, 2206,    0,
 2206, 2206, 2206, 2206,    0, 2206,    0, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206,    0, 2206, 2206, 2206, 2206,
    0, 2206, 2206, 2206, 2206, 2206,    0, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206,    0, 2206, 2206,    0,
    0,    0, 2206, 2206, 2206, 2206, 2206, 2206,    0,    0,
 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,    0,
    0,    0,    0, 2971, 2206,    0, 2206, 2206, 2206, 2206,
 2206, 2206,    0, 2206,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2206, 2206,    0, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206,    0, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206,    0, 2206, 2206, 2206,    0,
 2206,    0,    0,    0, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206, 2206,    0, 2206, 2206,
    0,    0,    0, 2209,    0, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206,    0,    0, 2206,    0, 2206, 2206, 2206,
 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206, 2206,
 2206, 2206, 2206, 1960, 2209, 2209, 2209, 2209, 2209, 2209,
 2209,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2209, 2209, 2209, 2209, 2209,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2209,    0, 2209,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2209,    0, 2209,    0,    0, 2209,    0, 2209, 2209, 2209,
    0, 2209, 2209,    0, 2209,    0,    0,    0,    0,    0,
 2209,    0, 2209, 2209, 2209, 2209,    0, 2209,    0,    0,
 2209,    0,    0, 2209, 1456,    0,    0,    0,    0,    0,
 2209,    0, 2209, 2209,    0, 2209, 2209, 2209, 2209, 2209,
    0,    0, 2209,    0,    0, 2209,    0, 2209, 2209,    0,
    0, 2209, 2209, 2209,    0, 2209, 2209, 2209, 2209, 2209,
 2209, 2209,    0,    0,    0, 2209, 2209, 2209, 2209, 2209,
    0,    0,    0, 2209,    0,    0,    0,    0,    0,    0,
    0, 2209, 2209, 2209, 2209, 2209, 2209,    0,    0,    0,
 2209, 2209,    0, 2209,    0, 2209,    0, 2209, 2209, 2209,
    0, 2209,    0, 2209, 2209, 2209,    0, 2209, 2209, 2209,
 2209, 2209,    0, 2209, 2209,    0,    0,    0,    0, 2209,
    0, 2209, 2209,    0,    0, 2209, 2209, 2209,    0,    0,
 2209,    0, 2209,    0,    0, 2209, 2209, 2209, 2209,    0,
 2209,    0, 2209,    0, 2209, 2209, 2209, 2209, 2209,    0,
    0, 2209, 2209, 2209,    0, 2209, 2209,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2209, 2209, 2209,    0,
 2209, 2209, 2209,    0,    0,    0,    0,    0, 2209,    0,
    0,    0, 2209,    0, 2209, 2209, 2209, 2209,    0, 2209,
    0, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2209, 2209,
    0, 2209, 2209, 2209, 1457,    0,    0,    0, 2209, 2209,
 2209, 2209, 2209,    0, 2209,    0,    0, 2209, 2209,    0,
    0,    0, 2209, 2209,    0,    0, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209,    0, 2209,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209,    0, 2209, 2209,
 2209, 2209, 2209, 2209,    0, 2209, 2209, 2209, 2209,    0,
 2209,    0, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
    0, 2209, 2209, 2209, 2209,    0, 2209, 2209, 2209, 2209,
 2209,    0, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209,    0, 2209, 2209,    0,    0,    0, 2209, 2209, 2209,
 2209, 2209, 2209,    0,    0, 2209, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209,    0,    0,    0,    0, 2972, 2209,
    0, 2209, 2209, 2209, 2209, 2209, 2209,    0, 2209,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2209, 2209,    0, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209,    0, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
    0, 2209, 2209, 2209,    0, 2209,    0,    0,    0, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209, 2209,    0, 2209, 2209,    0,    0,    0, 2211,    0,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,    0,    0,
 2209,    0, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209,
 2209, 2209, 2209, 2209, 2209, 2209, 2209, 2209, 1961, 2211,
 2211, 2211, 2211, 2211, 2211, 2211,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2211, 2211, 2211, 2211,
 2211,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2211,
    0, 2211,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2211,    0, 2211,    0,    0,
 2211,    0, 2211, 2211, 2211,    0, 2211, 2211,    0, 2211,
    0,    0,    0,    0,    0, 2211,    0, 2211, 2211, 2211,
 2211,    0, 2211,    0,    0, 2211,    0,    0, 2211, 1456,
    0,    0,    0,    0,    0, 2211,    0, 2211, 2211,    0,
 2211, 2211, 2211, 2211, 2211,    0,    0, 2211,    0,    0,
 2211,    0, 2211, 2211,    0,    0, 2211, 2211, 2211,    0,
 2211, 2211, 2211, 2211, 2211, 2211, 2211,    0,    0,    0,
 2211, 2211, 2211, 2211, 2211,    0,    0,    0, 2211,    0,
    0,    0,    0,    0,    0,    0, 2211, 2211, 2211, 2211,
 2211, 2211,    0,    0,    0, 2211, 2211,    0, 2211,    0,
 2211,    0, 2211, 2211, 2211,    0, 2211,    0, 2211, 2211,
 2211,    0, 2211, 2211, 2211, 2211, 2211,    0, 2211, 2211,
    0,    0,    0,    0, 2211,    0, 2211, 2211,    0,    0,
 2211, 2211, 2211,    0,    0, 2211,    0, 2211,    0,    0,
 2211, 2211, 2211, 2211,    0, 2211,    0, 2211,    0, 2211,
 2211, 2211, 2211, 2211,    0,    0, 2211, 2211, 2211,    0,
 2211, 2211,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2211, 2211, 2211,    0, 2211, 2211, 2211,    0,    0,
    0,    0,    0, 2211,    0,    0,    0, 2211,    0, 2211,
 2211, 2211, 2211,    0, 2211,    0, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2211, 2211,    0, 2211, 2211, 2211, 1457,
    0,    0,    0, 2211, 2211, 2211, 2211, 2211,    0, 2211,
    0,    0, 2211, 2211,    0,    0,    0, 2211, 2211,    0,
    0, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
    0, 2211,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211,    0, 2211, 2211, 2211, 2211, 2211, 2211,    0,
 2211, 2211, 2211, 2211,    0, 2211,    0, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211,    0, 2211, 2211, 2211, 2211,
    0, 2211, 2211, 2211, 2211, 2211,    0, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211,    0, 2211, 2211,    0,
    0,    0, 2211, 2211, 2211, 2211, 2211, 2211,    0,    0,
 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,    0,
    0,    0,    0, 2979, 2211,    0, 2211, 2211, 2211, 2211,
 2211, 2211,    0, 2211,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2211, 2211,    0, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211,    0, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211,    0, 2211, 2211, 2211,    0,
 2211,    0,    0,    0, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211, 2211,    0, 2211, 2211,
    0,    0,    0, 2213,    0, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211,    0,    0, 2211,    0, 2211, 2211, 2211,
 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211, 2211,
 2211, 2211, 2211, 1962, 2213, 2213, 2213, 2213, 2213, 2213,
 2213,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2213, 2213, 2213, 2213, 2213,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2213,    0, 2213,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2213,    0, 2213,    0,    0, 2213,    0, 2213, 2213, 2213,
    0, 2213, 2213,    0, 2213,    0,    0,    0,    0,    0,
 2213,    0, 2213, 2213, 2213, 2213,    0, 2213,    0,    0,
 2213,    0,    0, 2213, 1456,    0,    0,    0,    0,    0,
 2213,    0, 2213, 2213,    0, 2213, 2213, 2213, 2213, 2213,
    0,    0, 2213,    0,    0, 2213,    0, 2213, 2213,    0,
    0, 2213, 2213, 2213,    0, 2213, 2213, 2213, 2213, 2213,
 2213, 2213,    0,    0,    0, 2213, 2213, 2213, 2213, 2213,
    0,    0,    0, 2213,    0,    0,    0,    0,    0,    0,
    0, 2213, 2213, 2213, 2213, 2213, 2213,    0,    0,    0,
 2213, 2213,    0, 2213,    0, 2213,    0, 2213, 2213, 2213,
    0, 2213,    0, 2213, 2213, 2213,    0, 2213, 2213, 2213,
 2213, 2213,    0, 2213, 2213,    0,    0,    0,    0, 2213,
    0, 2213, 2213,    0,    0, 2213, 2213, 2213,    0,    0,
 2213,    0, 2213,    0,    0, 2213, 2213, 2213, 2213,    0,
 2213,    0, 2213,    0, 2213, 2213, 2213, 2213, 2213,    0,
    0, 2213, 2213, 2213,    0, 2213, 2213,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2213, 2213, 2213,    0,
 2213, 2213, 2213,    0,    0,    0,    0,    0, 2213,    0,
    0,    0, 2213,    0, 2213, 2213, 2213, 2213,    0, 2213,
    0, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2213, 2213,
    0, 2213, 2213, 2213, 1457,    0,    0,    0, 2213, 2213,
 2213, 2213, 2213,    0, 2213,    0,    0, 2213, 2213,    0,
    0,    0, 2213, 2213,    0,    0, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213,    0, 2213,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213,    0, 2213, 2213,
 2213, 2213, 2213, 2213,    0, 2213, 2213, 2213, 2213,    0,
 2213,    0, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
    0, 2213, 2213, 2213, 2213,    0, 2213, 2213, 2213, 2213,
 2213,    0, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213,    0, 2213, 2213,    0,    0,    0, 2213, 2213, 2213,
 2213, 2213, 2213,    0,    0, 2213, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213,    0,    0,    0,    0, 2982, 2213,
    0, 2213, 2213, 2213, 2213, 2213, 2213,    0, 2213,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2213, 2213,    0, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213,    0, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
    0, 2213, 2213, 2213,    0, 2213,    0,    0,    0, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213, 2213,    0, 2213, 2213,    0,    0,    0, 2214,    0,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,    0,    0,
 2213,    0, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213,
 2213, 2213, 2213, 2213, 2213, 2213, 2213, 2213, 1967, 2214,
 2214, 2214, 2214, 2214, 2214, 2214,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2214, 2214, 2214, 2214,
 2214,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2214,
    0, 2214,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2214,    0, 2214,    0,    0,
 2214,    0, 2214, 2214, 2214,    0, 2214, 2214,    0, 2214,
    0,    0,    0,    0,    0, 2214,    0, 2214, 2214, 2214,
 2214,    0, 2214,    0,    0, 2214,    0,    0, 2214, 1456,
    0,    0,    0,    0,    0, 2214,    0, 2214, 2214,    0,
 2214, 2214, 2214, 2214, 2214,    0,    0, 2214,    0,    0,
 2214,    0, 2214, 2214,    0,    0, 2214, 2214, 2214,    0,
 2214, 2214, 2214, 2214, 2214, 2214, 2214,    0,    0,    0,
 2214, 2214, 2214, 2214, 2214,    0,    0,    0, 2214,    0,
    0,    0,    0,    0,    0,    0, 2214, 2214, 2214, 2214,
 2214, 2214,    0,    0,    0, 2214, 2214,    0, 2214,    0,
 2214,    0, 2214, 2214, 2214,    0, 2214,    0, 2214, 2214,
 2214,    0, 2214, 2214, 2214, 2214, 2214,    0, 2214, 2214,
    0,    0,    0,    0, 2214,    0, 2214, 2214,    0,    0,
 2214, 2214, 2214,    0,    0, 2214,    0, 2214,    0,    0,
 2214, 2214, 2214, 2214,    0, 2214,    0, 2214,    0, 2214,
 2214, 2214, 2214, 2214,    0,    0, 2214, 2214, 2214,    0,
 2214, 2214,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2214, 2214, 2214,    0, 2214, 2214, 2214,    0,    0,
    0,    0,    0, 2214,    0,    0,    0, 2214,    0, 2214,
 2214, 2214, 2214,    0, 2214,    0, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2214, 2214,    0, 2214, 2214, 2214, 1457,
    0,    0,    0, 2214, 2214, 2214, 2214, 2214,    0, 2214,
    0,    0, 2214, 2214,    0,    0,    0, 2214, 2214,    0,
    0, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
    0, 2214,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214,    0, 2214, 2214, 2214, 2214, 2214, 2214,    0,
 2214, 2214, 2214, 2214,    0, 2214,    0, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214,    0, 2214, 2214, 2214, 2214,
    0, 2214, 2214, 2214, 2214, 2214,    0, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214,    0, 2214, 2214,    0,
    0,    0, 2214, 2214, 2214, 2214, 2214, 2214,    0,    0,
 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,    0,
    0,    0,    0, 2983, 2214,    0, 2214, 2214, 2214, 2214,
 2214, 2214,    0, 2214,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2214, 2214,    0, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214,    0, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214,    0, 2214, 2214, 2214,    0,
 2214,    0,    0,    0, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214, 2214,    0, 2214, 2214,
    0,    0,    0, 2215,    0, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214,    0,    0, 2214,    0, 2214, 2214, 2214,
 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214, 2214,
 2214, 2214, 2214, 1968, 2215, 2215, 2215, 2215, 2215, 2215,
 2215,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2215, 2215, 2215, 2215, 2215,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2215,    0, 2215,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2215,    0, 2215,    0,    0, 2215,    0, 2215, 2215, 2215,
    0, 2215, 2215,    0, 2215,    0,    0,    0,    0,    0,
 2215,    0, 2215, 2215, 2215, 2215,    0, 2215,    0,    0,
 2215,    0,    0, 2215, 1456,    0,    0,    0,    0,    0,
 2215,    0, 2215, 2215,    0, 2215, 2215, 2215, 2215, 2215,
    0,    0, 2215,    0,    0, 2215,    0, 2215, 2215,    0,
    0, 2215, 2215, 2215,    0, 2215, 2215, 2215, 2215, 2215,
 2215, 2215,    0,    0,    0, 2215, 2215, 2215, 2215, 2215,
    0,    0,    0, 2215,    0,    0,    0,    0,    0,    0,
    0, 2215, 2215, 2215, 2215, 2215, 2215,    0,    0,    0,
 2215, 2215,    0, 2215,    0, 2215,    0, 2215, 2215, 2215,
    0, 2215,    0, 2215, 2215, 2215,    0, 2215, 2215, 2215,
 2215, 2215,    0, 2215, 2215,    0,    0,    0,    0, 2215,
    0, 2215, 2215,    0,    0, 2215, 2215, 2215,    0,    0,
 2215,    0, 2215,    0,    0, 2215, 2215, 2215, 2215,    0,
 2215,    0, 2215,    0, 2215, 2215, 2215, 2215, 2215,    0,
    0, 2215, 2215, 2215,    0, 2215, 2215,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2215, 2215, 2215,    0,
 2215, 2215, 2215,    0,    0,    0,    0,    0, 2215,    0,
    0,    0, 2215,    0, 2215, 2215, 2215, 2215,    0, 2215,
    0, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2215, 2215,
    0, 2215, 2215, 2215, 1457,    0,    0,    0, 2215, 2215,
 2215, 2215, 2215,    0, 2215,    0,    0, 2215, 2215,    0,
    0,    0, 2215, 2215,    0,    0, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215,    0, 2215,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215,    0, 2215, 2215,
 2215, 2215, 2215, 2215,    0, 2215, 2215, 2215, 2215,    0,
 2215,    0, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
    0, 2215, 2215, 2215, 2215,    0, 2215, 2215, 2215, 2215,
 2215,    0, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215,    0, 2215, 2215,    0,    0,    0, 2215, 2215, 2215,
 2215, 2215, 2215,    0,    0, 2215, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215,    0,    0,    0,    0, 2996, 2215,
    0, 2215, 2215, 2215, 2215, 2215, 2215,    0, 2215,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2215, 2215,    0, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215,    0, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
    0, 2215, 2215, 2215,    0, 2215,    0,    0,    0, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215, 2215,    0, 2215, 2215,    0,    0,    0, 2216,    0,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,    0,    0,
 2215,    0, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215,
 2215, 2215, 2215, 2215, 2215, 2215, 2215, 2215, 1970, 2216,
 2216, 2216, 2216, 2216, 2216, 2216,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2216, 2216, 2216, 2216,
 2216,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2216,
    0, 2216,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2216,    0, 2216,    0,    0,
 2216,    0, 2216, 2216, 2216,    0, 2216, 2216,    0, 2216,
    0,    0,    0,    0,    0, 2216,    0, 2216, 2216, 2216,
 2216,    0, 2216,    0,    0, 2216,    0,    0, 2216, 1456,
    0,    0,    0,    0,    0, 2216,    0, 2216, 2216,    0,
 2216, 2216, 2216, 2216, 2216,    0,    0, 2216,    0,    0,
 2216,    0, 2216, 2216,    0,    0, 2216, 2216, 2216,    0,
 2216, 2216, 2216, 2216, 2216, 2216, 2216,    0,    0,    0,
 2216, 2216, 2216, 2216, 2216,    0,    0,    0, 2216,    0,
    0,    0,    0,    0,    0,    0, 2216, 2216, 2216, 2216,
 2216, 2216,    0,    0,    0, 2216, 2216,    0, 2216,    0,
 2216,    0, 2216, 2216, 2216,    0, 2216,    0, 2216, 2216,
 2216,    0, 2216, 2216, 2216, 2216, 2216,    0, 2216, 2216,
    0,    0,    0,    0, 2216,    0, 2216, 2216,    0,    0,
 2216, 2216, 2216,    0,    0, 2216,    0, 2216,    0,    0,
 2216, 2216, 2216, 2216,    0, 2216,    0, 2216,    0, 2216,
 2216, 2216, 2216, 2216,    0,    0, 2216, 2216, 2216,    0,
 2216, 2216,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2216, 2216, 2216,    0, 2216, 2216, 2216,    0,    0,
    0,    0,    0, 2216,    0,    0,    0, 2216,    0, 2216,
 2216, 2216, 2216,    0, 2216,    0, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2216, 2216,    0, 2216, 2216, 2216, 1457,
    0,    0,    0, 2216, 2216, 2216, 2216, 2216,    0, 2216,
    0,    0, 2216, 2216,    0,    0,    0, 2216, 2216,    0,
    0, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
    0, 2216,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216,    0, 2216, 2216, 2216, 2216, 2216, 2216,    0,
 2216, 2216, 2216, 2216,    0, 2216,    0, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216,    0, 2216, 2216, 2216, 2216,
    0, 2216, 2216, 2216, 2216, 2216,    0, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216,    0, 2216, 2216,    0,
    0,    0, 2216, 2216, 2216, 2216, 2216, 2216,    0,    0,
 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,    0,
    0,    0,    0, 3001, 2216,    0, 2216, 2216, 2216, 2216,
 2216, 2216,    0, 2216,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2216, 2216,    0, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216,    0, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216,    0, 2216, 2216, 2216,    0,
 2216,    0,    0,    0, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216, 2216,    0, 2216, 2216,
    0,    0,    0, 2217,    0, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216,    0,    0, 2216,    0, 2216, 2216, 2216,
 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216, 2216,
 2216, 2216, 2216, 1971, 2217, 2217, 2217, 2217, 2217, 2217,
 2217,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2217, 2217, 2217, 2217, 2217,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2217,    0, 2217,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2217,    0, 2217,    0,    0, 2217,    0, 2217, 2217, 2217,
    0, 2217, 2217,    0, 2217,    0,    0,    0,    0,    0,
 2217,    0, 2217, 2217, 2217, 2217,    0, 2217,    0,    0,
 2217,    0,    0, 2217, 1456,    0,    0,    0,    0,    0,
 2217,    0, 2217, 2217,    0, 2217, 2217, 2217, 2217, 2217,
    0,    0, 2217,    0,    0, 2217,    0, 2217, 2217,    0,
    0, 2217, 2217, 2217,    0, 2217, 2217, 2217, 2217, 2217,
 2217, 2217,    0,    0,    0, 2217, 2217, 2217, 2217, 2217,
    0,    0,    0, 2217,    0,    0,    0,    0,    0,    0,
    0, 2217, 2217, 2217, 2217, 2217, 2217,    0,    0,    0,
 2217, 2217,    0, 2217,    0, 2217,    0, 2217, 2217, 2217,
    0, 2217,    0, 2217, 2217, 2217,    0, 2217, 2217, 2217,
 2217, 2217,    0, 2217, 2217,    0,    0,    0,    0, 2217,
    0, 2217, 2217,    0,    0, 2217, 2217, 2217,    0,    0,
 2217,    0, 2217,    0,    0, 2217, 2217, 2217, 2217,    0,
 2217,    0, 2217,    0, 2217, 2217, 2217, 2217, 2217,    0,
    0, 2217, 2217, 2217,    0, 2217, 2217,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2217, 2217, 2217,    0,
 2217, 2217, 2217,    0,    0,    0,    0,    0, 2217,    0,
    0,    0, 2217,    0, 2217, 2217, 2217, 2217,    0, 2217,
    0, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2217, 2217,
    0, 2217, 2217, 2217, 1457,    0,    0,    0, 2217, 2217,
 2217, 2217, 2217,    0, 2217,    0,    0, 2217, 2217,    0,
    0,    0, 2217, 2217,    0,    0, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217,    0, 2217,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217,    0, 2217, 2217,
 2217, 2217, 2217, 2217,    0, 2217, 2217, 2217, 2217,    0,
 2217,    0, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
    0, 2217, 2217, 2217, 2217,    0, 2217, 2217, 2217, 2217,
 2217,    0, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217,    0, 2217, 2217,    0,    0,    0, 2217, 2217, 2217,
 2217, 2217, 2217,    0,    0, 2217, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217,    0,    0,    0,    0, 3002, 2217,
    0, 2217, 2217, 2217, 2217, 2217, 2217,    0, 2217,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2217, 2217,    0, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217,    0, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
    0, 2217, 2217, 2217,    0, 2217,    0,    0,    0, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217, 2217,    0, 2217, 2217,    0,    0,    0, 2218,    0,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,    0,    0,
 2217,    0, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217,
 2217, 2217, 2217, 2217, 2217, 2217, 2217, 2217, 1974, 2218,
 2218, 2218, 2218, 2218, 2218, 2218,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2218, 2218, 2218, 2218,
 2218,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2218,
    0, 2218,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2218,    0, 2218,    0,    0,
 2218,    0, 2218, 2218, 2218,    0, 2218, 2218,    0, 2218,
    0,    0,    0,    0,    0, 2218,    0, 2218, 2218, 2218,
 2218,    0, 2218,    0,    0, 2218,    0,    0, 2218, 1456,
    0,    0,    0,    0,    0, 2218,    0, 2218, 2218,    0,
 2218, 2218, 2218, 2218, 2218,    0,    0, 2218,    0,    0,
 2218,    0, 2218, 2218,    0,    0, 2218, 2218, 2218,    0,
 2218, 2218, 2218, 2218, 2218, 2218, 2218,    0,    0,    0,
 2218, 2218, 2218, 2218, 2218,    0,    0,    0, 2218,    0,
    0,    0,    0,    0,    0,    0, 2218, 2218, 2218, 2218,
 2218, 2218,    0,    0,    0, 2218, 2218,    0, 2218,    0,
 2218,    0, 2218, 2218, 2218,    0, 2218,    0, 2218, 2218,
 2218,    0, 2218, 2218, 2218, 2218, 2218,    0, 2218, 2218,
    0,    0,    0,    0, 2218,    0, 2218, 2218,    0,    0,
 2218, 2218, 2218,    0,    0, 2218,    0, 2218,    0,    0,
 2218, 2218, 2218, 2218,    0, 2218,    0, 2218,    0, 2218,
 2218, 2218, 2218, 2218,    0,    0, 2218, 2218, 2218,    0,
 2218, 2218,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2218, 2218, 2218,    0, 2218, 2218, 2218,    0,    0,
    0,    0,    0, 2218,    0,    0,    0, 2218,    0, 2218,
 2218, 2218, 2218,    0, 2218,    0, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2218, 2218,    0, 2218, 2218, 2218, 1457,
    0,    0,    0, 2218, 2218, 2218, 2218, 2218,    0, 2218,
    0,    0, 2218, 2218,    0,    0,    0, 2218, 2218,    0,
    0, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
    0, 2218,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218,    0, 2218, 2218, 2218, 2218, 2218, 2218,    0,
 2218, 2218, 2218, 2218,    0, 2218,    0, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218,    0, 2218, 2218, 2218, 2218,
    0, 2218, 2218, 2218, 2218, 2218,    0, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218,    0, 2218, 2218,    0,
    0,    0, 2218, 2218, 2218, 2218, 2218, 2218,    0,    0,
 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,    0,
    0,    0,    0, 3003, 2218,    0, 2218, 2218, 2218, 2218,
 2218, 2218,    0, 2218,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2218, 2218,    0, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218,    0, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218,    0, 2218, 2218, 2218,    0,
 2218,    0,    0,    0, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218, 2218,    0, 2218, 2218,
    0,    0,    0, 2219,    0, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218,    0,    0, 2218,    0, 2218, 2218, 2218,
 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218, 2218,
 2218, 2218, 2218, 1975, 2219, 2219, 2219, 2219, 2219, 2219,
 2219,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2219, 2219, 2219, 2219, 2219,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2219,    0, 2219,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2219,    0, 2219,    0,    0, 2219,    0, 2219, 2219, 2219,
    0, 2219, 2219,    0, 2219,    0,    0,    0,    0,    0,
 2219,    0, 2219, 2219, 2219, 2219,    0, 2219,    0,    0,
 2219,    0,    0, 2219, 1456,    0,    0,    0,    0,    0,
 2219,    0, 2219, 2219,    0, 2219, 2219, 2219, 2219, 2219,
    0,    0, 2219,    0,    0, 2219,    0, 2219, 2219,    0,
    0, 2219, 2219, 2219,    0, 2219, 2219, 2219, 2219, 2219,
 2219, 2219,    0,    0,    0, 2219, 2219, 2219, 2219, 2219,
    0,    0,    0, 2219,    0,    0,    0,    0,    0,    0,
    0, 2219, 2219, 2219, 2219, 2219, 2219,    0,    0,    0,
 2219, 2219,    0, 2219,    0, 2219,    0, 2219, 2219, 2219,
    0, 2219,    0, 2219, 2219, 2219,    0, 2219, 2219, 2219,
 2219, 2219,    0, 2219, 2219,    0,    0,    0,    0, 2219,
    0, 2219, 2219,    0,    0, 2219, 2219, 2219,    0,    0,
 2219,    0, 2219,    0,    0, 2219, 2219, 2219, 2219,    0,
 2219,    0, 2219,    0, 2219, 2219, 2219, 2219, 2219,    0,
    0, 2219, 2219, 2219,    0, 2219, 2219,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2219, 2219, 2219,    0,
 2219, 2219, 2219,    0,    0,    0,    0,    0, 2219,    0,
    0,    0, 2219,    0, 2219, 2219, 2219, 2219,    0, 2219,
    0, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2219, 2219,
    0, 2219, 2219, 2219, 1457,    0,    0,    0, 2219, 2219,
 2219, 2219, 2219,    0, 2219,    0,    0, 2219, 2219,    0,
    0,    0, 2219, 2219,    0,    0, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219,    0, 2219,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219,    0, 2219, 2219,
 2219, 2219, 2219, 2219,    0, 2219, 2219, 2219, 2219,    0,
 2219,    0, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
    0, 2219, 2219, 2219, 2219,    0, 2219, 2219, 2219, 2219,
 2219,    0, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219,    0, 2219, 2219,    0,    0,    0, 2219, 2219, 2219,
 2219, 2219, 2219,    0,    0, 2219, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219,    0,    0,    0,    0, 3004, 2219,
    0, 2219, 2219, 2219, 2219, 2219, 2219,    0, 2219,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2219, 2219,    0, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219,    0, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
    0, 2219, 2219, 2219,    0, 2219,    0,    0,    0, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219, 2219,    0, 2219, 2219,    0,    0,    0, 2220,    0,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,    0,    0,
 2219,    0, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219,
 2219, 2219, 2219, 2219, 2219, 2219, 2219, 2219, 1976, 2220,
 2220, 2220, 2220, 2220, 2220, 2220,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2220, 2220, 2220, 2220,
 2220,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2220,
    0, 2220,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2220,    0, 2220,    0,    0,
 2220,    0, 2220, 2220, 2220,    0, 2220, 2220,    0, 2220,
    0,    0,    0,    0,    0, 2220,    0, 2220, 2220, 2220,
 2220,    0, 2220,    0,    0, 2220,    0,    0, 2220, 1456,
    0,    0,    0,    0,    0, 2220,    0, 2220, 2220,    0,
 2220, 2220, 2220, 2220, 2220,    0,    0, 2220,    0,    0,
 2220,    0, 2220, 2220,    0,    0, 2220, 2220, 2220,    0,
 2220, 2220, 2220, 2220, 2220, 2220, 2220,    0,    0,    0,
 2220, 2220, 2220, 2220, 2220,    0,    0,    0, 2220,    0,
    0,    0,    0,    0,    0,    0, 2220, 2220, 2220, 2220,
 2220, 2220,    0,    0,    0, 2220, 2220,    0, 2220,    0,
 2220,    0, 2220, 2220, 2220,    0, 2220,    0, 2220, 2220,
 2220,    0, 2220, 2220, 2220, 2220, 2220,    0, 2220, 2220,
    0,    0,    0,    0, 2220,    0, 2220, 2220,    0,    0,
 2220, 2220, 2220,    0,    0, 2220,    0, 2220,    0,    0,
 2220, 2220, 2220, 2220,    0, 2220,    0, 2220,    0, 2220,
 2220, 2220, 2220, 2220,    0,    0, 2220, 2220, 2220,    0,
 2220, 2220,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2220, 2220, 2220,    0, 2220, 2220, 2220,    0,    0,
    0,    0,    0, 2220,    0,    0,    0, 2220,    0, 2220,
 2220, 2220, 2220,    0, 2220,    0, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2220, 2220,    0, 2220, 2220, 2220, 1457,
    0,    0,    0, 2220, 2220, 2220, 2220, 2220,    0, 2220,
    0,    0, 2220, 2220,    0,    0,    0, 2220, 2220,    0,
    0, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
    0, 2220,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220,    0, 2220, 2220, 2220, 2220, 2220, 2220,    0,
 2220, 2220, 2220, 2220,    0, 2220,    0, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220,    0, 2220, 2220, 2220, 2220,
    0, 2220, 2220, 2220, 2220, 2220,    0, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220,    0, 2220, 2220,    0,
    0,    0, 2220, 2220, 2220, 2220, 2220, 2220,    0,    0,
 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,    0,
    0,    0,    0, 3005, 2220,    0, 2220, 2220, 2220, 2220,
 2220, 2220,    0, 2220,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2220, 2220,    0, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220,    0, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220,    0, 2220, 2220, 2220,    0,
 2220,    0,    0,    0, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220, 2220,    0, 2220, 2220,
    0,    0,    0, 2222,    0, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220,    0,    0, 2220,    0, 2220, 2220, 2220,
 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220, 2220,
 2220, 2220, 2220, 1977, 2222, 2222, 2222, 2222, 2222, 2222,
 2222,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2222, 2222, 2222, 2222, 2222,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2222,    0, 2222,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2222,    0, 2222,    0,    0, 2222,    0, 2222, 2222, 2222,
    0, 2222, 2222,    0, 2222,    0,    0,    0,    0,    0,
 2222,    0, 2222, 2222, 2222, 2222,    0, 2222,    0,    0,
 2222,    0,    0, 2222, 1456,    0,    0,    0,    0,    0,
 2222,    0, 2222, 2222,    0, 2222, 2222, 2222, 2222, 2222,
    0,    0, 2222,    0,    0, 2222,    0, 2222, 2222,    0,
    0, 2222, 2222, 2222,    0, 2222, 2222, 2222, 2222, 2222,
 2222, 2222,    0,    0,    0, 2222, 2222, 2222, 2222, 2222,
    0,    0,    0, 2222,    0,    0,    0,    0,    0,    0,
    0, 2222, 2222, 2222, 2222, 2222, 2222,    0,    0,    0,
 2222, 2222,    0, 2222,    0, 2222,    0, 2222, 2222, 2222,
    0, 2222,    0, 2222, 2222, 2222,    0, 2222, 2222, 2222,
 2222, 2222,    0, 2222, 2222,    0,    0,    0,    0, 2222,
    0, 2222, 2222,    0,    0, 2222, 2222, 2222,    0,    0,
 2222,    0, 2222,    0,    0, 2222, 2222, 2222, 2222,    0,
 2222,    0, 2222,    0, 2222, 2222, 2222, 2222, 2222,    0,
    0, 2222, 2222, 2222,    0, 2222, 2222,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2222, 2222, 2222,    0,
 2222, 2222, 2222,    0,    0,    0,    0,    0, 2222,    0,
    0,    0, 2222,    0, 2222, 2222, 2222, 2222,    0, 2222,
    0, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2222, 2222,
    0, 2222, 2222, 2222, 1457,    0,    0,    0, 2222, 2222,
 2222, 2222, 2222,    0, 2222,    0,    0, 2222, 2222,    0,
    0,    0, 2222, 2222,    0,    0, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222,    0, 2222,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222,    0, 2222, 2222,
 2222, 2222, 2222, 2222,    0, 2222, 2222, 2222, 2222,    0,
 2222,    0, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
    0, 2222, 2222, 2222, 2222,    0, 2222, 2222, 2222, 2222,
 2222,    0, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222,    0, 2222, 2222,    0,    0,    0, 2222, 2222, 2222,
 2222, 2222, 2222,    0,    0, 2222, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222,    0,    0,    0,    0, 3006, 2222,
    0, 2222, 2222, 2222, 2222, 2222, 2222,    0, 2222,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2222, 2222,    0, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222,    0, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
    0, 2222, 2222, 2222,    0, 2222,    0,    0,    0, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222, 2222,    0, 2222, 2222,    0,    0,    0, 2223,    0,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,    0,    0,
 2222,    0, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222,
 2222, 2222, 2222, 2222, 2222, 2222, 2222, 2222, 1978, 2223,
 2223, 2223, 2223, 2223, 2223, 2223,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2223, 2223, 2223, 2223,
 2223,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2223,
    0, 2223,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2223,    0, 2223,    0,    0,
 2223,    0, 2223, 2223, 2223,    0, 2223, 2223,    0, 2223,
    0,    0,    0,    0,    0, 2223,    0, 2223, 2223, 2223,
 2223,    0, 2223,    0,    0, 2223,    0,    0, 2223, 1456,
    0,    0,    0,    0,    0, 2223,    0, 2223, 2223,    0,
 2223, 2223, 2223, 2223, 2223,    0,    0, 2223,    0,    0,
 2223,    0, 2223, 2223,    0,    0, 2223, 2223, 2223,    0,
 2223, 2223, 2223, 2223, 2223, 2223, 2223,    0,    0,    0,
 2223, 2223, 2223, 2223, 2223,    0,    0,    0, 2223,    0,
    0,    0,    0,    0,    0,    0, 2223, 2223, 2223, 2223,
 2223, 2223,    0,    0,    0, 2223, 2223,    0, 2223,    0,
 2223,    0, 2223, 2223, 2223,    0, 2223,    0, 2223, 2223,
 2223,    0, 2223, 2223, 2223, 2223, 2223,    0, 2223, 2223,
    0,    0,    0,    0, 2223,    0, 2223, 2223,    0,    0,
 2223, 2223, 2223,    0,    0, 2223,    0, 2223,    0,    0,
 2223, 2223, 2223, 2223,    0, 2223,    0, 2223,    0, 2223,
 2223, 2223, 2223, 2223,    0,    0, 2223, 2223, 2223,    0,
 2223, 2223,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2223, 2223, 2223,    0, 2223, 2223, 2223,    0,    0,
    0,    0,    0, 2223,    0,    0,    0, 2223,    0, 2223,
 2223, 2223, 2223,    0, 2223,    0, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2223, 2223,    0, 2223, 2223, 2223, 1457,
    0,    0,    0, 2223, 2223, 2223, 2223, 2223,    0, 2223,
    0,    0, 2223, 2223,    0,    0,    0, 2223, 2223,    0,
    0, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
    0, 2223,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223,    0, 2223, 2223, 2223, 2223, 2223, 2223,    0,
 2223, 2223, 2223, 2223,    0, 2223,    0, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223,    0, 2223, 2223, 2223, 2223,
    0, 2223, 2223, 2223, 2223, 2223,    0, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223,    0, 2223, 2223,    0,
    0,    0, 2223, 2223, 2223, 2223, 2223, 2223,    0,    0,
 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,    0,
    0,    0,    0, 3007, 2223,    0, 2223, 2223, 2223, 2223,
 2223, 2223,    0, 2223,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2223, 2223,    0, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223,    0, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223,    0, 2223, 2223, 2223,    0,
 2223,    0,    0,    0, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223, 2223,    0, 2223, 2223,
    0,    0,    0, 2224,    0, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223,    0,    0, 2223,    0, 2223, 2223, 2223,
 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223, 2223,
 2223, 2223, 2223, 1979, 2224, 2224, 2224, 2224, 2224, 2224,
 2224,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2224, 2224, 2224, 2224, 2224,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2224,    0, 2224,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2224,    0, 2224,    0,    0, 2224,    0, 2224, 2224, 2224,
    0, 2224, 2224,    0, 2224,    0,    0,    0,    0,    0,
 2224,    0, 2224, 2224, 2224, 2224,    0, 2224,    0,    0,
 2224,    0,    0, 2224, 1456,    0,    0,    0,    0,    0,
 2224,    0, 2224, 2224,    0, 2224, 2224, 2224, 2224, 2224,
    0,    0, 2224,    0,    0, 2224,    0, 2224, 2224,    0,
    0, 2224, 2224, 2224,    0, 2224, 2224, 2224, 2224, 2224,
 2224, 2224,    0,    0,    0, 2224, 2224, 2224, 2224, 2224,
    0,    0,    0, 2224,    0,    0,    0,    0,    0,    0,
    0, 2224, 2224, 2224, 2224, 2224, 2224,    0,    0,    0,
 2224, 2224,    0, 2224,    0, 2224,    0, 2224, 2224, 2224,
    0, 2224,    0, 2224, 2224, 2224,    0, 2224, 2224, 2224,
 2224, 2224,    0, 2224, 2224,    0,    0,    0,    0, 2224,
    0, 2224, 2224,    0,    0, 2224, 2224, 2224,    0,    0,
 2224,    0, 2224,    0,    0, 2224, 2224, 2224, 2224,    0,
 2224,    0, 2224,    0, 2224, 2224, 2224, 2224, 2224,    0,
    0, 2224, 2224, 2224,    0, 2224, 2224,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2224, 2224, 2224,    0,
 2224, 2224, 2224,    0,    0,    0,    0,    0, 2224,    0,
    0,    0, 2224,    0, 2224, 2224, 2224, 2224,    0, 2224,
    0, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2224, 2224,
    0, 2224, 2224, 2224, 1457,    0,    0,    0, 2224, 2224,
 2224, 2224, 2224,    0, 2224,    0,    0, 2224, 2224,    0,
    0,    0, 2224, 2224,    0,    0, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224,    0, 2224,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224,    0, 2224, 2224,
 2224, 2224, 2224, 2224,    0, 2224, 2224, 2224, 2224,    0,
 2224,    0, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
    0, 2224, 2224, 2224, 2224,    0, 2224, 2224, 2224, 2224,
 2224,    0, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224,    0, 2224, 2224,    0,    0,    0, 2224, 2224, 2224,
 2224, 2224, 2224,    0,    0, 2224, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224,    0,    0,    0,    0, 3008, 2224,
    0, 2224, 2224, 2224, 2224, 2224, 2224,    0, 2224,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2224, 2224,    0, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224,    0, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
    0, 2224, 2224, 2224,    0, 2224,    0,    0,    0, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224, 2224,    0, 2224, 2224,    0,    0,    0, 2226,    0,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,    0,    0,
 2224,    0, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224,
 2224, 2224, 2224, 2224, 2224, 2224, 2224, 2224, 1980, 2226,
 2226, 2226, 2226, 2226, 2226, 2226,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2226, 2226, 2226, 2226,
 2226,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2226,
    0, 2226,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2226,    0, 2226,    0,    0,
 2226,    0, 2226, 2226, 2226,    0, 2226, 2226,    0, 2226,
    0,    0,    0,    0,    0, 2226,    0, 2226, 2226, 2226,
 2226,    0, 2226,    0,    0, 2226,    0,    0, 2226, 1456,
    0,    0,    0,    0,    0, 2226,    0, 2226, 2226,    0,
 2226, 2226, 2226, 2226, 2226,    0,    0, 2226,    0,    0,
 2226,    0, 2226, 2226,    0,    0, 2226, 2226, 2226,    0,
 2226, 2226, 2226, 2226, 2226, 2226, 2226,    0,    0,    0,
 2226, 2226, 2226, 2226, 2226,    0,    0,    0, 2226,    0,
    0,    0,    0,    0,    0,    0, 2226, 2226, 2226, 2226,
 2226, 2226,    0,    0,    0, 2226, 2226,    0, 2226,    0,
 2226,    0, 2226, 2226, 2226,    0, 2226,    0, 2226, 2226,
 2226,    0, 2226, 2226, 2226, 2226, 2226,    0, 2226, 2226,
    0,    0,    0,    0, 2226,    0, 2226, 2226,    0,    0,
 2226, 2226, 2226,    0,    0, 2226,    0, 2226,    0,    0,
 2226, 2226, 2226, 2226,    0, 2226,    0, 2226,    0, 2226,
 2226, 2226, 2226, 2226,    0,    0, 2226, 2226, 2226,    0,
 2226, 2226,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2226, 2226, 2226,    0, 2226, 2226, 2226,    0,    0,
    0,    0,    0, 2226,    0,    0,    0, 2226,    0, 2226,
 2226, 2226, 2226,    0, 2226,    0, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2226, 2226,    0, 2226, 2226, 2226, 1457,
    0,    0,    0, 2226, 2226, 2226, 2226, 2226,    0, 2226,
    0,    0, 2226, 2226,    0,    0,    0, 2226, 2226,    0,
    0, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
    0, 2226,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226,    0, 2226, 2226, 2226, 2226, 2226, 2226,    0,
 2226, 2226, 2226, 2226,    0, 2226,    0, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226,    0, 2226, 2226, 2226, 2226,
    0, 2226, 2226, 2226, 2226, 2226,    0, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226,    0, 2226, 2226,    0,
    0,    0, 2226, 2226, 2226, 2226, 2226, 2226,    0,    0,
 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,    0,
    0,    0,    0, 3009, 2226,    0, 2226, 2226, 2226, 2226,
 2226, 2226,    0, 2226,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2226, 2226,    0, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226,    0, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226,    0, 2226, 2226, 2226,    0,
 2226,    0,    0,    0, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226, 2226,    0, 2226, 2226,
    0,    0,    0, 2228,    0, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226,    0,    0, 2226,    0, 2226, 2226, 2226,
 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226, 2226,
 2226, 2226, 2226, 1942, 2228, 2228, 2228, 2228, 2228, 2228,
 2228,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2228, 2228, 2228, 2228, 2228,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2228,    0, 2228,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2228,    0, 2228,    0,    0, 2228,    0, 2228, 2228, 2228,
    0, 2228, 2228,    0, 2228,    0,    0,    0,    0,    0,
 2228,    0, 2228, 2228, 2228, 2228,    0, 2228,    0,    0,
 2228,    0,    0, 2228, 1456,    0,    0,    0,    0,    0,
 2228,    0, 2228, 2228,    0, 2228, 2228, 2228, 2228, 2228,
    0,    0, 2228,    0,    0, 2228,    0, 2228, 2228,    0,
    0, 2228, 2228, 2228,    0, 2228, 2228, 2228, 2228, 2228,
 2228, 2228,    0,    0,    0, 2228, 2228, 2228, 2228, 2228,
    0,    0,    0, 2228,    0,    0,    0,    0,    0,    0,
    0, 2228, 2228, 2228, 2228, 2228, 2228,    0,    0,    0,
 2228, 2228,    0, 2228,    0, 2228,    0, 2228, 2228, 2228,
    0, 2228,    0, 2228, 2228, 2228,    0, 2228, 2228, 2228,
 2228, 2228,    0, 2228, 2228,    0,    0,    0,    0, 2228,
    0, 2228, 2228,    0,    0, 2228, 2228, 2228,    0,    0,
 2228,    0, 2228,    0,    0, 2228, 2228, 2228, 2228,    0,
 2228,    0, 2228,    0, 2228, 2228, 2228, 2228, 2228,    0,
    0, 2228, 2228, 2228,    0, 2228, 2228,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2228, 2228, 2228,    0,
 2228, 2228, 2228,    0,    0,    0,    0,    0, 2228,    0,
    0,    0, 2228,    0, 2228, 2228, 2228, 2228,    0, 2228,
    0, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2228, 2228,
    0, 2228, 2228, 2228, 1457,    0,    0,    0, 2228, 2228,
 2228, 2228, 2228,    0, 2228,    0,    0, 2228, 2228,    0,
    0,    0, 2228, 2228,    0,    0, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228,    0, 2228,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228,    0, 2228, 2228,
 2228, 2228, 2228, 2228,    0, 2228, 2228, 2228, 2228,    0,
 2228,    0, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
    0, 2228, 2228, 2228, 2228,    0, 2228, 2228, 2228, 2228,
 2228,    0, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228,    0, 2228, 2228,    0,    0,    0, 2228, 2228, 2228,
 2228, 2228, 2228,    0,    0, 2228, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228,    0,    0,    0,    0, 3010, 2228,
    0, 2228, 2228, 2228, 2228, 2228, 2228,    0, 2228,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2228, 2228,    0, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228,    0, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
    0, 2228, 2228, 2228,    0, 2228,    0,    0,    0, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228, 2228,    0, 2228, 2228,    0,    0,    0, 2234,    0,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,    0,    0,
 2228,    0, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228,
 2228, 2228, 2228, 2228, 2228, 2228, 2228, 2228, 1983, 2234,
 2234, 2234, 2234, 2234, 2234, 2234,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2234, 2234, 2234, 2234,
 2234,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2234,
    0, 2234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2234,    0, 2234,    0,    0,
 2234,    0, 2234, 2234, 2234,    0, 2234, 2234,    0, 2234,
    0,    0,    0,    0,    0, 2234,    0, 2234, 2234, 2234,
 2234,    0, 2234,    0,    0, 2234,    0,    0, 2234, 1456,
    0,    0,    0,    0,    0, 2234,    0, 2234, 2234,    0,
 2234, 2234, 2234, 2234, 2234,    0,    0, 2234,    0,    0,
 2234,    0, 2234, 2234,    0,    0, 2234, 2234, 2234,    0,
 2234, 2234, 2234, 2234, 2234, 2234, 2234,    0,    0,    0,
 2234, 2234, 2234, 2234, 2234,    0,    0,    0, 2234,    0,
    0,    0,    0,    0,    0,    0, 2234, 2234, 2234, 2234,
 2234, 2234,    0,    0,    0, 2234, 2234,    0, 2234,    0,
 2234,    0, 2234, 2234, 2234,    0, 2234,    0, 2234, 2234,
 2234,    0, 2234, 2234, 2234, 2234, 2234,    0, 2234, 2234,
    0,    0,    0,    0, 2234,    0, 2234, 2234,    0,    0,
 2234, 2234, 2234,    0,    0, 2234,    0, 2234,    0,    0,
 2234, 2234, 2234, 2234,    0, 2234,    0, 2234,    0, 2234,
 2234, 2234, 2234, 2234,    0,    0, 2234, 2234, 2234,    0,
 2234, 2234,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2234, 2234, 2234,    0, 2234, 2234, 2234,    0,    0,
    0,    0,    0, 2234,    0,    0,    0, 2234,    0, 2234,
 2234, 2234, 2234,    0, 2234,    0, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2234, 2234,    0, 2234, 2234, 2234, 1457,
    0,    0,    0, 2234, 2234, 2234, 2234, 2234,    0, 2234,
    0,    0, 2234, 2234,    0,    0,    0, 2234, 2234,    0,
    0, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
    0, 2234,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234,    0, 2234, 2234, 2234, 2234, 2234, 2234,    0,
 2234, 2234, 2234, 2234,    0, 2234,    0, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234,    0, 2234, 2234, 2234, 2234,
    0, 2234, 2234, 2234, 2234, 2234,    0, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234,    0, 2234, 2234,    0,
    0,    0, 2234, 2234, 2234, 2234, 2234, 2234,    0,    0,
 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,    0,
    0,    0,    0, 3011, 2234,    0, 2234, 2234, 2234, 2234,
 2234, 2234,    0, 2234,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2234, 2234,    0, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234,    0, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234,    0, 2234, 2234, 2234,    0,
 2234,    0,    0,    0, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234, 2234,    0, 2234, 2234,
    0,    0,    0, 2241,    0, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234,    0,    0, 2234,    0, 2234, 2234, 2234,
 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234, 2234,
 2234, 2234, 2234, 1935, 2241, 2241, 2241, 2241, 2241, 2241,
 2241,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2241, 2241, 2241, 2241, 2241,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2241,    0, 2241,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2241,    0, 2241,    0,    0, 2241,    0, 2241, 2241, 2241,
    0, 2241, 2241,    0, 2241,    0,    0,    0,    0,    0,
 2241,    0, 2241, 2241, 2241, 2241,    0, 2241,    0,    0,
 2241,    0,    0, 2241, 1456,    0,    0,    0,    0,    0,
 2241,    0, 2241, 2241,    0, 2241, 2241, 2241, 2241, 2241,
    0,    0, 2241,    0,    0, 2241,    0, 2241, 2241,    0,
    0, 2241, 2241, 2241,    0, 2241, 2241, 2241, 2241, 2241,
 2241, 2241,    0,    0,    0, 2241, 2241, 2241, 2241, 2241,
    0,    0,    0, 2241,    0,    0,    0,    0,    0,    0,
    0, 2241, 2241, 2241, 2241, 2241, 2241,    0,    0,    0,
 2241, 2241,    0, 2241,    0, 2241,    0, 2241, 2241, 2241,
    0, 2241,    0, 2241, 2241, 2241,    0, 2241, 2241, 2241,
 2241, 2241,    0, 2241, 2241,    0,    0,    0,    0, 2241,
    0, 2241, 2241,    0,    0, 2241, 2241, 2241,    0,    0,
 2241,    0, 2241,    0,    0, 2241, 2241, 2241, 2241,    0,
 2241,    0, 2241,    0, 2241, 2241, 2241, 2241, 2241,    0,
    0, 2241, 2241, 2241,    0, 2241, 2241,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2241, 2241, 2241,    0,
 2241, 2241, 2241,    0,    0,    0,    0,    0, 2241,    0,
    0,    0, 2241,    0, 2241, 2241, 2241, 2241,    0, 2241,
    0, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2241, 2241,
    0, 2241, 2241, 2241, 1457,    0,    0,    0, 2241, 2241,
 2241, 2241, 2241,    0, 2241,    0,    0, 2241, 2241,    0,
    0,    0, 2241, 2241,    0,    0, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241,    0, 2241,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241,    0, 2241, 2241,
 2241, 2241, 2241, 2241,    0, 2241, 2241, 2241, 2241,    0,
 2241,    0, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
    0, 2241, 2241, 2241, 2241,    0, 2241, 2241, 2241, 2241,
 2241,    0, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241,    0, 2241, 2241,    0,    0,    0, 2241, 2241, 2241,
 2241, 2241, 2241,    0,    0, 2241, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241,    0,    0,    0,    0, 3012, 2241,
    0, 2241, 2241, 2241, 2241, 2241, 2241,    0, 2241,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2241, 2241,    0, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241,    0, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
    0, 2241, 2241, 2241,    0, 2241,    0,    0,    0, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241, 2241,    0, 2241, 2241,    0,    0,    0, 2309,    0,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,    0,    0,
 2241,    0, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241,
 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2241, 2030, 2309,
 2309, 2309, 2309, 2309, 2309, 2309,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2309, 2309, 2309, 2309,
 2309,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2309,
    0, 2309,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2309,    0, 2309,    0,    0,
 2309,    0, 2309, 2309, 2309,    0, 2309, 2309,    0, 2309,
    0,    0,    0,    0,    0, 2309,    0, 2309, 2309, 2309,
 2309,    0, 2309,    0,    0, 2309,    0,    0, 2309, 1456,
    0,    0,    0,    0,    0, 2309,    0, 2309, 2309,    0,
 2309, 2309, 2309, 2309, 2309,    0,    0, 2309,    0,    0,
 2309,    0, 2309, 2309,    0,    0, 2309, 2309, 2309,    0,
 2309, 2309, 2309, 2309, 2309, 2309, 2309,    0,    0,    0,
 2309, 2309, 2309, 2309, 2309,    0,    0,    0, 2309,    0,
    0,    0,    0,    0,    0,    0, 2309, 2309, 2309, 2309,
 2309, 2309,    0,    0,    0, 2309, 2309,    0, 2309,    0,
 2309,    0, 2309, 2309, 2309,    0, 2309,    0, 2309, 2309,
 2309,    0, 2309, 2309, 2309, 2309, 2309,    0, 2309, 2309,
    0,    0,    0,    0, 2309,    0, 2309, 2309,    0,    0,
 2309, 2309, 2309,    0,    0, 2309,    0, 2309,    0,    0,
 2309, 2309, 2309, 2309,    0, 2309,    0, 2309,    0, 2309,
 2309, 2309, 2309, 2309,    0,    0, 2309, 2309, 2309,    0,
 2309, 2309,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2309, 2309, 2309,    0, 2309, 2309, 2309,    0,    0,
    0,    0,    0, 2309,    0,    0,    0, 2309,    0, 2309,
 2309, 2309, 2309,    0, 2309,    0, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2309, 2309,    0, 2309, 2309, 2309, 1457,
    0,    0,    0, 2309, 2309, 2309, 2309, 2309,    0, 2309,
    0,    0, 2309, 2309,    0,    0,    0, 2309, 2309,    0,
    0, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
    0, 2309,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309,    0, 2309, 2309, 2309, 2309, 2309, 2309,    0,
 2309, 2309, 2309, 2309,    0, 2309,    0, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309,    0, 2309, 2309, 2309, 2309,
    0, 2309, 2309, 2309, 2309, 2309,    0, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309,    0, 2309, 2309,    0,
    0,    0, 2309, 2309, 2309, 2309, 2309, 2309,    0,    0,
 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,    0,
    0,    0,    0, 3013, 2309,    0, 2309, 2309, 2309, 2309,
 2309, 2309,    0, 2309,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2309, 2309,    0, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309,    0, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309,    0, 2309, 2309, 2309,    0,
 2309,    0,    0,    0, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309, 2309,    0, 2309, 2309,
    0,    0,    0, 2310,    0, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309,    0,    0, 2309,    0, 2309, 2309, 2309,
 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309, 2309,
 2309, 2309, 2309, 2029, 2310, 2310, 2310, 2310, 2310, 2310,
 2310,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2310, 2310, 2310, 2310, 2310,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2310,    0, 2310,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2310,    0, 2310,    0,    0, 2310,    0, 2310, 2310, 2310,
    0, 2310, 2310,    0, 2310,    0,    0,    0,    0,    0,
 2310,    0, 2310, 2310, 2310, 2310,    0, 2310,    0,    0,
 2310,    0,    0, 2310, 1456,    0,    0,    0,    0,    0,
 2310,    0, 2310, 2310,    0, 2310, 2310, 2310, 2310, 2310,
    0,    0, 2310,    0,    0, 2310,    0, 2310, 2310,    0,
    0, 2310, 2310, 2310,    0, 2310, 2310, 2310, 2310, 2310,
 2310, 2310,    0,    0,    0, 2310, 2310, 2310, 2310, 2310,
    0,    0,    0, 2310,    0,    0,    0,    0,    0,    0,
    0, 2310, 2310, 2310, 2310, 2310, 2310,    0,    0,    0,
 2310, 2310,    0, 2310,    0, 2310,    0, 2310, 2310, 2310,
    0, 2310,    0, 2310, 2310, 2310,    0, 2310, 2310, 2310,
 2310, 2310,    0, 2310, 2310,    0,    0,    0,    0, 2310,
    0, 2310, 2310,    0,    0, 2310, 2310, 2310,    0,    0,
 2310,    0, 2310,    0,    0, 2310, 2310, 2310, 2310,    0,
 2310,    0, 2310,    0, 2310, 2310, 2310, 2310, 2310,    0,
    0, 2310, 2310, 2310,    0, 2310, 2310,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2310, 2310, 2310,    0,
 2310, 2310, 2310,    0,    0,    0,    0,    0, 2310,    0,
    0,    0, 2310,    0, 2310, 2310, 2310, 2310,    0, 2310,
    0, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2310, 2310,
    0, 2310, 2310, 2310, 1457,    0,    0,    0, 2310, 2310,
 2310, 2310, 2310,    0, 2310,    0,    0, 2310, 2310,    0,
    0,    0, 2310, 2310,    0,    0, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310,    0, 2310,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310,    0, 2310, 2310,
 2310, 2310, 2310, 2310,    0, 2310, 2310, 2310, 2310,    0,
 2310,    0, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
    0, 2310, 2310, 2310, 2310,    0, 2310, 2310, 2310, 2310,
 2310,    0, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310,    0, 2310, 2310,    0,    0,    0, 2310, 2310, 2310,
 2310, 2310, 2310,    0,    0, 2310, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310,    0,    0,    0,    0, 3014, 2310,
    0, 2310, 2310, 2310, 2310, 2310, 2310,    0, 2310,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2310, 2310,    0, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310,    0, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
    0, 2310, 2310, 2310,    0, 2310,    0,    0,    0, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310, 2310,    0, 2310, 2310,    0,    0,    0, 2303,    0,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,    0,    0,
 2310,    0, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310,
 2310, 2310, 2310, 2310, 2310, 2310, 2310, 2310, 1924, 2303,
 2303, 2303, 2303, 2303, 2303, 2303,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2303, 2303, 2303, 2303,
 2303,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2303,
    0, 2303,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2303,    0, 2303,    0,    0,
 2303,    0, 2303, 2303, 2303,    0, 2303, 2303,    0, 2303,
    0,    0,    0,    0,    0, 2303,    0, 2303, 2303, 2303,
 2303,    0, 2303,    0,    0, 2303,    0,    0, 2303, 1456,
    0,    0,    0,    0,    0, 2303,    0, 2303, 2303,    0,
 2303, 2303, 2303, 2303, 2303,    0,    0, 2303,    0,    0,
 2303,    0, 2303, 2303,    0,    0, 2303, 2303, 2303,    0,
 2303, 2303, 2303, 2303, 2303, 2303, 2303,    0,    0,    0,
 2303, 2303, 2303, 2303, 2303,    0,    0,    0, 2303,    0,
    0,    0,    0,    0,    0,    0, 2303, 2303, 2303, 2303,
 2303, 2303,    0,    0,    0, 2303, 2303,    0, 2303,    0,
 2303,    0, 2303, 2303, 2303,    0, 2303,    0, 2303, 2303,
 2303,    0, 2303, 2303, 2303, 2303, 2303,    0, 2303, 2303,
    0,    0,    0,    0, 2303,    0, 2303, 2303,    0,    0,
 2303, 2303, 2303,    0,    0, 2303,    0, 2303,    0,    0,
 2303, 2303, 2303, 2303,    0, 2303,    0, 2303,    0, 2303,
 2303, 2303, 2303, 2303,    0,    0, 2303, 2303, 2303,    0,
 2303, 2303,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2303, 2303, 2303,    0, 2303, 2303, 2303,    0,    0,
    0,    0,    0, 2303,    0,    0,    0, 2303,    0, 2303,
 2303, 2303, 2303,    0, 2303,    0, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2303, 2303,    0, 2303, 2303, 2303, 1457,
    0,    0,    0, 2303, 2303, 2303, 2303, 2303,    0, 2303,
    0,    0, 2303, 2303,    0,    0,    0, 2303, 2303,    0,
    0, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
    0, 2303,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303,    0, 2303, 2303, 2303, 2303, 2303, 2303,    0,
 2303, 2303, 2303, 2303,    0, 2303,    0, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303,    0, 2303, 2303, 2303, 2303,
    0, 2303, 2303, 2303, 2303, 2303,    0, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303,    0, 2303, 2303,    0,
    0,    0, 2303, 2303, 2303, 2303, 2303, 2303,    0,    0,
 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,    0,
    0,    0,    0, 3312, 2303,    0, 2303, 2303, 2303, 2303,
 2303, 2303,    0, 2303,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2303, 2303,    0, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303,    0, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303,    0, 2303, 2303, 2303,    0,
 2303,    0,    0,    0, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303, 2303,    0, 2303, 2303,
    0,    0,    0, 2304,    0, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303,    0,    0, 2303,    0, 2303, 2303, 2303,
 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303, 2303,
 2303, 2303, 2303, 1928, 2304, 2304, 2304, 2304, 2304, 2304,
 2304,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2304, 2304, 2304, 2304, 2304,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2304,    0, 2304,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2304,    0, 2304,    0,    0, 2304,    0, 2304, 2304, 2304,
    0, 2304, 2304,    0, 2304,    0,    0,    0,    0,    0,
 2304,    0, 2304, 2304, 2304, 2304,    0, 2304,    0,    0,
 2304,    0,    0, 2304, 1456,    0,    0,    0,    0,    0,
 2304,    0, 2304, 2304,    0, 2304, 2304, 2304, 2304, 2304,
    0,    0, 2304,    0,    0, 2304,    0, 2304, 2304,    0,
    0, 2304, 2304, 2304,    0, 2304, 2304, 2304, 2304, 2304,
 2304, 2304,    0,    0,    0, 2304, 2304, 2304, 2304, 2304,
    0,    0,    0, 2304,    0,    0,    0,    0,    0,    0,
    0, 2304, 2304, 2304, 2304, 2304, 2304,    0,    0,    0,
 2304, 2304,    0, 2304,    0, 2304,    0, 2304, 2304, 2304,
    0, 2304,    0, 2304, 2304, 2304,    0, 2304, 2304, 2304,
 2304, 2304,    0, 2304, 2304,    0,    0,    0,    0, 2304,
    0, 2304, 2304,    0,    0, 2304, 2304, 2304,    0,    0,
 2304,    0, 2304,    0,    0, 2304, 2304, 2304, 2304,    0,
 2304,    0, 2304,    0, 2304, 2304, 2304, 2304, 2304,    0,
    0, 2304, 2304, 2304,    0, 2304, 2304,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2304, 2304, 2304,    0,
 2304, 2304, 2304,    0,    0,    0,    0,    0, 2304,    0,
    0,    0, 2304,    0, 2304, 2304, 2304, 2304,    0, 2304,
    0, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2304, 2304,
    0, 2304, 2304, 2304, 1457,    0,    0,    0, 2304, 2304,
 2304, 2304, 2304,    0, 2304,    0,    0, 2304, 2304,    0,
    0,    0, 2304, 2304,    0,    0, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304,    0, 2304,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304,    0, 2304, 2304,
 2304, 2304, 2304, 2304,    0, 2304, 2304, 2304, 2304,    0,
 2304,    0, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
    0, 2304, 2304, 2304, 2304,    0, 2304, 2304, 2304, 2304,
 2304,    0, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304,    0, 2304, 2304,    0,    0,    0, 2304, 2304, 2304,
 2304, 2304, 2304,    0,    0, 2304, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304,    0,    0,    0,    0, 3313, 2304,
    0, 2304, 2304, 2304, 2304, 2304, 2304,    0, 2304,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2304, 2304,    0, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304,    0, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
    0, 2304, 2304, 2304,    0, 2304,    0,    0,    0, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304, 2304,    0, 2304, 2304,    0,    0,    0, 2305,    0,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,    0,    0,
 2304,    0, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304,
 2304, 2304, 2304, 2304, 2304, 2304, 2304, 2304, 1931, 2305,
 2305, 2305, 2305, 2305, 2305, 2305,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2305, 2305, 2305, 2305,
 2305,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2305,
    0, 2305,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2305,    0, 2305,    0,    0,
 2305,    0, 2305, 2305, 2305,    0, 2305, 2305,    0, 2305,
    0,    0,    0,    0,    0, 2305,    0, 2305, 2305, 2305,
 2305,    0, 2305,    0,    0, 2305,    0,    0, 2305, 1456,
    0,    0,    0,    0,    0, 2305,    0, 2305, 2305,    0,
 2305, 2305, 2305, 2305, 2305,    0,    0, 2305,    0,    0,
 2305,    0, 2305, 2305,    0,    0, 2305, 2305, 2305,    0,
 2305, 2305, 2305, 2305, 2305, 2305, 2305,    0,    0,    0,
 2305, 2305, 2305, 2305, 2305,    0,    0,    0, 2305,    0,
    0,    0,    0,    0,    0,    0, 2305, 2305, 2305, 2305,
 2305, 2305,    0,    0,    0, 2305, 2305,    0, 2305,    0,
 2305,    0, 2305, 2305, 2305,    0, 2305,    0, 2305, 2305,
 2305,    0, 2305, 2305, 2305, 2305, 2305,    0, 2305, 2305,
    0,    0,    0,    0, 2305,    0, 2305, 2305,    0,    0,
 2305, 2305, 2305,    0,    0, 2305,    0, 2305,    0,    0,
 2305, 2305, 2305, 2305,    0, 2305,    0, 2305,    0, 2305,
 2305, 2305, 2305, 2305,    0,    0, 2305, 2305, 2305,    0,
 2305, 2305,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2305, 2305, 2305,    0, 2305, 2305, 2305,    0,    0,
    0,    0,    0, 2305,    0,    0,    0, 2305,    0, 2305,
 2305, 2305, 2305,    0, 2305,    0, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2305, 2305,    0, 2305, 2305, 2305, 1457,
    0,    0,    0, 2305, 2305, 2305, 2305, 2305,    0, 2305,
    0,    0, 2305, 2305,    0,    0,    0, 2305, 2305,    0,
    0, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
    0, 2305,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305,    0, 2305, 2305, 2305, 2305, 2305, 2305,    0,
 2305, 2305, 2305, 2305,    0, 2305,    0, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305,    0, 2305, 2305, 2305, 2305,
    0, 2305, 2305, 2305, 2305, 2305,    0, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305,    0, 2305, 2305,    0,
    0,    0, 2305, 2305, 2305, 2305, 2305, 2305,    0,    0,
 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,    0,
    0,    0,    0, 3518, 2305,    0, 2305, 2305, 2305, 2305,
 2305, 2305,    0, 2305,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2305, 2305,    0, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305,    0, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305,    0, 2305, 2305, 2305,    0,
 2305,    0,    0,    0, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305, 2305,    0, 2305, 2305,
    0,    0,    0, 2333,    0, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305,    0,    0, 2305,    0, 2305, 2305, 2305,
 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305, 2305,
 2305, 2305, 2305, 1932, 2333, 2333, 2333, 2333, 2333, 2333,
 2333,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2333, 2333, 2333, 2333, 2333,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2333,    0, 2333,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2333,    0, 2333,    0,    0, 2333,    0, 2333, 2333, 2333,
    0, 2333, 2333,    0, 2333,    0,    0,    0,    0,    0,
 2333,    0, 2333, 2333, 2333, 2333,    0, 2333,    0,    0,
 2333,    0,    0, 2333, 1456,    0,    0,    0,    0,    0,
 2333,    0, 2333, 2333,    0, 2333, 2333, 2333, 2333, 2333,
    0,    0, 2333,    0,    0, 2333,    0, 2333, 2333,    0,
    0, 2333, 2333, 2333,    0, 2333, 2333, 2333, 2333, 2333,
 2333, 2333,    0,    0,    0, 2333, 2333, 2333, 2333, 2333,
    0,    0,    0, 2333,    0,    0,    0,    0,    0,    0,
    0, 2333, 2333, 2333, 2333, 2333, 2333,    0,    0,    0,
 2333, 2333,    0, 2333,    0, 2333,    0, 2333, 2333, 2333,
    0, 2333,    0, 2333, 2333, 2333,    0, 2333, 2333, 2333,
 2333, 2333,    0, 2333, 2333,    0,    0,    0,    0, 2333,
    0, 2333, 2333,    0,    0, 2333, 2333, 2333,    0,    0,
 2333,    0, 2333,    0,    0, 2333, 2333, 2333, 2333,    0,
 2333,    0, 2333,    0, 2333, 2333, 2333, 2333, 2333,    0,
    0, 2333, 2333, 2333,    0, 2333, 2333,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2333, 2333, 2333,    0,
 2333, 2333, 2333,    0,    0,    0,    0,    0, 2333,    0,
    0,    0, 2333,    0, 2333, 2333, 2333, 2333,    0, 2333,
    0, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2333, 2333,
    0, 2333, 2333, 2333, 1457,    0,    0,    0, 2333, 2333,
 2333, 2333, 2333,    0, 2333,    0,    0, 2333, 2333,    0,
    0,    0, 2333, 2333,    0,    0, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333,    0, 2333,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333,    0, 2333, 2333,
 2333, 2333, 2333, 2333,    0, 2333, 2333, 2333, 2333,    0,
 2333,    0, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
    0, 2333, 2333, 2333, 2333,    0, 2333, 2333, 2333, 2333,
 2333,    0, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333,    0, 2333, 2333,    0,    0,    0, 2333, 2333, 2333,
 2333, 2333, 2333,    0,    0, 2333, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333,    0,    0,    0,    0, 3523, 2333,
    0, 2333, 2333, 2333, 2333, 2333, 2333,    0, 2333,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2333, 2333,    0, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333,    0, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
    0, 2333, 2333, 2333,    0, 2333,    0,    0,    0, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333, 2333,    0, 2333, 2333,    0,    0,    0, 2334,    0,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,    0,    0,
 2333,    0, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333,
 2333, 2333, 2333, 2333, 2333, 2333, 2333, 2333, 1933, 2334,
 2334, 2334, 2334, 2334, 2334, 2334,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2334, 2334, 2334, 2334,
 2334,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2334,
    0, 2334,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2334,    0, 2334,    0,    0,
 2334,    0, 2334, 2334, 2334,    0, 2334, 2334,    0, 2334,
    0,    0,    0,    0,    0, 2334,    0, 2334, 2334, 2334,
 2334,    0, 2334,    0,    0, 2334,    0,    0, 2334, 1456,
    0,    0,    0,    0,    0, 2334,    0, 2334, 2334,    0,
 2334, 2334, 2334, 2334, 2334,    0,    0, 2334,    0,    0,
 2334,    0, 2334, 2334,    0,    0, 2334, 2334, 2334,    0,
 2334, 2334, 2334, 2334, 2334, 2334, 2334,    0,    0,    0,
 2334, 2334, 2334, 2334, 2334,    0,    0,    0, 2334,    0,
    0,    0,    0,    0,    0,    0, 2334, 2334, 2334, 2334,
 2334, 2334,    0,    0,    0, 2334, 2334,    0, 2334,    0,
 2334,    0, 2334, 2334, 2334,    0, 2334,    0, 2334, 2334,
 2334,    0, 2334, 2334, 2334, 2334, 2334,    0, 2334, 2334,
    0,    0,    0,    0, 2334,    0, 2334, 2334,    0,    0,
 2334, 2334, 2334,    0,    0, 2334,    0, 2334,    0,    0,
 2334, 2334, 2334, 2334,    0, 2334,    0, 2334,    0, 2334,
 2334, 2334, 2334, 2334,    0,    0, 2334, 2334, 2334,    0,
 2334, 2334,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2334, 2334, 2334,    0, 2334, 2334, 2334,    0,    0,
    0,    0,    0, 2334,    0,    0,    0, 2334,    0, 2334,
 2334, 2334, 2334,    0, 2334,    0, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2334, 2334,    0, 2334, 2334, 2334, 1457,
    0,    0,    0, 2334, 2334, 2334, 2334, 2334,    0, 2334,
    0,    0, 2334, 2334,    0,    0,    0, 2334, 2334,    0,
    0, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
    0, 2334,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334,    0, 2334, 2334, 2334, 2334, 2334, 2334,    0,
 2334, 2334, 2334, 2334,    0, 2334,    0, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334,    0, 2334, 2334, 2334, 2334,
    0, 2334, 2334, 2334, 2334, 2334,    0, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334,    0, 2334, 2334,    0,
    0,    0, 2334, 2334, 2334, 2334, 2334, 2334,    0,    0,
 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,    0,
    0,    0,    0, 3524, 2334,    0, 2334, 2334, 2334, 2334,
 2334, 2334,    0, 2334,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2334, 2334,    0, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334,    0, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334,    0, 2334, 2334, 2334,    0,
 2334,    0,    0,    0, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334, 2334,    0, 2334, 2334,
    0,    0,    0, 2338,    0, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334,    0,    0, 2334,    0, 2334, 2334, 2334,
 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334, 2334,
 2334, 2334, 2334, 1987, 2338, 2338, 2338, 2338, 2338, 2338,
 2338,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2338, 2338, 2338, 2338, 2338,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2338,    0, 2338,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2338,    0, 2338,    0,    0, 2338,    0, 2338, 2338, 2338,
    0, 2338, 2338,    0, 2338,    0,    0,    0,    0,    0,
 2338,    0, 2338, 2338, 2338, 2338,    0, 2338,    0,    0,
 2338,    0,    0, 2338, 1456,    0,    0,    0,    0,    0,
 2338,    0, 2338, 2338,    0, 2338, 2338, 2338, 2338, 2338,
    0,    0, 2338,    0,    0, 2338,    0, 2338, 2338,    0,
    0, 2338, 2338, 2338,    0, 2338, 2338, 2338, 2338, 2338,
 2338, 2338,    0,    0,    0, 2338, 2338, 2338, 2338, 2338,
    0,    0,    0, 2338,    0,    0,    0,    0,    0,    0,
    0, 2338, 2338, 2338, 2338, 2338, 2338,    0,    0,    0,
 2338, 2338,    0, 2338,    0, 2338,    0, 2338, 2338, 2338,
    0, 2338,    0, 2338, 2338, 2338,    0, 2338, 2338, 2338,
 2338, 2338,    0, 2338, 2338,    0,    0,    0,    0, 2338,
    0, 2338, 2338,    0,    0, 2338, 2338, 2338,    0,    0,
 2338,    0, 2338,    0,    0, 2338, 2338, 2338, 2338,    0,
 2338,    0, 2338,    0, 2338, 2338, 2338, 2338, 2338,    0,
    0, 2338, 2338, 2338,    0, 2338, 2338,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2338, 2338, 2338,    0,
 2338, 2338, 2338,    0,    0,    0,    0,    0, 2338,    0,
    0,    0, 2338,    0, 2338, 2338, 2338, 2338,    0, 2338,
    0, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2338, 2338,
    0, 2338, 2338, 2338, 1457,    0,    0,    0, 2338, 2338,
 2338, 2338, 2338,    0, 2338,    0,    0, 2338, 2338,    0,
    0,    0, 2338, 2338,    0,    0, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338,    0, 2338,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338,    0, 2338, 2338,
 2338, 2338, 2338, 2338,    0, 2338, 2338, 2338, 2338,    0,
 2338,    0, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
    0, 2338, 2338, 2338, 2338,    0, 2338, 2338, 2338, 2338,
 2338,    0, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338,    0, 2338, 2338,    0,    0,    0, 2338, 2338, 2338,
 2338, 2338, 2338,    0,    0, 2338, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338,    0,    0,    0,    0, 3525, 2338,
    0, 2338, 2338, 2338, 2338, 2338, 2338,    0, 2338,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2338, 2338,    0, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338,    0, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
    0, 2338, 2338, 2338,    0, 2338,    0,    0,    0, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338, 2338,    0, 2338, 2338,    0,    0,    0, 2341,    0,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,    0,    0,
 2338,    0, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338,
 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2338, 2006, 2341,
 2341, 2341, 2341, 2341, 2341, 2341,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2341, 2341, 2341, 2341,
 2341,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2341,
    0, 2341,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2341,    0, 2341,    0,    0,
 2341,    0, 2341, 2341, 2341,    0, 2341, 2341,    0, 2341,
    0,    0,    0,    0,    0, 2341,    0, 2341, 2341, 2341,
 2341,    0, 2341,    0,    0, 2341,    0,    0, 2341, 1456,
    0,    0,    0,    0,    0, 2341,    0, 2341, 2341,    0,
 2341, 2341, 2341, 2341, 2341,    0,    0, 2341,    0,    0,
 2341,    0, 2341, 2341,    0,    0, 2341, 2341, 2341,    0,
 2341, 2341, 2341, 2341, 2341, 2341, 2341,    0,    0,    0,
 2341, 2341, 2341, 2341, 2341,    0,    0,    0, 2341,    0,
    0,    0,    0,    0,    0,    0, 2341, 2341, 2341, 2341,
 2341, 2341,    0,    0,    0, 2341, 2341,    0, 2341,    0,
 2341,    0, 2341, 2341, 2341,    0, 2341,    0, 2341, 2341,
 2341,    0, 2341, 2341, 2341, 2341, 2341,    0, 2341, 2341,
    0,    0,    0,    0, 2341,    0, 2341, 2341,    0,    0,
 2341, 2341, 2341,    0,    0, 2341,    0, 2341,    0,    0,
 2341, 2341, 2341, 2341,    0, 2341,    0, 2341,    0, 2341,
 2341, 2341, 2341, 2341,    0,    0, 2341, 2341, 2341,    0,
 2341, 2341,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2341, 2341, 2341,    0, 2341, 2341, 2341,    0,    0,
    0,    0,    0, 2341,    0,    0,    0, 2341,    0, 2341,
 2341, 2341, 2341,    0, 2341,    0, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2341, 2341,    0, 2341, 2341, 2341, 1457,
    0,    0,    0, 2341, 2341, 2341, 2341, 2341,    0, 2341,
    0,    0, 2341, 2341,    0,    0,    0, 2341, 2341,    0,
    0, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
    0, 2341,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341,    0, 2341, 2341, 2341, 2341, 2341, 2341,    0,
 2341, 2341, 2341, 2341,    0, 2341,    0, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341,    0, 2341, 2341, 2341, 2341,
    0, 2341, 2341, 2341, 2341, 2341,    0, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341,    0, 2341, 2341,    0,
    0,    0, 2341, 2341, 2341, 2341, 2341, 2341,    0,    0,
 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,    0,
    0,    0,    0, 3526, 2341,    0, 2341, 2341, 2341, 2341,
 2341, 2341,    0, 2341,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2341, 2341,    0, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341,    0, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341,    0, 2341, 2341, 2341,    0,
 2341,    0,    0,    0, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341, 2341,    0, 2341, 2341,
    0,    0,    0, 2354,    0, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341,    0,    0, 2341,    0, 2341, 2341, 2341,
 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341, 2341,
 2341, 2341, 2341, 1949, 2354, 2354, 2354, 2354, 2354, 2354,
 2354,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2354, 2354, 2354, 2354, 2354,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2354,    0, 2354,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2354,    0, 2354,    0,    0, 2354,    0, 2354, 2354, 2354,
    0, 2354, 2354,    0, 2354,    0,    0,    0,    0,    0,
 2354,    0, 2354, 2354, 2354, 2354,    0, 2354,    0,    0,
 2354,    0,    0, 2354, 1456,    0,    0,    0,    0,    0,
 2354,    0, 2354, 2354,    0, 2354, 2354, 2354, 2354, 2354,
    0,    0, 2354,    0,    0, 2354,    0, 2354, 2354,    0,
    0, 2354, 2354, 2354,    0, 2354, 2354, 2354, 2354, 2354,
 2354, 2354,    0,    0,    0, 2354, 2354, 2354, 2354, 2354,
    0,    0,    0, 2354,    0,    0,    0,    0,    0,    0,
    0, 2354, 2354, 2354, 2354, 2354, 2354,    0,    0,    0,
 2354, 2354,    0, 2354,    0, 2354,    0, 2354, 2354, 2354,
    0, 2354,    0, 2354, 2354, 2354,    0, 2354, 2354, 2354,
 2354, 2354,    0, 2354, 2354,    0,    0,    0,    0, 2354,
    0, 2354, 2354,    0,    0, 2354, 2354, 2354,    0,    0,
 2354,    0, 2354,    0,    0, 2354, 2354, 2354, 2354,    0,
 2354,    0, 2354,    0, 2354, 2354, 2354, 2354, 2354,    0,
    0, 2354, 2354, 2354,    0, 2354, 2354,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2354, 2354, 2354,    0,
 2354, 2354, 2354,    0,    0,    0,    0,    0, 2354,    0,
    0,    0, 2354,    0, 2354, 2354, 2354, 2354,    0, 2354,
    0, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2354, 2354,
    0, 2354, 2354, 2354, 1457,    0,    0,    0, 2354, 2354,
 2354, 2354, 2354,    0, 2354,    0,    0, 2354, 2354,    0,
    0,    0, 2354, 2354,    0,    0, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354,    0, 2354,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354,    0, 2354, 2354,
 2354, 2354, 2354, 2354,    0, 2354, 2354, 2354, 2354,    0,
 2354,    0, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
    0, 2354, 2354, 2354, 2354,    0, 2354, 2354, 2354, 2354,
 2354,    0, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354,    0, 2354, 2354,    0,    0,    0, 2354, 2354, 2354,
 2354, 2354, 2354,    0,    0, 2354, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354,    0,    0,    0,    0, 3533, 2354,
    0, 2354, 2354, 2354, 2354, 2354, 2354,    0, 2354,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2354, 2354,    0, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354,    0, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
    0, 2354, 2354, 2354,    0, 2354,    0,    0,    0, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354, 2354,    0, 2354, 2354,    0,    0,    0, 2355,    0,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,    0,    0,
 2354,    0, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354,
 2354, 2354, 2354, 2354, 2354, 2354, 2354, 2354, 1950, 2355,
 2355, 2355, 2355, 2355, 2355, 2355,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2355, 2355, 2355, 2355,
 2355,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2355,
    0, 2355,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2355,    0, 2355,    0,    0,
 2355,    0, 2355, 2355, 2355,    0, 2355, 2355,    0, 2355,
    0,    0,    0,    0,    0, 2355,    0, 2355, 2355, 2355,
 2355,    0, 2355,    0,    0, 2355,    0,    0, 2355, 1456,
    0,    0,    0,    0,    0, 2355,    0, 2355, 2355,    0,
 2355, 2355, 2355, 2355, 2355,    0,    0, 2355,    0,    0,
 2355,    0, 2355, 2355,    0,    0, 2355, 2355, 2355,    0,
 2355, 2355, 2355, 2355, 2355, 2355, 2355,    0,    0,    0,
 2355, 2355, 2355, 2355, 2355,    0,    0,    0, 2355,    0,
    0,    0,    0,    0,    0,    0, 2355, 2355, 2355, 2355,
 2355, 2355,    0,    0,    0, 2355, 2355,    0, 2355,    0,
 2355,    0, 2355, 2355, 2355,    0, 2355,    0, 2355, 2355,
 2355,    0, 2355, 2355, 2355, 2355, 2355,    0, 2355, 2355,
    0,    0,    0,    0, 2355,    0, 2355, 2355,    0,    0,
 2355, 2355, 2355,    0,    0, 2355,    0, 2355,    0,    0,
 2355, 2355, 2355, 2355,    0, 2355,    0, 2355,    0, 2355,
 2355, 2355, 2355, 2355,    0,    0, 2355, 2355, 2355,    0,
 2355, 2355,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2355, 2355, 2355,    0, 2355, 2355, 2355,    0,    0,
    0,    0,    0, 2355,    0,    0,    0, 2355,    0, 2355,
 2355, 2355, 2355,    0, 2355,    0, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2355, 2355,    0, 2355, 2355, 2355, 1457,
    0,    0,    0, 2355, 2355, 2355, 2355, 2355,    0, 2355,
    0,    0, 2355, 2355,    0,    0,    0, 2355, 2355,    0,
    0, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
    0, 2355,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355,    0, 2355, 2355, 2355, 2355, 2355, 2355,    0,
 2355, 2355, 2355, 2355,    0, 2355,    0, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355,    0, 2355, 2355, 2355, 2355,
    0, 2355, 2355, 2355, 2355, 2355,    0, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355,    0, 2355, 2355,    0,
    0,    0, 2355, 2355, 2355, 2355, 2355, 2355,    0,    0,
 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,    0,
    0,    0,    0, 3534, 2355,    0, 2355, 2355, 2355, 2355,
 2355, 2355,    0, 2355,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2355, 2355,    0, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355,    0, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355,    0, 2355, 2355, 2355,    0,
 2355,    0,    0,    0, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355, 2355,    0, 2355, 2355,
    0,    0,    0, 2364,    0, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355,    0,    0, 2355,    0, 2355, 2355, 2355,
 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355, 2355,
 2355, 2355, 2355, 1956, 2364, 2364, 2364, 2364, 2364, 2364,
 2364,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2364, 2364, 2364, 2364, 2364,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2364,    0, 2364,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2364,    0, 2364,    0,    0, 2364,    0, 2364, 2364, 2364,
    0, 2364, 2364,    0, 2364,    0,    0,    0,    0,    0,
 2364,    0, 2364, 2364, 2364, 2364,    0, 2364,    0,    0,
 2364,    0,    0, 2364, 1456,    0,    0,    0,    0,    0,
 2364,    0, 2364, 2364,    0, 2364, 2364, 2364, 2364, 2364,
    0,    0, 2364,    0,    0, 2364,    0, 2364, 2364,    0,
    0, 2364, 2364, 2364,    0, 2364, 2364, 2364, 2364, 2364,
 2364, 2364,    0,    0,    0, 2364, 2364, 2364, 2364, 2364,
    0,    0,    0, 2364,    0,    0,    0,    0,    0,    0,
    0, 2364, 2364, 2364, 2364, 2364, 2364,    0,    0,    0,
 2364, 2364,    0, 2364,    0, 2364,    0, 2364, 2364, 2364,
    0, 2364,    0, 2364, 2364, 2364,    0, 2364, 2364, 2364,
 2364, 2364,    0, 2364, 2364,    0,    0,    0,    0, 2364,
    0, 2364, 2364,    0,    0, 2364, 2364, 2364,    0,    0,
 2364,    0, 2364,    0,    0, 2364, 2364, 2364, 2364,    0,
 2364,    0, 2364,    0, 2364, 2364, 2364, 2364, 2364,    0,
    0, 2364, 2364, 2364,    0, 2364, 2364,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2364, 2364, 2364,    0,
 2364, 2364, 2364,    0,    0,    0,    0,    0, 2364,    0,
    0,    0, 2364,    0, 2364, 2364, 2364, 2364,    0, 2364,
    0, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2364, 2364,
    0, 2364, 2364, 2364, 1457,    0,    0,    0, 2364, 2364,
 2364, 2364, 2364,    0, 2364,    0,    0, 2364, 2364,    0,
    0,    0, 2364, 2364,    0,    0, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364,    0, 2364,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364,    0, 2364, 2364,
 2364, 2364, 2364, 2364,    0, 2364, 2364, 2364, 2364,    0,
 2364,    0, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
    0, 2364, 2364, 2364, 2364,    0, 2364, 2364, 2364, 2364,
 2364,    0, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364,    0, 2364, 2364,    0,    0,    0, 2364, 2364, 2364,
 2364, 2364, 2364,    0,    0, 2364, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364,    0,    0,    0,    0, 3638, 2364,
    0, 2364, 2364, 2364, 2364, 2364, 2364,    0, 2364,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2364, 2364,    0, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364,    0, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
    0, 2364, 2364, 2364,    0, 2364,    0,    0,    0, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364, 2364,    0, 2364, 2364,    0,    0,    0, 2368,    0,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,    0,    0,
 2364,    0, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364,
 2364, 2364, 2364, 2364, 2364, 2364, 2364, 2364, 1986, 2368,
 2368, 2368, 2368, 2368, 2368, 2368,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2368, 2368, 2368, 2368,
 2368,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2368,
    0, 2368,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2368,    0, 2368,    0,    0,
 2368,    0, 2368, 2368, 2368,    0, 2368, 2368,    0, 2368,
    0,    0,    0,    0,    0, 2368,    0, 2368, 2368, 2368,
 2368,    0, 2368,    0,    0, 2368,    0,    0, 2368, 1456,
    0,    0,    0,    0,    0, 2368,    0, 2368, 2368,    0,
 2368, 2368, 2368, 2368, 2368,    0,    0, 2368,    0,    0,
 2368,    0, 2368, 2368,    0,    0, 2368, 2368, 2368,    0,
 2368, 2368, 2368, 2368, 2368, 2368, 2368,    0,    0,    0,
 2368, 2368, 2368, 2368, 2368,    0,    0,    0, 2368,    0,
    0,    0,    0,    0,    0,    0, 2368, 2368, 2368, 2368,
 2368, 2368,    0,    0,    0, 2368, 2368,    0, 2368,    0,
 2368,    0, 2368, 2368, 2368,    0, 2368,    0, 2368, 2368,
 2368,    0, 2368, 2368, 2368, 2368, 2368,    0, 2368, 2368,
    0,    0,    0,    0, 2368,    0, 2368, 2368,    0,    0,
 2368, 2368, 2368,    0,    0, 2368,    0, 2368,    0,    0,
 2368, 2368, 2368, 2368,    0, 2368,    0, 2368,    0, 2368,
 2368, 2368, 2368, 2368,    0,    0, 2368, 2368, 2368,    0,
 2368, 2368,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2368, 2368, 2368,    0, 2368, 2368, 2368,    0,    0,
    0,    0,    0, 2368,    0,    0,    0, 2368,    0, 2368,
 2368, 2368, 2368,    0, 2368,    0, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2368, 2368,    0, 2368, 2368, 2368, 1457,
    0,    0,    0, 2368, 2368, 2368, 2368, 2368,    0, 2368,
    0,    0, 2368, 2368,    0,    0,    0, 2368, 2368,    0,
    0, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
    0, 2368,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368,    0, 2368, 2368, 2368, 2368, 2368, 2368,    0,
 2368, 2368, 2368, 2368,    0, 2368,    0, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368,    0, 2368, 2368, 2368, 2368,
    0, 2368, 2368, 2368, 2368, 2368,    0, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368,    0, 2368, 2368,    0,
    0,    0, 2368, 2368, 2368, 2368, 2368, 2368,    0,    0,
 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,    0,
    0,    0,    0, 3815, 2368,    0, 2368, 2368, 2368, 2368,
 2368, 2368,    0, 2368,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2368, 2368,    0, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368,    0, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368,    0, 2368, 2368, 2368,    0,
 2368,    0,    0,    0, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368, 2368,    0, 2368, 2368,
    0,    0,    0, 2378,    0, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368,    0,    0, 2368,    0, 2368, 2368, 2368,
 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368, 2368,
 2368, 2368, 2368, 1984, 2378, 2378, 2378, 2378, 2378, 2378,
 2378,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2378, 2378, 2378, 2378, 2378,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2378,    0, 2378,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2378,    0, 2378,    0,    0, 2378,    0, 2378, 2378, 2378,
    0, 2378, 2378,    0, 2378,    0,    0,    0,    0,    0,
 2378,    0, 2378, 2378, 2378, 2378,    0, 2378,    0,    0,
 2378,    0,    0, 2378, 1456,    0,    0,    0,    0,    0,
 2378,    0, 2378, 2378,    0, 2378, 2378, 2378, 2378, 2378,
    0,    0, 2378,    0,    0, 2378,    0, 2378, 2378,    0,
    0, 2378, 2378, 2378,    0, 2378, 2378, 2378, 2378, 2378,
 2378, 2378,    0,    0,    0, 2378, 2378, 2378, 2378, 2378,
    0,    0,    0, 2378,    0,    0,    0,    0,    0,    0,
    0, 2378, 2378, 2378, 2378, 2378, 2378,    0,    0,    0,
 2378, 2378,    0, 2378,    0, 2378,    0, 2378, 2378, 2378,
    0, 2378,    0, 2378, 2378, 2378,    0, 2378, 2378, 2378,
 2378, 2378,    0, 2378, 2378,    0,    0,    0,    0, 2378,
    0, 2378, 2378,    0,    0, 2378, 2378, 2378,    0,    0,
 2378,    0, 2378,    0,    0, 2378, 2378, 2378, 2378,    0,
 2378,    0, 2378,    0, 2378, 2378, 2378, 2378, 2378,    0,
    0, 2378, 2378, 2378,    0, 2378, 2378,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2378, 2378, 2378,    0,
 2378, 2378, 2378,    0,    0,    0,    0,    0, 2378,    0,
    0,    0, 2378,    0, 2378, 2378, 2378, 2378,    0, 2378,
    0, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2378, 2378,
    0, 2378, 2378, 2378, 1457,    0,    0,    0, 2378, 2378,
 2378, 2378, 2378,    0, 2378,    0,    0, 2378, 2378,    0,
    0,    0, 2378, 2378,    0,    0, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378,    0, 2378,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378,    0, 2378, 2378,
 2378, 2378, 2378, 2378,    0, 2378, 2378, 2378, 2378,    0,
 2378,    0, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
    0, 2378, 2378, 2378, 2378,    0, 2378, 2378, 2378, 2378,
 2378,    0, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378,    0, 2378, 2378,    0,    0,    0, 2378, 2378, 2378,
 2378, 2378, 2378,    0,    0, 2378, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378,    0,    0,    0,    0, 3860, 2378,
    0, 2378, 2378, 2378, 2378, 2378, 2378,    0, 2378,    0,
    0,    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,
    0,    0,    0,    0,    0, 2378, 2378,    0, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378,    0, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
    0, 2378, 2378, 2378,    0, 2378,    0,    0,    0, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378, 2378,    0, 2378, 2378,    0,    0,    0, 2381,    0,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,    0,    0,
 2378,    0, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378,
 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2378, 2007, 2381,
 2381, 2381, 2381, 2381, 2381, 2381,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2381, 2381, 2381, 2381,
 2381,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2381,
    0, 2381,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1443,
    0,    0,    0,    0,    0,    0,    0,    0, 1444,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1445,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1446,    0,    0,    0,    0,
    0,    0, 1447,    0,    0,    0,    0,    0,    0,    0,
 1448,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1449,    0, 1450,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1451,
    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,
    0,    0,    0,    0,    0, 2381,    0, 2381,    0,    0,
 2381,    0, 2381, 2381, 2381,    0, 2381, 2381,    0, 2381,
    0,    0,    0,    0,    0, 2381,    0, 2381, 2381, 2381,
 2381,    0, 2381,    0,    0, 2381,    0,    0, 2381, 1456,
    0,    0,    0,    0,    0, 2381,    0, 2381, 2381,    0,
 2381, 2381, 2381, 2381, 2381,    0,    0, 2381,    0,    0,
 2381,    0, 2381, 2381,    0,    0, 2381, 2381, 2381,    0,
 2381, 2381, 2381, 2381, 2381, 2381, 2381,    0,    0,    0,
 2381, 2381, 2381, 2381, 2381,    0,    0,    0, 2381,    0,
    0,    0,    0,    0,    0,    0, 2381, 2381, 2381, 2381,
 2381, 2381,    0,    0,    0, 2381, 2381,    0, 2381,    0,
 2381,    0, 2381, 2381, 2381,    0, 2381,    0, 2381, 2381,
 2381,    0, 2381, 2381, 2381, 2381, 2381,    0, 2381, 2381,
    0,    0,    0,    0, 2381,    0, 2381, 2381,    0,    0,
 2381, 2381, 2381,    0,    0, 2381,    0, 2381,    0,    0,
 2381, 2381, 2381, 2381,    0, 2381,    0, 2381,    0, 2381,
 2381, 2381, 2381, 2381,    0,    0, 2381, 2381, 2381,    0,
 2381, 2381,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2381, 2381, 2381,    0, 2381, 2381, 2381,    0,    0,
    0,    0,    0, 2381,    0,    0,    0, 2381,    0, 2381,
 2381, 2381, 2381,    0, 2381,    0, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2381, 2381,    0, 2381, 2381, 2381, 1457,
    0,    0,    0, 2381, 2381, 2381, 2381, 2381,    0, 2381,
    0,    0, 2381, 2381,    0,    0,    0, 2381, 2381,    0,
    0, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
    0, 2381,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381,    0, 2381, 2381, 2381, 2381, 2381, 2381,    0,
 2381, 2381, 2381, 2381,    0, 2381,    0, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381,    0, 2381, 2381, 2381, 2381,
    0, 2381, 2381, 2381, 2381, 2381,    0, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381,    0, 2381, 2381,    0,
    0,    0, 2381, 2381, 2381, 2381, 2381, 2381,    0,    0,
 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,    0,
    0,    0,    0, 3906, 2381,    0, 2381, 2381, 2381, 2381,
 2381, 2381,    0, 2381,    0,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
 2381, 2381,    0, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381,    0, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381,    0, 2381, 2381, 2381,    0,
 2381,    0,    0,    0, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381, 2381,    0, 2381, 2381,
    0,    0,    0, 2382,    0, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381,    0,    0, 2381,    0, 2381, 2381, 2381,
 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381, 2381,
 2381, 2381, 2381, 2008, 2382, 2382, 2382, 2382, 2382, 2382,
 2382,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2382, 2382, 2382, 2382, 2382,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2382,    0, 2382,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2382,    0, 2382,    0,    0, 2382,    0, 2382, 2382, 2382,
    0, 2382, 2382,    0, 2382,    0,    0,    0,    0,    0,
 2382,    0, 2382, 2382, 2382, 2382,    0, 2382,    0,    0,
 2382,    0,    0, 2382, 1456,    0,    0,    0,    0,    0,
 2382,    0, 2382, 2382,    0, 2382, 2382, 2382, 2382, 2382,
    0,    0, 2382,    0,    0, 2382,    0, 2382, 2382,    0,
    0, 2382, 2382, 2382,    0, 2382, 2382, 2382, 2382, 2382,
 2382, 2382,    0,    0,    0, 2382, 2382, 2382, 2382, 2382,
    0,    0,    0, 2382,    0,    0,    0,    0,    0,    0,
    0, 2382, 2382, 2382, 2382, 2382, 2382,    0,    0,    0,
 2382, 2382,    0, 2382,    0, 2382,    0, 2382, 2382, 2382,
    0, 2382,    0, 2382, 2382, 2382,    0, 2382, 2382, 2382,
 2382, 2382,    0, 2382, 2382,    0,    0,    0,    0, 2382,
    0, 2382, 2382,    0,    0, 2382, 2382, 2382,    0,    0,
 2382,    0, 2382,    0,    0, 2382, 2382, 2382, 2382,    0,
 2382,    0, 2382,    0, 2382, 2382, 2382, 2382, 2382,    0,
    0, 2382, 2382, 2382,    0, 2382, 2382,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2382, 2382, 2382,    0,
 2382, 2382, 2382,    0,    0,    0,    0,    0, 2382,    0,
    0,    0, 2382,    0, 2382, 2382, 2382, 2382,    0, 2382,
    0, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2382, 2382,
    0, 2382, 2382, 2382, 1457,    0,    0,    0, 2382, 2382,
 2382, 2382, 2382,    0, 2382,    0,    0, 2382, 2382,    0,
    0,    0, 2382, 2382,    0,    0, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382,    0, 2382,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382,    0, 2382, 2382,
 2382, 2382, 2382, 2382,    0, 2382, 2382, 2382, 2382,    0,
 2382,    0, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
    0, 2382, 2382, 2382, 2382,    0, 2382, 2382, 2382, 2382,
 2382,    0, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382,    0, 2382, 2382,    0,    0,    0, 2382, 2382, 2382,
 2382, 2382, 2382,    0,    0, 2382, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382,    0,    0,    0,    0,    0, 2382,
    0, 2382, 2382, 2382, 2382, 2382, 2382,    0, 2382, 1459,
 1458, 1460,    0,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0, 2382, 2382,    0, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382,    0, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
    0, 2382, 2382, 2382,    0, 2382,    0,    0,    0, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382, 2382,    0, 2382, 2382,    0,    0,    0, 2383,    0,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,    0,    0,
 2382,    0, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382,
 2382, 2382, 2382, 2382, 2382, 2382, 2382, 2382, 1972, 2383,
 2383, 2383, 2383, 2383, 2383, 2383,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2383, 2383, 2383, 2383,
 2383,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2383,
    0, 2383,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1443,    0,    0,  657,    0,    0,    0,    0,    0,
 1444,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0, 1445,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,  658,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1446,    0,    0,
    0,    0,    0,    0, 1447,    0,    0,    0, 1446,    0,
    0,    0, 1448,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0,    0,    0,    0, 1449,
    0, 1450,    0,    0,    0,    0,    0,    0,    0,    0,
 1449, 1451, 1450, 1452, 1453, 1454,    0,    0,    0,    0,
 1455,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0, 2383,    0, 2383,    0,    0,
 2383,    0, 2383, 2383, 2383,    0, 2383, 2383,    0, 2383,
    0,    0,    0,    0,    0, 2383,    0, 2383, 2383, 2383,
 2383, 1456, 2383,    0,    0, 2383,    0,    0, 2383,    0,
    0,    0, 1456,    0,    0, 2383,    0, 2383, 2383, 2018,
 2383, 2383, 2383, 2383, 2383,    0,    0, 2383,    0,    0,
 2383,    0, 2383, 2383,    0,    0, 2383, 2383, 2383,    0,
 2383, 2383, 2383, 2383, 2383, 2383, 2383,    0,    0,    0,
 2383, 2383, 2383, 2383, 2383,    0,    0,    0, 2383,    0,
    0,    0,    0,    0,    0,    0, 2383, 2383, 2383, 2383,
 2383, 2383,    0,    0,    0, 2383, 2383,    0, 2383,    0,
 2383,    0, 2383, 2383, 2383,    0, 2383,    0, 2383, 2383,
 2383,    0, 2383, 2383, 2383, 2383, 2383,    0, 2383, 2383,
    0,    0,    0,    0, 2383,    0, 2383, 2383,    0,    0,
 2383, 2383, 2383,    0,    0, 2383,    0, 2383,    0,    0,
 2383, 2383, 2383, 2383,    0, 2383,    0, 2383,    0, 2383,
 2383, 2383, 2383, 2383,    0,    0, 2383, 2383, 2383,    0,
 2383, 2383,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2383, 2383, 2383,    0, 2383, 2383, 2383,    0,    0,
    0,    0,    0, 2383,    0,    0,    0, 2383,    0, 2383,
 2383, 2383, 2383,    0, 2383,    0, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383,    0,    0,    0,    0,    0,    0,
    0, 1457,    0, 2383, 2383,    0, 2383, 2383, 2383,    0,
    0,    0, 1457, 2383, 2383, 2383, 2383, 2383,    0, 2383,
    0,    0, 2383, 2383,    0,    0,    0, 2383, 2383,    0,
    0, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
    0, 2383,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383,    0, 2383, 2383, 2383, 2383, 2383, 2383,    0,
 2383, 2383, 2383, 2383,    0, 2383,    0, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383,    0, 2383, 2383, 2383, 2383,
    0, 2383, 2383, 2383, 2383, 2383,    0, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383,    0, 2383, 2383,    0,
    0,    0, 2383, 2383, 2383, 2383, 2383, 2383,    0,    0,
 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,    0,
    0,    0,    0,    0, 2383,    0, 2383, 2383, 2383, 2383,
 2383, 2383,    0, 2383, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,
 2383, 2383,    0, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383,    0, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383,    0, 2383, 2383, 2383,    0,
 2383,    0,    0,    0, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383, 2383,    0, 2383, 2383,
    0,    0,    0, 2384,    0, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383,    0,    0, 2383,    0, 2383, 2383, 2383,
 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383, 2383,
 2383, 2383, 2383, 1973, 2384, 2384, 2384, 2384, 2384, 2384,
 2384,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2384, 2384, 2384, 2384, 2384,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2384,    0, 2384,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0, 2165,    0,
    0,    0,    0,    0,    0, 1444,    0, 1443,    0,    0,
    0,    0,    0,    0,    0,    0, 1444,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1445,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0, 1446,    0,    0,    0, 1448,    0,    0,
 1447,    0,    0,    0,    0,    0,    0,    0, 1448,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0, 1449, 1451, 1450, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0, 1451,    0, 1452,
 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,    0,
 2384,    0, 2384,    0,    0, 2384,    0, 2384, 2384, 2384,
    0, 2384, 2384,    0, 2384,    0,    0,    0,    0,    0,
 2384,    0, 2384, 2384, 2384, 2384, 1456, 2384,    0,    0,
 2384,    0,    0, 2384,    0,    0,    0, 1456,    0,    0,
 2384,    0, 2384, 2384,    0, 2384, 2384, 2384, 2384, 2384,
    0,    0, 2384,    0,    0, 2384,    0, 2384, 2384,    0,
    0, 2384, 2384, 2384,    0, 2384, 2384, 2384, 2384, 2384,
 2384, 2384,    0,    0,    0, 2384, 2384, 2384, 2384, 2384,
    0,    0,    0, 2384,    0,    0,    0,    0,    0,    0,
    0, 2384, 2384, 2384, 2384, 2384, 2384,    0,    0,    0,
 2384, 2384,    0, 2384,    0, 2384,    0, 2384, 2384, 2384,
    0, 2384,    0, 2384, 2384, 2384,    0, 2384, 2384, 2384,
 2384, 2384,    0, 2384, 2384,    0,    0,    0,    0, 2384,
    0, 2384, 2384,    0,    0, 2384, 2384, 2384,    0,    0,
 2384,    0, 2384,    0,    0, 2384, 2384, 2384, 2384,    0,
 2384,    0, 2384,    0, 2384, 2384, 2384, 2384, 2384,    0,
    0, 2384, 2384, 2384,    0, 2384, 2384,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2384, 2384, 2384,    0,
 2384, 2384, 2384,    0,    0,    0,    0,    0, 2384,    0,
    0,    0, 2384,    0, 2384, 2384, 2384, 2384, 2212, 2384,
    0, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,    0,
    0,    0,    0,    0,    0,    0, 1457,    0, 2384, 2384,
    0, 2384, 2384, 2384,    0,    0,    0, 1457, 2384, 2384,
 2384, 2384, 2384,    0, 2384,    0,    0, 2384, 2384,    0,
    0,    0, 2384, 2384,    0,    0, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384,    0, 2384,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384,    0, 2384, 2384,
 2384, 2384, 2384, 2384,    0, 2384, 2384, 2384, 2384,    0,
 2384,    0, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
    0, 2384, 2384, 2384, 2384,    0, 2384, 2384, 2384, 2384,
 2384,    0, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384,    0, 2384, 2384,    0,    0,    0, 2384, 2384, 2384,
 2384, 2384, 2384,    0,    0, 2384, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384,    0,    0,    0,    0,    0, 2384,
    0, 2384, 2384, 2384, 2384, 2384, 2384,    0, 2384, 1459,
 1458, 1460,    0,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0, 2384, 2384,    0, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384,    0, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
    0, 2384, 2384, 2384,    0, 2384,    0,    0,    0, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384, 2384,    0, 2384, 2384,    0,    0,    0, 2394,    0,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,    0,    0,
 2384,    0, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384,
 2384, 2384, 2384, 2384, 2384, 2384, 2384, 2384, 1985, 2394,
 2394, 2394, 2394, 2394, 2394, 2394,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2394, 2394, 2394, 2394,
 2394,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2394,
    0, 2394,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1443,    0,    0,    0,    0,    0,    0,    0,    0,
 1444,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0, 1445,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2184,    0,    0,    0, 1446,    0,    0,
    0,    0,    0,    0, 1447,    0,    0,    0, 1446,    0,
    0,    0, 1448,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0, 2245,    0,    0, 1449,
    0, 1450,    0,    0,    0,    0,    0,    0,    0,    0,
 1449, 1451, 1450, 1452, 1453, 1454,    0,    0,    0,    0,
 1455,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0, 2394,    0, 2394,    0,    0,
 2394,    0, 2394, 2394, 2394,    0, 2394, 2394,    0, 2394,
    0,    0,    0,    0,    0, 2394,    0, 2394, 2394, 2394,
 2394, 1456, 2394,    0,    0, 2394,    0,    0, 2394,    0,
    0,    0, 1456,    0,    0, 2394,    0, 2394, 2394,    0,
 2394, 2394, 2394, 2394, 2394,    0,    0, 2394,    0,    0,
 2394,    0, 2394, 2394,    0,    0, 2394, 2394, 2394,    0,
 2394, 2394, 2394, 2394, 2394, 2394, 2394,    0,    0,    0,
 2394, 2394, 2394, 2394, 2394,    0,    0,    0, 2394,    0,
    0,    0,    0,    0,    0,    0, 2394, 2394, 2394, 2394,
 2394, 2394,    0,    0,    0, 2394, 2394,    0, 2394,    0,
 2394,    0, 2394, 2394, 2394,    0, 2394,    0, 2394, 2394,
 2394,    0, 2394, 2394, 2394, 2394, 2394,    0, 2394, 2394,
    0,    0,    0,    0, 2394,    0, 2394, 2394,    0,    0,
 2394, 2394, 2394,    0,    0, 2394,    0, 2394,    0,    0,
 2394, 2394, 2394, 2394,    0, 2394,    0, 2394,    0, 2394,
 2394, 2394, 2394, 2394,    0,    0, 2394, 2394, 2394,    0,
 2394, 2394,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2394, 2394, 2394,    0, 2394, 2394, 2394,    0,    0,
    0,    0,    0, 2394,    0,    0,    0, 2394,    0, 2394,
 2394, 2394, 2394,    0, 2394,    0, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394,    0,    0,    0,    0,    0,    0,
    0, 2185,    0, 2394, 2394,    0, 2394, 2394, 2394,    0,
    0,    0, 1457, 2394, 2394, 2394, 2394, 2394,    0, 2394,
    0,    0, 2394, 2394,    0,    0,    0, 2394, 2394,    0,
    0, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
    0, 2394,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394,    0, 2394, 2394, 2394, 2394, 2394, 2394,    0,
 2394, 2394, 2394, 2394,    0, 2394,    0, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394,    0, 2394, 2394, 2394, 2394,
    0, 2394, 2394, 2394, 2394, 2394,    0, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394,    0, 2394, 2394,    0,
    0,    0, 2394, 2394, 2394, 2394, 2394, 2394,    0,    0,
 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,    0,
    0,    0,    0,    0, 2394,    0, 2394, 2394, 2394, 2394,
 2394, 2394,    0, 2394,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,
 2394, 2394,    0, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394,    0, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394,    0, 2394, 2394, 2394,    0,
 2394,    0,    0,    0, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394, 2394,    0, 2394, 2394,
    0,    0,    0, 2399,    0, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394,    0,    0, 2394,    0, 2394, 2394, 2394,
 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394, 2394,
 2394, 2394, 2394, 1940, 2399, 2399, 2399, 2399, 2399, 2399,
 2399,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2399, 2399, 2399, 2399, 2399,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2399,    0, 2399,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1443,    0,    0,    0,    0,
    0,    0,    0,    0, 1444,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1445,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1446,    0,    0,    0,    0,    0,    0, 1447,    0,
    0,    0,    0,    0,    0,    0, 1448,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1449,    0, 1450,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,
    0,    0,    0,    0, 1455,    0,    0,    0,    0,    0,
 2399,    0, 2399,    0,    0, 2399,    0, 2399, 2399, 2399,
    0, 2399, 2399,    0, 2399,    0,    0,    0,    0,    0,
 2399,    0, 2399, 2399, 2399, 2399,    0, 2399,    0,    0,
 2399,    0,    0, 2399,    0, 1456,    0,    0,    0,    0,
 2399,    0, 2399, 2399,    0, 2399, 2399, 2399, 2399, 2399,
    0,    0, 2399,    0,    0, 2399,    0, 2399, 2399,    0,
    0, 2399, 2399, 2399,    0, 2399, 2399, 2399, 2399, 2399,
 2399, 2399,    0,    0,    0, 2399, 2399, 2399, 2399, 2399,
    0,    0,    0, 2399,    0,    0,    0,    0,    0,    0,
    0, 2399, 2399, 2399, 2399, 2399, 2399,    0,    0,    0,
 2399, 2399,    0, 2399,    0, 2399,    0, 2399, 2399, 2399,
 2275, 2399,    0, 2399, 2399, 2399,    0, 2399, 2399, 2399,
 2399, 2399,    0, 2399, 2399,    0,    0,    0,    0, 2399,
    0, 2399, 2399,    0,    0, 2399, 2399, 2399,    0,    0,
 2399,    0, 2399,    0,    0, 2399, 2399, 2399, 2399,    0,
 2399,    0, 2399,    0, 2399, 2399, 2399, 2399, 2399,    0,
    0, 2399, 2399, 2399,    0, 2399, 2399,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2399, 2399, 2399,    0,
 2399, 2399, 2399,    0,    0,    0,    0,    0, 2399,    0,
    0,    0, 2399,    0, 2399, 2399, 2399, 2399,    0, 2399,
    0, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2399, 2399,
    0, 2399, 2399, 2399,    0, 1457,    0,    0, 2399, 2399,
 2399, 2399, 2399,    0, 2399,    0,    0, 2399, 2399,    0,
    0,    0, 2399, 2399,    0,    0, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399,    0, 2399,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399,    0, 2399, 2399,
 2399, 2399, 2399, 2399,    0, 2399, 2399, 2399, 2399,    0,
 2399,    0, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
    0, 2399, 2399, 2399, 2399,    0, 2399, 2399, 2399, 2399,
 2399,    0, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399,    0, 2399, 2399,    0,    0,    0, 2399, 2399, 2399,
 2399, 2399, 2399,    0,    0, 2399, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399,    0,    0,    0,    0,    0, 2399,
    0, 2399, 2399, 2399, 2399, 2399, 2399,    0, 2399, 1459,
 1458, 1460,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2399, 2399,    0, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399,    0, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
    0, 2399, 2399, 2399,    0, 2399,    0,    0,    0, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399, 2399,    0, 2399, 2399,    0,    0,    0, 2405,    0,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,    0,    0,
 2399,    0, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399,
 2399, 2399, 2399, 2399, 2399, 2399, 2399, 2399, 1981, 2405,
 2405, 2405, 2405, 2405, 2405, 2405,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2405, 2405, 2405, 2405,
 2405,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2405,
    0, 2405,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1443,    0,    0,    0,    0,    0,    0,    0,    0,
 1444,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1445,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1446,    0,    0,
    0,    0,    0,    0, 1447,    0,    0,    0,    0,    0,
    0,    0, 1448,    0,    0, 2276,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1449,
    0, 1450,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0,
 1455,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2405,    0, 2405,    0,    0,
 2405,    0, 2405, 2405, 2405,    0, 2405, 2405,    0, 2405,
    0,    0,    0,    0,    0, 2405,    0, 2405, 2405, 2405,
 2405, 1456, 2405,    0,    0, 2405,    0,    0, 2405,    0,
    0,    0,    0,    0,    0, 2405,    0, 2405, 2405,    0,
 2405, 2405, 2405, 2405, 2405,    0,    0, 2405,    0,    0,
 2405,    0, 2405, 2405,    0,    0, 2405, 2405, 2405,    0,
 2405, 2405, 2405, 2405, 2405, 2405, 2405,    0,    0,    0,
 2405, 2405, 2405, 2405, 2405,    0,    0,    0, 2405,    0,
    0,    0,    0,    0,    0,    0, 2405, 2405, 2405, 2405,
 2405, 2405,    0,    0,    0, 2405, 2405,    0, 2405,    0,
 2405,    0, 2405, 2405, 2405,    0, 2405,    0, 2405, 2405,
 2405,    0, 2405, 2405, 2405, 2405, 2405,    0, 2405, 2405,
    0,    0,    0,    0, 2405,    0, 2405, 2405,    0,    0,
 2405, 2405, 2405,    0,    0, 2405,    0, 2405,    0,    0,
 2405, 2405, 2405, 2405,    0, 2405,    0, 2405,    0, 2405,
 2405, 2405, 2405, 2405,    0,    0, 2405, 2405, 2405,    0,
 2405, 2405,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2405, 2405, 2405,    0, 2405, 2405, 2405,    0,    0,
    0,    0,    0, 2405,    0,    0,    0, 2405,    0, 2405,
 2405, 2405, 2405,    0, 2405,    0, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405,    0,    0,    0,    0,    0,    0,
    0, 1457,    0, 2405, 2405,    0, 2405, 2405, 2405,    0,
    0,    0,    0, 2405, 2405, 2405, 2405, 2405,    0, 2405,
    0,    0, 2405, 2405,    0,    0,    0, 2405, 2405,    0,
    0, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
    0, 2405,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405,    0, 2405, 2405, 2405, 2405, 2405, 2405,    0,
 2405, 2405, 2405, 2405,    0, 2405,    0, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405,    0, 2405, 2405, 2405, 2405,
    0, 2405, 2405, 2405, 2405, 2405,    0, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405,    0, 2405, 2405,    0,
    0,    0, 2405, 2405, 2405, 2405, 2405, 2405,    0,    0,
 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,    0,
    0,    0,    0,    0, 2405,    0, 2405, 2405, 2405, 2405,
 2405, 2405,    0, 2405,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,
 2405, 2405,    0, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405,    0, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405,    0, 2405, 2405, 2405,    0,
 2405,    0,    0,    0, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405, 2405,    0, 2405, 2405,
    0,    0,    0, 2406,    0, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405,    0,    0, 2405,    0, 2405, 2405, 2405,
 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405, 2405,
 2405, 2405, 2405, 1982, 2406, 2406, 2406, 2406, 2406, 2406,
 2406,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2406, 2406, 2406, 2406, 2406,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2406,    0, 2406,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1443,    0,    0,    0,    0,
    0,    0,    0,    0, 1444,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1445,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1446,    0,    0,    0,    0,    0,    0, 1447,    0,
    0,    0,    0,    0,    0,    0, 1448,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1449,    0, 1450,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,
    0,    0,    0,    0, 1455,    0,    0,    0,    0,    0,
 2406,    0, 2406,    0,    0, 2406,    0, 2406, 2406, 2406,
    0, 2406, 2406,    0, 2406,    0,    0,    0,    0,    0,
 2406,    0, 2406, 2406, 2406, 2406,    0, 2406,    0,    0,
 2406,    0,    0, 2406,    0, 1456,    0,    0,    0,    0,
 2406,    0, 2406, 2406,    0, 2406, 2406, 2406, 2406, 2406,
    0,    0, 2406,    0,    0, 2406,    0, 2406, 2406,    0,
    0, 2406, 2406, 2406,    0, 2406, 2406, 2406, 2406, 2406,
 2406, 2406,    0,    0,    0, 2406, 2406, 2406, 2406, 2406,
    0,    0,    0, 2406,    0,    0,    0,    0,    0,    0,
    0, 2406, 2406, 2406, 2406, 2406, 2406,    0,    0,    0,
 2406, 2406,    0, 2406,    0, 2406,    0, 2406, 2406, 2406,
 2277, 2406,    0, 2406, 2406, 2406,    0, 2406, 2406, 2406,
 2406, 2406,    0, 2406, 2406,    0,    0,    0,    0, 2406,
    0, 2406, 2406,    0,    0, 2406, 2406, 2406,    0,    0,
 2406,    0, 2406,    0,    0, 2406, 2406, 2406, 2406,    0,
 2406,    0, 2406,    0, 2406, 2406, 2406, 2406, 2406,    0,
    0, 2406, 2406, 2406,    0, 2406, 2406,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2406, 2406, 2406,    0,
 2406, 2406, 2406,    0,    0,    0,    0,    0, 2406,    0,
    0,    0, 2406,    0, 2406, 2406, 2406, 2406,    0, 2406,
    0, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2406, 2406,
    0, 2406, 2406, 2406,    0, 1457,    0,    0, 2406, 2406,
 2406, 2406, 2406,    0, 2406,    0,    0, 2406, 2406,    0,
    0,    0, 2406, 2406,    0,    0, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406,    0, 2406,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406,    0, 2406, 2406,
 2406, 2406, 2406, 2406,    0, 2406, 2406, 2406, 2406,    0,
 2406,    0, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
    0, 2406, 2406, 2406, 2406,    0, 2406, 2406, 2406, 2406,
 2406,    0, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406,    0, 2406, 2406,    0,    0,    0, 2406, 2406, 2406,
 2406, 2406, 2406,    0,    0, 2406, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406,    0,    0,    0, 2217,    0, 2406,
 2216, 2406, 2406, 2406, 2406, 2406, 2406,    0, 2406,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 2406, 2406,    0, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406,    0, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
    0, 2406, 2406, 2406,    0, 2406,    0,    0,    0, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406, 2406,    0, 2406, 2406,    0,    0, 2143,    0,    0,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,    0,    0,
 2406,    0, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406,
 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2406, 2143, 2143,
 2143, 2143, 2143, 2143, 2143,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2143, 2143, 2143, 2143, 2143,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2143,    0,
 2143,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 1447,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 2143,    0, 2143,    0,    0, 2143,
    0, 2143, 2143, 2143,    0, 2143, 2143,    0, 2143,    0,
    0,    0,    0,    0, 2143,    0, 2143, 2143, 2143, 2143,
    0, 2143,    0,    0, 2143,    0,    0, 2143, 1456,    0,
    0,    0,    0,    0, 2143,    0, 2143, 2143,    0, 2143,
 2143, 2143, 2143, 2143,    0,    0, 2143,    0,    0, 2143,
    0, 2143, 2143,    0,    0, 2143, 2143, 2143,    0, 2143,
 2143, 2143, 2143, 2143, 2143, 2143,    0,    0,    0, 2143,
 2143, 2143, 2143, 2143,    0,    0,    0, 2143,    0,    0,
    0,    0,    0,    0,    0, 2143, 2143, 2143, 2143, 2143,
 2143,    0,    0,    0, 2143, 2143,    0, 2143,    0, 2143,
    0, 2143, 2143, 2143,    0, 2143,    0, 2143, 2143, 2143,
    0, 2143, 2143, 2143, 2143, 2143,    0, 2143, 2143,    0,
    0,    0,    0, 2143,    0, 2143, 2143,    0,    0, 2143,
 2143, 2143,    0,    0, 2143,    0, 2143,    0,    0, 2143,
 2143, 2143, 2143,    0, 2143,    0, 2143,    0, 2143, 2143,
 2143, 2143, 2143,    0,    0, 2143, 2143, 2143,    0, 2143,
 2143,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2143, 2143, 2143,    0, 2143, 2143, 2143,    0,    0,    0,
    0,    0, 2143,    0,    0,    0, 2143,    0, 2143, 2143,
 2143, 2143,    0, 2143,    0, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2143, 2143,    0, 2143, 2143, 2143, 1457,    0,
    0,    0, 2143, 2143, 2143, 2143, 2143,    0, 2143,    0,
    0, 2143, 2143,    0,    0,    0, 2143, 2143,    0,    0,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,    0,
 2143,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143,    0, 2143, 2143, 2143, 2143, 2143, 2143,    0, 2143,
 2143, 2143, 2143,    0, 2143,    0, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143,    0, 2143, 2143, 2143, 2143,    0,
 2143, 2143, 2143, 2143, 2143,    0, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143,    0, 2143, 2143,    0,    0,
    0, 2143, 2143, 2143, 2143, 2143, 2143,    0,    0, 2143,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,    0,    0,
    0, 2219,    0, 2143, 2218, 2143, 2143, 2143, 2143, 2143,
 2143,    0, 2143,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0,    0,    0,    0,    0, 2143,
 2143,    0, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143,    0, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143,    0, 2143, 2143, 2143,    0, 2143,
    0,    0,    0, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143, 2143,    0, 2143, 2143,    0,
    0, 2188,    0,    0, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143,    0,    0, 2143,    0, 2143, 2143, 2143, 2143,
 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143, 2143,
 2143, 2143, 2188, 2188, 2188, 2188, 2188, 2188, 2188,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2188,
 2188, 2188, 2188, 2188,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2188,    0, 2188,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1446,    0,
    0,    0,    0,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1449,    0, 1450,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0,    0,    0,    0, 2188,    0,
 2188,    0,    0, 2188,    0, 2188, 2188, 2188,    0, 2188,
 2188,    0, 2188,    0,    0,    0,    0,    0, 2188,    0,
 2188, 2188, 2188, 2188,    0, 2188,    0,    0, 2188,    0,
    0, 2188, 1456,    0,    0,    0,    0,    0, 2188,    0,
 2188, 2188,    0, 2188, 2188, 2188, 2188, 2188,    0,    0,
 2188,    0,    0, 2188,    0, 2188, 2188,    0,    0, 2188,
 2188, 2188,    0, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
    0,    0,    0, 2188, 2188, 2188, 2188, 2188,    0,    0,
    0, 2188,    0,    0,    0,    0,    0,    0,    0, 2188,
 2188, 2188, 2188, 2188, 2188,    0,    0,    0, 2188, 2188,
    0, 2188,    0, 2188,    0, 2188, 2188, 2188,    0, 2188,
    0, 2188, 2188, 2188,    0, 2188, 2188, 2188, 2188, 2188,
    0, 2188, 2188,    0,    0,    0,    0, 2188,    0, 2188,
 2188,    0,    0, 2188, 2188, 2188,    0,    0, 2188,    0,
 2188,    0,    0, 2188, 2188, 2188, 2188,    0, 2188,    0,
 2188,    0, 2188, 2188, 2188, 2188, 2188,    0,    0, 2188,
 2188, 2188,    0, 2188, 2188,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2188, 2188, 2188,    0, 2188, 2188,
 2188,    0,    0,    0,    0,    0, 2188,    0,    0,    0,
 2188,    0, 2188, 2188, 2188, 2188,    0, 2188,    0, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2188, 2188,    0, 2188,
 2188, 2188, 1457,    0,    0,    0, 2188, 2188, 2188, 2188,
 2188,    0, 2188,    0,    0, 2188, 2188,    0,    0,    0,
 2188, 2188,    0,    0, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188,    0, 2188,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188,    0, 2188, 2188, 2188, 2188,
 2188, 2188,    0, 2188, 2188, 2188, 2188,    0, 2188,    0,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,    0, 2188,
 2188, 2188, 2188,    0, 2188, 2188, 2188, 2188, 2188,    0,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,    0,
 2188, 2188,    0,    0,    0, 2188, 2188, 2188, 2188, 2188,
 2188,    0,    0, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188,    0,    0,    0, 2998,    0, 2188, 2997, 2188,
 2188, 2188, 2188, 2188, 2188,    0, 2188,    0,    0,    0,
    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0, 2188, 2188,    0, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188,    0, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,    0, 2188,
 2188, 2188,    0, 2188,    0,    0,    0, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
    0, 2188, 2188,    0,    0, 2189,    0,    0, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188,    0,    0, 2188,    0,
 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188, 2188,
 2188, 2188, 2188, 2188, 2188, 2188, 2189, 2189, 2189, 2189,
 2189, 2189, 2189,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2189, 2189, 2189, 2189, 2189,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2189,    0, 2189,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0,    0,    0,
    0,    0,    0,    0,    0, 1444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0,    0,    0,    0,    0, 1448,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1451,    0, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0,    0,    0,    0,
    0,    0, 2189,    0, 2189,    0,    0, 2189,    0, 2189,
 2189, 2189,    0, 2189, 2189,    0, 2189,    0,    0,    0,
    0,    0, 2189,    0, 2189, 2189, 2189, 2189,    0, 2189,
    0,    0, 2189,    0,    0, 2189, 1456,    0,    0,    0,
    0,    0, 2189,    0, 2189, 2189,    0, 2189, 2189, 2189,
 2189, 2189,    0,    0, 2189,    0,    0, 2189,    0, 2189,
 2189,    0,    0, 2189, 2189, 2189,    0, 2189, 2189, 2189,
 2189, 2189, 2189, 2189,    0,    0,    0, 2189, 2189, 2189,
 2189, 2189,    0,    0,    0, 2189,    0,    0,    0,    0,
    0,    0,    0, 2189, 2189, 2189, 2189, 2189, 2189,    0,
    0,    0, 2189, 2189,    0, 2189,    0, 2189,    0, 2189,
 2189, 2189,    0, 2189,    0, 2189, 2189, 2189,    0, 2189,
 2189, 2189, 2189, 2189,    0, 2189, 2189,    0,    0,    0,
    0, 2189,    0, 2189, 2189,    0,    0, 2189, 2189, 2189,
    0,    0, 2189,    0, 2189,    0,    0, 2189, 2189, 2189,
 2189,    0, 2189,    0, 2189,    0, 2189, 2189, 2189, 2189,
 2189,    0,    0, 2189, 2189, 2189,    0, 2189, 2189,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2189, 2189,
 2189,    0, 2189, 2189, 2189,    0,    0,    0,    0,    0,
 2189,    0,    0,    0, 2189,    0, 2189, 2189, 2189, 2189,
    0, 2189,    0, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2189, 2189,    0, 2189, 2189, 2189, 1457,    0,    0,    0,
 2189, 2189, 2189, 2189, 2189,    0, 2189,    0,    0, 2189,
 2189,    0,    0,    0, 2189, 2189,    0,    0, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189,    0, 2189,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,    0,
 2189, 2189, 2189, 2189, 2189, 2189,    0, 2189, 2189, 2189,
 2189,    0, 2189,    0, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189,    0, 2189, 2189, 2189, 2189,    0, 2189, 2189,
 2189, 2189, 2189,    0, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189,    0, 2189, 2189,    0,    0,    0, 2189,
 2189, 2189, 2189, 2189, 2189,    0,    0, 2189, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189,    0,    0,    0, 3000,
    0, 2189, 2999, 2189, 2189, 2189, 2189, 2189, 2189,    0,
 2189,    0,    0,    0,    0,    0,    0,    0, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0, 2189, 2189,    0,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,    0,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189,    0, 2189, 2189, 2189,    0, 2189,    0,    0,
    0, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189, 2189,    0, 2189, 2189,    0,    0, 2195,
    0,    0, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
    0,    0, 2189,    0, 2189, 2189, 2189, 2189, 2189, 2189,
 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189, 2189,
 2195, 2195, 2195, 2195, 2195, 2195, 2195,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2195, 2195, 2195,
 2195, 2195,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2195,    0, 2195,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1443,    0,    0,    0,    0,    0,    0,    0,    0, 1444,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1445,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1446,    0,    0,    0,
    0,    0,    0, 1447,    0,    0,    0,    0,    0,    0,
    0, 1448,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1449,    0,
 1450,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,
    0,    0,    0,    0,    0,    0, 2195,    0, 2195,    0,
    0, 2195,    0, 2195, 2195, 2195,    0, 2195, 2195,    0,
 2195,    0,    0,    0,    0,    0, 2195,    0, 2195, 2195,
 2195, 2195,    0, 2195,    0,    0, 2195,    0,    0, 2195,
 1456,    0,    0,    0,    0,    0, 2195,    0, 2195, 2195,
    0, 2195, 2195, 2195, 2195, 2195,    0,    0, 2195,    0,
    0, 2195,    0, 2195, 2195,    0,    0, 2195, 2195, 2195,
    0, 2195, 2195, 2195, 2195, 2195, 2195, 2195,    0,    0,
    0, 2195, 2195, 2195, 2195, 2195,    0,    0,    0, 2195,
    0,    0,    0,    0,    0,    0,    0, 2195, 2195, 2195,
 2195, 2195, 2195,    0,    0,    0, 2195, 2195,    0, 2195,
    0, 2195,    0, 2195, 2195, 2195,    0, 2195,    0, 2195,
 2195, 2195,    0, 2195, 2195, 2195, 2195, 2195,    0, 2195,
 2195,    0,    0,    0,    0, 2195,    0, 2195, 2195,    0,
    0, 2195, 2195, 2195,    0,    0, 2195,    0, 2195,    0,
    0, 2195, 2195, 2195, 2195,    0, 2195,    0, 2195,    0,
 2195, 2195, 2195, 2195, 2195,    0,    0, 2195, 2195, 2195,
    0, 2195, 2195,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2195, 2195, 2195,    0, 2195, 2195, 2195,    0,
    0,    0,    0,    0, 2195,    0,    0,    0, 2195,    0,
 2195, 2195, 2195, 2195,    0, 2195,    0, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2195, 2195,    0, 2195, 2195, 2195,
 1457,    0,    0,    0, 2195, 2195, 2195, 2195, 2195,    0,
 2195,    0,    0, 2195, 2195,    0,    0,    0, 2195, 2195,
    0,    0, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195,    0, 2195,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195,    0, 2195, 2195, 2195, 2195, 2195, 2195,
    0, 2195, 2195, 2195, 2195,    0, 2195,    0, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195,    0, 2195, 2195, 2195,
 2195,    0, 2195, 2195, 2195, 2195, 2195,    0, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195,    0, 2195, 2195,
    0,    0,    0, 2195, 2195, 2195, 2195, 2195, 2195,    0,
    0, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
    0,    0,    0, 3528,    0, 2195, 3527, 2195, 2195, 2195,
 2195, 2195, 2195,    0, 2195,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 2195, 2195,    0, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195,    0, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195,    0, 2195, 2195, 2195,
    0, 2195,    0,    0,    0, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,    0, 2195,
 2195,    0,    0, 2207,    0,    0, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195,    0,    0, 2195,    0, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2207, 2207, 2207, 2207, 2207, 2207,
 2207,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2207, 2207, 2207, 2207, 2207,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2207,    0, 2207,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2207,    0, 2207,    0,    0, 2207,    0, 2207, 2207, 2207,
    0, 2207, 2207,    0, 2207,    0,    0,    0,    0,    0,
 2207,    0, 2207, 2207, 2207, 2207,    0, 2207,    0,    0,
 2207,    0,    0, 2207, 1456,    0,    0,    0,    0,    0,
 2207,    0, 2207, 2207,    0, 2207, 2207, 2207, 2207, 2207,
    0,    0, 2207,    0,    0, 2207,    0, 2207, 2207,    0,
    0, 2207, 2207, 2207,    0, 2207, 2207, 2207, 2207, 2207,
 2207, 2207,    0,    0,    0, 2207, 2207, 2207, 2207, 2207,
    0,    0,    0, 2207,    0,    0,    0,    0,    0,    0,
    0, 2207, 2207, 2207, 2207, 2207, 2207,    0,    0,    0,
 2207, 2207,    0, 2207,    0, 2207,    0, 2207, 2207, 2207,
    0, 2207,    0, 2207, 2207, 2207,    0, 2207, 2207, 2207,
 2207, 2207,    0, 2207, 2207,    0,    0,    0,    0, 2207,
    0, 2207, 2207,    0,    0, 2207, 2207, 2207,    0,    0,
 2207,    0, 2207,    0,    0, 2207, 2207, 2207, 2207,    0,
 2207,    0, 2207,    0, 2207, 2207, 2207, 2207, 2207,    0,
    0, 2207, 2207, 2207,    0, 2207, 2207,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2207, 2207, 2207,    0,
 2207, 2207, 2207,    0,    0,    0,    0,    0, 2207,    0,
    0,    0, 2207,    0, 2207, 2207, 2207, 2207,    0, 2207,
    0, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2207, 2207,
    0, 2207, 2207, 2207, 1457,    0,    0,    0, 2207, 2207,
 2207, 2207, 2207,    0, 2207,    0,    0, 2207, 2207,    0,
    0,    0, 2207, 2207,    0,    0, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207,    0, 2207,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207,    0, 2207, 2207,
 2207, 2207, 2207, 2207,    0, 2207, 2207, 2207, 2207,    0,
 2207,    0, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
    0, 2207, 2207, 2207, 2207,    0, 2207, 2207, 2207, 2207,
 2207,    0, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207,    0, 2207, 2207,    0,    0,    0, 2207, 2207, 2207,
 2207, 2207, 2207,    0,    0, 2207, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207,    0,    0,    0,    0,    0, 2207,
 2191, 2207, 2207, 2207, 2207, 2207, 2207,    0, 2207,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 2207, 2207,    0, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207,    0, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
    0, 2207, 2207, 2207,    0, 2207,    0,    0,    0, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207, 2207,    0, 2207, 2207,    0,    0, 2352,    0,    0,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,    0,    0,
 2207,    0, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207,
 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2207, 2352, 2352,
 2352, 2352, 2352, 2352, 2352,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2352, 2352, 2352, 2352, 2352,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2352,    0,
 2352,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 1447,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 2352,    0, 2352,    0,    0, 2352,
    0, 2352, 2352, 2352,    0, 2352, 2352,    0, 2352,    0,
    0,    0,    0,    0, 2352,    0, 2352, 2352, 2352, 2352,
    0, 2352,    0,    0, 2352,    0,    0, 2352, 1456,    0,
    0,    0,    0,    0, 2352,    0, 2352, 2352,    0, 2352,
 2352, 2352, 2352, 2352,    0,    0, 2352,    0,    0, 2352,
    0, 2352, 2352,    0,    0, 2352, 2352, 2352,    0, 2352,
 2352, 2352, 2352, 2352, 2352, 2352,    0,    0,    0, 2352,
 2352, 2352, 2352, 2352,    0,    0,    0, 2352,    0,    0,
    0,    0,    0,    0,    0, 2352, 2352, 2352, 2352, 2352,
 2352,    0,    0,    0, 2352, 2352,    0, 2352,    0, 2352,
    0, 2352, 2352, 2352,    0, 2352,    0, 2352, 2352, 2352,
    0, 2352, 2352, 2352, 2352, 2352,    0, 2352, 2352,    0,
    0,    0,    0, 2352,    0, 2352, 2352,    0,    0, 2352,
 2352, 2352,    0,    0, 2352,    0, 2352,    0,    0, 2352,
 2352, 2352, 2352,    0, 2352,    0, 2352,    0, 2352, 2352,
 2352, 2352, 2352,    0,    0, 2352, 2352, 2352,    0, 2352,
 2352,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2352, 2352, 2352,    0, 2352, 2352, 2352,    0,    0,    0,
    0,    0, 2352,    0,    0,    0, 2352,    0, 2352, 2352,
 2352, 2352,    0, 2352,    0, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2352, 2352,    0, 2352, 2352, 2352, 1457,    0,
    0,    0, 2352, 2352, 2352, 2352, 2352,    0, 2352,    0,
    0, 2352, 2352,    0,    0,    0, 2352, 2352,    0,    0,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,    0,
 2352,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352,    0, 2352, 2352, 2352, 2352, 2352, 2352,    0, 2352,
 2352, 2352, 2352,    0, 2352,    0, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352,    0, 2352, 2352, 2352, 2352,    0,
 2352, 2352, 2352, 2352, 2352,    0, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352,    0, 2352, 2352,    0,    0,
    0, 2352, 2352, 2352, 2352, 2352, 2352,    0,    0, 2352,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,    0,    0,
    0,    0,    0, 2352, 2192, 2352, 2352, 2352, 2352, 2352,
 2352,    0, 2352,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0,    0,    0,    0,    0, 2352,
 2352,    0, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352,    0, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352,    0, 2352, 2352, 2352,    0, 2352,
    0,    0,    0, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352, 2352,    0, 2352, 2352,    0,
    0, 2360,    0,    0, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352,    0,    0, 2352,    0, 2352, 2352, 2352, 2352,
 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352, 2352,
 2352, 2352, 2360, 2360, 2360, 2360, 2360, 2360, 2360,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2360,
 2360, 2360, 2360, 2360,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2360,    0, 2360,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1446,    0,
    0,    0,    0,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1449,    0, 1450,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0,    0,    0,    0, 2360,    0,
 2360,    0,    0, 2360,    0, 2360, 2360, 2360,    0, 2360,
 2360,    0, 2360,    0,    0,    0,    0,    0, 2360,    0,
 2360, 2360, 2360, 2360,    0, 2360,    0,    0, 2360,    0,
    0, 2360, 1456,    0,    0,    0,    0,    0, 2360,    0,
 2360, 2360,    0, 2360, 2360, 2360, 2360, 2360,    0,    0,
 2360,    0,    0, 2360,    0, 2360, 2360,    0,    0, 2360,
 2360, 2360,    0, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
    0,    0,    0, 2360, 2360, 2360, 2360, 2360,    0,    0,
    0, 2360,    0,    0,    0,    0,    0,    0,    0, 2360,
 2360, 2360, 2360, 2360, 2360,    0,    0,    0, 2360, 2360,
    0, 2360,    0, 2360,    0, 2360, 2360, 2360,    0, 2360,
    0, 2360, 2360, 2360,    0, 2360, 2360, 2360, 2360, 2360,
    0, 2360, 2360,    0,    0,    0,    0, 2360,    0, 2360,
 2360,    0,    0, 2360, 2360, 2360,    0,    0, 2360,    0,
 2360,    0,    0, 2360, 2360, 2360, 2360,    0, 2360,    0,
 2360,    0, 2360, 2360, 2360, 2360, 2360,    0,    0, 2360,
 2360, 2360,    0, 2360, 2360,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2360, 2360, 2360,    0, 2360, 2360,
 2360,    0,    0,    0,    0,    0, 2360,    0,    0,    0,
 2360,    0, 2360, 2360, 2360, 2360,    0, 2360,    0, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2360, 2360,    0, 2360,
 2360, 2360, 1457,    0,    0,    0, 2360, 2360, 2360, 2360,
 2360,    0, 2360,    0,    0, 2360, 2360,    0,    0,    0,
 2360, 2360,    0,    0, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360,    0, 2360,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360,    0, 2360, 2360, 2360, 2360,
 2360, 2360,    0, 2360, 2360, 2360, 2360,    0, 2360,    0,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,    0, 2360,
 2360, 2360, 2360,    0, 2360, 2360, 2360, 2360, 2360,    0,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,    0,
 2360, 2360,    0,    0,    0, 2360, 2360, 2360, 2360, 2360,
 2360,    0,    0, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360,    0,    0,    0,    0,    0, 2360, 2208, 2360,
 2360, 2360, 2360, 2360, 2360,    0, 2360,    0,    0,    0,
    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0, 2360, 2360,    0, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360,    0, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,    0, 2360,
 2360, 2360,    0, 2360,    0,    0,    0, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
    0, 2360, 2360,    0,    0, 2385,    0,    0, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360,    0,    0, 2360,    0,
 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360, 2360,
 2360, 2360, 2360, 2360, 2360, 2360, 2385, 2385, 2385, 2385,
 2385, 2385, 2385,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2385, 2385, 2385, 2385, 2385,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2385,    0, 2385,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0,    0,    0,
    0,    0,    0,    0,    0, 1444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0,    0,    0,    0,    0, 1448,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1451,    0, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0,    0,    0,    0,
    0,    0, 2385,    0, 2385,    0,    0, 2385,    0, 2385,
 2385, 2385,    0, 2385, 2385,    0, 2385,    0,    0,    0,
    0,    0, 2385,    0, 2385, 2385, 2385, 2385,    0, 2385,
    0,    0, 2385,    0,    0, 2385, 1456,    0,    0,    0,
    0,    0, 2385,    0, 2385, 2385,    0, 2385, 2385, 2385,
 2385, 2385,    0,    0, 2385,    0,    0, 2385,    0, 2385,
 2385,    0,    0, 2385, 2385, 2385,    0, 2385, 2385, 2385,
 2385, 2385, 2385, 2385,    0,    0,    0, 2385, 2385, 2385,
 2385, 2385,    0,    0,    0, 2385,    0,    0,    0,    0,
    0,    0,    0, 2385, 2385, 2385, 2385, 2385, 2385,    0,
    0,    0, 2385, 2385,    0, 2385,    0, 2385,    0, 2385,
 2385, 2385,    0, 2385,    0, 2385, 2385, 2385,    0, 2385,
 2385, 2385, 2385, 2385,    0, 2385, 2385,    0,    0,    0,
    0, 2385,    0, 2385, 2385,    0,    0, 2385, 2385, 2385,
    0,    0, 2385,    0, 2385,    0,    0, 2385, 2385, 2385,
 2385,    0, 2385,    0, 2385,    0, 2385, 2385, 2385, 2385,
 2385,    0,    0, 2385, 2385, 2385,    0, 2385, 2385,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2385, 2385,
 2385,    0, 2385, 2385, 2385,    0,    0,    0,    0,    0,
 2385,    0,    0,    0, 2385,    0, 2385, 2385, 2385, 2385,
    0, 2385,    0, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2385, 2385,    0, 2385, 2385, 2385, 1457,    0,    0,    0,
 2385, 2385, 2385, 2385, 2385,    0, 2385,    0,    0, 2385,
 2385,    0,    0,    0, 2385, 2385,    0,    0, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385,    0, 2385,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,    0,
 2385, 2385, 2385, 2385, 2385, 2385,    0, 2385, 2385, 2385,
 2385,    0, 2385,    0, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385,    0, 2385, 2385, 2385, 2385,    0, 2385, 2385,
 2385, 2385, 2385,    0, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385,    0, 2385, 2385,    0,    0,    0, 2385,
 2385, 2385, 2385, 2385, 2385,    0,    0, 2385, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385,    0,    0,    0,    0,
    0, 2385, 2214, 2385, 2385, 2385, 2385, 2385, 2385,    0,
 2385,    0,    0,    0,    0,    0,    0,    0, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0, 2385, 2385,    0,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,    0,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385,    0, 2385, 2385, 2385,    0, 2385,    0,    0,
    0, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385, 2385,    0, 2385, 2385,    0,    0, 2386,
    0,    0, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
    0,    0, 2385,    0, 2385, 2385, 2385, 2385, 2385, 2385,
 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385, 2385,
 2386, 2386, 2386, 2386, 2386, 2386, 2386,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2386, 2386, 2386,
 2386, 2386,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2386,    0, 2386,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1443,    0,    0,    0,    0,    0,    0,    0,    0, 1444,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1445,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1446,    0,    0,    0,
    0,    0,    0, 1447,    0,    0,    0,    0,    0,    0,
    0, 1448,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1449,    0,
 1450,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,
    0,    0,    0,    0,    0,    0, 2386,    0, 2386,    0,
    0, 2386,    0, 2386, 2386, 2386,    0, 2386, 2386,    0,
 2386,    0,    0,    0,    0,    0, 2386,    0, 2386, 2386,
 2386, 2386,    0, 2386,    0,    0, 2386,    0,    0, 2386,
 1456,    0,    0,    0,    0,    0, 2386,    0, 2386, 2386,
    0, 2386, 2386, 2386, 2386, 2386,    0,    0, 2386,    0,
    0, 2386,    0, 2386, 2386,    0,    0, 2386, 2386, 2386,
    0, 2386, 2386, 2386, 2386, 2386, 2386, 2386,    0,    0,
    0, 2386, 2386, 2386, 2386, 2386,    0,    0,    0, 2386,
    0,    0,    0,    0,    0,    0,    0, 2386, 2386, 2386,
 2386, 2386, 2386,    0,    0,    0, 2386, 2386,    0, 2386,
    0, 2386,    0, 2386, 2386, 2386,    0, 2386,    0, 2386,
 2386, 2386,    0, 2386, 2386, 2386, 2386, 2386,    0, 2386,
 2386,    0,    0,    0,    0, 2386,    0, 2386, 2386,    0,
    0, 2386, 2386, 2386,    0,    0, 2386,    0, 2386,    0,
    0, 2386, 2386, 2386, 2386,    0, 2386,    0, 2386,    0,
 2386, 2386, 2386, 2386, 2386,    0,    0, 2386, 2386, 2386,
    0, 2386, 2386,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2386, 2386, 2386,    0, 2386, 2386, 2386,    0,
    0,    0,    0,    0, 2386,    0,    0,    0, 2386,    0,
 2386, 2386, 2386, 2386,    0, 2386,    0, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2386, 2386,    0, 2386, 2386, 2386,
 1457,    0,    0,    0, 2386, 2386, 2386, 2386, 2386,    0,
 2386,    0,    0, 2386, 2386,    0,    0,    0, 2386, 2386,
    0,    0, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386,    0, 2386,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386,    0, 2386, 2386, 2386, 2386, 2386, 2386,
    0, 2386, 2386, 2386, 2386,    0, 2386,    0, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386,    0, 2386, 2386, 2386,
 2386,    0, 2386, 2386, 2386, 2386, 2386,    0, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386,    0, 2386, 2386,
    0,    0,    0, 2386, 2386, 2386, 2386, 2386, 2386,    0,
    0, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
    0,    0,    0,    0,    0, 2386, 2224, 2386, 2386, 2386,
 2386, 2386, 2386,    0, 2386,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 2386, 2386,    0, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386,    0, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386,    0, 2386, 2386, 2386,
    0, 2386,    0,    0,    0, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,    0, 2386,
 2386,    0,    0, 2261,    0,    0, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386,    0,    0, 2386,    0, 2386, 2386,
 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386, 2386,
 2386, 2386, 2386, 2386, 2261, 2261, 2261, 2261, 2261, 2261,
 2261,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2261, 2261, 2261, 2261,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2261,    0, 2261,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2261,    0, 2261,    0,    0, 2261,    0, 2261, 2261, 2261,
    0, 2261, 2261,    0, 2261,    0,    0,    0,    0,    0,
 2261,    0, 2261, 2261, 2261, 2261,    0, 2261,    0,    0,
 2261,    0,    0, 2261, 1456,    0,    0,    0,    0,    0,
 2261,    0, 2261, 2261,    0, 2261, 2261, 2261, 2261, 2261,
    0,    0, 2261,    0,    0, 2261,    0, 2261, 2261,    0,
    0, 2261, 2261, 2261,    0, 2261, 2261, 2261, 2261, 2261,
 2261, 2261,    0,    0,    0, 2261, 2261, 2261, 2261, 2261,
    0,    0,    0, 2261,    0,    0,    0,    0,    0,    0,
    0, 2261, 2261, 2261, 2261, 2261, 2261,    0,    0,    0,
 2261, 2261,    0, 2261,    0, 2261,    0, 2261, 2261, 2261,
    0, 2261,    0, 2261, 2261, 2261,    0, 2261, 2261, 2261,
 2261, 2261,    0, 2261, 2261,    0,    0,    0,    0, 2261,
    0, 2261, 2261,    0,    0, 2261, 2261, 2261,    0,    0,
 2261,    0, 2261,    0,    0, 2261, 2261, 2261, 2261,    0,
 2261,    0, 2261,    0, 2261, 2261, 2261, 2261, 2261,    0,
    0, 2261, 2261, 2261,    0, 2261, 2261,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2261, 2261, 2261,    0,
 2261, 2261, 2261,    0,    0,    0,    0,    0, 2261,    0,
    0,    0, 2261,    0, 2261, 2261, 2261, 2261,    0, 2261,
    0, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2261, 2261,
    0, 2261, 2261, 2261, 1457,    0,    0,    0, 2261, 2261,
 2261, 2261, 2261,    0, 2261,    0,    0, 2261, 2261,    0,
    0,    0, 2261, 2261,    0,    0, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261,    0, 2261,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261,    0, 2261, 2261,
 2261, 2261, 2261, 2261,    0, 2261, 2261, 2261, 2261,    0,
 2261,    0, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
    0, 2261, 2261, 2261, 2261,    0, 2261, 2261, 2261, 2261,
 2261,    0, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261,    0, 2261, 2261,    0,    0,    0, 2261, 2261, 2261,
 2261, 2261, 2261,    0,    0, 2261, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261,    0,    0,    0,    0,    0, 2261,
 2225, 2261, 2261, 2261, 2261, 2261, 2261,    0, 2261,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 2261, 2261,    0, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261,    0, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
    0, 2261, 2261, 2261,    0, 2261,    0,    0,    0, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261, 2261,    0, 2261, 2261,    0,    0, 2287,    0,    0,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,    0,    0,
 2261,    0, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261,
 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2261, 2287, 2287,
 2287, 2287, 2287, 2287, 2287,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2287, 2287, 2287, 2287,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2287,    0,
 2287,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 1447,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 2287,    0, 2287,    0,    0, 2287,
    0, 2287, 2287, 2287,    0, 2287, 2287,    0, 2287,    0,
    0,    0,    0,    0, 2287,    0, 2287, 2287, 2287, 2287,
    0, 2287,    0,    0, 2287,    0,    0, 2287, 1456,    0,
    0,    0,    0,    0, 2287,    0, 2287, 2287,    0, 2287,
 2287, 2287, 2287, 2287,    0,    0, 2287,    0,    0, 2287,
    0, 2287, 2287,    0,    0, 2287, 2287, 2287,    0, 2287,
 2287, 2287, 2287, 2287, 2287, 2287,    0,    0,    0, 2287,
 2287, 2287, 2287, 2287,    0,    0,    0, 2287,    0,    0,
    0,    0,    0,    0,    0, 2287, 2287, 2287, 2287, 2287,
 2287,    0,    0,    0, 2287, 2287,    0, 2287,    0, 2287,
    0, 2287, 2287, 2287,    0, 2287,    0, 2287, 2287, 2287,
    0, 2287, 2287, 2287, 2287, 2287,    0, 2287, 2287,    0,
    0,    0,    0, 2287,    0, 2287, 2287,    0,    0, 2287,
 2287, 2287,    0,    0, 2287,    0, 2287,    0,    0, 2287,
 2287, 2287, 2287,    0, 2287,    0, 2287,    0, 2287, 2287,
 2287, 2287, 2287,    0,    0, 2287, 2287, 2287,    0, 2287,
 2287,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2287, 2287, 2287,    0, 2287, 2287, 2287,    0,    0,    0,
    0,    0, 2287,    0,    0,    0, 2287,    0, 2287, 2287,
 2287, 2287,    0, 2287,    0, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2287, 2287,    0, 2287, 2287, 2287, 1457,    0,
    0,    0, 2287, 2287, 2287, 2287, 2287,    0, 2287,    0,
    0, 2287, 2287,    0,    0,    0, 2287, 2287,    0,    0,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,    0,
 2287,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287,    0, 2287, 2287, 2287, 2287, 2287, 2287,    0, 2287,
 2287, 2287, 2287,    0, 2287,    0, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287,    0, 2287, 2287, 2287, 2287,    0,
 2287, 2287, 2287, 2287, 2287,    0, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287,    0, 2287, 2287,    0,    0,
    0, 2287, 2287, 2287, 2287, 2287, 2287,    0,    0, 2287,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,    0,    0,
    0,    0,    0, 2287, 2226, 2287, 2287, 2287, 2287, 2287,
 2287,    0, 2287,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0,    0,    0,    0,    0, 2287,
 2287,    0, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287,    0, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287,    0, 2287, 2287, 2287,    0, 2287,
    0,    0,    0, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287, 2287,    0, 2287, 2287,    0,
    0, 2148,    0,    0, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287,    0,    0, 2287,    0, 2287, 2287, 2287, 2287,
 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287, 2287,
 2287, 2287, 2148, 2148, 2148, 2148, 2148, 2148, 2148,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2148, 2148, 2148, 2148,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2148,    0, 2148,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1446,    0,
    0,    0,    0,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1449,    0, 1450,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0,    0,    0,    0, 2148,    0,
 2148,    0,    0, 2148,    0, 2148, 2148, 2148,    0, 2148,
 2148,    0, 2148,    0,    0,    0,    0,    0, 2148,    0,
 2148, 2148, 2148, 2148,    0, 2148,    0,    0, 2148,    0,
    0, 2148, 1456,    0,    0,    0,    0,    0, 2148,    0,
 2148, 2148,    0, 2148, 2148, 2148, 2148, 2148,    0,    0,
 2148,    0,    0, 2148,    0, 2148, 2148,    0,    0, 2148,
 2148, 2148,    0, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
    0,    0,    0, 2148, 2148, 2148, 2148, 2148,    0,    0,
    0, 2148,    0,    0,    0,    0,    0,    0,    0, 2148,
 2148, 2148, 2148, 2148, 2148,    0,    0,    0, 2148, 2148,
    0, 2148,    0, 2148,    0, 2148, 2148, 2148,    0, 2148,
    0, 2148, 2148, 2148,    0, 2148, 2148, 2148, 2148, 2148,
    0, 2148, 2148,    0,    0,    0,    0, 2148,    0, 2148,
 2148,    0,    0, 2148, 2148, 2148,    0,    0, 2148,    0,
 2148,    0,    0, 2148, 2148, 2148, 2148,    0, 2148,    0,
 2148,    0, 2148, 2148, 2148, 2148, 2148,    0,    0, 2148,
 2148, 2148,    0, 2148, 2148,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2148, 2148, 2148,    0, 2148, 2148,
 2148,    0,    0,    0,    0,    0, 2148,    0,    0,    0,
 2148,    0, 2148, 2148, 2148, 2148,    0, 2148,    0, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2148, 2148,    0, 2148,
 2148, 2148, 1457,    0,    0,    0, 2148, 2148, 2148, 2148,
 2148,    0, 2148,    0,    0, 2148, 2148,    0,    0,    0,
 2148, 2148,    0,    0, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148,    0, 2148,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148,    0, 2148, 2148, 2148, 2148,
 2148, 2148,    0, 2148, 2148, 2148, 2148,    0, 2148,    0,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,    0, 2148,
 2148, 2148, 2148,    0, 2148, 2148, 2148, 2148, 2148,    0,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,    0,
 2148, 2148,    0,    0,    0, 2148, 2148, 2148, 2148, 2148,
 2148,    0,    0, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148,    0,    0,    0,    0,    0, 2148, 2227, 2148,
 2148, 2148, 2148, 2148, 2148,    0, 2148,    0,    0,    0,
    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0, 2148, 2148,    0, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148,    0, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,    0, 2148,
 2148, 2148,    0, 2148,    0,    0,    0, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
    0, 2148, 2148,    0,    0, 2144,    0,    0, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148,    0,    0, 2148,    0,
 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148, 2148,
 2148, 2148, 2148, 2148, 2148, 2148, 2144, 2144, 2144, 2144,
 2144, 2144, 2144,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2144, 2144, 2144, 2144,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2144,    0, 2144,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0,    0,    0,
    0,    0,    0,    0,    0, 1444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0,    0,    0,    0,    0, 1448,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1451,    0, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0,    0,    0,    0,
    0,    0, 2144,    0, 2144,    0,    0, 2144,    0, 2144,
 2144, 2144,    0, 2144, 2144,    0, 2144,    0,    0,    0,
    0,    0, 2144,    0, 2144, 2144, 2144, 2144,    0, 2144,
    0,    0, 2144,    0,    0, 2144, 1456,    0,    0,    0,
    0,    0, 2144,    0, 2144, 2144,    0, 2144, 2144, 2144,
 2144, 2144,    0,    0, 2144,    0,    0, 2144,    0, 2144,
 2144,    0,    0, 2144, 2144, 2144,    0, 2144, 2144, 2144,
 2144, 2144, 2144, 2144,    0,    0,    0, 2144, 2144, 2144,
 2144, 2144,    0,    0,    0, 2144,    0,    0,    0,    0,
    0,    0,    0, 2144, 2144, 2144, 2144, 2144, 2144,    0,
    0,    0, 2144, 2144,    0, 2144,    0, 2144,    0, 2144,
 2144, 2144,    0, 2144,    0, 2144, 2144, 2144,    0, 2144,
 2144, 2144, 2144, 2144,    0, 2144, 2144,    0,    0,    0,
    0, 2144,    0, 2144, 2144,    0,    0, 2144, 2144, 2144,
    0,    0, 2144,    0, 2144,    0,    0, 2144, 2144, 2144,
 2144,    0, 2144,    0, 2144,    0, 2144, 2144, 2144, 2144,
 2144,    0,    0, 2144, 2144, 2144,    0, 2144, 2144,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2144, 2144,
 2144,    0, 2144, 2144, 2144,    0,    0,    0,    0,    0,
 2144,    0,    0,    0, 2144,    0, 2144, 2144, 2144, 2144,
    0, 2144,    0, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2144, 2144,    0, 2144, 2144, 2144, 1457,    0,    0,    0,
 2144, 2144, 2144, 2144, 2144,    0, 2144,    0,    0, 2144,
 2144,    0,    0,    0, 2144, 2144,    0,    0, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144,    0, 2144,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,    0,
 2144, 2144, 2144, 2144, 2144, 2144,    0, 2144, 2144, 2144,
 2144,    0, 2144,    0, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144,    0, 2144, 2144, 2144, 2144,    0, 2144, 2144,
 2144, 2144, 2144,    0, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144,    0, 2144, 2144,    0,    0,    0, 2144,
 2144, 2144, 2144, 2144, 2144,    0,    0, 2144, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144,    0,    0,    0,    0,
    0, 2144, 2228, 2144, 2144, 2144, 2144, 2144, 2144,    0,
 2144,    0,    0,    0,    0,    0,    0,    0, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0, 2144, 2144,    0,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,    0,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144,    0, 2144, 2144, 2144,    0, 2144,    0,    0,
    0, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144, 2144,    0, 2144, 2144,    0,    0, 2156,
    0,    0, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
    0,    0, 2144,    0, 2144, 2144, 2144, 2144, 2144, 2144,
 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144, 2144,
 2156, 2156, 2156, 2156, 2156, 2156, 2156,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2156, 2156,
 2156, 2156,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2156,    0, 2156,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1443,    0,    0,    0,    0,    0,    0,    0,    0, 1444,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1445,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1446,    0,    0,    0,
    0,    0,    0, 1447,    0,    0,    0,    0,    0,    0,
    0, 1448,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1449,    0,
 1450,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,
    0,    0,    0,    0,    0,    0, 2156,    0, 2156,    0,
    0, 2156,    0, 2156, 2156, 2156,    0, 2156, 2156,    0,
 2156,    0,    0,    0,    0,    0, 2156,    0, 2156, 2156,
 2156, 2156,    0, 2156,    0,    0, 2156,    0,    0, 2156,
 1456,    0,    0,    0,    0,    0, 2156,    0, 2156, 2156,
    0, 2156, 2156, 2156, 2156, 2156,    0,    0, 2156,    0,
    0, 2156,    0, 2156, 2156,    0,    0, 2156, 2156, 2156,
    0, 2156, 2156, 2156, 2156, 2156, 2156, 2156,    0,    0,
    0, 2156, 2156, 2156, 2156, 2156,    0,    0,    0, 2156,
    0,    0,    0,    0,    0,    0,    0, 2156, 2156, 2156,
 2156, 2156, 2156,    0,    0,    0, 2156, 2156,    0, 2156,
    0, 2156,    0, 2156, 2156, 2156,    0, 2156,    0, 2156,
 2156, 2156,    0, 2156, 2156, 2156, 2156, 2156,    0, 2156,
 2156,    0,    0,    0,    0, 2156,    0, 2156, 2156,    0,
    0, 2156, 2156, 2156,    0,    0, 2156,    0, 2156,    0,
    0, 2156, 2156, 2156, 2156,    0, 2156,    0, 2156,    0,
 2156, 2156, 2156, 2156, 2156,    0,    0, 2156, 2156, 2156,
    0, 2156, 2156,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2156, 2156, 2156,    0, 2156, 2156, 2156,    0,
    0,    0,    0,    0, 2156,    0,    0,    0, 2156,    0,
 2156, 2156, 2156, 2156,    0, 2156,    0, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2156, 2156,    0, 2156, 2156, 2156,
 1457,    0,    0,    0, 2156, 2156, 2156, 2156, 2156,    0,
 2156,    0,    0, 2156, 2156,    0,    0,    0, 2156, 2156,
    0,    0, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156,    0, 2156,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156,    0, 2156, 2156, 2156, 2156, 2156, 2156,
    0, 2156, 2156, 2156, 2156,    0, 2156,    0, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156,    0, 2156, 2156, 2156,
 2156,    0, 2156, 2156, 2156, 2156, 2156,    0, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156,    0, 2156, 2156,
    0,    0,    0, 2156, 2156, 2156, 2156, 2156, 2156,    0,
    0, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
    0,    0,    0,    0,    0, 2156, 2229, 2156, 2156, 2156,
 2156, 2156, 2156,    0, 2156,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 2156, 2156,    0, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156,    0, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156,    0, 2156, 2156, 2156,
    0, 2156,    0,    0,    0, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,    0, 2156,
 2156,    0,    0, 2161,    0,    0, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156,    0,    0, 2156,    0, 2156, 2156,
 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156, 2156,
 2156, 2156, 2156, 2156, 2161, 2161, 2161, 2161, 2161, 2161,
 2161,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2161, 2161, 2161, 2161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2161,    0, 2161,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2161,    0, 2161,    0,    0, 2161,    0, 2161, 2161, 2161,
    0, 2161, 2161,    0, 2161,    0,    0,    0,    0,    0,
 2161,    0, 2161, 2161, 2161, 2161,    0, 2161,    0,    0,
 2161,    0,    0, 2161, 1456,    0,    0,    0,    0,    0,
 2161,    0, 2161, 2161,    0, 2161, 2161, 2161, 2161, 2161,
    0,    0, 2161,    0,    0, 2161,    0, 2161, 2161,    0,
    0, 2161, 2161, 2161,    0, 2161, 2161, 2161, 2161, 2161,
 2161, 2161,    0,    0,    0, 2161, 2161, 2161, 2161, 2161,
    0,    0,    0, 2161,    0,    0,    0,    0,    0,    0,
    0, 2161, 2161, 2161, 2161, 2161, 2161,    0,    0,    0,
 2161, 2161,    0, 2161,    0, 2161,    0, 2161, 2161, 2161,
    0, 2161,    0, 2161, 2161, 2161,    0, 2161, 2161, 2161,
 2161, 2161,    0, 2161, 2161,    0,    0,    0,    0, 2161,
    0, 2161, 2161,    0,    0, 2161, 2161, 2161,    0,    0,
 2161,    0, 2161,    0,    0, 2161, 2161, 2161, 2161,    0,
 2161,    0, 2161,    0, 2161, 2161, 2161, 2161, 2161,    0,
    0, 2161, 2161, 2161,    0, 2161, 2161,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2161, 2161, 2161,    0,
 2161, 2161, 2161,    0,    0,    0,    0,    0, 2161,    0,
    0,    0, 2161,    0, 2161, 2161, 2161, 2161,    0, 2161,
    0, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2161, 2161,
    0, 2161, 2161, 2161, 1457,    0,    0,    0, 2161, 2161,
 2161, 2161, 2161,    0, 2161,    0,    0, 2161, 2161,    0,
    0,    0, 2161, 2161,    0,    0, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161,    0, 2161,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161,    0, 2161, 2161,
 2161, 2161, 2161, 2161,    0, 2161, 2161, 2161, 2161,    0,
 2161,    0, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
    0, 2161, 2161, 2161, 2161,    0, 2161, 2161, 2161, 2161,
 2161,    0, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161,    0, 2161, 2161,    0,    0,    0, 2161, 2161, 2161,
 2161, 2161, 2161,    0,    0, 2161, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161,    0,    0,    0,    0,    0, 2161,
 2230, 2161, 2161, 2161, 2161, 2161, 2161,    0, 2161,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 2161, 2161,    0, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161,    0, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
    0, 2161, 2161, 2161,    0, 2161,    0,    0,    0, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161, 2161,    0, 2161, 2161,    0,    0, 2190,    0,    0,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,    0,    0,
 2161,    0, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161,
 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2161, 2190, 2190,
 2190, 2190, 2190, 2190, 2190,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2190, 2190, 2190, 2190,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2190,    0,
 2190,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 1447,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 2190,    0, 2190,    0,    0, 2190,
    0, 2190, 2190, 2190,    0, 2190, 2190,    0, 2190,    0,
    0,    0,    0,    0, 2190,    0, 2190, 2190, 2190, 2190,
    0, 2190,    0,    0, 2190,    0,    0, 2190, 1456,    0,
    0,    0,    0,    0, 2190,    0, 2190, 2190,    0, 2190,
 2190, 2190, 2190, 2190,    0,    0, 2190,    0,    0, 2190,
    0, 2190, 2190,    0,    0, 2190, 2190, 2190,    0, 2190,
 2190, 2190, 2190, 2190, 2190, 2190,    0,    0,    0, 2190,
 2190, 2190, 2190, 2190,    0,    0,    0, 2190,    0,    0,
    0,    0,    0,    0,    0, 2190, 2190, 2190, 2190, 2190,
 2190,    0,    0,    0, 2190, 2190,    0, 2190,    0, 2190,
    0, 2190, 2190, 2190,    0, 2190,    0, 2190, 2190, 2190,
    0, 2190, 2190, 2190, 2190, 2190,    0, 2190, 2190,    0,
    0,    0,    0, 2190,    0, 2190, 2190,    0,    0, 2190,
 2190, 2190,    0,    0, 2190,    0, 2190,    0,    0, 2190,
 2190, 2190, 2190,    0, 2190,    0, 2190,    0, 2190, 2190,
 2190, 2190, 2190,    0,    0, 2190, 2190, 2190,    0, 2190,
 2190,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2190, 2190, 2190,    0, 2190, 2190, 2190,    0,    0,    0,
    0,    0, 2190,    0,    0,    0, 2190,    0, 2190, 2190,
 2190, 2190,    0, 2190,    0, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2190, 2190,    0, 2190, 2190, 2190, 1457,    0,
    0,    0, 2190, 2190, 2190, 2190, 2190,    0, 2190,    0,
    0, 2190, 2190,    0,    0,    0, 2190, 2190,    0,    0,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,    0,
 2190,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190,    0, 2190, 2190, 2190, 2190, 2190, 2190,    0, 2190,
 2190, 2190, 2190,    0, 2190,    0, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190,    0, 2190, 2190, 2190, 2190,    0,
 2190, 2190, 2190, 2190, 2190,    0, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190,    0, 2190, 2190,    0,    0,
    0, 2190, 2190, 2190, 2190, 2190, 2190,    0,    0, 2190,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,    0,    0,
    0,    0,    0, 2190, 2231, 2190, 2190, 2190, 2190, 2190,
 2190,    0, 2190,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0,    0,    0,    0,    0, 2190,
 2190,    0, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190,    0, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190,    0, 2190, 2190, 2190,    0, 2190,
    0,    0,    0, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190, 2190,    0, 2190, 2190,    0,
    0, 2196,    0,    0, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190,    0,    0, 2190,    0, 2190, 2190, 2190, 2190,
 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190, 2190,
 2190, 2190, 2196, 2196, 2196, 2196, 2196, 2196, 2196,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2196, 2196, 2196, 2196,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2196,    0, 2196,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1446,    0,
    0,    0,    0,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1449,    0, 1450,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0,    0,    0,    0, 2196,    0,
 2196,    0,    0, 2196,    0, 2196, 2196, 2196,    0, 2196,
 2196,    0, 2196,    0,    0,    0,    0,    0, 2196,    0,
 2196, 2196, 2196, 2196,    0, 2196,    0,    0, 2196,    0,
    0, 2196, 1456,    0,    0,    0,    0,    0, 2196,    0,
 2196, 2196,    0, 2196, 2196, 2196, 2196, 2196,    0,    0,
 2196,    0,    0, 2196,    0, 2196, 2196,    0,    0, 2196,
 2196, 2196,    0, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
    0,    0,    0, 2196, 2196, 2196, 2196, 2196,    0,    0,
    0, 2196,    0,    0,    0,    0,    0,    0,    0, 2196,
 2196, 2196, 2196, 2196, 2196,    0,    0,    0, 2196, 2196,
    0, 2196,    0, 2196,    0, 2196, 2196, 2196,    0, 2196,
    0, 2196, 2196, 2196,    0, 2196, 2196, 2196, 2196, 2196,
    0, 2196, 2196,    0,    0,    0,    0, 2196,    0, 2196,
 2196,    0,    0, 2196, 2196, 2196,    0,    0, 2196,    0,
 2196,    0,    0, 2196, 2196, 2196, 2196,    0, 2196,    0,
 2196,    0, 2196, 2196, 2196, 2196, 2196,    0,    0, 2196,
 2196, 2196,    0, 2196, 2196,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2196, 2196, 2196,    0, 2196, 2196,
 2196,    0,    0,    0,    0,    0, 2196,    0,    0,    0,
 2196,    0, 2196, 2196, 2196, 2196,    0, 2196,    0, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2196, 2196,    0, 2196,
 2196, 2196, 1457,    0,    0,    0, 2196, 2196, 2196, 2196,
 2196,    0, 2196,    0,    0, 2196, 2196,    0,    0,    0,
 2196, 2196,    0,    0, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196,    0, 2196,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196,    0, 2196, 2196, 2196, 2196,
 2196, 2196,    0, 2196, 2196, 2196, 2196,    0, 2196,    0,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,    0, 2196,
 2196, 2196, 2196,    0, 2196, 2196, 2196, 2196, 2196,    0,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,    0,
 2196, 2196,    0,    0,    0, 2196, 2196, 2196, 2196, 2196,
 2196,    0,    0, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196,    0,    0,    0,    0,    0, 2196, 2232, 2196,
 2196, 2196, 2196, 2196, 2196,    0, 2196,    0,    0,    0,
    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0, 2196, 2196,    0, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196,    0, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,    0, 2196,
 2196, 2196,    0, 2196,    0,    0,    0, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
    0, 2196, 2196,    0,    0, 2318,    0,    0, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196,    0,    0, 2196,    0,
 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196, 2196,
 2196, 2196, 2196, 2196, 2196, 2196, 2318, 2318, 2318, 2318,
 2318, 2318, 2318,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2318, 2318, 2318, 2318,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2318,    0, 2318,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0,    0,    0,
    0,    0,    0,    0,    0, 1444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0,    0,    0,    0,    0, 1448,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1451,    0, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0,    0,    0,    0,
    0,    0, 2318,    0, 2318,    0,    0, 2318,    0, 2318,
 2318, 2318,    0, 2318, 2318,    0, 2318,    0,    0,    0,
    0,    0, 2318,    0, 2318, 2318, 2318, 2318,    0, 2318,
    0,    0, 2318,    0,    0, 2318, 1456,    0,    0,    0,
    0,    0, 2318,    0, 2318, 2318,    0, 2318, 2318, 2318,
 2318, 2318,    0,    0, 2318,    0,    0, 2318,    0, 2318,
 2318,    0,    0, 2318, 2318, 2318,    0, 2318, 2318, 2318,
 2318, 2318, 2318, 2318,    0,    0,    0, 2318, 2318, 2318,
 2318, 2318,    0,    0,    0, 2318,    0,    0,    0,    0,
    0,    0,    0, 2318, 2318, 2318, 2318, 2318, 2318,    0,
    0,    0, 2318, 2318,    0, 2318,    0, 2318,    0, 2318,
 2318, 2318,    0, 2318,    0, 2318, 2318, 2318,    0, 2318,
 2318, 2318, 2318, 2318,    0, 2318, 2318,    0,    0,    0,
    0, 2318,    0, 2318, 2318,    0,    0, 2318, 2318, 2318,
    0,    0, 2318,    0, 2318,    0,    0, 2318, 2318, 2318,
 2318,    0, 2318,    0, 2318,    0, 2318, 2318, 2318, 2318,
 2318,    0,    0, 2318, 2318, 2318,    0, 2318, 2318,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2318, 2318,
 2318,    0, 2318, 2318, 2318,    0,    0,    0,    0,    0,
 2318,    0,    0,    0, 2318,    0, 2318, 2318, 2318, 2318,
    0, 2318,    0, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2318, 2318,    0, 2318, 2318, 2318, 1457,    0,    0,    0,
 2318, 2318, 2318, 2318, 2318,    0, 2318,    0,    0, 2318,
 2318,    0,    0,    0, 2318, 2318,    0,    0, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318,    0, 2318,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,    0,
 2318, 2318, 2318, 2318, 2318, 2318,    0, 2318, 2318, 2318,
 2318,    0, 2318,    0, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318,    0, 2318, 2318, 2318, 2318,    0, 2318, 2318,
 2318, 2318, 2318,    0, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318,    0, 2318, 2318,    0,    0,    0, 2318,
 2318, 2318, 2318, 2318, 2318,    0,    0, 2318, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318,    0,    0,    0,    0,
    0, 2318, 2233, 2318, 2318, 2318, 2318, 2318, 2318,    0,
 2318,    0,    0,    0,    0,    0,    0,    0, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0, 2318, 2318,    0,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,    0,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318,    0, 2318, 2318, 2318,    0, 2318,    0,    0,
    0, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318, 2318,    0, 2318, 2318,    0,    0, 2319,
    0,    0, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
    0,    0, 2318,    0, 2318, 2318, 2318, 2318, 2318, 2318,
 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318, 2318,
 2319, 2319, 2319, 2319, 2319, 2319, 2319,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2319, 2319,
 2319, 2319,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2319,    0, 2319,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1443,    0,    0,    0,    0,    0,    0,    0,    0, 1444,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1445,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1446,    0,    0,    0,
    0,    0,    0, 1447,    0,    0,    0,    0,    0,    0,
    0, 1448,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1449,    0,
 1450,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,
    0,    0,    0,    0,    0,    0, 2319,    0, 2319,    0,
    0, 2319,    0, 2319, 2319, 2319,    0, 2319, 2319,    0,
 2319,    0,    0,    0,    0,    0, 2319,    0, 2319, 2319,
 2319, 2319,    0, 2319,    0,    0, 2319,    0,    0, 2319,
 1456,    0,    0,    0,    0,    0, 2319,    0, 2319, 2319,
    0, 2319, 2319, 2319, 2319, 2319,    0,    0, 2319,    0,
    0, 2319,    0, 2319, 2319,    0,    0, 2319, 2319, 2319,
    0, 2319, 2319, 2319, 2319, 2319, 2319, 2319,    0,    0,
    0, 2319, 2319, 2319, 2319, 2319,    0,    0,    0, 2319,
    0,    0,    0,    0,    0,    0,    0, 2319, 2319, 2319,
 2319, 2319, 2319,    0,    0,    0, 2319, 2319,    0, 2319,
    0, 2319,    0, 2319, 2319, 2319,    0, 2319,    0, 2319,
 2319, 2319,    0, 2319, 2319, 2319, 2319, 2319,    0, 2319,
 2319,    0,    0,    0,    0, 2319,    0, 2319, 2319,    0,
    0, 2319, 2319, 2319,    0,    0, 2319,    0, 2319,    0,
    0, 2319, 2319, 2319, 2319,    0, 2319,    0, 2319,    0,
 2319, 2319, 2319, 2319, 2319,    0,    0, 2319, 2319, 2319,
    0, 2319, 2319,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2319, 2319, 2319,    0, 2319, 2319, 2319,    0,
    0,    0,    0,    0, 2319,    0,    0,    0, 2319,    0,
 2319, 2319, 2319, 2319,    0, 2319,    0, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2319, 2319,    0, 2319, 2319, 2319,
 1457,    0,    0,    0, 2319, 2319, 2319, 2319, 2319,    0,
 2319,    0,    0, 2319, 2319,    0,    0,    0, 2319, 2319,
    0,    0, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319,    0, 2319,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319,    0, 2319, 2319, 2319, 2319, 2319, 2319,
    0, 2319, 2319, 2319, 2319,    0, 2319,    0, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319,    0, 2319, 2319, 2319,
 2319,    0, 2319, 2319, 2319, 2319, 2319,    0, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319,    0, 2319, 2319,
    0,    0,    0, 2319, 2319, 2319, 2319, 2319, 2319,    0,
    0, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
    0,    0,    0,    0,    0, 2319, 2234, 2319, 2319, 2319,
 2319, 2319, 2319,    0, 2319,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 2319, 2319,    0, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319,    0, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319,    0, 2319, 2319, 2319,
    0, 2319,    0,    0,    0, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,    0, 2319,
 2319,    0,    0, 2320,    0,    0, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319,    0,    0, 2319,    0, 2319, 2319,
 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319, 2319,
 2319, 2319, 2319, 2319, 2320, 2320, 2320, 2320, 2320, 2320,
 2320,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2320, 2320, 2320, 2320,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2320,    0, 2320,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2320,    0, 2320,    0,    0, 2320,    0, 2320, 2320, 2320,
    0, 2320, 2320,    0, 2320,    0,    0,    0,    0,    0,
 2320,    0, 2320, 2320, 2320, 2320,    0, 2320,    0,    0,
 2320,    0,    0, 2320, 1456,    0,    0,    0,    0,    0,
 2320,    0, 2320, 2320,    0, 2320, 2320, 2320, 2320, 2320,
    0,    0, 2320,    0,    0, 2320,    0, 2320, 2320,    0,
    0, 2320, 2320, 2320,    0, 2320, 2320, 2320, 2320, 2320,
 2320, 2320,    0,    0,    0, 2320, 2320, 2320, 2320, 2320,
    0,    0,    0, 2320,    0,    0,    0,    0,    0,    0,
    0, 2320, 2320, 2320, 2320, 2320, 2320,    0,    0,    0,
 2320, 2320,    0, 2320,    0, 2320,    0, 2320, 2320, 2320,
    0, 2320,    0, 2320, 2320, 2320,    0, 2320, 2320, 2320,
 2320, 2320,    0, 2320, 2320,    0,    0,    0,    0, 2320,
    0, 2320, 2320,    0,    0, 2320, 2320, 2320,    0,    0,
 2320,    0, 2320,    0,    0, 2320, 2320, 2320, 2320,    0,
 2320,    0, 2320,    0, 2320, 2320, 2320, 2320, 2320,    0,
    0, 2320, 2320, 2320,    0, 2320, 2320,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2320, 2320, 2320,    0,
 2320, 2320, 2320,    0,    0,    0,    0,    0, 2320,    0,
    0,    0, 2320,    0, 2320, 2320, 2320, 2320,    0, 2320,
    0, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2320, 2320,
    0, 2320, 2320, 2320, 1457,    0,    0,    0, 2320, 2320,
 2320, 2320, 2320,    0, 2320,    0,    0, 2320, 2320,    0,
    0,    0, 2320, 2320,    0,    0, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320,    0, 2320,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320,    0, 2320, 2320,
 2320, 2320, 2320, 2320,    0, 2320, 2320, 2320, 2320,    0,
 2320,    0, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
    0, 2320, 2320, 2320, 2320,    0, 2320, 2320, 2320, 2320,
 2320,    0, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320,    0, 2320, 2320,    0,    0,    0, 2320, 2320, 2320,
 2320, 2320, 2320,    0,    0, 2320, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320,    0,    0,    0,    0,    0, 2320,
 2235, 2320, 2320, 2320, 2320, 2320, 2320,    0, 2320,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 2320, 2320,    0, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320,    0, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
    0, 2320, 2320, 2320,    0, 2320,    0,    0,    0, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320, 2320,    0, 2320, 2320,    0,    0, 2321,    0,    0,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,    0,    0,
 2320,    0, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320,
 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2320, 2321, 2321,
 2321, 2321, 2321, 2321, 2321,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2321, 2321, 2321, 2321,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2321,    0,
 2321,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 1447,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 2321,    0, 2321,    0,    0, 2321,
    0, 2321, 2321, 2321,    0, 2321, 2321,    0, 2321,    0,
    0,    0,    0,    0, 2321,    0, 2321, 2321, 2321, 2321,
    0, 2321,    0,    0, 2321,    0,    0, 2321, 1456,    0,
    0,    0,    0,    0, 2321,    0, 2321, 2321,    0, 2321,
 2321, 2321, 2321, 2321,    0,    0, 2321,    0,    0, 2321,
    0, 2321, 2321,    0,    0, 2321, 2321, 2321,    0, 2321,
 2321, 2321, 2321, 2321, 2321, 2321,    0,    0,    0, 2321,
 2321, 2321, 2321, 2321,    0,    0,    0, 2321,    0,    0,
    0,    0,    0,    0,    0, 2321, 2321, 2321, 2321, 2321,
 2321,    0,    0,    0, 2321, 2321,    0, 2321,    0, 2321,
    0, 2321, 2321, 2321,    0, 2321,    0, 2321, 2321, 2321,
    0, 2321, 2321, 2321, 2321, 2321,    0, 2321, 2321,    0,
    0,    0,    0, 2321,    0, 2321, 2321,    0,    0, 2321,
 2321, 2321,    0,    0, 2321,    0, 2321,    0,    0, 2321,
 2321, 2321, 2321,    0, 2321,    0, 2321,    0, 2321, 2321,
 2321, 2321, 2321,    0,    0, 2321, 2321, 2321,    0, 2321,
 2321,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2321, 2321, 2321,    0, 2321, 2321, 2321,    0,    0,    0,
    0,    0, 2321,    0,    0,    0, 2321,    0, 2321, 2321,
 2321, 2321,    0, 2321,    0, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2321, 2321,    0, 2321, 2321, 2321, 1457,    0,
    0,    0, 2321, 2321, 2321, 2321, 2321,    0, 2321,    0,
    0, 2321, 2321,    0,    0,    0, 2321, 2321,    0,    0,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,    0,
 2321,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321,    0, 2321, 2321, 2321, 2321, 2321, 2321,    0, 2321,
 2321, 2321, 2321,    0, 2321,    0, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321,    0, 2321, 2321, 2321, 2321,    0,
 2321, 2321, 2321, 2321, 2321,    0, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321,    0, 2321, 2321,    0,    0,
    0, 2321, 2321, 2321, 2321, 2321, 2321,    0,    0, 2321,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,    0,    0,
    0,    0,    0, 2321, 2644, 2321, 2321, 2321, 2321, 2321,
 2321,    0, 2321,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0,    0,    0,    0,    0, 2321,
 2321,    0, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321,    0, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321,    0, 2321, 2321, 2321,    0, 2321,
    0,    0,    0, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321, 2321,    0, 2321, 2321,    0,
    0, 2322,    0,    0, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321,    0,    0, 2321,    0, 2321, 2321, 2321, 2321,
 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321, 2321,
 2321, 2321, 2322, 2322, 2322, 2322, 2322, 2322, 2322,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2322, 2322, 2322, 2322,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2322,    0, 2322,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1446,    0,
    0,    0,    0,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1449,    0, 1450,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0,    0,    0,    0, 2322,    0,
 2322,    0,    0, 2322,    0, 2322, 2322, 2322,    0, 2322,
 2322,    0, 2322,    0,    0,    0,    0,    0, 2322,    0,
 2322, 2322, 2322, 2322,    0, 2322,    0,    0, 2322,    0,
    0, 2322, 1456,    0,    0,    0,    0,    0, 2322,    0,
 2322, 2322,    0, 2322, 2322, 2322, 2322, 2322,    0,    0,
 2322,    0,    0, 2322,    0, 2322, 2322,    0,    0, 2322,
 2322, 2322,    0, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
    0,    0,    0, 2322, 2322, 2322, 2322, 2322,    0,    0,
    0, 2322,    0,    0,    0,    0,    0,    0,    0, 2322,
 2322, 2322, 2322, 2322, 2322,    0,    0,    0, 2322, 2322,
    0, 2322,    0, 2322,    0, 2322, 2322, 2322,    0, 2322,
    0, 2322, 2322, 2322,    0, 2322, 2322, 2322, 2322, 2322,
    0, 2322, 2322,    0,    0,    0,    0, 2322,    0, 2322,
 2322,    0,    0, 2322, 2322, 2322,    0,    0, 2322,    0,
 2322,    0,    0, 2322, 2322, 2322, 2322,    0, 2322,    0,
 2322,    0, 2322, 2322, 2322, 2322, 2322,    0,    0, 2322,
 2322, 2322,    0, 2322, 2322,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2322, 2322, 2322,    0, 2322, 2322,
 2322,    0,    0,    0,    0,    0, 2322,    0,    0,    0,
 2322,    0, 2322, 2322, 2322, 2322,    0, 2322,    0, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2322, 2322,    0, 2322,
 2322, 2322, 1457,    0,    0,    0, 2322, 2322, 2322, 2322,
 2322,    0, 2322,    0,    0, 2322, 2322,    0,    0,    0,
 2322, 2322,    0,    0, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322,    0, 2322,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322,    0, 2322, 2322, 2322, 2322,
 2322, 2322,    0, 2322, 2322, 2322, 2322,    0, 2322,    0,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,    0, 2322,
 2322, 2322, 2322,    0, 2322, 2322, 2322, 2322, 2322,    0,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,    0,
 2322, 2322,    0,    0,    0, 2322, 2322, 2322, 2322, 2322,
 2322,    0,    0, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322,    0,    0,    0,    0,    0, 2322, 2981, 2322,
 2322, 2322, 2322, 2322, 2322,    0, 2322,    0,    0,    0,
    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0, 2322, 2322,    0, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322,    0, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,    0, 2322,
 2322, 2322,    0, 2322,    0,    0,    0, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
    0, 2322, 2322,    0,    0, 2323,    0,    0, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322,    0,    0, 2322,    0,
 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322, 2322,
 2322, 2322, 2322, 2322, 2322, 2322, 2323, 2323, 2323, 2323,
 2323, 2323, 2323,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2323, 2323, 2323, 2323,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2323,    0, 2323,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0,    0,    0,
    0,    0,    0,    0,    0, 1444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0,    0,    0,    0,    0, 1448,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1451,    0, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0,    0,    0,    0,
    0,    0, 2323,    0, 2323,    0,    0, 2323,    0, 2323,
 2323, 2323,    0, 2323, 2323,    0, 2323,    0,    0,    0,
    0,    0, 2323,    0, 2323, 2323, 2323, 2323,    0, 2323,
    0,    0, 2323,    0,    0, 2323, 1456,    0,    0,    0,
    0,    0, 2323,    0, 2323, 2323,    0, 2323, 2323, 2323,
 2323, 2323,    0,    0, 2323,    0,    0, 2323,    0, 2323,
 2323,    0,    0, 2323, 2323, 2323,    0, 2323, 2323, 2323,
 2323, 2323, 2323, 2323,    0,    0,    0, 2323, 2323, 2323,
 2323, 2323,    0,    0,    0, 2323,    0,    0,    0,    0,
    0,    0,    0, 2323, 2323, 2323, 2323, 2323, 2323,    0,
    0,    0, 2323, 2323,    0, 2323,    0, 2323,    0, 2323,
 2323, 2323,    0, 2323,    0, 2323, 2323, 2323,    0, 2323,
 2323, 2323, 2323, 2323,    0, 2323, 2323,    0,    0,    0,
    0, 2323,    0, 2323, 2323,    0,    0, 2323, 2323, 2323,
    0,    0, 2323,    0, 2323,    0,    0, 2323, 2323, 2323,
 2323,    0, 2323,    0, 2323,    0, 2323, 2323, 2323, 2323,
 2323,    0,    0, 2323, 2323, 2323,    0, 2323, 2323,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2323, 2323,
 2323,    0, 2323, 2323, 2323,    0,    0,    0,    0,    0,
 2323,    0,    0,    0, 2323,    0, 2323, 2323, 2323, 2323,
    0, 2323,    0, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2323, 2323,    0, 2323, 2323, 2323, 1457,    0,    0,    0,
 2323, 2323, 2323, 2323, 2323,    0, 2323,    0,    0, 2323,
 2323,    0,    0,    0, 2323, 2323,    0,    0, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323,    0, 2323,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,    0,
 2323, 2323, 2323, 2323, 2323, 2323,    0, 2323, 2323, 2323,
 2323,    0, 2323,    0, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323,    0, 2323, 2323, 2323, 2323,    0, 2323, 2323,
 2323, 2323, 2323,    0, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323,    0, 2323, 2323,    0,    0,    0, 2323,
 2323, 2323, 2323, 2323, 2323,    0,    0, 2323, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323,    0,    0,    0,    0,
    0, 2323, 2986, 2323, 2323, 2323, 2323, 2323, 2323,    0,
 2323,    0,    0,    0,    0,    0,    0,    0, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0, 2323, 2323,    0,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,    0,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323,    0, 2323, 2323, 2323,    0, 2323,    0,    0,
    0, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323, 2323,    0, 2323, 2323,    0,    0, 2324,
    0,    0, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
    0,    0, 2323,    0, 2323, 2323, 2323, 2323, 2323, 2323,
 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323, 2323,
 2324, 2324, 2324, 2324, 2324, 2324, 2324,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2324, 2324,
 2324, 2324,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2324,    0, 2324,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1443,    0,    0,    0,    0,    0,    0,    0,    0, 1444,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1445,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1446,    0,    0,    0,
    0,    0,    0, 1447,    0,    0,    0,    0,    0,    0,
    0, 1448,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1449,    0,
 1450,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,
    0,    0,    0,    0,    0,    0, 2324,    0, 2324,    0,
    0, 2324,    0, 2324, 2324, 2324,    0, 2324, 2324,    0,
 2324,    0,    0,    0,    0,    0, 2324,    0, 2324, 2324,
 2324, 2324,    0, 2324,    0,    0, 2324,    0,    0, 2324,
 1456,    0,    0,    0,    0,    0, 2324,    0, 2324, 2324,
    0, 2324, 2324, 2324, 2324, 2324,    0,    0, 2324,    0,
    0, 2324,    0, 2324, 2324,    0,    0, 2324, 2324, 2324,
    0, 2324, 2324, 2324, 2324, 2324, 2324, 2324,    0,    0,
    0, 2324, 2324, 2324, 2324, 2324,    0,    0,    0, 2324,
    0,    0,    0,    0,    0,    0,    0, 2324, 2324, 2324,
 2324, 2324, 2324,    0,    0,    0, 2324, 2324,    0, 2324,
    0, 2324,    0, 2324, 2324, 2324,    0, 2324,    0, 2324,
 2324, 2324,    0, 2324, 2324, 2324, 2324, 2324,    0, 2324,
 2324,    0,    0,    0,    0, 2324,    0, 2324, 2324,    0,
    0, 2324, 2324, 2324,    0,    0, 2324,    0, 2324,    0,
    0, 2324, 2324, 2324, 2324,    0, 2324,    0, 2324,    0,
 2324, 2324, 2324, 2324, 2324,    0,    0, 2324, 2324, 2324,
    0, 2324, 2324,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2324, 2324, 2324,    0, 2324, 2324, 2324,    0,
    0,    0,    0,    0, 2324,    0,    0,    0, 2324,    0,
 2324, 2324, 2324, 2324,    0, 2324,    0, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2324, 2324,    0, 2324, 2324, 2324,
 1457,    0,    0,    0, 2324, 2324, 2324, 2324, 2324,    0,
 2324,    0,    0, 2324, 2324,    0,    0,    0, 2324, 2324,
    0,    0, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324,    0, 2324,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324,    0, 2324, 2324, 2324, 2324, 2324, 2324,
    0, 2324, 2324, 2324, 2324,    0, 2324,    0, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324,    0, 2324, 2324, 2324,
 2324,    0, 2324, 2324, 2324, 2324, 2324,    0, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324,    0, 2324, 2324,
    0,    0,    0, 2324, 2324, 2324, 2324, 2324, 2324,    0,
    0, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
    0,    0,    0,    0,    0, 2324, 2988, 2324, 2324, 2324,
 2324, 2324, 2324,    0, 2324,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 2324, 2324,    0, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324,    0, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324,    0, 2324, 2324, 2324,
    0, 2324,    0,    0,    0, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,    0, 2324,
 2324,    0,    0, 2325,    0,    0, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324,    0,    0, 2324,    0, 2324, 2324,
 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324, 2324,
 2324, 2324, 2324, 2324, 2325, 2325, 2325, 2325, 2325, 2325,
 2325,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2325, 2325, 2325, 2325,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2325,    0, 2325,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 2325,    0, 2325,    0,    0, 2325,    0, 2325, 2325, 2325,
    0, 2325, 2325,    0, 2325,    0,    0,    0,    0,    0,
 2325,    0, 2325, 2325, 2325, 2325,    0, 2325,    0,    0,
 2325,    0,    0, 2325, 1456,    0,    0,    0,    0,    0,
 2325,    0, 2325, 2325,    0, 2325, 2325, 2325, 2325, 2325,
    0,    0, 2325,    0,    0, 2325,    0, 2325, 2325,    0,
    0, 2325, 2325, 2325,    0, 2325, 2325, 2325, 2325, 2325,
 2325, 2325,    0,    0,    0, 2325, 2325, 2325, 2325, 2325,
    0,    0,    0, 2325,    0,    0,    0,    0,    0,    0,
    0, 2325, 2325, 2325, 2325, 2325, 2325,    0,    0,    0,
 2325, 2325,    0, 2325,    0, 2325,    0, 2325, 2325, 2325,
    0, 2325,    0, 2325, 2325, 2325,    0, 2325, 2325, 2325,
 2325, 2325,    0, 2325, 2325,    0,    0,    0,    0, 2325,
    0, 2325, 2325,    0,    0, 2325, 2325, 2325,    0,    0,
 2325,    0, 2325,    0,    0, 2325, 2325, 2325, 2325,    0,
 2325,    0, 2325,    0, 2325, 2325, 2325, 2325, 2325,    0,
    0, 2325, 2325, 2325,    0, 2325, 2325,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2325, 2325, 2325,    0,
 2325, 2325, 2325,    0,    0,    0,    0,    0, 2325,    0,
    0,    0, 2325,    0, 2325, 2325, 2325, 2325,    0, 2325,
    0, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2325, 2325,
    0, 2325, 2325, 2325, 1457,    0,    0,    0, 2325, 2325,
 2325, 2325, 2325,    0, 2325,    0,    0, 2325, 2325,    0,
    0,    0, 2325, 2325,    0,    0, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325,    0, 2325,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325,    0, 2325, 2325,
 2325, 2325, 2325, 2325,    0, 2325, 2325, 2325, 2325,    0,
 2325,    0, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
    0, 2325, 2325, 2325, 2325,    0, 2325, 2325, 2325, 2325,
 2325,    0, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325,    0, 2325, 2325,    0,    0,    0, 2325, 2325, 2325,
 2325, 2325, 2325,    0,    0, 2325, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325,    0,    0,    0,    0,    0, 2325,
 2989, 2325, 2325, 2325, 2325, 2325, 2325,    0, 2325,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 2325, 2325,    0, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325,    0, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
    0, 2325, 2325, 2325,    0, 2325,    0,    0,    0, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325, 2325,    0, 2325, 2325,    0,    0, 2345,    0,    0,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,    0,    0,
 2325,    0, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325,
 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2325, 2345, 2345,
 2345, 2345, 2345, 2345, 2345,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2345, 2345, 2345, 2345,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2345,    0,
 2345,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 1447,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 2345,    0, 2345,    0,    0, 2345,
    0, 2345, 2345, 2345,    0, 2345, 2345,    0, 2345,    0,
    0,    0,    0,    0, 2345,    0, 2345, 2345, 2345, 2345,
    0, 2345,    0,    0, 2345,    0,    0, 2345, 1456,    0,
    0,    0,    0,    0, 2345,    0, 2345, 2345,    0, 2345,
 2345, 2345, 2345, 2345,    0,    0, 2345,    0,    0, 2345,
    0, 2345, 2345,    0,    0, 2345, 2345, 2345,    0, 2345,
 2345, 2345, 2345, 2345, 2345, 2345,    0,    0,    0, 2345,
 2345, 2345, 2345, 2345,    0,    0,    0, 2345,    0,    0,
    0,    0,    0,    0,    0, 2345, 2345, 2345, 2345, 2345,
 2345,    0,    0,    0, 2345, 2345,    0, 2345,    0, 2345,
    0, 2345, 2345, 2345,    0, 2345,    0, 2345, 2345, 2345,
    0, 2345, 2345, 2345, 2345, 2345,    0, 2345, 2345,    0,
    0,    0,    0, 2345,    0, 2345, 2345,    0,    0, 2345,
 2345, 2345,    0,    0, 2345,    0, 2345,    0,    0, 2345,
 2345, 2345, 2345,    0, 2345,    0, 2345,    0, 2345, 2345,
 2345, 2345, 2345,    0,    0, 2345, 2345, 2345,    0, 2345,
 2345,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2345, 2345, 2345,    0, 2345, 2345, 2345,    0,    0,    0,
    0,    0, 2345,    0,    0,    0, 2345,    0, 2345, 2345,
 2345, 2345,    0, 2345,    0, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2345, 2345,    0, 2345, 2345, 2345, 1457,    0,
    0,    0, 2345, 2345, 2345, 2345, 2345,    0, 2345,    0,
    0, 2345, 2345,    0,    0,    0, 2345, 2345,    0,    0,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,    0,
 2345,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345,    0, 2345, 2345, 2345, 2345, 2345, 2345,    0, 2345,
 2345, 2345, 2345,    0, 2345,    0, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345,    0, 2345, 2345, 2345, 2345,    0,
 2345, 2345, 2345, 2345, 2345,    0, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345,    0, 2345, 2345,    0,    0,
    0, 2345, 2345, 2345, 2345, 2345, 2345,    0,    0, 2345,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,    0,    0,
    0,    0,    0, 2345, 1714, 2345, 2345, 2345, 2345, 2345,
 2345,    0, 2345,    0,    0,    0,    0,    0,    0,    0,
 1714, 1714, 1714,    0,    0,    0,    0,    0,    0, 2345,
 2345,    0, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345,    0, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345,    0, 2345, 2345, 2345,    0, 2345,
    0,    0,    0, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345, 2345,    0, 2345, 2345,    0,
    0, 2369,    0,    0, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345,    0,    0, 2345,    0, 2345, 2345, 2345, 2345,
 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345,
 2345, 2345, 2369, 2369, 2369, 2369, 2369, 2369, 2369,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2369, 2369, 2369, 2369,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2369,    0, 2369,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1714,    0,    0,    0,    0,    0,    0,    0,
    0, 1714,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1714,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1714,    0,
    0,    0,    0,    0,    0, 1714,    0,    0,    0,    0,
    0,    0,    0, 1714,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1714,    0, 1714,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1714,    0, 1714, 1714, 1714,    0,    0,    0,
    0, 1714,    0,    0,    0,    0,    0,    0, 2369,    0,
 2369,    0,    0, 2369,    0, 2369, 2369, 2369,    0, 2369,
 2369,    0, 2369,    0,    0,    0,    0,    0, 2369,    0,
 2369, 2369, 2369, 2369,    0, 2369,    0,    0, 2369,    0,
    0, 2369, 1714,    0,    0,    0,    0,    0, 2369,    0,
 2369, 2369,    0, 2369, 2369, 2369, 2369, 2369,    0,    0,
 2369,    0,    0, 2369,    0, 2369, 2369,    0,    0, 2369,
 2369, 2369,    0, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
    0,    0,    0, 2369, 2369, 2369, 2369, 2369,    0,    0,
    0, 2369,    0,    0,    0,    0,    0,    0,    0, 2369,
 2369, 2369, 2369, 2369, 2369,    0,    0,    0, 2369, 2369,
    0, 2369,    0, 2369,    0, 2369, 2369, 2369,    0, 2369,
    0, 2369, 2369, 2369,    0, 2369, 2369, 2369, 2369, 2369,
    0, 2369, 2369,    0,    0,    0,    0, 2369,    0, 2369,
 2369,    0,    0, 2369, 2369, 2369,    0,    0, 2369,    0,
 2369,    0,    0, 2369, 2369, 2369, 2369,    0, 2369,    0,
 2369,    0, 2369, 2369, 2369, 2369, 2369,    0,    0, 2369,
 2369, 2369,    0, 2369, 2369,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2369, 2369, 2369,    0, 2369, 2369,
 2369,    0,    0,    0,    0,    0, 2369,    0,    0,    0,
 2369,    0, 2369, 2369, 2369, 2369,    0, 2369,    0, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2369, 2369,    0, 2369,
 2369, 2369, 1714,    0,    0,    0, 2369, 2369, 2369, 2369,
 2369,    0, 2369,    0,    0, 2369, 2369,    0,    0,    0,
 2369, 2369,    0,    0, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369,    0, 2369,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369,    0, 2369, 2369, 2369, 2369,
 2369, 2369,    0, 2369, 2369, 2369, 2369,    0, 2369,    0,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,    0, 2369,
 2369, 2369, 2369,    0, 2369, 2369, 2369, 2369, 2369,    0,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,    0,
 2369, 2369,    0,    0,    0, 2369, 2369, 2369, 2369, 2369,
 2369,    0,    0, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369,    0,    0,    0,    0,    0, 2369,    0, 2369,
 2369, 2369, 2369, 2369, 2369,    0, 2369,    0,    0,    0,
    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0, 2369, 2369,    0, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369,    0, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,    0, 2369,
 2369, 2369,    0, 2369,    0,    0,    0, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
    0, 2369, 2369,    0,    0, 2374,    0,    0, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369,    0,    0, 2369,    0,
 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369, 2369,
 2369, 2369, 2369, 2369, 2369, 2369, 2374, 2374, 2374, 2374,
 2374, 2374, 2374,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2374, 2374, 2374, 2374,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2374,    0, 2374,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0,    0,    0,
    0,    0,    0,    0,    0, 1444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2637,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0,    0,    0,    0,    0, 1448,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1451,    0, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0,    0,    0,    0,
    0,    0, 2374,    0, 2374,    0,    0, 2374,    0, 2374,
 2374, 2374,    0, 2374, 2374,    0, 2374,    0,    0,    0,
    0,    0, 2374,    0, 2374, 2374, 2374, 2374,    0, 2374,
    0,    0, 2374,    0,    0, 2374, 1456,    0,    0,    0,
    0,    0, 2374,    0, 2374, 2374,    0, 2374, 2374, 2374,
 2374, 2374,    0,    0, 2374,    0,    0, 2374,    0, 2374,
 2374,    0,    0, 2374, 2374, 2374,    0, 2374, 2374, 2374,
 2374, 2374, 2374, 2374,    0,    0,    0, 2374, 2374, 2374,
 2374, 2374,    0,    0,    0, 2374,    0,    0,    0,    0,
    0,    0,    0, 2374, 2374, 2374, 2374, 2374, 2374,    0,
    0,    0, 2374, 2374,    0, 2374,    0, 2374,    0, 2374,
 2374, 2374,    0, 2374,    0, 2374, 2374, 2374,    0, 2374,
 2374, 2374, 2374, 2374,    0, 2374, 2374,    0,    0,    0,
    0, 2374,    0, 2374, 2374,    0,    0, 2374, 2374, 2374,
    0,    0, 2374,    0, 2374,    0,    0, 2374, 2374, 2374,
 2374,    0, 2374,    0, 2374,    0, 2374, 2374, 2374, 2374,
 2374,    0,    0, 2374, 2374, 2374,    0, 2374, 2374,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2374, 2374,
 2374,    0, 2374, 2374, 2374,    0,    0,    0,    0,    0,
 2374,    0,    0,    0, 2374,    0, 2374, 2374, 2374, 2374,
    0, 2374,    0, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2374, 2374,    0, 2374, 2374, 2374, 1457,    0,    0,    0,
 2374, 2374, 2374, 2374, 2374,    0, 2374,    0,    0, 2374,
 2374,    0,    0,    0, 2374, 2374,    0,    0, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374,    0, 2374,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,    0,
 2374, 2374, 2374, 2374, 2374, 2374,    0, 2374, 2374, 2374,
 2374,    0, 2374,    0, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374,    0, 2374, 2374, 2374, 2374,    0, 2374, 2374,
 2374, 2374, 2374,    0, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374,    0, 2374, 2374,    0,    0,    0, 2374,
 2374, 2374, 2374, 2374, 2374,    0,    0, 2374, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374,    0,    0,    0,    0,
    0, 2374,    0, 2374, 2374, 2374, 2374, 2374, 2374,    0,
 2374,    0,    0,    0,    0,    0,    0,    0, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0, 2374, 2374,    0,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,    0,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374,    0, 2374, 2374, 2374,    0, 2374,    0,    0,
    0, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374, 2374,    0, 2374, 2374,    0,    0, 1819,
    0,    0, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
    0,    0, 2374,    0, 2374, 2374, 2374, 2374, 2374, 2374,
 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374, 2374,
 1819, 1819, 1819, 1819, 1819,    0, 1819,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1819, 1819,
 1819, 1819,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1819,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1443,    0,    0,    0,    0,    0,    0,    0,    0, 1444,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1445,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2973,    0,    0,    0,    0,    0, 1446,    0,    0,    0,
    0,    0,    0, 1447,    0,    0,    0,    0,    0,    0,
    0, 1448,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1449,    0,
 1450,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,
    0,    0,    0,    0,    0,    0, 1819,    0, 1819,    0,
    0, 1819,    0, 1819, 1819, 1819,    0, 1819, 1819,    0,
 1819,    0,    0,    0,    0,    0, 1819,    0, 1819, 1819,
 1819, 1819,    0, 1819,    0,    0, 1819,    0,    0, 1819,
 1456,    0,    0,    0,    0,    0, 1819,    0, 1819, 1819,
    0, 1819, 1819, 1819, 1819, 1819,    0,    0, 1819,    0,
    0, 1819,    0, 1819, 1819,    0,    0, 1819, 1819, 1819,
    0, 1819, 1819, 1819, 1819, 1819, 1819, 1819,    0,    0,
    0, 1819, 1819, 1819, 1819, 1819,    0,    0,    0, 1819,
    0,    0,    0,    0,    0,    0,    0, 1819, 1819, 1819,
 1819, 1819, 1819,    0,    0,    0, 1819, 1819,    0, 1819,
    0, 1819,    0, 1819, 1819, 1819,    0, 1819,    0, 1819,
 1819, 1819,    0, 1819, 1819, 1819, 1819, 1819,    0, 1819,
 1819,    0,    0,    0,    0, 1819,    0, 1819, 1819,    0,
    0, 1819, 1819, 1819,    0,    0, 1819,    0, 1819,    0,
    0, 1819, 1819, 1819, 1819,    0, 1819,    0, 1819,    0,
 1819, 1819, 1819, 1819, 1819,    0,    0, 1819, 1819, 1819,
    0, 1819, 1819,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1819, 1819, 1819,    0, 1819, 1819, 1819,    0,
    0,    0,    0,    0, 1819,    0,    0,    0, 1819,    0,
 1819, 1819, 1819, 1819,    0, 1819,    0, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1819, 1819,    0, 1819, 1819, 1819,
 1457,    0,    0,    0, 1819, 1819, 1819, 1819, 1819,    0,
 1819,    0,    0, 1819, 1819,    0,    0,    0, 1819, 1819,
    0,    0, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819,    0, 1819,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819,    0, 1819, 1819, 1819, 1819, 1819, 1819,
    0, 1819, 1819, 1819, 1819,    0, 1819,    0, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819,    0, 1819, 1819, 1819,
 1819,    0, 1819, 1819, 1819, 1819, 1819,    0, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,    0, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1877, 1819, 1819,
    0,    0,    0, 1819, 1819, 1819, 1819, 1819, 1819,    0,
    0, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
    0,    0,    0,    0,    0, 1819,    0, 1819, 1819, 1819,
 1819, 1819, 1819,    0, 1819,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 1819, 1819,    0, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819,    0, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819,    0, 1819, 1819, 1819,
    0, 1819,    0,    0,    0, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,    0, 1819,
 1819,    0,    0, 1825,    0,    0, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819,    0,    0, 1819,    0, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1825, 1825, 1825, 1825, 1825,    0,
 1825,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1825, 1825, 1825, 1825,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1825,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2975,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 1825,    0, 1825,    0,    0, 1825,    0, 1825, 1825, 1825,
    0, 1825, 1825,    0, 1825,    0,    0,    0,    0,    0,
 1825,    0, 1825, 1825, 1825, 1825,    0, 1825,    0,    0,
 1825,    0,    0, 1825, 1456,    0,    0,    0,    0,    0,
 1825,    0, 1825, 1825,    0, 1825, 1825, 1825, 1825, 1825,
    0,    0, 1825,    0,    0, 1825,    0, 1825, 1825,    0,
    0, 1825, 1825, 1825,    0, 1825, 1825, 1825, 1825, 1825,
 1825, 1825,    0,    0,    0, 1825, 1825, 1825, 1825, 1825,
    0,    0,    0, 1825,    0,    0,    0,    0,    0,    0,
    0, 1825, 1825, 1825, 1825, 1825, 1825,    0,    0,    0,
 1825, 1825,    0, 1825,    0, 1825,    0, 1825, 1825, 1825,
    0, 1825,    0, 1825, 1825, 1825,    0, 1825, 1825, 1825,
 1825, 1825,    0, 1825, 1825,    0,    0,    0,    0, 1825,
    0, 1825, 1825,    0,    0, 1825, 1825, 1825,    0,    0,
 1825,    0, 1825,    0,    0, 1825, 1825, 1825, 1825,    0,
 1825,    0, 1825,    0, 1825, 1825, 1825, 1825, 1825,    0,
    0, 1825, 1825, 1825,    0, 1825, 1825,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1825, 1825, 1825,    0,
 1825, 1825, 1825,    0,    0,    0,    0,    0, 1825,    0,
    0,    0, 1825,    0, 1825, 1825, 1825, 1825,    0, 1825,
    0, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1825, 1825,
    0, 1825, 1825, 1825, 1457,    0,    0,    0, 1825, 1825,
 1825, 1825, 1825,    0, 1825,    0,    0, 1825, 1825,    0,
    0,    0, 1825, 1825,    0,    0, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825,    0, 1825,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0, 1825, 1825,
 1825, 1825, 1825, 1825,    0, 1825, 1825, 1825, 1825,    0,
 1825,    0, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
    0, 1825, 1825, 1825, 1825,    0, 1825, 1825, 1825, 1825,
 1825,    0, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825,    0, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825,    0,    0,    0, 1825, 1825, 1825,
 1825, 1825, 1825,    0,    0, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825,    0,    0,    0,    0,    0, 1825,
    0, 1825, 1825, 1825, 1825, 1825, 1825,    0, 1825,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 1825, 1825,    0, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
    0, 1825, 1825, 1825,    0, 1825,    0,    0,    0, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825,    0, 1825, 1825,    0,    0, 2110,    0,    0,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0,    0,
 1825,    0, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 2110, 2110,
 2110, 2110, 2110,    0, 2110,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2110, 2110, 2110, 2110,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2110,    0,
 2110,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 1447,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 2110,    0, 2110,    0,    0, 2110,
    0, 2110, 2110, 2110,    0, 2110, 2110,    0, 2110,    0,
    0,    0,    0,    0, 2110,    0, 2110, 2110, 2110, 2110,
    0, 2110,    0,    0, 2110,    0,    0, 2110, 1456,    0,
    0,    0,    0,    0, 2110, 2987, 2110, 2110,    0, 2110,
 2110, 2110, 2110, 2110,    0,    0, 2110,    0,    0, 2110,
    0, 2110, 2110,    0,    0, 2110, 2110, 2110,    0, 2110,
 2110, 2110, 2110, 2110, 2110, 2110,    0,    0,    0, 2110,
 2110, 2110, 2110, 2110,    0,    0,    0, 2110,    0,    0,
    0,    0,    0,    0,    0, 2110, 2110, 2110, 2110, 2110,
 2110,    0,    0,    0, 2110, 2110,    0, 2110,    0, 2110,
    0, 2110, 2110, 2110,    0, 2110,    0, 2110, 2110, 2110,
    0, 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110,    0,
    0,    0,    0, 2110,    0, 2110, 2110,    0,    0, 2110,
 2110, 2110,    0,    0, 2110,    0, 2110,    0,    0, 2110,
 2110, 2110, 2110,    0, 2110,    0, 2110,    0, 2110, 2110,
 2110, 2110, 2110,    0,    0, 2110, 2110, 2110,    0, 2110,
 2110,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2110, 2110, 2110,    0, 2110, 2110, 2110,    0,    0,    0,
    0,    0, 2110,    0,    0,    0, 2110,    0, 2110, 2110,
 2110, 2110,    0, 2110,    0, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2110, 2110,    0, 2110, 2110, 2110, 1457,    0,
    0,    0, 2110, 2110, 2110, 2110, 2110,    0, 2110,    0,
    0, 2110, 2110,    0,    0,    0, 2110, 2110,    0,    0,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,    0,
 2110,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110,    0, 2110, 2110, 2110, 2110, 2110, 2110,    0, 2110,
 2110, 2110, 2110,    0, 2110,    0, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110,    0, 2110, 2110, 2110, 2110,    0,
 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110,    0,    0,
    0, 2110, 2110, 2110, 2110, 2110, 2110,    0,    0, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,    0,    0,
    0,    0,    0, 2110,    0, 2110, 2110, 2110, 2110, 2110,
 2110,    0, 2110,    0,    0,    0,    0,    0,    0,    0,
 1459, 1458, 1460,    0,    0,    0,    0,    0,    0, 2110,
 2110,    0, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110,    0, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110,    0, 2110, 2110, 2110,    0, 2110,
    0,    0,    0, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110,    0,
    0, 2128,    0,    0, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110,    0,    0, 2110,    0, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2128, 2128, 2128, 2128, 2128, 2124, 2128,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2128, 2128, 2128, 2128,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2128,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1443,    0,    0,    0,    0,    0,    0,    0,
    0, 1444,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2995,    0,    0,    0, 1446,    0,
    0,    0,    0,    0,    0, 1447,    0,    0,    0,    0,
    0,    0,    0, 1448,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1449,    0, 1450,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1451,    0, 1452, 1453, 1454,    0,    0,    0,
    0, 1455,    0,    0,    0,    0,    0,    0, 2128,    0,
 2128,    0,    0, 2128,    0, 2128, 2128, 2128,    0, 2128,
 2128,    0, 2128,    0,    0,    0,    0,    0, 2128,    0,
 2128, 2128, 2128, 2128,    0, 2128,    0,    0, 2128,    0,
    0, 2128, 1456,    0,    0,    0,    0,    0, 2128,    0,
 2128, 2128,    0, 2128, 2128, 2128, 2128, 2128,    0,    0,
 2128,    0,    0, 2128,    0, 2128, 2128,    0,    0, 2128,
 2128, 2128,    0, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
    0,    0,    0, 2128, 2128, 2128, 2128, 2128,    0,    0,
    0, 2128,    0,    0,    0,    0,    0,    0,    0, 2128,
 2128, 2128, 2128, 2128, 2128,    0,    0,    0, 2128, 2128,
    0, 2128,    0, 2128,    0, 2128, 2128, 2128,    0, 2128,
    0, 2128, 2128, 2128,    0, 2128, 2128, 2128, 2128, 2128,
    0, 2128, 2128,    0,    0,    0,    0, 2128,    0, 2128,
 2128,    0,    0, 2128, 2128, 2128,    0,    0, 2128,    0,
 2128,    0,    0, 2128, 2128, 2128, 2128,    0, 2128,    0,
 2128,    0, 2128, 2128, 2128, 2128, 2128,    0,    0, 2128,
 2128, 2128,    0, 2128, 2128,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2128, 2128, 2128,    0, 2128, 2128,
 2128,    0,    0,    0,    0,    0, 2128,    0,    0,    0,
 2128,    0, 2128, 2128, 2128, 2128,    0, 2128,    0, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 2128,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2128, 2128,    0, 2128,
 2128, 2128, 1457,    0,    0,    0, 2128, 2128, 2128, 2128,
 2128,    0, 2128,    0,    0, 2128, 2128,    0,    0,    0,
 2128, 2128,    0,    0, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128, 2128,    0, 2128,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128,    0, 2128, 2128, 2128, 2128,
 2128, 2128,    0, 2128, 2128, 2128, 2128,    0, 2128,    0,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,    0, 2128,
 2128, 2128, 2128,    0, 2128, 2128, 2128, 2128, 2128,    0,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
    0, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,    0,
 2128, 2128,    0,    0,    0, 2128, 2128, 2128, 2128, 2128,
 2128,    0,    0, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128,    0,    0,    0,    0,    0, 2128,    0, 2128,
 2128, 2128, 2128, 2128, 2128,    0, 2128,    0,    0,    0,
    0,    0,    0,    0, 1459, 1458, 1460,    0,    0,    0,
    0,    0,    0, 2128, 2128,    0, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128,    0, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,    0, 2128,
 2128, 2128,    0, 2128,    0,    0,    0, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
    0, 2128, 2128,    0,    0, 1722,    0,    0, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128,    0,    0, 2128,    0,
 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128, 2128,
 2128, 2128, 2128, 2128, 2128, 2128, 1722, 1722, 1722, 1722,
 1722,    0, 1722,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1722, 1722, 1722, 1722,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1722,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1443,    0,    0,    0,
    0,    0,    0,    0,    0, 1444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1445,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1446,    0,    0,    0,    0,    0,    0, 1447,
    0,    0,    0,    0,    0,    0,    0, 1448,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1449,    0, 1450,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1451,    0, 1452, 1453,
 1454,    0,    0,    0,    0, 1455,    0,    0,    0,    0,
    0,    0, 1722,    0, 1722,    0,    0, 1722,    0, 1722,
 1722, 1722,    0, 1722, 1722,    0, 1722,    0,    0,    0,
    0,    0, 1722,    0, 1722, 1722, 1722, 1722,    0, 1722,
    0,    0, 1722,    0,    0, 1722, 1456,    0,    0,    0,
    0,    0, 1722,    0, 1722, 1722,    0, 1722, 1722, 1722,
 1722, 1722,    0,    0, 1722,    0,    0, 1722,    0, 1722,
 1722,    0,    0, 1722, 1722, 1722,    0, 1722, 1722, 1722,
 1722, 1722, 1722, 1722,    0,    0,    0, 1722, 1722, 1722,
 1722, 1722,    0,    0,    0, 1722,    0,    0,    0,    0,
    0,    0,    0, 1722, 1722, 1722, 1722, 1722, 1722,    0,
    0,    0, 1722, 1722,    0, 1722,    0, 1722,    0, 1722,
 1722, 1722,    0, 1722,    0, 1722, 1722, 1722,    0, 1722,
 1722, 1722, 1722, 1722,    0, 1722, 1722,    0,    0,    0,
    0, 1722,    0, 1722, 1722,    0,    0, 1722, 1722, 1722,
    0,    0, 1722,    0, 1722,    0,    0, 1722, 1722, 1722,
 1722,    0, 1722,    0, 1722,    0, 1722, 1722, 1722, 1722,
 1722,    0,    0, 1722, 1722, 1722,    0, 1722, 1722,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1722, 1722,
 1722, 3015, 1722, 1722, 1722,    0,    0,    0,    0,    0,
 1722,    0,    0,    0, 1722,    0, 1722, 1722, 1722, 1722,
    0, 1722,    0, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1722,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1722, 1722,    0, 1722, 1722, 1722, 1457,    0,    0,    0,
 1722, 1722, 1722, 1722, 1722,    0, 1722,    0,    0, 1722,
 1722,    0,    0,    0, 1722, 1722,    0,    0, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722, 1722,    0, 1722,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,    0,
 1722, 1722, 1722, 1722, 1722, 1722,    0, 1722, 1722, 1722,
 1722,    0, 1722,    0, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722,    0, 1722, 1722, 1722, 1722,    0, 1722, 1722,
 1722, 1722, 1722,    0, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722, 1722,    0, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722,    0, 1722, 1722,    0,    0,    0, 1722,
 1722, 1722, 1722, 1722, 1722,    0,    0, 1722, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722,    0,    0,    0,    0,
    0, 1722,    0, 1722, 1722, 1722, 1722, 1722, 1722,    0,
 1722,    0,    0,    0,    0,    0,    0,    0, 1459, 1458,
 1460,    0,    0,    0,    0,    0,    0, 1722, 1722,    0,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,    0,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722,    0, 1722, 1722, 1722,    0, 1722,    0,    0,
    0, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722, 1722,    0, 1722, 1722,    0,    0, 1713,
    0,    0, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
    0,    0, 1722,    0, 1722, 1722, 1722, 1722, 1722, 1722,
 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722, 1722,
 1713, 1715, 1715, 1713, 1715,    0, 1715,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1713, 1713,
 1713, 1713,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1713,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1443,    0,    0,    0,    0,    0,    0,    0,    0, 1444,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1445,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1446,    0,    0,    0,
    0,    0,    0, 1447,    0,    0,    0,    0,    0,    0,
    0, 1448,    0,    0, 3017,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1449,    0,
 1450,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1451,    0, 1452, 1453, 1454,    0,    0,    0,    0, 1455,
    0,    0,    0,    0,    0,    0, 1713,    0, 1713,    0,
    0, 1713,    0, 1713, 1713, 1715,    0, 1713, 1713,    0,
 1713,    0,    0,    0,    0,    0, 1715,    0, 1713, 1713,
 1715, 1713,    0, 1713,    0,    0, 1713,    0,    0, 1713,
 1456,    0,    0,    0,    0,    0, 1713,    0, 1713, 1713,
    0, 1713, 1713, 1713, 1713, 1713,    0,    0, 1713,    0,
    0, 1713,    0, 1713, 1713,    0,    0, 1713, 1713, 1713,
    0, 1713, 1713, 1713, 1713, 1713, 1713, 1713,    0,    0,
    0, 1713, 1713, 1713, 1713, 1713,    0,    0,    0, 1713,
    0,    0,    0,    0,    0,    0,    0, 1713, 1713, 1713,
 1713, 1713, 1713,    0,    0,    0, 1713, 1713,    0, 1713,
    0, 1713,    0, 1713, 1713, 1713,    0, 1713,    0, 1713,
 1713, 1713,    0, 1713, 1713, 1713, 1713, 1713,    0, 1713,
 1713,    0,    0,    0,    0, 1713,    0, 1713, 1713,    0,
    0, 1713, 1713, 1713,    0,    0, 1713,    0, 1713,    0,
    0, 1713, 1713, 1713, 1713,    0, 1713,    0, 1713,    0,
 1713, 1713, 1713, 1713, 1713,    0,    0, 1713, 1713, 1713,
    0, 1713, 1713,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1713, 1713, 1713,    0, 1713, 1713, 1713,    0,
    0,    0,    0,    0, 1713,    0,    0,    0, 1713,    0,
 1713, 1713, 1713, 1713,    0, 1713,    0, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1713,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1713, 1713,    0, 1713, 1713, 1713,
 1457,    0,    0,    0, 1713, 1713, 1713, 1713, 1713,    0,
 1713,    0,    0, 1713, 1713,    0,    0,    0, 1713, 1713,
    0,    0, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
 1713,    0, 1713,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713,    0, 1713, 1713, 1713, 1713, 1713, 1713,
    0, 1713, 1713, 1713, 1713,    0, 1713,    0, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713,    0, 1713, 1713, 1713,
 1713,    0, 1713, 1713, 1713, 1713, 1713,    0, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,    0, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713,    0, 1713, 1713,
    0,    0,    0, 1713, 1713, 1713, 1713, 1713, 1713,    0,
    0, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
    0,    0,    0,    0,    0, 1713,    0, 1713, 1713, 1713,
 1713, 1713, 1713,    0, 1713,    0,    0,    0,    0,    0,
    0,    0, 1459, 1458, 1460,    0,    0,    0,    0,    0,
    0, 1713, 1713,    0, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713,    0, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713,    0, 1713, 1713, 1713,
    0, 1713,    0,    0,    0, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,    0, 1713,
 1713,    0,    0, 1585,    0,    0, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713,    0,    0, 1713,    0, 1713, 1713,
 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713, 1713,
 1713, 1713, 1713, 1713, 1585, 1716, 1716, 1585, 1716,    0,
 1716,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1585, 1585, 1585, 1585,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1585,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1443,    0,    0,    0,    0,    0,
    0,    0,    0, 1444,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1445,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1446,    0,    0,    0,    0,    0,    0, 1447,    0,    0,
    0,    0,    0,    0,    0, 1448,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1449,    0, 1450,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,    0,
    0,    0,    0, 1455,    0,    0,    0,    0,    0,    0,
 1585,    0, 1585,    0,    0, 1585,    0, 1585, 1585, 1716,
    0, 1585, 1585,    0, 1585,    0,    0,    0,    0,    0,
 1716,    0, 1585, 1585, 1716, 1585,    0, 1585,    0,    0,
 1585,    0,    0, 1585, 1456,    0,    0,    0,    0,    0,
 1585,    0, 1585, 1585,    0, 1585, 1585, 1585, 1585, 1585,
    0,    0, 1585,    0,    0, 1585,    0, 1585, 1585,    0,
    0, 1585, 1585, 1585,    0, 1585, 1585, 1585, 1585, 1585,
 1585, 1585,    0,    0,    0, 1585, 1585, 1585, 1585, 1585,
    0,    0,    0, 1585,    0,    0,    0,    0,    0,    0,
    0, 1585, 1585, 1585, 1585, 1585, 1585,    0,    0,    0,
 1585, 1585,    0, 1585,    0, 1585,    0, 1585, 1585, 1585,
    0, 1585,    0, 1585, 1585, 1585,    0, 1585, 1585, 1585,
 1585, 1585,    0, 1585, 1585,    0,    0,    0,    0, 1585,
    0, 1585, 1585,    0,    0, 1585, 1585, 1585,    0,    0,
 1585,    0, 1585,    0,    0, 1585, 1585, 1585, 1585,    0,
 1585,    0, 1585,    0, 1585, 1585, 1585, 1585, 1585,    0,
    0, 1585, 1585, 1585,    0, 1585, 1585,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1585, 1585, 1585,    0,
 1585, 1585, 1585,    0,    0,    0,    0,    0, 1585,    0,
    0,    0, 1585,    0, 1585, 1585, 1585, 1585,    0, 1585,
    0, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1585, 1585,
    0, 1585, 1585, 1585, 1457,    0,    0,    0, 1585, 1585,
 1585, 1585, 1585,    0, 1585,    0,    0, 1585, 1585,    0,
    0,    0, 1585, 1585,    0,    0, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585, 1585,    0, 1585,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585,    0, 1585, 1585,
 1585, 1585, 1585, 1585,    0, 1585, 1585, 1585, 1585,    0,
 1585,    0, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
    0, 1585, 1585, 1585, 1585,    0, 1585, 1585, 1585, 1585,
 1585,    0, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585, 1585,    0, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585,    0, 1585, 1585,    0,    0,    0, 1585, 1585, 1585,
 1585, 1585, 1585,    0,    0, 1585, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585,    0,    0,    0,    0,    0, 1585,
    0, 1585, 1585, 1585, 1585, 1585, 1585,    0, 1585,    0,
    0,    0,    0,    0,    0,    0, 1459, 1458, 1460,    0,
    0,    0,    0,    0,    0, 1585, 1585,    0, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585,    0, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
    0, 1585, 1585, 1585,    0, 1585,    0,    0,    0, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585, 1585,    0, 1585, 1585,    0,    0, 1736,    0,    0,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,    0,    0,
 1585,    0, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585,
 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1585, 1736, 1736,
 1736, 1736, 1736,    0, 1736,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1736, 1736, 1736, 1736,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1736,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1443,    0,
    0,    0,    0,    0,    0,    0,    0, 1444,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1445,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1446,    0,    0,    0,    0,    0,
    0, 2176,    0,    0,    0,    0,    0,    0,    0, 1448,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1449,    0, 1450,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1451,    0,
 1452, 1453, 1454,    0,    0,    0,    0, 1455,    0,    0,
    0,    0,    0,    0, 1736,    0, 1736,    0,    0, 1736,
    0, 1736, 1736,    0,    0, 1736, 1736,    0, 1736,    0,
    0,    0,    0,    0,    0,    0, 1736, 1736, 1736, 1736,
    0, 1736,    0,    0, 1736,    0,    0, 1736, 1456,    0,
    0,    0,    0,    0, 1736,    0, 1736, 1736,    0, 1736,
 1736, 1736, 1736, 1736,    0,    0, 1736,    0,    0, 1736,
    0, 1736, 1736,    0,    0, 1736, 1736, 1736,    0, 1736,
 1736, 1736, 1736, 1736, 1736, 1736,    0,    0,    0, 1736,
 1736, 1736, 1736, 1736,    0,    0,    0, 1736,    0,    0,
    0,    0,    0,    0,    0, 1736, 1736, 1736, 1736, 1736,
 1736,    0,    0,    0, 1736, 1736,    0, 1736,    0, 1736,
    0, 1736, 1736, 1736,    0, 1736,    0, 1736, 1736, 1736,
    0, 1736, 1736, 1736, 1736, 1736,    0, 1736, 1736,    0,
    0,    0,    0, 1736,    0, 1736, 1736,    0,    0, 1736,
 1736, 1736,    0,    0, 1736,    0, 1736,    0,    0, 1736,
 1736, 1736, 1736,    0, 1736,    0, 1736,    0, 1736, 1736,
 1736, 1736, 1736,    0,    0, 1736, 1736, 1736,    0, 1736,
 1736,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1736, 1736, 1736,    0, 1736, 1736, 1736,    0,    0,    0,
    0,    0, 1736,    0,    0,    0, 1736,    0, 1736, 1736,
 1736, 1736,    0, 1736,    0, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1736,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1736, 1736,    0, 1736, 1736, 1736, 1457,    0,
    0,    0, 1736, 1736, 1736, 1736, 1736,    0, 1736,    0,
    0, 1736, 1736,    0,    0,    0, 1736, 1736,    0,    0,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,    0,
 1736,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736,    0, 1736, 1736, 1736, 1736, 1736, 1736,    0, 1736,
 1736, 1736, 1736,    0, 1736,    0, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736,    0, 1736, 1736, 1736, 1736,    0,
 1736, 1736, 1736, 1736, 1736,    0, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736, 1736,    0, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736,    0, 1736, 1736,    0,    0,
    0, 1736, 1736, 1736, 1736, 1736, 1736,    0,    0, 1736,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,    0,    0,
    0,    0,    0, 1736,    0, 1736, 1736, 1736, 1736, 1736,
 1736,    0, 1736,    0,    0,    0,    0,    0,    0,    0,
 1714, 1714, 1714,    0,    0,    0,    0,    0,    0, 1736,
 1736,    0, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736,    0, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736,    0, 1736, 1736, 1736,    0, 1736,
    0,    0,    0, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736, 1736,    0, 1736, 1736,    0,
    0, 1734,    0,    0, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736,    0,    0, 1736,    0, 1736, 1736, 1736, 1736,
 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736, 1736,
 1736, 1736, 1734, 1734, 1734, 1734, 1734,    0, 1734,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1734, 1734, 1734, 1734,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1734,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1714,    0,    0,    0,    0,    0,    0,    0,
    0, 1714,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1714,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1714,    0,
    0,    0,    0,    0,    0, 1714,    0,    0,    0,    0,
    0,    0,    0, 1714,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1714,    0, 1714,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1714,    0, 1714, 1714, 1714,    0,    0,    0,
    0, 1714,    0,    0,    0,    0,    0,    0, 1734,    0,
 1734,    0,    0, 1734,    0, 1734, 1734,    0,    0, 1734,
 1734,    0, 1734,    0,    0,    0,    0,    0,    0,    0,
 1734, 1734,    0, 1734,    0, 1734,    0,    0, 1734,    0,
    0, 1734, 1714,    0,    0,    0,    0,    0, 1734,    0,
 1734, 1734,    0, 1734, 1734, 1734, 1734, 1734,    0,    0,
 1734,    0,    0, 1734,    0, 1734, 1734,    0,    0, 1734,
 1734, 1734,    0, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
    0,    0,    0, 1734, 1734, 1734, 1734, 1734,    0,    0,
    0, 1734,    0,    0,    0,    0,    0,    0,    0, 1734,
 1734, 1734, 1734, 1734, 1734,    0,    0,    0, 1734, 1734,
    0, 1734,    0, 1734,    0, 1734, 1734, 1734,    0, 1734,
    0, 1734, 1734, 1734,    0, 1734, 1734, 1734, 1734, 1734,
    0, 1734, 1734,    0,    0,    0,    0, 1734,    0, 1734,
 1734,    0,    0, 1734, 1734, 1734,    0,    0, 1734,    0,
 1734,    0,    0, 1734, 1734, 1734, 1734,    0, 1734,    0,
 1734,    0, 1734, 1734, 1734, 1734, 1734,    0,    0, 1734,
 1734, 1734,    0, 1734, 1734,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1734, 1734, 1734,    0, 1734, 1734,
 1734,    0,    0,    0,    0,    0, 1734,    0,    0,    0,
 1734,    0, 1734, 1734, 1734, 1734,    0, 1734,    0, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1734,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1734, 1734,    0, 1734,
 1734, 1734, 1714,    0,    0,    0, 1734, 1734, 1734, 1734,
 1734,    0, 1734,    0,    0, 1734, 1734,    0,    0,    0,
 1734, 1734,    0,    0, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734, 1734,    0, 1734,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734,    0, 1734, 1734, 1734, 1734,
 1734, 1734,    0, 1734, 1734, 1734, 1734,    0, 1734,    0,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,    0, 1734,
 1734, 1734, 1734,    0, 1734, 1734, 1734, 1734, 1734,    0,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
    0, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,    0,
 1734, 1734,    0,    0,    0, 1734, 1734, 1734, 1734, 1734,
 1734,    0,    0, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734,    0,    0,    0,    0,    0, 1734,    0, 1734,
 1734, 1734, 1734, 1734, 1734,    0, 1734,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1734, 1734,    0, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734,    0, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,    0, 1734,
 1734, 1734,    0, 1734,    0,    0,    0, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
    0, 1734, 1734,    0,    0, 1733,    0,    0, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734,    0,    0, 1734,    0,
 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734, 1734,
 1734, 1734, 1734, 1734, 1734, 1734, 1733, 1733, 1733, 1733,
 1733,    0, 1733,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1733, 1733, 1733, 1733,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1733,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1733,    0, 1733,    0,    0, 1733,    0, 1733,
 1733,    0,    0, 1733, 1733,    0, 1733,    0,    0,    0,
    0,    0,    0,    0, 1733, 1733,    0, 1733,    0, 1733,
    0,    0, 1733,    0,    0, 1733,    0,    0,    0,    0,
    0,    0, 1733,    0, 1733, 1733,    0, 1733, 1733, 1733,
 1733, 1733,    0,    0, 1733,    0,    0, 1733,    0, 1733,
 1733,    0,    0, 1733, 1733, 1733,    0, 1733, 1733, 1733,
 1733, 1733, 1733, 1733,    0,    0,    0, 1733, 1733, 1733,
 1733, 1733,    0,    0,    0, 1733,    0,    0,    0,    0,
    0,    0,    0, 1733, 1733, 1733, 1733, 1733, 1733,    0,
    0,    0, 1733, 1733,    0, 1733,    0, 1733,    0, 1733,
 1733, 1733,    0, 1733,    0, 1733, 1733, 1733,    0, 1733,
 1733, 1733, 1733, 1733,    0, 1733, 1733,    0,    0,    0,
    0, 1733,    0, 1733, 1733,    0,    0, 1733, 1733, 1733,
    0,    0, 1733,    0, 1733,    0,    0, 1733, 1733, 1733,
 1733,    0, 1733,    0, 1733,    0, 1733, 1733, 1733, 1733,
 1733,    0,    0, 1733, 1733, 1733,    0, 1733, 1733,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1733, 1733,
 1733,    0, 1733, 1733, 1733,    0,    0,    0,    0,    0,
 1733,    0,    0,    0, 1733,    0, 1733, 1733, 1733, 1733,
    0, 1733,    0, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1733,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1733, 1733,    0, 1733, 1733, 1733,    0,    0,    0,    0,
 1733, 1733, 1733, 1733, 1733,    0, 1733,    0,    0, 1733,
 1733,    0,    0,    0, 1733, 1733,    0,    0, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733, 1733,    0, 1733,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,    0,
 1733, 1733, 1733, 1733, 1733, 1733,    0, 1733, 1733, 1733,
 1733,    0, 1733,    0, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733,    0, 1733, 1733, 1733, 1733,    0, 1733, 1733,
 1733, 1733, 1733,    0, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733, 1733,    0, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733,    0, 1733, 1733,    0,    0,    0, 1733,
 1733, 1733, 1733, 1733, 1733,    0,    0, 1733, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733,    0,    0,    0,    0,
    0, 1733,    0, 1733, 1733, 1733, 1733, 1733, 1733,    0,
 1733,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1733, 1733,    0,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,    0,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733,    0, 1733, 1733, 1733,    0, 1733,    0,    0,
    0, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733, 1733,    0, 1733, 1733,    0,    0, 1741,
    0,    0, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
    0,    0, 1733,    0, 1733, 1733, 1733, 1733, 1733, 1733,
 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733, 1733,
 1741, 1741, 1741, 1741, 1741,    0, 1741,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1741, 1741,
 1741, 1741,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1741,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1741,    0, 1741,    0,
    0, 1741,    0, 1741, 1741,    0,    0, 1741, 1741,    0,
 1741,    0,    0,    0,    0,    0,    0,    0, 1741, 1741,
    0, 1741,    0, 1741,    0,    0, 1741,    0,    0, 1741,
    0,    0,    0,    0,    0,    0, 1741,    0, 1741, 1741,
    0, 1741, 1741, 1741, 1741, 1741,    0,    0, 1741,    0,
    0, 1741,    0, 1741, 1741,    0,    0, 1741, 1741, 1741,
    0, 1741, 1741, 1741, 1741, 1741, 1741, 1741,    0,    0,
    0, 1741, 1741, 1741, 1741, 1741,    0,    0,    0, 1741,
    0,    0,    0,    0,    0,    0,    0, 1741, 1741, 1741,
 1741, 1741, 1741,    0,    0,    0, 1741, 1741,    0, 1741,
    0, 1741,    0, 1741, 1741, 1741,    0, 1741,    0, 1741,
 1741, 1741,    0, 1741, 1741, 1741, 1741, 1741,    0, 1741,
 1741,    0,    0,    0,    0, 1741,    0, 1741, 1741,    0,
    0, 1741, 1741, 1741,    0,    0, 1741,    0, 1741,    0,
    0, 1741, 1741, 1741, 1741,    0, 1741,    0, 1741,    0,
 1741, 1741, 1741, 1741, 1741,    0,    0, 1741, 1741, 1741,
    0, 1741, 1741,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1741, 1741, 1741,    0, 1741, 1741, 1741,    0,
    0,    0,    0,    0, 1741,    0,    0,    0, 1741,    0,
 1741, 1741, 1741, 1741,    0, 1741,    0, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1741,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1741, 1741,    0, 1741, 1741, 1741,
    0,    0,    0,    0, 1741, 1741, 1741, 1741, 1741,    0,
 1741,    0,    0, 1741, 1741,    0,    0,    0, 1741, 1741,
    0,    0, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
 1741,    0, 1741,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741,    0, 1741, 1741, 1741, 1741, 1741, 1741,
    0, 1741, 1741, 1741, 1741,    0, 1741,    0, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741,    0, 1741, 1741, 1741,
 1741,    0, 1741, 1741, 1741, 1741, 1741,    0, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,    0, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741,    0, 1741, 1741,
    0,    0,    0, 1741, 1741, 1741, 1741, 1741, 1741,    0,
    0, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
    0,    0,    0,    0,    0, 1741,    0, 1741, 1741, 1741,
 1741, 1741, 1741,    0, 1741,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1741, 1741,    0, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741,    0, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741,    0, 1741, 1741, 1741,
    0, 1741,    0,    0,    0, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,    0, 1741,
 1741,    0,    0, 1742,    0,    0, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741,    0,    0, 1741,    0, 1741, 1741,
 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741, 1741,
 1741, 1741, 1741, 1741, 1742, 1742, 1742, 1742, 1742,    0,
 1742,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1742, 1742, 1742, 1742,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1742,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1742,    0, 1742,    0,    0, 1742,    0, 1742, 1742,    0,
    0, 1742, 1742,    0, 1742,    0,    0,    0,    0,    0,
    0,    0, 1742, 1742,    0, 1742,    0, 1742,    0,    0,
 1742,    0,    0, 1742,    0,    0,    0,    0,    0,    0,
 1742,    0, 1742, 1742,    0, 1742, 1742, 1742, 1742, 1742,
    0,    0, 1742,    0,    0, 1742,    0, 1742, 1742,    0,
    0, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742,    0,    0,    0, 1742, 1742, 1742, 1742, 1742,
    0,    0,    0, 1742,    0,    0,    0,    0,    0,    0,
    0, 1742, 1742, 1742, 1742, 1742, 1742,    0,    0,    0,
 1742, 1742,    0, 1742,    0, 1742,    0, 1742, 1742, 1742,
    0, 1742,    0, 1742, 1742, 1742,    0, 1742, 1742, 1742,
 1742, 1742,    0, 1742, 1742,    0,    0,    0,    0, 1742,
    0, 1742, 1742,    0,    0, 1742, 1742, 1742,    0,    0,
 1742,    0, 1742,    0,    0, 1742, 1742, 1742, 1742,    0,
 1742,    0, 1742,    0, 1742, 1742, 1742, 1742, 1742,    0,
    0, 1742, 1742, 1742,    0, 1742, 1742,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1742, 1742, 1742,    0,
 1742, 1742, 1742,    0,    0,    0,    0,    0, 1742,    0,
    0,    0, 1742,    0, 1742, 1742, 1742, 1742,    0, 1742,
    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1742, 1742,
    0, 1742, 1742, 1742,    0,    0,    0,    0, 1742, 1742,
 1742, 1742, 1742,    0, 1742,    0,    0, 1742, 1742,    0,
    0,    0, 1742, 1742,    0,    0, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742,    0, 1742,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742,
 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,    0,
 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
    0, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,
 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742,    0, 1742, 1742,    0,    0,    0, 1742, 1742, 1742,
 1742, 1742, 1742,    0,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742,    0,    0,    0,    0,    0, 1742,
    0, 1742, 1742, 1742, 1742, 1742, 1742,    0, 1742,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1742, 1742,    0, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
    0, 1742, 1742, 1742,    0, 1742,    0,    0,    0, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742,    0, 1742, 1742,    0,    0, 1735,    0,    0,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0,    0,
 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1735,    0,
 1735, 1735, 1735,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1735, 1735, 1735, 1735,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1735,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1735,    0, 1735,    0,    0, 1735,
    0, 1735, 1735,    0,    0, 1735, 1735,    0, 1735,    0,
    0,    0,    0,    0,    0,    0, 1735, 1735,    0, 1735,
    0, 1735,    0,    0, 1735,    0,    0, 1735,    0,    0,
    0,    0,    0,    0, 1735,    0, 1735, 1735,    0, 1735,
 1735, 1735, 1735, 1735,    0,    0, 1735,    0,    0, 1735,
    0, 1735, 1735,    0,    0, 1735, 1735, 1735,    0, 1735,
 1735, 1735, 1735, 1735, 1735, 1735,    0,    0,    0, 1735,
 1735, 1735, 1735, 1735,    0,    0,    0, 1735,    0,    0,
    0,    0,    0,    0,    0, 1735, 1735, 1735, 1735, 1735,
 1735,    0,    0,    0, 1735, 1735,    0, 1735,    0, 1735,
    0, 1735, 1735, 1735,    0, 1735,    0, 1735, 1735, 1735,
    0, 1735, 1735, 1735, 1735, 1735,    0, 1735, 1735,    0,
    0,    0,    0, 1735,    0, 1735, 1735,    0,    0, 1735,
 1735, 1735,    0,    0, 1735,    0, 1735,    0,    0, 1735,
 1735, 1735, 1735,    0, 1735,    0, 1735,    0, 1735, 1735,
 1735, 1735, 1735,    0,    0, 1735, 1735, 1735,    0, 1735,
 1735,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1735, 1735, 1735,    0, 1735, 1735, 1735,    0,    0,    0,
    0,    0, 1735,    0,    0,    0, 1735,    0, 1735, 1735,
 1735, 1735,    0, 1735,    0, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1735,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1735, 1735,    0, 1735, 1735, 1735,    0,    0,
    0,    0, 1735, 1735, 1735, 1735, 1735,    0, 1735,    0,
    0, 1735, 1735,    0,    0,    0, 1735, 1735,    0,    0,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,    0,
 1735,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735,    0, 1735, 1735, 1735, 1735, 1735, 1735,    0, 1735,
 1735, 1735, 1735,    0, 1735,    0, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735,    0, 1735, 1735, 1735, 1735,    0,
 1735, 1735, 1735, 1735, 1735,    0, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735, 1735,    0, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735,    0, 1735, 1735,    0,    0,
    0, 1735, 1735, 1735, 1735, 1735, 1735,    0,    0, 1735,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,    0,    0,
    0,    0,    0, 1735,    0, 1735, 1735, 1735, 1735, 1735,
 1735,    0, 1735,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1735,
 1735,    0, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735,    0, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735,    0, 1735, 1735, 1735,    0, 1735,
    0,    0,    0, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735, 1735,    0, 1735, 1735,    0,
    0, 1740,    0,    0, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735,    0,    0, 1735,    0, 1735, 1735, 1735, 1735,
 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735, 1735,
 1735, 1735, 1740,    0, 1740, 1740, 1740,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1740, 1740, 1740, 1740,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1740,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1740,    0,
 1740,    0,    0, 1740,    0, 1740, 1740,    0,    0, 1740,
 1740,    0, 1740,    0,    0,    0,    0,    0,    0,    0,
 1740, 1740,    0, 1740,    0, 1740,    0,    0, 1740,    0,
    0, 1740,    0,    0,    0,    0,    0,    0, 1740,    0,
 1740, 1740,    0, 1740, 1740, 1740, 1740, 1740,    0,    0,
 1740,    0,    0, 1740,    0, 1740, 1740,    0,    0, 1740,
 1740, 1740,    0, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
    0,    0,    0, 1740, 1740, 1740, 1740, 1740,    0,    0,
    0, 1740,    0,    0,    0,    0,    0,    0,    0, 1740,
 1740, 1740, 1740, 1740, 1740,    0,    0,    0, 1740, 1740,
    0, 1740,    0, 1740,    0, 1740, 1740, 1740,    0, 1740,
    0, 1740, 1740, 1740,    0, 1740, 1740, 1740, 1740, 1740,
    0, 1740, 1740,    0,    0,    0,    0, 1740,    0, 1740,
 1740,    0,    0, 1740, 1740, 1740,    0,    0, 1740,    0,
 1740,    0,    0, 1740, 1740, 1740, 1740,    0, 1740,    0,
 1740,    0, 1740, 1740, 1740, 1740, 1740,    0,    0, 1740,
 1740, 1740,    0, 1740, 1740,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1740, 1740, 1740,    0, 1740, 1740,
 1740,    0,    0,    0,    0,    0, 1740,    0,    0,    0,
 1740,    0, 1740, 1740, 1740, 1740,    0, 1740,    0, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1740,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1740, 1740,    0, 1740,
 1740, 1740,    0,    0,    0,    0, 1740, 1740, 1740, 1740,
 1740,    0, 1740,    0,    0, 1740, 1740,    0,    0,    0,
 1740, 1740,    0,    0, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740, 1740,    0, 1740,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740,    0, 1740, 1740, 1740, 1740,
 1740, 1740,    0, 1740, 1740, 1740, 1740,    0, 1740,    0,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,    0, 1740,
 1740, 1740, 1740,    0, 1740, 1740, 1740, 1740, 1740,    0,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
    0, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,    0,
 1740, 1740,    0,    0,    0, 1740, 1740, 1740, 1740, 1740,
 1740,    0,    0, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740,    0,    0,    0,    0,    0, 1740,    0, 1740,
 1740, 1740, 1740, 1740, 1740,    0, 1740,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1740, 1740,    0, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740,    0, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,    0, 1740,
 1740, 1740,    0, 1740,    0,    0,    0, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
    0, 1740, 1740,    0,    0, 1615,    0,    0, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740,    0,    0, 1740,    0,
 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740, 1740,
 1740, 1740, 1740, 1740, 1740, 1740, 1615,    0,    0, 1615,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1615, 1615, 1615, 1615,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1615,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1615,    0, 1615,    0,    0, 1615,    0, 1615,
 1615,    0,    0, 1615, 1615,    0, 1615,    0,    0,    0,
    0,    0,    0,    0, 1615, 1615,    0, 1615,    0, 1615,
    0,    0, 1615,    0,    0, 1615,    0,    0,    0,    0,
    0,    0, 1615,    0, 1615, 1615,    0, 1615, 1615, 1615,
 1615, 1615,    0,    0, 1615,    0,    0, 1615,    0, 1615,
 1615,    0,    0, 1615, 1615, 1615,    0, 1615, 1615, 1615,
 1615, 1615, 1615, 1615,    0,    0,    0, 1615, 1615, 1615,
 1615, 1615,    0,    0,    0, 1615,    0,    0,    0,    0,
    0,    0,    0, 1615, 1615, 1615, 1615, 1615, 1615,    0,
    0,    0, 1615, 1615,    0, 1615,    0, 1615,    0, 1615,
 1615, 1615,    0, 1615,    0, 1615, 1615, 1615,    0, 1615,
 1615, 1615, 1615, 1615,    0, 1615, 1615,    0,    0,    0,
    0, 1615,    0, 1615, 1615,    0,    0, 1615, 1615, 1615,
    0,    0, 1615,    0, 1615,    0,    0, 1615, 1615, 1615,
 1615,    0, 1615,    0, 1615,    0, 1615, 1615, 1615, 1615,
 1615,    0,    0, 1615, 1615, 1615,    0, 1615, 1615,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1615, 1615,
 1615,    0, 1615, 1615, 1615,    0,    0,    0,    0,    0,
 1615,    0,    0,    0, 1615,    0, 1615, 1615, 1615, 1615,
    0, 1615,    0, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1615,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1615, 1615,    0, 1615, 1615, 1615,    0,    0,    0,    0,
 1615, 1615, 1615, 1615, 1615,    0, 1615,    0,    0, 1615,
 1615,    0,    0,    0, 1615, 1615,    0,    0, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615, 1615,    0, 1615,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,    0,
 1615, 1615, 1615, 1615, 1615, 1615,    0, 1615, 1615, 1615,
 1615,    0, 1615,    0, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615,    0, 1615, 1615, 1615, 1615,    0, 1615, 1615,
 1615, 1615, 1615,    0, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615, 1615,    0, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615,    0, 1615, 1615,    0,    0,    0, 1615,
 1615, 1615, 1615, 1615, 1615,    0,    0, 1615, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615,    0,    0,    0,    0,
    0, 1615,    0, 1615, 1615, 1615, 1615, 1615, 1615,    0,
 1615,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1615, 1615,    0,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,    0,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615,    0, 1615, 1615, 1615,    0, 1615,    0,    0,
    0, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615, 1615,    0, 1615, 1615,    0,    0, 1616,
    0,    0, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
    0,    0, 1615,    0, 1615, 1615, 1615, 1615, 1615, 1615,
 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615, 1615,
 1616,    0,    0, 1616,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1616, 1616,
 1616, 1616,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1616,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1616,    0, 1616,    0,
    0, 1616,    0, 1616, 1616,    0,    0, 1616, 1616,    0,
 1616,    0,    0,    0,    0,    0,    0,    0, 1616, 1616,
    0, 1616,    0, 1616,    0,    0, 1616,    0,    0, 1616,
    0,    0,    0,    0,    0,    0, 1616,    0, 1616, 1616,
    0, 1616, 1616, 1616, 1616, 1616,    0,    0, 1616,    0,
    0, 1616,    0, 1616, 1616,    0,    0, 1616, 1616, 1616,
    0, 1616, 1616, 1616, 1616, 1616, 1616, 1616,    0,    0,
    0, 1616, 1616, 1616, 1616, 1616,    0,    0,    0, 1616,
    0,    0,    0,    0,    0,    0,    0, 1616, 1616, 1616,
 1616, 1616, 1616,    0,    0,    0, 1616, 1616,    0, 1616,
    0, 1616,    0, 1616, 1616, 1616,    0, 1616,    0, 1616,
 1616, 1616,    0, 1616, 1616, 1616, 1616, 1616,    0, 1616,
 1616,    0,    0,    0,    0, 1616,    0, 1616, 1616,    0,
    0, 1616, 1616, 1616,    0,    0, 1616,    0, 1616,    0,
    0, 1616, 1616, 1616, 1616,    0, 1616,    0, 1616,    0,
 1616, 1616, 1616, 1616, 1616,    0,    0, 1616, 1616, 1616,
    0, 1616, 1616,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1616, 1616, 1616,    0, 1616, 1616, 1616,    0,
    0,    0,    0,    0, 1616,    0,    0,    0, 1616,    0,
 1616, 1616, 1616, 1616,    0, 1616,    0, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1616,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1616, 1616,    0, 1616, 1616, 1616,
    0,    0,    0,    0, 1616, 1616, 1616, 1616, 1616,    0,
 1616,    0,    0, 1616, 1616,    0,    0,    0, 1616, 1616,
    0,    0, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
 1616,    0, 1616,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616,    0, 1616, 1616, 1616, 1616, 1616, 1616,
    0, 1616, 1616, 1616, 1616,    0, 1616,    0, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616,    0, 1616, 1616, 1616,
 1616,    0, 1616, 1616, 1616, 1616, 1616,    0, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,    0, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616,    0, 1616, 1616,
    0,    0,    0, 1616, 1616, 1616, 1616, 1616, 1616,    0,
    0, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
    0,    0,    0,    0,    0, 1616,    0, 1616, 1616, 1616,
 1616, 1616, 1616,    0, 1616,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1616, 1616,    0, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616,    0, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616,    0, 1616, 1616, 1616,
    0, 1616,    0,    0,    0, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,    0, 1616,
 1616,    0,    0, 1600,    0,    0, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616,    0,    0, 1616,    0, 1616, 1616,
 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616, 1616,
 1616, 1616, 1616, 1616, 1600,    0,    0, 1600,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1600, 1600, 1600, 1600,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1600,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1600,    0, 1600,    0,    0, 1600,    0, 1600, 1600,    0,
    0, 1600, 1600,    0, 1600,    0,    0,    0,    0,    0,
    0,    0, 1600, 1600,    0, 1600,    0, 1600,    0,    0,
 1600,    0,    0, 1600,    0,    0,    0,    0,    0,    0,
 1600,    0, 1600, 1600,    0, 1600, 1600, 1600, 1600, 1600,
    0,    0, 1600,    0,    0, 1600,    0, 1600, 1600,    0,
    0, 1600, 1600, 1600,    0, 1600, 1600, 1600, 1600, 1600,
 1600, 1600,    0,    0,    0, 1600,    0, 1600, 1600, 1600,
    0,    0,    0, 1600,    0,    0,    0,    0,    0,    0,
    0, 1600, 1600, 1600, 1600, 1600, 1600,    0,    0,    0,
 1600, 1600,    0, 1600,    0, 1600,    0, 1600, 1600, 1600,
    0, 1600,    0, 1600, 1600, 1600,    0, 1600, 1600, 1600,
 1600, 1600,    0, 1600, 1600,    0,    0,    0,    0, 1600,
    0, 1600, 1600,    0,    0, 1600, 1600, 1600,    0,    0,
 1600,    0, 1600,    0,    0, 1600, 1600, 1600, 1600,    0,
 1600,    0, 1600,    0, 1600, 1600, 1600, 1600, 1600,    0,
    0, 1600, 1600, 1600,    0, 1600, 1600,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1600, 1600, 1600,    0,
 1600, 1600, 1600,    0,    0,    0,    0,    0, 1600,    0,
    0,    0, 1600,    0, 1600, 1600, 1600, 1600,    0, 1600,
    0, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1600, 1600,
    0, 1600, 1600, 1600,    0,    0,    0,    0, 1600, 1600,
 1600, 1600, 1600,    0, 1600,    0,    0, 1600, 1600,    0,
    0,    0, 1600, 1600,    0,    0, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600, 1600,    0, 1600,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600,    0, 1600, 1600,
 1600, 1600, 1600, 1600,    0, 1600, 1600, 1600, 1600,    0,
 1600,    0, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
    0, 1600, 1600, 1600, 1600,    0, 1600, 1600, 1600, 1600,
 1600,    0, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600, 1600,    0, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600,    0, 1600, 1600,    0,    0,    0, 1600, 1600, 1600,
 1600, 1600, 1600,    0,    0, 1600, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600,    0,    0,    0,    0,    0, 1600,
    0, 1600, 1600, 1600, 1600, 1600, 1600,    0, 1600,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1600, 1600,    0, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600,    0, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
    0, 1600, 1600, 1600,    0, 1600,    0,    0,    0, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600, 1600,    0, 1600, 1600,    0,    0, 1617,    0,    0,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,    0,    0,
 1600,    0, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600,
 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1600, 1617,    0,
    0, 1617,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1617, 1617, 1617, 1617,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1617,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1617,    0, 1617,    0,    0, 1617,
    0, 1617, 1617,    0,    0, 1617, 1617,    0, 1617,    0,
    0,    0,    0,    0,    0,    0, 1617, 1617,    0, 1617,
    0, 1617,    0,    0, 1617,    0,    0, 1617,    0,    0,
    0,    0,    0,    0, 1617,    0, 1617, 1617,    0, 1617,
 1617, 1617, 1617, 1617,    0,    0, 1617,    0,    0, 1617,
    0, 1617, 1617,    0,    0, 1617, 1617, 1617,    0, 1617,
 1617, 1617, 1617, 1617, 1617, 1617,    0,    0,    0, 1617,
    0, 1617, 1617, 1617,    0,    0,    0, 1617,    0,    0,
    0,    0,    0,    0,    0, 1617, 1617, 1617, 1617, 1617,
 1617,    0,    0,    0, 1617, 1617,    0, 1617,    0, 1617,
    0, 1617, 1617, 1617,    0, 1617,    0, 1617, 1617, 1617,
    0, 1617, 1617, 1617, 1617, 1617,    0, 1617, 1617,    0,
    0,    0,    0, 1617,    0, 1617, 1617,    0,    0, 1617,
 1617, 1617,    0,    0, 1617,    0, 1617,    0,    0, 1617,
 1617, 1617, 1617,    0, 1617,    0, 1617,    0, 1617, 1617,
 1617, 1617, 1617,    0,    0, 1617, 1617, 1617,    0, 1617,
 1617,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1617, 1617, 1617,    0, 1617, 1617, 1617,    0,    0,    0,
    0,    0, 1617,    0,    0,    0, 1617,    0, 1617, 1617,
 1617, 1617,    0, 1617,    0, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1617,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1617, 1617,    0, 1617, 1617, 1617,    0,    0,
    0,    0, 1617, 1617, 1617, 1617, 1617,    0, 1617,    0,
    0, 1617, 1617,    0,    0,    0, 1617, 1617,    0,    0,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,    0,
 1617,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617,    0, 1617, 1617, 1617, 1617, 1617, 1617,    0, 1617,
 1617, 1617, 1617,    0, 1617,    0, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617,    0, 1617, 1617, 1617, 1617,    0,
 1617, 1617, 1617, 1617, 1617,    0, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617, 1617,    0, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617,    0, 1617, 1617,    0,    0,
    0, 1617, 1617, 1617, 1617, 1617, 1617,    0,    0, 1617,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,    0,    0,
    0,    0,    0, 1617,    0, 1617, 1617, 1617, 1617, 1617,
 1617,    0, 1617,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1617,
 1617,    0, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617,    0, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617,    0, 1617, 1617, 1617,    0, 1617,
    0,    0,    0, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617, 1617,    0, 1617, 1617,    0,
    0, 1618,    0,    0, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617,    0,    0, 1617,    0, 1617, 1617, 1617, 1617,
 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617, 1617,
 1617, 1617, 1618,    0,    0, 1618,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1618, 1618, 1618, 1618,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1618,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1618,    0,
 1618,    0,    0, 1618,    0, 1618, 1618,    0,    0, 1618,
 1618,    0, 1618,    0,    0,    0,    0,    0,    0,    0,
 1618, 1618,    0, 1618,    0, 1618,    0,    0, 1618,    0,
    0, 1618,    0,    0,    0,    0,    0,    0, 1618,    0,
 1618, 1618,    0, 1618, 1618, 1618, 1618, 1618,    0,    0,
 1618,    0,    0, 1618,    0, 1618, 1618,    0,    0, 1618,
 1618, 1618,    0, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
    0,    0,    0, 1618,    0, 1618, 1618, 1618,    0,    0,
    0, 1618,    0,    0,    0,    0,    0,    0,    0, 1618,
 1618, 1618, 1618, 1618, 1618,    0,    0,    0, 1618, 1618,
    0, 1618,    0, 1618,    0, 1618, 1618, 1618,    0, 1618,
    0, 1618, 1618, 1618,    0, 1618, 1618, 1618, 1618, 1618,
    0, 1618, 1618,    0,    0,    0,    0, 1618,    0, 1618,
 1618,    0,    0, 1618, 1618, 1618,    0,    0, 1618,    0,
 1618,    0,    0, 1618, 1618, 1618, 1618,    0, 1618,    0,
 1618,    0, 1618, 1618, 1618, 1618, 1618,    0,    0, 1618,
 1618, 1618,    0, 1618, 1618,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1618, 1618, 1618,    0, 1618, 1618,
 1618,    0,    0,    0,    0,    0, 1618,    0,    0,    0,
 1618,    0, 1618, 1618, 1618, 1618,    0, 1618,    0, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1618,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1618, 1618,    0, 1618,
 1618, 1618,    0,    0,    0,    0, 1618, 1618, 1618, 1618,
 1618,    0, 1618,    0,    0, 1618, 1618,    0,    0,    0,
 1618, 1618,    0,    0, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618, 1618,    0, 1618,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618,    0, 1618, 1618, 1618, 1618,
 1618, 1618,    0, 1618, 1618, 1618, 1618,    0, 1618,    0,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,    0, 1618,
 1618, 1618, 1618,    0, 1618, 1618, 1618, 1618, 1618,    0,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
    0, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,    0,
 1618, 1618,    0,    0,    0, 1618, 1618, 1618, 1618, 1618,
 1618,    0,    0, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618,    0,    0,    0,    0,    0, 1618,    0, 1618,
 1618, 1618, 1618, 1618, 1618,    0, 1618,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1618, 1618,    0, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618,    0, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,    0, 1618,
 1618, 1618,    0, 1618,    0,    0,    0, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
    0, 1618, 1618,    0,    0, 1623,    0,    0, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618,    0,    0, 1618,    0,
 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618, 1618,
 1618, 1618, 1618, 1618, 1618, 1618, 1623,    0,    0, 1623,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1623, 1623, 1623, 1623,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1623,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1623,    0, 1623,    0,    0, 1623,    0, 1623,
 1623,    0,    0, 1623, 1623,    0, 1623,    0,    0,    0,
    0,    0,    0,    0, 1623, 1623,    0, 1623,    0, 1623,
    0,    0, 1623,    0,    0, 1623,    0,    0,    0,    0,
    0,    0, 1623,    0, 1623, 1623,    0, 1623, 1623, 1623,
 1623, 1623,    0,    0, 1623,    0,    0, 1623,    0, 1623,
 1623,    0,    0, 1623, 1623, 1623,    0, 1623, 1623, 1623,
 1623, 1623, 1623, 1623,    0,    0,    0, 1623,    0, 1623,
 1623, 1623,    0,    0,    0, 1623,    0,    0,    0,    0,
    0,    0,    0, 1623, 1623, 1623, 1623, 1623, 1623,    0,
    0,    0, 1623, 1623,    0, 1623,    0, 1623,    0, 1623,
 1623, 1623,    0, 1623,    0, 1623, 1623, 1623,    0, 1623,
 1623, 1623, 1623, 1623,    0, 1623, 1623,    0,    0,    0,
    0, 1623,    0, 1623, 1623,    0,    0, 1623, 1623, 1623,
    0,    0, 1623,    0, 1623,    0,    0, 1623, 1623, 1623,
 1623,    0, 1623,    0, 1623,    0, 1623, 1623, 1623, 1623,
 1623,    0,    0, 1623, 1623, 1623,    0, 1623, 1623,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1623, 1623,
 1623,    0, 1623, 1623, 1623,    0,    0,    0,    0,    0,
 1623,    0,    0,    0, 1623,    0, 1623, 1623, 1623, 1623,
    0, 1623,    0, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1623,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1623, 1623,    0, 1623, 1623, 1623,    0,    0,    0,    0,
 1623, 1623, 1623, 1623, 1623,    0, 1623,    0,    0, 1623,
 1623,    0,    0,    0, 1623, 1623,    0,    0, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623, 1623,    0, 1623,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,    0,
 1623, 1623, 1623, 1623, 1623, 1623,    0, 1623, 1623, 1623,
 1623,    0, 1623,    0, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623,    0, 1623, 1623, 1623, 1623,    0, 1623, 1623,
 1623, 1623, 1623,    0, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623, 1623,    0, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623,    0, 1623, 1623,    0,    0,    0, 1623,
 1623, 1623, 1623, 1623, 1623,    0,    0, 1623, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623,    0,    0,    0,    0,
    0, 1623,    0, 1623, 1623, 1623, 1623, 1623, 1623,    0,
 1623,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1623, 1623,    0,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,    0,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623,    0, 1623, 1623, 1623,    0, 1623,    0,    0,
    0, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623, 1623,    0, 1623, 1623,    0,    0, 1625,
    0,    0, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
    0,    0, 1623,    0, 1623, 1623, 1623, 1623, 1623, 1623,
 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623, 1623,
 1625,    0,    0, 1625,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1625, 1625,
 1625, 1625,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1625,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1625,    0, 1625,    0,
    0, 1625,    0, 1625, 1625,    0,    0, 1625, 1625,    0,
 1625,    0,    0,    0,    0,    0,    0,    0, 1625, 1625,
    0, 1625,    0, 1625,    0,    0, 1625,    0,    0, 1625,
    0,    0,    0,    0,    0,    0, 1625,    0, 1625, 1625,
    0, 1625, 1625, 1625, 1625, 1625,    0,    0, 1625,    0,
    0, 1625,    0, 1625, 1625,    0,    0, 1625, 1625, 1625,
    0, 1625, 1625, 1625, 1625, 1625, 1625, 1625,    0,    0,
    0, 1625,    0, 1625, 1625, 1625,    0,    0,    0, 1625,
    0,    0,    0,    0,    0,    0,    0, 1625, 1625, 1625,
 1625, 1625, 1625,    0,    0,    0, 1625, 1625,    0, 1625,
    0, 1625,    0, 1625, 1625, 1625,    0, 1625,    0, 1625,
 1625, 1625,    0, 1625, 1625, 1625, 1625, 1625,    0, 1625,
 1625,    0,    0,    0,    0, 1625,    0, 1625, 1625,    0,
    0, 1625, 1625, 1625,    0,    0, 1625,    0, 1625,    0,
    0, 1625, 1625, 1625, 1625,    0, 1625,    0, 1625,    0,
 1625, 1625, 1625, 1625, 1625,    0,    0, 1625, 1625, 1625,
    0, 1625, 1625,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1625, 1625, 1625,    0, 1625, 1625, 1625,    0,
    0,    0,    0,    0, 1625,    0,    0,    0, 1625,    0,
 1625, 1625, 1625, 1625,    0, 1625,    0, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1625,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1625, 1625,    0, 1625, 1625, 1625,
    0,    0,    0,    0, 1625, 1625, 1625, 1625, 1625,    0,
 1625,    0,    0, 1625, 1625,    0,    0,    0, 1625, 1625,
    0,    0, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
 1625,    0, 1625,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625,    0, 1625, 1625, 1625, 1625, 1625, 1625,
    0, 1625, 1625, 1625, 1625,    0, 1625,    0, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625,    0, 1625, 1625, 1625,
 1625,    0, 1625, 1625, 1625, 1625, 1625,    0, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,    0, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625,    0, 1625, 1625,
    0,    0,    0, 1625, 1625, 1625, 1625, 1625, 1625,    0,
    0, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
    0,    0,    0,    0,    0, 1625,    0, 1625, 1625, 1625,
 1625, 1625, 1625,    0, 1625,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1625, 1625,    0, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625,    0, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625,    0, 1625, 1625, 1625,
    0, 1625,    0,    0,    0, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,    0, 1625,
 1625,    0,    0, 1627,    0,    0, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625,    0,    0, 1625,    0, 1625, 1625,
 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625, 1625,
 1625, 1625, 1625, 1625, 1627,    0,    0, 1627,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1627, 1627, 1627, 1627,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1627,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1627,    0, 1627,    0,    0, 1627,    0, 1627, 1627,    0,
    0, 1627, 1627,    0, 1627,    0,    0,    0,    0,    0,
    0,    0, 1627, 1627,    0, 1627,    0, 1627,    0,    0,
 1627,    0,    0, 1627,    0,    0,    0,    0,    0,    0,
 1627,    0, 1627, 1627,    0, 1627, 1627, 1627, 1627, 1627,
    0,    0, 1627,    0,    0, 1627,    0, 1627, 1627,    0,
    0, 1627, 1627, 1627,    0, 1627, 1627, 1627, 1627, 1627,
 1627, 1627,    0,    0,    0, 1627,    0, 1627, 1627, 1627,
    0,    0,    0, 1627,    0,    0,    0,    0,    0,    0,
    0, 1627, 1627, 1627, 1627, 1627, 1627,    0,    0,    0,
 1627, 1627,    0, 1627,    0, 1627,    0, 1627, 1627, 1627,
    0, 1627,    0, 1627, 1627, 1627,    0, 1627, 1627, 1627,
 1627, 1627,    0, 1627, 1627,    0,    0,    0,    0, 1627,
    0, 1627, 1627,    0,    0, 1627, 1627, 1627,    0,    0,
 1627,    0, 1627,    0,    0, 1627, 1627, 1627, 1627,    0,
 1627,    0, 1627,    0, 1627, 1627, 1627, 1627, 1627,    0,
    0, 1627, 1627, 1627,    0, 1627, 1627,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1627, 1627, 1627,    0,
 1627, 1627, 1627,    0,    0,    0,    0,    0, 1627,    0,
    0,    0, 1627,    0, 1627, 1627, 1627, 1627,    0, 1627,
    0, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1627, 1627,
    0, 1627, 1627, 1627,    0,    0,    0,    0, 1627, 1627,
 1627, 1627, 1627,    0, 1627,    0,    0, 1627, 1627,    0,
    0,    0, 1627, 1627,    0,    0, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627, 1627,    0, 1627,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627,    0, 1627, 1627,
 1627, 1627, 1627, 1627,    0, 1627, 1627, 1627, 1627,    0,
 1627,    0, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
    0, 1627, 1627, 1627, 1627,    0, 1627, 1627, 1627, 1627,
 1627,    0, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627, 1627,    0, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627,    0, 1627, 1627,    0,    0,    0, 1627, 1627, 1627,
 1627, 1627, 1627,    0,    0, 1627, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627,    0,    0,    0,    0,    0, 1627,
    0, 1627, 1627, 1627, 1627, 1627, 1627,    0, 1627,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1627, 1627,    0, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627,    0, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
    0, 1627, 1627, 1627,    0, 1627,    0,    0,    0, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627, 1627,    0, 1627, 1627,    0,    0, 1629,    0,    0,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,    0,    0,
 1627,    0, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627,
 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1627, 1629,    0,
    0, 1629,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1629, 1629, 1629, 1629,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1629,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1629,    0, 1629,    0,    0, 1629,
    0, 1629, 1629,    0,    0, 1629, 1629,    0, 1629,    0,
    0,    0,    0,    0,    0,    0, 1629, 1629,    0, 1629,
    0, 1629,    0,    0, 1629,    0,    0, 1629,    0,    0,
    0,    0,    0,    0, 1629,    0, 1629, 1629,    0, 1629,
 1629, 1629, 1629, 1629,    0,    0, 1629,    0,    0, 1629,
    0, 1629, 1629,    0,    0, 1629, 1629, 1629,    0, 1629,
 1629, 1629, 1629, 1629, 1629, 1629,    0,    0,    0, 1629,
    0, 1629, 1629, 1629,    0,    0,    0, 1629,    0,    0,
    0,    0,    0,    0,    0, 1629, 1629, 1629, 1629, 1629,
 1629,    0,    0,    0, 1629, 1629,    0, 1629,    0, 1629,
    0, 1629, 1629, 1629,    0, 1629,    0, 1629, 1629, 1629,
    0, 1629, 1629, 1629, 1629, 1629,    0, 1629, 1629,    0,
    0,    0,    0, 1629,    0, 1629, 1629,    0,    0, 1629,
 1629, 1629,    0,    0, 1629,    0, 1629,    0,    0, 1629,
 1629, 1629, 1629,    0, 1629,    0, 1629,    0, 1629, 1629,
 1629, 1629, 1629,    0,    0, 1629, 1629, 1629,    0, 1629,
 1629,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1629, 1629, 1629,    0, 1629, 1629, 1629,    0,    0,    0,
    0,    0, 1629,    0,    0,    0, 1629,    0, 1629, 1629,
 1629, 1629,    0, 1629,    0, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1629,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1629, 1629,    0, 1629, 1629, 1629,    0,    0,
    0,    0, 1629, 1629, 1629, 1629, 1629,    0, 1629,    0,
    0, 1629, 1629,    0,    0,    0, 1629, 1629,    0,    0,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,    0,
 1629,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629,    0, 1629, 1629, 1629, 1629, 1629, 1629,    0, 1629,
 1629, 1629, 1629,    0, 1629,    0, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629,    0, 1629, 1629, 1629, 1629,    0,
 1629, 1629, 1629, 1629, 1629,    0, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629, 1629,    0, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629,    0, 1629, 1629,    0,    0,
    0, 1629, 1629, 1629, 1629, 1629, 1629,    0,    0, 1629,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,    0,    0,
    0,    0,    0, 1629,    0, 1629, 1629, 1629, 1629, 1629,
 1629,    0, 1629,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1629,
 1629,    0, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629,    0, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629,    0, 1629, 1629, 1629,    0, 1629,
    0,    0,    0, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629, 1629,    0, 1629, 1629,    0,
    0, 1622,    0,    0, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629,    0,    0, 1629,    0, 1629, 1629, 1629, 1629,
 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629, 1629,
 1629, 1629, 1622,    0,    0, 1622,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1622, 1622, 1622, 1622,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1622,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1622,    0,
 1622,    0,    0, 1622,    0, 1622, 1622,    0,    0, 1622,
 1622,    0, 1622,    0,    0,    0,    0,    0,    0,    0,
 1622, 1622,    0, 1622,    0, 1622,    0,    0, 1622,    0,
    0, 1622,    0,    0,    0,    0,    0,    0, 1622,    0,
 1622, 1622,    0, 1622, 1622, 1622,    0, 1622,    0,    0,
 1622,    0,    0, 1622,    0, 1622, 1622,    0,    0, 1622,
 1622, 1622,    0, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
    0,    0,    0, 1622,    0, 1622, 1622, 1622,    0,    0,
    0, 1622,    0,    0,    0,    0,    0,    0,    0, 1622,
 1622, 1622, 1622, 1622, 1622,    0,    0,    0, 1622, 1622,
    0, 1622,    0, 1622,    0, 1622, 1622, 1622,    0, 1622,
    0, 1622, 1622, 1622,    0, 1622, 1622, 1622, 1622, 1622,
    0, 1622, 1622,    0,    0,    0,    0, 1622,    0, 1622,
 1622,    0,    0, 1622, 1622, 1622,    0,    0, 1622,    0,
 1622,    0,    0, 1622, 1622, 1622, 1622,    0, 1622,    0,
 1622,    0, 1622, 1622, 1622, 1622, 1622,    0,    0, 1622,
 1622, 1622,    0, 1622, 1622,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1622, 1622, 1622,    0, 1622, 1622,
 1622,    0,    0,    0,    0,    0, 1622,    0,    0,    0,
 1622,    0, 1622, 1622, 1622, 1622,    0, 1622,    0, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1622,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1622, 1622,    0, 1622,
 1622, 1622,    0,    0,    0,    0, 1622, 1622, 1622, 1622,
 1622,    0, 1622,    0,    0, 1622, 1622,    0,    0,    0,
 1622, 1622,    0,    0, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622, 1622,    0, 1622,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622,    0, 1622, 1622, 1622, 1622,
 1622, 1622,    0, 1622, 1622, 1622, 1622,    0, 1622,    0,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,    0, 1622,
 1622, 1622, 1622,    0, 1622, 1622, 1622, 1622, 1622,    0,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
    0, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,    0,
 1622, 1622,    0,    0,    0, 1622, 1622, 1622, 1622, 1622,
 1622,    0,    0, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622,    0,    0,    0,    0,    0, 1622,    0, 1622,
 1622, 1622, 1622, 1622, 1622,    0, 1622,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1622, 1622,    0, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622,    0, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,    0, 1622,
 1622, 1622,    0, 1622,    0,    0,    0, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
    0, 1622, 1622,    0,    0, 1624,    0,    0, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622,    0,    0, 1622,    0,
 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622, 1622,
 1622, 1622, 1622, 1622, 1622, 1622, 1624,    0,    0, 1624,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1624, 1624, 1624, 1624,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1624,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1624,    0, 1624,    0,    0, 1624,    0, 1624,
 1624,    0,    0, 1624, 1624,    0, 1624,    0,    0,    0,
    0,    0,    0,    0, 1624, 1624,    0, 1624,    0, 1624,
    0,    0, 1624,    0,    0, 1624,    0,    0,    0,    0,
    0,    0, 1624,    0, 1624, 1624,    0, 1624, 1624, 1624,
    0, 1624,    0,    0, 1624,    0,    0, 1624,    0, 1624,
 1624,    0,    0, 1624, 1624, 1624,    0, 1624, 1624, 1624,
 1624, 1624, 1624, 1624,    0,    0,    0, 1624,    0, 1624,
 1624, 1624,    0,    0,    0, 1624,    0,    0,    0,    0,
    0,    0,    0, 1624, 1624, 1624, 1624, 1624, 1624,    0,
    0,    0, 1624, 1624,    0, 1624,    0, 1624,    0, 1624,
 1624, 1624,    0, 1624,    0, 1624, 1624, 1624,    0, 1624,
 1624, 1624, 1624, 1624,    0, 1624, 1624,    0,    0,    0,
    0, 1624,    0, 1624, 1624,    0,    0, 1624, 1624, 1624,
    0,    0, 1624,    0, 1624,    0,    0, 1624, 1624, 1624,
 1624,    0, 1624,    0, 1624,    0, 1624, 1624, 1624, 1624,
 1624,    0,    0, 1624, 1624, 1624,    0, 1624, 1624,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1624, 1624,
 1624,    0, 1624, 1624, 1624,    0,    0,    0,    0,    0,
 1624,    0,    0,    0, 1624,    0, 1624, 1624, 1624, 1624,
    0, 1624,    0, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1624,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1624, 1624,    0, 1624, 1624, 1624,    0,    0,    0,    0,
 1624, 1624, 1624, 1624, 1624,    0, 1624,    0,    0, 1624,
 1624,    0,    0,    0, 1624, 1624,    0,    0, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624, 1624,    0, 1624,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,    0,
 1624, 1624, 1624, 1624, 1624, 1624,    0, 1624, 1624, 1624,
 1624,    0, 1624,    0, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624,    0, 1624, 1624, 1624, 1624,    0, 1624, 1624,
 1624, 1624, 1624,    0, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624, 1624,    0, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624,    0, 1624, 1624,    0,    0,    0, 1624,
 1624, 1624, 1624, 1624, 1624,    0,    0, 1624, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624,    0,    0,    0,    0,
    0, 1624,    0, 1624, 1624, 1624, 1624, 1624, 1624,    0,
 1624,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1624, 1624,    0,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,    0,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624,    0, 1624, 1624, 1624,    0, 1624,    0,    0,
    0, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624, 1624,    0, 1624, 1624,    0,    0, 1626,
    0,    0, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
    0,    0, 1624,    0, 1624, 1624, 1624, 1624, 1624, 1624,
 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624, 1624,
 1626,    0,    0, 1626,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1626, 1626,
 1626, 1626,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1626,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1626,    0, 1626,    0,
    0, 1626,    0, 1626, 1626,    0,    0, 1626, 1626,    0,
 1626,    0,    0,    0,    0,    0,    0,    0, 1626, 1626,
    0, 1626,    0, 1626,    0,    0, 1626,    0,    0, 1626,
    0,    0,    0,    0,    0,    0, 1626,    0, 1626, 1626,
    0, 1626, 1626, 1626,    0, 1626,    0,    0, 1626,    0,
    0, 1626,    0, 1626, 1626,    0,    0, 1626, 1626, 1626,
    0, 1626, 1626, 1626, 1626, 1626, 1626, 1626,    0,    0,
    0, 1626,    0, 1626, 1626, 1626,    0,    0,    0, 1626,
    0,    0,    0,    0,    0,    0,    0, 1626, 1626, 1626,
 1626, 1626, 1626,    0,    0,    0, 1626, 1626,    0, 1626,
    0, 1626,    0, 1626, 1626, 1626,    0, 1626,    0, 1626,
 1626, 1626,    0, 1626, 1626, 1626, 1626, 1626,    0, 1626,
 1626,    0,    0,    0,    0, 1626,    0, 1626, 1626,    0,
    0, 1626, 1626, 1626,    0,    0, 1626,    0, 1626,    0,
    0, 1626, 1626, 1626, 1626,    0, 1626,    0, 1626,    0,
 1626, 1626, 1626, 1626, 1626,    0,    0, 1626, 1626, 1626,
    0, 1626, 1626,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1626, 1626, 1626,    0, 1626, 1626, 1626,    0,
    0,    0,    0,    0, 1626,    0,    0,    0, 1626,    0,
 1626, 1626, 1626, 1626,    0, 1626,    0, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1626,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1626, 1626,    0, 1626, 1626, 1626,
    0,    0,    0,    0, 1626, 1626, 1626, 1626, 1626,    0,
 1626,    0,    0, 1626, 1626,    0,    0,    0, 1626, 1626,
    0,    0, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
 1626,    0, 1626,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626,    0, 1626, 1626, 1626, 1626, 1626, 1626,
    0, 1626, 1626, 1626, 1626,    0, 1626,    0, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626,    0, 1626, 1626, 1626,
 1626,    0, 1626, 1626, 1626, 1626, 1626,    0, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,    0, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626,    0, 1626, 1626,
    0,    0,    0, 1626, 1626, 1626, 1626, 1626, 1626,    0,
    0, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
    0,    0,    0,    0,    0, 1626,    0, 1626, 1626, 1626,
 1626, 1626, 1626,    0, 1626,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1626, 1626,    0, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626,    0, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626,    0, 1626, 1626, 1626,
    0, 1626,    0,    0,    0, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,    0, 1626,
 1626,    0,    0, 1628,    0,    0, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626,    0,    0, 1626,    0, 1626, 1626,
 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626, 1626,
 1626, 1626, 1626, 1626, 1628,    0,    0, 1628,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1628, 1628, 1628, 1628,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1628,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1628,    0, 1628,    0,    0, 1628,    0, 1628, 1628,    0,
    0, 1628, 1628,    0, 1628,    0,    0,    0,    0,    0,
    0,    0, 1628, 1628,    0, 1628,    0, 1628,    0,    0,
 1628,    0,    0, 1628,    0,    0,    0,    0,    0,    0,
 1628,    0, 1628, 1628,    0, 1628, 1628, 1628,    0, 1628,
    0,    0, 1628,    0,    0, 1628,    0, 1628, 1628,    0,
    0, 1628, 1628, 1628,    0, 1628, 1628, 1628, 1628, 1628,
 1628, 1628,    0,    0,    0, 1628,    0, 1628, 1628, 1628,
    0,    0,    0, 1628,    0,    0,    0,    0,    0,    0,
    0, 1628, 1628, 1628, 1628, 1628, 1628,    0,    0,    0,
 1628, 1628,    0, 1628,    0, 1628,    0, 1628, 1628, 1628,
    0, 1628,    0, 1628, 1628, 1628,    0, 1628, 1628, 1628,
 1628, 1628,    0, 1628, 1628,    0,    0,    0,    0, 1628,
    0, 1628, 1628,    0,    0, 1628, 1628, 1628,    0,    0,
 1628,    0, 1628,    0,    0, 1628, 1628, 1628, 1628,    0,
 1628,    0, 1628,    0, 1628, 1628, 1628, 1628, 1628,    0,
    0, 1628, 1628, 1628,    0, 1628, 1628,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1628, 1628, 1628,    0,
 1628, 1628, 1628,    0,    0,    0,    0,    0, 1628,    0,
    0,    0, 1628,    0, 1628, 1628, 1628, 1628,    0, 1628,
    0, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1628, 1628,
    0, 1628, 1628, 1628,    0,    0,    0,    0, 1628, 1628,
 1628, 1628, 1628,    0, 1628,    0,    0, 1628, 1628,    0,
    0,    0, 1628, 1628,    0,    0, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628, 1628,    0, 1628,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628,    0, 1628, 1628,
 1628, 1628, 1628, 1628,    0, 1628, 1628, 1628, 1628,    0,
 1628,    0, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
    0, 1628, 1628, 1628, 1628,    0, 1628, 1628, 1628, 1628,
 1628,    0, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628, 1628,    0, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628,    0, 1628, 1628,    0,    0,    0, 1628, 1628, 1628,
 1628, 1628, 1628,    0,    0, 1628, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628,    0,    0,    0,    0,    0, 1628,
    0, 1628, 1628, 1628, 1628, 1628, 1628,    0, 1628,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1628, 1628,    0, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628,    0, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
    0, 1628, 1628, 1628,    0, 1628,    0,    0,    0, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628, 1628,    0, 1628, 1628,    0,    0, 1584,    0,    0,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,    0,    0,
 1628,    0, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628,
 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1628, 1584,    0,
    0, 1584,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1584,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1584,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1584,    0, 1584,    0,    0, 1584,
    0, 1584, 1584,    0,    0, 1584, 1584,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1584, 1584,    0, 1584,
    0,    0,    0,    0, 1584,    0,    0, 1584,    0,    0,
    0,    0,    0,    0, 1584,    0, 1584, 1584,    0, 1584,
 1584, 1584, 1584, 1584,    0,    0, 1584,    0,    0, 1584,
    0, 1584, 1584,    0,    0,    0, 1584, 1584,    0, 1584,
 1584, 1584,    0, 1584, 1584, 1584,    0,    0,    0, 1584,
    0, 1584, 1584, 1584,    0,    0,    0, 1584,    0,    0,
    0,    0,    0,    0,    0, 1584, 1584,    0, 1584,    0,
 1584,    0,    0,    0, 1584, 1584,    0, 1584,    0,    0,
    0, 1584,    0,    0,    0, 1584,    0, 1584, 1584, 1584,
    0, 1584, 1584, 1584, 1584, 1584,    0, 1584, 1584,    0,
    0,    0,    0, 1584,    0, 1584, 1584,    0,    0, 1584,
 1584, 1584,    0,    0, 1584,    0, 1584,    0,    0, 1584,
 1584, 1584, 1584,    0, 1584,    0, 1584,    0, 1584,    0,
 1584, 1584, 1584,    0,    0, 1584, 1584, 1584,    0, 1584,
 1584,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1584, 1584, 1584,    0, 1584, 1584, 1584,    0,    0,    0,
    0,    0, 1584,    0,    0,    0, 1584,    0, 1584, 1584,
 1584, 1584,    0, 1584,    0, 1584, 1584, 1584, 1584, 1584,
 1584, 1584, 1584,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1584, 1584,    0, 1584, 1584, 1584,    0,    0,
    0,    0, 1584, 1584, 1584, 1584, 1584,    0, 1584,    0,
    0, 1584, 1584,    0,    0,    0, 1584, 1584,    0,    0,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,    0,
 1584,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584,    0, 1584, 1584, 1584, 1584, 1584, 1584,    0, 1584,
 1584, 1584, 1584,    0, 1584,    0, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584,    0, 1584, 1584, 1584, 1584,    0,
 1584, 1584, 1584, 1584, 1584,    0, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,    0,
 1584, 1584, 1584, 1584, 1584, 1584,    0, 1584, 1584, 1584,
 1584, 1584, 1584, 1584, 1584,    0, 1584, 1584,    0,    0,
    0, 1584, 1584, 1584, 1584, 1584, 1584,    0,    0, 1584,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,    0,    0,
    0,    0,    0, 1584,    0, 1584, 1584, 1584, 1584, 1584,
 1584,    0, 1584,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1584,
 1584,    0, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584,    0, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584,    0, 1584, 1584, 1584,    0, 1584,
    0,    0,    0, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584, 1584, 1584,    0, 1584, 1584,    0,
    0, 1583,    0,    0, 1584, 1584, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584, 1584,    0,    0, 1584,    0, 1584, 1584, 1584, 1584,
 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584, 1584,
 1584, 1584, 1583,    0,    0, 1583,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1583,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1583,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1583,    0,
 1583,    0,    0, 1583,    0, 1583, 1583,    0,    0, 1583,
 1583,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1583, 1583,    0, 1583,    0,    0,    0,    0, 1583,    0,
    0, 1583,    0,    0,    0,    0,    0,    0, 1583,    0,
 1583, 1583,    0, 1583, 1583, 1583, 1583, 1583,    0,    0,
 1583,    0,    0, 1583,    0, 1583, 1583,    0,    0,    0,
 1583, 1583,    0, 1583, 1583, 1583,    0, 1583, 1583, 1583,
    0,    0,    0, 1583,    0, 1583, 1583, 1583,    0,    0,
    0, 1583,    0,    0,    0,    0,    0,    0,    0, 1583,
 1583,    0, 1583,    0, 1583,    0,    0,    0, 1583, 1583,
    0, 1583,    0,    0,    0,    0,    0,    0,    0, 1583,
    0, 1583, 1583, 1583,    0, 1583, 1583, 1583, 1583, 1583,
    0, 1583, 1583,    0,    0,    0,    0, 1583,    0, 1583,
 1583,    0,    0, 1583, 1583, 1583,    0,    0, 1583,    0,
 1583,    0,    0, 1583, 1583, 1583, 1583,    0, 1583,    0,
 1583,    0, 1583,    0, 1583, 1583, 1583,    0,    0, 1583,
 1583, 1583,    0, 1583, 1583,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1583, 1583, 1583,    0, 1583, 1583,
 1583,    0,    0,    0,    0,    0, 1583,    0,    0,    0,
 1583,    0, 1583, 1583, 1583, 1583,    0, 1583,    0, 1583,
 1583, 1583, 1583, 1583, 1583, 1583, 1583,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1583, 1583,    0, 1583,
 1583, 1583,    0,    0,    0,    0, 1583, 1583, 1583, 1583,
 1583,    0, 1583,    0,    0, 1583, 1583,    0,    0,    0,
 1583, 1583,    0,    0, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583, 1583,    0, 1583,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583,    0, 1583, 1583, 1583, 1583,
 1583, 1583,    0, 1583, 1583, 1583, 1583,    0, 1583,    0,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,    0, 1583,
 1583, 1583, 1583,    0, 1583, 1583, 1583, 1583, 1583,    0,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583, 1583,    0, 1583, 1583, 1583, 1583, 1583, 1583,
    0, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,    0,
 1583, 1583,    0,    0,    0, 1583, 1583, 1583, 1583, 1583,
 1583,    0,    0, 1583, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583,    0,    0,    0,    0,    0, 1583,    0, 1583,
 1583, 1583, 1583, 1583, 1583,    0, 1583,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1583, 1583,    0, 1583, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583,    0, 1583, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,    0, 1583,
 1583, 1583,    0, 1583,    0,    0,    0, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,
    0, 1583, 1583,    0,    0, 1582,    0,    0, 1583, 1583,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583, 1583,    0,    0, 1583,    0,
 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583, 1583,
 1583, 1583, 1583, 1583, 1583, 1583, 1582,    0,    0, 1582,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1582,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1582,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1582,    0, 1582,    0,    0,    0,    0, 1582,
 1582,    0,    0, 1582, 1582,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1582, 1582,    0, 1582,    0,    0,
    0,    0, 1582,    0,    0, 1582,    0,    0,    0,    0,
    0,    0, 1582,    0, 1582, 1582,    0, 1582, 1582, 1582,
 1582, 1582,    0,    0, 1582,    0,    0, 1582,    0, 1582,
 1582,    0,    0,    0, 1582, 1582,    0, 1582, 1582, 1582,
    0, 1582, 1582, 1582,    0,    0,    0, 1582,    0, 1582,
 1582, 1582,    0,    0,    0, 1582,    0,    0,    0,    0,
    0,    0,    0, 1582, 1582,    0, 1582,    0, 1582,    0,
    0,    0, 1582, 1582,    0, 1582,    0,    0,    0,    0,
    0,    0,    0, 1582,    0, 1582, 1582, 1582,    0, 1582,
 1582, 1582, 1582, 1582,    0, 1582, 1582,    0,    0,    0,
    0, 1582,    0, 1582, 1582,    0,    0, 1582, 1582, 1582,
    0,    0, 1582,    0, 1582,    0,    0, 1582, 1582, 1582,
 1582,    0, 1582,    0, 1582,    0, 1582,    0, 1582, 1582,
 1582,    0,    0, 1582, 1582, 1582,    0, 1582, 1582,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1582, 1582,
 1582,    0, 1582, 1582, 1582,    0,    0,    0,    0,    0,
 1582,    0,    0,    0, 1582,    0, 1582, 1582, 1582, 1582,
    0, 1582,    0, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
 1582,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1582, 1582,    0, 1582, 1582, 1582,    0,    0,    0,    0,
 1582, 1582, 1582, 1582, 1582,    0, 1582,    0,    0, 1582,
 1582,    0,    0,    0, 1582, 1582,    0,    0, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582, 1582,    0, 1582,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,    0,
 1582, 1582, 1582, 1582, 1582, 1582,    0, 1582, 1582, 1582,
 1582,    0, 1582,    0, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582,    0, 1582, 1582, 1582, 1582,    0, 1582, 1582,
 1582, 1582, 1582,    0, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582, 1582,    0, 1582, 1582,
 1582, 1582, 1582, 1582,    0, 1582, 1582, 1582, 1582, 1582,
 1582, 1582, 1582,    0, 1582, 1582,    0,    0,    0, 1582,
 1582, 1582, 1582, 1582, 1582,    0,    0, 1582, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582,    0,    0,    0,    0,
    0, 1582,    0, 1582, 1582, 1582, 1582, 1582, 1582,    0,
 1582,    0,    0,    0,    0, 1410,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1582, 1582,    0,
 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,    0,
 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582,    0, 1582, 1582, 1582, 1410, 1582,    0, 1410,
    0, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582, 1582, 1582, 1410, 1582, 1582,    0,    0,    0,
    0,    0, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
    0,    0, 1582,    0, 1582, 1582, 1582, 1582, 1582, 1582,
 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582, 1582,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1410,    0, 1410,    0,    0,    0,    0, 1410,
 1410,    0,    0, 1410, 1410,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1410, 1410,    0, 1410,    0, 1410,
    0,    0, 1410,    0,    0, 1410,    0,    0,    0,    0,
    0,    0, 1410,    0, 1410, 1410,    0,    0,    0, 1410,
    0, 1410,    0,    0, 1410,    0,    0, 1410,    0,    0,
 1410,    0,    0,    0, 1410, 1410,    0, 1410, 1410, 1410,
    0, 1410, 1410, 1410,    0,    0,    0, 1410,    0, 1410,
 1410, 1410,    0,    0,    0, 1410,    0,    0,    0,    0,
    0,    0,    0, 1410, 1410,    0, 1410,    0, 1410,    0,
    0,    0, 1410, 1410,    0, 1410,    0,    0,    0,    0,
    0,    0,    0, 1410,    0, 1410,    0, 1410,    0, 1410,
 1410, 1410, 1410, 1410,    0, 1410, 1410,    0,    0,    0,
    0, 1410,    0, 1410, 1410,    0,    0, 1410, 1410,    0,
    0,    0, 1410,    0, 1410,    0,    0, 1410, 1410, 1410,
 1410,    0, 1410,    0, 1410,    0, 1410, 1410, 1410, 1410,
 1410,    0,    0,    0,    0, 1410,    0, 1410, 1410,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1410,    0,
 1410,    0, 1410, 1410, 1410,    0,    0,    0,    0,    0,
 1410,    0,    0,    0, 1410,    0, 1410, 1410, 1410, 1410,
    0, 1410,    0,    0,    0,    0,    0,    0,    0, 1410,
 1410,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1410, 1410,    0, 1410, 1410, 1410,    0,    0,    0,    0,
 1410, 1410, 1410, 1410, 1410,    0, 1410,    0,    0, 1410,
 1410,    0,    0,    0, 1410, 1410,    0,    0, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410, 1410,    0, 1410,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,    0,
 1410, 1410, 1410, 1410, 1410, 1410,    0, 1410, 1410, 1410,
 1410,    0, 1410,    0, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410,    0, 1410, 1410, 1410, 1410,    0, 1410, 1410,
 1410, 1410, 1410,    0, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410, 1410,    0, 1410, 1410,
 1410, 1410, 1410, 1410,    0, 1410, 1410, 1410, 1410, 1410,
 1410, 1410, 1410,    0, 1410, 1410,    0,    0,    0, 1410,
 1410, 1410, 1410, 1410, 1410,    0,    0, 1410, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410,    0,    0,    0,    0,
    0, 1410,    0, 1410, 1410, 1410, 1410, 1410, 1410,    0,
 1410,    0,    0,    0,    0, 1407,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1410, 1410,    0,
 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,    0,
 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410,    0, 1410, 1410, 1410, 1407, 1410,    0, 1407,
    0, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410, 1410, 1410, 1407, 1410, 1410,    0,    0,    0,
    0,    0, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410,    0,    0, 1410, 1410,
    0,    0, 1410,    0, 1410, 1410, 1410, 1410, 1410, 1410,
 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410, 1410,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1375,    0, 1375,    0,    0,    0,    0,    0,
 1375,    0,    0, 1375, 1375,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1375, 1375,    0, 1375,    0, 1375,
    0,    0, 1375,    0,    0, 1375,    0,    0,    0,    0,
    0,    0, 1375,    0, 1375, 1375,    0,    0,    0, 1375,
    0, 1375,    0,    0, 1375,    0,    0, 1407,    0,    0,
 1407,    0,    0,    0, 1375, 1375,    0, 1407, 1407, 1375,
    0, 1375, 1407, 1375,    0,    0,    0, 1407,    0, 1375,
 1407, 1375,    0,    0,    0, 1375,    0,    0,    0,    0,
    0,    0,    0, 1375, 1407,    0, 1375,    0, 1375,    0,
    0,    0, 1375, 1375,    0, 1407,    0,    0,    0,    0,
    0,    0,    0, 1407,    0, 1375,    0, 1407,    0, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1407,    0,    0,    0,
    0, 1375,    0, 1375, 1375,    0,    0, 1375, 1375,    0,
    0,    0, 1407,    0, 1375,    0,    0, 1375, 1375, 1375,
 1375,    0, 1375,    0, 1375,    0, 1375, 1375, 1375, 1375,
 1375,    0,    0,    0,    0, 1375,    0, 1375, 1407,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1375,    0,
 1407,    0, 1407, 1375, 1375,    0,    0,    0,    0,    0,
 1375,    0,    0,    0, 1375,    0, 1375, 1375, 1375, 1375,
    0, 1375,    0,    0,    0,    0,    0,    0,    0, 1375,
 1375,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1375, 1375,    0, 1375, 1375, 1375,    0,    0,    0,    0,
 1375, 1375, 1407, 1375, 1375,    0, 1375,    0,    0, 1375,
 1375,    0,    0,    0, 1375, 1375,    0,    0, 1407, 1407,
 1375, 1375, 1375, 1407, 1375, 1375, 1375,    0, 1375,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0,
 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375,
 1375,    0, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375,    0, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375,    0, 1375, 1375,    0,    0,    0, 1375,
 1375, 1375, 1375, 1375, 1375,    0,    0, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375,    0,    0,    0,    0,
    0, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375,    0,
 1407,    0,    0,    0,    0, 1409,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1375, 1375,    0,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375,    0, 1375, 1375, 1375, 1409, 1375,    0, 1409,
    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1409, 1375, 1375,    0,    0,    0,
    0,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375,    0,    0, 1375, 1375,
    0,    0, 1407,    0, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1375,    0, 1375,    0,    0,    0,    0,    0,
 1375,    0,    0, 1375, 1375,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1375, 1375,    0, 1375,    0, 1375,
    0,    0, 1375,    0,    0, 1375,    0,    0,    0,    0,
    0,    0, 1375,    0, 1375, 1375,    0,    0,    0, 1375,
    0, 1375,    0,    0, 1375,    0,    0, 1409,    0,    0,
 1409,    0,    0,    0, 1375, 1375,    0, 1409, 1409, 1375,
    0, 1375, 1409, 1375,    0,    0,    0, 1409,    0, 1375,
 1409, 1375,    0,    0,    0, 1375,    0,    0,    0,    0,
    0,    0,    0, 1375, 1409,    0, 1375,    0, 1375,    0,
    0,    0, 1375, 1375,    0, 1409,    0,    0,    0,    0,
    0,    0,    0, 1409,    0, 1375,    0, 1409,    0, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1409,    0,    0,    0,
    0, 1375,    0, 1375, 1375,    0,    0, 1375, 1375,    0,
    0,    0, 1409,    0, 1375,    0,    0, 1375, 1375, 1375,
 1375,    0, 1375,    0, 1375,    0, 1375, 1375, 1375, 1375,
 1375,    0,    0,    0,    0, 1375,    0, 1375, 1409,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1375,    0,
 1409,    0, 1409, 1375, 1375,    0,    0,    0,    0,    0,
 1375,    0,    0,    0, 1375,    0, 1375, 1375, 1375, 1375,
    0, 1375,    0,    0,    0,    0,    0,    0,    0, 1375,
 1375,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1375, 1375,    0, 1375, 1375, 1375,    0,    0,    0,    0,
 1375, 1375, 1409, 1375, 1375,    0, 1375,    0,    0, 1375,
 1375,    0,    0,    0, 1375, 1375,    0,    0, 1409, 1409,
 1375, 1375, 1375, 1409, 1375, 1375, 1375,    0, 1375,    0,
    0,    0,    0, 1414,    0,    0,    0,    0,    0, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0,
 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375,
 1375,    0, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1414, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375,    0, 1375, 1375,    0,    0,    0, 1375,
 1375, 1375, 1375, 1375, 1375,    0,    0, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375,    0,    0,    0,    0,
    0, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375,    0,
 1409,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1375, 1375,    0,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375,    0, 1375, 1375, 1375,    0, 1375,    0,    0,
    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1375,    0,    0,    0,
    0,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375,    0,    0, 1375, 1375,
    0,    0, 1409,    0, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1375,    0, 1375,    0,    0,    0,    0,    0, 1375,    0,
    0, 1375, 1375,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1375, 1375,    0, 1375,    0, 1375,    0,    0,
 1375,    0,    0, 1375,    0,    0,    0,    0,    0,    0,
 1375,    0, 1375, 1375,    0,    0,    0, 1375,    0, 1375,
    0,    0, 1375,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1375, 1375,    0,    0,    0, 1375,    0, 1375,
    0, 1375,    0,    0,    0,    0,    0, 1375,    0, 1375,
    0,    0,    0, 1375,    0,    0,    0,    0,    0,    0,
    0, 1375,    0,    0, 1375,    0, 1375,    0,    0,    0,
 1375, 1375,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1375,    0, 1414,    0, 1375, 1375, 1375,
 1375, 1375,    0, 1375, 1414,    0,    0,    0,    0, 1375,
    0, 1375, 1375,    0,    0, 1375, 1375,    0,    0,    0,
    0,    0, 1375,    0, 1414, 1375, 1375, 1375, 1375,    0,
 1375,    0, 1375,    0, 1375, 1375, 1375, 1375, 1375,    0,
    0,    0,    0, 1375,    0, 1375,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1375,    0, 1414,    0,
    0, 1375, 1375,    0,    0,    0,    0,    0, 1375,    0,
    0,    0, 1375,    0, 1375, 1375, 1375, 1375,    0, 1375,
    0,    0,    0,    0,    0,    0,    0, 1375, 1375,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1375, 1375,
    0, 1375, 1375,    0,    0,    0,    0,    0, 1375, 1375,
 1414, 1375, 1375,    0, 1375,    0,    0, 1375, 1375,    0,
    0,    0, 1375, 1375,    0,    0,    0, 1414, 1375, 1375,
 1375,    0, 1375, 1375, 1375,    0, 1375,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375,    0,
 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
    0, 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375,
 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375,
 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375,    0, 1375, 1375,    0,    0,    0, 1375, 1375, 1375,
 1375, 1375, 1375,    0,    0, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375,    0,    0,    0,    0,    0, 1375,
    0, 1375, 1375, 1375, 1375, 1375, 1375,    0,    0,    0,
    0,    0,    0, 1373,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
    0, 1375, 1375, 1375,    0, 1375,    0, 1373,    0, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1373, 1375, 1375,    0,    0,    0,    0,    0,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375,    0,    0, 1375, 1375,    0,    0,
    0,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1375,    0, 1375,    0,    0,    0,    0,    0, 1375,    0,
    0, 1375, 1375,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1375, 1375,    0, 1375,    0,    0,    0,    0,
 1375,    0,    0, 1375,    0,    0,    0,    0,    0,    0,
 1375,    0, 1375, 1375,    0,    0,    0, 1375,    0, 1375,
    0,    0, 1375,    0,    0,    0,    0, 1373,    0,    0,
    0,    0, 1375, 1375,    0,    0,    0, 1375,    0, 1375,
    0, 1375,    0,    0,    0, 1373,    0, 1375,    0, 1375,
    0,    0,    0, 1375,    0,    0,    0,    0,    0,    0,
    0, 1375,    0,    0, 1375,    0, 1375,    0,    0,    0,
 1375, 1375,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1375,    0,    0,    0, 1375, 1375, 1375,
 1375, 1375,    0, 1375,    0,    0,    0,    0,    0, 1375,
    0, 1375, 1375,    0,    0, 1375, 1375,    0,    0,    0,
    0,    0, 1375,    0,    0, 1375, 1375, 1375, 1375,    0,
 1375,    0, 1375,    0, 1375,    0, 1375, 1375, 1375,    0,
    0,    0,    0, 1375,    0, 1375,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1375,    0,    0,    0,
    0, 1375, 1375,    0,    0,    0,    0,    0, 1375,    0,
    0,    0, 1375,    0, 1375, 1375, 1375, 1375,    0, 1375,
    0,    0,    0,    0,    0,    0,    0, 1375, 1375,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1375, 1375,
    0, 1375, 1375, 1375,    0,    0,    0,    0, 1375, 1375,
    0, 1375, 1375,    0, 1375,    0,    0, 1375, 1375,    0,
    0,    0, 1375, 1375,    0,    0,    0,    0, 1375, 1375,
 1375,    0, 1375, 1375, 1375,    0, 1375,    0,    0,    0,
    0, 1219,    0,    0,    0,    0, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375,    0,
 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1219, 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375,
 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375, 1375, 1375,
 1375, 1375,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375,    0, 1375, 1375,    0,    0,    0, 1375, 1375, 1375,
 1375, 1375, 1375,    0,    0, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375,    0,    0,    0,    0,    0, 1375,
    0, 1375, 1375, 1375, 1375, 1375, 1375,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
    0, 1375, 1375, 1375,    0, 1375,    0,    0,    0, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375,    0, 1375, 1375,    0,    0,    0,    0,    0,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375,    0,    0, 1375, 1375,    0,    0,
    0,    0, 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,
 1375, 1375, 1375, 1375, 1375, 1375, 1375, 1375,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1206,    0,
 1206,    0,    0,    0,    0,    0, 1206,    0,    0, 1206,
 1206,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1206, 1206,    0, 1206,    0, 1206,    0,    0, 1206,    0,
    0, 1206,    0,    0,    0,    0,    0,    0, 1206,    0,
 1206, 1206,    0,    0,    0, 1206,    0, 1206,    0,    0,
 1206,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1206, 1206,    0,    0,    0, 1206,    0, 1206,    0, 1206,
    0,    0,    0,    0,    0, 1206,    0, 1206,    0,    0,
    0, 1206,    0,    0,    0,    0,    0,    0,    0, 1206,
    0,    0, 1206,    0, 1206,    0,    0,    0, 1206, 1206,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1206,    0,    0,    0, 1206, 1206, 1206, 1206, 1206,
    0, 1206,    0,    0,    0,    0,    0, 1206,    0, 1206,
 1206,    0,    0, 1206, 1206,    0,    0,    0,    0,    0,
 1206,    0,    0, 1206, 1206, 1206, 1206,    0, 1206,    0,
 1206,    0, 1206, 1206, 1206, 1206, 1206,    0,    0,    0,
    0, 1206,    0, 1206,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1206,    0,    0,    0,    0, 1206,
 1206,    0,    0,    0,    0,    0, 1206,    0,    0,    0,
 1206,    0, 1206, 1206, 1206, 1206,    0, 1206,    0,    0,
    0,    0,    0,    0,    0, 1206, 1206,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1206, 1206,    0, 1206,
 1206, 1206,    0,    0,    0,    0, 1206, 1206,    0, 1206,
 1206,    0, 1206,    0,    0, 1206, 1206,    0,    0,    0,
 1206, 1206,    0,    0,    0,    0, 1206, 1206, 1206,    0,
 1206, 1206, 1206,    0, 1206,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1206, 1206, 1206, 1206, 1206, 1206,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,
 1206, 1206, 1206, 1206, 1206,    0, 1206, 1206, 1206, 1206,
 1206, 1206,    0, 1206, 1206, 1206, 1206,    0, 1206,    0,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,    0, 1206,
 1206, 1206, 1206,    0, 1206, 1206, 1206, 1206, 1206,    0,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,
 1206, 1206, 1206,    0, 1206, 1206, 1206, 1206, 1206, 1206,
    0, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,    0,
 1206, 1206,    0,    0,    0, 1206, 1206, 1206, 1206, 1206,
 1206,    0,    0, 1206, 1206, 1206, 1206, 1206, 1206, 1206,
 1206, 1206,    0, 1341,    0,    0, 1044, 1206, 1045, 1206,
 1206, 1206, 1206, 1206, 1206,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0, 1206, 1206,    0, 1206, 1206, 1206, 1206,
 1206, 1206, 1206, 1206, 1206,    0, 1206, 1206, 1206, 1206,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,    0, 1206,
 1206, 1206,    0, 1206,    0,    0,    0, 1206, 1206, 1206,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,
    0, 1206, 1206,    0,    0,    0,    0,    0, 1206, 1206,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,
 1206, 1206,    0,    0, 1206, 1206,    0,    0,    0,    0,
 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206, 1206,
 1206, 1206, 1206, 1206, 1206, 1206,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900, 1743, 1744, 1745, 1746, 1747, 1748, 1780, 1781,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
 1782,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973, 1783,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412, 1753, 1754, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425, 1784,  427, 1785, 1042, 1043, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
 1775,  246,    0,  922,  923,  924, 1776,  925,  926, 1777,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158, 1868,    0,    0, 1869,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205, 1870,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0, 3817,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166, 3818,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414, 3819,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166, 3912,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414, 3913,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166, 3998,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414, 3999,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1357,    0, 1357, 1357,    0, 1357,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1357,    0,    0,    0,    0, 1357,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1357,    0, 1357, 1357,    0,    0,    0,    0, 1357,    0,
 1357, 1357, 1357,    0,    0,    0,    0, 1357,    0,    0,
    0,    0, 1357, 1357,    0, 1357,    0, 1357, 1357,    0,
 1357,    0,    0, 1357, 1357, 1357,    0,    0,    0,    0,
 1357, 1357, 1357, 1357,    0,    0,    0, 1357,    0, 1357,
    0, 1357, 1357,    0,    0,    0,    0,    0,    0,    0,
 1357,    0, 1357, 1357,    0,    0,    0, 1357,    0, 1357,
    0, 1357,    0,    0,    0,    0,    0, 1357,    0, 1357,
    0,    0,    0, 1357,    0,    0,    0, 1357,    0, 1357,
 1357, 1357, 1357,    0, 1357,    0, 1357, 1357,    0, 1357,
 1357, 1357,    0,    0,    0,    0,    0, 1357,    0,    0,
    0,    0,    0, 1357,    0,    0,    0, 1357, 1357, 1357,
 1357, 1357,    0, 1357,    0, 1357,    0,    0,    0, 1357,
    0, 1357, 1357,    0,    0, 1357, 1357,    0,    0,    0,
 1357,    0, 1357,    0,    0, 1357, 1357, 1357, 1357,    0,
 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
    0,    0,    0, 1357,    0, 1357,    0,    0,    0, 1357,
    0,    0,    0,    0,    0,    0, 1357,    0,    0,    0,
    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357,    0, 1357, 1357, 1357, 1357,    0, 1357,
 1357,    0,    0,    0,    0,    0,    0, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
    0, 1357, 1357,    0, 1357, 1357,    0, 1357, 1357, 1357,
    0, 1357, 1357, 1357, 1357,    0,    0, 1357, 1357, 1357,
 1357, 1357, 1357, 1357,    0,    0,    0,    0, 1357, 1357,
 1357,    0, 1357, 1357, 1357,    0, 1357,    0, 1357, 1357,
 1357,    0, 1357, 1357,    0, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357,    0, 1357, 1357,
 1357, 1357, 1357, 1357,    0, 1357, 1357, 1357, 1357,    0,
 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
    0, 1357, 1357, 1357, 1357,    0, 1357, 1357, 1357, 1357,
 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357,    0, 1357, 1357, 1357, 1357,
 1357, 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357,    0, 1357, 1357, 1357, 1357,    0, 1357, 1357, 1357,
 1357, 1357, 1357,    0,    0, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357,    0, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357,    0,    0, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357,    0, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357,    0, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
    0, 1357, 1357, 1357,    0, 1357,    0, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357,    0,    0, 1357, 1357,    0,    0,
    0,    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166, 1340,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214, 1372,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1341,    0,
 1587, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341, 1891,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0, 2202,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0, 2394,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0, 1867,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341, 2689,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0, 2768,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
 3123,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1355,    0,
 1355, 1355,    0, 1355,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1355,    0,    0,    0,
    0, 1355,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1355,    0, 1355, 1355,    0,
    0,    0,    0, 1355,    0, 1355, 1355, 1355,    0,    0,
    0,    0, 1355,    0,    0,    0,    0, 1355, 1355,    0,
 1355,    0, 1355, 1355,    0, 1355,    0,    0, 1355, 1355,
 1355,    0,    0,    0,    0, 1355, 1355, 1355, 1355,    0,
    0,    0, 1355,    0, 1355,    0, 1355, 1355,    0,    0,
    0,    0,    0,    0,    0, 1355,    0, 1355, 1355,    0,
    0,    0, 1355,    0, 1355,    0, 1355,    0,    0,    0,
    0,    0, 1355,    0, 1355,    0,    0,    0, 1355,    0,
    0,    0, 1355,    0, 1355, 1355, 1355, 1355,    0, 1355,
    0, 1355, 1355,    0, 1355, 1355, 1355,    0,    0,    0,
    0,    0, 1355,    0,    0,    0,    0,    0, 1355,    0,
    0,    0, 1355, 1355, 1355, 1355, 1355,    0, 1355,    0,
 1355,    0,    0,    0, 1355,    0, 1355, 1355,    0,    0,
 1355, 1355,    0,    0,    0, 1355,    0, 1355,    0,    0,
 1355, 1355, 1355, 1355,    0, 1355,    0, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355,    0,    0,    0, 1355,    0,
 1355,    0,    0,    0, 1355,    0,    0,    0,    0,    0,
    0, 1355,    0,    0,    0,    0, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,    0, 1355,
 1355, 1355, 1355,    0, 1355, 1355,    0,    0,    0,    0,
    0,    0, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355,    0, 1355,    0,    0, 1355,
 1355,    0, 1355, 1355, 1355,    0, 1355, 1355, 1355, 1355,
    0,    0, 1355, 1355, 1355, 1355, 1355, 1355, 1355,    0,
    0,    0,    0, 1355, 1355, 1355,    0, 1355, 1355, 1355,
    0, 1355,    0, 1355, 1355, 1355,    0, 1355, 1355,    0,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355,    0, 1355, 1355, 1355, 1355, 1355, 1355,    0,
 1355, 1355, 1355, 1355,    0, 1355,    0, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355,    0, 1355, 1355, 1355, 1355,
    0, 1355, 1355, 1355, 1355, 1355,    0, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
    0, 1355, 1355, 1355, 1355, 1355, 1355,    0, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355,    0, 1355, 1355, 1355,
 1355,    0, 1355, 1355, 1355, 1355, 1355, 1355,    0,    0,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
    0, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355,    0,    0, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355,    0, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355,    0, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355,    0, 1355, 1355, 1355,    0,
 1355,    0, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355,    0, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,    0,
    0, 1355, 1355,    0,    0,    0,    0, 1355, 1355, 1355,
 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355, 1355,
 1355, 1355, 1355, 2097,    0, 2097, 2097,    0, 2097,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2097,    0,    0,    0,    0, 2097,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2097,    0, 2097,    0,    0,    0,    0,    0, 2097,    0,
 2097, 2097, 2097,    0,    0,    0,    0, 2097,    0,    0,
    0,    0, 2097, 2097,    0, 2097,    0, 2097, 2097,    0,
 2097,    0,    0, 2097, 2097, 2097,    0,    0,    0,    0,
 2097,    0, 2097, 2097,    0,    0,    0, 2097,    0, 2097,
    0, 2097, 2097,    0,    0,    0,    0,    0,    0,    0,
 2097,    0, 2097, 2097,    0,    0,    0, 2097,    0, 2097,
    0, 2097,    0,    0,    0,    0,    0, 2097,    0, 2097,
    0,    0,    0, 2097,    0,    0,    0, 2097,    0, 2097,
 2097, 2097, 2097,    0, 2097,    0, 2097, 2097,    0, 2097,
 2097, 2097,    0,    0,    0,    0,    0, 2097,    0,    0,
    0,    0,    0, 2097,    0,    0,    0, 2097, 2097, 2097,
 2097, 2097,    0, 2097,    0, 2097,    0,    0,    0, 2097,
    0, 2097, 2097,    0,    0, 2097, 2097,    0,    0,    0,
 2097,    0, 2097,    0,    0, 2097, 2097, 2097, 2097,    0,
 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
    0,    0,    0, 2097,    0, 2097,    0,    0,    0, 2097,
    0,    0,    0,    0,    0,    0, 2097,    0,    0,    0,
    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,    0, 2097,
 2097,    0,    0,    0,    0,    0,    0, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
    0, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097,
    0, 2097, 2097, 2097, 2097,    0,    0, 2097, 2097, 2097,
 2097, 2097, 2097, 2097,    0,    0,    0,    0, 2097, 2097,
 2097,    0, 2097, 2097, 2097,    0, 2097,    0, 2097, 2097,
 2097,    0, 2097, 2097,    0, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097,
 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,    0,
 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
    0, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,
 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,
 2097, 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097,    0, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097,
 2097, 2097, 2097,    0,    0, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0,    0, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
    0, 2097, 2097, 2097,    0, 2097,    0, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097,    0,    0, 2097, 2097,    0,    0,
    0,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 1046,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 2621,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 3613,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 1341,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,  877,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,  886,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  889,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,  917,  918,  919,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370, 4048,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,    0,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,    0, 1028,  398,
 1029, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043, 1341,    0,    0, 1044,    0, 1045,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1047,    0,    0,    0,    0, 1048,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0,    0,    0,    0,    0,  159,    0,
  872,  160,  161,    0,    0,    0,    0,  873,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,  874,    0,
  166,    0,    0,  167,  875,  876,    0,    0,    0,    0,
  168,    0,  169,  170,    0,    0,    0,  171,    0,  172,
    0,  877,  173,    0,    0,    0,    0,    0,    0,    0,
  878,    0,  174,  879,    0,    0,    0,  176,    0,  177,
    0,  178,    0,    0,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,  880,    0,  881,
  882,  182,  883,    0,  183,    0,  184,  884,    0,  885,
  185,  186,    0,    0,    0,    0,    0,  886,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887,    0,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0,    0,  201,  202,  889,  204,    0,
  205,    0,  206,  890,  207,  208,  209,  210,  211,  891,
    0,    0,    0,  212,    0,  213,    0,    0,    0,  892,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
    0,  215,  216,  893,  894,  895,  896,  897,  898,  899,
  618,  619,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,  901,
  902,  903,  904,  905,  906,  907,  908,  909,  226,  910,
    0,  228,  229,  230,  911,  912,    0,  913,  914,  915,
    0,  233,  234,  916,  235,    0,    0,  236,  237,  917,
  918,  919,  238,  239,    0,    0,    0,    0,  240,  920,
  242,    0,  921,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  946,    0,  947,  948,  949,  277,    0,
  950,    0,  951,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
    0,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321, 4055,
  323,  324,    0,  976,  326,  327,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983,    0,    0,  341,  984,  985,  986,  987,
  988,  989,  990,  991,  992,    0,  993,  994,  995,  350,
  996,  351,  352,  353,  354,  355,  356,    0,    0,  997,
  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007,
 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366, 1017,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,  384,
 1024,  386,  387,  388, 1025, 1026,  391,  392,  393, 1027,
  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043, 2097,    0,
    0, 2097,    0, 2097,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2097,    0,    0,    0,
    0, 2097,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2097,    0, 2097,    0,    0,
    0,    0,    0, 2097,    0, 2097, 2097, 2097,    0,    0,
    0,    0, 2097,    0,    0,    0,    0, 2097, 2097,    0,
 2097,    0, 2097, 2097,    0, 2097,    0,    0, 2097, 2097,
 2097,    0,    0,    0,    0, 2097,    0, 2097, 2097,    0,
    0,    0, 2097,    0, 2097,    0, 2097, 2097,    0,    0,
    0,    0,    0,    0,    0, 2097,    0, 2097, 2097,    0,
    0,    0, 2097,    0, 2097,    0, 2097,    0,    0,    0,
    0,    0, 2097,    0, 2097,    0,    0,    0, 2097,    0,
    0,    0, 2097,    0, 2097, 2097, 2097, 2097,    0, 2097,
    0, 2097, 2097,    0, 2097, 2097, 2097,    0,    0,    0,
    0,    0, 2097,    0,    0,    0,    0,    0, 2097,    0,
    0,    0, 2097, 2097, 2097, 2097, 2097,    0, 2097,    0,
 2097,    0,    0,    0, 2097,    0, 2097, 2097,    0,    0,
 2097, 2097,    0,    0,    0, 2097,    0, 2097,    0,    0,
 2097, 2097, 2097, 2097,    0, 2097,    0, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097,    0,    0,    0, 2097,    0,
 2097,    0,    0,    0, 2097,    0,    0,    0,    0,    0,
    0, 2097,    0,    0,    0,    0, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0, 2097,
 2097, 2097, 2097,    0, 2097, 2097,    0,    0,    0,    0,
    0,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,
 2097,    0, 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,
    0,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0,
    0,    0,    0, 2097, 2097, 2097,    0, 2097, 2097, 2097,
    0, 2097,    0, 2097, 2097, 2097,    0, 2097, 2097,    0,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097,    0,
 2097, 2097, 2097, 2097,    0, 2097,    0, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097, 2097,
    0, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
    0, 2097, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097,
 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097,    0,    0,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097,    0,    0, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097, 2097,    0,
 2097,    0, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,    0,
    0, 2097, 2097,    0,    0,    0,    0, 2097, 2097, 2097,
 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097, 2097,
 2097, 2097, 2097, 1620,    0,    0, 1620,    0, 1620,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1620,    0,    0,    0,    0, 1620,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1620,    0, 1620,    0,    0,    0,    0,    0, 1620,    0,
 1620, 1620, 1620,    0,    0,    0,    0, 1620,    0,    0,
    0,    0, 1620, 1620,    0, 1620,    0, 1620, 1620,    0,
 1620,    0,    0, 1620, 1620, 1620,    0,    0,    0,    0,
 1620,    0, 1620, 1620,    0,    0,    0, 1620,    0, 1620,
    0, 1620, 1620,    0,    0,    0,    0,    0,    0,    0,
 1620,    0, 1620, 1620,    0,    0,    0, 1620,    0, 1620,
    0, 1620,    0,    0,    0,    0,    0, 1620,    0, 1620,
    0,    0,    0, 1620,    0,    0,    0, 1620,    0, 1620,
 1620, 1620, 1620,    0, 1620,    0, 1620, 1620,    0, 1620,
 1620, 1620,    0,    0,    0,    0,    0, 1620,    0,    0,
    0,    0,    0, 1620,    0,    0,    0, 1620, 1620, 1620,
 1620, 1620,    0, 1620,    0, 1620,    0,    0,    0, 1620,
    0, 1620, 1620,    0,    0, 1620, 1620,    0,    0,    0,
 1620,    0, 1620,    0,    0, 1620, 1620, 1620, 1620,    0,
 1620,    0, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
    0,    0,    0, 1620,    0, 1620,    0,    0,    0, 1620,
    0,    0,    0,    0,    0,    0, 1620,    0,    0,    0,
    0, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620,    0, 1620, 1620, 1620, 1620,    0, 1620,
 1620,    0,    0,    0,    0,    0,    0, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
    0, 1620, 1620, 1620, 1620, 1620,    0, 1620, 1620, 1620,
    0, 1620, 1620, 1620, 1620,    0,    0, 1620, 1620, 1620,
 1620, 1620, 1620, 1620,    0,    0,    0,    0, 1620, 1620,
 1620,    0, 1620, 1620, 1620,    0, 1620,    0, 1620, 1620,
 1620,    0, 1620, 1620,    0, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620,    0, 1620, 1620,
 1620, 1620, 1620, 1620,    0, 1620, 1620, 1620, 1620,    0,
 1620,    0, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
    0, 1620, 1620, 1620, 1620,    0, 1620, 1620, 1620, 1620,
 1620,    0, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620,    0, 1620, 1620, 1620, 1620,
 1620, 1620,    0, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620,    0, 1620, 1620, 1620, 1620,    0, 1620, 1620, 1620,
 1620, 1620, 1620,    0,    0, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620,    0, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620,    0,    0, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620,    0, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620,    0, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
    0, 1620, 1620, 1620,    0, 1620,    0, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620,    0, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620,    0,    0, 1620, 1620,    0,    0,
    0,    0, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620,
 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1620, 1427,    0,
    0, 1044,    0, 1045,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1047,    0,    0,    0,
    0, 1048,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,  872,  160,  161,    0,    0,
    0,    0,  873,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,  874,    0,  166,    0,    0,  167,  875,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,    0,  173,    0,    0,
    0,    0,    0,    0,    0,  878,    0,  174,  879,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,  880,    0,  881,  882,  182,  883,    0,  183,
    0,  184,  884,    0,  885,  185,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
  887,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,  888,    0,  200,    0,    0,
  201,  202,  203,  204,    0,  205,    0,  206,  890,  207,
  208,  209,  210,  211,  891,    0,    0,    0,  212,    0,
  213,    0,    0,    0,  892,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,  893,  894,
  895,  896,  897,  898,  899,  618,  619,  218,    0,  219,
  220,  221,  222,    0,  223,  900,    0,    0,    0,    0,
    0,    0,  224,  225,  901,  902,  903,  904,  905,  906,
  907,  908,  909,  226,  910,    0,  228,  229,  230,  911,
  912,    0,  913,  914,  915,    0,  233,  234,  916,  235,
    0,    0,  236,  237,    0,    0,    0,  238,  239,    0,
    0,    0,    0,  240,  920,  242,    0,  921,  244,  245,
    0,  246,    0,  922,  923,  924,    0,  925,  926,    0,
  927,  247,  248,  249,  250,  251,  252,  928,  254,  929,
  256,  930,  931,  932,  933,  934,  935,  936,  937,  938,
  939,  940,    0,  941,  942,  943,  944,  945,  946,    0,
  947,  948,  949,  277,    0,  950,    0,  951,  952,  953,
  954,  955,  284,  285,  956,  287,  957,  958,  959,  291,
  960,  293,  961,  295,  962,    0,  963,  964,  965,  966,
    0,  967,  968,  969,  304,  970,    0,  971,  972,  308,
  973,  310,  311,  974,  313,  314,  315,  316,  317,  318,
    0,  975,  320,  321,  322,  323,  324,    0,  976,  326,
  327,  328,  977,  978,  331,  332,    0,  333,  334,  979,
  980,    0,  335,  336,  337,  981,  982,  983,    0,    0,
  341,  984,  985,  986,  987,  988,  989,  990,  991,  992,
    0,  993,  994,  995,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,  997,  998,  999, 1000, 1001, 1002,
 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
 1013, 1014,    0,  359, 1015,  361,  362, 1016,  364,  365,
  366, 1017,    0,  368,  369,  370,  371,  372,  373, 1018,
  375, 1019, 1020,  378,  379,    0,  380,  381, 1021,  690,
  383,    0, 1022, 1023,  384, 1024,  386,  387,  388, 1025,
 1026,  391,  392,  393, 1027,  395,  396,  690, 1028,  398,
    0, 1030, 1031, 1032, 1033,  399, 1034, 1035, 1036, 1037,
 1038, 1039,  406,  407,  408,  409,  410,  411,  412,    0,
    0, 1040,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422, 1041,  424,  425,  426,  427,
  428, 1042, 1043,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  690,    0,  690,    0,
  690,    0,    0,    0,  690,    0,    0,  690,  690,  690,
    0,    0,    0,    0,    0,    0,    0,    0,  690,  690,
    0,  690,    0,  690,    0,    0,  690,    0,    0,  690,
    0,    0,    0,    0,    0,  690,  690,    0,  690,  690,
    0,    0,  690,  690,    0,  690,  690,    0,  690,    0,
    0,  690,    0,    0,    0,    0,    0,    0,  690,  690,
    0,    0,    0,  690,  690,  690,    0,  690,    0,  690,
    0,    0,    0,  690,    0,  690,    0,    0,    0,  690,
    0,    0,    0,    0,    0,  690,    0,  690,  690,    0,
  690,    0,  690,    0,  690,    0,  690,  690,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  690,
    0,    0,    0,  690,  690,  690,  690,  690,    0,  690,
    0,  690,  690,    0,    0,  690,    0,  690,  690,    0,
    0,  690,  690,    0,    0,    0,  690,    0,  690,  690,
  690,  690,  690,  690,  690,    0,  690,    0,  690,    0,
  690,  690,  690,  690,  690,    0,    0,    0,    0,  690,
    0,  690,    0,    0,  690,    0,    0,    0,    0,    0,
    0,    0,  690,  690,    0,  690,  690,  690,  690,    0,
    0,    0,    0,    0,  690,    0,    0,    0,  690,    0,
  690,  690,  690,  690,    0,  690,  690,    0,    0,    0,
    0,    0,    0,  690,  690,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  690,  690,    0,  690,  690,  690,
    0,    0,    0,    0,  690,  690,    0,  690,  690,    0,
  690,    0,    0,  690,  690,    0,    0,    0,  690,  690,
  690,  690,  690,    0,  690,  690,  690,    0,  690,  690,
  690,    0,  690,    0,  690,  690,  690,    0,  690,  690,
    0,  690,  690,  690,  690,  690,  690,  690,  690,  690,
  690,  690,  690,  690,  690,  690,  690,  690,  690,  690,
  690,  690,  690,    0,  690,  690,  690,  690,  690,  690,
 1448,  690,  690,  690,  690,    0,  690,    0,  690,  690,
  690,  690,  690,  690,  690,  690,  690,  690,  690,  690,
  690,  690,  690,  690,  690,  690,    0,  690,  690,  690,
  690,    0,  690,  690,  690,  690,  690,    0,  690,  690,
  690,  690,  690,  690,  690,  690,  690,  690,  690,  690,
  690,    0,  690,  690,  690,  690,  690,  690,    0,  690,
  690,  690,  690,  690,  690,  690,  690,    0,  690,  690,
  690,  690,    0,  690,  690,  690,  690,  690,  690,  690,
    0,  690,  690,  690,  690,  690,  690,  690,  690,  690,
    0,    0,    0,    0,    0,  690,    0,  690,  690,  690,
  690,  690,  690,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  690,  690,    0,  690,  690,  690,  690,  690,  690,
  690,  690,  690,    0,  690,  690,  690,  690,  690,  690,
  690,  690,  690,  690,  690,  690,    0,  690,  690,  690,
    0,  690,    0,    0,    0,  690,  690,  690,  690,  690,
  690,  690,  690,  690,  690,  690,  690,  690,    0,  690,
  690,    0,    0,  690,  690,  690,  690,  690,  690,  690,
  690,  690,  690,  690,  690,  690,  690,  690,  690,  690,
    0,    0,  690,  690,    0,    0,    0,    0,  690,  690,
  690,  690,  690,  690,  690,  690,  690,  690,  690,  690,
  690,  690,  690,  690,    0,    0,    0, 2290,    0, 2290,
    0,    0,    0,    0,    0, 2290,    0,    0, 2290, 2290,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2290,
 2290,    0, 2290,    0, 2290,    0,    0, 2290,    0,    0,
 2290,    0,    0,    0,    0,    0,    0, 2290,    0, 2290,
 2290,    0,    0,    0, 2290,    0, 2290,    0,    0, 2290,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2290,
 2290,    0,    0,    0, 2290,    0, 2290,    0, 2290, 2290,
    0,    0,    0,    0, 2290,    0, 2290,    0,    0,    0,
 2290,    0,    0,    0,    0,    0,    0,    0, 2290,    0,
    0, 2290,    0, 2290,    0,    0,    0, 2290, 2290,    0,
 2290,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2290,    0, 2290,    0, 2290, 2290, 2290, 2290, 2290,    0,
 2290,    0,    0,    0,    0,    0, 2290,    0, 2290, 2290,
    0,    0, 2290, 2290,    0,    0,    0,    0,    0, 2290,
    0,    0, 2290, 2290, 2290, 2290,    0, 2290,    0, 2290,
    0, 2290, 2290, 2290, 2290, 2290,    0,    0,    0,    0,
 2290,    0, 2290,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2290,    0,    0,    0,    0, 2290, 2290,
    0,    0,    0,    0,    0, 2290,    0,    0,    0, 2290,
    0, 2290, 2290, 2290, 2290,    0, 2290,    0,    0,    0,
    0,    0,    0,    0, 2290, 2290,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2290, 2290,    0, 2290, 2290,
 2290,    0,    0,    0,    0, 2290, 2290,    0, 2290, 2290,
    0, 2290,    0,    0, 2290, 2290,    0,    0,    0, 2290,
 2290,    0,    0,    0,    0, 2290, 2290, 2290,    0, 2290,
 2290, 2290,    0, 2290,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2290, 2290, 2290, 2290, 2290, 2290, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,
 2290, 2290, 2290, 2290,    0, 2290, 2290, 2290, 2290, 2290,
 2290, 1447, 2290, 2290, 2290, 2290,    0, 2290,    0, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290,    0, 2290, 2290,
 2290, 2290,    0, 2290, 2290, 2290, 2290, 2290,    0, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,
 2290, 2290,    0, 2290, 2290, 2290, 2290, 2290, 2290,    0,
 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,    0, 2290,
 2290,    0,    0,    0, 2290, 2290, 2290, 2290, 2290, 2290,
    0,    0, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,
 2290,    0,    0,    0,    0,    0, 2290,    0, 2290, 2290,
 2290, 2290, 2290, 2290,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2290, 2290,    0, 2290, 2290, 2290, 2290, 2290,
 2290, 2290, 2290, 2290,    0, 2290, 2290, 2290, 2290, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290,    0, 2290, 2290,
 2290,    0, 2290,    0,    0,    0, 2290, 2290, 2290, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,    0,
 2290, 2290,    0,    0,    0,    0,    0, 2290, 2290, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,
 2290,    0,    0, 2290, 2290,    0,    0,    0,    0, 2290,
 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290, 2290,
 2290, 2290, 2290, 2290, 2290,    0,    0,    0, 2195,    0,
 2195,    0,    0,    0,    0,    0, 2195,    0,    0, 2195,
 2195,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2195, 2195,    0, 2195,    0, 2195,    0,    0, 2195,    0,
    0, 2195,    0,    0,    0,    0,    0,    0, 2195,    0,
 2195, 2195,    0,    0,    0, 2195,    0, 2195,    0,    0,
 2195,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2195, 2195,    0,    0,    0, 2195,    0, 2195,    0, 2195,
 2195,    0,    0,    0,    0, 2195,    0, 2195,    0,    0,
    0, 2195,    0,    0,    0,    0,    0,    0,    0, 2195,
    0,    0, 2195,    0, 2195,    0,    0,    0, 2195, 2195,
    0, 2195,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2195,    0, 2195,    0, 2195, 2195, 2195, 2195, 2195,
    0, 2195,    0,    0,    0,    0,    0, 2195,    0, 2195,
 2195,    0,    0, 2195, 2195,    0,    0,    0,    0,    0,
 2195,    0,    0, 2195, 2195, 2195, 2195,    0, 2195,    0,
 2195,    0, 2195, 2195, 2195, 2195, 2195,    0,    0,    0,
    0, 2195,    0, 2195,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2195,    0,    0,    0,    0, 2195,
 2195,    0,    0,    0,    0,    0, 2195,    0,    0,    0,
 2195,    0, 2195, 2195, 2195, 2195,    0, 2195,    0,    0,
    0,    0,    0,    0,    0, 2195, 2195,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2195, 2195,    0, 2195,
 2195, 2195,    0,    0,    0,    0, 2195, 2195,    0, 2195,
 2195,    0, 2195,    0,    0, 2195, 2195,    0,    0,    0,
 2195, 2195,    0,    0,    0,    0, 2195, 2195, 2195,    0,
 2195, 2195, 2195,    0, 2195,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195,    0, 2195, 2195, 2195, 2195,
 2195, 2195, 1495, 2195, 2195, 2195, 2195,    0, 2195,    0,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,    0, 2195,
 2195, 2195, 2195,    0, 2195, 2195, 2195, 2195, 2195,    0,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195,    0, 2195, 2195, 2195, 2195, 2195, 2195,
    0, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,    0,
 2195, 2195,    0,    0,    0, 2195, 2195, 2195, 2195, 2195,
 2195,    0,    0, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195,    0,    0,    0,    0,    0, 2195,    0, 2195,
 2195, 2195, 2195, 2195, 2195,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2195, 2195,    0, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195,    0, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,    0, 2195,
 2195, 2195,    0, 2195,    0,    0,    0, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
    0, 2195, 2195,    0,    0,    0,    0,    0, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195,    0,    0, 2195, 2195,    0,    0,    0,    0,
 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195, 2195,
 2195, 2195, 2195, 2195, 2195, 2195,    0,    0,    0,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0, 1842,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349, 1047,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382, 1494,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  157,    0,  158,    0,    0,    0,    0,    0,
  159,    0,    0,  160,  161,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  162,  163,    0,  164,    0,  165,
    0,    0,  166,    0,    0,  167,    0,    0,    0,    0,
    0,    0,  168,    0,  169,  170,    0,    0,    0,  171,
    0,  172,    0,    0,  173,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  174,  175,    0,    0,    0,  176,
    0,  177,    0,  178,    0,    0,    0,    0,    0,  179,
    0,  180,    0,    0,    0,  181,    0,    0,    0,    0,
    0,    0,    0,  182,    0,    0,  183,    0,  184,    0,
    0,    0,  185,  186,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  187,    0,    0,    0,  189,
  190,  191,  192,  193,    0,  194,    0,    0,    0,    0,
    0,  195,    0,  196,  197,    0,    0,  198,  199,    0,
    0,    0,    0,    0,  200,    0,    0,  201,  202,  203,
  204,    0,  205,    0,  206,    0,  207,  208,  209,  210,
  211,    0,    0,    0,    0,  212,    0,  213,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  214,    0,
    0,    0,    0,  215,  216,    0,    0,    0,    0,    0,
  217,    0,    0,    0,  218,    0,  219,  220,  221,  222,
    0,  223,    0,    0,    0,    0,    0,    0,    0,  224,
  225,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  226,  227,    0,  228,  229,  230,    0,    0,    0,    0,
  231,  232,    0,  233,  234,    0,  235,    0,    0,  236,
  237,    0,    0,    0,  238,  239,    0,    0,    0,    0,
  240,  241,  242,    0,  243,  244,  245,    0,  246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  247,  248,
  249,  250,  251,  252,  253,  254,  255,  256,  257,  258,
  259,  260,  261,  262,  263,  264,  265,  266,  267,    0,
  268,  269,  270,  271,  272,  273, 1884,  274,  275,  276,
  277,    0,  278,    0,  279,  280,  281,  282,  283,  284,
  285,  286,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296,    0,  297,  298,  299,  300,    0,  301,  302,
  303,  304,  305,    0,  306,  307,  308,  309,  310,  311,
  312,  313,  314,  315,  316,  317,  318,    0,  319,  320,
  321,  322,  323,  324,    0,  325,  326,  327,  328,  329,
  330,  331,  332,    0,  333,  334,    0,    0,    0,  335,
  336,  337,  338,  339,  340,    0,    0,  341,  342,  343,
  344,  345,  346,  347,  348,  349,    0,    0,    0,    0,
    0,  350,    0,  351,  352,  353,  354,  355,  356,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  357,  358,    0,
  359,  360,  361,  362,  363,  364,  365,  366,  367,    0,
  368,  369,  370,  371,  372,  373,  374,  375,  376,  377,
  378,  379,    0,  380,  381,  382,    0,  383,    0,    0,
    0,  384,  385,  386,  387,  388,  389,  390,  391,  392,
  393,  394,  395,  396,    0,  397,  398,    0,    0,    0,
    0,    0,  399,  400,  401,  402,  403,  404,  405,  406,
  407,  408,  409,  410,  411,  412,    0,    0,  413,  414,
    0,    0,    0,    0,  415,  416,  417,  418,  419,  420,
  421,  422,  423,  424,  425,  426,  427,  428,  429,  430,
    0,    0,    0,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167,    0,    0,    0,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,  806,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0,    0,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,    0,    0,    0,  157,    0,  158,    0,    0,    0,
    0,    0,  159,    0,    0,  160,  161,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  162, 3163,    0,  164,
    0,  165,    0,    0,  166,    0,    0,  167,    0,    0,
    0,    0,    0,    0,  168,    0,  169,  170,    0,    0,
    0,  171,    0,  172,    0,    0,  173,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  174,  175,    0,    0,
    0,  176,    0,  177,    0,  178,    0,    0,    0,    0,
    0,  179,    0,  180,    0,    0,    0,  181,    0,    0,
    0,    0,    0,    0,    0,  182,    0,    0,  183,    0,
  184,    0,    0,    0,  185,  186,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  187,    0,    0,
    0,  189,  190,  191,  192,  193,    0,  194,    0,    0,
    0,    0,    0,  195,    0,  196,  197,    0,    0,  198,
  199,    0,    0,    0,    0,    0,  200,    0,    0,  201,
  202,  203,  204,    0,  205,    0,  206,    0,  207,  208,
  209,  210,  211,    0,    0,    0,    0,  212,    0,  213,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  214,    0,    0,    0,    0,  215,  216,    0,    0,    0,
    0,    0,  217,    0,    0,    0,  218,    0,  219,  220,
  221,  222,    0,  223,    0,    0,    0,    0,    0,    0,
    0,  224,  225,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  226,  227,    0,  228,  229,  230,    0,    0,
    0,    0,  231,  232,    0,  233,  234,    0,  235,    0,
    0,  236,  237,    0,    0,    0,  238,  239,    0,    0,
    0,    0,  240,  241,  242,    0,  243,  244,  245,    0,
  246,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  247,  248,  249,  250,  251,  252,  253,  254,  255,  256,
  257,  258,  259,  260,  261,  262,  263,  264,  265,  266,
  267,    0,  268,  269,  270,  271,  272,  273, 2385,  274,
  275,  276,  277,    0,  278,    0,  279,  280,  281,  282,
  283,  284,  285,  286,  287,  288,  289,  290,  291,  292,
  293,  294,  295,  296,    0,  297,  298,  299,  300,    0,
  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,    0,
  319,  320,  321,  322,  323,  324,    0,  325,  326,  327,
  328,  329,  330,  331,  332,    0,  333,  334,    0,    0,
    0,  335,  336,  337,  338,  339,  340,    0,    0,  341,
  342,  343,  344,  345,  346,  347,  348,  349,    0,    0,
    0,    0,    0,  350,    0,  351,  352,  353,  354,  355,
  356,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  357,
  358,    0,  359,  360,  361,  362,  363,  364,  365,  366,
  367,    0,  368,  369,  370,  371,  372,  373,  374,  375,
  376,  377,  378,  379,    0,  380,  381,  382,    0,  383,
    0,    0,    0,  384,  385,  386,  387,  388,  389,  390,
  391,  392,  393,  394,  395,  396,    0,  397,  398,    0,
    0,    0,    0,    0,  399,  400,  401,  402,  403,  404,
  405,  406,  407,  408,  409,  410,  411,  412,    0,    0,
  413,  414,    0,    0,    0,    0,  415,  416,  417,  418,
  419,  420,  421,  422,  423,  424,  425,  426,  427,  428,
  429,  430,    0,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167,    0,  876,    0,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349, 1044,    0, 1045,
    0,    0,  350,  996,  351,  352,  353,  354,  355,  356,
    0,    0, 1047,    0,    0,    0,    0, 1048,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,  872,  160,  161,    0,    0,    0,    0,  873,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,  874,
    0,  166,    0,    0,  167,  875,  876,    0,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,  877,  173,    0,    0,    0,    0,    0,    0,
    0,  878,    0,  174,  879,    0,    0,    0,  176,    0,
  177,    0,  178,    0,    0,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,  880,    0,
  881,  882,  182,  883,    0,  183,    0,  184,  884,    0,
  885,  185,  186,    0,    0,    0,    0,    0,  886,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,  887,    0,    0,    0,
  195,    0,  196,  197,    0,    0,  198,  199,    0,    0,
    0,  888,    0,  200,    0,    0,  201,  202,  889,  204,
    0,  205,    0,  206,  890,  207,  208,  209,  210,  211,
  891,    0,    0,    0,  212,    0,  213,    0,    0,    0,
  892,    0,    0,    0,    0,    0,    0,  214,    0,    0,
    0,   15,  215,  216,  893,  894,  895,  896,  897,  898,
  899,  618,  619,  218,    0,  219,  220,  221,  222,    0,
  223,  900,    0,    0,    0,    0,    0,    0,  224,  225,
  901,  902,  903,  904,  905,  906,  907,  908,  909,  226,
  910,    0,  228,  229,  230,  911,  912,    0,  913,  914,
  915,    0,  233,  234,  916,  235,    0,    0,  236,  237,
  917,  918,  919,  238,  239,    0,    0,    0,    0,  240,
  920,  242,    0,  921,  244,  245,    0,  246,    0,  922,
  923,  924,    0,  925,  926,    0,  927,  247,  248,  249,
  250,  251,  252,  928,  254,  929,  256,  930,  931,  932,
  933,  934,  935,  936,  937,  938,  939,  940,    0,  941,
  942,  943,  944,  945,  946,    0,  947,  948,  949,  277,
    0,  950,    0,  951,  952,  953,  954,  955,  284,  285,
  956,  287,  957,  958,  959,  291,  960,  293,  961,  295,
  962,    0,  963,  964,  965,  966,    0,  967,  968,  969,
  304,  970,    0,  971,  972,  308,  973,  310,  311,  974,
  313,  314,  315,  316,  317,  318,    0,  975,  320,  321,
  322,  323,  324,    0,  976,  326,  327,  328,  977,  978,
  331,  332,    0,  333,  334,  979,  980,    0,  335,  336,
  337,  981,  982,  983,    0,    0,  341,  984,  985,  986,
  987,  988,  989,  990,  991,  992,    0,  993,  994,  995,
  350,  996,  351,  352,  353,  354,  355,  356,    0,    0,
  997,  998,  999, 1000, 1001, 1002, 1003, 1004, 1005, 1006,
 1007, 1008, 1009, 1010, 1011, 1012, 1013, 1014,    0,  359,
 1015,  361,  362, 1016,  364,  365,  366, 1017,    0,  368,
  369,  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,
  379,    0,  380,  381, 1021,    0,  383,    0, 1022, 1023,
  384, 1024,  386,  387,  388, 1025, 1026,  391,  392,  393,
 1027,  395,  396,    0, 1028,  398, 1029, 1030, 1031, 1032,
 1033,  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,
  408,  409,  410,  411,  412,    0,    0, 1040,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422, 1041,  424,  425,  426,  427,  428, 1042, 1043, 2402,
 2402, 2084, 2402, 2402, 2402,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2402, 2402, 2402,
    0,    0, 2404, 2404, 2093, 2404, 2404, 2404,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2404, 2404, 2404,    0,    0,    0,    0,    0, 2402, 2349,
 2349, 2349, 2349, 2349, 2349, 2349,    0,    0, 1143,    0,
    0, 1144,    0,    0,    0,    0,    0,    0, 2349, 2349,
 2349, 2404,    0,    0,    0,    0,    0, 1145,    0, 1146,
    0,    0,    0, 2959,    0,    0, 1144,    0,    0,    0,
    0,    0,  169,    0,    0,  169,    0,    0, 2960, 2349,
    0,    0, 1145,    0, 1146,    0,    0,  169,    0,    0,
    0,  169,    0,  169,    0,    0,    0, 1147,    0,    0,
    0,    0,    0, 1148, 1149, 1150,    0, 1151, 1152, 1153,
    0, 1154,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1147,    0,    0, 1155,    0, 1156, 1148, 1149,
 1150,  169, 1151, 1152, 1153,    0, 1154,  169,  169,  169,
    0,  169,  169,  169,    0,  169,    0,    0,    0,    0,
 1155,    0, 1156,    0,    0,    0, 1157,    0,    0,  169,
    0,  169,    0,    0,    0,    0,    0,    0,    0,    0,
 3502,    0, 1158,    0,    0,    0,    0,    0,    0,    0,
    0, 1157,    0,    0,    0,    0,    0,    0,    0,    0,
  169,    0,    0,    0, 1159,  168,    0, 1158,    0, 2402,
    0,    0,    0, 2402,    0,    0,  169,    0, 2402,    0,
    0,    0,    0,    0, 2402,    0,    0,    0, 2402, 1159,
    0, 2402, 2404,    0,    0,    0, 2404,    0,  169,    0,
    0, 2404, 1287, 1161,    0,    0,    0, 2404,    0,    0,
    0, 2404,    0,    0, 2404,    0,    0,    0,    0, 1162,
 2349,    0,    0,    0, 2349, 2402,    0, 1287, 1161, 2349,
    0,    0, 2402,    0,    0, 2349,  169,  169,    0, 2349,
 2402,    0,    0,    0, 1162,    0,    0,    0, 2404,    0,
    0,    0,    0,  169,    0, 2404,    0, 2402,    0, 2402,
    0,    0,    0, 2404,    0,    0,    0,    0,    0, 2402,
    0, 2402, 2402, 2402,    0,    0, 2349,    0, 2402,    0,
 2404,    0, 2404, 2349,    0,    0,    0,    0,    0,    0,
    0, 2349, 2404,    0, 2404, 2404, 2404,    0,    0,    0,
    0, 2404,    0,    0,    0,    0,    0,    0, 2349,    0,
 2349,    0,    0,    0,    0,    0,    0,    0,    0, 2402,
 2349,    0, 2349, 2349, 2349,    0,    0,    0,    0, 2349,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2404,    0,    0,    0,    0,    0,    0,    0,
 1334,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2402, 2402, 2402, 2402, 2402,
 2402, 2402, 2402,    0,    0,    0,    0,    0, 1163,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2404, 2404,
 2404, 2404, 2404, 2404, 2404, 2404,    0,    0,    0,    0,
    0,    0,    0, 1163,    0,    0,    0,    0,    0,    0,
 1164,    0,  169,    0,    0,    0,    0,    0,    0,    0,
 1165,    0,    0,    0,    0,    0, 1971,    0,    0,    0,
    0,    0,    0, 1166,    0, 1164,    0,    0, 1972,    0,
    0,    0,    0,    0,  169, 1165, 1973,    0,    0,    0,
    0,    0,    0,    0,  169,    0,    0,    0, 1166,    0,
    0,    0,    0,    0,    0,    0,    0,  169,    0,    0,
    0,    0,    0,    0, 2402,    0,    0,    0,    0,    0,
    0,    0, 1167,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2404,    0,    0,
 2402,    0,    0,    0,    0,    0,    0, 1167,    0, 2402,
    0,    0,    0,    0,    0,    0,  169,    0,    0,    0,
    0,    0,    0, 2404,    0,    0,    0,    0,    0,    0,
    0,    0, 2404,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2349,    0,  157,    0,  158,    0,    0, 2349,    0,    0,
  159,    0,    0,  160,  161,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  162,  163,    0,  164,    0,  165,
    0,    0,  166,    0,    0,  167,    0,    0,    0,    0,
    0,    0,  168,    0,  169,  170,    0,    0,    0,  171,
    0,  172,    0,    0,  173,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  174,  175,    0,    0,    0,  176,
    0,  177,    0,  178,    0,    0,    0,    0,    0,  179,
    0,  180,    0,    0,    0,  181,    0,    0, 2402, 2402,
    0,    0,    0,  182,    0,    0,  183,    0,  184,    0,
    0,    0,  185,  186,    0,    0,    0, 2402,    0, 2402,
    0, 2404, 2404,    0,    0,  187,    0,    0,    0,  189,
  190,  191,  192,  193,    0,  194,    0,    0,    0,    0,
 2404,  195, 2404,  196,  197,    0,    0,  198,  199,    0,
    0,    0,    0,    0,  200,    0,    0,  201,  202,  203,
  204,    0,  205,    0,  206,    0,  207,  208,  209,  210,
  211,    0,    0,    0,    0,  212,    0,  213,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  214,    0,
    0,    0,    0,  215,  216,    0,    0,    0,    0,    0,
  217,  634,    0,    0,  218,    0,  219,  220,  221,  222,
    0,  223,    0,    0,    0,    0,    0,    0,    0,  224,
  225,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  226,  227,    0,  228,  229,  230,    0,    0,    0,    0,
  231,  232,    0,  233,  234,    0,  235,    0,    0,  236,
  237,    0,    0,    0,  238,  239,    0,    0,    0,    0,
  240,  241,  242,    0,  243,  244,  245,    0,  246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  247,  248,
  249,  250,  251,  252,  253,  254,  255,  256,  257,  258,
  259,  260,  261,  262,  263,  264,  265,  266,  267,    0,
  268,  269,  270,  271,  272,  273,    0,  274,  275,  276,
  277,    0,  278,    0,  279,  280,  281,  282,  283,  284,
  285,  286,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296, 2448,  297,  298,  299,  300,    0,  301,  302,
  303,  304,  305,    0,  306,  307,  308,  309,  310,  311,
  312,  313,  314,  315,  316,  317,  318,    0,  319,  320,
  321,  322,  323,  324,    0,  325,  326,  327,  328,  329,
  330,  331,  332,    0,  333,  334,    0,    0,    0,  335,
  336,  337,  338,  339,  340,    0,    0,  341,  342,  343,
  344,  345,  346,  347,  348,  349,    0,    0,    0,    0,
    0,  350,    0,  351,  352,  353,  354,  355,  356,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  357,  358, 1333,
  359,  360,  361,  362,  363,  364,  365,  366,  367,    0,
  368,  369,  370,  371,  372,  373,  374,  375,  376,  377,
  378,  379,    0,  380,  381,  382,    0,  383,    0,    0,
    0,  384,  385,  386,  387,  388,  389,  390,  391,  392,
  393,  394,  395,  396,    0,  397,  398,    0,    0,    0,
    0,    0,  399,  400,  401,  402,  403,  404,  405,  406,
  407,  408,  409,  410,  411,  412,    0,    0,  413,  414,
    0,    0,    0,    0,  415,  416,  417,  418,  419,  420,
  421,  422,  423,  424,  425,  426,  427,  428,  429,  430,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  157,    0,  158,    0, 2432,    0,    0,    0,  159,
    0,    0,  160,  161, 2066,    0,    0,    0,    0,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167,    0,    0,    0,    0,    0,
    5,  168,    0,  169,  170,    0,    0,    0,  171,    0,
 2433, 2434,    0, 2435,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,  175,    0,    0,    0, 2436, 2437,
  177,    0,  178,    0,    9,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,    0,    0,
  881,    0,  182,  883,    0,  183,    0,  184,    0,   10,
    0,  185,  186,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,  887, 2438,    0,    0,
  195,    0,  196,  197,    0,    0,  198,  199,    0,    0,
    0,  888,    0,  200,    0, 2439,  201,  202,  203,  204,
    0,  205,    0,  206,    0,  207,  208,  209,  210, 2440,
    0,    0,    0,    0,  212,    0,  213,    0,    0,   14,
    0,    0,    0,    0,    0,    0,    0,  214, 2816,    0,
    0,   15,  215,  216,    0,    0,    0,    0,    0,  898,
    0,    0,    0,  218,    0,  219,  220,  221,  222,    0,
  223,  900,    0,    0,    0,    0,    0,    0,  224,  225,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2441,
  910,    0,  228,  229,  230,    0,    0,    0,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236, 2442,
    0,    0,    0,  238,  239, 2443, 2444, 2445,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,  922,
  923,  924,    0,  925,  926,    0,  927,  247,  248,  249,
  250,  251,  252,  928,  254,  929,  256,  930,  931,  932,
  933,  934,  935,  936,  937,  938,  939,  940,    0,  941,
  942,  943,  944,  945,  273,    0,  947,  948,  949,  277,
    0,  950,    0,  279,  952,  953,  954,  955,  284,  285,
  956,  287,  957,  958,  959,  291,  960,  293,  961,  295,
  962, 2448,  963,  964,  965,  966,    0,  967,  968,  969,
  304,  970,    0,  971,  972,  308,  973,  310,  311,  974,
  313,  314,  315,  316,  317,  318,    0,  975,  320,  321,
  322,  323,  324,    0,  976,  326, 2446,  328,  977,  978,
  331,  332,    0,  333,  334,  979,  980,    0,  335,  336,
  337,  981,  982,  983, 2447,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0,    0,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1013, 1014,    0,  359,
 1015,  361,  362, 1016,  364,  365,  366,  367,    0,  368,
  369,  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,
  379,    0,  380,  381, 1021,    0,  383,    0,    0,    0,
  384, 1024,  386,  387,  388, 1025,  390,  391,  392,  393,
  394,  395,  396,    0, 1028,  398,    0,    0, 1031, 1032,
 1033,  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,
  408,  409,  410,  411,  412,    0,    0, 1040,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422, 1041,  424,  425,  426,  427,  428, 1042, 1043,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  157,    0,  158,    0, 2432,    0,    0,    0,  159,    0,
    0,  160,  161, 2066,    0,    0,    0,    0,    0,    0,
    0,    0,  162,  163,    0,  164,    0,  165,    0,    0,
  166,    0,    0,  167,    0,    0,    0,    0,    0,    5,
  168,    0,  169,  170,    0,    0,    0,  171,    0, 2433,
 2434,    0, 2435,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  174,  175,    0,    0,    0, 2436, 2437,  177,
    0,  178,    0,    9,    0,    0,    0,  179,    0,  180,
    0,    0,    0,  181,    0,    0,    0,    0,    0,  881,
    0,  182,  883,    0,  183,    0,  184,    0,   10,    0,
  185,  186,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  187,    0,    0,    0,  189,  190,  191,
  192,  193,    0,  194,    0,  887, 2438,    0,    0,  195,
    0,  196,  197,    0,    0,  198,  199,    0,    0,    0,
  888,    0,  200,    0, 2439,  201,  202,  203,  204,    0,
  205,    0,  206,    0,  207,  208,  209,  210, 2440,    0,
    0,    0,    0,  212,    0,  213,    0,    0,   14,    0,
    0,    0,    0,    0,    0,    0,  214,    0,    0,    0,
   15,  215,  216,    0,    0,    0,    0,    0,  898,    0,
    0,    0,  218,    0,  219,  220,  221,  222,    0,  223,
  900,    0,    0,    0,    0,    0,    0,  224,  225,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2441,  910,
    0,  228,  229,  230,    0,    0,    0,    0,  231,  232,
    0,  233,  234,    0,  235,    0,    0,  236, 2442,    0,
    0,    0,  238,  239, 2443, 2444, 2445,    0,  240,  241,
  242,    0,  243,  244,  245,    0,  246,    0,  922,  923,
  924,    0,  925,  926,    0,  927,  247,  248,  249,  250,
  251,  252,  928,  254,  929,  256,  930,  931,  932,  933,
  934,  935,  936,  937,  938,  939,  940,    0,  941,  942,
  943,  944,  945,  273,    0,  947,  948,  949,  277,    0,
  950,    0,  279,  952,  953,  954,  955,  284,  285,  956,
  287,  957,  958,  959,  291,  960,  293,  961,  295,  962,
 1047,  963,  964,  965,  966,    0,  967,  968,  969,  304,
  970,    0,  971,  972,  308,  973,  310,  311,  974,  313,
  314,  315,  316,  317,  318,    0,  975,  320,  321,  322,
  323,  324,    0,  976,  326, 2446,  328,  977,  978,  331,
  332,    0,  333,  334,  979,  980,    0,  335,  336,  337,
  981,  982,  983, 2447,    0,  341,  342,  343,  344,  345,
  346,  347,  348,  349,    0,    0,    0,    0,    0,  350,
    0,  351,  352,  353,  354,  355,  356,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1013, 1014,    0,  359, 1015,
  361,  362, 1016,  364,  365,  366,  367,    0,  368,  369,
  370,  371,  372,  373, 1018,  375, 1019, 1020,  378,  379,
    0,  380,  381, 1021,    0,  383,    0,    0,    0,  384,
 1024,  386,  387,  388, 1025,  390,  391,  392,  393,  394,
  395,  396,    0, 1028,  398,    0,    0, 1031, 1032, 1033,
  399, 1034, 1035, 1036, 1037, 1038, 1039,  406,  407,  408,
  409,  410,  411,  412,    0,    0, 1040,  414,    0,    0,
    0,    0,  415,  416,  417,  418,  419,  420,  421,  422,
 1041,  424,  425,  426,  427,  428, 1042, 1043,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334, 1459, 1458, 1460,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349, 1459, 1458, 1460,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430, 2110, 2110, 2110,
 2110, 2110,    0, 2110,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2110, 2110, 2110,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2238, 2238,    0, 2238, 2238, 2238, 2110,    0,    0,
    0,    0,    0,    0,    0, 1443,    0,    0, 2238, 2238,
 2238, 2238,    0,    0, 1444,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1443,    0, 1445,    0,    0,
    0,    0,    0,    0, 1444,    0,    0,    0,    0,    0,
 2238,    0,    0,    0,    0,    0,    0, 1445,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1446,    0,    0,    0,    0,    0,    0, 1447,    0,
    0,    0,    0,    0,    0,    0, 1448,    0,    0,    0,
    0, 1446,    0,    0,    0,    0,    0,    0, 1447,    0,
    0,    0,    0, 1449,    0, 1450, 1448,    0,    0, 1715,
 1715, 1713, 1715,    0, 1715, 1451,    0, 1452, 1453, 1454,
    0,    0,    0, 1449, 1455, 1450,    0, 1713, 1713, 1713,
    0,    0,    0,    0,    0, 1451,    0, 1452, 1453, 1454,
    0,    0,    0,    0, 1455,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1456,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2110,    0,
    0,    0, 2110,    0,    0, 1456,    0, 2110,    0,    0,
    0,    0,    0, 2110,    0,    0,    0, 2110,    0,    0,
 2110,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1743, 1744, 1745, 1746, 1747, 1748, 1749, 1750,    0,
    0, 2238,    0, 2238,    0, 2238,    0,    0,    0,    0,
 2238,    0,    0,    0, 2110,    0, 2238,    0,    0,    0,
 2238, 2110,    0, 2238,    0,    0,    0,    0,    0, 2110,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2110,    0, 2110,    0,
    0,    0,    0,    0,    0,    0,    0, 2238, 2110,    0,
 2110, 2110, 2110,    0, 2238,    0,    0, 2110,    0,    0,
    0, 2238, 2238,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2238,
    0, 2238,    0,    0,    0,    0,    0,    0,    0,    0,
 1751, 2238,    0, 2238, 2238, 2238,    0, 2238, 2110, 1713,
 2238,    0,    0, 1715,    0,    0,    0,    0, 1713, 2238,
    0,    0,    0,    0, 1715,    0, 1752,    0, 1715,    0,
    0, 1713,    0,    0,    0, 1457,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2238,    0,    0,    0, 1457,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1713,    0,    0,    0,    0,
    0,    0, 1713,    0,    0,    0, 2238,    0,    0,    0,
 1713,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2238,    0,    0,    0, 1713,    0, 1713,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1713,
    0, 1713, 1713, 1713,    0,    0,    0,    0, 1713,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 4033,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1713,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  157,
 4034,  158,    0,    0, 1753, 1754,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163, 1755,  164, 1756,  165,    0, 2110,  166,
    0,    0,  167,    0,    0, 2128,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178, 2238,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0, 1713,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,  471,  219,  220,  221,  222,  472,  223,  473,
  474,  475,  476,  477,  478,  479,  224,  225,  480,  481,
  482,  483,  484,    0,    0,  485,  486,  226,  227,  487,
  228,  229,  230,  488,  489,  490,  491,  231,  232,  492,
  233,  234,  493,  235,  494,  495,  236,  237,  496,  497,
  498,  238,  239,  499,  500,  501,  502,  240,  241,  242,
  503,  243,  244,  245,  504,  246,  505,  506,  507,  508,
  509,  510,  511,  512,  513,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,  514,  268,  269,  270,
  271,  272,  273,  515,  274,  275,  276,  277,  516,  278,
  517,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,  518,
  297,  298,  299,  300,  519,  301,  302,  303,  304,  305,
  520,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  521,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
  522,  333,  334,    0,    0,  523,  335,  336,  337,  338,
  339,  340,  524,  525,  341,  342,  343,  344,  345,  346,
  347,  348,  349,  526,  527,  528,  529,  530,  350,    0,
  351,  352,  353,  354,  355,  356,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,  544,
  545,  546,  547,  548,  357,  358,  549,  359,  360,  361,
  362,  363,  364,  365,  366,  367,  550,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,  551,
  380,  381,  382,  552,  383,  553,  554,  555,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,  556,  397,  398,  557,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,  558,  559,  413,  414,  560,  561,  562,
  563,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430, 1005,    0, 1005,
    0,    0,    0,    0,    0, 1005,    0,    0, 1005, 1005,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1005,
 1005,    0, 1005,    0, 1005,    0,    0, 1005,    0,    0,
 1005,    0,    0,    0,    0,    0,    0, 1005,    0, 1005,
 1005,    0,    0,    0, 1005,    0, 1005,    0,    0, 1005,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1005,
 1005,    0,    0,    0, 1005,    0, 1005,    0, 1005,    0,
    0,    0,    0,    0, 1005,    0, 1005,    0,    0,    0,
 1005,    0,    0,    0,    0,    0,    0,    0, 1005,    0,
    0, 1005,    0, 1005,    0,    0,    0, 1005, 1005,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1005,    0,    0,    0, 1005, 1005, 1005, 1005, 1005,    0,
 1005,    0,    0,    0,    0,    0, 1005,    0, 1005, 1005,
    0,    0, 1005, 1005,    0,    0,    0,    0,    0, 1005,
    0,    0, 1005, 1005, 1005, 1005,    0, 1005,    0, 1005,
    0, 1005, 1005, 1005, 1005, 1005,    0,    0,    0,    0,
 1005,    0, 1005,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1005,    0,    0,    0,    0, 1005, 1005,
    0,    0,    0,    0,    0, 1005,    0,    0,    0, 1005,
 1005, 1005, 1005, 1005, 1005,    0, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005,    0,    0, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,    0,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005,    0,    0, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005,    0, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005,    0,    0,    0,    0, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,    0, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005, 1005,
 1005, 1005, 1005, 1005, 1005, 2110,    0, 2110,    0,    0,
    0,    0,    0, 2110,    0,    0, 2110, 2110,    0,    0,
 2110,    0,    0, 2110,    0,    0,    0, 2110, 2110,    0,
 2110,    0, 2110,    0,    0, 2110,    0, 2112, 2110, 2110,
    0, 2110,    0,    0,    0, 2110,    0, 2110, 2110,    0,
    0,    0, 2110,    0, 2110,    0,    0, 2110,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2110, 2110,    0,
    0,    0, 2110,    0, 2110,    0, 2110,    0,    0, 2110,
    0,    0, 2110,    0, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110,    0, 2110,    0,    0, 2110,    0,    0, 2110,
    0, 2110,    0,    0,    0, 2110, 2110, 2110,    0, 2110,
    0, 2112,    0,    0,    0,    0,    0,    0, 2110,    0,
    0,    0, 2110, 2110, 2110, 2110, 2110,    0, 2110,    0,
    0,    0,    0,    0, 2110,    0, 2110, 2110, 2110,    0,
 2110, 2110,    0,    0,    0,    0,    0, 2110,    0,    0,
 2110, 2110, 2110, 2110, 2110, 2110,    0, 2110,    0, 2110,
 2110, 2110, 2110, 2110,    0,    0,    0,    0, 2110,    0,
 2110,    0,    0,    0,    0,    0, 2110,    0,    0,    0,
    0, 2110,    0,    0,    0,    0, 2110, 2110,    0,    0,
    0,    0,    0, 2110,    0,    0,    0, 2110,    0, 2110,
 2110, 2110, 2110,    0, 2110,    0,    0,    0,    0,    0,
    0,    0, 2110, 2110, 2110, 2110,    0,    0,    0,    0,
    0,    0,    0, 2110, 2110,    0, 2110, 2110, 2110,    0,
    0, 2110,    0, 2110, 2110,    0, 2110, 2110,    0, 2110,
    0,    0, 2110, 2110,    0,    0,    0, 2110, 2110,    0,
    0,    0,    0, 2110, 2110, 2110,    0, 2110, 2110, 2110,
    0, 2110,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110,    0, 2110, 2110, 2110, 2110, 2110, 2110,    0,
 2110, 2110, 2110, 2110,    0, 2110,    0, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110, 2110, 2110,
    0, 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
    0, 2110, 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110,    0,
    0, 2112, 2110, 2110, 2110, 2110, 2110, 2110,    0,    0,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,    0,
 2110,    0,    0,    0, 2110,    0, 2110, 2110, 2110, 2110,
 2110, 2110,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,    0,
 2110,    0,    0,    0, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110,    0, 2110, 2110,
    0,    0,    0,    0,    0, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,    0,
    0, 2110, 2110,    0, 2110,    0,    0, 2110, 2110, 2110,
 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110, 2110,
 2110, 2110, 2110,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0, 1143,    0,
    0, 1144,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167, 1145,    0, 1146,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0, 1147,    0,    0,
  179,    0,  180, 1148, 1149, 1150,  181, 1151, 1152, 1153,
    0, 1154,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186, 1155,    0, 1156,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197, 1157,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204, 1158,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0, 1159,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0, 2497,    0,    0,    0,    0,    0,    0,    0,
  224,  225, 1287, 1161,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0, 1162,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0, 1163,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
 1164,  359,  360,  361,  362,  363,  364,  365,  366,  367,
 1165,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379, 1166,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0, 1167,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,    0,  160,  161,    0,    0, 1143,    0,    0, 1144,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167, 1145,    0, 1146,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,    0,  173,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,  175,    0,    0,    0,  176,    0,
  177,    0,  178,    0,    0, 1147,    0,    0,  179,    0,
  180, 1148, 1149, 1150,  181, 1151, 1152, 1153,    0, 1154,
    0,    0,  182,    0,    0,  183,    0,  184,    0,    0,
    0,  185,  186, 1155,    0, 1156,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,    0,    0,    0,    0,
  195,    0,  196,  197, 1157,    0,  198,  199,    0,    0,
    0,    0,    0,  200,    0,    0,  201,  202,  203,  204,
 1158,  205,    0,  206,    0,  207,  208,  209,  210,  211,
    0,    0,    0,    0,  212,    0,  213,    0,    0,    0,
    0,    0, 1159,    0,    0,    0,    0,  214,    0,    0,
    0,    0,  215,  216,    0,    0,    0,    0,    0,  217,
    0,    0,    0,  218,    0,  219,  220,  221,  222,    0,
 2610,    0,    0,    0,    0,    0,    0,    0,  224,  225,
 1287, 1161,    0,    0,    0,    0,    0,    0,    0,  226,
  227,    0,  228,  229,  230,    0,    0, 1162,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236,  237,
    0,    0,    0,  238,  239,    0,    0,    0,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  247,  248,  249,
  250,  251,  252,  253,  254,  255,  256,  257,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,    0,  268,
  269,  270,  271,  272,  273,    0,  274,  275,  276,  277,
    0,  278,    0,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,    0,  297,  298,  299,  300,    0,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,    0,  319,  320,  321,
  322,  323,  324,    0,  325,  326,  327,  328,  329,  330,
  331,  332,    0,  333,  334,    0,    0,    0,  335,  336,
  337,  338,  339,  340,    0,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0, 1163,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  357,  358, 1164,  359,
  360,  361,  362,  363,  364,  365,  366,  367, 1165,  368,
  369,  370,  371,  372,  373,  374,  375,  376,  377,  378,
  379, 1166,  380,  381,  382,    0,  383,    0,    0,    0,
  384,  385,  386,  387,  388,  389,  390,  391,  392,  393,
  394,  395,  396,    0,  397,  398,    0,    0,    0,    0,
    0,  399,  400,  401,  402,  403,  404,  405,  406,  407,
  408,  409,  410,  411,  412,    0,    0,  413,  414,    0,
 1167,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422,  423,  424,  425,  426,  427,  428,  429,  430,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0, 1143,    0,    0, 1144,    0,    0,
    0,  162, 3163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167, 1145,    0, 1146,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0, 1147,    0,    0,  179,    0,  180, 1148,
 1149, 1150,  181, 1151, 1152, 1153,    0, 1154,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186, 1155,    0, 1156,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197, 1157,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204, 1158,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
 1159,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225, 1287, 1161,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0, 1162,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276, 3164,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0, 1163,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358, 1164,  359,  360,  361,
  362,  363,  364,  365,  366,  367, 1165,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379, 1166,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0, 1167,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  157,    0,  158,
    0,    0,    0,    0,    0,  159,    0,    0,  160,  161,
    0,    0, 1143,    0,    0, 1144,    0,    0,    0,  162,
  163,    0,  164,    0,  165,    0,    0,  166,    0,    0,
  167, 1145,    0, 1146,    0,    0,    0,  168,    0,  169,
  170,    0,    0,    0,  171,    0,  172,    0,    0,  173,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  175,    0,    0,    0,  176,    0,  177,    0,  178,    0,
    0, 1147,    0,    0,  179,    0,  180, 1148, 1149, 1150,
  181, 1151, 1152, 1153,    0, 1154,    0,    0,  182,    0,
    0,  183,    0,  184,    0,    0,    0,  185,  186, 1155,
    0, 1156,    0,    0,    0,    0,    0,    0,    0,    0,
  187,    0,    0,    0,  189,  190,  191,  192,  193,    0,
  194,    0,    0,    0,    0,    0,  195,    0,  196,  197,
 1157,    0,  198,  199,    0,    0,    0,    0,    0,  200,
    0,    0,  201,  202,  203,  204, 1158,  205,    0,  206,
    0,  207,  208,  209,  210,  211,    0,    0,    0,    0,
  212,    0,  213,    0,    0,    0,    0,    0, 1159,    0,
    0,    0,    0,  214,    0,    0,    0,    0,  215,  216,
    0,    0,    0,    0,    0,  217,    0,    0,    0,  218,
    0,  219,  220,  221,  222,    0,  223,    0,    0,    0,
    0,    0,    0,    0,  224,  225, 1287, 1161,    0,    0,
    0,    0,    0,    0,    0,  226,  227,    0,  228,  229,
  230,    0,    0, 1162,    0,  231,  232,    0,  233,  234,
    0,  235,    0,    0,  236,  237,    0,    0,    0,  238,
  239,    0,    0,    0,    0,  240,  241,  242,    0,  243,
  244,  245,    0,  246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  247,  248,  249,  250,  251,  252,  253,
  254,  255,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,    0,  268,  269,  270,  271,  272,
  273,    0,  274,  275,  276,  277,    0,  278,    0,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,    0,  297,  298,
  299,  300,    0,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,  314,  315,  316,
  317,  318,    0,  319,  320,  321,  322,  323,  324,    0,
  325,  326,  327,  328,  329,  330,  331,  332,    0,  333,
  334,    0,    0,    0,  335,  336,  337,  338,  339,  340,
    0,    0,  341,  342,  343,  344,  345,  346,  347,  348,
  349,    0, 1163,    0,    0,    0,  350,    0,  351,  352,
  353,  354,  355,  356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  357,  358, 1164,  359,  360,  361,  362,  363,
  364,  365,  366,  367, 1165,  368,  369,  370,  371,  372,
  373,  374,  375,  376,  377,  378,  379, 1166,  380,  381,
  382,    0,  383,    0,    0,    0,  384,  385,  386,  387,
  388,  389,  390,  391,  392,  393,  394,  395,  396,    0,
  397,  398,    0,    0,    0,    0,    0,  399,  400,  401,
  402,  403,  404,  405,  406,  407,  408,  409,  410,  411,
  412,    0,    0,  413,  414,    0, 1167,    0,    0,  415,
  416,  417,  418,  419,  420,  421,  422,  423,  424,  425,
  426,  427,  428,  429,  430,   53,    0,   53,   41,   41,
    0,    0,    0,   53,    0,    0,   53,   53,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   53,   53,    0,
   53,    0,   53,    0,   41,   53,    0,    0,   53,    0,
    0,    0,    0,   53,   41,   53,    0,   53,   53,   41,
    0,    0,   53,    0,   53,   41,    0,   53,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   53,   53,    0,
    0,    0,   53,    0,   53,    0,   53,    0,   41,    0,
    0,    0,   53,    0,   53,    0,    0,    0,   53,    0,
    0,    0,    0,    0,    0,    0,   53,    0,    0,   53,
    0,   53,    0,    0,    0,   53,   53,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   53,    0,
    0,    0,   53,   53,   53,   53,   53,    0,   53,    0,
    0,    0,    0,    0,   53,    0,   53,   53,    0,   41,
   53,   53,    0,    0,    0,    0,    0,   53,   41,    0,
   53,   53,   53,   53,    0,   53,    0,   53,    0,   53,
   53,   53,   53,   53,    0,    0,    0,    0,   53,    0,
   53,    0,    0,   41,    0,    0,    0,    0,    0,    0,
    0,   53,    0,    0,    0,    0,   53,   53,    0,    0,
    0,    0,    0,   53,    0,    0,    0,   53,    0,   53,
   53,   53,   53,    0,   53,    0,    0,    0,    0,    0,
    0,    0,   53,   53,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   53,   53,    0,   53,   53,   53,    0,
    0,    0,    0,   53,   53,    0,   53,   53,    0,   53,
    0,    0,   53,   53,    0,    0,    0,   53,   53,    0,
    0,    0,    0,   53,   53,   53,    0,   53,   53,   53,
    0,   53,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   53,   53,   53,   53,   53,   53,   53,   53,   53,
   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
   53,   53,    0,   53,   53,   53,   53,   53,   53,    0,
   53,   53,   53,   53,    0,   53,    0,   53,   53,   53,
   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
   53,   53,   53,   53,   53,    0,   53,   53,   53,   53,
    0,   53,   53,   53,   53,   53,    0,   53,   53,   53,
   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
    0,   53,   53,   53,   53,   53,   53,    0,   53,   53,
   53,   53,   53,   53,   53,   53,    0,   53,   53,    0,
    0,    0,   53,   53,   53,   53,   53,   53,    0,    0,
   53,   53,   53,   53,   53,   53,   53,   53,   53,    0,
    0,    0,    0,    0,    0,    0,   53,   53,   53,   53,
   53,   53,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   53,   53,    0,   53,   53,   53,   53,   53,   53,   53,
   53,   53,    0,   53,   53,   53,   53,   53,   53,   53,
   53,   53,   53,   53,   53,    0,   53,   53,   53,    0,
   53,    0,    0,    0,   53,   53,   53,   53,   53,   53,
   53,   53,   53,   53,   53,   53,   53,    0,   53,   53,
    0,    0,    0,    0,    0,   53,   53,   53,   53,   53,
   53,   53,   53,   53,   53,   53,   53,   53,   53,    0,
    0,   53,   53,    0,    0,    0,    0,   53,   53,   53,
   53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
   53,   53,   53,  130,    0,  130,    0,  128,    0,    0,
    0,  130,    0,    0,  130,  130,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  130,  130,    0,  130,    0,
  130,    0,  128,  130,    0,    0,  130,    0,    0,    0,
    0,  130,  128,  130,    0,  130,  130,  128,    0,    0,
  130,    0,  130,  128,    0,  130,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  130,  130,    0,    0,    0,
  130,    0,  130,    0,  130,    0,  128,    0,    0,    0,
  130,    0,  130,    0,    0,    0,  130,    0,    0,    0,
    0,    0,    0,    0,  130,    0,    0,  130,    0,  130,
    0,    0,    0,  130,  130,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  130,    0,    0,    0,
  130,  130,  130,  130,  130,    0,  130,    0,    0,    0,
    0,    0,  130,    0,  130,  130,    0,  128,  130,  130,
    0,    0,    0,    0,    0,  130,  128,    0,  130,  130,
  130,  130,    0,  130,    0,  130,    0,  130,  130,  130,
  130,  130,    0,    0,    0,    0,  130,    0,  130,    0,
    0,  128,    0,    0,    0,    0,    0,    0,    0,  130,
    0,    0,    0,    0,  130,  130,    0,    0,    0,    0,
    0,  130,    0,    0,    0,  130,    0,  130,  130,  130,
  130,    0,  130,    0,    0,    0,    0,    0,    0,    0,
  130,  130,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  130,  130,    0,  130,  130,  130,    0,    0,    0,
    0,  130,  130,    0,  130,  130,    0,  130,    0,    0,
  130,  130,    0,    0,    0,  130,  130,    0,    0,    0,
    0,  130,  130,  130,    0,  130,  130,  130,    0,  130,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  130,
  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,
  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,
    0,  130,  130,  130,  130,  130,  130,    0,  130,  130,
  130,  130,    0,  130,    0,  130,  130,  130,  130,  130,
  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,
  130,  130,  130,    0,  130,  130,  130,  130,    0,  130,
  130,  130,  130,  130,    0,  130,  130,  130,  130,  130,
  130,  130,  130,  130,  130,  130,  130,  130,    0,  130,
  130,  130,  130,  130,  130,    0,  130,  130,  130,  130,
  130,  130,  130,  130,    0,  130,  130,    0,    0,    0,
  130,  130,  130,  130,  130,  130,    0,    0,  130,  130,
  130,  130,  130,  130,  130,  130,  130,    0,    0,    0,
    0,    0,    0,    0,  130,  130,  130,  130,  130,  130,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  130,  130,
    0,  130,  130,  130,  130,  130,  130,  130,  130,  130,
    0,  130,  130,  130,  130,  130,  130,  130,  130,  130,
  130,  130,  130,    0,  130,  130,  130,    0,  130,    0,
    0,    0,  130,  130,  130,  130,  130,  130,  130,  130,
  130,  130,  130,  130,  130,    0,  130,  130,    0,    0,
    0,    0,    0,  130,  130,  130,  130,  130,  130,  130,
  130,  130,  130,  130,  130,  130,  130,    0,    0,  130,
  130,    0,    0,    0,    0,  130,  130,  130,  130,  130,
  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,
  130,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167,    0,    0,    0,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,    0,  173,    0,    0,    0,    0,    0,    0,
 3353,    0,    0,  174,  175,    0, 3354,    0,  176,    0,
  177,    0,  178,    0,    0,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,    0,    0,
    0,    0,  182,    0,    0,  183,    0,  184,    0,    0,
    0,  185,  186,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,    0,    0,    0,    0,
  195, 3355,  196,  197,    0,    0,  198,  199,    0,    0,
    0,    0,    0,  200,    0,    0,  201,  202,  203,  204,
    0,  205,    0,  206,    0,  207,  208,  209,  210,  211,
    0,    0,    0,    0,  212, 3356,  213,    0,    0,    0,
 3357,    0,    0,    0,    0,    0, 3358,  214,    0,    0,
    0,    0,  215,  216,    0,    0,    0,    0,    0,  217,
    0,    0,    0,  218,    0,  219,  220,  221, 3359,    0,
  223,    0,    0,    0,    0,    0,    0,    0,  224,  225,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  226,
  227,    0,  228,  229,  230,    0,    0,    0,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236,  237,
    0,    0,    0,  238,  239,    0,    0,    0,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  247,  248,  249,
  250,  251,  252,  253,  254,  255,  256,  257,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,    0,  268,
  269,  270,  271,  272,  273,    0,  274,  275,  276,  277,
    0,  278,    0,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,    0,  297,  298,  299,  300,    0,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,    0,  319,  320,  321,
  322,  323,  324,    0,  325,  326,  327,  328,  329,  330,
  331,  332,    0, 3360,  334,    0,    0,    0,  335,  336,
  337,  338,  339,  340,    0,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0,    0,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  357,  358,    0,  359,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  368,
  369,  370,  371,  372,  373,  374,  375,  376,  377,  378,
  379,    0,  380,  381,  382,    0,  383,    0,    0,    0,
  384,  385,  386,  387,  388,  389,  390,  391,  392,  393,
  394,  395,  396,    0,  397,  398,    0,    0,    0,    0,
    0,  399,  400,  401,  402,  403,  404,  405,  406,  407,
  408,  409,  410, 3361,  412,    0,    0,  413,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422,  423,  424,  425,  426,  427,  428,  429,  430,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178, 3512,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0, 3513,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0, 3514,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  157,    0,  158,
    0,    0,    0, 3645,    0,  159,    0,    0,  160,  161,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
  163,    0,  164,    0,  165,    0,    0,  166,    0,    0,
  167,    0,    0,    0,    0,    0,    0,  168,    0,  169,
  170,    0,    0,    0,  171,    0,  172,    0,    0,  173,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  175,    0, 3646,    0,  176,    0,  177,    0,  178,    0,
    0,    0,    0,    0,  179,    0,  180,    0,    0,    0,
  181,    0,    0,    0,    0,    0,    0,    0,  182,    0,
    0,  183,    0,  184,    0,    0,    0,  185,  186,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  187,    0,    0,    0,  189,  190,  191,  192,  193,    0,
  194,    0,    0,    0,    0,    0,  195,    0,  196,  197,
    0,    0,  198,  199,    0,    0,    0,    0,    0,  200,
    0,    0,  201,  202,  203,  204,    0,  205,    0,  206,
    0,  207,  208,  209,  210,  211,    0,    0,    0,    0,
  212,    0,  213,    0,    0,    0, 3647,    0,    0,    0,
    0,    0,    0,  214,    0,    0,    0,    0,  215,  216,
    0,    0,    0,    0,    0,  217,    0,    0,    0,  218,
    0,  219,  220,  221,  222,    0,  223,    0,    0,    0,
    0,    0,    0,    0,  224,  225,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  226,  227,    0,  228,  229,
  230,    0,    0,    0,    0,  231,  232,    0,  233,  234,
    0,  235,    0,    0,  236,  237,    0,    0,    0,  238,
  239,    0,    0,    0,    0,  240,  241,  242,    0,  243,
  244,  245,    0,  246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  247,  248,  249,  250,  251,  252,  253,
  254,  255,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,    0,  268,  269,  270,  271,  272,
  273,    0,  274,  275,  276,  277,    0,  278,    0,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,    0,  297,  298,
  299,  300,    0,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,  314,  315,  316,
  317,  318,    0,  319,  320,  321,  322,  323,  324,    0,
  325,  326,  327,  328,  329,  330,  331,  332,    0,  333,
  334,    0,    0,    0,  335,  336,  337,  338,  339,  340,
    0,    0,  341,  342,  343,  344,  345,  346,  347,  348,
  349,    0,    0,    0,    0,    0,  350,    0,  351,  352,
  353,  354,  355,  356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  357,  358,    0,  359,  360,  361,  362,  363,
  364,  365,  366,  367,    0,  368,  369,  370,  371,  372,
  373,  374,  375,  376,  377,  378,  379,    0,  380,  381,
  382,    0,  383,    0,    0,    0,  384,  385,  386,  387,
  388,  389,  390,  391,  392,  393,  394,  395,  396,    0,
  397,  398,    0,    0,    0,    0,    0,  399,  400,  401,
  402,  403,  404,  405,  406,  407,  408,  409,  410,  411,
  412,    0,    0,  413,  414,    0,    0,    0,    0,  415,
  416,  417,  418,  419,  420,  421,  422,  423,  424,  425,
  426,  427,  428,  429,  430,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,    0,  160,  161,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,    0,    0,  166,    0,    0,  167,    0,
  876,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,    0,  173,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  174,  175,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,    0,    0,    0,    0,  182,    0,    0,  183,
    0,  184,    0,    0,    0,  185,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
    0,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,    0,    0,  200,    0,    0,
  201,  202,  203,  204,    0,  205,    0,  206,    0,  207,
  208,  209,  210,  211,    0,    0,    0,    0,  212,    0,
  213,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,    0,    0,
    0,    0,    0,  217,    0,    0,    0,  218,    0,  219,
  220,  221,  222,    0,  223,    0,    0,    0,    0,    0,
    0,    0,  224,  225,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  226,  227,    0,  228,  229,  230,    0,
    0,    0,    0,  231,  232,    0,  233,  234,    0,  235,
    0,    0,  236,  237,    0,    0,    0,  238,  239,    0,
    0,    0,    0,  240,  241,  242,    0,  243,  244,  245,
    0,  246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  247,  248,  249,  250,  251,  252,  253,  254,  255,
  256,  257,  258,  259,  260,  261,  262,  263,  264,  265,
  266,  267,    0,  268,  269,  270,  271,  272,  273,    0,
  274,  275,  276,  277,    0,  278,    0,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,    0,  297,  298,  299,  300,
    0,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
    0,  319,  320,  321,  322,  323,  324,    0,  325,  326,
  327,  328,  329,  330,  331,  332,    0,  333,  334,    0,
    0,    0,  335,  336,  337,  338,  339,  340,    0,    0,
  341,  342,  343,  344,  345,  346,  347,  348,  349,    0,
    0,    0,    0,    0,  350,  996,  351,  352,  353,  354,
  355,  356,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  357,  358,    0,  359,  360,  361,  362,  363,  364,  365,
  366,  367,    0,  368,  369,  370,  371,  372,  373,  374,
  375,  376,  377,  378,  379,    0,  380,  381,  382,    0,
  383,    0,    0,    0,  384,  385,  386,  387,  388,  389,
  390,  391,  392,  393,  394,  395,  396,    0,  397,  398,
    0,    0,    0,    0,    0,  399,  400,  401,  402,  403,
  404,  405,  406,  407,  408,  409,  410,  411,  412,    0,
    0,  413,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422,  423,  424,  425,  426,  427,
  428,  429,  430,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167,    0,    0,    0,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,   15,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0,    0,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382, 1494,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167,    0,    0,    0,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,    0,  173,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,  175,    0,    0,    0,  176,    0,
  177,    0,  178,    0,    0,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,    0,    0,
    0,    0,  182,    0,    0,  183,    0,  184,    0,    0,
    0,  185,  186,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,    0,    0,    0,    0,
  195,    0,  196,  197,    0,    0,  198,  199,    0,    0,
    0,    0,    0,  200,    0,    0,  201,  202,  203,  204,
    0,  205,    0,  206,    0,  207,  208,  209,  210,  211,
    0,    0,    0,    0,  212,    0,  213,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  214,    0,    0,
    0,    0,  215,  216,    0,    0,    0,    0,    0,  217,
    0,  618,  619,  218,    0,  219,  220,  221,  222,    0,
  223,    0,    0,    0,    0,    0,    0,    0,  224,  225,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  226,
  227,    0,  228,  229,  230,    0,    0,    0,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236,  237,
    0,    0,    0,  238,  239,    0,    0,    0,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  247,  248,  249,
  250,  251,  252,  253,  254,  255,  256,  257,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,    0,  268,
  269,  270,  271,  272,  273,    0,  274,  275,  276,  277,
    0,  278,    0,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,    0,  297,  298,  299,  300,    0,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,    0,  319,  320,  321,
  322,  323,  324,    0,  325,  326,  327,  328,  329,  330,
  331,  332,    0,  333,  334,    0,    0,    0,  335,  336,
  337,  338,  339,  340,    0,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0,    0,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  357,  358,    0,  359,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  368,
  369,  370,  371,  372,  373,  374,  375,  376,  377,  378,
  379,    0,  380,  381,  382,    0,  383,    0,    0,    0,
  384,  385,  386,  387,  388,  389,  390,  391,  392,  393,
  394,  395,  396,    0,  397,  398,    0,    0,    0,    0,
    0,  399,  400,  401,  402,  403,  404,  405,  406,  407,
  408,  409,  410,  411,  412,    0,    0,  413,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422,  423,  424,  425,  426,  427,  428,  429,  430,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179, 1716,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0, 1717,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0, 3294,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0, 3295,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  157,    0,  158,
    0,    0,    0,    0,    0,  159,    0,    0,  160,  161,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
  163,    0,  164,    0,  165,    0,    0,  166,    0,    0,
  167,    0,    0,    0,    0,    0,    0,  168,    0,  169,
  170,    0,    0,    0,  171,    0,  172,    0,    0,  173,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  175,    0, 3646,    0,  176,    0,  177,    0,  178,    0,
    0,    0,    0,    0,  179,    0,  180,    0,    0,    0,
  181,    0,    0,    0,    0,    0,    0,    0,  182,    0,
    0,  183,    0,  184,    0,    0,    0,  185,  186,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  187,    0,    0,    0,  189,  190,  191,  192,  193,    0,
  194,    0,    0,    0,    0,    0,  195,    0,  196,  197,
    0,    0,  198,  199,    0,    0,    0,    0,    0,  200,
    0,    0,  201,  202,  203,  204,    0,  205,    0,  206,
    0,  207,  208,  209,  210,  211,    0,    0,    0,    0,
  212,    0,  213,    0,    0,    0, 3647,    0,    0,    0,
    0,    0,    0,  214,    0,    0,    0,    0,  215,  216,
    0,    0,    0,    0,    0,  217,    0,    0,    0,  218,
    0,  219,  220,  221,  222,    0,  223,    0,    0,    0,
    0,    0,    0,    0,  224,  225,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  226,  227,    0,  228,  229,
  230,    0,    0,    0,    0,  231,  232,    0,  233,  234,
    0,  235,    0,    0,  236,  237,    0,    0,    0,  238,
  239,    0,    0,    0,    0,  240,  241,  242,    0,  243,
  244,  245,    0,  246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  247,  248,  249,  250,  251,  252,  253,
  254,  255,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,    0,  268,  269,  270,  271,  272,
  273,    0,  274,  275,  276,  277,    0,  278,    0,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,    0,  297,  298,
  299,  300,    0,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,  314,  315,  316,
  317,  318,    0,  319,  320,  321,  322,  323,  324,    0,
  325,  326,  327,  328,  329,  330,  331,  332,    0,  333,
  334,    0,    0,    0,  335,  336,  337,  338,  339,  340,
    0,    0,  341,  342,  343,  344,  345,  346,  347,  348,
  349,    0,    0,    0,    0,    0,  350,    0,  351,  352,
  353,  354,  355,  356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  357,  358,    0,  359,  360,  361,  362,  363,
  364,  365,  366,  367,    0,  368,  369,  370,  371,  372,
  373,  374,  375,  376,  377,  378,  379,    0,  380,  381,
  382,    0,  383,    0,    0,    0,  384,  385,  386,  387,
  388,  389,  390,  391,  392,  393,  394,  395,  396,    0,
  397,  398,    0,    0,    0,    0,    0,  399,  400,  401,
  402,  403,  404,  405,  406,  407,  408,  409,  410,  411,
  412,    0,    0,  413,  414,    0,    0,    0,    0,  415,
  416,  417,  418,  419,  420,  421,  422,  423,  424,  425,
  426,  427,  428,  429,  430,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,    0,  160,  161,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,    0,    0,  166,    0,    0,  167,    0,
    0,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,    0,  173,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  174,  175,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,    0,    0,    0,    0,  182,    0,    0,  183,
    0,  184,    0,    0,    0,  185,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,  188,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
    0,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,    0,    0,  200,    0,    0,
  201,  202,  203,  204,    0,  205,    0,  206,    0,  207,
  208,  209,  210,  211,    0,    0,    0,    0,  212,    0,
  213,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,    0,    0,
    0,    0,    0,  217,    0,    0,    0,  218,    0,  219,
  220,  221,  222,    0,  223,    0,    0,    0,    0,    0,
    0,    0,  224,  225,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  226,  227,    0,  228,  229,  230,    0,
    0,    0,    0,  231,  232,    0,  233,  234,    0,  235,
    0,    0,  236,  237,    0,    0,    0,  238,  239,    0,
    0,    0,    0,  240,  241,  242,    0,  243,  244,  245,
    0,  246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  247,  248,  249,  250,  251,  252,  253,  254,  255,
  256,  257,  258,  259,  260,  261,  262,  263,  264,  265,
  266,  267,    0,  268,  269,  270,  271,  272,  273,    0,
  274,  275,  276,  277,    0,  278,    0,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,    0,  297,  298,  299,  300,
    0,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
    0,  319,  320,  321,  322,  323,  324,    0,  325,  326,
  327,  328,  329,  330,  331,  332,    0,  333,  334,    0,
    0,    0,  335,  336,  337,  338,  339,  340,    0,    0,
  341,  342,  343,  344,  345,  346,  347,  348,  349,    0,
    0,    0,    0,    0,  350,    0,  351,  352,  353,  354,
  355,  356,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  357,  358,    0,  359,  360,  361,  362,  363,  364,  365,
  366,  367,    0,  368,  369,  370,  371,  372,  373,  374,
  375,  376,  377,  378,  379,    0,  380,  381,  382,    0,
  383,    0,    0,    0,  384,  385,  386,  387,  388,  389,
  390,  391,  392,  393,  394,  395,  396,    0,  397,  398,
    0,    0,    0,    0,    0,  399,  400,  401,  402,  403,
  404,  405,  406,  407,  408,  409,  410,  411,  412,    0,
    0,  413,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422,  423,  424,  425,  426,  427,
  428,  429,  430,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167,    0,    0,    0,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,  436,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0,    0,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167,    0,    0,    0,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,    0,  173,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,  175,    0,    0,    0,  176,    0,
  177,    0,  178,    0,    0,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,    0,    0,
    0,    0,  182,    0,    0,  183,    0,  184,    0,    0,
    0,  185,  186,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,    0,    0,    0,    0,
  195,    0,  196,  197,    0,    0,  198,  199,    0,    0,
    0,    0,    0,  200,    0,    0,  201,  202,  203,  204,
    0,  205,    0,  206,    0,  207,  208,  209,  210,  211,
    0,    0,    0,    0,  212,    0,  213,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  214,    0,    0,
    0,    0,  215,  216,    0,    0,    0,    0,    0,  217,
    0,    0,    0,  218,    0,  219,  220,  221,  222,    0,
  223,    0,    0,    0,    0,    0,    0,    0,  224,  225,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  226,
  227,    0,  228,  229,  230,    0,    0,    0,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236,  237,
    0,    0,    0,  238,  239,    0,    0,    0,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  247,  248,  249,
  250,  251,  252,  253,  254,  255,  256,  257,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,    0,  268,
  269,  270,  271,  272,  273,    0,  274,  275,  276,  277,
    0,  278,    0,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,    0,  297,  298,  299,  300,    0,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,    0,  319,  320,  321,
  322,  323,  324,    0,  325,  326,  327,  328,  329,  330,
  331,  332,    0,  333,  334,    0,    0,    0,  335,  336,
  337,  338,  339,  340,    0,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0,    0,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  357,  358,  776,  359,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  368,
  369,  370,  371,  372,  373,  374,  375,  376,  377,  378,
  379,    0,  380,  381,  382,    0,  383,    0,    0,    0,
  384,  385,  386,  387,  388,  389,  390,  391,  392,  393,
  394,  395,  396,    0,  397,  398,    0,    0,    0,    0,
    0,  399,  400,  401,  402,  403,  404,  405,  406,  407,
  408,  409,  410,  411,  412,    0,    0,  413,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422,  423,  424,  425,  426,  427,  428,  429,  430,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0, 1925,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  157,    0,  158,
    0,    0,    0,    0,    0,  159,    0,    0,  160,  161,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
  163,    0,  164, 2513,  165,    0,    0,  166,    0,    0,
  167,    0,    0,    0,    0,    0,    0,  168,    0,  169,
  170,    0,    0,    0,  171,    0,  172,    0,    0,  173,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  175,    0,    0,    0,  176,    0,  177,    0,  178,    0,
    0,    0,    0,    0,  179,    0,  180,    0,    0,    0,
  181,    0,    0,    0,    0,    0,    0,    0,  182,    0,
    0,  183,    0,  184,    0,    0,    0,  185,  186,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  187,    0,    0,    0,  189,  190,  191,  192,  193,    0,
  194,    0,    0,    0,    0,    0,  195,    0,  196,  197,
    0,    0,  198,  199,    0,    0,    0,    0,    0,  200,
    0,    0,  201,  202,  203,  204,    0,  205,    0,  206,
    0,  207,  208,  209,  210,  211,    0,    0,    0,    0,
  212,    0,  213,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  214,    0,    0,    0,    0,  215,  216,
    0,    0,    0,    0,    0,  217,    0,    0,    0,  218,
    0,  219,  220,  221,  222,    0,  223,    0,    0,    0,
    0,    0,    0,    0,  224,  225,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  226,  227,    0,  228,  229,
  230,    0,    0,    0,    0,  231,  232,    0,  233,  234,
    0,  235,    0,    0,  236,  237,    0,    0,    0,  238,
  239,    0,    0,    0,    0,  240,  241,  242,    0,  243,
  244,  245,    0,  246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  247,  248,  249,  250,  251,  252,  253,
  254,  255,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,    0,  268,  269,  270,  271,  272,
  273,    0,  274,  275,  276,  277,    0,  278,    0,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,    0,  297,  298,
  299,  300,    0,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,  314,  315,  316,
  317,  318,    0,  319,  320,  321,  322,  323,  324,    0,
  325,  326,  327,  328,  329,  330,  331,  332,    0,  333,
  334,    0,    0,    0,  335,  336,  337,  338,  339,  340,
    0,    0,  341,  342,  343,  344,  345,  346,  347,  348,
  349,    0,    0,    0,    0,    0,  350,    0,  351,  352,
  353,  354,  355,  356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  357,  358,    0,  359,  360,  361,  362,  363,
  364,  365,  366,  367,    0,  368,  369,  370,  371,  372,
  373,  374,  375,  376,  377,  378,  379,    0,  380,  381,
  382,    0,  383,    0,    0,    0,  384,  385,  386,  387,
  388,  389,  390,  391,  392,  393,  394,  395,  396,    0,
  397,  398,    0,    0,    0,    0,    0,  399,  400,  401,
  402,  403,  404,  405,  406,  407,  408,  409, 2514,  411,
  412,    0,    0,  413,  414,    0,    0,    0,    0,  415,
  416,  417,  418,  419,  420,  421,  422,  423,  424,  425,
  426,  427,  428,  429,  430,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,    0,  160,  161,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  162,  163,    0,
  164, 2849,  165,    0,    0,  166,    0,    0,  167,    0,
    0,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,    0,  173,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  174,  175,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,    0,    0,    0,    0,  182,    0,    0,  183,
    0,  184,    0,    0,    0,  185,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
    0,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,    0,    0,  200,    0,    0,
  201,  202,  203,  204,    0,  205,    0,  206,    0,  207,
  208,  209,  210,  211,    0,    0,    0,    0,  212,    0,
  213,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,    0,    0,
    0,    0,    0,  217,    0,    0,    0,  218,    0,  219,
  220,  221,  222,    0,  223,    0,    0,    0,    0,    0,
    0,    0,  224,  225,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  226,  227,    0,  228,  229,  230,    0,
    0,    0,    0,  231,  232,    0,  233,  234,    0,  235,
    0,    0,  236,  237,    0,    0,    0,  238,  239,    0,
    0,    0,    0,  240,  241,  242,    0,  243,  244,  245,
    0,  246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  247,  248,  249,  250,  251,  252,  253,  254,  255,
  256,  257,  258,  259,  260,  261,  262,  263,  264,  265,
  266,  267,    0,  268,  269,  270,  271,  272,  273,    0,
  274,  275,  276,  277,    0,  278,    0,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,    0,  297,  298,  299,  300,
    0,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
    0,  319,  320,  321,  322,  323,  324,    0,  325,  326,
  327,  328,  329,  330,  331,  332,    0,  333,  334,    0,
    0,    0,  335,  336,  337,  338,  339,  340,    0,    0,
  341,  342,  343,  344,  345,  346,  347,  348,  349,    0,
    0,    0,    0,    0,  350,    0,  351,  352,  353,  354,
  355,  356,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  357,  358,    0,  359,  360,  361,  362,  363,  364,  365,
  366,  367,    0,  368,  369,  370,  371,  372,  373,  374,
  375,  376,  377,  378,  379,    0,  380,  381,  382,    0,
  383,    0,    0,    0,  384,  385,  386,  387,  388,  389,
  390,  391,  392,  393,  394,  395,  396,    0,  397,  398,
    0,    0,    0,    0,    0,  399,  400,  401,  402,  403,
  404,  405,  406,  407,  408,  409,  410,  411,  412,    0,
    0,  413,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422,  423,  424,  425,  426,  427,
  428,  429,  430,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167,    0,    0,    0,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173, 2948,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0,    0,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167,    0,    0,    0,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,    0,  173,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,  175,    0,    0,    0,  176,    0,
  177,    0,  178,    0,    0,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,    0,    0,
    0,    0,  182,    0,    0,  183,    0,  184,    0,    0,
    0,  185,  186,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,    0,    0,    0,    0,
  195,    0,  196,  197,    0,    0,  198,  199,    0,    0,
    0,    0,    0,  200,    0,    0,  201,  202,  203,  204,
    0,  205,    0,  206,    0,  207,  208,  209,  210,  211,
    0,    0,    0,    0,  212,    0,  213,    0,    0,    0,
 3060,    0,    0,    0,    0,    0,    0,  214,    0,    0,
    0,    0,  215,  216,    0,    0,    0,    0,    0,  217,
    0,    0,    0,  218,    0,  219,  220,  221, 3061,    0,
  223,    0,    0,    0,    0,    0,    0,    0,  224,  225,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  226,
  227,    0,  228,  229,  230,    0,    0,    0,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236,  237,
    0,    0,    0,  238,  239,    0,    0,    0,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  247,  248,  249,
  250,  251,  252,  253,  254,  255,  256,  257,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,    0,  268,
  269,  270,  271,  272,  273,    0,  274,  275,  276,  277,
    0,  278,    0,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,    0,  297,  298,  299,  300,    0,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,    0,  319,  320,  321,
  322,  323,  324,    0,  325,  326,  327,  328,  329,  330,
  331,  332,    0,  333,  334,    0,    0,    0,  335,  336,
  337,  338,  339,  340,    0,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0,    0,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  357,  358,    0,  359,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  368,
  369,  370,  371,  372,  373,  374,  375,  376,  377,  378,
  379,    0,  380,  381,  382,    0,  383,    0,    0,    0,
  384,  385,  386,  387,  388,  389,  390,  391,  392,  393,
  394,  395,  396,    0,  397,  398,    0,    0,    0,    0,
    0,  399,  400,  401,  402,  403,  404,  405,  406,  407,
  408,  409,  410,  411,  412,    0,    0,  413,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422,  423,  424,  425,  426,  427,  428,  429,  430,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222, 3152,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  157,    0,  158,
    0,    0,    0,    0,    0,  159,    0,    0,  160,  161,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
  163,    0,  164,    0,  165,    0,    0,  166,    0,    0,
  167,    0,    0,    0,    0,    0,    0,  168,    0,  169,
  170,    0,    0,    0,  171,    0,  172,    0,    0,  173,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  175,    0,    0,    0,  176,    0,  177,    0,  178,    0,
    0,    0,    0,    0,  179,    0,  180,    0,    0,    0,
  181,    0,    0,    0,    0,    0,    0,    0,  182,    0,
    0,  183,    0,  184,    0,    0,    0,  185,  186,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  187,    0,    0,    0,  189,  190,  191,  192,  193,    0,
  194,    0,    0,    0,    0,    0,  195,    0,  196,  197,
    0,    0,  198,  199,    0,    0,    0,    0,    0,  200,
    0,    0,  201,  202,  203,  204,    0,  205,    0,  206,
    0,  207,  208,  209,  210,  211,    0,    0,    0,    0,
  212,    0,  213,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  214,    0,    0,    0,    0,  215,  216,
    0,    0,    0,    0,    0,  217,    0,    0,    0,  218,
    0,  219,  220,  221,  222, 3301,  223,    0,    0,    0,
    0,    0,    0,    0,  224,  225,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  226,  227,    0,  228,  229,
  230,    0,    0,    0,    0,  231,  232,    0,  233,  234,
    0,  235,    0,    0,  236,  237,    0,    0,    0,  238,
  239,    0,    0,    0,    0,  240,  241,  242,    0,  243,
  244,  245,    0,  246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  247,  248,  249,  250,  251,  252,  253,
  254,  255,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,    0,  268,  269,  270,  271,  272,
  273,    0,  274,  275,  276,  277,    0,  278,    0,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,    0,  297,  298,
  299,  300,    0,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,  314,  315,  316,
  317,  318,    0,  319,  320,  321,  322,  323,  324,    0,
  325,  326,  327,  328,  329,  330,  331,  332,    0,  333,
  334,    0,    0,    0,  335,  336,  337,  338,  339,  340,
    0,    0,  341,  342,  343,  344,  345,  346,  347,  348,
  349,    0,    0,    0,    0,    0,  350,    0,  351,  352,
  353,  354,  355,  356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  357,  358,    0,  359,  360,  361,  362,  363,
  364,  365,  366,  367,    0,  368,  369,  370,  371,  372,
  373,  374,  375,  376,  377,  378,  379,    0,  380,  381,
  382,    0,  383,    0,    0,    0,  384,  385,  386,  387,
  388,  389,  390,  391,  392,  393,  394,  395,  396,    0,
  397,  398,    0,    0,    0,    0,    0,  399,  400,  401,
  402,  403,  404,  405,  406,  407,  408,  409,  410,  411,
  412,    0,    0,  413,  414,    0,    0,    0,    0,  415,
  416,  417,  418,  419,  420,  421,  422,  423,  424,  425,
  426,  427,  428,  429,  430,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,    0,  160,  161,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,    0,    0,  166,    0,    0,  167,    0,
    0,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,    0,  173,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  174,  175,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,    0,    0,    0,    0,  182,    0,    0,  183,
    0,  184,    0,    0,    0,  185,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
    0,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,    0,    0,  200,    0,    0,
  201,  202,  203,  204,    0,  205,    0,  206,    0,  207,
  208,  209,  210,  211,    0,    0,    0,    0,  212,    0,
  213,    0,    0,    0, 3572,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,    0,    0,
    0,    0,    0,  217,    0,    0,    0,  218,    0,  219,
  220,  221,  222,    0,  223,    0,    0,    0,    0,    0,
    0,    0,  224,  225,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  226,  227,    0,  228,  229,  230,    0,
    0,    0,    0,  231,  232,    0,  233,  234,    0,  235,
    0,    0,  236,  237,    0,    0,    0,  238,  239,    0,
    0,    0,    0,  240,  241,  242,    0,  243,  244,  245,
    0,  246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  247,  248,  249,  250,  251,  252,  253,  254,  255,
  256,  257,  258,  259,  260,  261,  262,  263,  264,  265,
  266,  267,    0,  268,  269,  270,  271,  272,  273,    0,
  274,  275,  276,  277,    0,  278,    0,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,    0,  297,  298,  299,  300,
    0,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
    0,  319,  320,  321,  322,  323,  324,    0,  325,  326,
  327,  328,  329,  330,  331,  332,    0,  333,  334,    0,
    0,    0,  335,  336,  337,  338,  339,  340,    0,    0,
  341,  342,  343,  344,  345,  346,  347,  348,  349,    0,
    0,    0,    0,    0,  350,    0,  351,  352,  353,  354,
  355,  356,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  357,  358,    0,  359,  360,  361,  362,  363,  364,  365,
  366,  367,    0,  368,  369,  370,  371,  372,  373,  374,
  375,  376,  377,  378,  379,    0,  380,  381,  382,    0,
  383,    0,    0,    0,  384,  385,  386,  387,  388,  389,
  390,  391,  392,  393,  394,  395,  396,    0,  397,  398,
    0,    0,    0,    0,    0,  399,  400,  401,  402,  403,
  404,  405,  406,  407,  408,  409,  410,  411,  412,    0,
    0,  413,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422,  423,  424,  425,  426,  427,
  428,  429,  430,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167,    0,    0,    0,
    0,    0,    0,  168,    0,  169,  170, 3662,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0,    0,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167,    0,    0,    0,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,    0,  173,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,  175,    0,    0,    0,  176,    0,
  177,    0,  178,    0,    0,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,    0,    0,
    0,    0,  182,    0,    0,  183,    0,  184,    0,    0,
    0,  185,  186,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,    0,    0,    0,    0,
  195,    0,  196,  197,    0,    0,  198,  199,    0,    0,
    0,    0,    0,  200,    0,    0,  201,  202,  203,  204,
    0,  205,    0,  206,    0,  207,  208,  209,  210,  211,
    0,    0,    0,    0,  212,    0,  213,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  214,    0,    0,
    0,    0,  215,  216,    0,    0,    0,    0,    0,  217,
    0,    0,    0,  218,    0,  219,  220,  221,  222,    0,
  223,    0,    0,    0,    0,    0,    0,    0,  224,  225,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  226,
  227,    0,  228,  229,  230,    0,    0,    0,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236,  237,
    0,    0,    0,  238,  239,    0,    0,    0,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  247,  248,  249,
  250,  251,  252,  253,  254,  255,  256,  257,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,    0,  268,
  269,  270,  271,  272,  273,    0,  274,  275,  276,  277,
    0,  278,    0,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,    0,  297,  298,  299,  300,    0,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,    0,  319,  320,  321,
  322,  323,  324,    0,  325,  326,  327,  328,  329,  330,
  331,  332,    0,  333,  334,    0,    0,    0,  335,  336,
  337,  338,  339,  340,    0,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0,    0,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  357,  358,    0,  359,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  368,
  369,  370,  371,  372,  373,  374,  375,  376,  377,  378,
  379,    0,  380,  381,  382,    0,  383,    0,    0,    0,
  384,  385,  386,  387,  388,  389,  390,  391,  392,  393,
  394,  395,  396,    0,  397,  398,    0,    0,    0,    0,
    0,  399,  400,  401,  402,  403,  404,  405,  406,  407,
  408,  409,  410,  411,  412,    0,    0,  413,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422,  423,  424,  425,  426,  427,  428,  429,  430,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  655,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  157,    0,  158,
    0,    0,    0,    0,    0,  159,    0,    0,  160,  161,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
  163,    0,  164,    0,  165,    0,    0,  166,    0,    0,
  167,    0,    0,    0,    0,    0,    0,  168,    0,  169,
  170,    0,    0,    0,  171,    0,  172,    0,    0,  173,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  175,    0,    0,    0,  176,    0,  177,    0,  178,    0,
    0,    0,    0,    0,  179,    0,  180,    0,    0,    0,
  181,    0,    0,    0,    0,    0,    0,    0,  182,    0,
    0,  183,    0,  184,    0,    0,    0,  185,  186,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  187,    0,    0,    0,  189,  190,  191,  192,  193,    0,
  194,    0,    0,    0,    0,    0,  195,    0,  196,  197,
    0,    0,  198,  199,    0,    0,    0,    0,    0,  200,
    0,    0,  201,  202,  203,  204,    0,  205,    0,  206,
    0,  207,  208,  209,  210,  211,    0,    0,    0,    0,
  212,    0,  213,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  214,    0,    0,    0,    0,  215,  216,
    0,    0,    0,    0,    0,  217,    0,    0,    0,  218,
    0,  219,  220,  221,  222,    0,  223,    0,    0,    0,
    0,    0,    0,    0,  224,  225,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  226,  227,    0,  228,  229,
  230,    0,    0,    0,    0,  231,  232,    0,  233,  234,
    0,  235,    0,    0,  236,  237,    0,    0,    0,  238,
  239,    0,    0,    0,    0,  240,  241,  242,    0,  243,
  244,  245,    0,  246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  247,  248,  249,  250,  251,  252,  253,
  254,  255,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,    0,  268,  269,  270,  271,  272,
  273,    0,  274,  275,  276,  277,    0,  278,    0,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,    0,  297,  298,
  299,  300,    0,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,  314,  315,  316,
  317,  318,    0,  319,  320,  321,  322,  323,  324,    0,
  325,  685,  327,  328,  329,  330,  331,  332,    0,  333,
  334,    0,    0,    0,  335,  336,  337,  338,  339,  340,
    0,    0,  341,  342,  343,  344,  345,  346,  347,  348,
  349,    0,    0,    0,    0,    0,  350,    0,  351,  352,
  353,  354,  355,  356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  357,  358,    0,  359,  360,  361,  362,  363,
  364,  365,  366,  367,    0,  368,  369,  370,  371,  372,
  373,  374,  375,  376,  377,  378,  379,    0,  380,  381,
  382,    0,  383,    0,    0,    0,  384,  385,  386,  387,
  388,  389,  390,  391,  392,  393,  394,  395,  396,    0,
  397,  398,    0,    0,    0,    0,    0,  399,  400,  401,
  402,  403,  404,  405,  406,  407,  408,  409,  410,  411,
  412,    0,    0,  413,  414,    0,    0,    0,    0,  415,
  416,  417,  418,  419,  420,  421,  422,  423,  424,  425,
  426,  427,  428,  429,  430,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,    0,  160,  161,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,    0,    0,  166,    0,    0,  167,    0,
    0,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,    0,  173,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  174,  175,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,    0,    0,    0,    0,  182,    0,    0,  183,
    0,  184,    0,    0,    0,  185,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
    0,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,    0,    0,  200,    0,    0,
  201,  202,  203,  204,    0,  205,    0,  206,    0,  207,
  208,  209,  210,  211,    0,    0,    0,    0,  212,    0,
  213,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,    0,    0,
    0,    0,    0,  217,    0,    0,    0,  218,    0,  219,
  220,  221,  222,    0,  223,    0,    0,    0,    0,    0,
    0,    0,  224,  225,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  226,  227,    0,  228,  229,  230,    0,
    0,    0,    0,  231,  232,    0,  233,  234,    0,  235,
    0,    0,  236,  237,    0,    0,    0,  238,  239,    0,
    0,    0,    0,  240,  241,  242,    0,  243,  244,  245,
    0,  246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  247,  248,  249,  250,  251,  252,  253,  254,  255,
  256,  257,  258,  259,  260,  261,  262,  263,  264,  265,
  266,  267,    0,  268,  269,  270,  271,  272,  273,    0,
  274,  275,  276,  277,    0,  278,    0,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,    0,  297,  298,  299,  300,
    0,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
    0,  319,  320,  321,  322,  323,  324,    0,  325,  730,
  327,  328,  329,  330,  331,  332,    0,  333,  334,    0,
    0,    0,  335,  336,  337,  338,  339,  340,    0,    0,
  341,  342,  343,  344,  345,  346,  347,  348,  349,    0,
    0,    0,    0,    0,  350,    0,  351,  352,  353,  354,
  355,  356,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  357,  358,    0,  359,  360,  361,  362,  363,  364,  365,
  366,  367,    0,  368,  369,  370,  371,  372,  373,  374,
  375,  376,  377,  378,  379,    0,  380,  381,  382,    0,
  383,    0,    0,    0,  384,  385,  386,  387,  388,  389,
  390,  391,  392,  393,  394,  395,  396,    0,  397,  398,
    0,    0,    0,    0,    0,  399,  400,  401,  402,  403,
  404,  405,  406,  407,  408,  409,  410,  411,  412,    0,
    0,  413,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422,  423,  424,  425,  426,  427,
  428,  429,  430,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
  165,    0,    0,  166,    0,    0,  167,    0,    0,    0,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,  208,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0,    0,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370, 4048,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430, 1187,    0, 1187,    0,    0,    0,    0,    0, 1187,
    0,    0, 1187, 1187,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1187, 1187,    0, 1187,    0, 1187,    0,
    0, 1187,    0,    0, 1187,    0,    0,    0,    0,    0,
    0, 1187,    0, 1187, 1187,    0,    0,    0, 1187,    0,
 1187,    0,    0, 1187,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1187, 1187,    0,    0,    0, 1187,    0,
 1187,    0, 1187,    0,    0,    0,    0,    0, 1187,    0,
 1187,    0,    0,    0, 1187,    0,    0,    0,    0,    0,
    0,    0, 1187,    0,    0, 1187,    0, 1187,    0,    0,
    0, 1187, 1187,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1187,    0,    0,    0, 1187, 1187,
 1187, 1187, 1187,    0, 1187,    0,    0,    0,    0,    0,
 1187,    0, 1187, 1187,    0,    0, 1187, 1187,    0,    0,
    0,    0,    0, 1187,    0,    0, 1187, 1187, 1187, 1187,
    0, 1187,    0, 1187,    0, 1187, 1187, 1187, 1187, 1187,
    0,    0,    0,    0, 1187,    0, 1187,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1187,    0,    0,
    0,    0, 1187, 1187,    0,    0,    0,    0,    0, 1187,
    0,    0,    0, 1187,    0, 1187, 1187, 1187, 1187,    0,
 1187,    0,    0,    0,    0,    0,    0,    0, 1187, 1187,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1187,
 1187,    0, 1187, 1187, 1187,    0,    0,    0,    0, 1187,
 1187,    0, 1187, 1187,    0, 1187,    0,    0, 1187, 1187,
    0,    0,    0, 1187, 1187,    0,    0,    0,    0, 1187,
 1187, 1187,    0, 1187, 1187, 1187,    0, 1187,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1187, 1187, 1187,
 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187,    0, 1187,
 1187, 1187, 1187, 1187, 1187,    0, 1187, 1187, 1187, 1187,
    0, 1187,    0, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187,    0, 1187, 1187, 1187, 1187,    0, 1187, 1187, 1187,
 1187, 1187,    0, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187, 1187, 1187, 1187, 1187, 1187,    0, 1187, 1187, 1187,
 1187, 1187, 1187,    0, 1187, 1187, 1187, 1187, 1187, 1187,
 1187, 1187,    0, 1187, 1187,    0,    0,    0, 1187, 1187,
 1187, 1187, 1187, 1187,    0,    0, 1187, 1187, 1187, 1187,
 1187, 1187, 1187, 1187, 1187,    0,    0,    0,    0,    0,
 1187,    0, 1187, 1187, 1187, 1187, 1187, 1187,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1187, 1187,    0, 1187,
 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187,    0, 1187,
 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187,    0, 1187, 1187, 1187,    0, 1187,    0,    0,    0,
 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187, 1187, 1187,    0, 1187, 1187,    0,    0,    0,    0,
    0, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187, 1187, 1187, 1187, 1187,    0,    0, 1187, 1187,    0,
    0,    0,    0, 1187, 1187, 1187, 1187, 1187, 1187, 1187,
 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1187, 1319,
    0, 1319,    0,    0,    0,    0,    0, 1319,    0,    0,
 1319, 1319,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1319, 1319,    0, 1319,    0, 1319,    0,    0, 1319,
    0,    0, 1319,    0,    0,    0,    0,    0,    0, 1319,
    0, 1319, 1319,    0,    0,    0, 1319,    0, 1319,    0,
    0, 1319,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1319, 1319,    0,    0,    0, 1319,    0, 1319,    0,
 1319,    0,    0,    0,    0,    0, 1319,    0, 1319,    0,
    0,    0, 1319,    0,    0,    0,    0,    0,    0,    0,
 1319,    0,    0, 1319,    0, 1319,    0,    0,    0, 1319,
 1319,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1319,    0,    0,    0, 1319, 1319, 1319, 1319,
 1319,    0, 1319,    0,    0,    0,    0,    0, 1319,    0,
 1319, 1319,    0,    0, 1319, 1319,    0,    0,    0,    0,
    0, 1319,    0,    0, 1319, 1319, 1319, 1319,    0, 1319,
    0, 1319,    0, 1319, 1319, 1319, 1319, 1319,    0,    0,
    0,    0, 1319,    0, 1319,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1319,    0,    0,    0,    0,
 1319, 1319,    0,    0,    0,    0,    0, 1319,    0,    0,
    0, 1319,    0, 1319, 1319, 1319, 1319,    0, 1319,    0,
    0,    0,    0,    0,    0,    0, 1319, 1319,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1319, 1319,    0,
 1319, 1319, 1319,    0,    0,    0,    0, 1319, 1319,    0,
 1319, 1319,    0, 1319,    0,    0, 1319, 1319,    0,    0,
    0, 1319, 1319,    0,    0,    0,    0, 1319, 1319, 1319,
    0, 1319, 1319, 1319,    0, 1319,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1319, 1319, 1319, 1319, 1319,
 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,
 1319, 1319, 1319, 1319, 1319, 1319,    0, 1319, 1319, 1319,
 1319, 1319, 1319,    0, 1319, 1319, 1319, 1319,    0, 1319,
    0, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,
 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,    0,
 1319, 1319, 1319, 1319,    0, 1319, 1319, 1319, 1319, 1319,
    0, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,
 1319, 1319, 1319, 1319,    0, 1319, 1319, 1319, 1319, 1319,
 1319,    0, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,
    0, 1319, 1319,    0,    0,    0, 1319, 1319, 1319, 1319,
 1319, 1319,    0,    0, 1319, 1319, 1319, 1319, 1319, 1319,
 1319, 1319, 1319,    0,    0,    0,    0,    0, 1319,    0,
 1319, 1319, 1319, 1319, 1319, 1319,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1319, 1319,    0, 1319, 1319, 1319,
 1319, 1319, 1319, 1319, 1319, 1319,    0, 1319, 1319, 1319,
 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,    0,
 1319, 1319, 1319,    0, 1319,    0,    0,    0, 1319, 1319,
 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,
 1319,    0, 1319, 1319,    0,    0,    0,    0,    0, 1319,
 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,
 1319, 1319, 1319,    0,    0, 1319, 1319,    0,    0,    0,
    0, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1319,
 1319, 1319, 1319, 1319, 1319, 1319, 1319, 1317,    0, 1317,
    0,    0,    0,    0,    0, 1317,    0,    0, 1317, 1317,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1317,
 1317,    0, 1317,    0, 1317,    0,    0, 1317,    0,    0,
 1317,    0,    0,    0,    0,    0,    0, 1317,    0, 1317,
 1317,    0,    0,    0, 1317,    0, 1317,    0,    0, 1317,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1317,
 1317,    0,    0,    0, 1317,    0, 1317,    0, 1317,    0,
    0,    0,    0,    0, 1317,    0, 1317,    0,    0,    0,
 1317,    0,    0,    0,    0,    0,    0,    0, 1317,    0,
    0, 1317,    0, 1317,    0,    0,    0, 1317, 1317,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1317,    0,    0,    0, 1317, 1317, 1317, 1317, 1317,    0,
 1317,    0,    0,    0,    0,    0, 1317,    0, 1317, 1317,
    0,    0, 1317, 1317,    0,    0,    0,    0,    0, 1317,
    0,    0, 1317, 1317, 1317, 1317,    0, 1317,    0, 1317,
    0, 1317, 1317, 1317, 1317, 1317,    0,    0,    0,    0,
 1317,    0, 1317,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1317,    0,    0,    0,    0, 1317, 1317,
    0,    0,    0,    0,    0, 1317,    0,    0,    0, 1317,
    0, 1317, 1317, 1317, 1317,    0, 1317,    0,    0,    0,
    0,    0,    0,    0, 1317, 1317,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1317, 1317,    0, 1317, 1317,
 1317,    0,    0,    0,    0, 1317, 1317,    0, 1317, 1317,
    0, 1317,    0,    0, 1317, 1317,    0,    0,    0, 1317,
 1317,    0,    0,    0,    0, 1317, 1317, 1317,    0, 1317,
 1317, 1317,    0, 1317,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1317, 1317, 1317, 1317, 1317, 1317, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,
 1317, 1317, 1317, 1317,    0, 1317, 1317, 1317, 1317, 1317,
 1317,    0, 1317, 1317, 1317, 1317,    0, 1317,    0, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317,    0, 1317, 1317,
 1317, 1317,    0, 1317, 1317, 1317, 1317, 1317,    0, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,
 1317, 1317,    0, 1317, 1317, 1317, 1317, 1317, 1317,    0,
 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,    0, 1317,
 1317,    0,    0,    0, 1317, 1317, 1317, 1317, 1317, 1317,
    0,    0, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,
 1317,    0,    0,    0,    0,    0, 1317,    0, 1317, 1317,
 1317, 1317, 1317, 1317,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1317, 1317,    0, 1317, 1317, 1317, 1317, 1317,
 1317, 1317, 1317, 1317,    0, 1317, 1317, 1317, 1317, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317,    0, 1317, 1317,
 1317,    0, 1317,    0,    0,    0, 1317, 1317, 1317, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,    0,
 1317, 1317,    0,    0,    0,    0,    0, 1317, 1317, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,
 1317,    0,    0, 1317, 1317,    0,    0,    0,    0, 1317,
 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317, 1317,
 1317, 1317, 1317, 1317, 1317,  171,    0,  171,    0,    0,
    0,    0,    0,  171,    0,    0,  171,  171,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  171,  171,    0,
  171,    0,  171,    0,    0,  171,    0,    0,  171,    0,
    0,    0,    0,    0,    0,  171,    0,  171,  171,    0,
    0,    0,  171,    0,  171,    0,    0,  171,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  171,  171,    0,
    0,    0,  171,    0,  171,    0,  171,    0,    0,    0,
    0,    0,  171,    0,  171,    0,    0,    0,  171,    0,
    0,    0,    0,    0,    0,    0,  171,    0,    0,  171,
    0,  171,    0,    0,    0,  171,  171,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  171,    0,
    0,    0,  171,  171,  171,  171,  171,    0,  171,    0,
    0,    0,    0,    0,  171,    0,  171,  171,    0,    0,
  171,  171,    0,    0,    0,    0,    0,  171,    0,    0,
  171,  171,  171,  171,    0,  171,    0,  171,    0,  171,
  171,  171,  171,  171,    0,    0,    0,    0,  171,    0,
  171,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  171,    0,    0,    0,    0,  171,  171,    0,    0,
    0,    0,    0,  171,    0,    0,    0,  171,    0,  171,
  171,  171,  171,    0,  171,    0,    0,    0,    0,    0,
    0,    0,  171,  171,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  171,  171,    0,  171,  171,  171,    0,
    0,    0,    0,  171,  171,    0,  171,  171,    0,  171,
    0,    0,  171,  171,    0,    0,    0,  171,  171,    0,
    0,    0,    0,  171,  171,  171,    0,  171,  171,  171,
    0,  171,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  171,  171,  171,  171,  171,  171,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,
  171,  171,    0,  171,  171,  171,  171,  171,  171,    0,
  171,  171,  171,  171,    0,  171,    0,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,
  171,  171,  171,  171,  171,    0,  171,  171,  171,  171,
    0,  171,  171,  171,  171,  171,    0,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,
    0,  171,  171,  171,  171,  171,  171,    0,  171,  171,
  171,  171,  171,  171,  171,  171,    0,  171,  171,    0,
    0,    0,  171,  171,  171,  171,  171,  171,    0,    0,
  171,  171,  171,  171,  171,  171,  171,  171,  171,    0,
    0,    0,    0,    0,  171,    0,  171,  171,  171,  171,
  171,  171,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  171,  171,    0,  171,  171,  171,  171,  171,  171,  171,
  171,  171,    0,  171,  171,  171,  171,  171,  171,  171,
  171,  171,  171,  171,  171,    0,  171,  171,  171,    0,
  171,    0,    0,    0,  171,  171,  171,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,    0,  171,  171,
    0,    0,    0,    0,    0,  171,  171,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  171,    0,
    0,  171,  171,    0,    0,    0,    0,  171,  171,  171,
  171,  171,  171,  171,  171,  171,  171,  171,  171,  171,
  171,  171,  171,   69,    0,   69,    0,    0,    0,    0,
    0,   69,    0,    0,   69,   69,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   69,   69,    0,   69,    0,
   69,    0,    0,   69,    0,    0,   69,    0,    0,    0,
    0,    0,    0,   69,    0,   69,   69,    0,    0,    0,
   69,    0,   69,    0,    0,   69,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   69,   69,    0,    0,    0,
   69,    0,   69,    0,   69,    0,    0,    0,    0,    0,
   69,    0,   69,    0,    0,    0,   69,    0,    0,    0,
    0,    0,    0,    0,   69,    0,    0,   69,    0,   69,
    0,    0,    0,   69,   69,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   69,    0,    0,    0,
   69,   69,   69,   69,   69,    0,   69,    0,    0,    0,
    0,    0,   69,    0,   69,   69,    0,    0,   69,   69,
    0,    0,    0,    0,    0,   69,    0,    0,   69,   69,
   69,   69,    0,   69,    0,   69,    0,   69,   69,   69,
   69,   69,    0,    0,    0,    0,   69,    0,   69,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   69,
    0,    0,    0,    0,   69,   69,    0,    0,    0,    0,
    0,   69,    0,    0,    0,   69,    0,   69,   69,   69,
   69,    0,   69,    0,    0,    0,    0,    0,    0,    0,
   69,   69,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   69,   69,    0,   69,   69,   69,    0,    0,    0,
    0,   69,   69,    0,   69,   69,    0,   69,    0,    0,
   69,   69,    0,    0,    0,   69,   69,    0,    0,    0,
    0,   69,   69,   69,    0,   69,   69,   69,    0,   69,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   69,
   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,
   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,
    0,   69,   69,   69,   69,   69,   69,    0,   69,   69,
   69,   69,    0,   69,    0,   69,   69,   69,   69,   69,
   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,
   69,   69,   69,    0,   69,   69,   69,   69,    0,   69,
   69,   69,   69,   69,    0,   69,   69,   69,   69,   69,
   69,   69,   69,   69,   69,   69,   69,   69,    0,   69,
   69,   69,   69,   69,   69,    0,   69,   69,   69,   69,
   69,   69,   69,   69,    0,   69,   69,    0,    0,    0,
   69,   69,   69,   69,   69,   69,    0,    0,   69,   69,
   69,   69,   69,   69,   69,   69,   69,    0,    0,    0,
    0,    0,   69,    0,   69,   69,   69,   69,   69,   69,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   69,   69,
    0,   69,   69,   69,   69,   69,   69,   69,   69,   69,
    0,   69,   69,   69,   69,   69,   69,   69,   69,   69,
   69,   69,   69,    0,   69,   69,   69,    0,   69,    0,
    0,    0,   69,   69,   69,   69,   69,   69,   69,   69,
   69,   69,   69,   69,   69,    0,   69,   69,    0,    0,
    0,    0,    0,   69,   69,   69,   69,   69,   69,   69,
   69,   69,   69,   69,   69,   69,   69,    0,    0,   69,
   69,    0,    0,    0,    0,   69,   69,   69,   69,   69,
   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,
   69,  602,    0,  602,    0,    0,    0,    0,    0,  602,
    0,    0,  602,  602,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  602,  602,    0,  602,    0,  602,    0,
    0,  602,    0,    0,  602,    0,    0,    0,    0,    0,
    0,  602,    0,  602,  602,    0,    0,    0,  602,    0,
  602,    0,    0,  602,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  602,  602,    0,    0,    0,  602,    0,
  602,    0,  602,    0,    0,    0,    0,    0,  602,    0,
  602,    0,    0,    0,  602,    0,    0,    0,    0,    0,
    0,    0,  602,    0,    0,  602,    0,  602,    0,    0,
    0,  602,  602,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  602,    0,    0,    0,  602,  602,
  602,  602,  602,    0,  602,    0,    0,    0,    0,    0,
  602,    0,  602,  602,    0,    0,  602,  602,    0,    0,
    0,    0,    0,  602,    0,    0,  602,  602,  602,  602,
    0,  602,    0,  602,    0,  602,  602,  602,  602,  602,
    0,    0,    0,    0,  602,    0,  602,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  602,    0,    0,
    0,    0,  602,  602,    0,    0,    0,    0,    0,  602,
    0,    0,    0,  602,    0,  602,  602,  602,  602,    0,
  602,    0,    0,    0,    0,    0,    0,    0,  602,  602,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  602,
  602,    0,  602,  602,  602,    0,    0,    0,    0,  602,
  602,    0,  602,  602,    0,  602,    0,    0,  602,  602,
    0,    0,    0,  602,  602,    0,    0,    0,    0,  602,
  602,  602,    0,  602,  602,  602,    0,  602,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  602,  602,  602,
  602,  602,  602,  602,  602,  602,  602,  602,  602,  602,
  602,  602,  602,  602,  602,  602,  602,  602,    0,  602,
  602,  602,  602,  602,  602,    0,  602,  602,  602,  602,
    0,  602,    0,  602,  602,  602,  602,  602,  602,  602,
  602,  602,  602,  602,  602,  602,  602,  602,  602,  602,
  602,    0,  602,  602,  602,  602,    0,  602,  602,  602,
  602,  602,    0,  602,  602,  602,  602,  602,  602,  602,
  602,  602,  602,  602,  602,  602,    0,  602,  602,  602,
  602,  602,  602,    0,  602,  602,  602,  602,  602,  602,
  602,  602,    0,  602,  602,    0,    0,    0,  602,  602,
  602,  602,  602,  602,    0,    0,  602,  602,  602,  602,
  602,  602,  602,  602,  602,    0,    0,    0,    0,    0,
  602,    0,  602,  602,  602,  602,  602,  602,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  602,  602,    0,  602,
  602,  602,  602,  602,  602,  602,  602,  602,    0,  602,
  602,  602,  602,  602,  602,  602,  602,  602,  602,  602,
  602,    0,  602,  602,  602,    0,  602,    0,    0,    0,
  602,  602,  602,  602,  602,  602,  602,  602,  602,  602,
  602,  602,  602,    0,  602,  602,    0,    0,    0,    0,
    0,  602,  602,  602,  602,  602,  602,  602,  602,  602,
  602,  602,  602,  602,  602,    0,    0,  602,  602,    0,
    0,    0,    0,  602,  602,  602,  602,  602,  602,  602,
  602,  602,  602,  602,  602,  602,  602,  602,  602,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,  229,  230,    0,    0,    0,    0,  231,  232,    0,
  233,  234,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,  243,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,  335,  336,  337,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
    0,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  157,    0,  158,
    0,    0,    0,    0,    0,  159,    0,    0,  160,  161,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,
  163,    0,  164,    0,  165,    0,    0,  166,    0,    0,
  167,    0,    0,    0,    0,    0,    0,  168,    0,  169,
  170,    0,    0,    0,  171,    0,  172,    0,    0,  173,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  175,    0,    0,    0,  176,    0,  177,    0,  178,    0,
    0,    0,    0,    0,  179,    0,  180,    0,    0,    0,
  181,    0,    0,    0,    0,    0,    0,    0,  182,    0,
    0,  183,    0,  184,    0,    0,    0,  185,  186,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  187,    0,    0,    0,  189,  190,  191,  192,  193,    0,
  194,    0,    0,    0,    0,    0,  195,    0,  196,  197,
    0,    0,  198,  199,    0,    0,    0,    0,    0,  200,
    0,    0,  201,  202,  203,  204,    0,  205,    0,  206,
    0,  207,  208,  209,  210,  211,    0,    0,    0,    0,
  212,    0,  213,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  214,    0,    0,    0,    0,  215,  216,
    0,    0,    0,    0,    0,  217,    0,    0,    0,  218,
    0,  219,  220,  221,  222,    0,  223,    0,    0,    0,
    0,    0,    0,    0,  224,  225,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  226,  227,    0,  228,  229,
  230,    0,    0,    0,    0,  231,  232,    0,  233,  234,
    0,  235,    0,    0,  236,  237,    0,    0,    0,  238,
  239,    0,    0,    0,    0,  240,  241,  242,    0,  243,
  244,  245,    0,  246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  247,  248,  249,  250,  251,  252,  253,
  254,  255,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,    0,  268,  269,  270,  271,  272,
  273,    0,  274,  275,  276,  277,    0,  278,    0,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,    0,  297,  298,
  299,  300,    0,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,    0,  315,  316,
  317,  318,    0,  319,  320,  321,  322,  323,  324,    0,
  325,  326,  327,  328,  329,  330,  331,  332,    0,  333,
  334,    0,    0,    0,  335,  336,  337,  338,  339,  340,
    0,    0,  341,  342,  343,  344,  345,  346,  347,  348,
  349,    0,    0,    0,    0,    0,  350,    0,  351,  352,
  353,  354,  355,  356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  357,  358,    0,  359,  360,  361,  362,  363,
  364,  365,  366,  367,    0,  368,  369,  370,  371,  372,
  373,  374,  375,  376,  377,  378,  379,    0,  380,  381,
  382,    0,  383,    0,    0,    0,  384,  385,  386,  387,
  388,  389,  390,  391,  392,  393,  394,  395,  396,    0,
  397,  398,    0,    0,    0,    0,    0,  399,  400,  401,
  402,  403,  404,  405,  406,  407,  408,  409,  410,  411,
  412,    0,    0,  413,  414,    0,    0,    0,    0,  415,
  416,  417,  418,  419,  420,  421,  422,  423,  424,  425,
  426,  427,  428,  429,  430,  157,    0,  158,    0,    0,
    0,    0,    0,  159,    0,    0,  160,  161,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  162,  163,    0,
  164,    0,  165,    0,    0,  166,    0,    0,  167,    0,
    0,    0,    0,    0,    0,  168,    0,  169,  170,    0,
    0,    0,  171,    0,  172,    0,    0,  173,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  174,  175,    0,
    0,    0,  176,    0,  177,    0,  178,    0,    0,    0,
    0,    0,  179,    0,  180,    0,    0,    0,  181,    0,
    0,    0,    0,    0,    0,    0,  182,    0,    0,  183,
    0,  184,    0,    0,    0,  185,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,    0,
    0,    0,  189,  190,  191,  192,  193,    0,  194,    0,
    0,    0,    0,    0,  195,    0,  196,  197,    0,    0,
  198,  199,    0,    0,    0,    0,    0,  200,    0,    0,
  201,  202,  203,  204,    0,  205,    0,  206,    0,  207,
  208,  209,  210,  211,    0,    0,    0,    0,  212,    0,
  213,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  214,    0,    0,    0,    0,  215,  216,    0,    0,
    0,    0,    0,  217,    0,    0,    0,  218,    0,  219,
  220,  221,  222,    0,  223,    0,    0,    0,    0,    0,
    0,    0,  224,  225,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  226,  227,    0,  228,  229,  230,    0,
    0,    0,    0,  231,  232,    0,  233,  234,    0,  235,
    0,    0,  236,  237,    0,    0,    0,  238,  239,    0,
    0,    0,    0,  240,  241,  242,    0,  243,  244,  245,
    0,  246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  248,  249,  250,  251,  252,  253,  254,  255,
  256,  257,  258,  259,  260,  261,  262,  263,  264,  265,
  266,  267,    0,  268,  269,  270,  271,  272,  273,    0,
  274,  275,  276,  277,    0,  278,    0,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,    0,  297,  298,  299,  300,
    0,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
    0,  319,  320,  321,  322,  323,  324,    0,  325,  326,
  327,  328,  329,  330,  331,  332,    0,  333,  334,    0,
    0,    0,  335,  336,  337,  338,  339,  340,    0,    0,
  341,  342,  343,  344,  345,  346,  347,  348,  349,    0,
    0,    0,    0,    0,  350,    0,  351,  352,  353,  354,
  355,  356,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  357,  358,    0,  359,  360,  361,  362,  363,  364,  365,
  366,  367,    0,  368,  369,  370,  371,  372,  373,  374,
  375,  376,  377,  378,  379,    0,  380,  381,  382,    0,
  383,    0,    0,    0,  384,  385,  386,  387,  388,  389,
  390,  391,  392,  393,  394,  395,  396,    0,  397,  398,
    0,    0,    0,    0,    0,  399,  400,  401,  402,  403,
  404,  405,  406,  407,  408,  409,  410,  411,  412,    0,
    0,  413,  414,    0,    0,    0,    0,  415,  416,  417,
  418,  419,  420,  421,  422,  423,  424,    0,  426,  427,
  428,  429,  430,  157,    0,  158,    0,    0,    0,    0,
    0,  159,    0,    0,  160,  161,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  162,  163,    0,  164,    0,
    0,    0,    0,  166,    0,    0,  167,    0,    0,    0,
    0,    0,    0,  168,    0,  169,  170,    0,    0,    0,
  171,    0,  172,    0,    0,  173,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  174,  175,    0,    0,    0,
  176,    0,  177,    0,  178,    0,    0,    0,    0,    0,
  179,    0,  180,    0,    0,    0,  181,    0,    0,    0,
    0,    0,    0,    0,  182,    0,    0,  183,    0,  184,
    0,    0,    0,  185,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  187,    0,    0,    0,
  189,  190,  191,  192,  193,    0,  194,    0,    0,    0,
    0,    0,  195,    0,  196,  197,    0,    0,  198,  199,
    0,    0,    0,    0,    0,  200,    0,    0,  201,  202,
  203,  204,    0,  205,    0,  206,    0,  207,    0,  209,
  210,  211,    0,    0,    0,    0,  212,    0,  213,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,    0,    0,  215,  216,    0,    0,    0,    0,
    0,  217,    0,    0,    0,  218,    0,  219,  220,  221,
  222,    0,  223,    0,    0,    0,    0,    0,    0,    0,
  224,  225,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  226,  227,    0,  228,  229,  230,    0,    0,    0,
    0,  231,  232,    0,  233,  234,    0,  235,    0,    0,
  236,  237,    0,    0,    0,  238,  239,    0,    0,    0,
    0,  240,  241,  242,    0,  243,  244,  245,    0,  246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  247,
  248,  249,  250,  251,  252,  253,  254,  255,  256,  257,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
    0,  268,  269,  270,  271,  272,  273,    0,  274,  275,
  276,  277,    0,  278,    0,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,    0,  297,  298,  299,  300,    0,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,  314,  315,  316,  317,  318,    0,  319,
  320,  321,  322,  323,  324,    0,  325,  326,  327,  328,
  329,  330,  331,  332,    0,  333,  334,    0,    0,    0,
  335,  336,  337,  338,  339,  340,    0,    0,  341,  342,
  343,  344,  345,  346,  347,  348,  349,    0,    0,    0,
    0,    0,  350,    0,  351,  352,  353,  354,  355,  356,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  357,  358,
    0,  359,  360,  361,  362,  363,  364,  365,  366,  367,
    0,  368,  369,  370,  371,  372,  373,  374,  375,  376,
  377,  378,  379,    0,  380,  381,  382,    0,  383,    0,
    0,    0,  384,  385,  386,  387,  388,  389,  390,  391,
  392,  393,  394,  395,  396,    0,  397,  398,    0,    0,
    0,    0,    0,  399,  400,  401,  402,  403,  404,  405,
  406,  407,  408,  409,  410,  411,  412,    0,    0,  413,
  414,    0,    0,    0,    0,  415,  416,  417,  418,  419,
  420,  421,  422,  423,  424,  425,  426,  427,  428,  429,
  430,  157,    0,  158,    0,    0,    0,    0,    0,  159,
    0,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  162,  163,    0,  164,    0,  165,    0,
    0,  166,    0,    0,  167,    0,    0,    0,    0,    0,
    0,  168,    0,  169,  170,    0,    0,    0,  171,    0,
  172,    0,    0,  173,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,  175,    0,    0,    0,  176,    0,
  177,    0,  178,    0,    0,    0,    0,    0,  179,    0,
  180,    0,    0,    0,  181,    0,    0,    0,    0,    0,
    0,    0,  182,    0,    0,  183,    0,  184,    0,    0,
    0,  185,  186,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  187,    0,    0,    0,  189,  190,
  191,  192,  193,    0,  194,    0,    0,    0,    0,    0,
  195,    0,  196,  197,    0,    0,  198,  199,    0,    0,
    0,    0,    0,  200,    0,    0,  201,  202,  203,  204,
    0,  205,    0,  206,    0,  207,  208,  209,  210,  211,
    0,    0,    0,    0,  212,    0,  213,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  214,    0,    0,
    0,    0,  215,  216,    0,    0,    0,    0,    0,  217,
    0,    0,    0,  218,    0,  219,  220,  221,  222,    0,
  223,    0,    0,    0,    0,    0,    0,    0,  224,  225,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  226,
  227,    0,  228,  229,  230,    0,    0,    0,    0,  231,
  232,    0,  233,  234,    0,  235,    0,    0,  236,  237,
    0,    0,    0,  238,  239,    0,    0,    0,    0,  240,
  241,  242,    0,  243,  244,  245,    0,  246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  247,  248,  249,
  250,  251,  252,  253,  254,  255,  256,  257,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,    0,  268,
  269,  270,  271,  272,  273,    0,  274,  275,  276,  277,
    0,  278,    0,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,    0,  297,  298,  299,  300,    0,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,  314,  315,  316,  317,  318,    0,  319,  320,  321,
  322,  323,  324,    0,  325,  326,  327,  328,  329,  330,
  331,  332,    0,  333,    0,    0,    0,    0,  335,  336,
  337,  338,  339,  340,    0,    0,  341,  342,  343,  344,
  345,  346,  347,  348,  349,    0,    0,    0,    0,    0,
  350,    0,  351,  352,  353,  354,  355,  356,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  357,  358,    0,  359,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  368,
  369,  370,  371,    0,  373,  374,  375,  376,  377,  378,
  379,    0,  380,  381,  382,    0,  383,    0,    0,    0,
  384,  385,  386,  387,  388,  389,  390,  391,  392,  393,
  394,  395,  396,    0,  397,    0,    0,    0,    0,    0,
    0,  399,  400,  401,  402,  403,  404,  405,  406,  407,
  408,  409,  410,  411,  412,    0,    0,  413,  414,    0,
    0,    0,    0,  415,  416,  417,  418,  419,  420,  421,
  422,  423,  424,  425,  426,  427,  428,  429,  430,  157,
    0,  158,    0,    0,    0,    0,    0,  159,    0,    0,
  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  162,  163,    0,  164,    0,  165,    0,    0,  166,
    0,    0,  167,    0,    0,    0,    0,    0,    0,  168,
    0,  169,  170,    0,    0,    0,  171,    0,  172,    0,
    0,  173,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  174,  175,    0,    0,    0,  176,    0,  177,    0,
  178,    0,    0,    0,    0,    0,  179,    0,  180,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
  182,    0,    0,  183,    0,  184,    0,    0,    0,  185,
  186,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  187,    0,    0,    0,  189,  190,  191,  192,
  193,    0,  194,    0,    0,    0,    0,    0,  195,    0,
  196,  197,    0,    0,  198,  199,    0,    0,    0,    0,
    0,  200,    0,    0,  201,  202,  203,  204,    0,  205,
    0,  206,    0,  207,  208,  209,  210,  211,    0,    0,
    0,    0,  212,    0,  213,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  214,    0,    0,    0,    0,
  215,  216,    0,    0,    0,    0,    0,  217,    0,    0,
    0,  218,    0,  219,  220,  221,  222,    0,  223,    0,
    0,    0,    0,    0,    0,    0,  224,  225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  226,  227,    0,
  228,    0,  230,    0,    0,    0,    0,  231,  232,    0,
  233,    0,    0,  235,    0,    0,  236,  237,    0,    0,
    0,  238,  239,    0,    0,    0,    0,  240,  241,  242,
    0,    0,  244,  245,    0,  246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  247,  248,  249,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,    0,  268,  269,  270,
  271,  272,  273,    0,  274,  275,  276,  277,    0,  278,
    0,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,    0,
  297,  298,  299,  300,    0,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,    0,  319,  320,  321,  322,  323,
  324,    0,  325,  326,  327,  328,  329,  330,  331,  332,
    0,  333,  334,    0,    0,    0,    0,    0,    0,  338,
  339,  340,    0,    0,  341,  342,  343,  344,  345,  346,
  347,  348,  349,    0,    0,    0,    0,    0,  350,    0,
  351,  352,  353,  354,  355,  356,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  357,  358,    0,  359,  360,  361,
  362,  363,  364,  365,  366,  367,    0,  368,  369,  370,
  371,  372,  373,  374,  375,  376,  377,  378,  379,    0,
  380,  381,  382,    0,  383,    0,    0,    0,  384,  385,
  386,  387,  388,  389,  390,  391,  392,  393,  394,  395,
  396,    0,  397,  398,    0,    0,    0,    0,    0,  399,
  400,  401,  402,  403,  404,  405,  406,  407,  408,  409,
  410,  411,  412,    0,    0,  413,  414,    0,    0,    0,
    0,  415,  416,  417,  418,  419,  420,  421,  422,  423,
  424,  425,  426,  427,  428,  429,  430,  181,    0,    0,
  181,    0,    0,    0,    0,    0, 2959,    0,    0, 1144,
    0,    0,  181,    0,    0,    0,  181,    0,  181,    0,
    0, 2960,    0,    0,    0, 1145,    0, 1146,    0,    0,
    0,    0, 1143,    0,    0, 1144,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1145,    0, 1146,    0,    0,  181,    0,    0,    0,
    0,    0,  181,  181,  181, 1147,  181,  181,  181,    0,
  181, 1148, 1149, 1150,    0, 1151, 1152, 1153,    0, 1154,
    0,    0,    0,    0,  181,    0,  181,    0,    0,    0,
    0, 1147,    0, 1155,    0, 1156,    0, 1148, 1149, 1150,
    0, 1151, 1152, 1153,  181, 1154, 1143,    0,    0, 1144,
    0,    0,    0,    0,    0,  181,    0,    0,    0, 1155,
    0, 1156,    0,    0, 1157, 1145,    0, 1146,    0,    0,
    0,  181,    0,    0,    0,    0,    0,    0,    0,    0,
 1158,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1157,    0,    0,  181,    0,    0,    0,    0,    0,    0,
    0,    0, 1159,    0,  282, 1147, 1158,  282,    0,    0,
    0, 1148, 1149, 1150,    0, 1151, 1152, 1153,    0, 1154,
    0,    0,    0,  282,    0,  282,    0,    0, 1159,    0,
    0,  181,  181, 1155,    0, 1156,    0,    0,    0,    0,
 1287, 1161,    0,    0,    0,    0,    0,    0,  181,    0,
    0,    0,    0,    0,    0,    0,    0, 1162,    0,    0,
    0,    0,    0,  282, 1157,    0, 1160, 1161,    0,  282,
  282,  282,    0,  282,  282,  282,    0,  282,    0,    0,
 1158,    0,    0, 1162,    0,    0,    0,    0,    0,    0,
    0,  282,    0,  282,    0,    0,    0,    0,    0,    0,
    0,    0, 1159,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  282,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  282,    0,
 1287, 1161,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1162,    0,    0,
  282,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  282,  282,
    0,    0,    0,    0,    0,    0,    0,  181,    0,    0,
    0,    0,    0,    0,    0,  282, 1163,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  181,
    0,    0, 1163,    0,    0,    0,    0,    0, 1164,  181,
    0,    0,    0,    0,    0,    0,    0,    0, 1165,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
    0, 1166,    0,    0, 1164,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1165,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1166,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  181,    0,    0,    0,    0, 1163,    0,    0,    0,
 1167,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1167,    0, 1164,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1165,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1166,    0,    0,  282,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  282,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  282,    0,    0,    0,
 1167,    0,    0,    0,    0,    0,    0,    0,    0,  282,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  282,
};
static int yycheck[] = {                                      14,
   15,   78,   17,   74,  133,   93,    0,   82,  126,  768,
  588,  122,  861,   93,  702,  858,  125,  848,  699, 1046,
   15,   14, 1372, 1046,   84,   81,  795, 1534, 1378,  138,
  763,  764, 1597, 1472, 1241,  613, 1281, 1219,  704,  822,
 1241,  588, 1990,  847,  822, 2054,  106,  124,  119,  806,
 2367, 2304, 1270, 2306,  129,  824, 1960, 1241,   73,   74,
 1680,   76,   77,   78,   79,   80,   81,   82,   83,   84,
   85,  131,  128,   88, 1680,  763,  764, 1705, 1046,   94,
   95,   96,   97, 2186,   99, 1680,  101,  102,  783,  104,
  105,  106,  107,  108,  641, 1680,  111, 1680, 2083, 2066,
  115, 2066,  117,  118,  119,   14,  121,  122,  123,  124,
  125,   77,  127,  128,  129,  130,  131,  132,  133,   85,
 1440,  136, 2765,  138,  117,  820,  141,  142, 1637, 1638,
  145, 2046, 2453,  148,  127, 2443, 2444, 2445, 1044, 1045,
 1046, 1268, 2543, 2544, 1044, 1045,  799, 2844, 1495,  142,
 1647,   88, 1842, 1575, 2767, 1592,  822,  123, 1580, 2491,
 2693, 1658,   94, 2554, 2496, 2874,  132, 2403, 2162, 2695,
  684,   80, 1618, 2604,  111, 1707, 1683, 1684,  704,  145,
 1629, 1630, 1323, 2491, 2696,  852, 1563,  756,  757, 2056,
 1241,  844,  101, 1851, 2130, 1322, 2129,   96, 2145,  136,
 3206,  138, 1699,  105,   44,    0,    0,  111,  117,   41,
  110,  455,  653,    0,  598, 2912, 2165,   41,   41,    0,
 1247,    0, 1668,   41,   59,   41,    0,   41,   41,   99,
 3600,   41,  141,  142,  264,  135,   41, 1421,    0, 1616,
    0, 1448,  683, 1478,   41,   44,   41,    0,   40,   44,
    0,  275, 1279,  316,   41,  801, 1440, 1648,    0, 1654,
   40, 2147,   41,    0,   59,   59,   59,   41, 2706,   44,
 1710,   42,   59,   41,    0,  264, 2147,  310,   59,   41,
   59,   41,   59, 3334,    0,   59,    0,   44,   41, 2727,
  446,  264,   40,    0, 2827,  466,    0,   59, 1562,   59,
    0,  264,    0,   41, 1165, 2838,   59,    0,  260,   59,
 2146, 3029,  301,    0, 1341,    0,    0,   59, 1341,    0,
   40,  310,   59, 2835,  363,   41, 2146,   41, 1355,    0,
  364,    0,   41,   59,   44, 1362,    0, 1564,  301,    0,
 2030,   59,  364,   59,   44,   59,  296,  310,  268,  273,
  288, 1558,   59,  867, 2140,   59,  262, 1558,    0,   59,
   41,   59,  500, 1648,    0,  426,   59, 2758,   59, 2322,
  264,   41,   59, 2906,   59,   59,  607,  264,   59,   41,
 1386, 1387,   44,  504,  322,   41,  302,  303,   59,  401,
   59,    0,  347, 1361,  301,   59,  330,   59,   59,   61,
 1427,    0,    0,   44, 1427,   41,  259, 2416,  371,  257,
  276,  334,  334,  389, 1764,  480,  269,   59,  316,  382,
   41,  436,   41,   59,  264,  363,  441,  442,  443,  444,
  445,  446,  447,  448,  449, 1341,  451,    0,  275,  301,
  455,  436, 2346,   40, 2348,  117, 2958, 1251,  310,   41,
   59, 2310,   41,  468,  438,   44, 1137, 1897,  523,  570,
   59,   59, 1219,  439,  312,  264,   63,  389, 1495,  264,
  142,  264, 1276, 1680, 1442,  310, 3185,  264,  326, 1680,
  568,   44,    0,  264,  407,  264, 2453,  264, 2453,  387,
  264,  425,  263,  523,  401,    0,   59, 1982,  364,  511,
  337, 1189,  264,   44,  264, 1473, 1943, 1558, 1241, 1477,
  648,  264,  418,  433,  451,  368,  368,  312,  434,   59,
  301, 1427, 1842,  478, 1367, 1294, 1369, 2630,  452,  310,
  761,    0,  703,  310,  655,    0,  445,  332, 1444, 1566,
  628,   59,  564,  442, 1444,  332,  448,  451,  264,  660,
  264,   45,  312,  332,   59,  570, 2029,  668,  332, 1465,
 1466, 1467, 1468, 1469,  264, 1465, 1466, 1467, 1468, 1469,
  332,  364,  332,  264,  444,  259,   40,  264, 2000,  332,
  264, 2003, 2004,  264,  623,  269, 1345,  368, 1938,  299,
   59, 1286,  310,  264,   59,  304,  312,  631,  312,   63,
  264,  382,  264,  264,  372,    0,  322,  323,    0, 3327,
 2268,   40,   44,  701,  301,  277,  332,  301,  332,  310,
  400,  698,  264,  310,  597,  575,  310,    0,  264,  523,
  301,  312,   40,  295,  304,  427,  523,  301, 2078,  310,
  655, 3692,  437, 3081, 3082,  660,  310,  427,  310, 1856,
  437,  332,   45,  668,    0,  264,  372,  437, 1842,  301,
 2896,  295,  437, 2010,   59,  381, 2115,   59,  310,  684,
  685,  746, 1533,  829,   45,  437,  312,  437,  739, 2070,
  400,  368,  430,  698,  437,  480,   59,  750, 1421,  446,
  705,    0,    0,  708,  646,  382,  332, 1211,  382,  504,
  715,  264,  717,  718, 3036,  738,  503,  423,  503,  504,
 2030,  382, 2598,   59,  738,  730,  372,    0,  655,  435,
  715,  437,  717,  437,  264, 1271,  368, 2598,  523,  759,
  480,  746,   41,   41,  557,   44,  504,  437,  480,  557,
  382,  756,  757,  758,  759,   59,  264, 2011,  685,  738,
   59,   59,  840,  372,  825,  830,  437,  597, 2153,  264,
  840, 2597,  504,  523, 1568,  503, 1615, 2720, 4138, 1210,
  310, 4141,    0,  523,  511,  738,   59, 2597,  855, 2014,
 1786,  523, 2568,  301,   93, 2070, 2013,  503,  504,  503,
  504,  806,  310,  730,  435,  264,  301,  523,  597,  480,
  815,  437,  740,    0,  511,  310, 1563,  523, 3341,  523,
  825,  826,  827,  828,  829,  830,  831,  832, 3150,  523,
  835,    0, 2029, 1666, 1851,  523,  511, 1515, 2029,  600,
  295,   59,  301,  848,  849,  504,  435,  852,  631, 1520,
  855,  310,  523,  858, 2351, 2029, 2030,   44,  504,  446,
  368,  588,  867,  753,  523,  395, 3287, 3288,  655, 1616,
  655,  827,   59,  368,  382,  759,  832,  503,  678, 1310,
 1713,  681,  759,  689,  437,  340,  738,  382, 2737,  650,
   59, 2366, 1548, 1851,  689,  504,  738,  523, 1657,    0,
 1134,  264,  686,  687,  689,  588,  437,  597,  835,  368,
 1704, 1928,  734,  738, 2853, 3343, 3344, 3345, 3346, 3347,
 3348, 3349, 1669,  382,    0,  652,  721, 1686, 2391, 1688,
 2270,  689, 1306,  715,  756,  325,  721,  826,  301,  831,
 2250,  835,  756,  756,  834,  715,  597,  310,  756,  655,
  849,  655,  756,  756,  738,  652,   40,   45,   59,    0,
  264,  405,  446,  721, 1860,   41,  264,  738,  828,  652,
 1860,  738, 2411, 2621,  759, 3201,   40,  652, 3974,  631,
 2917,  400,  759,   59, 1942,  401, 2323,  372, 2029, 2915,
  759, 2914,  446,  280, 1679,  759,  381,  419,  597,    0,
   41, 2368,  400,   44,   40,  368,  310,  759, 2139,  759,
 2867,  393,  260,  260,  312,  260,  759, 1576,   59,  382,
 1579,   59, 1121,    0,  648,   40,   44, 1126,    0, 2550,
  738, 2398, 1548,  295,  332, 1692,  372,  419,  437,  306,
 2562,  747, 1047,  689, 3546,  500,  264,   44,   41,    0,
  317,   44,   93,  759, 2551,  759, 2553,  738,   59,  307,
  364,  738, 1047,  446,  738, 1569,   59,  678,   61,  680,
  681,  682,    0,  372, 3773,  721, 2250,  738,  107, 3500,
  273,   44,   59,  301,  738,  446,  738,   59,  759,   44,
  363, 3287,  310,   44, 3610,  394,  678, 3414,  680,  681,
  682,    0, 3624,    0, 2570, 2571,  738,    0,   59,  580,
    0,  478,    0,  262, 3425,  278, 1121,    0, 3102,  504,
    0, 1126,  260,   44, 3435, 2561,   59, 1960,  295, 1134,
 1667,   59,  387,  759, 1212,  504,    0,  268,  523,  437,
  507, 2821,   41,  410, 2824,   44,   40,   44,  389, 1970,
  368,    0, 1991, 3581, 3582, 3583, 3584, 3585, 3586, 3587,
   59,   44,   59, 3162,  382,  312,   59,  279,  504,   59,
  264,   59,  417,    0,  447,  448,   59, 1255,  437,   59,
 1258,    0,  480, 1261,  437, 1255, 1264,  523,  264, 1194,
    0, 1261,  296, 3891, 1264,   59,  264, 1202,  439, 1126,
  447,  448,  400,  372, 1998,  504, 1211,  301,  738, 2853,
   59,  484,  381,  312, 1219, 3308,  310,    0, 1296,    0,
 3743,   44,  353,    0,  314,  523,  264, 3749,  312,  427,
  738,    0,   59,  301, 2192,  398,  312, 1315, 1316,  437,
   59,  273,  310,  738, 3943, 1315, 1316, 1232,    0,   59,
  505, 2268,    0, 3479,  262, 1260,  332,  530,   41, 1244,
   41,  651,   40,  264,  451,  434,  453,   44,  555,  418,
   93,  264,   41,  744,    0,   44,   59,  371,   59,  738,
  458,  459,   59,   41,  277,  654,   44, 1965,  382,  436,
   59, 3608,   44,  290,  332, 1194,  314,    0,  571, 2726,
  301, 3682,  295, 1202, 3500, 3616, 2029,   59,  752,  310,
  422,   59,  333,  264,  382,   41,  400,  310,  419, 2066,
  419, 2279, 3620,    0,  504,    0,  258,  631,    0,  261,
  368,  372, 2950,   59,  280,  290,  400,  753, 2132,  532,
  268,  479,  380,   59, 1349, 1464, 2833, 2834, 2142,  748,
 2847, 1356,  436,  394,  523,  301,   59, 2253,  446, 2771,
 3883,  437, 3885, 2253,  400,    0,  299,  368, 4067,  301,
  632,  364, 2131, 2269,  641,  738, 2493, 2494, 1479, 2269,
 3702,  382,   59,  471,   59,  400, 2907,   59,  555,    0,
  689,  427, 2165,  314, 2881,  372, 1495, 2165,  646,  437,
 1356,  437, 2058, 2891,  480,  368,  334,  295,  301,  303,
  418,  298, 2529,    0,  443,  593,  454, 4115,  312, 1424,
  449,  393,  721, 1531,   59,  588, 3519,  503,  504, 2114,
  257,  511,    0,    0,  738, 1440, 3374, 2774, 4125, 3750,
 4043,  708, 1550, 1551, 1552, 1553, 1554,  523,   59, 3129,
 3761, 3283,  340, 2096, 2794, 1440, 2895,  344, 2924, 1464,
   61,  759, 2802,  504,  392, 3856,  487, 1472, 3425,    0,
 3425,  575,   59, 1478, 1479, 3413,  295,  357, 3435,  407,
 3435,  264,  372,  264,    0,   91,  437, 4120, 1493,  748,
 1495,   59,   59,  332,  321,  748,  749, 1575,  387,  326,
  532, 4134, 1580,  480,  401,  433,  400,  754,  401, 2165,
  456, 2821, 3047, 2346, 2824, 2348,  266,  715,  401,  263,
  738,  340,  316,  372, 3168,  303,  588,  504,   59,  312,
  394,  312,  381,   42,  312,  447,  448, 3181,  264,  322,
  323,  721,  436,   59,  434,   59,  523,   61,  264,  332,
 3037,  332, 3863,  380,  257,  264, 1478, 1635, 1563, 1564,
 3747, 3872,  306,  491, 1569, 1635,    0,  321, 1495,  396,
    0, 1576,  652,  317, 1579,    0,    0, 3482, 1563,  655,
 2697,  316,  760,  511,  394,  434,  312,  264,  487,  372,
  652,  372,  302,  387, 3093, 3688,  524, 1602,  381,  368,
  381,  276,  277,  296, 2621,  494,  332,   41, 2621,  555,
   44, 1616,  500,  689,  511,  363,   41, 3449,  321,   44,
 2369, 2368,  400,  326,  401,   59,    0,   41,  511,   59,
   44, 1616,  332,  523,   59,   59, 3611,  395,  631,  276,
  423, 3939,  423,  257,  738,  721,  372, 2821,  689,  401,
 2824, 2398,  387,  301,  437,  381,  437,   59,  436, 3616,
  487, 3616, 3280, 2621, 1669,   59,  410,  417,   59,  275,
  738,  258,    0, 2344,  523, 3862,  313,  380,   59,  395,
  721,  500,    0,  759,  492,   59,  395, 1692,  296,  364,
  258,  363,  368,  396,  295, 1700,  450,  423, 3941,  447,
  448, 2448,  270,    0, 1709,  340, 2453, 3195, 1713, 1714,
 3198,  437,  326,   41,  301,  558,   44, 1700,  487,  307,
  503,  504,  503,  504,  434, 2621,   44,  340, 3068,  340,
 2408,   59,  425,  301,  511,  303,  484,  738,  301,  582,
  523,   59,  523,   41,  282,  738,   44,   44,  385,  302,
  303,  260,   45,  401,  686,  687,  643,  644,  645,  511,
  276,  277,   59,   61, 1769,   58,  295, 3107, 2607,  596,
   63,  598,  599,  340,  284,  447,  448,  503,  504, 2573,
 2676,  295,  530,  363,  487,    0, 2676,  424,  348,  298,
 3977,  703, 4099, 3750,  276, 3750, 2555,  523, 3630, 3631,
 3140, 2560,  296,  296, 3761, 3765, 3761,  369,    0,  316,
 1709,  340,  484, 1769,  401,   42,   59,  425,   59, 3129,
   47, 4128,  649,  571,  284,  619,   41,  306, 3438, 3439,
  264,  313,  419,  401,  447,  448, 1841, 1842,  401,  264,
  319,  372, 3438, 3439,   59, 1944,  276,  277,  364,  437,
  381, 4094,  412, 3438, 3439,  420, 2541, 1842,  530,  273,
  312, 3755,  264, 3438, 3439, 3438, 3439,   59, 1976,  456,
  264,  295,  655,  264,  655, 1925,  295,  439,  312, 1884,
  387,  434,    0,  264,  619,  314, 2564,  312,  322,  323,
  304,  298,  620,  596, 4025,  598,  599,  641,  332,  571,
  257,  335,  412,  385,  484,  299, 3863,  332, 3863,  447,
  448, 2010, 2745, 3121, 3122, 3872,  340, 3872, 2037, 1841,
 1925,    0, 2000,   41,  348, 2003, 2004, 3946,  446,  655,
  344,  425,  425,  259,  364, 2684,  264,  344,  372, 1944,
  314,   59,  424,  269, 2783,  364,  649,  381,  335,  480,
  530,  332,  412,  471,  378, 3129,  380,  364,  332,  271,
 4091,  689,   41, 2064,  747, 1970, 2065, 2788,  355,  326,
 3989,  500,  284,  504, 2110,  295,  759,  401,  759,  544,
   59,  395,    0, 3412,  312,  388,  551,  368,  401,  423,
  419,  571,  523,  721,  322,  323,  301,  295,  291,  380,
  403,  435,  426,  437,  332, 2010, 2442,  335, 2013, 2014,
 2446, 2147,  437,  325, 3549, 3550, 3551, 3552, 3553, 3554,
 3555,  747,  409,  264,  280, 2030, 2594, 2012,  264, 4028,
  454,  510, 2037,  759, 3972,  306,  372,  296,  452,  266,
  352,   59,  368, 2048,  372, 2030,  614,  615,  319,  264,
  277,  620, 2057,  381,  281,  480,  437, 2594, 2063, 2064,
 2065, 2066, 4022,  488,    0,  257,  426, 3025,  344,  503,
  504,  330,    0,  454, 2079,  372,  500,  382,  503,  504,
 2853, 2066, 2087, 2010,  381, 2853, 2091,  423,  316,  523,
  265,  332, 2014,  426,  454,  423,  401,  316,  523,    0,
  412,  600,    0,  268, 2109, 2110, 2791,  435,   44,  437,
 4070, 3283,  283,  258,  419,  479,  316,  332,  686,  687,
  689,  689,  297,   59,  295,  489,  382,  368,  696,  321,
  296,   59,  368, 4093,  326, 2057,  393,  434,  435,  380,
    0, 2146, 2147,   41,  380,  509,  264,  440,  441,  442,
  443,  444,  721,  446,  447,  448,  301, 2079,   59,  387,
 2165,   59,  419,  419,  330, 2087,  425, 4127,  387, 2091,
  557,  447,  448,  466,  467,  503,  504,  276,  471,  472,
  273,    0,   44,  480,  283,  264, 2287,  387,  380, 2288,
    0,  727,  479,  364,  312,  523,  437, 2853,  363,   59,
  456,  437,  489,  588,  396, 3377,  276,  504,  393,   44,
  429, 3766, 2868,  454,  332,  588,  588,   59,  454,  363,
 2898,  655,  509, 2322, 2323,  391,  523,  392, 3463, 2290,
  655,   93,    0,  312,  400, 4012,  643,  644,  645,  510,
   59, 3438, 3439, 2165,  323, 2250, 2251, 3438, 3439,   59,
  268,  344,  295,  332,  372,  511,  401,  401,   93,  425,
  624,  625,  626,  381,  689, 2250,  315,  652,  433, 4102,
 2275, 4048, 2277,   41,  419, 2109,   44, 2282,  327,  652,
  652, 2286, 2287, 2288, 2289, 2290, 2291, 2292,    0,  261,
  602,   59,   44,  372,  364,  487,  721, 4130,  558,  555,
  349,  344,  381, 2286, 4081,  423, 2311,  451, 2313, 3738,
  446,  456, 2146,  747,    0,  385,  334, 2322, 2323,  437,
 2325,  391,  582, 2401, 2402,  759,  491,  655, 2313, 2251,
  316, 2401, 2402, 3673,  759, 2291, 2292,  624,  625,  626,
 2325,   93,  316, 2348,  423,  363,  511,   59,  397,    0,
  643,  644,    0,  296,  424,   41,  265,  258,  437,  524,
 2282,  600, 2289, 2368,  315,  260,  264, 2372,    0,  270,
  266,  389, 2433,   59,  392,  329,  327,    0,  461,  462,
  463,  277,  749, 2368,  335,  503,  504,  330,  297,  407,
    0, 2295,    0, 2398,  260, 2322, 2323,   59,  349,    0,
  301,    0,  303,  298,  596,  523,  598,  599,   59,  360,
  454,   59,   44, 2398,  312,  433, 2112,   40,   41,  747,
  329,   44,  264, 2327,  503,  504,    0,   59, 2433, 2333,
    0,  759,  298,    0,  332,  295,   59, 2442, 2443, 2444,
 2445, 2446,   41, 2448,  523,   44,  397,   46, 2453,   59,
 2372,   59, 2148, 3950, 3213,    0,   41,  649,   59,   44,
   59,  363,   61, 2448, 3422,    0, 2585, 2545, 2453, 2547,
 2548,    0,  276,  491,  372, 2545,  295, 2547, 2548,  260,
  340,  260, 2547, 3277,  567,   59, 2491,  389,  348,   59,
  332, 2496,   59,  511,    0,    0,  264,    0,  479, 2577,
  401, 2579, 2580, 2581,  257, 2510,  524, 2577, 2513, 2579,
 2580, 2581,   41,  260,   59,   44,  319,  298,  378,  298,
  380,  340,  261,  363,   59,  423,  368, 2605,  509,  348,
   59,    0,  685,  348,  528, 2605, 3283,  655,  380,  437,
    0,  401, 2547, 2548,  312,  698,  258,  286,  695,  389,
    0,  298,  370,   59,   59,  323,   59,  508,  270,  378,
  364,  380,  301,  378,  332,  380,  426, 3406,  321, 2491,
  686,  687,   41,  326, 2496,   44,  655,    0,  264,    0,
 2585,  385,  401,  261,   44,  263,  401,  391, 2510,  301,
   59,  303, 2597, 2598,  454,  437, 3613,    0,   59,   59,
 3613,  367,  264,    0,  372,  503,  504,  426,  286,   59,
  413,  426,  454,  381,    0, 3338,  330,  420,  296,  685,
  424, 2720,  738,  301,    0,  523,  312,  380, 2729, 4126,
 3377, 2730,  698,    0,  401,  454,   59,  323,   59,  454,
  500,  759,    0,  396,   41,    0,  332,   44,    0,    0,
 2715,  454,  330,  276,  277,  423,   59,    0,  310,  311,
  283,    0,   59,  466, 4004,  264,  259,   46,  747,  437,
  332, 2732,  295,   59,   41, 2774,  269,   44, 3425,   58,
  759,  500,   61,   59,    0,  315,  372,  264, 3435,  401,
 2695,  494,   59, 2771,   61,  381, 4036,  327,  685,  427,
  299,   59,    0,   44,   59,    0,  368,   59,   59,  437,
 2715,  698, 2717,  614,  615, 2720,   59, 3613,  380,  349,
   59, 2840, 3393, 2728, 2729, 2730, 2731, 2732, 2733, 2734,
  360,    0, 2717,   40,  260,  503,  504,  423,  368,  447,
  448,  364,  594,   59,  260, 2728,  315,  425,   60,   61,
   62,  437,  604,  605, 2862,  523,  425,  655,  327, 2764,
  479,   59,  385,  451,   59,  453,  335,  397,  391, 2774,
  489,  261,  298, 2695,  261,  437,  263, 2733, 2734,  479,
  349,    0,  298, 2788,  591,  686,  687,  266,  689,  489,
   59,  360,  454, 2720,    0,  696,  286,   59,  277,  286,
  259,  424,  281,  264, 2731,  443,  591,  680,  446,  682,
  269,  301,  435,  437,  301,  600, 2821,  503,  504, 2824,
  461,  462,  463, 2901,   42,   43, 3575,   45,  397,   47,
 3651, 2901,  427,  471, 2839, 2840, 2821,  523, 2822, 2824,
   59,  616,  617,  596, 2849,  598,  599, 2774, 2853,   41,
  653,   41,   44,   59,   44,  258,  738,   42,   43, 2864,
   45,  759,   47, 2868, 4044,   41, 4046,  270,   44, 3616,
   41,  332,  675,   44,   58,  751, 3715,   61,  508,  276,
  277,  684, 2887,  466,  467, 2890,  283,  655,  376,  377,
   41,  258,  295,   44,    0,   40,  649,  264,  301,   46,
  303,   41,  614,  615,   44,  447,  448,  368,  275,  276,
  277,   58,   59,  280,   61,  401,  283, 2839, 2923,  380,
   41,   41, 3755,   44,   44,   59,   41,   41,  295,   44,
   44, 2853,  555,  736,  301,   41,   46,  340,   44,  508,
  407,   41, 2864,  310,   44,  348, 2868,   41,   58,   59,
   44,   61,  755,   59,   41,   41,  759,   44,   44,  271,
 2965,   41, 2967,   41,   44,   41,   44,  364,   44,  655,
   41,  368,  284,   44,  686,  687,  437,  689,  315,  747,
   41, 3046, 2965,   44,  696, 1237,   41, 1239,  385,   44,
  327,  759,   46,  454,  391, 3060, 3061,  364,  401,  268,
  447, 2923,  264, 3750,   58,   59,  318,   61,   41,  412,
 3015,   44,  349,  325, 3761,  264,   41,  310,  385,   44,
  306,  333,  312,  360,  391, 3030,  417,  424, 3033,    0,
  316, 3036,  399,  319,  401,  461,  462,  463,  350,  258,
  352, 3046,  479,  480,  262, 2967,    0,  414,  266,  329,
  362,  270,  419,  365,  366, 3060, 3061,  424,  328,  277,
  397,  747,  268,  281,  435,  334,   41, 3072,  487,   44,
  332,  678,   41,  759,  681,   44,  295,  262, 3438, 3439,
 3085,  266,  301,   41,  303,   46,   44, 3072,   59,  456,
  487,  487,  277,  492,  363,   41,  281,  500,   44, 3104,
  412,  387, 3085,  528, 3672,   59,  368,  315,    0, 3114,
   41, 3033, 3117,   44, 3036,  419, 3863,  446,  380,  327,
  389,  340,  419,  392, 3129, 3872, 2297, 2298,  334,  348,
  264,  417,  295,  332, 3117,  421,   46,  264,  407,   41,
  426,  349,   44,  417, 3129, 3150,  432, 3152,   58,   59,
   41,   61,  360,   44,   60,   61,   62,  363,  264,   41,
  368,    0,   44,  401,  433,   42,   43,   59,   45,  369,
   47,  508,   46,   41,   41,  437,   44,   44,  316,  333,
  600, 3289,  401,  389,   58,   59,  392,   61,  555,  397,
  379, 3196,  454,  412,   41,  367,   41,   44,  332,   44,
 3278,  407,   41,  491,    0,   44,  312,   41, 3278,  315,
   44,  614,  615,  379,  273, 3293,  322,  323,  759,  715,
   59,  327,  491, 3293,  510, 3230,  332,  433, 3150,  335,
 3152,  523,   41,   40,  368,   44,    0,   41,  304,  401,
   44, 4080,  511,  349,  735,   41,  380, 3230,   44,  687,
  264, 3929, 3353, 3931,  360,  524, 3355,   41,  264, 4008,
   44,   46,  368,   59,  631,   41,  372,  264,   44,  140,
  556,  264,  143,   58,   59,  381,   61,   41, 3283,  487,
   44,  500,  268,  686,  687,  491,  689,  454,  264, 3354,
  602,  397, 3357,  696, 3299,   59, 3301,  268,    0,  466,
  508,  600, 3367,  437, 1936,  511, 3371, 1939,  282, 3359,
 3356,  738,  418,  735,  268,  264, 3299,  423,  524,  264,
  454, 3039, 3040, 3041, 3042, 3043, 3044,  494,  434,  435,
  381,  437,  340,  619,   40,  340, 3904,  650,  447,   41,
   40,   40,   44,   40,    0,   40,   40,   40, 3353, 3354,
 3355, 3356, 3357, 3358, 3359, 3360,  262,   59,   40,   40,
   40,   40, 3367,  334,   40,  271, 3371,   40,  447,   40,
  447,  738, 3377,   40,  480,   40,  268,   40,  284, 3301,
  334,  487,   40,  347,   40,   41,   58,   40,   44,  266,
   40,   63,  363, 3471, 3502,  614,  615,  503,  504, 3404,
  277, 3471,  508,   59,  281,   40,   40, 3412,   40,  363,
   40,   40,  318,   40, 3492,   40,   40,  523,  389,  325,
 3425,  392, 3492, 3501,   40,  264,  261,  333,   40,   40,
 3435, 3501,   40, 3360,   40,  389,  407,   40,  392,   40,
 3425,   40,  334,  394,  350,   40,  352,   40,  886,   40,
 3435,  286,   40,  407,   91,   40,  362,   40,  364,  365,
  366,  296,  433, 3468,   40,  371,  301,  686,  687,   40,
  689,  363,   40,  312,   40,  913,  315,  696,   40,  433,
  276,  277,   40,  322,  323, 3490,  653,  283,  327,   40,
   40,   40, 3557,  332,   40,  330,  335,  389,   40,  295,
  392,   40,   40,   40,   40, 3510,  412, 3572,  675,   40,
  349,   40,  276,  277,   40,  407,  280,  684,   40,  283,
  491,  360,   40,   44,  311,  618,   40, 3510,   40,  368,
   40,  295,   40,  372,   40,   46,   40,  491,  368,   46,
  511,  433,  381, 1351,  370, 1353,  370,   40,  384,  655,
 1358, 1359, 3557,  524,  430, 1363,  391,  511,  397,   40,
  430,  341, 3567,   40,  430,  400,  275, 3572,  364,  736,
  524, 3649,   40,  419,  276,  277,   40,   40,   40,   61,
  504,  283, 1390,  504,  423, 3590,   40, 3592,  755,  385,
  425,  333,  759,   46,  333,  391,  435,  480,  437,  491,
  364,  312,  655,  503, 3609, 3610,  357, 3590, 1046, 3592,
  295, 3616,   40,  382,  437, 3620,  270,  450, 3685,  511,
  295,  385,   91,   40, 3629,  270, 3609,  391,  424,  446,
  735, 3616,  524,  368,  374,  367,  372,   41,  312, 3644,
 3645,  747,   41,   41, 3649,   41, 3651,  367,  487,   41,
   41,   46,   40,  759,  437,   40,  312, 3662,  419,  315,
  424,  386,  364,  368,  503,  504,  322,  323,  290,  508,
  368,  327,  368,   40,   40,  261,  332,   46,  275,  335,
 3685, 3686,  260,  385,  523,  466,  312,  466, 3610,  391,
   40,  312, 3753,  349,  466,  333,  602, 3702,  401,   44,
  286,  758,  333,  300,  360,  333,   44, 3629,   44,  306,
  296,   46,  368, 3791,  314,  301,  372,  332,   40,  316,
  513, 3791,  424,  500,  347,  381,  499,  347,  499, 3807,
  437,  719,  329, 3738,  719,  616,  446, 3807,  270,  294,
   59,  397,   59,   41,  330, 3750,  395,   44, 3753, 3754,
  437,  440,  441,  442,  443,  444, 3761,  446,  371,  555,
  600,  371,  371,  487, 3686, 3750,  348,  423,  440,  441,
  442,  443,  444,   40,  446, 3894, 3761,   40, 3762,  435,
 3702,  437,  471,  472, 3789,  487, 1224,  504,  355, 1227,
  387,  555,    0,   41, 1232,   41,   41,    0, 1236,  471,
  472,   44, 1240,   41,   41,  391, 1244,   41,   44,  314,
   41,  717,   44,  418,  400,  314,  655,  314,   41,   41,
  417,   41,    0,   44,  421,  314,   40,   46,  435,  426,
  333,  487,   40,   41,  749,  432,   44,   41,   41,  425,
  419,   44,   40,  395,  417,  401,   41,  503,  504, 3854,
    0,   59,  508,   61,  419,  419,   59,  454, 3863,   44,
   44,   44,   41,   41,   41,  335,   44, 3872, 3380, 3381,
 3382, 3383, 3384, 3385, 3879, 3942,  335,  373, 3863,  314,
  368,   59,  368,   91,  368,  290,  414, 3872, 3893, 3894,
   40,   41,   41, 3083,   44,  749, 3879,  749,  749,   41,
   41,   41,   41, 1341, 3909,   41,   61,  488,  747,   59,
   41,   61,   46,  510, 1352,  435,   41,  340,  340, 1357,
  759,  340,  264, 1361,  419,  264,  739,  719, 3995,  719,
  455,  264,  276, 1371, 3939,  303,  739, 3942, 1376, 1377,
  419,   91, 1380, 1381, 1382, 1383, 1384, 1385, 1386,   58,
 1388, 1389,  314, 1391,    0, 1393, 1394, 1395, 1396, 1397,
   61,  395, 1400, 1401, 1402, 1403, 1404, 1405, 1406, 1407,
 1408, 1409, 1410, 1411, 1412, 1413, 1414, 1415,  303,   40,
  304,  418,  303,  419,   41,  314,   41, 1425, 1426, 1427,
 3995,   40,  264,  263,  263,   41,  314,  278,   44,  655,
  314,  620,  525,  600, 1442, 1443,   41, 4012,  335,  528,
  527,  437, 1450,   59,   41,   61,  399, 1455,  333,  437,
  487,   44,  619, 1461, 1462, 4030,  322,  401,  369,  314,
  419,   40, 1470,  344,  276, 1473, 1474, 1475, 1476, 1477,
  344,   40, 1480, 4048,  736,  303,  594,   59,  332,  332,
  258,    0,   58,    0,   61,  367,  264,  437,  273,  277,
  386,  371,  371,  371, 4069,  273,  735,  275,  276,  277,
  371,  735,  280,  276,  277,  283, 4081,  371,  371,  370,
  283,  386,   59,   59,  358,  437, 4069,  295,    0,  450,
  511,  747,   41,  301,   41,   44,  304,   44,  276,  277,
   40,  450,  310,   59,   59,  283,  303,   44,  258,   41,
   59,  367,   59,   41,  264,   41,   41,  295,  434,   41,
  273,   41,  709,  273,  419,  275,  276,  277,   40,   41,
  280,  419,   44,  283,  351,   41,  344,  419,   41,  419,
  419,  419,  419,  407,  526,  295,    0,   59,  405,   61,
   44,  301,  650,  466,  304,  273,  364,  323,  369,  275,
  310,  364, 3352,  466,  336,  719,  719,  420,  332,  434,
   40,   59,  364,  344,  290, 3365,  364,  385,  273,   91,
   40,    0,  385,  391,  300,  532,  364,  395,  391,  296,
  306,  296,  425,  401,  344,  311, 3386, 3387,  330,  425,
  316,  330,  650,  319,  325,   59,  419,  385,  314,  368,
  613,  419,  258,  391,  364,  371,  424,  446,  264,  454,
   40,  424,   41,   40,  454,   44,   40,  357,   41,  275,
  276,  277,  314,  734,  280,  385,  336,  283,   61,   40,
   59,  391,   61,  450,  452,  395,  424,   44,  456,  295,
  557,  401,  368,  747,  314,  301,  314,  431,  314,  749,
  314,  314,  314,  314,  310,  427,  749,   44,   41,  419,
   40,  387,   91,   41,  424,  299,  289,  299,  494,  280,
  623,  344,  344,  631,  487,  264, 1724, 1725,  275, 1727,
  340, 1729,  494,  401,  304, 1733, 1734, 1735, 1736,  418,
  303,  417,  452, 1741, 1742,  611,  456,   45,  689,  611,
  756,   63,  718,  321,  273,   44,  432,  434,  364,  312,
   58,  389,  370,  677, 1762,   63,  713,  276,  277,  276,
  277,  411,  431,   40,  283,   44,  283,  418,  454,  385,
  314, 1779,   41,  312,   46,  391,  258,  555,  295,   60,
   61,   62,  264, 1791, 1792,  401,  395,  401,  336,    0,
  344,  273,   41,  336,  276,  277,  435,   40,  280,   41,
   46,  283,   41,  419,   41,  418,  370,  555,  424,   44,
  264,  299,   40,  295,  487,  295,  264,  555,   44,  301,
   61,  719,  304,   44,  510,   41,  719,  513,  310,  357,
   41,   40,  654,   44,  258,  555,  564,   40,  299,  302,
  456,  447,  264, 1851,  714,  364,  270,  364,   59,  391,
  632,  513,  278,  631, 1862,  258, 1864,  652,  571, 1867,
   41,  452,  344,  273,  586,  266,  385,  270,  385,  258,
   40,  295,  391,  289,  391,  264,  717,  301,  654,  303,
   41,  295,  364,  329,  273,  329,  275,  276,  277,  262,
  654,  280,  717,   41,  283,   41,  504,  565,  301,  376,
  303,  283,  504,  385,   40,  424,  295,  424,  426,  391,
  310,  631,  301,  395,   40,  304,  340,  368,  386,  401,
  420,  310,  692,  654,  348,  368,  420,  368,  330,  418,
  401,  296,  425,  619, 1942,  609,  449,  419,  336,  555,
  607,   41,  424, 3703, 3704, 3705, 3706, 3707, 3708, 3709,
  609,  418,  330,  437,  401,  344,   44,  427,   40,   40,
  738, 3721, 3722, 3723, 3724, 3725, 3726, 3727,  487,    0,
  452,  427,  329,  264,  456,  364,  264,  401,  264,   40,
  264,  262,  264,  291,  419,  400,  395,  333,  412,   40,
  271,   41,  270,  335,   41,  434,  385,    0,  401,  340,
  270,  306,  391,  284, 2012,  270,  395,  303, 2016,  303,
 2018,  316,  401,   40,  319,  631,   41,  264,  738,  368,
  335,  446,  290,  303,   41, 2033,  270,  487,  303,   59,
  419,  407,  511,    0,  368,  424,  395,  318,   41,   41,
   41,   44,   61,  336,  325,   59,  427,   59,   59,   59,
  303,  289,  333,   40,  314,  303,   59,   41,   61,  618,
   59,   59,   59,  452,   59,  276,  277,  456,   41,  350,
  418,  352,  283,  555,   41,   41,  500,   44,   41,  511,
  314,  362,  387,  364,  365,  366,  264,  314,   91,  299,
  371,   41,   59,  446,   61,  446,   41,   40,  738,   40,
  480,  646,  738,   40,   46,  294,   46,  294,  294,   40,
    0,   40,  294,   41,   41,   40,  421,   40,  389,  294,
  305,  426,  738,   45,   91,  305,  738,  432,  305,   44,
  389,  412,  440,  441,  442,  443,  444,   46,  446,  447,
  448,  294,  364,  401,  294,   40,  305,  294,   41,  631,
  555,   41,  597,  364,   44,  597,   44,   40,  466,  467,
  597,  401, 2170,  471,  472,  264,  555,   46, 2176,   59,
    0,   61, 1951, 2181,  385, 1511, 2184, 2185, 2186, 1511,
  391, 1511, 2707, 2191, 2192, 2193, 1511,    0, 2708, 1511,
  614,  615, 2200, 2709, 2202, 3557,  504, 2205, 2206, 2207,
 2208,   91, 3557, 2311, 2212,  510, 2214, 2737, 2216, 1512,
 2218,  614,  615,  424, 3367, 2601, 2224, 2225, 2226, 2227,
 2228, 2229, 2230, 2231, 2232, 2233, 2234, 2235,   41, 3288,
 1713,   44, 2240, 2241, 3500,  441, 3221, 2245, 2246, 3473,
 3166, 2555,  631, 2915,  832, 3213,   59,   85, 2256, 2503,
 3776,  556,  106, 2261, 3893,  258,  738, 2265, 3668,    0,
 3201,  264,  686,  687, 3788,  689, 2542, 1202, 2276, 3855,
  273, 2279,  696,  276,  277, 3904,  445,  280, 2923, 2587,
  283, 3771, 3222,  686,  687, 4113,  689, 3989, 2923, 4024,
 4061,  258,  295,  696, 4118,  600, 4023,  264,  301, 3150,
   41,  304, 3702,   44, 3151, 3859,  273,  310, 2150,  276,
  277, 1708,  730,  280,  619, 2148,  283, 2070,   59, 2822,
   61,  602, 1649,  275, 3624, 2822, 4081, 3978,  295,   60,
   61,   62, 2446, 4082,  301,  643,  644,  304,  290, 3760,
 2818,  344,    0,  310, 3866, 1896,  654, 1480,  300,  738,
   91, 3033, 2568, 3274,  306, 2053, 2091, 2509, 1274,  311,
 3889,  364, 3774, 2371,  316, 1619, 2052,  319,  258,  291,
 3609, 1667, 2551, 1684,  264, 1186,  144,  344, 3404, 2334,
 1548, 3393,  385,  273, 3089, 1216,  276,  277,  391, 3590,
  280,  759,  395,  283,  436,  461,  717,  364,  401,    0,
  766, 1222, 2012, 2718, 1937,  295, 1244, 2326, 3602, 1495,
 1495,  301, 3738, 3510,  304, 1494,  419, 1344,  385, 2016,
  310,  424, 1594, 4030,  391, 1235,  771,    0,  395, 2782,
 2438,    0, 3576, 3576,  401,  387,    0,    0, 4075, 2447,
   41,    0,    0,   44, 2448, 1562, 4007, 1861, 2368,  452,
 1452,  447,  419,  456,  344, 2573,  737,  424,   59, 2584,
   61,  813, 3789,  276,  277,  417, 1569, 2029, 1690,    0,
  283,  850,  867, 3645,  364, 1421, 1473, 3854,   41, 3694,
  432,   44, 1022, 2211, 3538,  452, 1023, 1601, 1601,  456,
   91, 3309, 1418, 2348,  121,  385,   59, 2849,   61, 3358,
 2328,  391,  454, 2736, 3283,  395, 1491, 3351, 1332, 4028,
   41,  401, 3416,   44, 2144, 3657, 3495,  258,  440,  441,
  442,  443,  444,  264,  446,  447,  448,    0,   59,  419,
   61,  262,  273, 3800,  424,  276,  277, 3637, 3884,  280,
  271, 2852,  283, 3991,  466,  467, 1254,   12, 3087,  471,
  472,  364,  555,  284,  295, 3812, 3067, 1161,  510, 4002,
  301,  513,  452,  304, 3971, 3660,  456,   -1,   41,  310,
 3540,   44,  385,   -1,   -1,   -1,   -1,   -1,  391,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,  318,  555,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  333,  344,   60,   61,   62,   -1,   -1,   -1,
   -1,  424,   -1, 2621,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352, 2630,  364,   -1,   -1,   -1,   -1,  631, 2637,
 2638,  362,   -1,  364,  365,  366, 2644,   -1,   -1,   -1,
  371,   -1,   -1,    0,  385,   -1,   -1,   -1,   -1,   -1,
  391,   -1,   -1,   -1,  395,   -1,   -1,  258,   -1,   -1,
  401,   -1,   -1,  264,  631,  555, 2674,  619,   -1, 2677,
   -1, 2679,  273,   -1,   -1,  276,  277,   -1,  419,  280,
   -1,  412,  283,  424,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,  275,  295,  258,   -1,   -1,   -1,   -1,
  301,  264,   59,  304,   61,   -1,   -1,   -1,  290,  310,
   -1,  452,  275,  276,  277,  456,   -1,  280,  300,   -1,
  283,  643,  644,   -1,  306,   -1,   -1,  258,    0,  311,
   -1,   -1,  295,  264,  316,  738,   -1,  319,  301,   -1,
   -1,  631,  273,  344,   -1,   -1,  277,  310,   -1,  280,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  364,  295,   -1,   -1,   -1,   -1,   41,
  301,  738,   44,  304, 2782,   -1,   -1,   -1,   -1,  310,
   -1,   -1,   -1,   -1,  385,   -1,  368,   59,   -1,   -1,
  391,   -1, 2800,   -1,  395,   -1,   -1,   -1,   -1,   -1,
  401,  364,   -1,  276,  277,  387,  262,   -1,  264,   -1,
  283,   -1,   -1,  344,  555,  271,   -1, 2825,  419,   -1,
   -1,   -1,  385,  424,   -1,   -1,   -1,   -1,  391,   -1,
   -1,   -1,   -1,  364,   -1,  417,  399,   -1,  401,   -1,
   42,   43,   44,   45,   46,   47,   -1,   -1,  738,   -1,
  432,  452,   60,   61,   62,  456,  419,   -1,   60,   61,
   62,  424,  318,   -1,  395,   -1,   -1,   -1,   -1,  325,
  401,  602,  454,   -1,   -1,   -1,   -1,  333,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  419,   91,
  631,  364,   -1,  456,  350,   -1,  352,   -1,   -1,   -1,
   -1,  258,   -1,   -1,   -1,   -1,  362,  264,  364,  365,
  366,    0,  385,   -1,   -1,  371,   -1,   -1,  391,  276,
  277,  452,   -1,  280,   -1,  456,  283,   -1,  510,   -1,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,  295,   -1,
   -1,   -1,   -1,   -1,  301,   -1,   -1,   -1,   -1,   -1,
   -1,  424,   41,  310,  555,   44,   -1,   42,   43,   44,
   45,   46,   47,   -1,   -1, 2973,   -1, 2975,  699, 2977,
   59,   -1,   -1, 2981,   -1,   60,   61,   62, 2986, 2987,
 2988, 2989, 2990,   -1,   -1,   -1,   -1, 2995,   -1, 2997,
   -1, 2999,  555,   -1,   -1,   -1,   -1,  738,   -1,   -1,
   -1,   -1,   -1,   -1,  276,  277,   91,  364,  280, 3017,
   -1,  283,   -1, 3021,   -1,   -1,   -1, 3025,   -1,   -1,
 3028,   -1,   -1,  295,  555,   -1,   -1,   -1,  385,   -1,
  631,   -1,   -1,   -1,  391,   -1,   -1,  619,   -1,   -1,
   -1,   -1,    0,   -1,  401,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
  262,   -1,  419,  271,  266,   -1,   -1,  424,  631,  271,
   -1,   -1,   -1,   -1,   -1,  277,  284,   -1,   -1,  281,
  262,   -1,  284,   41,   -1,   -1,   44,   -1,   -1,  271,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1, 3106,  456,
  631,   59,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,  385,   -1,   -1,  318,  325,   -1,  391,
   -1,   -1,   -1,  325,   -1,  333,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,  318,  738,   -1,   -1,
   -1,   -1,  350,  325,  352,   -1,  602,   -1,  350,   -1,
  352,  333,  424,   -1,  362,   -1,  364,  365,  366,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,  350,  371,
  352,   60,   61,   62,   -1,  738, 3184,  262,   -1,   -1,
  362,  266,  364,  365,  366,   -1,  271,  276,  277,  371,
   -1,   -1,  277,   -1,  283,   -1,  281,   -1,  555,  284,
   -1,   -1,   -1,   -1,  412,   -1,   -1,  738,   -1,   -1,
  412,   -1,   -1,   -1,  458,  459,  460,  461,  462,  463,
  464,  465,   -1,   42,   43,   44,   45,   46,   47,   -1,
  412,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,
  325,   60,   61,   62,   -1,   -1,   -1,   -1,  333,   -1,
   -1,  275,   -1,   -1,   -1,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,  350,  290,  352,   -1,   -1,
   -1,   -1,   91,   -1,  631,  364,  300,  362,   -1,  364,
  365,  366,  306,  555,   -1,   -1,  371,  311,   -1,   -1,
   -1,   -1,  316,   -1,   -1,  319,  385,   -1,   -1,   -1,
 3308, 3309,  391,   -1,   -1,   -1,   -1,   45,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1, 3325,  276,  277,
   -1,   -1,   -1,  567,   -1,  283,   -1,  412, 3336,   -1,
   -1,   -1,   -1,   -1,   -1,  424,   -1,   -1,   -1,   -1,
   -1,   42,   43,   44,   45,   46,   47,   -1,   -1,  593,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,
   61,   62,   -1,  387,   -1,  567,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,  738,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,  593,   -1,  417,  602,  284,   -1,   -1,   -1,   -1,
  602,   -1,   -1,   -1,   -1,   -1,  364,   -1,  432,   -1,
   -1,   -1,   -1,   -1, 3422, 3423,   -1,   -1,   -1,   -1,
  602,   -1,   -1, 3431, 3432,   -1,   -1,  385,   -1,  318,
  454,   -1,   -1,  391,   -1,   -1,  325,   -1,   -1,   -1,
   -1,   -1,   -1,  262,  333,   -1,   -1,  266,   -1,   -1,
   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,  275,
   -1,  350,  281,  352,   -1,  284,  424,   -1,   -1,   -1,
   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,
   -1,   -1,  567,   -1,  300,   -1,  510,   -1,   -1,  513,
  306,   -1,   -1,   -1,   -1,  311,   -1,  741,  742,  318,
  316,   -1,   -1,  319,   -1,   -1,  325,   -1,  593,   -1,
  692, 3519,   -1,   -1,  333,   -1,  760,  602,  762, 3527,
   -1, 3529,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  350,   -1,  352,   -1,  717,   -1,   -1,   -1,  741,
  742,  275,   -1,  362,   -1,  364,  365,  366,   -1,   -1,
   -1,   -1,  371,  291,   -1,   -1,   -1,   -1,  760,   -1,
  762,  262,   -1,   -1,   -1,  266,  300,   -1,   -1,   -1,
  271,  387,  306,   -1,   -1,   -1,  277,  311,   -1,  317,
  281,   -1,  316,  284,   -1,  319,   -1,   -1,   -1,   -1,
   -1,   -1, 3600,  412, 3602,  619,   -1,   -1,   -1,   -1,
   -1,  417,   -1,   -1,   -1, 3613,  344,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  432,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   60,   61,   62,   -1,   -1,
   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,  454,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,  350,
   -1,  352,   -1,  387,   -1,   -1,  741,  742,  290,   -1,
   -1,  362,   -1,  364,  365,  366,   -1, 3675,  300,   -1,
  371,   -1,  410,   -1,  306,  760,   -1,  762,   -1,   -1,
 3688,   -1,   -1,  417,  316, 3693, 3694,  319,  426,   -1,
   -1,   -1, 3700,   -1,  510,   -1,   -1,  513,  432,   -1,
   -1,   -1,  440,  441,  442,  443,  444,   -1,  446,  447,
  448,  412,   -1,  602,   -1,   -1,   -1,   -1,   -1,   -1,
  454,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  466,  467,
  468,  469,  470,  471,  472,  473,  474,   -1,   -1,  371,
   -1,   -1,   -1,  481,  482,   -1,   -1,   -1,  567,   -1,
   -1,   -1,  490,   -1,   -1,  387,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
  402,   -1,   -1,   -1,  593,   -1,  510,   -1,   -1,  513,
   -1,   -1,   -1,  602,   -1,  417,   -1,   -1,    0,  421,
   -1,   -1,  424,   -1,  426,   -1,   -1,   -1,   -1,   -1,
  432,   -1,   -1,  619,   -1,   -1,   -1,   -1,   -1, 3817,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  454,   -1,   -1,   -1,   -1,  262,   40,   41,
   -1,   43,   44,   45,   46,   -1,  271,   -1,  300,   -1,
   -1,   -1,   -1,   -1,  306,   -1,   58,   59,   -1,  284,
   -1,   63,   -1,   -1,  316,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  567,  329,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  510,   -1,
   -1,  513,   -1,  318,   -1,  619,   -1,   -1,   -1,   -1,
  325,   -1,  593,   -1,   -1,   -1,   -1,   -1,  333,   -1,
   -1,  602,   -1,  641,   -1,  643,  644,  645,   -1,   -1,
   -1,   -1,   -1,   -1, 3922,  350,   -1,  352,   -1,   -1,
   -1,   -1,  741,  742,  556,  387,   -1,  362,   -1,  364,
  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,
   -1,  760,   -1,  762,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  417, 3964,   -1,   -1,  421,
   -1,   -1,   -1,   -1,  426,   -1,  704,  705,  600,   -1,
  432,   -1,   -1, 3981,   -1, 3983,   -1,  412,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  723,   -1,  619,   -1,   -1,
   -1,   -1,  454,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1, 4012,   -1, 4014, 4015,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  741,  742,   -1,   -1,   -1,  257,   -1,  259,  510,   -1,
   -1,  513,  264,  265,   -1,  267,  268,  269,   -1,  760,
   -1,  762,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285, 4082,  287, 4084, 4085,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309, 4106,   -1,
  312,   -1,   -1,  315,   -1,  317,   -1,  319,  320,   -1,
  322,  323,  324,   -1,  326,  327,  328,   -1,   -1,   -1,
  332,   -1,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
 4138,   -1,  344, 4141,  346,  347,  348,  349,  600,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,  360,   -1,
   -1,   -1,  364,   -1,   -1,   -1,  368,  619,  370,   -1,
  372,   -1,  374,  375,  376,  377,  378,  602,  380,  381,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,  423,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,  435,   -1,  437,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,  706,  484,  485,  486,  487,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,  655,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,    0,
   -1,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  275,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,  300,   -1,   -1,
   -1,   -1,   -1,  306,   -1,   -1,   -1,  310,  311,   -1,
   -1,   -1,   -1,  316,   -1,   -1,  319,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  329,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  379,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  387,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  417,   -1,   -1,   -1,  421,   -1,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,  432,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  454,  455,   -1,   -1,   -1,  257,  258,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,  276,  277,   -1,  279,  280,
  281,  282,  283,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
  301,  302,  303,  304,  305,  306,   -1,  510,  309,   -1,
  513,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,  556,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,  385,  386,   -1,  388,  389,   -1,
  391,  392,  393,  394,   -1,   -1,  397,  600,  399,   -1,
  401,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,  619,  418,  419,  420,
   -1,  422,  423,  424,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
   -1,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   41,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   -1,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,  300,   -1,   -1,   -1,   -1,   -1,  306,   -1,   -1,
   -1,  310,  311,   -1,   -1,   -1,   -1,  316,   -1,   -1,
  319,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  329,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,   -1,   -1,
  318,   -1,   -1,  402,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,  417,   -1,
   -1,   -1,  421,   -1,   -1,   -1,   -1,  426,   -1,   -1,
   -1,   -1,  350,  432,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,  454,   -1,   -1,   -1,   -1,
  257,  258,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,  276,
  277,   -1,  279,  280,  281,  282,  283,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,  301,  302,  303,  304,  305,  306,
   -1,  510,  309,   -1,  513,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,  556,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,  385,  386,
   -1,  388,  389,   -1,  391,  392,  393,  394,   -1,   -1,
  397,  600,  399,   -1,  401,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
  619,  418,  419,  420,   -1,  422,  423,  424,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  275,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  300,   -1,   -1,   -1,   -1,   -1,  306,   -1,   -1,   91,
  310,   93,   -1,   -1,   -1,   -1,  316,   -1,   -1,  319,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  329,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  312,
   -1,   -1,   -1,   -1,   -1,  318,   -1,  417,   -1,   -1,
   -1,  421,  325,   -1,   -1,   -1,  426,   -1,   -1,   -1,
  333,   -1,  432,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,  454,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
  510,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,  556,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
  600,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,  619,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,  736,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  300,   -1,   -1,   -1,   -1,   -1,  306,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,  316,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  329,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,  387,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  417,
  318,   -1,   -1,  421,   -1,   -1,   -1,  325,  426,   -1,
   -1,   -1,   -1,   -1,  432,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,  454,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,  510,  290,  412,  513,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,  600,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,  619,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,  261,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  278,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  286,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  294,   -1,  296,   -1,   91,
   -1,   93,  301,   -1,   -1,   -1,   -1,   -1,  307,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  321,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  330,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,  355,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  396,   -1,  398,
   -1,   -1,  401,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,  425,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,  437,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,  477,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,  492,  493,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,  514,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   41,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   41,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,  265,   -1,   -1,   -1,   -1,   -1,
  271,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,  297,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,  318,   -1,
   -1,   -1,  333,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,  362,  352,  364,  365,  366,   -1,   -1,   -1,   -1,
  371,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,  412,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,  412,   -1,   -1,  297,   -1,  299,  300,  419,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  602,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,  602,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,  264,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,  262,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,  318,   -1,   -1,   -1,  333,   -1,   -1,
  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  350,  362,  352,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,  362,   -1,  364,
  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,  412,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,  412,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,  573,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  602,   -1,  475,  476,
   -1,  478,  479,  480,   -1,   -1,   -1,  602,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  271,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  314,   -1,   -1,   -1,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,  318,   -1,
   -1,   -1,  333,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,  336,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,  362,  352,  364,  365,  366,   -1,   -1,   -1,   -1,
  371,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,  412,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,  412,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  602,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,  602,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,
   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,  412,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
  487,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,  602,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,    0,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  333,   -1,   -1,  336,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,
  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,  412,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,  458,  459,  460,  461,
  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  602,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,  503,  504,  505,  506,  507,  508,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,    0,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,  741,
  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,
   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,  412,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
  487,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,  602,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   41,   -1,  646,
   44,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,  609,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   41,   -1,  646,   44,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,  266,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
  609,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   41,   -1,  646,   44,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,  266,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,  412,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,  602,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,  609,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   41,
   -1,  646,   44,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
  602,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,  609,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   41,   -1,  646,   44,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   44,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,  609,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   44,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,  266,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
  609,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   44,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,  266,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,  412,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,  602,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,  609,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   44,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
  602,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,  609,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   44,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   44,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,  609,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   44,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,  266,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
  609,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   44,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,  266,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,  412,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,  602,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,  609,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   44,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
  602,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,  609,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   44,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   44,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,  609,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   44,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,  266,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
  609,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   44,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,  266,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,  412,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,  602,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,  609,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   44,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
  602,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,  609,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   44,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   44,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,  609,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   44,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,  266,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
  609,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   44,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,  266,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,  412,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,  602,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,  609,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   44,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
  602,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,  609,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   44,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   44,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,  609,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   44,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,  266,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
  609,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,  266,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,  412,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,  602,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,  609,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  312,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
  602,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,  618,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   -1,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,  618,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,  266,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,  419,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,   -1,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  314,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,  266,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,  266,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,  412,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,  557,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,  602,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,  336,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,  266,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
  602,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   -1,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,  266,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,  602,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   42,
   43,   44,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,  281,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  412,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,  602,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,   -1,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   42,   43,   44,   45,   -1,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  412,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,  602,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   42,   43,   44,
   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
   -1,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   42,   43,   44,   45,   -1,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   -1,
   43,   44,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,   -1,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   -1,   43,   44,   45,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,   -1,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
   -1,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,   -1,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   -1,
   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
   -1,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,   -1,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,   -1,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,   -1,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
   -1,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,   -1,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   -1,
   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
   -1,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,  605,  606,  607,  608,   -1,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,   -1,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,    0,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,   -1,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,   -1,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,   -1,  458,  459,  460,
  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
   -1,   -1,   -1,   -1,  485,  486,  487,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
   -1,   -1,  503,  504,  505,  506,  507,  508,  509,  510,
  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,    0,   -1,   -1,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
  741,  742,  743,  744,   -1,   -1,  747,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,   -1,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,   -1,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,  503,  504,  505,  506,
  507,  508,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,  655,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,    0,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
  747,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   41,   -1,
   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,   -1,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,   -1,  319,  320,   -1,  322,
  323,  324,   -1,  326,  327,  328,   -1,   -1,   -1,  332,
   -1,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351,   -1,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,   -1,
   -1,  364,   -1,   -1,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,   -1,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,
   -1,   -1,  485,  486,  487,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
  503,  504,  505,  506,  507,  508,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   -1,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,   -1,
  603,  604,  605,  606,  607,  608,   -1,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,
  653,   -1,  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,    0,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,  741,  742,
  743,  744,   -1,   -1,  747,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,   -1,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,   -1,
  319,  320,   -1,  322,  323,  324,   -1,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,   -1,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,  458,
  459,  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,   -1,   -1,   -1,   -1,  485,  486,  487,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,  503,  504,  505,  506,  507,  508,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,   -1,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,  655,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,    0,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,  741,  742,  743,  744,   -1,   -1,  747,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,   -1,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,   -1,  319,  320,   -1,  322,  323,  324,
   -1,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,   -1,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,  458,  459,  460,  461,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,   -1,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   41,  702,   -1,   44,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   59,  720,  721,   -1,   -1,   -1,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,  741,  742,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,   -1,
  315,   -1,   -1,   -1,  319,  320,   -1,  322,  323,  324,
   -1,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  368,   -1,  370,   -1,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,   -1,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   41,  702,   -1,   44,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   59,  720,  721,   -1,   -1,   -1,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,   -1,   -1,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,   -1,
  315,   -1,   -1,   -1,  319,  320,   -1,  322,  323,  324,
   -1,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  368,   -1,  370,   -1,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,   -1,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   41,  702,   -1,   44,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   59,  720,  721,   -1,   -1,   -1,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,   -1,   -1,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,   -1,  760,  761,  762,  763,  764,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,   -1,
  315,   -1,   -1,   -1,  319,  320,   -1,  322,  323,  324,
   -1,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  368,   -1,  370,   -1,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
  435,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,  487,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,  503,  504,
  505,  506,  507,  508,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   59,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,   -1,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
  655,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,   -1,   -1,  743,  744,
   -1,   -1,  747,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,   -1,  760,  761,  762,  763,  764,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
   -1,   -1,  399,   -1,  401,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,  435,   -1,
   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,   -1,   -1,   -1,   -1,   -1,  485,  486,
  487,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,   -1,  504,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,
   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   44,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   59,  720,  721,   -1,   -1,   -1,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,   -1,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,   -1,  309,   -1,   -1,   -1,   -1,  314,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,  332,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,   -1,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,
   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,
   -1,    0,   -1,   -1,   -1,   -1,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   59,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,
   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,
   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,   -1,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   40,   -1,   -1,   43,  646,   45,  648,
  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,  458,  459,  460,  461,  462,  463,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,  741,  742,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
  512,  513,   -1,  515,  516,  517,  518,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,  260,   -1,   -1,  263,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,  408,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,  271,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,  288,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,  745,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,  288,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,  745,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,  288,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,  745,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   42,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,  260,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,  298,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,   -1,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,  288,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   42,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,  314,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,  295,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,  419,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   41,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,  295,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
  487,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   42,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,  260,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,  298,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,   -1,   -1,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   42,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,  496,  497,  498,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   40,   -1,   -1,   43,   -1,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,   -1,
  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,  346,
  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,  356,
  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,  410,  411,  412,  413,  414,  415,  416,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,  426,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,  440,  441,  442,  443,  444,  445,  446,
  447,  448,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,
  467,  468,  469,  470,  471,  472,  473,  474,  475,  476,
   -1,  478,  479,  480,  481,  482,   -1,  484,  485,  486,
   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,  496,
  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   -1,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,   -1,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,  641,   -1,  643,  644,  645,  646,
  647,  648,  649,  650,  651,  652,  653,   -1,   -1,  656,
  657,  658,  659,  660,  661,  662,  663,  664,  665,  666,
  667,  668,  669,  670,  671,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,  722,  723,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   40,   -1,
   -1,   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,
   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,
   -1,   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,
   -1,  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
  382,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,  410,  411,
  412,  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,
  442,  443,  444,  445,  446,  447,  448,  449,   -1,  451,
  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,  468,  469,  470,  471,
  472,  473,  474,  475,  476,   -1,  478,  479,  480,  481,
  482,   -1,  484,  485,  486,   -1,  488,  489,  490,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,   -1,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,  621,
  622,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,  641,
   -1,  643,  644,  645,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,  656,  657,  658,  659,  660,  661,
  662,  663,  664,  665,  666,  667,  668,  669,  670,  671,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   40,
  702,   -1,  704,  705,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   58,  720,  721,
   -1,  723,  724,  725,  726,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
  261,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,  270,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,  296,  297,   -1,  299,  300,
   -1,   -1,  303,  304,   -1,  306,  307,   -1,  309,   -1,
   -1,  312,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,
   -1,   -1,   -1,  324,  325,  326,   -1,  328,   -1,  330,
   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,  346,   -1,  348,  349,   -1,
  351,   -1,  353,   -1,  355,   -1,  357,  358,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,
   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,
   -1,  382,  383,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,   -1,   -1,   -1,  397,   -1,  399,  400,
  401,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,
   -1,  422,   -1,   -1,  425,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,   -1,  436,  437,  438,  439,   -1,
   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,
  451,  452,  453,  454,   -1,  456,  457,   -1,   -1,   -1,
   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,
   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,
  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,
  501,  502,  503,   -1,  505,  506,  507,   -1,  509,  510,
  511,   -1,  513,   -1,  515,  516,  517,   -1,  519,  520,
   -1,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,  540,
  541,  542,  543,   -1,  545,  546,  547,  548,  549,  550,
   40,  552,  553,  554,  555,   -1,  557,   -1,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,  573,  574,  575,  576,   -1,  578,  579,  580,
  581,   -1,  583,  584,  585,  586,  587,   -1,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,   -1,  603,  604,  605,  606,  607,  608,   -1,  610,
  611,  612,  613,  614,  615,  616,  617,   -1,  619,  620,
  621,  622,   -1,  624,  625,  626,  627,  628,  629,  630,
   -1,  632,  633,  634,  635,  636,  637,  638,  639,  640,
   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,
  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  672,  673,   -1,  675,  676,  677,  678,  679,  680,
  681,  682,  683,   -1,  685,  686,  687,  688,  689,  690,
  691,  692,  693,  694,  695,  696,   -1,  698,  699,  700,
   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,   -1,  720,
  721,   -1,   -1,  724,  725,  726,  727,  728,  729,  730,
  731,  732,  733,  734,  735,  736,  737,  738,  739,  740,
   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,
  751,  752,  753,  754,  755,  756,  757,  758,  759,  760,
  761,  762,  763,  764,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,  329,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   40,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,
  329,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  370,   -1,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,
  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,
  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,
  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,
  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,
  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,
  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,  540,  541,  542,  543,   -1,  545,  546,  547,  548,
  549,  550,   40,  552,  553,  554,  555,   -1,  557,   -1,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,  573,  574,  575,  576,   -1,  578,
  579,  580,  581,   -1,  583,  584,  585,  586,  587,   -1,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,   -1,  603,  604,  605,  606,  607,  608,
   -1,  610,  611,  612,  613,  614,  615,  616,  617,   -1,
  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,
  629,   -1,   -1,  632,  633,  634,  635,  636,  637,  638,
  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,
  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,
  679,  680,  681,  682,  683,   -1,  685,  686,  687,  688,
  689,  690,  691,  692,  693,  694,  695,  696,   -1,  698,
  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  715,  716,  717,  718,
   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,
  729,  730,  731,  732,  733,  734,  735,  736,  737,  738,
  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,
  749,  750,  751,  752,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  763,  764,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   40,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   58,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,  701,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,
   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,
  505,  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   40,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   -1,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,   -1,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,   -1,   -1,  743,  744,
   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   40,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,
  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,
   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,
  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,
   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,
   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,
   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,
  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,   -1,  545,  546,  547,  548,  549,  550,   42,  552,
  553,  554,  555,   -1,  557,   -1,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
  573,  574,  575,  576,   -1,  578,  579,  580,  581,   -1,
  583,  584,  585,  586,  587,   -1,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,   -1,
  603,  604,  605,  606,  607,  608,   -1,  610,  611,  612,
  613,  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,
   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,  632,
  633,  634,  635,  636,  637,  638,  639,  640,   -1,   -1,
   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,
  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,
  673,   -1,  675,  676,  677,  678,  679,  680,  681,  682,
  683,   -1,  685,  686,  687,  688,  689,  690,  691,  692,
  693,  694,  695,  696,   -1,  698,  699,  700,   -1,  702,
   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,  712,
  713,  714,  715,  716,  717,  718,   -1,  720,  721,   -1,
   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,
  733,  734,  735,  736,  737,  738,  739,  740,   -1,   -1,
  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,
  753,  754,  755,  756,  757,  758,  759,  760,  761,  762,
  763,  764,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,  292,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   43,   -1,   45,
   -1,   -1,  646,  647,  648,  649,  650,  651,  652,  653,
   -1,   -1,   58,   -1,   -1,   -1,   -1,   63,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,  267,  268,  269,   -1,   -1,   -1,   -1,  274,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,  285,
   -1,  287,   -1,   -1,  290,  291,  292,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  317,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,  344,   -1,
  346,  347,  348,  349,   -1,  351,   -1,  353,  354,   -1,
  356,  357,  358,   -1,   -1,   -1,   -1,   -1,  364,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,  382,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,  410,  411,  412,  413,  414,  415,
  416,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
  426,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,  437,  438,  439,  440,  441,  442,  443,  444,  445,
  446,  447,  448,  449,   -1,  451,  452,  453,  454,   -1,
  456,  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
  466,  467,  468,  469,  470,  471,  472,  473,  474,  475,
  476,   -1,  478,  479,  480,  481,  482,   -1,  484,  485,
  486,   -1,  488,  489,  490,  491,   -1,   -1,  494,  495,
  496,  497,  498,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,
  516,  517,   -1,  519,  520,   -1,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,  621,  622,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,  641,   -1,  643,  644,  645,
  646,  647,  648,  649,  650,  651,  652,  653,   -1,   -1,
  656,  657,  658,  659,  660,  661,  662,  663,  664,  665,
  666,  667,  668,  669,  670,  671,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,  704,  705,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,  722,  723,  724,  725,
  726,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,   -1,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   91,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,  272,   -1,
   -1,  275,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,   91,   -1,   -1,   -1,   -1,   -1,  291,   -1,  293,
   -1,   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,   -1,
   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,  287,   91,
   -1,   -1,  291,   -1,  293,   -1,   -1,  287,   -1,   -1,
   -1,  291,   -1,  293,   -1,   -1,   -1,  331,   -1,   -1,
   -1,   -1,   -1,  337,  338,  339,   -1,  341,  342,  343,
   -1,  345,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  331,   -1,   -1,  359,   -1,  361,  337,  338,
  339,  331,  341,  342,  343,   -1,  345,  337,  338,  339,
   -1,  341,  342,  343,   -1,  345,   -1,   -1,   -1,   -1,
  359,   -1,  361,   -1,   -1,   -1,  390,   -1,   -1,  359,
   -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  379,   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  390,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  390,   -1,   -1,   -1,  428,  395,   -1,  406,   -1,  262,
   -1,   -1,   -1,  266,   -1,   -1,  406,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,   -1,   -1,  281,  428,
   -1,  284,  262,   -1,   -1,   -1,  266,   -1,  428,   -1,
   -1,  271,  466,  467,   -1,   -1,   -1,  277,   -1,   -1,
   -1,  281,   -1,   -1,  284,   -1,   -1,   -1,   -1,  483,
  262,   -1,   -1,   -1,  266,  318,   -1,  466,  467,  271,
   -1,   -1,  325,   -1,   -1,  277,  466,  467,   -1,  281,
  333,   -1,   -1,   -1,  483,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,  483,   -1,  325,   -1,  350,   -1,  352,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,  318,   -1,  371,   -1,
  350,   -1,  352,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  458,  459,  460,  461,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,  642,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  458,  459,
  460,  461,  462,  463,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  642,   -1,   -1,   -1,   -1,   -1,   -1,
  674,   -1,  642,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  684,   -1,   -1,   -1,   -1,   -1,  690,   -1,   -1,   -1,
   -1,   -1,   -1,  697,   -1,  674,   -1,   -1,  702,   -1,
   -1,   -1,   -1,   -1,  674,  684,  710,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  684,   -1,   -1,   -1,  697,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  697,   -1,   -1,
   -1,   -1,   -1,   -1,  567,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  746,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  567,   -1,   -1,
  593,   -1,   -1,   -1,   -1,   -1,   -1,  746,   -1,  602,
   -1,   -1,   -1,   -1,   -1,   -1,  746,   -1,   -1,   -1,
   -1,   -1,   -1,  593,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  602,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  602,   -1,  257,   -1,  259,   -1,   -1,  609,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,
   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,  741,  742,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,   -1,   -1,  760,   -1,  762,
   -1,  741,  742,   -1,   -1,  370,   -1,   -1,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
  760,  386,  762,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,
  445,  446,   -1,   -1,  449,   -1,  451,  452,  453,  454,
   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,
  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,
  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,
  505,  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,   -1,
  545,  546,  547,  548,  549,  550,   -1,  552,  553,  554,
  555,   -1,  557,   -1,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,   58,  578,  579,  580,  581,   -1,  583,  584,
  585,  586,  587,   -1,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,   -1,  603,  604,
  605,  606,  607,  608,   -1,  610,  611,  612,  613,  614,
  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,
  625,  626,  627,  628,  629,   -1,   -1,  632,  633,  634,
  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,
   -1,  646,   -1,  648,  649,  650,  651,  652,  653,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,  674,
  675,  676,  677,  678,  679,  680,  681,  682,  683,   -1,
  685,  686,  687,  688,  689,  690,  691,  692,  693,  694,
  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,
   -1,  706,  707,  708,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,
   -1,   -1,  727,  728,  729,  730,  731,  732,  733,  734,
  735,  736,  737,  738,  739,  740,   -1,   -1,  743,  744,
   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,
  755,  756,  757,  758,  759,  760,  761,  762,  763,  764,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,  261,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,  270,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
  296,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,  307,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,  325,
  326,   -1,  328,   -1,  330,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
  346,   -1,  348,  349,   -1,  351,   -1,  353,   -1,  355,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,  382,  383,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,  397,   -1,  399,   -1,  401,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,  425,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,   -1,
   -1,  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,  501,  502,  503,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,
  516,  517,   -1,  519,  520,   -1,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   58,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,  621,  622,   -1,  624,  625,
  626,  627,  628,  629,  630,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,  724,  725,
  726,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,  261,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,  270,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,  296,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
  307,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,  325,  326,
   -1,  328,   -1,  330,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,  346,
   -1,  348,  349,   -1,  351,   -1,  353,   -1,  355,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,  383,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,  401,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,  425,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
  437,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,
   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,
  457,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,
   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,
   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,
   -1,   -1,  499,  500,  501,  502,  503,   -1,  505,  506,
  507,   -1,  509,  510,  511,   -1,  513,   -1,  515,  516,
  517,   -1,  519,  520,   -1,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,  540,  541,  542,  543,   -1,  545,  546,
  547,  548,  549,  550,   -1,  552,  553,  554,  555,   -1,
  557,   -1,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
   58,  578,  579,  580,  581,   -1,  583,  584,  585,  586,
  587,   -1,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,   -1,  603,  604,  605,  606,
  607,  608,   -1,  610,  611,  612,  613,  614,  615,  616,
  617,   -1,  619,  620,  621,  622,   -1,  624,  625,  626,
  627,  628,  629,  630,   -1,  632,  633,  634,  635,  636,
  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,
   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,
  677,  678,  679,  680,  681,  682,  683,   -1,  685,  686,
  687,  688,  689,  690,  691,  692,  693,  694,  695,  696,
   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,
  707,  708,  709,  710,  711,  712,  713,  714,  715,  716,
  717,  718,   -1,  720,  721,   -1,   -1,  724,  725,  726,
  727,  728,  729,  730,  731,  732,  733,  734,  735,  736,
  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,
   -1,   -1,  749,  750,  751,  752,  753,  754,  755,  756,
  757,  758,  759,  760,  761,  762,  763,  764,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   60,   61,   62,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   60,   61,   62,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,   41,   42,   43,
   44,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   42,   43,   -1,   45,   46,   47,   91,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,   -1,  350,   -1,  352,  333,   -1,   -1,   42,
   43,   44,   45,   -1,   47,  362,   -1,  364,  365,  366,
   -1,   -1,   -1,  350,  371,  352,   -1,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,
   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,  266,   -1,   -1,  412,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,   -1,   -1,  281,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  458,  459,  460,  461,  462,  463,  464,  465,   -1,
   -1,  262,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
  271,   -1,   -1,   -1,  318,   -1,  277,   -1,   -1,   -1,
  281,  325,   -1,  284,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  362,   -1,
  364,  365,  366,   -1,  325,   -1,   -1,  371,   -1,   -1,
   -1,  332,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  567,  362,   -1,  364,  365,  366,   -1,  368,  412,  262,
  371,   -1,   -1,  266,   -1,   -1,   -1,   -1,  271,  380,
   -1,   -1,   -1,   -1,  277,   -1,  593,   -1,  281,   -1,
   -1,  284,   -1,   -1,   -1,  602,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  412,   -1,   -1,   -1,  602,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,  437,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  454,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  692,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
  717,  259,   -1,   -1,  741,  742,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,  760,  282,  762,  284,   -1,  602,  287,
   -1,   -1,  290,   -1,   -1,  609,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,  602,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,  602,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,  450,  451,  452,  453,  454,  455,  456,  457,
  458,  459,  460,  461,  462,  463,  464,  465,  466,  467,
  468,  469,  470,   -1,   -1,  473,  474,  475,  476,  477,
  478,  479,  480,  481,  482,  483,  484,  485,  486,  487,
  488,  489,  490,  491,  492,  493,  494,  495,  496,  497,
  498,  499,  500,  501,  502,  503,  504,  505,  506,  507,
  508,  509,  510,  511,  512,  513,  514,  515,  516,  517,
  518,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,  544,  545,  546,  547,
  548,  549,  550,  551,  552,  553,  554,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,  577,
  578,  579,  580,  581,  582,  583,  584,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
  618,  619,  620,   -1,   -1,  623,  624,  625,  626,  627,
  628,  629,  630,  631,  632,  633,  634,  635,  636,  637,
  638,  639,  640,  641,  642,  643,  644,  645,  646,   -1,
  648,  649,  650,  651,  652,  653,  654,  655,  656,  657,
  658,  659,  660,  661,  662,  663,  664,  665,  666,  667,
  668,  669,  670,  671,  672,  673,  674,  675,  676,  677,
  678,  679,  680,  681,  682,  683,  684,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,  697,
  698,  699,  700,  701,  702,  703,  704,  705,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,  719,  720,  721,  722,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,  741,  742,  743,  744,  745,  746,  747,
  748,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
  450,  451,  452,  453,  454,   -1,  456,  457,  458,  459,
  460,  461,  462,  463,  464,  465,  466,  467,  468,  469,
  470,   -1,   -1,  473,  474,  475,  476,  477,  478,  479,
  480,  481,  482,  483,  484,  485,  486,  487,  488,  489,
  490,  491,  492,  493,  494,  495,  496,  497,  498,  499,
  500,  501,  502,  503,  504,  505,  506,  507,  508,  509,
  510,  511,  512,  513,  514,  515,  516,  517,  518,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,  544,  545,  546,  547,  548,  549,
  550,  551,  552,  553,  554,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,  577,  578,  579,
  580,  581,  582,  583,  584,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,  618,  619,
  620,   -1,   -1,  623,  624,  625,  626,  627,  628,  629,
  630,  631,  632,  633,  634,  635,  636,  637,  638,  639,
  640,  641,  642,  643,  644,  645,  646,   -1,  648,  649,
  650,  651,  652,  653,  654,  655,  656,  657,  658,  659,
  660,  661,  662,  663,  664,  665,  666,  667,  668,  669,
  670,  671,  672,  673,  674,  675,  676,  677,  678,  679,
  680,  681,  682,  683,  684,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,  697,  698,  699,
  700,  701,  702,  703,  704,  705,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,  719,
  720,  721,  722,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,   -1,  739,
  740,  741,  742,  743,  744,  745,  746,  747,  748,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
  272,   -1,   -1,  275,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,  289,  290,  291,
   -1,  293,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,  331,
   -1,   -1,  334,   -1,  336,  337,  338,  339,  340,  341,
  342,  343,   -1,  345,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,  359,   -1,  361,
   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,  390,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,  406,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,  428,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,  466,  467,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,  483,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,  623,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
  642,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,  674,  675,  676,  677,  678,  679,  680,  681,
  682,  683,  684,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,  697,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,  746,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,  272,   -1,
   -1,  275,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,  291,   -1,  293,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,  331,   -1,   -1,
  334,   -1,  336,  337,  338,  339,  340,  341,  342,  343,
   -1,  345,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,  359,   -1,  361,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,  390,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,  406,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,  428,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,  466,  467,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,  483,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,  642,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
  674,  675,  676,  677,  678,  679,  680,  681,  682,  683,
  684,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,  697,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,  746,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,  272,   -1,   -1,  275,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,  291,   -1,  293,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,  331,   -1,   -1,  334,   -1,
  336,  337,  338,  339,  340,  341,  342,  343,   -1,  345,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,  359,   -1,  361,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,  390,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
  406,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,  428,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
  466,  467,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,  483,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,  642,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,  674,  675,
  676,  677,  678,  679,  680,  681,  682,  683,  684,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,  697,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
  746,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,  291,   -1,  293,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,  331,   -1,   -1,  334,   -1,  336,  337,
  338,  339,  340,  341,  342,  343,   -1,  345,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,  359,   -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,  390,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,  406,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
  428,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,  466,  467,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,  483,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,  642,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,  674,  675,  676,  677,
  678,  679,  680,  681,  682,  683,  684,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,  697,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,  746,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,  291,   -1,  293,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,  331,   -1,   -1,  334,   -1,  336,  337,  338,  339,
  340,  341,  342,  343,   -1,  345,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,  359,
   -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
  390,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,  406,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,  428,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,  466,  467,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,  483,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,  642,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,  674,  675,  676,  677,  678,  679,
  680,  681,  682,  683,  684,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,  697,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,  746,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,  260,  261,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,  286,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,  295,  296,  297,   -1,  299,  300,  301,
   -1,   -1,  304,   -1,  306,  307,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,  330,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,  391,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,  400,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,  425,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,  261,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,  286,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,  295,  296,  297,   -1,  299,  300,  301,   -1,   -1,
  304,   -1,  306,  307,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,  330,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,  391,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,  400,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,  425,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
  316,   -1,   -1,  319,  320,   -1,  322,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,  387,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,  421,  422,   -1,   -1,   -1,
  426,   -1,   -1,   -1,   -1,   -1,  432,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,  329,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,  263,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,  322,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,  647,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,  437,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,  701,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,  355,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,  322,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,  371,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,  577,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,  674,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,  295,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,  283,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,  283,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,  310,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
  426,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,  455,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,  455,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,  301,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,  759,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,  620,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,  689,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,  721,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,  479,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,  489,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,  509,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,  624,  625,  626,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
   -1,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,
   -1,  451,  452,  453,  454,   -1,  456,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,  478,  479,
  480,   -1,   -1,   -1,   -1,  485,  486,   -1,  488,  489,
   -1,  491,   -1,   -1,  494,  495,   -1,   -1,   -1,  499,
  500,   -1,   -1,   -1,   -1,  505,  506,  507,   -1,  509,
  510,  511,   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
  540,  541,  542,  543,   -1,  545,  546,  547,  548,  549,
  550,   -1,  552,  553,  554,  555,   -1,  557,   -1,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,  573,  574,  575,  576,   -1,  578,  579,
  580,  581,   -1,  583,  584,  585,  586,  587,   -1,  589,
  590,  591,  592,  593,  594,  595,  596,   -1,  598,  599,
  600,  601,   -1,  603,  604,  605,  606,  607,  608,   -1,
  610,  611,  612,  613,  614,  615,  616,  617,   -1,  619,
  620,   -1,   -1,   -1,  624,  625,  626,  627,  628,  629,
   -1,   -1,  632,  633,  634,  635,  636,  637,  638,  639,
  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,  648,  649,
  650,  651,  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  672,  673,   -1,  675,  676,  677,  678,  679,
  680,  681,  682,  683,   -1,  685,  686,  687,  688,  689,
  690,  691,  692,  693,  694,  695,  696,   -1,  698,  699,
  700,   -1,  702,   -1,   -1,   -1,  706,  707,  708,  709,
  710,  711,  712,  713,  714,  715,  716,  717,  718,   -1,
  720,  721,   -1,   -1,   -1,   -1,   -1,  727,  728,  729,
  730,  731,  732,  733,  734,  735,  736,  737,  738,  739,
  740,   -1,   -1,  743,  744,   -1,   -1,   -1,   -1,  749,
  750,  751,  752,  753,  754,  755,  756,  757,  758,  759,
  760,  761,  762,  763,  764,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,
   -1,   -1,   -1,  445,   -1,   -1,   -1,  449,   -1,  451,
  452,  453,  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  475,  476,   -1,  478,  479,  480,   -1,
   -1,   -1,   -1,  485,  486,   -1,  488,  489,   -1,  491,
   -1,   -1,  494,  495,   -1,   -1,   -1,  499,  500,   -1,
   -1,   -1,   -1,  505,  506,  507,   -1,  509,  510,  511,
   -1,  513,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,  540,  541,
  542,  543,   -1,  545,  546,  547,  548,  549,  550,   -1,
  552,  553,  554,  555,   -1,  557,   -1,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,  573,  574,  575,  576,   -1,  578,  579,  580,  581,
   -1,  583,  584,  585,  586,  587,   -1,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
   -1,  603,  604,  605,  606,  607,  608,   -1,  610,  611,
  612,  613,  614,  615,  616,  617,   -1,  619,  620,   -1,
   -1,   -1,  624,  625,  626,  627,  628,  629,   -1,   -1,
  632,  633,  634,  635,  636,  637,  638,  639,  640,   -1,
   -1,   -1,   -1,   -1,  646,   -1,  648,  649,  650,  651,
  652,  653,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  672,  673,   -1,  675,  676,  677,  678,  679,  680,  681,
  682,  683,   -1,  685,  686,  687,  688,  689,  690,  691,
  692,  693,  694,  695,  696,   -1,  698,  699,  700,   -1,
  702,   -1,   -1,   -1,  706,  707,  708,  709,  710,  711,
  712,  713,  714,  715,  716,  717,  718,   -1,  720,  721,
   -1,   -1,   -1,   -1,   -1,  727,  728,  729,  730,  731,
  732,  733,  734,  735,  736,  737,  738,  739,  740,   -1,
   -1,  743,  744,   -1,   -1,   -1,   -1,  749,  750,  751,
  752,  753,  754,  755,  756,  757,  758,   -1,  760,  761,
  762,  763,  764,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
   -1,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,   -1,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,
   -1,  445,   -1,   -1,   -1,  449,   -1,  451,  452,  453,
  454,   -1,  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  475,  476,   -1,  478,  479,  480,   -1,   -1,   -1,
   -1,  485,  486,   -1,  488,  489,   -1,  491,   -1,   -1,
  494,  495,   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,
   -1,  505,  506,  507,   -1,  509,  510,  511,   -1,  513,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,  540,  541,  542,  543,
   -1,  545,  546,  547,  548,  549,  550,   -1,  552,  553,
  554,  555,   -1,  557,   -1,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,  573,
  574,  575,  576,   -1,  578,  579,  580,  581,   -1,  583,
  584,  585,  586,  587,   -1,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,   -1,  603,
  604,  605,  606,  607,  608,   -1,  610,  611,  612,  613,
  614,  615,  616,  617,   -1,  619,  620,   -1,   -1,   -1,
  624,  625,  626,  627,  628,  629,   -1,   -1,  632,  633,
  634,  635,  636,  637,  638,  639,  640,   -1,   -1,   -1,
   -1,   -1,  646,   -1,  648,  649,  650,  651,  652,  653,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,
   -1,  675,  676,  677,  678,  679,  680,  681,  682,  683,
   -1,  685,  686,  687,  688,  689,  690,  691,  692,  693,
  694,  695,  696,   -1,  698,  699,  700,   -1,  702,   -1,
   -1,   -1,  706,  707,  708,  709,  710,  711,  712,  713,
  714,  715,  716,  717,  718,   -1,  720,  721,   -1,   -1,
   -1,   -1,   -1,  727,  728,  729,  730,  731,  732,  733,
  734,  735,  736,  737,  738,  739,  740,   -1,   -1,  743,
  744,   -1,   -1,   -1,   -1,  749,  750,  751,  752,  753,
  754,  755,  756,  757,  758,  759,  760,  761,  762,  763,
  764,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,   -1,   -1,   -1,   -1,  445,
   -1,   -1,   -1,  449,   -1,  451,  452,  453,  454,   -1,
  456,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,
  476,   -1,  478,  479,  480,   -1,   -1,   -1,   -1,  485,
  486,   -1,  488,  489,   -1,  491,   -1,   -1,  494,  495,
   -1,   -1,   -1,  499,  500,   -1,   -1,   -1,   -1,  505,
  506,  507,   -1,  509,  510,  511,   -1,  513,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,   -1,  545,
  546,  547,  548,  549,  550,   -1,  552,  553,  554,  555,
   -1,  557,   -1,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,  573,  574,  575,
  576,   -1,  578,  579,  580,  581,   -1,  583,  584,  585,
  586,  587,   -1,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,   -1,  603,  604,  605,
  606,  607,  608,   -1,  610,  611,  612,  613,  614,  615,
  616,  617,   -1,  619,   -1,   -1,   -1,   -1,  624,  625,
  626,  627,  628,  629,   -1,   -1,  632,  633,  634,  635,
  636,  637,  638,  639,  640,   -1,   -1,   -1,   -1,   -1,
  646,   -1,  648,  649,  650,  651,  652,  653,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  672,  673,   -1,  675,
  676,  677,  678,  679,  680,  681,  682,  683,   -1,  685,
  686,  687,  688,   -1,  690,  691,  692,  693,  694,  695,
  696,   -1,  698,  699,  700,   -1,  702,   -1,   -1,   -1,
  706,  707,  708,  709,  710,  711,  712,  713,  714,  715,
  716,  717,  718,   -1,  720,   -1,   -1,   -1,   -1,   -1,
   -1,  727,  728,  729,  730,  731,  732,  733,  734,  735,
  736,  737,  738,  739,  740,   -1,   -1,  743,  744,   -1,
   -1,   -1,   -1,  749,  750,  751,  752,  753,  754,  755,
  756,  757,  758,  759,  760,  761,  762,  763,  764,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,   -1,   -1,   -1,   -1,  445,   -1,   -1,
   -1,  449,   -1,  451,  452,  453,  454,   -1,  456,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  475,  476,   -1,
  478,   -1,  480,   -1,   -1,   -1,   -1,  485,  486,   -1,
  488,   -1,   -1,  491,   -1,   -1,  494,  495,   -1,   -1,
   -1,  499,  500,   -1,   -1,   -1,   -1,  505,  506,  507,
   -1,   -1,  510,  511,   -1,  513,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,   -1,  545,  546,  547,
  548,  549,  550,   -1,  552,  553,  554,  555,   -1,  557,
   -1,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,   -1,
  578,  579,  580,  581,   -1,  583,  584,  585,  586,  587,
   -1,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,   -1,  603,  604,  605,  606,  607,
  608,   -1,  610,  611,  612,  613,  614,  615,  616,  617,
   -1,  619,  620,   -1,   -1,   -1,   -1,   -1,   -1,  627,
  628,  629,   -1,   -1,  632,  633,  634,  635,  636,  637,
  638,  639,  640,   -1,   -1,   -1,   -1,   -1,  646,   -1,
  648,  649,  650,  651,  652,  653,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  672,  673,   -1,  675,  676,  677,
  678,  679,  680,  681,  682,  683,   -1,  685,  686,  687,
  688,  689,  690,  691,  692,  693,  694,  695,  696,   -1,
  698,  699,  700,   -1,  702,   -1,   -1,   -1,  706,  707,
  708,  709,  710,  711,  712,  713,  714,  715,  716,  717,
  718,   -1,  720,  721,   -1,   -1,   -1,   -1,   -1,  727,
  728,  729,  730,  731,  732,  733,  734,  735,  736,  737,
  738,  739,  740,   -1,   -1,  743,  744,   -1,   -1,   -1,
   -1,  749,  750,  751,  752,  753,  754,  755,  756,  757,
  758,  759,  760,  761,  762,  763,  764,  272,   -1,   -1,
  275,   -1,   -1,   -1,   -1,   -1,  272,   -1,   -1,  275,
   -1,   -1,  287,   -1,   -1,   -1,  291,   -1,  293,   -1,
   -1,  287,   -1,   -1,   -1,  291,   -1,  293,   -1,   -1,
   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  291,   -1,  293,   -1,   -1,  331,   -1,   -1,   -1,
   -1,   -1,  337,  338,  339,  331,  341,  342,  343,   -1,
  345,  337,  338,  339,   -1,  341,  342,  343,   -1,  345,
   -1,   -1,   -1,   -1,  359,   -1,  361,   -1,   -1,   -1,
   -1,  331,   -1,  359,   -1,  361,   -1,  337,  338,  339,
   -1,  341,  342,  343,  379,  345,  272,   -1,   -1,  275,
   -1,   -1,   -1,   -1,   -1,  390,   -1,   -1,   -1,  359,
   -1,  361,   -1,   -1,  390,  291,   -1,  293,   -1,   -1,
   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  390,   -1,   -1,  428,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  428,   -1,  272,  331,  406,  275,   -1,   -1,
   -1,  337,  338,  339,   -1,  341,  342,  343,   -1,  345,
   -1,   -1,   -1,  291,   -1,  293,   -1,   -1,  428,   -1,
   -1,  466,  467,  359,   -1,  361,   -1,   -1,   -1,   -1,
  466,  467,   -1,   -1,   -1,   -1,   -1,   -1,  483,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  483,   -1,   -1,
   -1,   -1,   -1,  331,  390,   -1,  466,  467,   -1,  337,
  338,  339,   -1,  341,  342,  343,   -1,  345,   -1,   -1,
  406,   -1,   -1,  483,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  359,   -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  428,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  390,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  406,   -1,
  466,  467,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  483,   -1,   -1,
  428,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  466,  467,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  642,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  483,  642,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  674,
   -1,   -1,  642,   -1,   -1,   -1,   -1,   -1,  674,  684,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  684,   -1,
   -1,   -1,  697,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  697,   -1,   -1,  674,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  684,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  697,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  746,   -1,   -1,   -1,   -1,  642,   -1,   -1,   -1,
  746,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  746,   -1,  674,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  684,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  697,   -1,   -1,  642,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  674,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  684,   -1,   -1,   -1,
  746,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  697,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  746,
};
static int yyctable[] = {                                     -1,
 1375, 1414,   -1,  763, 1355,   -1,  764, 1355,   -1, 1568,
 1569,   -1,  809,  322,   -1,  810,  322,   -1,  410, 2306,
   -1,   41,   53,   -1, 1127,  128,   -1,  128,  130,   -1,
  764, 1357,   -1, 1389, 2005,   -1, 1341, 1340,   -1, 2124,
 2131,   -1, 1579, 1321,   -1, 1646,  579,   -1, 1838, 1714,
   -1, 1839, 1713,   -1, 1495, 1340,   -1, 1445, 1375,   -1,
 1456, 1375,   -1, 1646,  580,   -1, 2261, 1622,   -1,  334,
 1436,   -1,  372, 1436,   -1,  398, 1436,   -1,  714, 1407,
   -1, 1375, 1407,   -1, 2337, 1198,   -1, 2512, 1005,   -1,
 2677, 1624,   -1, 2679, 1626,   -1,   91,   98,   -1, 2717,
 1390,   -1,  247, 1390,   -1,  425, 1390,   -1,  229,  729,
   -1,  234,  729,   -1,  243,  729,   -1,  335,  729,   -1,
  336,  729,   -1,  337,  729,   -1, 2842, 1097,   -1, 2867,
  902,   -1,  911, 2110,   -1, 3021, 1628,   -1,  714, 1409,
   -1, 1375, 1409,   -1, 2671, 1347,   -1,  314, 2139,   -1,
 3394, 1270,   -1, 3395, 1270,   -1, 2869, 1095,   -1,  314,
 2314,   -1, 3592, 1282,   -1, 3639,  455,   -1,  424,  435,
   -1, 3873,  657,   -1,  165, 2349,   -1,  208, 2349,   -1,
};
#define YYFINAL 20
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 766
#if YYDEBUG
static const char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,"'('","')'","'*'","'+'","','","'-'","'.'","'/'",0,0,0,0,0,0,0,0,0,0,
"':'","';'","'<'","'='","'>'","'?'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,"'['",0,"']'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"ACTIVE","ADD","AFTER","ALL","ALTER","AND","ANY",
"AS","ASC","AT","AVG","AUTO","BEFORE","BEGIN","BETWEEN","BLOB","BY","CAST",
"CHARACTER","CHECK","COLLATE","COMMIT","COMMITTED","COMPUTED","CONCATENATE",
"CONDITIONAL","CONSTRAINT","CONTAINING","COUNT","CREATE","CSTRING","CURRENT",
"CURSOR","DATABASE","DATE","DB_KEY","DECIMAL","DECLARE","DEFAULT","DELETE",
"DESC","DISTINCT","DO","DOMAIN","DROP","ELSE","END","ENTRY_POINT","ESCAPE",
"EXCEPTION","EXECUTE","EXISTS","EXIT","EXTERNAL","FILTER","FOR","FOREIGN",
"FROM","FULL","FUNCTION","GDSCODE","GEQ","GENERATOR","GEN_ID","GRANT","GROUP",
"HAVING","IF","IN","INACTIVE","INNER","INPUT_TYPE","INDEX","INSERT","INTEGER",
"INTO","IS","ISOLATION","JOIN","KEY","CHAR","DEC","DOUBLE","FILE","FLOAT","INT",
"LONG","NULL","NUMERIC","UPPER","VALUE","LENGTH","LEFT","LEQ","LEVEL","LIKE",
"MANUAL","MAXIMUM","MERGE","MINIMUM","MODULE_NAME","NAMES","NATIONAL","NATURAL",
"NCHAR","NEQ","NO","NOT","NOT_GTR","NOT_LSS","OF","ON","ONLY","OPTION","OR",
"ORDER","OUTER","OUTPUT_TYPE","OVERFLOW","PAGE","PAGES","PAGE_SIZE","PARAMETER",
"PASSWORD","PLAN","POSITION","POST_EVENT","PRECISION","PRIMARY","PRIVILEGES",
"PROCEDURE","PROTECTED","READ","REAL","REFERENCES","RESERVING","RETAIN",
"RETURNING_VALUES","RETURNS","REVOKE","RIGHT","ROLLBACK","SEGMENT","SELECT",
"SET","SHADOW","SHARED","SINGULAR","SIZE","SMALLINT","SNAPSHOT","SOME","SORT",
"SQLCODE","STABILITY","STARTING","STATISTICS","SUB_TYPE","SUSPEND","SUM",
"TABLE","THEN","TO","TRANSACTION","TRIGGER","UNCOMMITTED","UNION","UNIQUE",
"UPDATE","USER","VALUES","VARCHAR","VARIABLE","VARYING","VERSION","VIEW","WAIT",
"WHEN","WHERE","WHILE","WITH","WORK","WRITE","FLOAT_NUMBER","DECIMAL_NUMBER",
"LIMIT64_NUMBER","LIMIT64_INT","NUM128","SYMBOL","NUMBER32BIT","STRING",
"INTRODUCER","ACTION","ADMIN","CASCADE","FREE_IT","RESTRICT","ROLE","COLUMN",
"TYPE","EXTRACT","YEAR","MONTH","DAY","HOUR","MINUTE","SECOND","WEEKDAY",
"YEARDAY","TIME","TIMESTAMP","CURRENT_DATE","CURRENT_TIME","CURRENT_TIMESTAMP",
"NUMBER64BIT","SCALEDINT","CURRENT_USER","CURRENT_ROLE","BREAK","SUBSTRING",
"RECREATE","DESCRIPTOR","FIRST","SKIP","CURRENT_CONNECTION",
"CURRENT_TRANSACTION","BIGINT","CASE","NULLIF","COALESCE","USING","NULLS",
"LAST","ROW_COUNT","LOCK","SAVEPOINT","RELEASE","STATEMENT","LEAVE","INSERTING",
"UPDATING","DELETING","BACKUP","DIFFERENCE","OPEN","CLOSE","FETCH","ROWS",
"BLOCK","IIF","SCALAR_ARRAY","CROSS","NEXT","SEQUENCE","RESTART","BOTH",
"COLLATION","COMMENT","BIT_LENGTH","CHAR_LENGTH","CHARACTER_LENGTH","LEADING",
"LOWER","OCTET_LENGTH","TRAILING","TRIM","RETURNING","IGNORE","LIMBO","UNDO",
"REQUESTS","TIMEOUT","ABS","ACCENT","ACOS","ALWAYS","ASCII_CHAR","ASCII_VAL",
"ASIN","ATAN","ATAN2","BIN_AND","BIN_OR","BIN_SHL","BIN_SHR","BIN_XOR","CEIL",
"CONNECT","COS","COSH","COT","DATEADD","DATEDIFF","DECODE","DISCONNECT","EXP",
"FLOOR","GEN_UUID","GENERATED","GLOBAL","HASH","INSENSITIVE","LIST","LN","LOG",
"LOG10","LPAD","MATCHED","MATCHING","MAXVALUE","MILLISECOND","MINVALUE","MOD",
"OVERLAY","PAD","PI","PLACING","POWER","PRESERVE","RAND","RECURSIVE","REPLACE",
"REVERSE","ROUND","RPAD","SENSITIVE","SIGN","SIN","SINH","SPACE","SQRT","START",
"TAN","TANH","TEMPORARY","TRUNC","WEEK","AUTONOMOUS","CHAR_TO_UUID","FIRSTNAME",
"GRANTED","LASTNAME","MIDDLENAME","MAPPING","OS_NAME","SIMILAR","UUID_TO_CHAR",
"CALLER","COMMON","DATA","SOURCE","TWO_PHASE","BIND_PARAM","BIN_NOT","BODY",
"CONTINUE","DDL","DECRYPT","ENCRYPT","ENGINE","NAME","OVER","PACKAGE",
"PARTITION","RDB_GET_CONTEXT","RDB_SET_CONTEXT","SCROLL","PRIOR","ABSOLUTE",
"RELATIVE","ACOSH","ASINH","ATANH","RETURN","DETERMINISTIC","IDENTITY",
"DENSE_RANK","FIRST_VALUE","NTH_VALUE","LAST_VALUE","LAG","LEAD","RANK",
"ROW_NUMBER","SQLSTATE","BOOLEAN","FALSE","TRUE","UNKNOWN","USAGE",
"RDB_RECORD_VERSION","LINGER","TAGS","PLUGIN","SERVERWIDE","INCREMENT",
"TRUSTED","ROW","OFFSET","STDDEV_SAMP","STDDEV_POP","VAR_SAMP","VAR_POP",
"COVAR_SAMP","COVAR_POP","CORR","REGR_AVGX","REGR_AVGY","REGR_COUNT",
"REGR_INTERCEPT","REGR_R2","REGR_SLOPE","REGR_SXX","REGR_SXY","REGR_SYY",
"BASE64_DECODE","BASE64_ENCODE","BINARY","BIND","COMPARE_DECFLOAT",
"CONSISTENCY","COUNTER","CRYPT_HASH","CTR_BIG_ENDIAN","CTR_LENGTH",
"CTR_LITTLE_ENDIAN","CUME_DIST","DECFLOAT","DEFINER","DISABLE","ENABLE",
"EXCESS","EXCLUDE","EXTENDED","FIRST_DAY","FOLLOWING","HEX_DECODE","HEX_ENCODE",
"IDLE","INCLUDE","INT128","INVOKER","IV","LAST_DAY","LATERAL","LEGACY","LOCAL",
"LOCALTIME","LOCALTIMESTAMP","LPARAM","MAKE_DBKEY","MESSAGE","MODE","NATIVE",
"NORMALIZE_DECFLOAT","NTILE","NUMBER","OTHERS","OVERRIDING","PERCENT_RANK",
"PRECEDING","PRIVILEGE","PUBLICATION","QUANTIZE","RANGE","RESETTING",
"RDB_ERROR","RDB_GET_TRANSACTION_CN","RDB_ROLE_IN_USE","RDB_SYSTEM_PRIVILEGE",
"RESET","RSA_DECRYPT","RSA_ENCRYPT","RSA_PRIVATE","RSA_PUBLIC","RSA_SIGN_HASH",
"RSA_VERIFY_HASH","SALT_LENGTH","SECURITY","SESSION","SIGNATURE","SQL","SYSTEM",
"TIES","TIMEZONE_HOUR","TIMEZONE_MINUTE","TOTALORDER","TRAPS","UNBOUNDED",
"VARBINARY","WINDOW","WITHOUT","ZONE","CONNECTIONS","POOL","LIFETIME","CLEAR",
"OLDEST","DEBUG","PKCS_1_5","BLOB_APPEND","LOCKED","OPTIMIZE","QUARTER",
"TARGET","TIMEZONE_NAME","UNICODE_CHAR","UNICODE_VAL","UMINUS","UPLUS",
};
static const char *yyrule[] = {
"$accept : top",
"top : statement",
"top : statement ';'",
"statement : dml_statement",
"statement : ddl_statement",
"statement : tra_statement",
"statement : mng_statement",
"dml_statement : delete",
"dml_statement : insert",
"dml_statement : merge",
"dml_statement : exec_procedure",
"dml_statement : exec_block",
"dml_statement : select",
"dml_statement : update",
"dml_statement : update_or_insert",
"ddl_statement : alter",
"ddl_statement : comment",
"ddl_statement : create",
"ddl_statement : create_or_alter",
"ddl_statement : declare",
"ddl_statement : drop",
"ddl_statement : grant",
"ddl_statement : recreate",
"ddl_statement : revoke",
"ddl_statement : set_statistics",
"tra_statement : set_transaction",
"tra_statement : savepoint",
"tra_statement : commit",
"tra_statement : rollback",
"mng_statement : set_debug_option",
"mng_statement : set_decfloat_round",
"mng_statement : set_decfloat_traps",
"mng_statement : session_statement",
"mng_statement : set_role",
"mng_statement : session_reset",
"mng_statement : set_time_zone",
"mng_statement : set_bind",
"mng_statement : set_optimize",
"$$1 :",
"$$2 :",
"grant : GRANT $$1 $$2 grant0",
"$$3 :",
"$$4 :",
"grant0 : $$3 privileges ON table_noise symbol_table_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 execute_privilege ON PROCEDURE symbol_procedure_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 execute_privilege ON FUNCTION symbol_UDF_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 execute_privilege ON PACKAGE symbol_package_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 usage_privilege ON EXCEPTION symbol_exception_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 usage_privilege ON GENERATOR symbol_generator_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 usage_privilege ON SEQUENCE symbol_generator_name TO $$4 non_role_grantee_list grant_option granted_by",
"$$5 :",
"grant0 : $$3 ddl_privileges object TO $$5 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 db_ddl_privileges DATABASE TO $$5 non_role_grantee_list grant_option granted_by",
"$$6 :",
"$$7 :",
"grant0 : $$6 role_name_list TO $$7 role_grantee_list role_admin_option granted_by",
"object : TABLE",
"object : VIEW",
"object : PROCEDURE",
"object : FUNCTION",
"object : PACKAGE",
"object : GENERATOR",
"object : SEQUENCE",
"object : DOMAIN",
"object : EXCEPTION",
"object : ROLE",
"object : CHARACTER SET",
"object : COLLATION",
"object : FILTER",
"table_noise :",
"table_noise : TABLE",
"privileges : ALL",
"privileges : ALL PRIVILEGES",
"$$8 :",
"privileges : $$8 privilege_list",
"privilege_list : $$8 privilege",
"$$9 :",
"privilege_list : privilege_list ',' $$9 privilege",
"execute_privilege : EXECUTE",
"usage_privilege : USAGE",
"privilege : SELECT",
"privilege : INSERT",
"privilege : DELETE",
"privilege : UPDATE column_parens_opt",
"privilege : REFERENCES column_parens_opt",
"ddl_privileges : ALL privileges_opt",
"ddl_privileges : $$8 ddl_privilege_list",
"privileges_opt :",
"privileges_opt : PRIVILEGES",
"ddl_privilege_list : $$8 ddl_privilege",
"ddl_privilege_list : ddl_privilege_list ',' $$9 ddl_privilege",
"ddl_privilege : CREATE",
"ddl_privilege : ALTER ANY",
"ddl_privilege : DROP ANY",
"db_ddl_privileges : ALL privileges_opt",
"db_ddl_privileges : $$8 db_ddl_privilege_list",
"db_ddl_privilege_list : $$8 db_ddl_privilege",
"db_ddl_privilege_list : db_ddl_privilege_list ',' $$9 db_ddl_privilege",
"db_ddl_privilege : CREATE",
"db_ddl_privilege : ALTER",
"db_ddl_privilege : DROP",
"grant_option :",
"grant_option : WITH GRANT OPTION",
"role_admin_option :",
"role_admin_option : WITH ADMIN OPTION",
"granted_by :",
"granted_by : granted_by_text grantor",
"granted_by_text : GRANTED BY",
"granted_by_text : AS",
"grantor : symbol_user_name",
"grantor : USER symbol_user_name",
"$$10 :",
"revoke : REVOKE $$10 $$2 revoke0",
"$$11 :",
"$$12 :",
"revoke0 : rev_grant_option $$11 privileges ON table_noise symbol_table_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 execute_privilege ON PROCEDURE symbol_procedure_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 execute_privilege ON FUNCTION symbol_UDF_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 execute_privilege ON PACKAGE symbol_package_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 usage_privilege ON EXCEPTION symbol_exception_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 usage_privilege ON GENERATOR symbol_generator_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 usage_privilege ON SEQUENCE symbol_generator_name FROM $$12 non_role_grantee_list granted_by",
"$$13 :",
"revoke0 : rev_grant_option $$11 ddl_privileges object FROM $$13 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 db_ddl_privileges DATABASE FROM $$13 non_role_grantee_list granted_by",
"$$14 :",
"revoke0 : rev_admin_option $$14 role_name_list FROM $$5 role_grantee_list granted_by",
"revoke0 : ALL ON ALL FROM $$5 non_role_grantee_list",
"rev_grant_option :",
"rev_grant_option : GRANT OPTION FOR",
"rev_admin_option :",
"rev_admin_option : ADMIN OPTION FOR",
"$$15 :",
"non_role_grantee_list : $$15 grantee",
"non_role_grantee_list : $$15 user_grantee",
"$$16 :",
"non_role_grantee_list : non_role_grantee_list ',' $$16 grantee",
"non_role_grantee_list : non_role_grantee_list ',' $$16 user_grantee",
"grantee : PROCEDURE symbol_procedure_name",
"grantee : FUNCTION symbol_UDF_name",
"grantee : PACKAGE symbol_package_name",
"grantee : TRIGGER symbol_trigger_name",
"grantee : VIEW symbol_view_name",
"grantee : ROLE symbol_role_name",
"grantee : SYSTEM PRIVILEGE valid_symbol_name",
"user_grantee : symbol_user_name",
"user_grantee : USER symbol_user_name",
"user_grantee : GROUP symbol_user_name",
"role_name_list : $$2 role_name",
"$$17 :",
"role_name_list : role_name_list ',' $$17 role_name",
"role_name : symbol_role_name",
"role_name : DEFAULT symbol_role_name",
"role_grantee_list : $$15 role_grantee",
"role_grantee_list : role_grantee_list ',' $$16 role_grantee",
"role_grantee : symbol_user_name",
"role_grantee : USER symbol_user_name",
"role_grantee : ROLE symbol_user_name",
"declare : DECLARE declare_clause",
"declare_clause : FILTER filter_decl_clause",
"declare_clause : EXTERNAL FUNCTION udf_decl_clause",
"$$18 :",
"$$19 :",
"$$20 :",
"udf_decl_clause : symbol_UDF_name $$18 $$19 arg_desc_list1 RETURNS $$20 return_value1 ENTRY_POINT utf_string MODULE_NAME utf_string",
"udf_data_type : simple_type",
"udf_data_type : BLOB",
"udf_data_type : CSTRING '(' pos_short_integer ')' charset_clause",
"arg_desc_list1 :",
"$$21 :",
"arg_desc_list1 : $$21 arg_desc_list",
"$$22 :",
"arg_desc_list1 : '(' $$22 arg_desc_list ')'",
"arg_desc_list : $$21 arg_desc",
"$$23 :",
"arg_desc_list : arg_desc_list ',' $$23 arg_desc",
"arg_desc : udf_data_type param_mechanism",
"param_mechanism :",
"param_mechanism : BY DESCRIPTOR",
"param_mechanism : BY SCALAR_ARRAY",
"param_mechanism : NULL",
"$$24 :",
"return_value1 : $$24 return_value",
"$$25 :",
"return_value1 : '(' $$25 return_value ')'",
"return_value : udf_data_type return_mechanism",
"return_value : PARAMETER pos_short_integer",
"return_mechanism :",
"return_mechanism : BY VALUE",
"return_mechanism : BY DESCRIPTOR",
"return_mechanism : FREE_IT",
"return_mechanism : BY DESCRIPTOR FREE_IT",
"filter_decl_clause : symbol_filter_name INPUT_TYPE blob_filter_subtype OUTPUT_TYPE blob_filter_subtype ENTRY_POINT utf_string MODULE_NAME utf_string",
"blob_filter_subtype : symbol_blob_subtype_name",
"blob_filter_subtype : signed_short_integer",
"create : CREATE create_clause",
"create_clause : EXCEPTION exception_clause",
"$$26 :",
"$$27 :",
"create_clause : unique_opt order_direction INDEX symbol_index_name ON simple_table_name $$26 $$27 index_definition",
"create_clause : FUNCTION function_clause",
"create_clause : PROCEDURE procedure_clause",
"create_clause : TABLE table_clause",
"create_clause : GLOBAL TEMPORARY TABLE gtt_table_clause",
"create_clause : TRIGGER trigger_clause",
"create_clause : VIEW view_clause",
"create_clause : GENERATOR generator_clause",
"create_clause : SEQUENCE generator_clause",
"create_clause : DATABASE db_clause",
"create_clause : DOMAIN domain_clause",
"create_clause : SHADOW shadow_clause",
"create_clause : ROLE role_clause",
"create_clause : COLLATION collation_clause",
"create_clause : USER create_user_clause",
"create_clause : PACKAGE package_clause",
"create_clause : PACKAGE BODY package_body_clause",
"$$28 :",
"create_clause : MAPPING $$28 create_map_clause",
"$$29 :",
"create_clause : GLOBAL MAPPING $$29 create_map_clause",
"recreate : RECREATE recreate_clause",
"recreate_clause : PROCEDURE procedure_clause",
"recreate_clause : FUNCTION function_clause",
"recreate_clause : TABLE table_clause",
"recreate_clause : GLOBAL TEMPORARY TABLE gtt_table_clause",
"recreate_clause : VIEW view_clause",
"recreate_clause : TRIGGER trigger_clause",
"recreate_clause : PACKAGE package_clause",
"recreate_clause : PACKAGE BODY package_body_clause",
"recreate_clause : EXCEPTION exception_clause",
"recreate_clause : GENERATOR generator_clause",
"recreate_clause : SEQUENCE generator_clause",
"recreate_clause : USER create_user_clause",
"create_or_alter : CREATE OR ALTER replace_clause",
"replace_clause : PROCEDURE replace_procedure_clause",
"replace_clause : FUNCTION replace_function_clause",
"replace_clause : TRIGGER replace_trigger_clause",
"replace_clause : PACKAGE replace_package_clause",
"replace_clause : VIEW replace_view_clause",
"replace_clause : EXCEPTION replace_exception_clause",
"replace_clause : GENERATOR replace_sequence_clause",
"replace_clause : SEQUENCE replace_sequence_clause",
"replace_clause : USER replace_user_clause",
"replace_clause : MAPPING $$28 replace_map_clause",
"replace_clause : GLOBAL MAPPING $$29 replace_map_clause",
"exception_clause : symbol_exception_name sql_string",
"replace_exception_clause : symbol_exception_name sql_string",
"alter_exception_clause : symbol_exception_name sql_string",
"unique_opt :",
"unique_opt : UNIQUE",
"$$30 :",
"index_definition : $$30 index_column_expr index_condition_opt",
"index_column_expr : column_list",
"index_column_expr : column_parens",
"index_column_expr : computed_by '(' value ')'",
"index_condition_opt :",
"index_condition_opt : WHERE search_condition",
"$$31 :",
"$$32 :",
"shadow_clause : pos_short_integer manual_auto conditional utf_string first_file_length $$31 $$32 sec_shadow_files",
"manual_auto :",
"manual_auto : MANUAL",
"manual_auto : AUTO",
"conditional :",
"conditional : CONDITIONAL",
"first_file_length :",
"first_file_length : LENGTH equals long_integer page_noise",
"sec_shadow_files :",
"$$33 :",
"sec_shadow_files : $$33 db_file_list",
"db_file_list : db_file",
"db_file_list : db_file_list db_file",
"$$34 :",
"$$35 :",
"domain_clause : symbol_column_name as_opt data_type domain_default_opt $$34 $$35 domain_constraints_opt collate_clause",
"domain_constraints_opt :",
"domain_constraints_opt : $$35 domain_constraints",
"domain_constraints : $$35 domain_constraint",
"$$36 :",
"domain_constraints : domain_constraints $$36 domain_constraint",
"domain_constraint : null_constraint",
"domain_constraint : check_constraint",
"as_opt :",
"as_opt : AS",
"domain_default : DEFAULT default_value",
"domain_default_opt :",
"domain_default_opt : domain_default",
"null_constraint : NOT NULL",
"check_constraint : CHECK '(' search_condition ')'",
"$$37 :",
"$$38 :",
"generator_clause : symbol_generator_name $$37 $$38 create_sequence_options",
"create_sequence_options :",
"$$39 :",
"create_sequence_options : $$38 create_seq_option $$39 create_sequence_options",
"create_seq_option : $$38 start_with_opt",
"create_seq_option : $$38 step_option",
"start_with_opt : START WITH sequence_value",
"step_option : INCREMENT by_noise signed_long_integer",
"by_noise :",
"by_noise : BY",
"$$40 :",
"replace_sequence_clause : symbol_generator_name $$40 $$38 replace_sequence_options",
"replace_sequence_options :",
"replace_sequence_options : $$38 replace_seq_option $$39 replace_sequence_options",
"replace_seq_option : RESTART",
"replace_seq_option : $$38 start_with_opt",
"replace_seq_option : $$38 step_option",
"$$41 :",
"alter_sequence_clause : symbol_generator_name $$41 $$38 alter_sequence_options",
"alter_sequence_options :",
"alter_sequence_options : $$38 alter_seq_option $$39 alter_sequence_options",
"alter_seq_option : $$38 restart_option",
"alter_seq_option : $$38 step_option",
"restart_option : RESTART with_opt",
"with_opt :",
"with_opt : WITH sequence_value",
"set_generator_clause : SET GENERATOR symbol_generator_name TO sequence_value",
"sequence_value : signed_long_integer",
"sequence_value : NUMBER64BIT",
"sequence_value : '-' NUMBER64BIT",
"sequence_value : '-' LIMIT64_INT",
"$$42 :",
"$$43 :",
"role_clause : symbol_role_name $$42 $$43 opt_system_privileges",
"opt_system_privileges :",
"opt_system_privileges : $$43 set_system_privileges",
"opt_system_privileges : $$43 drop_system_privileges",
"$$44 :",
"set_system_privileges : SET SYSTEM PRIVILEGES TO $$44 system_privileges_list",
"drop_system_privileges : DROP SYSTEM PRIVILEGES",
"system_privileges_list : $$43 system_privilege",
"$$45 :",
"system_privileges_list : system_privileges_list ',' $$45 system_privilege",
"system_privilege : valid_symbol_name",
"$$46 :",
"$$47 :",
"$$48 :",
"$$49 :",
"collation_clause : symbol_collation_name FOR symbol_character_set_name $$46 $$47 collation_sequence_definition $$48 collation_attribute_list_opt $$49 collation_specific_attribute_opt",
"collation_sequence_definition :",
"collation_sequence_definition : FROM symbol_collation_name",
"collation_sequence_definition : FROM EXTERNAL '(' utf_string ')'",
"collation_attribute_list_opt :",
"collation_attribute_list_opt : $$47 collation_attribute_list",
"collation_attribute_list : $$47 collation_attribute",
"$$50 :",
"collation_attribute_list : collation_attribute_list $$50 collation_attribute",
"collation_attribute : $$47 collation_pad_attribute",
"collation_attribute : $$47 collation_case_attribute",
"collation_attribute : $$47 collation_accent_attribute",
"collation_pad_attribute : NO PAD",
"collation_pad_attribute : PAD SPACE",
"collation_case_attribute : CASE SENSITIVE",
"collation_case_attribute : CASE INSENSITIVE",
"collation_accent_attribute : ACCENT SENSITIVE",
"collation_accent_attribute : ACCENT INSENSITIVE",
"collation_specific_attribute_opt :",
"collation_specific_attribute_opt : utf_string",
"alter_charset_clause : symbol_character_set_name SET DEFAULT COLLATION symbol_collation_name",
"alter_eds_conn_pool_clause : SET SIZE unsigned_short_integer",
"alter_eds_conn_pool_clause : SET LIFETIME unsigned_short_integer eds_pool_lifetime_mult",
"alter_eds_conn_pool_clause : CLEAR sql_string",
"alter_eds_conn_pool_clause : CLEAR ALL",
"alter_eds_conn_pool_clause : CLEAR OLDEST",
"eds_pool_lifetime_mult : HOUR",
"eds_pool_lifetime_mult : MINUTE",
"eds_pool_lifetime_mult : SECOND",
"$$51 :",
"$$52 :",
"$$53 :",
"db_clause : db_name $$51 $$52 db_initial_desc1 $$53 db_rem_desc1",
"equals :",
"equals : '='",
"db_name : utf_string",
"db_initial_desc1 :",
"db_initial_desc1 : $$52 db_initial_desc",
"db_initial_desc : $$52 db_initial_option",
"$$54 :",
"db_initial_desc : db_initial_desc $$54 db_initial_option",
"db_initial_option : PAGE_SIZE equals NUMBER32BIT",
"db_initial_option : USER symbol_user_name",
"db_initial_option : USER utf_string",
"db_initial_option : ROLE valid_symbol_name",
"db_initial_option : ROLE utf_string",
"db_initial_option : PASSWORD utf_string",
"db_initial_option : SET NAMES utf_string",
"db_initial_option : LENGTH equals long_integer page_noise",
"db_rem_desc1 :",
"db_rem_desc1 : $$52 db_rem_desc",
"db_rem_desc : $$52 db_rem_option",
"db_rem_desc : db_rem_desc $$54 db_rem_option",
"db_rem_option : db_file",
"db_rem_option : DEFAULT CHARACTER SET symbol_character_set_name",
"db_rem_option : DEFAULT CHARACTER SET symbol_character_set_name COLLATION symbol_collation_name",
"db_rem_option : DIFFERENCE FILE utf_string",
"$$55 :",
"$$56 :",
"db_file : FILE utf_string $$55 $$56 file_desc1",
"file_desc1 :",
"file_desc1 : $$56 file_desc",
"file_desc : $$56 file_clause",
"$$57 :",
"file_desc : file_desc $$57 file_clause",
"file_clause : STARTING file_clause_noise long_integer",
"file_clause : LENGTH equals long_integer page_noise",
"file_clause_noise :",
"file_clause_noise : AT",
"file_clause_noise : AT PAGE",
"page_noise :",
"page_noise : PAGE",
"page_noise : PAGES",
"$$58 :",
"$$59 :",
"$$60 :",
"table_clause : simple_table_name external_file $$58 '(' $$59 table_elements ')' $$60 table_attributes",
"table_attributes :",
"$$61 :",
"$$62 :",
"table_attributes : $$61 table_attribute $$62 table_attributes",
"table_attribute : sql_security_clause",
"table_attribute : publication_state",
"sql_security_clause : SQL SECURITY DEFINER",
"sql_security_clause : SQL SECURITY INVOKER",
"sql_security_clause_opt :",
"sql_security_clause_opt : sql_security_clause",
"publication_state : ENABLE PUBLICATION",
"publication_state : DISABLE PUBLICATION",
"$$63 :",
"$$64 :",
"gtt_table_clause : simple_table_name $$63 '(' $$59 table_elements ')' $$64 gtt_ops",
"$$65 :",
"gtt_ops : $$65 gtt_op",
"$$66 :",
"gtt_ops : gtt_ops ',' $$66 gtt_op",
"gtt_op :",
"gtt_op : sql_security_clause_opt",
"gtt_op : ON COMMIT DELETE ROWS",
"gtt_op : ON COMMIT PRESERVE ROWS",
"external_file :",
"external_file : EXTERNAL FILE utf_string",
"external_file : EXTERNAL utf_string",
"table_elements : $$65 table_element",
"table_elements : table_elements ',' $$66 table_element",
"$$67 :",
"table_element : $$67 column_def",
"table_element : $$67 table_constraint_definition",
"$$68 :",
"$$69 :",
"column_def : symbol_column_name data_type_or_domain domain_default_opt $$68 $$69 column_constraint_clause collate_clause",
"$$70 :",
"column_def : symbol_column_name data_type_or_domain identity_clause $$70 $$69 column_constraint_clause collate_clause",
"column_def : symbol_column_name non_array_type def_computed",
"column_def : symbol_column_name def_computed",
"$$71 :",
"$$72 :",
"identity_clause : GENERATED identity_clause_type AS IDENTITY $$71 $$72 identity_clause_options_opt",
"identity_clause_type : BY DEFAULT",
"identity_clause_type : ALWAYS",
"identity_clause_options_opt :",
"$$73 :",
"identity_clause_options_opt : '(' $$73 identity_clause_options ')'",
"identity_clause_options : identity_clause_options $$73 identity_clause_option",
"identity_clause_options : $$72 identity_clause_option",
"identity_clause_option : START WITH sequence_value",
"identity_clause_option : INCREMENT by_noise signed_long_integer",
"def_computed : computed_clause '(' value ')'",
"computed_clause : computed_by",
"computed_clause : generated_always_clause",
"generated_always_clause : GENERATED ALWAYS AS",
"computed_by : COMPUTED BY",
"computed_by : COMPUTED",
"data_type_or_domain : data_type",
"data_type_or_domain : symbol_column_name",
"collate_clause :",
"collate_clause : COLLATE symbol_collation_name",
"data_type_descriptor : data_type",
"data_type_descriptor : TYPE OF symbol_column_name",
"data_type_descriptor : TYPE OF COLUMN symbol_column_name '.' symbol_column_name",
"data_type_descriptor : symbol_column_name",
"default_value : constant",
"default_value : current_user",
"default_value : current_role",
"default_value : internal_info",
"default_value : null_value",
"default_value : datetime_value_expression",
"column_constraint_clause :",
"$$74 :",
"column_constraint_clause : $$74 column_constraint_list",
"column_constraint_list : $$74 column_constraint_def",
"$$75 :",
"column_constraint_list : column_constraint_list $$75 column_constraint_def",
"column_constraint_def : constraint_name_opt $$75 column_constraint",
"column_constraint : null_constraint",
"column_constraint : check_constraint",
"column_constraint : REFERENCES symbol_table_name column_parens_opt referential_trigger_action constraint_index_opt",
"column_constraint : UNIQUE constraint_index_opt",
"column_constraint : PRIMARY KEY constraint_index_opt",
"$$76 :",
"table_constraint_definition : constraint_name_opt $$76 table_constraint",
"constraint_name_opt :",
"constraint_name_opt : CONSTRAINT symbol_constraint_name",
"table_constraint : UNIQUE column_parens constraint_index_opt",
"table_constraint : PRIMARY KEY column_parens constraint_index_opt",
"table_constraint : FOREIGN KEY column_parens REFERENCES symbol_table_name column_parens_opt referential_trigger_action constraint_index_opt",
"table_constraint : check_constraint",
"constraint_index_opt :",
"constraint_index_opt : USING order_direction INDEX symbol_index_name",
"referential_trigger_action :",
"referential_trigger_action : update_rule",
"referential_trigger_action : delete_rule",
"referential_trigger_action : delete_rule update_rule",
"referential_trigger_action : update_rule delete_rule",
"update_rule : ON UPDATE referential_action",
"delete_rule : ON DELETE referential_action",
"referential_action : CASCADE",
"referential_action : SET DEFAULT",
"referential_action : SET NULL",
"referential_action : NO ACTION",
"procedure_clause : psql_procedure_clause",
"procedure_clause : external_procedure_clause",
"psql_procedure_clause : procedure_clause_start sql_security_clause_opt AS local_declarations_opt full_proc_block",
"external_procedure_clause : procedure_clause_start external_clause external_body_clause_opt",
"$$77 :",
"$$78 :",
"$$79 :",
"procedure_clause_start : symbol_procedure_name $$77 $$78 input_parameters $$79 output_parameters",
"alter_procedure_clause : procedure_clause",
"replace_procedure_clause : procedure_clause",
"input_parameters :",
"input_parameters : '(' ')'",
"input_parameters : '(' $$22 input_proc_parameters ')'",
"output_parameters :",
"output_parameters : RETURNS '(' $$23 output_proc_parameters ')'",
"input_proc_parameters : $$21 input_proc_parameter",
"input_proc_parameters : input_proc_parameters ',' $$23 input_proc_parameter",
"input_proc_parameter : column_domain_or_non_array_type collate_clause default_par_opt",
"output_proc_parameters : output_proc_parameter",
"output_proc_parameters : output_proc_parameters ',' $$23 output_proc_parameter",
"output_proc_parameter : column_domain_or_non_array_type collate_clause",
"column_domain_or_non_array_type : symbol_column_name domain_or_non_array_type",
"default_par_opt :",
"default_par_opt : DEFAULT default_value",
"default_par_opt : '=' default_value",
"function_clause : psql_function_clause",
"function_clause : external_function_clause",
"psql_function_clause : function_clause_start sql_security_clause_opt AS local_declarations_opt full_proc_block",
"external_function_clause : function_clause_start external_clause external_body_clause_opt",
"$$80 :",
"function_clause_start : symbol_UDF_name $$80 $$19 input_parameters RETURNS domain_or_non_array_type collate_clause deterministic_opt",
"deterministic_opt :",
"deterministic_opt : NOT DETERMINISTIC",
"deterministic_opt : DETERMINISTIC",
"external_clause : EXTERNAL NAME utf_string ENGINE valid_symbol_name",
"external_clause : EXTERNAL ENGINE valid_symbol_name",
"external_body_clause_opt :",
"external_body_clause_opt : AS utf_string",
"alter_function_clause : function_clause",
"replace_function_clause : function_clause",
"package_clause : symbol_package_name sql_security_clause_opt AS BEGIN package_items_opt END",
"package_items_opt : package_items",
"package_items_opt :",
"package_items : package_item",
"package_items : package_items package_item",
"package_item : FUNCTION function_clause_start ';'",
"package_item : PROCEDURE procedure_clause_start ';'",
"alter_package_clause : package_clause",
"replace_package_clause : package_clause",
"package_body_clause : symbol_package_name AS BEGIN package_items package_body_items_opt END",
"package_body_clause : symbol_package_name AS BEGIN package_body_items_opt END",
"package_body_items_opt :",
"package_body_items_opt : package_body_items",
"package_body_items : package_body_item",
"package_body_items : package_body_items package_body_item",
"package_body_item : FUNCTION psql_function_clause",
"package_body_item : FUNCTION external_function_clause ';'",
"package_body_item : PROCEDURE psql_procedure_clause",
"package_body_item : PROCEDURE external_procedure_clause ';'",
"local_declarations_opt : local_forward_declarations_opt local_nonforward_declarations_opt",
"local_forward_declarations_opt :",
"local_forward_declarations_opt : local_forward_declarations",
"local_forward_declarations : local_forward_declaration",
"local_forward_declarations : local_forward_declarations local_forward_declaration",
"local_forward_declaration : local_declaration_subproc_start ';'",
"local_forward_declaration : local_declaration_subfunc_start ';'",
"local_nonforward_declarations_opt :",
"local_nonforward_declarations_opt : local_nonforward_declarations",
"local_nonforward_declarations : local_nonforward_declaration",
"local_nonforward_declarations : local_nonforward_declarations local_nonforward_declaration",
"local_nonforward_declaration : DECLARE var_decl_opt local_declaration_item ';'",
"local_nonforward_declaration : local_declaration_subproc_start AS local_declarations_opt full_proc_block",
"local_nonforward_declaration : local_declaration_subfunc_start AS local_declarations_opt full_proc_block",
"$$81 :",
"$$82 :",
"$$83 :",
"local_declaration_subproc_start : DECLARE PROCEDURE symbol_procedure_name $$81 $$82 input_parameters $$83 output_parameters",
"$$84 :",
"$$85 :",
"local_declaration_subfunc_start : DECLARE FUNCTION symbol_UDF_name $$84 $$85 input_parameters RETURNS domain_or_non_array_type collate_clause deterministic_opt",
"local_declaration_item : var_declaration_item",
"local_declaration_item : cursor_declaration_item",
"var_declaration_item : column_domain_or_non_array_type collate_clause default_par_opt",
"var_decl_opt :",
"var_decl_opt : VARIABLE",
"cursor_declaration_item : symbol_cursor_name scroll_opt CURSOR FOR '(' select ')'",
"scroll_opt :",
"scroll_opt : NO SCROLL",
"scroll_opt : SCROLL",
"proc_block : proc_statement",
"proc_block : full_proc_block",
"full_proc_block : BEGIN full_proc_block_body END",
"full_proc_block_body :",
"full_proc_block_body : proc_statements",
"full_proc_block_body : proc_statements excp_hndl_statements",
"proc_statements : proc_block",
"proc_statements : proc_statements proc_block",
"proc_statement : simple_proc_statement ';'",
"proc_statement : complex_proc_statement",
"simple_proc_statement : assignment_statement",
"simple_proc_statement : insert",
"simple_proc_statement : merge",
"simple_proc_statement : update",
"simple_proc_statement : update_or_insert",
"simple_proc_statement : delete",
"simple_proc_statement : singleton_select",
"simple_proc_statement : exec_procedure",
"simple_proc_statement : exec_sql",
"simple_proc_statement : exec_into",
"simple_proc_statement : exec_function",
"simple_proc_statement : excp_statement",
"simple_proc_statement : raise_statement",
"simple_proc_statement : post_event",
"simple_proc_statement : cursor_statement",
"simple_proc_statement : breakleave",
"simple_proc_statement : continue",
"simple_proc_statement : SUSPEND",
"simple_proc_statement : EXIT",
"simple_proc_statement : RETURN value",
"simple_proc_statement : mng_statement",
"assignment_statement : assignment",
"assignment_statement : ':' assignment",
"complex_proc_statement : in_autonomous_transaction",
"complex_proc_statement : if_then_else",
"complex_proc_statement : while",
"complex_proc_statement : for_select",
"complex_proc_statement : for_exec_into",
"in_autonomous_transaction : IN AUTONOMOUS TRANSACTION DO proc_block",
"excp_statement : EXCEPTION symbol_exception_name",
"excp_statement : EXCEPTION symbol_exception_name value",
"excp_statement : EXCEPTION symbol_exception_name USING '(' value_list ')'",
"raise_statement : EXCEPTION",
"$$86 :",
"$$87 :",
"for_select : label_def_opt FOR select $$86 $$87 for_select_into_cursor DO proc_block",
"for_select_into_cursor : into_variable_list cursor_def_opt",
"for_select_into_cursor : into_variable_list_opt cursor_def",
"into_variable_list_opt :",
"into_variable_list_opt : into_variable_list",
"into_variable_list : INTO variable_list",
"$$88 :",
"$$89 :",
"$$90 :",
"exec_sql : EXECUTE STATEMENT $$88 $$89 exec_stmt_inputs $$90 exec_stmt_options",
"exec_into : exec_sql INTO variable_list",
"for_exec_into : label_def_opt FOR exec_into DO proc_block",
"exec_stmt_inputs : value",
"$$91 :",
"exec_stmt_inputs : '(' value ')' '(' $$91 named_params_list ')'",
"exec_stmt_inputs : '(' value ')' '(' $$91 not_named_params_list ')'",
"named_params_list : $$89 named_param",
"named_params_list : named_params_list ',' $$90 named_param",
"named_param : symbol_variable_name BIND_PARAM value",
"named_param : EXCESS symbol_variable_name BIND_PARAM value",
"not_named_params_list : $$89 not_named_param",
"not_named_params_list : not_named_params_list ',' $$90 not_named_param",
"not_named_param : value",
"exec_stmt_options :",
"exec_stmt_options : $$89 exec_stmt_options_list",
"$$92 :",
"exec_stmt_options_list : exec_stmt_options_list $$92 exec_stmt_option",
"exec_stmt_options_list : $$89 exec_stmt_option",
"exec_stmt_option : ON EXTERNAL DATA SOURCE value",
"exec_stmt_option : ON EXTERNAL value",
"exec_stmt_option : AS USER value",
"exec_stmt_option : PASSWORD value",
"exec_stmt_option : ROLE value",
"exec_stmt_option : WITH AUTONOMOUS TRANSACTION",
"exec_stmt_option : WITH COMMON TRANSACTION",
"exec_stmt_option : WITH CALLER PRIVILEGES",
"if_then_else : IF '(' search_condition ')' THEN proc_block ELSE proc_block",
"if_then_else : IF '(' search_condition ')' THEN proc_block",
"post_event : POST_EVENT value event_argument_opt",
"event_argument_opt :",
"singleton_select : select INTO variable_list",
"variable : ':' symbol_variable_name",
"variable_list : variable",
"variable_list : column_name",
"variable_list : variable_list ',' column_name",
"variable_list : variable_list ',' variable",
"while : label_def_opt WHILE '(' search_condition ')' DO proc_block",
"label_def_opt :",
"label_def_opt : symbol_label_name ':'",
"breakleave : BREAK",
"breakleave : LEAVE label_use_opt",
"continue : CONTINUE label_use_opt",
"label_use_opt :",
"label_use_opt : symbol_label_name",
"cursor_def_opt :",
"cursor_def_opt : cursor_def",
"cursor_def : AS CURSOR symbol_cursor_name",
"excp_hndl_statements : excp_hndl_statement",
"excp_hndl_statements : excp_hndl_statements excp_hndl_statement",
"$$93 :",
"$$94 :",
"excp_hndl_statement : WHEN $$93 $$94 errors DO proc_block",
"$$95 :",
"errors : $$95 err",
"$$96 :",
"errors : errors ',' $$96 err",
"err : SQLCODE signed_short_integer",
"err : SQLSTATE STRING",
"err : GDSCODE symbol_gdscode_name",
"err : EXCEPTION symbol_exception_name",
"err : ANY",
"cursor_statement : open_cursor",
"cursor_statement : fetch_cursor",
"cursor_statement : close_cursor",
"open_cursor : OPEN symbol_cursor_name",
"close_cursor : CLOSE symbol_cursor_name",
"$$97 :",
"$$98 :",
"fetch_cursor : FETCH $$97 $$98 fetch_scroll FROM symbol_cursor_name into_variable_list_opt",
"fetch_cursor : FETCH symbol_cursor_name into_variable_list_opt",
"fetch_scroll : FIRST",
"fetch_scroll : LAST",
"fetch_scroll : PRIOR",
"fetch_scroll : NEXT",
"fetch_scroll : ABSOLUTE value",
"fetch_scroll : RELATIVE value",
"exec_procedure : EXECUTE PROCEDURE symbol_procedure_name proc_inputs proc_outputs_opt",
"exec_procedure : EXECUTE PROCEDURE symbol_package_name '.' symbol_procedure_name proc_inputs proc_outputs_opt",
"proc_inputs :",
"proc_inputs : value_list",
"proc_inputs : '(' value_list ')'",
"proc_outputs_opt :",
"proc_outputs_opt : RETURNING_VALUES variable_list",
"proc_outputs_opt : RETURNING_VALUES '(' variable_list ')'",
"$$99 :",
"$$100 :",
"$$101 :",
"exec_block : EXECUTE BLOCK $$99 $$100 block_input_params $$101 output_parameters AS local_declarations_opt full_proc_block",
"block_input_params :",
"block_input_params : '(' $$22 block_parameters ')'",
"block_parameters : $$21 block_parameter",
"block_parameters : block_parameters ',' $$23 block_parameter",
"block_parameter : column_domain_or_non_array_type collate_clause '=' parameter",
"view_clause : simple_table_name column_parens_opt AS select_expr check_opt",
"replace_view_clause : view_clause",
"alter_view_clause : view_clause",
"check_opt :",
"check_opt : WITH CHECK OPTION",
"trigger_clause : create_trigger_start trg_sql_security_clause AS local_declarations_opt full_proc_block",
"trigger_clause : create_trigger_start external_clause external_body_clause_opt",
"$$102 :",
"$$103 :",
"create_trigger_start : symbol_trigger_name $$102 $$103 create_trigger_common",
"$$104 :",
"create_trigger_common : trigger_active $$104 trigger_type trigger_position",
"create_trigger_common : FOR symbol_table_name trigger_active table_trigger_type trigger_position",
"replace_trigger_clause : trigger_clause",
"trigger_active : ACTIVE",
"trigger_active : INACTIVE",
"trigger_active :",
"trigger_type : table_trigger_type trigger_position ON symbol_table_name",
"trigger_type : ON trigger_db_type",
"trigger_type : trigger_type_prefix trigger_ddl_type",
"table_trigger_type : trigger_type_prefix trigger_type_suffix",
"trigger_db_type : CONNECT",
"trigger_db_type : DISCONNECT",
"trigger_db_type : TRANSACTION START",
"trigger_db_type : TRANSACTION COMMIT",
"trigger_db_type : TRANSACTION ROLLBACK",
"trigger_ddl_type : trigger_ddl_type_items",
"trigger_ddl_type : ANY DDL STATEMENT",
"trigger_ddl_type_items : CREATE TABLE",
"trigger_ddl_type_items : ALTER TABLE",
"trigger_ddl_type_items : DROP TABLE",
"trigger_ddl_type_items : CREATE PROCEDURE",
"trigger_ddl_type_items : ALTER PROCEDURE",
"trigger_ddl_type_items : DROP PROCEDURE",
"trigger_ddl_type_items : CREATE FUNCTION",
"trigger_ddl_type_items : ALTER FUNCTION",
"trigger_ddl_type_items : DROP FUNCTION",
"trigger_ddl_type_items : CREATE TRIGGER",
"trigger_ddl_type_items : ALTER TRIGGER",
"trigger_ddl_type_items : DROP TRIGGER",
"trigger_ddl_type_items : CREATE EXCEPTION",
"trigger_ddl_type_items : ALTER EXCEPTION",
"trigger_ddl_type_items : DROP EXCEPTION",
"trigger_ddl_type_items : CREATE VIEW",
"trigger_ddl_type_items : ALTER VIEW",
"trigger_ddl_type_items : DROP VIEW",
"trigger_ddl_type_items : CREATE DOMAIN",
"trigger_ddl_type_items : ALTER DOMAIN",
"trigger_ddl_type_items : DROP DOMAIN",
"trigger_ddl_type_items : CREATE ROLE",
"trigger_ddl_type_items : ALTER ROLE",
"trigger_ddl_type_items : DROP ROLE",
"trigger_ddl_type_items : CREATE SEQUENCE",
"trigger_ddl_type_items : ALTER SEQUENCE",
"trigger_ddl_type_items : DROP SEQUENCE",
"trigger_ddl_type_items : CREATE USER",
"trigger_ddl_type_items : ALTER USER",
"trigger_ddl_type_items : DROP USER",
"trigger_ddl_type_items : CREATE INDEX",
"trigger_ddl_type_items : ALTER INDEX",
"trigger_ddl_type_items : DROP INDEX",
"trigger_ddl_type_items : CREATE COLLATION",
"trigger_ddl_type_items : DROP COLLATION",
"trigger_ddl_type_items : ALTER CHARACTER SET",
"trigger_ddl_type_items : CREATE PACKAGE",
"trigger_ddl_type_items : ALTER PACKAGE",
"trigger_ddl_type_items : DROP PACKAGE",
"trigger_ddl_type_items : CREATE PACKAGE BODY",
"trigger_ddl_type_items : DROP PACKAGE BODY",
"trigger_ddl_type_items : CREATE MAPPING",
"trigger_ddl_type_items : ALTER MAPPING",
"trigger_ddl_type_items : DROP MAPPING",
"trigger_ddl_type_items : trigger_ddl_type OR trigger_ddl_type",
"trigger_type_prefix : BEFORE",
"trigger_type_prefix : AFTER",
"trigger_type_suffix : INSERT",
"trigger_type_suffix : UPDATE",
"trigger_type_suffix : DELETE",
"trigger_type_suffix : INSERT OR UPDATE",
"trigger_type_suffix : INSERT OR DELETE",
"trigger_type_suffix : UPDATE OR INSERT",
"trigger_type_suffix : UPDATE OR DELETE",
"trigger_type_suffix : DELETE OR INSERT",
"trigger_type_suffix : DELETE OR UPDATE",
"trigger_type_suffix : INSERT OR UPDATE OR DELETE",
"trigger_type_suffix : INSERT OR DELETE OR UPDATE",
"trigger_type_suffix : UPDATE OR INSERT OR DELETE",
"trigger_type_suffix : UPDATE OR DELETE OR INSERT",
"trigger_type_suffix : DELETE OR INSERT OR UPDATE",
"trigger_type_suffix : DELETE OR UPDATE OR INSERT",
"trigger_position :",
"trigger_position : POSITION nonneg_short_integer",
"alter : ALTER alter_clause",
"alter : set_generator_clause",
"alter_clause : EXCEPTION alter_exception_clause",
"$$105 :",
"alter_clause : TABLE simple_table_name $$105 $$61 alter_ops",
"alter_clause : VIEW alter_view_clause",
"alter_clause : TRIGGER alter_trigger_clause",
"alter_clause : PROCEDURE alter_procedure_clause",
"alter_clause : PACKAGE alter_package_clause",
"$$106 :",
"alter_clause : DATABASE $$106 $$52 alter_db",
"alter_clause : DOMAIN alter_domain",
"alter_clause : INDEX alter_index_clause",
"alter_clause : EXTERNAL FUNCTION alter_udf_clause",
"alter_clause : FUNCTION alter_function_clause",
"alter_clause : ROLE alter_role_clause",
"alter_clause : USER alter_user_clause",
"alter_clause : CURRENT USER alter_cur_user_clause",
"alter_clause : CHARACTER SET alter_charset_clause",
"alter_clause : GENERATOR alter_sequence_clause",
"alter_clause : SEQUENCE alter_sequence_clause",
"alter_clause : MAPPING $$28 alter_map_clause",
"alter_clause : GLOBAL MAPPING $$29 alter_map_clause",
"alter_clause : EXTERNAL CONNECTIONS POOL alter_eds_conn_pool_clause",
"$$107 :",
"$$108 :",
"alter_domain : keyword_or_column $$107 $$108 alter_domain_ops",
"alter_domain_ops : $$108 alter_domain_op",
"$$109 :",
"alter_domain_ops : alter_domain_ops $$109 alter_domain_op",
"alter_domain_op : SET domain_default",
"alter_domain_op : ADD CONSTRAINT check_constraint",
"alter_domain_op : ADD check_constraint",
"alter_domain_op : DROP DEFAULT",
"alter_domain_op : DROP CONSTRAINT",
"alter_domain_op : DROP NOT NULL",
"alter_domain_op : SET NOT NULL",
"alter_domain_op : TO symbol_column_name",
"alter_domain_op : TYPE non_array_type",
"alter_ops : $$61 alter_op",
"alter_ops : alter_ops ',' $$62 alter_op",
"alter_op : DROP symbol_column_name drop_behaviour",
"alter_op : DROP CONSTRAINT symbol_constraint_name",
"alter_op : ADD $$76 column_def",
"alter_op : ADD $$76 table_constraint_definition",
"alter_op : col_opt alter_column_name POSITION pos_short_integer",
"alter_op : col_opt alter_column_name TO symbol_column_name",
"alter_op : col_opt alter_column_name DROP NOT NULL",
"alter_op : col_opt alter_column_name SET NOT NULL",
"alter_op : col_opt symbol_column_name TYPE alter_data_type_or_domain",
"alter_op : col_opt symbol_column_name TYPE non_array_type def_computed",
"alter_op : col_opt symbol_column_name def_computed",
"alter_op : col_opt symbol_column_name SET domain_default",
"alter_op : col_opt symbol_column_name DROP DEFAULT",
"$$110 :",
"alter_op : col_opt symbol_column_name $$110 $$72 alter_identity_clause_spec",
"alter_op : col_opt symbol_column_name DROP IDENTITY",
"alter_op : ALTER SQL SECURITY DEFINER",
"alter_op : ALTER SQL SECURITY INVOKER",
"alter_op : DROP SQL SECURITY",
"alter_op : ENABLE PUBLICATION",
"alter_op : DISABLE PUBLICATION",
"alter_column_name : keyword_or_column",
"keyword_or_column : valid_symbol_name",
"keyword_or_column : ADMIN",
"keyword_or_column : COLUMN",
"keyword_or_column : EXTRACT",
"keyword_or_column : YEAR",
"keyword_or_column : MONTH",
"keyword_or_column : DAY",
"keyword_or_column : HOUR",
"keyword_or_column : MINUTE",
"keyword_or_column : SECOND",
"keyword_or_column : TIME",
"keyword_or_column : TIMESTAMP",
"keyword_or_column : CURRENT_DATE",
"keyword_or_column : CURRENT_TIME",
"keyword_or_column : CURRENT_TIMESTAMP",
"keyword_or_column : CURRENT_USER",
"keyword_or_column : CURRENT_ROLE",
"keyword_or_column : RECREATE",
"keyword_or_column : CURRENT_CONNECTION",
"keyword_or_column : CURRENT_TRANSACTION",
"keyword_or_column : BIGINT",
"keyword_or_column : CASE",
"keyword_or_column : RELEASE",
"keyword_or_column : ROW_COUNT",
"keyword_or_column : SAVEPOINT",
"keyword_or_column : OPEN",
"keyword_or_column : CLOSE",
"keyword_or_column : FETCH",
"keyword_or_column : ROWS",
"keyword_or_column : USING",
"keyword_or_column : CROSS",
"keyword_or_column : BIT_LENGTH",
"keyword_or_column : BOTH",
"keyword_or_column : CHAR_LENGTH",
"keyword_or_column : CHARACTER_LENGTH",
"keyword_or_column : COMMENT",
"keyword_or_column : LEADING",
"keyword_or_column : LOWER",
"keyword_or_column : OCTET_LENGTH",
"keyword_or_column : TRAILING",
"keyword_or_column : TRIM",
"keyword_or_column : CONNECT",
"keyword_or_column : DISCONNECT",
"keyword_or_column : GLOBAL",
"keyword_or_column : INSENSITIVE",
"keyword_or_column : RECURSIVE",
"keyword_or_column : SENSITIVE",
"keyword_or_column : START",
"keyword_or_column : SIMILAR",
"keyword_or_column : BOOLEAN",
"keyword_or_column : CORR",
"keyword_or_column : COVAR_POP",
"keyword_or_column : COVAR_SAMP",
"keyword_or_column : DELETING",
"keyword_or_column : DETERMINISTIC",
"keyword_or_column : FALSE",
"keyword_or_column : INSERTING",
"keyword_or_column : OFFSET",
"keyword_or_column : OVER",
"keyword_or_column : REGR_AVGX",
"keyword_or_column : REGR_AVGY",
"keyword_or_column : REGR_COUNT",
"keyword_or_column : REGR_INTERCEPT",
"keyword_or_column : REGR_R2",
"keyword_or_column : REGR_SLOPE",
"keyword_or_column : REGR_SXX",
"keyword_or_column : REGR_SXY",
"keyword_or_column : REGR_SYY",
"keyword_or_column : RETURN",
"keyword_or_column : ROW",
"keyword_or_column : SCROLL",
"keyword_or_column : SQLSTATE",
"keyword_or_column : STDDEV_SAMP",
"keyword_or_column : STDDEV_POP",
"keyword_or_column : TRUE",
"keyword_or_column : UNKNOWN",
"keyword_or_column : UPDATING",
"keyword_or_column : VAR_SAMP",
"keyword_or_column : VAR_POP",
"keyword_or_column : BINARY",
"keyword_or_column : DECFLOAT",
"keyword_or_column : INT128",
"keyword_or_column : LATERAL",
"keyword_or_column : LOCAL",
"keyword_or_column : LOCALTIME",
"keyword_or_column : LOCALTIMESTAMP",
"keyword_or_column : PUBLICATION",
"keyword_or_column : RESETTING",
"keyword_or_column : TIMEZONE_HOUR",
"keyword_or_column : TIMEZONE_MINUTE",
"keyword_or_column : UNBOUNDED",
"keyword_or_column : VARBINARY",
"keyword_or_column : WINDOW",
"keyword_or_column : WITHOUT",
"col_opt : ALTER",
"col_opt : ALTER COLUMN",
"alter_data_type_or_domain : non_array_type",
"alter_data_type_or_domain : symbol_column_name",
"$$111 :",
"alter_identity_clause_spec : $$72 alter_identity_clause_generation $$111 alter_identity_clause_options_opt",
"alter_identity_clause_spec : $$72 alter_identity_clause_options",
"alter_identity_clause_generation : SET GENERATED ALWAYS",
"alter_identity_clause_generation : SET GENERATED BY DEFAULT",
"alter_identity_clause_options_opt :",
"alter_identity_clause_options_opt : $$72 alter_identity_clause_options",
"alter_identity_clause_options : alter_identity_clause_options $$73 alter_identity_clause_option",
"alter_identity_clause_options : $$72 alter_identity_clause_option",
"alter_identity_clause_option : RESTART with_opt",
"alter_identity_clause_option : SET INCREMENT by_noise signed_long_integer",
"drop_behaviour :",
"drop_behaviour : RESTRICT",
"drop_behaviour : CASCADE",
"alter_index_clause : symbol_index_name ACTIVE",
"alter_index_clause : symbol_index_name INACTIVE",
"alter_udf_clause : symbol_UDF_name entry_op module_op",
"entry_op :",
"entry_op : ENTRY_POINT utf_string",
"module_op :",
"module_op : MODULE_NAME utf_string",
"alter_role_2X_compatibility : symbol_role_name alter_role_enable AUTO ADMIN MAPPING",
"alter_role_clause : role_clause",
"alter_role_clause : alter_role_2X_compatibility",
"alter_role_enable : SET",
"alter_role_enable : DROP",
"alter_db : $$52 db_alter_clause",
"alter_db : alter_db $$54 db_alter_clause",
"$$112 :",
"db_alter_clause : ADD $$112 db_file_list",
"db_alter_clause : ADD DIFFERENCE FILE utf_string",
"db_alter_clause : DROP DIFFERENCE FILE",
"db_alter_clause : BEGIN BACKUP",
"db_alter_clause : END BACKUP",
"db_alter_clause : SET DEFAULT CHARACTER SET symbol_character_set_name",
"$$113 :",
"db_alter_clause : ENCRYPT WITH valid_symbol_name $$113 crypt_key_clause",
"db_alter_clause : DECRYPT",
"db_alter_clause : SET LINGER TO long_integer",
"db_alter_clause : DROP LINGER",
"db_alter_clause : SET DEFAULT sql_security_clause",
"db_alter_clause : ENABLE PUBLICATION",
"db_alter_clause : DISABLE PUBLICATION",
"db_alter_clause : INCLUDE $$54 pub_table_filter TO PUBLICATION",
"db_alter_clause : EXCLUDE $$54 pub_table_filter FROM PUBLICATION",
"crypt_key_clause :",
"crypt_key_clause : KEY valid_symbol_name",
"pub_table_filter : ALL",
"pub_table_filter : TABLE $$54 pub_table_list",
"pub_table_list : $$52 pub_table_clause",
"pub_table_list : pub_table_list ',' $$53 pub_table_clause",
"pub_table_clause : symbol_table_name",
"alter_trigger_clause : symbol_trigger_name trigger_active trigger_type_opt trigger_position trg_sql_security_clause AS local_declarations_opt full_proc_block",
"alter_trigger_clause : symbol_trigger_name trigger_active trigger_type_opt trigger_position external_clause external_body_clause_opt",
"alter_trigger_clause : symbol_trigger_name trigger_active trigger_type_opt trigger_position trg_sql_security_clause",
"trigger_type_opt : trigger_type_prefix trigger_type_suffix",
"trigger_type_opt :",
"trg_sql_security_clause :",
"trg_sql_security_clause : SQL SECURITY DEFINER",
"trg_sql_security_clause : SQL SECURITY INVOKER",
"trg_sql_security_clause : DROP SQL SECURITY",
"drop : DROP drop_clause",
"drop_clause : EXCEPTION symbol_exception_name",
"drop_clause : INDEX symbol_index_name",
"drop_clause : PROCEDURE symbol_procedure_name",
"drop_clause : TABLE symbol_table_name",
"drop_clause : TRIGGER symbol_trigger_name",
"drop_clause : VIEW symbol_view_name",
"drop_clause : FILTER symbol_filter_name",
"drop_clause : DOMAIN symbol_domain_name",
"drop_clause : EXTERNAL FUNCTION symbol_UDF_name",
"drop_clause : FUNCTION symbol_UDF_name",
"drop_clause : SHADOW pos_short_integer opt_no_file_delete",
"drop_clause : ROLE symbol_role_name",
"drop_clause : GENERATOR symbol_generator_name",
"drop_clause : SEQUENCE symbol_generator_name",
"drop_clause : COLLATION symbol_collation_name",
"drop_clause : USER symbol_user_name USING PLUGIN valid_symbol_name",
"drop_clause : USER symbol_user_name",
"drop_clause : PACKAGE symbol_package_name",
"drop_clause : PACKAGE BODY symbol_package_name",
"drop_clause : MAPPING $$28 drop_map_clause",
"drop_clause : GLOBAL MAPPING $$29 drop_map_clause",
"opt_no_file_delete :",
"opt_no_file_delete : PRESERVE FILE",
"opt_no_file_delete : DELETE FILE",
"data_type : non_array_type",
"data_type : array_type",
"domain_or_non_array_type : domain_or_non_array_type_name",
"domain_or_non_array_type : domain_or_non_array_type_name NOT NULL",
"domain_or_non_array_type_name : non_array_type",
"domain_or_non_array_type_name : domain_type",
"domain_type : TYPE OF symbol_column_name",
"domain_type : TYPE OF COLUMN symbol_column_name '.' symbol_column_name",
"domain_type : symbol_column_name",
"non_array_type : simple_type",
"non_array_type : blob_type",
"array_type : non_charset_simple_type '[' array_spec ']'",
"array_type : character_type '[' array_spec ']' charset_clause",
"array_spec : array_range",
"array_spec : array_spec ',' array_range",
"array_range : signed_long_integer",
"array_range : signed_long_integer ':' signed_long_integer",
"simple_type : non_charset_simple_type",
"simple_type : character_type charset_clause",
"non_charset_simple_type : national_character_type",
"non_charset_simple_type : binary_character_type",
"non_charset_simple_type : numeric_type",
"non_charset_simple_type : float_type",
"non_charset_simple_type : decfloat_type",
"non_charset_simple_type : BIGINT",
"non_charset_simple_type : INT128",
"non_charset_simple_type : integer_keyword",
"non_charset_simple_type : SMALLINT",
"non_charset_simple_type : DATE",
"non_charset_simple_type : TIME without_time_zone_opt",
"non_charset_simple_type : TIME WITH TIME ZONE",
"non_charset_simple_type : TIMESTAMP without_time_zone_opt",
"non_charset_simple_type : TIMESTAMP WITH TIME ZONE",
"non_charset_simple_type : BOOLEAN",
"integer_keyword : INTEGER",
"integer_keyword : INT",
"without_time_zone_opt :",
"without_time_zone_opt : WITHOUT TIME ZONE",
"$$114 :",
"$$115 :",
"blob_type : BLOB $$114 $$115 blob_subtype blob_segsize charset_clause",
"blob_type : BLOB '(' unsigned_short_integer ')'",
"blob_type : BLOB '(' unsigned_short_integer ',' signed_short_integer ')'",
"blob_type : BLOB '(' ',' signed_short_integer ')'",
"blob_segsize :",
"blob_segsize : SEGMENT SIZE unsigned_short_integer",
"blob_subtype :",
"blob_subtype : SUB_TYPE signed_short_integer",
"blob_subtype : SUB_TYPE symbol_blob_subtype_name",
"charset_clause :",
"charset_clause : CHARACTER SET symbol_character_set_name",
"national_character_type : national_character_keyword '(' pos_short_integer ')'",
"national_character_type : national_character_keyword",
"national_character_type : national_character_keyword VARYING '(' pos_short_integer ')'",
"binary_character_type : binary_character_keyword '(' pos_short_integer ')'",
"binary_character_type : binary_character_keyword",
"binary_character_type : varbinary_character_keyword '(' pos_short_integer ')'",
"character_type : character_keyword '(' pos_short_integer ')'",
"character_type : character_keyword",
"character_type : varying_keyword '(' pos_short_integer ')'",
"varying_keyword : VARCHAR",
"varying_keyword : CHARACTER VARYING",
"varying_keyword : CHAR VARYING",
"character_keyword : CHARACTER",
"character_keyword : CHAR",
"national_character_keyword : NCHAR",
"national_character_keyword : NATIONAL CHARACTER",
"national_character_keyword : NATIONAL CHAR",
"binary_character_keyword : BINARY",
"varbinary_character_keyword : VARBINARY",
"varbinary_character_keyword : BINARY VARYING",
"decfloat_type : DECFLOAT precision_opt_nz",
"numeric_type : NUMERIC prec_scale",
"numeric_type : decimal_keyword prec_scale",
"prec_scale :",
"prec_scale : '(' signed_long_integer ')'",
"prec_scale : '(' signed_long_integer ',' signed_long_integer ')'",
"decimal_keyword : DECIMAL",
"decimal_keyword : DEC",
"float_type : FLOAT precision_opt_nz",
"float_type : LONG FLOAT precision_opt_nz",
"float_type : REAL",
"float_type : DOUBLE PRECISION",
"precision_opt_nz :",
"precision_opt_nz : '(' pos_short_integer ')'",
"savepoint : set_savepoint",
"savepoint : release_savepoint",
"savepoint : undo_savepoint",
"set_savepoint : SAVEPOINT symbol_savepoint_name",
"release_savepoint : RELEASE SAVEPOINT symbol_savepoint_name",
"release_savepoint : RELEASE SAVEPOINT symbol_savepoint_name ONLY",
"undo_savepoint : ROLLBACK optional_work TO optional_savepoint symbol_savepoint_name",
"optional_savepoint :",
"optional_savepoint : SAVEPOINT",
"commit : COMMIT optional_work optional_retain",
"rollback : ROLLBACK optional_work optional_retain",
"optional_work :",
"optional_work : WORK",
"optional_retain :",
"optional_retain : RETAIN opt_snapshot",
"opt_snapshot :",
"opt_snapshot : SNAPSHOT",
"$$116 :",
"$$117 :",
"set_transaction : SET TRANSACTION $$116 $$117 tran_option_list_opt",
"session_reset : ALTER SESSION RESET",
"set_role : SET ROLE valid_symbol_name",
"set_role : SET TRUSTED ROLE",
"set_debug_option : SET DEBUG OPTION valid_symbol_name '=' constant",
"set_decfloat_round : SET DECFLOAT ROUND valid_symbol_name",
"$$118 :",
"$$119 :",
"set_decfloat_traps : SET DECFLOAT TRAPS TO $$118 $$119 decfloat_traps_list_opt",
"set_bind : SET BIND OF set_bind_from TO set_bind_to",
"set_bind_from : bind_type",
"set_bind_from : TIME ZONE",
"bind_type : non_array_type",
"bind_type : varying_keyword",
"set_bind_to : bind_type",
"set_bind_to : LEGACY",
"set_bind_to : NATIVE",
"set_bind_to : EXTENDED",
"set_bind_to : EXTENDED TIME WITH TIME ZONE",
"set_bind_to : EXTENDED TIMESTAMP WITH TIME ZONE",
"decfloat_traps_list_opt :",
"decfloat_traps_list_opt : $$119 decfloat_traps_list",
"decfloat_traps_list : $$119 decfloat_trap",
"$$120 :",
"decfloat_traps_list : decfloat_traps_list ',' $$120 decfloat_trap",
"decfloat_trap : valid_symbol_name",
"set_optimize : SET OPTIMIZE optimize_mode",
"set_optimize : SET OPTIMIZE TO DEFAULT",
"session_statement : SET SESSION IDLE TIMEOUT long_integer timepart_sesion_idle_tout",
"session_statement : SET STATEMENT TIMEOUT long_integer timepart_ses_stmt_tout",
"timepart_sesion_idle_tout :",
"timepart_sesion_idle_tout : HOUR",
"timepart_sesion_idle_tout : MINUTE",
"timepart_sesion_idle_tout : SECOND",
"timepart_ses_stmt_tout :",
"timepart_ses_stmt_tout : HOUR",
"timepart_ses_stmt_tout : MINUTE",
"timepart_ses_stmt_tout : SECOND",
"timepart_ses_stmt_tout : MILLISECOND",
"set_time_zone : SET TIME ZONE set_time_zone_option",
"set_time_zone_option : sql_string",
"set_time_zone_option : LOCAL",
"tran_option_list_opt :",
"tran_option_list_opt : $$117 tran_option_list",
"tran_option_list : $$117 tran_option",
"$$121 :",
"tran_option_list : tran_option_list $$121 tran_option",
"tran_option : READ ONLY",
"tran_option : READ WRITE",
"tran_option : WAIT",
"tran_option : NO WAIT",
"tran_option : $$117 isolation_mode",
"tran_option : NO AUTO UNDO",
"tran_option : IGNORE LIMBO",
"tran_option : RESTART REQUESTS",
"tran_option : AUTO COMMIT",
"tran_option : LOCK TIMEOUT nonneg_short_integer",
"$$122 :",
"$$123 :",
"tran_option : RESERVING $$122 $$123 restr_list",
"isolation_mode : ISOLATION LEVEL $$123 iso_mode",
"isolation_mode : iso_mode",
"iso_mode : $$117 snap_shot",
"iso_mode : READ UNCOMMITTED version_mode",
"iso_mode : READ COMMITTED version_mode",
"snap_shot : SNAPSHOT",
"snap_shot : SNAPSHOT AT NUMBER snapshot_number",
"snap_shot : SNAPSHOT TABLE",
"snap_shot : SNAPSHOT TABLE STABILITY",
"snapshot_number : NUMBER32BIT",
"snapshot_number : NUMBER64BIT",
"version_mode :",
"version_mode : VERSION",
"version_mode : NO VERSION",
"version_mode : READ CONSISTENCY",
"lock_type :",
"lock_type : SHARED",
"lock_type : PROTECTED",
"lock_mode : READ",
"lock_mode : WRITE",
"restr_list : restr_option",
"restr_list : restr_list ',' restr_option",
"restr_option : table_list table_lock",
"table_lock :",
"table_lock : FOR lock_type lock_mode",
"table_list : symbol_table_name",
"table_list : table_list ',' symbol_table_name",
"set_statistics : SET STATISTICS INDEX symbol_index_name",
"comment : COMMENT ON ddl_type0 IS ddl_desc",
"comment : COMMENT ON ddl_type1 symbol_ddl_name IS ddl_desc",
"comment : COMMENT ON ddl_type2 symbol_ddl_name ddl_subname IS ddl_desc",
"comment : COMMENT ON ddl_type3 ddl_qualified_name ddl_subname IS ddl_desc",
"comment : COMMENT ON ddl_type4 ddl_qualified_name IS ddl_desc",
"comment : comment_on_user",
"comment : comment_on_mapping",
"$$124 :",
"$$125 :",
"comment_on_user : COMMENT ON USER symbol_user_name $$124 $$125 opt_use_plugin IS ddl_desc",
"opt_use_plugin :",
"opt_use_plugin : $$125 use_plugin",
"ddl_type0 : DATABASE",
"ddl_type1 : DOMAIN",
"ddl_type1 : TABLE",
"ddl_type1 : VIEW",
"ddl_type1 : TRIGGER",
"ddl_type1 : FILTER",
"ddl_type1 : EXCEPTION",
"ddl_type1 : GENERATOR",
"ddl_type1 : SEQUENCE",
"ddl_type1 : INDEX",
"ddl_type1 : ROLE",
"ddl_type1 : CHARACTER SET",
"ddl_type1 : COLLATION",
"ddl_type1 : PACKAGE",
"ddl_type2 : COLUMN",
"ddl_type3 : PARAMETER",
"ddl_type3 : PROCEDURE PARAMETER",
"ddl_type3 : FUNCTION PARAMETER",
"ddl_type4 : PROCEDURE",
"ddl_type4 : EXTERNAL FUNCTION",
"ddl_type4 : FUNCTION",
"ddl_subname : '.' symbol_ddl_name",
"ddl_qualified_name : symbol_ddl_name",
"ddl_qualified_name : symbol_ddl_name '.' symbol_ddl_name",
"ddl_desc : utf_string",
"ddl_desc : NULL",
"select : select_expr for_update_clause lock_clause optimize_clause",
"for_update_clause :",
"for_update_clause : FOR UPDATE for_update_list",
"for_update_list :",
"for_update_list : OF column_list",
"lock_clause :",
"lock_clause : WITH LOCK skip_locked_clause_opt",
"skip_locked_clause_opt :",
"skip_locked_clause_opt : SKIP LOCKED",
"optimize_clause : OPTIMIZE optimize_mode",
"optimize_clause :",
"optimize_mode : FOR FIRST ROWS",
"optimize_mode : FOR ALL ROWS",
"select_expr : with_clause select_expr_body order_clause_opt rows_clause",
"select_expr : with_clause select_expr_body order_clause_opt result_offset_clause fetch_first_clause",
"with_clause :",
"with_clause : WITH RECURSIVE with_list",
"with_clause : WITH with_list",
"with_list : with_item",
"with_list : with_list ',' with_item",
"with_item : symbol_table_alias_name derived_column_list AS '(' select_expr ')'",
"column_select : select_expr",
"column_singleton : column_select",
"select_expr_body : query_term",
"select_expr_body : select_expr_body UNION distinct_noise query_term",
"select_expr_body : select_expr_body UNION ALL query_term",
"query_term : query_primary",
"query_primary : query_spec",
"query_primary : '(' select_expr_body order_clause_opt result_offset_clause fetch_first_clause ')'",
"query_spec : SELECT limit_clause distinct_clause select_list from_clause where_clause group_clause having_clause named_windows_clause plan_clause",
"limit_clause :",
"limit_clause : first_clause skip_clause",
"limit_clause : first_clause",
"limit_clause : skip_clause",
"first_clause : FIRST long_integer",
"first_clause : FIRST '(' value ')'",
"first_clause : FIRST parameter",
"skip_clause : SKIP long_integer",
"skip_clause : SKIP '(' value ')'",
"skip_clause : SKIP parameter",
"distinct_clause : DISTINCT",
"distinct_clause : all_noise",
"select_list : select_items",
"select_list : '*'",
"select_items : select_item",
"select_items : select_items ',' select_item",
"select_item : value_opt_alias",
"select_item : symbol_table_alias_name '.' '*'",
"value_opt_alias : value",
"value_opt_alias : value as_noise symbol_item_alias_name",
"as_noise :",
"as_noise : AS",
"from_clause : FROM from_list",
"from_list : table_reference",
"from_list : from_list ',' table_reference",
"table_reference : joined_table",
"table_reference : table_primary",
"table_primary : table_proc",
"table_primary : derived_table",
"table_primary : lateral_derived_table",
"table_primary : parenthesized_joined_table",
"parenthesized_joined_table : '(' parenthesized_joined_table ')'",
"parenthesized_joined_table : '(' joined_table ')'",
"derived_table : '(' select_expr ')' correlation_name_opt derived_column_list",
"lateral_derived_table : LATERAL derived_table",
"correlation_name_opt :",
"correlation_name_opt : symbol_table_alias_name",
"correlation_name_opt : AS symbol_table_alias_name",
"derived_column_list :",
"derived_column_list : '(' alias_list ')'",
"alias_list : symbol_item_alias_name",
"alias_list : alias_list ',' symbol_item_alias_name",
"joined_table : cross_join",
"joined_table : natural_join",
"joined_table : qualified_join",
"cross_join : table_reference CROSS JOIN table_primary",
"natural_join : table_reference NATURAL join_type JOIN table_primary",
"qualified_join : table_reference join_type JOIN table_reference join_condition",
"qualified_join : table_reference join_type JOIN table_reference named_columns_join",
"join_condition : ON search_condition",
"named_columns_join : USING '(' column_list ')'",
"table_proc : symbol_procedure_name table_proc_inputs as_noise symbol_table_alias_name",
"table_proc : symbol_procedure_name table_proc_inputs",
"table_proc : symbol_package_name '.' symbol_procedure_name table_proc_inputs as_noise symbol_table_alias_name",
"table_proc : symbol_package_name '.' symbol_procedure_name table_proc_inputs",
"table_proc_inputs :",
"table_proc_inputs : '(' value_list ')'",
"table_name : simple_table_name",
"table_name : symbol_table_name as_noise symbol_table_alias_name",
"simple_table_name : symbol_table_name",
"join_type :",
"join_type : INNER",
"join_type : LEFT outer_noise",
"join_type : RIGHT outer_noise",
"join_type : FULL outer_noise",
"outer_noise :",
"outer_noise : OUTER",
"group_clause :",
"group_clause : GROUP BY group_by_list",
"group_by_list : group_by_item",
"group_by_list : group_by_list ',' group_by_item",
"group_by_item : value",
"having_clause :",
"having_clause : HAVING search_condition",
"where_clause :",
"where_clause : WHERE search_condition",
"named_windows_clause :",
"named_windows_clause : WINDOW window_definition_list",
"window_definition_list : window_definition",
"window_definition_list : window_definition_list ',' window_definition",
"window_definition : symbol_window_name AS '(' window_clause ')'",
"symbol_window_name_opt :",
"symbol_window_name_opt : symbol_window_name",
"plan_clause :",
"plan_clause : PLAN plan_expression",
"$$126 :",
"$$127 :",
"plan_expression : plan_type $$126 '(' $$127 plan_item_list ')'",
"plan_type :",
"plan_type : JOIN",
"plan_type : SORT MERGE",
"plan_type : MERGE",
"plan_type : HASH",
"plan_type : SORT",
"plan_item_list : plan_item",
"plan_item_list : plan_item_list ',' plan_item",
"plan_item : table_or_alias_list access_type",
"plan_item : plan_expression",
"table_or_alias_list : symbol_table_name",
"table_or_alias_list : table_or_alias_list symbol_table_name",
"access_type : NATURAL",
"$$128 :",
"$$129 :",
"access_type : INDEX $$128 '(' $$129 index_list ')'",
"$$130 :",
"access_type : ORDER $$130 symbol_index_name $$129 extra_indices_opt",
"index_list : symbol_index_name",
"index_list : index_list ',' symbol_index_name",
"extra_indices_opt :",
"$$131 :",
"extra_indices_opt : INDEX '(' $$131 index_list ')'",
"order_clause_opt :",
"order_clause_opt : order_clause",
"order_clause : ORDER BY order_list",
"order_list : order_item",
"order_list : order_list ',' order_item",
"order_item : value order_direction nulls_clause",
"order_direction :",
"order_direction : ASC",
"order_direction : DESC",
"nulls_clause :",
"nulls_clause : NULLS nulls_placement",
"nulls_placement : FIRST",
"nulls_placement : LAST",
"rows_clause : ROWS value",
"rows_clause : ROWS value TO value",
"rows_clause_optional :",
"rows_clause_optional : rows_clause",
"row_noise : ROW",
"row_noise : ROWS",
"result_offset_clause :",
"result_offset_clause : OFFSET simple_value_spec row_noise",
"first_next_noise : FIRST",
"first_next_noise : NEXT",
"fetch_first_clause :",
"fetch_first_clause : FETCH first_next_noise simple_value_spec row_noise ONLY",
"fetch_first_clause : FETCH first_next_noise row_noise ONLY",
"$$132 :",
"insert : insert_start $$132 ins_column_parens_opt override_opt VALUES '(' value_or_default_list ')' returning_clause",
"insert : insert_start $$132 ins_column_parens_opt override_opt select_expr returning_clause",
"insert : insert_start DEFAULT VALUES returning_clause",
"insert_start : INSERT INTO simple_table_name",
"override_opt :",
"override_opt : OVERRIDING USER VALUE",
"override_opt : OVERRIDING SYSTEM VALUE",
"value_or_default_list : value_or_default",
"value_or_default_list : value_or_default_list ',' value_or_default",
"value_or_default : value",
"value_or_default : DEFAULT",
"$$133 :",
"$$134 :",
"merge : MERGE INTO table_name USING table_reference ON search_condition $$133 $$134 merge_when_clause plan_clause order_clause_opt returning_clause",
"merge_when_clause : $$134 merge_when_matched_clause",
"merge_when_clause : $$134 merge_when_not_matched_clause",
"$$135 :",
"merge_when_clause : merge_when_clause $$135 merge_when_matched_clause",
"merge_when_clause : merge_when_clause $$135 merge_when_not_matched_clause",
"$$136 :",
"$$137 :",
"$$138 :",
"merge_when_matched_clause : WHEN MATCHED $$136 $$137 $$138 merge_update_specification",
"$$139 :",
"$$140 :",
"merge_when_not_matched_clause : WHEN NOT MATCHED by_target_noise $$139 $$140 merge_insert_specification",
"$$141 :",
"$$142 :",
"merge_when_not_matched_clause : WHEN NOT MATCHED BY SOURCE $$141 $$137 $$142 merge_update_specification",
"by_target_noise :",
"by_target_noise : BY TARGET",
"$$143 :",
"merge_update_specification : THEN UPDATE SET $$143 update_assignments",
"$$144 :",
"merge_update_specification : AND search_condition THEN UPDATE SET $$144 update_assignments",
"merge_update_specification : THEN DELETE",
"merge_update_specification : AND search_condition THEN DELETE",
"$$145 :",
"merge_insert_specification : THEN INSERT $$145 ins_column_parens_opt override_opt VALUES '(' value_or_default_list ')'",
"$$146 :",
"merge_insert_specification : AND search_condition THEN INSERT $$146 ins_column_parens_opt override_opt VALUES '(' value_or_default_list ')'",
"delete : delete_searched",
"delete : delete_positioned",
"delete_searched : DELETE FROM table_name where_clause plan_clause order_clause_opt rows_clause_optional skip_locked_clause_opt returning_clause",
"delete_positioned : DELETE FROM table_name cursor_clause returning_clause",
"update : update_searched",
"update : update_positioned",
"$$147 :",
"update_searched : UPDATE table_name SET $$147 update_assignments where_clause plan_clause order_clause_opt rows_clause_optional skip_locked_clause_opt returning_clause",
"update_positioned : UPDATE table_name SET $$147 update_assignments cursor_clause returning_clause",
"$$148 :",
"$$149 :",
"$$150 :",
"update_or_insert : UPDATE OR INSERT INTO simple_table_name $$148 $$149 ins_column_parens_opt override_opt VALUES '(' value_or_default_list ')' $$150 update_or_insert_matching_opt plan_clause order_clause_opt rows_clause_optional returning_clause",
"update_or_insert_matching_opt :",
"$$151 :",
"update_or_insert_matching_opt : MATCHING $$151 ins_column_parens",
"returning_clause :",
"returning_clause : RETURNING select_list",
"returning_clause : RETURNING select_list INTO variable_list",
"cursor_clause : WHERE CURRENT OF symbol_cursor_name",
"assignment : update_column_name '=' value",
"$$152 :",
"update_assignments : $$152 update_assignment",
"$$153 :",
"update_assignments : update_assignments ',' $$153 update_assignment",
"update_assignment : update_column_name '=' value",
"update_assignment : update_column_name '=' DEFAULT",
"exec_function : udf",
"exec_function : non_aggregate_function",
"column_parens_opt :",
"column_parens_opt : column_parens",
"column_parens : '(' column_list ')'",
"column_list : simple_column_name",
"column_list : column_list ',' simple_column_name",
"ins_column_parens_opt :",
"$$154 :",
"ins_column_parens_opt : $$154 ins_column_parens",
"ins_column_parens : '(' $$151 ins_column_list ')'",
"ins_column_list : update_column_name",
"ins_column_list : ins_column_list ',' update_column_name",
"column_name : simple_column_name",
"column_name : symbol_table_alias_name '.' symbol_column_name",
"column_name : ':' symbol_table_alias_name '.' symbol_column_name",
"simple_column_name : symbol_column_name",
"update_column_name : simple_column_name",
"update_column_name : symbol_table_alias_name '.' symbol_column_name",
"search_condition : value",
"boolean_value_expression : predicate",
"boolean_value_expression : value OR value",
"boolean_value_expression : value AND value",
"boolean_value_expression : NOT value",
"boolean_value_expression : '(' boolean_value_expression ')'",
"boolean_value_expression : value IS boolean_literal",
"boolean_value_expression : value IS NOT boolean_literal",
"predicate : comparison_predicate",
"predicate : distinct_predicate",
"predicate : between_predicate",
"predicate : binary_pattern_predicate",
"predicate : ternary_pattern_predicate",
"predicate : in_predicate",
"predicate : null_predicate",
"predicate : quantified_predicate",
"predicate : exists_predicate",
"predicate : singular_predicate",
"predicate : trigger_action_predicate",
"predicate : session_reset_predicate",
"comparison_predicate : value comparison_operator value",
"comparison_operator : '='",
"comparison_operator : '<'",
"comparison_operator : '>'",
"comparison_operator : GEQ",
"comparison_operator : LEQ",
"comparison_operator : NOT_GTR",
"comparison_operator : NOT_LSS",
"comparison_operator : NEQ",
"quantified_predicate : value comparison_operator quantified_flag '(' column_select ')'",
"quantified_flag : ALL",
"quantified_flag : SOME",
"quantified_flag : ANY",
"distinct_predicate : value IS DISTINCT FROM value",
"distinct_predicate : value IS NOT DISTINCT FROM value",
"between_predicate : value BETWEEN value_special AND value_special",
"between_predicate : value NOT BETWEEN value_special AND value_special",
"binary_pattern_predicate : value binary_pattern_operator value",
"binary_pattern_predicate : value NOT binary_pattern_operator value",
"binary_pattern_operator : CONTAINING",
"binary_pattern_operator : STARTING",
"binary_pattern_operator : STARTING WITH",
"ternary_pattern_predicate : value LIKE value",
"ternary_pattern_predicate : value LIKE value ESCAPE value",
"ternary_pattern_predicate : value NOT LIKE value",
"ternary_pattern_predicate : value NOT LIKE value ESCAPE value",
"ternary_pattern_predicate : value SIMILAR TO value",
"ternary_pattern_predicate : value SIMILAR TO value ESCAPE value",
"ternary_pattern_predicate : value NOT SIMILAR TO value",
"ternary_pattern_predicate : value NOT SIMILAR TO value ESCAPE value",
"in_predicate : value IN in_predicate_value",
"in_predicate : value NOT IN in_predicate_value",
"exists_predicate : EXISTS '(' select_expr ')'",
"singular_predicate : SINGULAR '(' select_expr ')'",
"trigger_action_predicate : INSERTING",
"trigger_action_predicate : UPDATING",
"trigger_action_predicate : DELETING",
"session_reset_predicate : RESETTING",
"null_predicate : value IS NULL",
"null_predicate : value IS UNKNOWN",
"null_predicate : value IS NOT NULL",
"null_predicate : value IS NOT UNKNOWN",
"in_predicate_value : table_subquery",
"in_predicate_value : '(' value_list ')'",
"table_subquery : '(' column_select ')'",
"$$155 :",
"create_user_clause : symbol_user_name $$155 $$125 user_fixed_list_opt",
"$$156 :",
"alter_user_clause : symbol_user_name set_noise $$156 $$125 user_fixed_list_opt",
"$$157 :",
"alter_cur_user_clause : set_noise $$157 $$125 user_fixed_list_opt",
"$$158 :",
"replace_user_clause : symbol_user_name set_noise $$158 $$125 user_fixed_list_opt",
"set_noise :",
"set_noise : SET",
"user_fixed_list_opt :",
"user_fixed_list_opt : $$125 user_fixed_list",
"user_fixed_list : $$125 user_fixed_option",
"$$159 :",
"user_fixed_list : user_fixed_list $$159 user_fixed_option",
"user_fixed_option : FIRSTNAME utf_string",
"user_fixed_option : MIDDLENAME utf_string",
"user_fixed_option : LASTNAME utf_string",
"user_fixed_option : PASSWORD utf_string",
"user_fixed_option : GRANT ADMIN ROLE",
"user_fixed_option : REVOKE ADMIN ROLE",
"user_fixed_option : ACTIVE",
"user_fixed_option : INACTIVE",
"user_fixed_option : $$125 use_plugin",
"$$160 :",
"user_fixed_option : TAGS '(' $$160 user_var_list ')'",
"use_plugin : USING PLUGIN valid_symbol_name",
"user_var_list : $$125 user_var_option",
"user_var_list : user_var_list ',' $$160 user_var_option",
"user_var_option : valid_symbol_name '=' utf_string",
"user_var_option : DROP valid_symbol_name",
"$$161 :",
"$$162 :",
"$$163 :",
"create_map_clause : $$161 map_clause $$162 $$163 map_to",
"$$164 :",
"$$165 :",
"alter_map_clause : $$164 map_clause $$165 $$163 map_to",
"$$166 :",
"$$167 :",
"replace_map_clause : $$166 map_clause $$167 $$163 map_to",
"drop_map_clause : map_name",
"comment_on_mapping : COMMENT ON MAPPING $$28 map_comment",
"comment_on_mapping : COMMENT ON GLOBAL MAPPING $$29 map_comment",
"map_comment : map_name IS ddl_desc",
"$$168 :",
"$$169 :",
"$$170 :",
"map_clause : map_name $$168 USING $$169 map_using FROM $$170 map_from",
"map_name : valid_symbol_name",
"map_from : map_from_symbol_name map_logoninfo",
"map_from : ANY map_from_symbol_name",
"map_from_symbol_name : valid_symbol_name",
"map_from_symbol_name : USER",
"map_from_symbol_name : GROUP",
"map_logoninfo : sql_string",
"map_logoninfo : valid_symbol_name",
"map_using : PLUGIN valid_symbol_name map_in",
"map_using : ANY PLUGIN map_in",
"map_using : ANY PLUGIN SERVERWIDE",
"map_using : MAPPING map_in",
"map_using : '*' map_in",
"map_in :",
"map_in : IN valid_symbol_name",
"map_to : TO map_role valid_symbol_name",
"map_to : TO map_role",
"map_role : ROLE",
"map_role : USER",
"value : value_primary",
"value : boolean_value_expression",
"value_special : value_primary",
"value_special : '(' boolean_value_expression ')'",
"value_primary : nonparenthesized_value",
"value_primary : '(' value_primary ')'",
"simple_value_spec : constant",
"simple_value_spec : variable",
"simple_value_spec : parameter",
"nonparenthesized_value : column_name",
"nonparenthesized_value : array_element",
"nonparenthesized_value : function",
"nonparenthesized_value : u_constant",
"nonparenthesized_value : boolean_literal",
"nonparenthesized_value : parameter",
"nonparenthesized_value : variable",
"nonparenthesized_value : cast_specification",
"nonparenthesized_value : case_expression",
"nonparenthesized_value : next_value_expression",
"nonparenthesized_value : udf",
"nonparenthesized_value : '-' value_special",
"nonparenthesized_value : '+' value_special",
"nonparenthesized_value : value_special '+' value_special",
"nonparenthesized_value : value_special CONCATENATE value_special",
"nonparenthesized_value : value_special COLLATE symbol_collation_name",
"nonparenthesized_value : value_special AT LOCAL",
"nonparenthesized_value : value_special AT TIME ZONE value_special",
"nonparenthesized_value : value_special '-' value_special",
"nonparenthesized_value : value_special '*' value_special",
"nonparenthesized_value : value_special '/' value_special",
"nonparenthesized_value : '(' column_singleton ')'",
"nonparenthesized_value : current_user",
"nonparenthesized_value : current_role",
"nonparenthesized_value : internal_info",
"nonparenthesized_value : recordKeyType",
"nonparenthesized_value : symbol_table_alias_name '.' recordKeyType",
"nonparenthesized_value : VALUE",
"nonparenthesized_value : datetime_value_expression",
"nonparenthesized_value : null_value",
"recordKeyType : DB_KEY",
"recordKeyType : RDB_RECORD_VERSION",
"datetime_value_expression : CURRENT_DATE",
"datetime_value_expression : LOCALTIME time_precision_opt",
"datetime_value_expression : CURRENT_TIME time_precision_opt",
"datetime_value_expression : LOCALTIMESTAMP timestamp_precision_opt",
"datetime_value_expression : CURRENT_TIMESTAMP timestamp_precision_opt",
"time_precision_opt :",
"time_precision_opt : '(' nonneg_short_integer ')'",
"timestamp_precision_opt :",
"timestamp_precision_opt : '(' nonneg_short_integer ')'",
"array_element : column_name '[' value_list ']'",
"value_list_opt :",
"value_list_opt : value_list",
"value_list : value",
"value_list : value_list ',' value",
"constant : u_constant",
"constant : '-' ul_numeric_constant",
"constant : '-' LIMIT64_INT",
"constant : '-' LIMIT64_NUMBER",
"constant : '-' u_constant_128",
"constant : boolean_literal",
"u_numeric_constant : ul_numeric_constant",
"u_numeric_constant : LIMIT64_NUMBER",
"u_numeric_constant : LIMIT64_INT",
"u_numeric_constant : u_constant_128",
"u_constant_128 : NUM128",
"ul_numeric_constant : NUMBER32BIT",
"ul_numeric_constant : FLOAT_NUMBER",
"ul_numeric_constant : DECIMAL_NUMBER",
"ul_numeric_constant : NUMBER64BIT",
"ul_numeric_constant : SCALEDINT",
"u_constant : u_numeric_constant",
"u_constant : sql_string",
"u_constant : DATE STRING",
"u_constant : TIME STRING",
"u_constant : TIMESTAMP STRING",
"boolean_literal : FALSE",
"boolean_literal : TRUE",
"parameter : '?'",
"current_user : USER",
"current_user : CURRENT_USER",
"current_role : CURRENT_ROLE",
"internal_info : CURRENT_CONNECTION",
"internal_info : CURRENT_TRANSACTION",
"internal_info : GDSCODE",
"internal_info : SQLCODE",
"internal_info : SQLSTATE",
"internal_info : ROW_COUNT",
"internal_info : RDB_ERROR '(' error_context ')'",
"error_context : GDSCODE",
"error_context : SQLCODE",
"error_context : SQLSTATE",
"error_context : EXCEPTION",
"error_context : MESSAGE",
"sql_string : STRING",
"sql_string : INTRODUCER STRING",
"utf_string : sql_string",
"signed_short_integer : nonneg_short_integer",
"signed_short_integer : '-' neg_short_integer",
"nonneg_short_integer : NUMBER32BIT",
"neg_short_integer : NUMBER32BIT",
"pos_short_integer : nonneg_short_integer",
"unsigned_short_integer : NUMBER32BIT",
"signed_long_integer : long_integer",
"signed_long_integer : '-' long_integer",
"long_integer : NUMBER32BIT",
"function : aggregate_function",
"function : non_aggregate_function",
"function : over_clause",
"non_aggregate_function : numeric_value_function",
"non_aggregate_function : string_value_function",
"non_aggregate_function : system_function_expression",
"aggregate_function : aggregate_function_prefix",
"aggregate_function : aggregate_function_prefix FILTER '(' WHERE search_condition ')'",
"aggregate_function_prefix : COUNT '(' '*' ')'",
"aggregate_function_prefix : COUNT '(' all_noise value ')'",
"aggregate_function_prefix : COUNT '(' DISTINCT value ')'",
"aggregate_function_prefix : SUM '(' all_noise value ')'",
"aggregate_function_prefix : SUM '(' DISTINCT value ')'",
"aggregate_function_prefix : AVG '(' all_noise value ')'",
"aggregate_function_prefix : AVG '(' DISTINCT value ')'",
"aggregate_function_prefix : MINIMUM '(' all_noise value ')'",
"aggregate_function_prefix : MINIMUM '(' DISTINCT value ')'",
"aggregate_function_prefix : MAXIMUM '(' all_noise value ')'",
"aggregate_function_prefix : MAXIMUM '(' DISTINCT value ')'",
"aggregate_function_prefix : LIST '(' all_noise value delimiter_opt ')'",
"aggregate_function_prefix : LIST '(' DISTINCT value delimiter_opt ')'",
"aggregate_function_prefix : STDDEV_SAMP '(' value ')'",
"aggregate_function_prefix : STDDEV_POP '(' value ')'",
"aggregate_function_prefix : VAR_SAMP '(' value ')'",
"aggregate_function_prefix : VAR_POP '(' value ')'",
"aggregate_function_prefix : COVAR_SAMP '(' value ',' value ')'",
"aggregate_function_prefix : COVAR_POP '(' value ',' value ')'",
"aggregate_function_prefix : CORR '(' value ',' value ')'",
"aggregate_function_prefix : REGR_AVGX '(' value ',' value ')'",
"aggregate_function_prefix : REGR_AVGY '(' value ',' value ')'",
"aggregate_function_prefix : REGR_COUNT '(' value ',' value ')'",
"aggregate_function_prefix : REGR_INTERCEPT '(' value ',' value ')'",
"aggregate_function_prefix : REGR_R2 '(' value ',' value ')'",
"aggregate_function_prefix : REGR_SLOPE '(' value ',' value ')'",
"aggregate_function_prefix : REGR_SXX '(' value ',' value ')'",
"aggregate_function_prefix : REGR_SXY '(' value ',' value ')'",
"aggregate_function_prefix : REGR_SYY '(' value ',' value ')'",
"window_function : DENSE_RANK '(' ')'",
"window_function : RANK '(' ')'",
"window_function : PERCENT_RANK '(' ')'",
"window_function : CUME_DIST '(' ')'",
"window_function : ROW_NUMBER '(' ')'",
"window_function : FIRST_VALUE '(' value ')'",
"window_function : LAST_VALUE '(' value ')'",
"window_function : NTH_VALUE '(' value ',' value ')' nth_from",
"window_function : LAG '(' value ',' value ',' value ')'",
"window_function : LAG '(' value ',' value ')'",
"window_function : LAG '(' value ')'",
"window_function : LEAD '(' value ',' value ',' value ')'",
"window_function : LEAD '(' value ',' value ')'",
"window_function : LEAD '(' value ')'",
"window_function : NTILE '(' ntile_arg ')'",
"nth_from :",
"nth_from : FROM FIRST",
"nth_from : FROM LAST",
"ntile_arg : u_numeric_constant",
"ntile_arg : variable",
"ntile_arg : parameter",
"aggregate_window_function : aggregate_function",
"aggregate_window_function : window_function",
"over_clause : aggregate_window_function OVER symbol_window_name",
"over_clause : aggregate_window_function OVER '(' window_clause ')'",
"window_clause : symbol_window_name_opt window_partition_opt order_clause_opt window_frame_extent window_frame_exclusion_opt",
"window_partition_opt :",
"window_partition_opt : PARTITION BY value_list",
"window_frame_extent :",
"$$171 :",
"$$172 :",
"window_frame_extent : RANGE $$171 $$172 window_frame",
"$$173 :",
"window_frame_extent : ROWS $$173 $$172 window_frame",
"window_frame : window_frame_start",
"window_frame : BETWEEN window_frame_between_bound1 AND window_frame_between_bound2",
"window_frame_start : UNBOUNDED PRECEDING",
"window_frame_start : CURRENT ROW",
"window_frame_start : value PRECEDING",
"window_frame_between_bound1 : UNBOUNDED PRECEDING",
"window_frame_between_bound1 : CURRENT ROW",
"window_frame_between_bound1 : value PRECEDING",
"window_frame_between_bound1 : value FOLLOWING",
"window_frame_between_bound2 : UNBOUNDED FOLLOWING",
"window_frame_between_bound2 : CURRENT ROW",
"window_frame_between_bound2 : value PRECEDING",
"window_frame_between_bound2 : value FOLLOWING",
"window_frame_exclusion_opt :",
"window_frame_exclusion_opt : EXCLUDE NO OTHERS",
"window_frame_exclusion_opt : EXCLUDE CURRENT ROW",
"window_frame_exclusion_opt : EXCLUDE GROUP",
"window_frame_exclusion_opt : EXCLUDE TIES",
"delimiter_opt :",
"delimiter_opt : ',' value",
"numeric_value_function : extract_expression",
"numeric_value_function : length_expression",
"extract_expression : EXTRACT '(' timestamp_part FROM value ')'",
"length_expression : bit_length_expression",
"length_expression : char_length_expression",
"length_expression : octet_length_expression",
"bit_length_expression : BIT_LENGTH '(' value ')'",
"char_length_expression : CHAR_LENGTH '(' value ')'",
"char_length_expression : CHARACTER_LENGTH '(' value ')'",
"octet_length_expression : OCTET_LENGTH '(' value ')'",
"system_function_expression : system_function_std_syntax '(' value_list_opt ')'",
"system_function_expression : system_function_special_syntax",
"system_function_std_syntax : ABS",
"system_function_std_syntax : ACOS",
"system_function_std_syntax : ACOSH",
"system_function_std_syntax : ASCII_CHAR",
"system_function_std_syntax : ASCII_VAL",
"system_function_std_syntax : ASIN",
"system_function_std_syntax : ASINH",
"system_function_std_syntax : ATAN",
"system_function_std_syntax : ATAN2",
"system_function_std_syntax : ATANH",
"system_function_std_syntax : BASE64_DECODE",
"system_function_std_syntax : BASE64_ENCODE",
"system_function_std_syntax : BIN_AND",
"system_function_std_syntax : BIN_NOT",
"system_function_std_syntax : BIN_OR",
"system_function_std_syntax : BIN_SHL",
"system_function_std_syntax : BIN_SHR",
"system_function_std_syntax : BIN_XOR",
"system_function_std_syntax : BLOB_APPEND",
"system_function_std_syntax : CEIL",
"system_function_std_syntax : CHAR_TO_UUID",
"system_function_std_syntax : COS",
"system_function_std_syntax : COSH",
"system_function_std_syntax : COT",
"system_function_std_syntax : EXP",
"system_function_std_syntax : FLOOR",
"system_function_std_syntax : GEN_UUID",
"system_function_std_syntax : HEX_DECODE",
"system_function_std_syntax : HEX_ENCODE",
"system_function_std_syntax : LEFT",
"system_function_std_syntax : LN",
"system_function_std_syntax : LOG",
"system_function_std_syntax : LOG10",
"system_function_std_syntax : LPAD",
"system_function_std_syntax : MAKE_DBKEY",
"system_function_std_syntax : MAXVALUE",
"system_function_std_syntax : MINVALUE",
"system_function_std_syntax : MOD",
"system_function_std_syntax : PI",
"system_function_std_syntax : POWER",
"system_function_std_syntax : RAND",
"system_function_std_syntax : RDB_GET_CONTEXT",
"system_function_std_syntax : RDB_GET_TRANSACTION_CN",
"system_function_std_syntax : RDB_ROLE_IN_USE",
"system_function_std_syntax : RDB_SET_CONTEXT",
"system_function_std_syntax : REPLACE",
"system_function_std_syntax : REVERSE",
"system_function_std_syntax : RIGHT",
"system_function_std_syntax : ROUND",
"system_function_std_syntax : RPAD",
"system_function_std_syntax : RSA_PRIVATE",
"system_function_std_syntax : RSA_PUBLIC",
"system_function_std_syntax : SIGN",
"system_function_std_syntax : SIN",
"system_function_std_syntax : SINH",
"system_function_std_syntax : SQRT",
"system_function_std_syntax : TAN",
"system_function_std_syntax : TANH",
"system_function_std_syntax : TRUNC",
"system_function_std_syntax : UNICODE_CHAR",
"system_function_std_syntax : UNICODE_VAL",
"system_function_std_syntax : UUID_TO_CHAR",
"system_function_std_syntax : QUANTIZE",
"system_function_std_syntax : TOTALORDER",
"system_function_std_syntax : NORMALIZE_DECFLOAT",
"system_function_std_syntax : COMPARE_DECFLOAT",
"system_function_special_syntax : DATEADD '(' value timestamp_part TO value ')'",
"system_function_special_syntax : DATEADD '(' timestamp_part ',' value ',' value ')'",
"system_function_special_syntax : DATEDIFF '(' timestamp_part FROM value TO value ')'",
"system_function_special_syntax : DATEDIFF '(' timestamp_part ',' value ',' value ')'",
"system_function_special_syntax : encrypt_decrypt '(' value USING valid_symbol_name crypt_opt_mode KEY value crypt_opt_iv crypt_opt_counter_type crypt_opt_counter ')'",
"system_function_special_syntax : FIRST_DAY '(' of_first_last_day_part FROM value ')'",
"system_function_special_syntax : HASH '(' value ')'",
"system_function_special_syntax : hash_func '(' value USING valid_symbol_name ')'",
"system_function_special_syntax : LAST_DAY '(' of_first_last_day_part FROM value ')'",
"system_function_special_syntax : OVERLAY '(' value PLACING value FROM value FOR value ')'",
"system_function_special_syntax : OVERLAY '(' value PLACING value FROM value ')'",
"system_function_special_syntax : POSITION '(' value IN value ')'",
"system_function_special_syntax : POSITION '(' value_list_opt ')'",
"system_function_special_syntax : rsa_encrypt_decrypt '(' value KEY value crypt_opt_lparam crypt_opt_hash crypt_opt_pkcs ')'",
"system_function_special_syntax : RSA_SIGN_HASH '(' value KEY value crypt_opt_hash crypt_opt_saltlen crypt_opt_pkcs ')'",
"system_function_special_syntax : RSA_VERIFY_HASH '(' value SIGNATURE value KEY value crypt_opt_hash crypt_opt_saltlen crypt_opt_pkcs ')'",
"system_function_special_syntax : RDB_SYSTEM_PRIVILEGE '(' valid_symbol_name ')'",
"hash_func : HASH",
"hash_func : CRYPT_HASH",
"rsa_encrypt_decrypt : RSA_DECRYPT",
"rsa_encrypt_decrypt : RSA_ENCRYPT",
"crypt_opt_lparam :",
"crypt_opt_lparam : LPARAM value",
"crypt_opt_pkcs :",
"crypt_opt_pkcs : PKCS_1_5",
"crypt_opt_hash :",
"crypt_opt_hash : HASH valid_symbol_name",
"crypt_opt_saltlen :",
"crypt_opt_saltlen : SALT_LENGTH value",
"crypt_opt_mode :",
"crypt_opt_mode : MODE valid_symbol_name",
"crypt_opt_iv :",
"crypt_opt_iv : IV value",
"crypt_opt_counter_type :",
"crypt_opt_counter_type : crypt_counter_type",
"crypt_counter_type : CTR_BIG_ENDIAN",
"crypt_counter_type : CTR_LITTLE_ENDIAN",
"crypt_opt_counter :",
"crypt_opt_counter : crypt_counter_name value",
"crypt_counter_name : COUNTER",
"crypt_counter_name : CTR_LENGTH",
"encrypt_decrypt : ENCRYPT",
"encrypt_decrypt : DECRYPT",
"of_first_last_day_part : OF YEAR",
"of_first_last_day_part : OF QUARTER",
"of_first_last_day_part : OF MONTH",
"of_first_last_day_part : OF WEEK",
"string_value_function : substring_function",
"string_value_function : trim_function",
"string_value_function : UPPER '(' value ')'",
"string_value_function : LOWER '(' value ')'",
"substring_function : SUBSTRING '(' value FROM value string_length_opt ')'",
"substring_function : SUBSTRING '(' value SIMILAR value ESCAPE value ')'",
"string_length_opt :",
"string_length_opt : FOR value",
"trim_function : TRIM '(' trim_specification value FROM value ')'",
"trim_function : TRIM '(' value FROM value ')'",
"trim_function : TRIM '(' trim_specification FROM value ')'",
"trim_function : TRIM '(' value ')'",
"trim_specification : BOTH",
"trim_specification : TRAILING",
"trim_specification : LEADING",
"udf : symbol_UDF_call_name '(' value_list ')'",
"udf : symbol_UDF_call_name '(' ')'",
"udf : symbol_package_name '.' symbol_UDF_name '(' value_list ')'",
"udf : symbol_package_name '.' symbol_UDF_name '(' ')'",
"cast_specification : CAST '(' value AS data_type_descriptor ')'",
"case_expression : case_abbreviation",
"case_expression : case_specification",
"case_abbreviation : NULLIF '(' value ',' value ')'",
"case_abbreviation : IIF '(' search_condition ',' value ',' value ')'",
"case_abbreviation : COALESCE '(' value ',' value_list ')'",
"case_abbreviation : DECODE '(' value ',' decode_pairs ')'",
"case_abbreviation : DECODE '(' value ',' decode_pairs ',' value ')'",
"case_specification : simple_case",
"case_specification : searched_case",
"$$174 :",
"$$175 :",
"$$176 :",
"simple_case : CASE case_operand $$174 $$175 $$176 simple_when_clause else_case_result_opt END",
"simple_when_clause : WHEN when_operand THEN case_result",
"simple_when_clause : simple_when_clause WHEN when_operand THEN case_result",
"else_case_result_opt :",
"else_case_result_opt : ELSE case_result",
"searched_case : CASE searched_when_clause END",
"searched_case : CASE searched_when_clause ELSE case_result END",
"searched_when_clause : WHEN search_condition THEN case_result",
"searched_when_clause : searched_when_clause WHEN search_condition THEN case_result",
"when_operand : value",
"case_operand : value",
"case_result : value",
"decode_pairs : value ',' value",
"decode_pairs : decode_pairs ',' value ',' value",
"next_value_expression : NEXT VALUE FOR symbol_generator_name",
"next_value_expression : GEN_ID '(' symbol_generator_name ',' value ')'",
"timestamp_part : YEAR",
"timestamp_part : QUARTER",
"timestamp_part : MONTH",
"timestamp_part : DAY",
"timestamp_part : HOUR",
"timestamp_part : MINUTE",
"timestamp_part : SECOND",
"timestamp_part : MILLISECOND",
"timestamp_part : TIMEZONE_HOUR",
"timestamp_part : TIMEZONE_MINUTE",
"timestamp_part : TIMEZONE_NAME",
"timestamp_part : WEEK",
"timestamp_part : WEEKDAY",
"timestamp_part : YEARDAY",
"all_noise :",
"all_noise : ALL",
"distinct_noise :",
"distinct_noise : DISTINCT",
"null_value : NULL",
"null_value : UNKNOWN",
"symbol_UDF_call_name : SYMBOL",
"symbol_UDF_name : valid_symbol_name",
"symbol_blob_subtype_name : valid_symbol_name",
"symbol_blob_subtype_name : BINARY",
"symbol_character_set_name : valid_symbol_name",
"symbol_character_set_name : BINARY",
"symbol_collation_name : valid_symbol_name",
"symbol_column_name : valid_symbol_name",
"symbol_constraint_name : valid_symbol_name",
"symbol_cursor_name : valid_symbol_name",
"symbol_domain_name : valid_symbol_name",
"symbol_exception_name : valid_symbol_name",
"symbol_filter_name : valid_symbol_name",
"symbol_gdscode_name : valid_symbol_name",
"symbol_generator_name : valid_symbol_name",
"symbol_index_name : valid_symbol_name",
"symbol_item_alias_name : valid_symbol_name",
"symbol_label_name : valid_symbol_name",
"symbol_ddl_name : valid_symbol_name",
"symbol_procedure_name : valid_symbol_name",
"symbol_role_name : valid_symbol_name",
"symbol_table_alias_name : valid_symbol_name",
"symbol_table_name : valid_symbol_name",
"symbol_trigger_name : valid_symbol_name",
"symbol_user_name : valid_symbol_name",
"symbol_variable_name : valid_symbol_name",
"symbol_view_name : valid_symbol_name",
"symbol_savepoint_name : valid_symbol_name",
"symbol_package_name : valid_symbol_name",
"symbol_window_name : valid_symbol_name",
"valid_symbol_name : SYMBOL",
"valid_symbol_name : non_reserved_word",
"non_reserved_word : ACTION",
"non_reserved_word : CASCADE",
"non_reserved_word : FREE_IT",
"non_reserved_word : RESTRICT",
"non_reserved_word : ROLE",
"non_reserved_word : TYPE",
"non_reserved_word : BREAK",
"non_reserved_word : DESCRIPTOR",
"non_reserved_word : SUBSTRING",
"non_reserved_word : COALESCE",
"non_reserved_word : LAST",
"non_reserved_word : LEAVE",
"non_reserved_word : LOCK",
"non_reserved_word : NULLIF",
"non_reserved_word : NULLS",
"non_reserved_word : STATEMENT",
"non_reserved_word : FIRST",
"non_reserved_word : SKIP",
"non_reserved_word : BLOCK",
"non_reserved_word : BACKUP",
"non_reserved_word : DIFFERENCE",
"non_reserved_word : IIF",
"non_reserved_word : SCALAR_ARRAY",
"non_reserved_word : WEEKDAY",
"non_reserved_word : YEARDAY",
"non_reserved_word : SEQUENCE",
"non_reserved_word : NEXT",
"non_reserved_word : RESTART",
"non_reserved_word : COLLATION",
"non_reserved_word : RETURNING",
"non_reserved_word : IGNORE",
"non_reserved_word : LIMBO",
"non_reserved_word : UNDO",
"non_reserved_word : REQUESTS",
"non_reserved_word : TIMEOUT",
"non_reserved_word : ABS",
"non_reserved_word : ACCENT",
"non_reserved_word : ACOS",
"non_reserved_word : ALWAYS",
"non_reserved_word : ASCII_CHAR",
"non_reserved_word : ASCII_VAL",
"non_reserved_word : ASIN",
"non_reserved_word : ATAN",
"non_reserved_word : ATAN2",
"non_reserved_word : BIN_AND",
"non_reserved_word : BIN_OR",
"non_reserved_word : BIN_SHL",
"non_reserved_word : BIN_SHR",
"non_reserved_word : BIN_XOR",
"non_reserved_word : CEIL",
"non_reserved_word : COS",
"non_reserved_word : COSH",
"non_reserved_word : COT",
"non_reserved_word : DATEADD",
"non_reserved_word : DATEDIFF",
"non_reserved_word : DECODE",
"non_reserved_word : EXP",
"non_reserved_word : FLOOR",
"non_reserved_word : GEN_UUID",
"non_reserved_word : GENERATED",
"non_reserved_word : HASH",
"non_reserved_word : LIST",
"non_reserved_word : LN",
"non_reserved_word : LOG",
"non_reserved_word : LOG10",
"non_reserved_word : LPAD",
"non_reserved_word : MATCHED",
"non_reserved_word : MATCHING",
"non_reserved_word : MAXVALUE",
"non_reserved_word : MILLISECOND",
"non_reserved_word : MINVALUE",
"non_reserved_word : MOD",
"non_reserved_word : OVERLAY",
"non_reserved_word : PAD",
"non_reserved_word : PI",
"non_reserved_word : PLACING",
"non_reserved_word : POWER",
"non_reserved_word : PRESERVE",
"non_reserved_word : RAND",
"non_reserved_word : REPLACE",
"non_reserved_word : REVERSE",
"non_reserved_word : ROUND",
"non_reserved_word : RPAD",
"non_reserved_word : SIGN",
"non_reserved_word : SIN",
"non_reserved_word : SINH",
"non_reserved_word : SPACE",
"non_reserved_word : SQRT",
"non_reserved_word : TAN",
"non_reserved_word : TANH",
"non_reserved_word : TEMPORARY",
"non_reserved_word : TRUNC",
"non_reserved_word : AUTONOMOUS",
"non_reserved_word : CHAR_TO_UUID",
"non_reserved_word : FIRSTNAME",
"non_reserved_word : MIDDLENAME",
"non_reserved_word : LASTNAME",
"non_reserved_word : MAPPING",
"non_reserved_word : OS_NAME",
"non_reserved_word : UUID_TO_CHAR",
"non_reserved_word : GRANTED",
"non_reserved_word : CALLER",
"non_reserved_word : COMMON",
"non_reserved_word : DATA",
"non_reserved_word : SOURCE",
"non_reserved_word : TWO_PHASE",
"non_reserved_word : BIN_NOT",
"non_reserved_word : ACTIVE",
"non_reserved_word : AFTER",
"non_reserved_word : ASC",
"non_reserved_word : AUTO",
"non_reserved_word : BEFORE",
"non_reserved_word : COMMITTED",
"non_reserved_word : COMPUTED",
"non_reserved_word : CONDITIONAL",
"non_reserved_word : CONTAINING",
"non_reserved_word : CSTRING",
"non_reserved_word : DATABASE",
"non_reserved_word : DESC",
"non_reserved_word : DO",
"non_reserved_word : DOMAIN",
"non_reserved_word : ENTRY_POINT",
"non_reserved_word : EXCEPTION",
"non_reserved_word : EXIT",
"non_reserved_word : FILE",
"non_reserved_word : GENERATOR",
"non_reserved_word : GEN_ID",
"non_reserved_word : IF",
"non_reserved_word : INACTIVE",
"non_reserved_word : INPUT_TYPE",
"non_reserved_word : ISOLATION",
"non_reserved_word : KEY",
"non_reserved_word : LENGTH",
"non_reserved_word : LEVEL",
"non_reserved_word : MANUAL",
"non_reserved_word : MODULE_NAME",
"non_reserved_word : NAMES",
"non_reserved_word : OPTION",
"non_reserved_word : OUTPUT_TYPE",
"non_reserved_word : OVERFLOW",
"non_reserved_word : PAGE",
"non_reserved_word : PAGES",
"non_reserved_word : PAGE_SIZE",
"non_reserved_word : PASSWORD",
"non_reserved_word : PRIVILEGES",
"non_reserved_word : PROTECTED",
"non_reserved_word : READ",
"non_reserved_word : RESERVING",
"non_reserved_word : RETAIN",
"non_reserved_word : SEGMENT",
"non_reserved_word : SHADOW",
"non_reserved_word : SHARED",
"non_reserved_word : SINGULAR",
"non_reserved_word : SIZE",
"non_reserved_word : SNAPSHOT",
"non_reserved_word : SORT",
"non_reserved_word : STABILITY",
"non_reserved_word : STARTING",
"non_reserved_word : STATISTICS",
"non_reserved_word : SUB_TYPE",
"non_reserved_word : SUSPEND",
"non_reserved_word : TRANSACTION",
"non_reserved_word : UNCOMMITTED",
"non_reserved_word : WAIT",
"non_reserved_word : WEEK",
"non_reserved_word : WORK",
"non_reserved_word : WRITE",
"non_reserved_word : ABSOLUTE",
"non_reserved_word : ACOSH",
"non_reserved_word : ASINH",
"non_reserved_word : ATANH",
"non_reserved_word : BODY",
"non_reserved_word : CONTINUE",
"non_reserved_word : DDL",
"non_reserved_word : DECRYPT",
"non_reserved_word : ENCRYPT",
"non_reserved_word : ENGINE",
"non_reserved_word : IDENTITY",
"non_reserved_word : NAME",
"non_reserved_word : PACKAGE",
"non_reserved_word : PARTITION",
"non_reserved_word : PRIOR",
"non_reserved_word : RELATIVE",
"non_reserved_word : DENSE_RANK",
"non_reserved_word : FIRST_VALUE",
"non_reserved_word : NTH_VALUE",
"non_reserved_word : LAST_VALUE",
"non_reserved_word : LAG",
"non_reserved_word : LEAD",
"non_reserved_word : RANK",
"non_reserved_word : ROW_NUMBER",
"non_reserved_word : USAGE",
"non_reserved_word : LINGER",
"non_reserved_word : TAGS",
"non_reserved_word : PLUGIN",
"non_reserved_word : SERVERWIDE",
"non_reserved_word : INCREMENT",
"non_reserved_word : TRUSTED",
"non_reserved_word : BASE64_DECODE",
"non_reserved_word : BASE64_ENCODE",
"non_reserved_word : BIND",
"non_reserved_word : CLEAR",
"non_reserved_word : COUNTER",
"non_reserved_word : COMPARE_DECFLOAT",
"non_reserved_word : CONNECTIONS",
"non_reserved_word : CONSISTENCY",
"non_reserved_word : CRYPT_HASH",
"non_reserved_word : CTR_BIG_ENDIAN",
"non_reserved_word : CTR_LENGTH",
"non_reserved_word : CTR_LITTLE_ENDIAN",
"non_reserved_word : CUME_DIST",
"non_reserved_word : DEFINER",
"non_reserved_word : DISABLE",
"non_reserved_word : ENABLE",
"non_reserved_word : EXCESS",
"non_reserved_word : EXCLUDE",
"non_reserved_word : EXTENDED",
"non_reserved_word : FIRST_DAY",
"non_reserved_word : FOLLOWING",
"non_reserved_word : HEX_DECODE",
"non_reserved_word : HEX_ENCODE",
"non_reserved_word : IDLE",
"non_reserved_word : INCLUDE",
"non_reserved_word : INVOKER",
"non_reserved_word : IV",
"non_reserved_word : LAST_DAY",
"non_reserved_word : LEGACY",
"non_reserved_word : LIFETIME",
"non_reserved_word : LPARAM",
"non_reserved_word : MAKE_DBKEY",
"non_reserved_word : MESSAGE",
"non_reserved_word : MODE",
"non_reserved_word : NATIVE",
"non_reserved_word : NORMALIZE_DECFLOAT",
"non_reserved_word : NTILE",
"non_reserved_word : NUMBER",
"non_reserved_word : OLDEST",
"non_reserved_word : OTHERS",
"non_reserved_word : OVERRIDING",
"non_reserved_word : PERCENT_RANK",
"non_reserved_word : POOL",
"non_reserved_word : PRECEDING",
"non_reserved_word : PRIVILEGE",
"non_reserved_word : QUANTIZE",
"non_reserved_word : RANGE",
"non_reserved_word : RESET",
"non_reserved_word : RSA_DECRYPT",
"non_reserved_word : RSA_ENCRYPT",
"non_reserved_word : RSA_PRIVATE",
"non_reserved_word : RSA_PUBLIC",
"non_reserved_word : RSA_SIGN_HASH",
"non_reserved_word : RSA_VERIFY_HASH",
"non_reserved_word : SALT_LENGTH",
"non_reserved_word : SECURITY",
"non_reserved_word : SESSION",
"non_reserved_word : SIGNATURE",
"non_reserved_word : SQL",
"non_reserved_word : SYSTEM",
"non_reserved_word : TIES",
"non_reserved_word : TOTALORDER",
"non_reserved_word : TRAPS",
"non_reserved_word : ZONE",
"non_reserved_word : DEBUG",
"non_reserved_word : PKCS_1_5",
"non_reserved_word : BLOB_APPEND",
"non_reserved_word : LOCKED",
"non_reserved_word : OPTIMIZE",
"non_reserved_word : QUARTER",
"non_reserved_word : TARGET",
"non_reserved_word : TIMEZONE_NAME",
"non_reserved_word : UNICODE_CHAR",
"non_reserved_word : UNICODE_VAL",
};
#endif

#ifdef YYREDUCEPOSNFUNC
#define YYCALLREDUCEPOSN(e)   \
	if (reduce_posn) {     \
	  YYREDUCEPOSNFUNC(yyps->pos, &(yyps->psp)[1 - yym], &(yyps->vsp)[1 - yym], \
			   yym, yyps->psp - yyps->ps, yychar, yyposn, e);       \
	  reduce_posn = 0;    \
	}

#ifndef YYCALLREDUCEPOSNARG
#define YYCALLREDUCEPOSNARG yyps->val
#endif


#define YYPOSNARG(n) ((yyps->psp)[1 - yym + (n) - 1])
#define YYPOSNOUT    (yyps->pos)
#endif

// If delete function is not defined by the user, do not deletions.
#ifndef YYDELETEVAL
#define YYDELETEVAL(v, n)
#endif

// If delete function is not defined by the user, do not deletions.
#ifndef YYDELETEPOSN
#define YYDELETEPOSN(v, n)
#endif

#define yyclearin (yychar = (-1))

#define yyerrok (yyps->errflag = 0)

#ifndef YYSTACKGROWTH
#define YYSTACKGROWTH 128
#endif

#ifndef YYDEFSTACKSIZE
#define YYDEFSTACKSIZE 12
#endif

#ifdef YYDEBUG
int yydebug;
#endif


#define YYABORT  goto yyabort
#define YYACCEPT goto yyaccept
#define YYVALID         do { if (yyps->save)          goto yyvalid; } while(0)
#define YYVALID_NESTED  do { if (yyps->save && \
                                 yyps->save->save == 0) goto yyvalid; } while(0)
//
// For use in generated program
//
#define yytrial (yyps->save)
#define yyvsp   (yyps->vsp)
#define yyval   (yyps->val)
#define yydepth (yyps->ssp - yyps->ss)




void Parser::yySCopy(YYSTYPE* to, YYSTYPE* from, int size)
{
  int i;
  for (i = size - 1; i >= 0; i--) {
    to[i] = from[i];
  }
}


void Parser::yyPCopy(YYPOSN* to, YYPOSN* from, int size)
{
  int i;
  for (i = size - 1; i >= 0; i--) {
    to[i] = from[i];
  }
}


void Parser::yyMoreStack(yyparsestate* yyps)
{
  int p = yyps->ssp - yyps->ss;
  Yshort  *tss = yyps->ss;
  YYSTYPE *tvs = yyps->vs;
  YYPOSN  *tps = yyps->ps;
  yyps->ss = FB_NEW_POOL(getPool()) Yshort [yyps->stacksize + YYSTACKGROWTH];
  yyps->vs = FB_NEW_POOL(getPool()) YYSTYPE[yyps->stacksize + YYSTACKGROWTH];
  yyps->ps = FB_NEW_POOL(getPool()) YYPOSN [yyps->stacksize + YYSTACKGROWTH];
  memcpy(yyps->ss, tss, yyps->stacksize * sizeof(Yshort));
  yySCopy(yyps->vs, tvs, yyps->stacksize);
  yyPCopy(yyps->ps, tps, yyps->stacksize);
  yyps->stacksize += YYSTACKGROWTH;
  delete[] tss;
  delete[] tvs;
  delete[] tps;
  yyps->ssp = yyps->ss + p;
  yyps->vsp = yyps->vs + p;
  yyps->psp = yyps->ps + p;
}


Parser::yyparsestate* Parser::yyNewState(int size)
{
  yyparsestate *p = FB_NEW_POOL(getPool()) yyparsestate;
  p->stacksize = size + 4;
  p->ss = FB_NEW_POOL(getPool()) Yshort [size + 4];
  p->vs = FB_NEW_POOL(getPool()) YYSTYPE[size + 4];
  p->ps = FB_NEW_POOL(getPool()) YYPOSN [size + 4];
  memset(&p->vs[0], 0, (size + 4) * sizeof(YYSTYPE));
  memset(&p->ps[0], 0, (size + 4) * sizeof(YYPOSN));
  return p;
}


void Parser::yyFreeState(Parser::yyparsestate* p)
{
  delete[] p->ss;
  delete[] p->vs;
  delete[] p->ps;
  delete p;
}


void Parser::setNodeLineColumn(Node* node)
{
	// Check if we are not accessing the position stack below its base.
	if (&YYPOSNARG(1) >= yyps->ps)
	{
		node->line = YYPOSNARG(1).firstLine;
		node->column = YYPOSNARG(1).firstColumn;
	}
}


int Parser::parseAux()
{
  // ASF: yym moved to Parser.h
  int /*yym, */yyn, yystate, yychar, yynewerrflag;
  yyparsestate *yyerrctx = NULL;
  int reduce_posn;

#if YYDEBUG
  const char *yys;

  if ((yys = getenv("YYDEBUG"))) {
    yyn = *yys;
    if (yyn >= '0' && yyn <= '9')
      yydebug = yyn - '0';
  }
#endif

  yyps = yyNewState(YYDEFSTACKSIZE);
  yyps->save = 0;
  yynerrs = 0;
  yyps->errflag = 0;
  yychar = (-1);

  yyps->ssp = yyps->ss;
  yyps->vsp = yyps->vs;
  yyps->psp = yyps->ps;
  *(yyps->ssp) = yystate = 0;


  //
  // Main parsing loop
  //
 yyloop:
  if ((yyn = yydefred[yystate])) {
    goto yyreduce;
  }

  //
  // Read one token
  //
  if (yychar < 0) {
    if ((yychar = yylex1()) < 0) yychar = 0;
#if YYDEBUG
    if (yydebug) {
      yys = 0;
      if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
      if (!yys) yys = "illegal-symbol";
      printf("yydebug[%d,%p]: state %d, reading %d (%s)",
	     int(yydepth), yytrial, yystate, yychar, yys);
#ifdef YYDBPR
      printf("<");
      YYDBPR(yylval);
      printf(">");
#endif
      printf("\n");
    }
#endif
  }

  //
  // Do we have a conflict?
  //
  if ((yyn = yycindex[yystate]) &&
      (yyn += yychar) >= 0 &&
      yyn <= YYTABLESIZE &&
      yycheck[yyn] == yychar) {
    int ctry;

    if (yypath) {
#if YYDEBUG
      if (yydebug) {
        printf("yydebug[%d,%p]: CONFLICT in state %d: following successful trial parse\n",
	       int(yydepth), yytrial, yystate);
      }
#endif
      // Switch to the next conflict context
      yyparsestate *save = yypath;
      yypath = save->save;
      ctry = save->ctry;
      if (save->state != yystate)
        goto yyabort;
      yyFreeState(save);

    } else {

#if YYDEBUG
      if (yydebug) {
        printf("yydebug[%d,%p]: CONFLICT in state %d. ",
	       int(yydepth), yytrial, yystate);
        if (yyps->save) {
          printf("ALREADY in conflict. Continue trial parse.");
        } else {
          printf("Start trial parse.");
        }
        printf("\n");
      }
#endif
      yyparsestate *save = yyNewState(yyps->ssp - yyps->ss);
      save->save    = yyps->save;
      save->state   = yystate;
      save->errflag = yyps->errflag;
      save->ssp     = save->ss + (yyps->ssp - yyps->ss);
      save->vsp     = save->vs + (yyps->vsp - yyps->vs);
      save->psp     = save->ps + (yyps->psp - yyps->ps);
      memcpy (save->ss, yyps->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
      yySCopy(save->vs, yyps->vs, (yyps->ssp - yyps->ss + 1));
      yyPCopy(save->ps, yyps->ps, (yyps->ssp - yyps->ss + 1));
      ctry = yytable[yyn];
      if (yyctable[ctry] == -1) {
#if YYDEBUG
        if (yydebug && yychar >= 0)
          printf("yydebug[%p]: backtracking 1 token\n", yytrial);
#endif
        ctry++;
      }
      save->ctry = ctry;
      if (!yyps->save) {
        // If this is a first conflict in the stack, start saving lexemes
        if (!yylexemes) {
          yylexemes = FB_NEW_POOL(getPool()) Yshort[YYSTACKGROWTH];
          yylvals = FB_NEW_POOL(getPool()) YYSTYPE[YYSTACKGROWTH];
          yylvlim = yylvals + YYSTACKGROWTH;
          yylpsns = FB_NEW_POOL(getPool()) YYPOSN[YYSTACKGROWTH];
          yylplim = yylpsns + YYSTACKGROWTH;
        }
        if (yylvp == yylve) {
          yylvp = yylve = yylvals;
	  yylpp = yylpe = yylpsns;
          yylexp = yylexemes;
          if (yychar >= 0) {
            *yylve++ = yylval;
            *yylpe++ = yyposn;
            *yylexp = yychar;
            yychar = -1;
          }
        }
      }
      if (yychar >= 0) {
        yylvp--, yylpp--, yylexp--;
        yychar = -1;
      }
      save->lexeme = yylvp - yylvals;
      yyps->save = save;
    }
    if (yytable[yyn] == ctry) {
#if YYDEBUG
      if (yydebug)
        printf("yydebug[%d,%p]: state %d, shifting to state %d\n",
               int(yydepth), yytrial, yystate, yyctable[ctry]);
#endif
      if (yychar < 0)
        yylvp++, yylpp++, yylexp++;
      yychar = -1;
      if (yyps->errflag > 0) --yyps->errflag;
      yystate = yyctable[ctry];
      goto yyshift;
    } else {
      yyn = yyctable[ctry];
      goto yyreduce;
    }
  }

  //
  // Is action a shift?
  //
  if ((yyn = yysindex[yystate]) &&
      (yyn += yychar) >= 0 &&
      yyn <= YYTABLESIZE &&
      yycheck[yyn] == yychar) {
#if YYDEBUG
    if (yydebug)
      printf("yydebug[%d,%p]: state %d, shifting to state %d\n",
             int(yydepth), yytrial, yystate, yytable[yyn]);
#endif
    yychar = (-1);
    if (yyps->errflag > 0)  --yyps->errflag;
    yystate = yytable[yyn];
  yyshift:
    if (yyps->ssp >= yyps->ss + yyps->stacksize - 1) {
      yyMoreStack(yyps);
    }
    *++(yyps->ssp) = yystate;
    *++(yyps->vsp) = yylval;
    *++(yyps->psp) = yyposn;
    goto yyloop;
  }
  if ((yyn = yyrindex[yystate]) &&
      (yyn += yychar) >= 0 &&
      yyn <= YYTABLESIZE &&
      yycheck[yyn] == yychar) {
    yyn = yytable[yyn];
    goto yyreduce;
  }

  //
  // Action: error
  //
  if (yyps->errflag) goto yyinrecovery;
  while (yyps->save) {
    int ctry;
    yyparsestate *save = yyps->save;
#if YYDEBUG
    if (yydebug)
      printf("yydebug[%d,%p]: ERROR in state %d, CONFLICT BACKTRACKING to state %d, %d tokens\n",
             int(yydepth), yytrial, yystate, yyps->save->state, int(yylvp - yylvals - yyps->save->lexeme));
#endif
    // Memorize most forward-looking error state in case
    // it's really an error.
    if (yyerrctx == NULL || yyerrctx->lexeme < yylvp-yylvals) {
      // Free old saved error context state
      if (yyerrctx) yyFreeState(yyerrctx);
      // Create and fill out new saved error context state
      yyerrctx = yyNewState(yyps->ssp - yyps->ss);
      yyerrctx->save = yyps->save;
      yyerrctx->state = yystate;
      yyerrctx->errflag = yyps->errflag;
      yyerrctx->ssp = yyerrctx->ss + (yyps->ssp - yyps->ss);
      yyerrctx->vsp = yyerrctx->vs + (yyps->vsp - yyps->vs);
      yyerrctx->psp = yyerrctx->ps + (yyps->psp - yyps->ps);
      memcpy (yyerrctx->ss, yyps->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
      yySCopy(yyerrctx->vs, yyps->vs, (yyps->ssp - yyps->ss + 1));
      yyPCopy(yyerrctx->ps, yyps->ps, (yyps->ssp - yyps->ss + 1));
      yyerrctx->lexeme = yylvp - yylvals;
    }
    yylvp  = yylvals   + save->lexeme;
    yylpp  = yylpsns   + save->lexeme;
    yylexp = yylexemes + save->lexeme;
    yychar = -1;
    yyps->ssp = yyps->ss + (save->ssp - save->ss);
    yyps->vsp = yyps->vs + (save->vsp - save->vs);
    yyps->psp = yyps->ps + (save->psp - save->ps);
    memcpy (yyps->ss, save->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
    yySCopy(yyps->vs, save->vs,  yyps->vsp - yyps->vs + 1);
    yyPCopy(yyps->ps, save->ps,  yyps->psp - yyps->ps + 1);
    ctry = ++save->ctry;
    yystate = save->state;
    // We tried shift, try reduce now
    if ((yyn = yyctable[ctry]) >= 0) {
      goto yyreduce;
    }
    yyps->save = save->save;
    yyFreeState(save);
    //
    // Nothing left on the stack -- error
    //
    if (!yyps->save) {
#if YYDEBUG
      if (yydebug) {
        printf("yydebug[%p]: trial parse FAILED, entering ERROR mode\n",
	       yytrial);
      }
#endif
      // Restore state as it was in the most forward-advanced error
      yylvp  = yylvals   + yyerrctx->lexeme;
      yylpp  = yylpsns   + yyerrctx->lexeme;
      yylexp = yylexemes + yyerrctx->lexeme;
      yychar = yylexp[-1];
      yylval = yylvp[-1];
      yyposn = yylpp[-1];
      yyps->ssp = yyps->ss + (yyerrctx->ssp - yyerrctx->ss);
      yyps->vsp = yyps->vs + (yyerrctx->vsp - yyerrctx->vs);
      yyps->psp = yyps->ps + (yyerrctx->psp - yyerrctx->ps);
      memcpy (yyps->ss, yyerrctx->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
      yySCopy(yyps->vs, yyerrctx->vs,  yyps->vsp - yyps->vs + 1);
      yyPCopy(yyps->ps, yyerrctx->ps,  yyps->psp - yyps->ps + 1);
      yystate = yyerrctx->state;
      yyFreeState(yyerrctx);
      yyerrctx = NULL;
    }
  }
  yyerror_detailed("syntax error", yychar, yylval, yyposn);
  ++yynerrs;

 yyinrecovery:
  if (yyps->errflag < 3) {
    yyps->errflag = 3;
    for (;;) {
      if ((yyn = yysindex[*(yyps->ssp)]) &&
	  (yyn += YYERRCODE) >= 0 &&
          yyn <= YYTABLESIZE &&
	  yycheck[yyn] == YYERRCODE) {
#if YYDEBUG
        if (yydebug)
          printf("yydebug[%d,%p]: state %d, ERROR recovery shifts to state %d\n",
                 int(yydepth), yytrial, *(yyps->ssp), yytable[yyn]);
#endif
        yystate = yytable[yyn];
        goto yyshift;
      } else {
#if YYDEBUG
        if (yydebug)
          printf("yydebug[%d,%p]: ERROR recovery discards state %d\n",
                 int(yydepth), yytrial, *(yyps->ssp));
#endif
        if (yyps->ssp <= yyps->ss) {
	  goto yyabort;
	}
	if (!yytrial) {
	  YYDELETEVAL(yyps->vsp[0], 1);
	  YYDELETEPOSN(yyps->psp[0], 1);
	}
        --(yyps->ssp);
        --(yyps->vsp);
        --(yyps->psp);
      }
    }
  } else {
    if (yychar == 0) goto yyabort;
#if YYDEBUG
    if (yydebug) {
      yys = 0;
      if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
      if (!yys) yys = "illegal-symbol";
      printf("yydebug[%d,%p]: state %d, ERROR recovery discards token %d (%s)\n",
             int(yydepth), yytrial, yystate, yychar, yys);
    }
#endif
    if (!yytrial) {
      YYDELETEVAL(yylval, 0);
      YYDELETEPOSN(yyposn, 0);
    }
    yychar = (-1);
    goto yyloop;
  }

  //
  // Reduce the rule
  //
yyreduce:
  yym = yylen[yyn];
#if YYDEBUG
  if (yydebug) {
    printf("yydebug[%d,%p]: state %d, reducing by rule %d (%s)",
           int(yydepth), yytrial, yystate, yyn, yyrule[yyn]);
#ifdef YYDBPR
    if (yym) {
      int i;
      printf("<");
      for (i = yym; i > 0; i--) {
        if (i != yym) printf(", ");
        YYDBPR((yyps->vsp)[1 - i]);
      }
      printf(">");
    }
#endif
    printf("\n");
  }
#endif
  if (yyps->ssp + 1 - yym >= yyps->ss + yyps->stacksize) {
    yyMoreStack(yyps);
  }

  /* default action - assign last argument as in standard yacc */
  yyps->val = yyvsp[1 - yym];

  /* default reduced position is NULL -- no position at all.
     no position will be assigned at trial time and if no position handling is present */
  memset(&yyps->pos, 0, sizeof(yyps->pos));

  reduce_posn = TRUE;

  switch (yyn) {

case 1:
  if (!yytrial)
{ parsedStatement = yyvsp[0].dsqlStatement; }
break;
case 2:
  if (!yytrial)
{ parsedStatement = yyvsp[-1].dsqlStatement; }
break;
case 3:
  if (!yytrial)
{ yyval.dsqlStatement = FB_NEW_POOL(*statementPool) DsqlDmlStatement(*statementPool, scratch->getAttachment(), yyvsp[0].stmtNode); }
break;
case 4:
  if (!yytrial)
{ yyval.dsqlStatement = FB_NEW_POOL(*statementPool) DsqlDdlStatement(*statementPool, scratch->getAttachment(), yyvsp[0].ddlNode); }
break;
case 5:
  if (!yytrial)
{ yyval.dsqlStatement = FB_NEW_POOL(*statementPool) DsqlTransactionStatement(*statementPool, scratch->getAttachment(), yyvsp[0].traNode); }
break;
case 6:
  if (!yytrial)
{
			yyval.dsqlStatement = FB_NEW_POOL(*statementPool) DsqlSessionManagementStatement(
				*statementPool, scratch->getAttachment(), yyvsp[0].mngNode);
		}
break;
case 7:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 8:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].storeNode; }
break;
case 9:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].mergeNode; }
break;
case 10:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 11:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execBlockNode; }
break;
case 12:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].selectNode; }
break;
case 13:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 14:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].updInsNode; }
break;
case 15:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 16:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 17:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 18:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 19:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 20:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 21:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].grantRevokeNode; }
break;
case 22:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 23:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].grantRevokeNode; }
break;
case 24:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 25:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].setTransactionNode; }
break;
case 26:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].traNode; }
break;
case 27:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].traNode; }
break;
case 28:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].traNode; }
break;
case 29:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].mngNode; }
break;
case 30:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].setDecFloatRoundNode; }
break;
case 31:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].setDecFloatTrapsNode; }
break;
case 32:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].setSessionNode; }
break;
case 33:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].setRoleNode; }
break;
case 34:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].sessionResetNode; }
break;
case 35:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].mngNode; }
break;
case 36:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].setBindNode; }
break;
case 37:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].mngNode; }
break;
case 38:
  if (!yytrial)
{ yyval.grantRevokeNode = newNode<GrantRevokeNode>(true); }
break;
case 39:
yyval.grantRevokeNode = yyvsp[0].grantRevokeNode;
break;
case 40:
  if (!yytrial)
{ yyval.grantRevokeNode = yyvsp[-2].grantRevokeNode; }
break;
case 41:
yyval.privilegeArray = NOTRIAL(&yyvsp[0].grantRevokeNode->privileges);
break;
case 42:
yyval.granteeArray = NOTRIAL(&yyvsp[-6].grantRevokeNode->users);
break;
case 43:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_relation, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 44:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_procedure, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 45:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_udf, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 46:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_package_header, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 47:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_exception, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 48:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 49:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 50:
yyval.granteeArray = NOTRIAL(&yyvsp[-4].grantRevokeNode->users);
break;
case 51:
  if (!yytrial)
{
			yyvsp[-8].grantRevokeNode->object = yyvsp[-5].granteeClause;
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 52:
  if (!yytrial)
{
			yyvsp[-8].grantRevokeNode->object = newNode<GranteeClause>(obj_database, getSecurityClassName(obj_database));
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 53:
yyval.grantRevokeNode = NOTRIAL(yyvsp[0].grantRevokeNode);
break;
case 54:
yyval.granteeArray = NOTRIAL(&yyvsp[-3].grantRevokeNode->users);
break;
case 55:
  if (!yytrial)
{
			yyvsp[-7].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-7].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 56:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_relations, getSecurityClassName(obj_relations)); }
break;
case 57:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_views, getSecurityClassName(obj_views)); }
break;
case 58:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_procedures, getSecurityClassName(obj_procedures)); }
break;
case 59:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_functions, getSecurityClassName(obj_functions)); }
break;
case 60:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_packages, getSecurityClassName(obj_packages)); }
break;
case 61:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_generators, getSecurityClassName(obj_generators)); }
break;
case 62:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_generators, getSecurityClassName(obj_generators)); }
break;
case 63:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_domains, getSecurityClassName(obj_domains)); }
break;
case 64:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_exceptions, getSecurityClassName(obj_exceptions)); }
break;
case 65:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_roles, getSecurityClassName(obj_roles)); }
break;
case 66:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_charsets, getSecurityClassName(obj_charsets)); }
break;
case 67:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_collations, getSecurityClassName(obj_collations)); }
break;
case 68:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_filters, getSecurityClassName(obj_filters)); }
break;
case 71:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('A', NULL)); }
break;
case 72:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('A', NULL)); }
break;
case 73:
yyval.privilegeArray = yyvsp[0].privilegeArray;
break;
case 76:
yyval.privilegeArray = yyvsp[-2].privilegeArray;
break;
case 78:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('X', NULL)); }
break;
case 79:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('G', NULL)); }
break;
case 80:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('S', NULL)); }
break;
case 81:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('I', NULL)); }
break;
case 82:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('D', NULL)); }
break;
case 83:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('U', yyvsp[0].valueListNode)); }
break;
case 84:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('R', yyvsp[0].valueListNode)); }
break;
case 85:
  if (!yytrial)
{
			yyvsp[-2].privilegeArray->add(PrivilegeClause('C', NULL));
			yyvsp[-2].privilegeArray->add(PrivilegeClause('L', NULL));
			yyvsp[-2].privilegeArray->add(PrivilegeClause('O', NULL));
		}
break;
case 91:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('C', NULL)); }
break;
case 92:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('L', NULL)); }
break;
case 93:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('O', NULL)); }
break;
case 94:
  if (!yytrial)
{
			yyvsp[-2].privilegeArray->add(PrivilegeClause('L', NULL));
			yyvsp[-2].privilegeArray->add(PrivilegeClause('O', NULL));
		}
break;
case 98:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('C', NULL)); }
break;
case 99:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('L', NULL)); }
break;
case 100:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('O', NULL)); }
break;
case 101:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 102:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 103:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 104:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 105:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 106:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 110:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 111:
  if (!yytrial)
{ yyval.grantRevokeNode = newNode<GrantRevokeNode>(false); }
break;
case 112:
  if (!yytrial)
{ yyval.grantRevokeNode = yyvsp[-2].grantRevokeNode; }
break;
case 113:
yyval.privilegeArray = NOTRIAL(&yyvsp[-1].grantRevokeNode->privileges);
break;
case 114:
yyval.granteeArray = NOTRIAL(&yyvsp[-7].grantRevokeNode->users);
break;
case 115:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_relation, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 116:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_procedure, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 117:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_udf, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 118:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_package_header, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 119:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_exception, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 120:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 121:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 122:
yyval.granteeArray = NOTRIAL(&yyvsp[-5].grantRevokeNode->users);
break;
case 123:
  if (!yytrial)
{
			yyvsp[-8].grantRevokeNode->object = yyvsp[-4].granteeClause;
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-7].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 124:
  if (!yytrial)
{
			yyvsp[-8].grantRevokeNode->object = newNode<GranteeClause>(obj_database, getSecurityClassName(obj_database));
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-7].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 125:
yyval.grantRevokeNode = NOTRIAL(yyvsp[-1].grantRevokeNode);
break;
case 126:
  if (!yytrial)
{
			yyvsp[-7].grantRevokeNode->grantAdminOption = yyvsp[-6].boolVal;
			yyvsp[-7].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 128:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 129:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 130:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 131:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 132:
yyval.granteeArray = yyvsp[0].granteeArray;
break;
case 135:
yyval.granteeArray = yyvsp[-2].granteeArray;
break;
case 138:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_procedure, *yyvsp[0].metaNamePtr)); }
break;
case 139:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_udf, *yyvsp[0].metaNamePtr)); }
break;
case 140:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_package_header, *yyvsp[0].metaNamePtr)); }
break;
case 141:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_trigger, *yyvsp[0].metaNamePtr)); }
break;
case 142:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_view, *yyvsp[0].metaNamePtr)); }
break;
case 143:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_sql_role, *yyvsp[0].metaNamePtr)); }
break;
case 144:
  if (!yytrial)
{ yyvsp[-3].granteeArray->add(GranteeClause(obj_privilege, *yyvsp[0].metaNamePtr)); }
break;
case 145:
  if (!yytrial)
{ yyvsp[-1].granteeArray->add(GranteeClause(obj_user_or_role, *yyvsp[0].metaNamePtr)); }
break;
case 146:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_user, *yyvsp[0].metaNamePtr)); }
break;
case 147:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_user_group, *yyvsp[0].metaNamePtr)); }
break;
case 149:
yyval.grantRevokeNode = yyvsp[-2].grantRevokeNode;
break;
case 151:
  if (!yytrial)
{
			yyvsp[-1].grantRevokeNode->roles.add(GranteeClause(obj_sql_role, *yyvsp[0].metaNamePtr));
			yyvsp[-1].grantRevokeNode->defaultRoles.add(false);
		}
break;
case 152:
  if (!yytrial)
{
			yyvsp[-2].grantRevokeNode->roles.add(GranteeClause(obj_sql_role, *yyvsp[0].metaNamePtr));
			yyvsp[-2].grantRevokeNode->defaultRoles.add(true);
		}
break;
case 155:
  if (!yytrial)
{ yyvsp[-1].granteeArray->add(GranteeClause(obj_user_or_role, *yyvsp[0].metaNamePtr)); }
break;
case 156:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_user, *yyvsp[0].metaNamePtr)); }
break;
case 157:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_sql_role, *yyvsp[0].metaNamePtr)); }
break;
case 158:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode;}
break;
case 159:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 160:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 161:
  if (!yytrial)
{ yyval.createAlterFunctionNode = newNode<CreateAlterFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 162:
yyval.parametersClause = NOTRIAL(&yyvsp[0].createAlterFunctionNode->parameters);
break;
case 163:
yyval.createAlterFunctionNode = yyvsp[-3].createAlterFunctionNode;
break;
case 164:
  if (!yytrial)
{
				yyval.createAlterFunctionNode = yyvsp[-9].createAlterFunctionNode;
				yyval.createAlterFunctionNode->external = newNode<ExternalClause>();
				yyval.createAlterFunctionNode->external->name = *yyvsp[-2].stringPtr;
				yyval.createAlterFunctionNode->external->udfModule = *yyvsp[0].stringPtr;
			}
break;
case 166:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(bid);
		}
break;
case 167:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_cstring;
			yyval.legacyField->charLength = (USHORT) yyvsp[-2].int32Val;
			if (yyvsp[0].metaNamePtr)
				yyval.legacyField->charSet = *yyvsp[0].metaNamePtr;
		}
break;
case 169:
yyval.parametersClause = yyvsp[0].parametersClause;
break;
case 171:
yyval.parametersClause = yyvsp[-1].parametersClause;
break;
case 174:
yyval.parametersClause = yyvsp[-2].parametersClause;
break;
case 176:
  if (!yytrial)
{
			yyvsp[-2].parametersClause->add(newNode<ParameterClause>(yyvsp[-1].legacyField, MetaName()));
			yyvsp[-2].parametersClause->back()->udfMechanism = yyvsp[0].nullableIntVal;
		}
break;
case 177:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::empty(); }
break;
case 178:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(FUN_descriptor); }
break;
case 179:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(FUN_scalar_array); }
break;
case 180:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(FUN_ref_with_null); }
break;
case 181:
yyval.createAlterFunctionNode = yyvsp[0].createAlterFunctionNode;
break;
case 183:
yyval.createAlterFunctionNode = yyvsp[-1].createAlterFunctionNode;
break;
case 185:
  if (!yytrial)
{
			yyvsp[-2].createAlterFunctionNode->returnType = newNode<ParameterClause>(yyvsp[-1].legacyField, MetaName());
			yyvsp[-2].createAlterFunctionNode->returnType->udfMechanism = yyvsp[0].int32Val;
		}
break;
case 186:
  if (!yytrial)
{ yyvsp[-2].createAlterFunctionNode->udfReturnPos = yyvsp[0].int32Val; }
break;
case 187:
  if (!yytrial)
{ yyval.int32Val = FUN_reference; }
break;
case 188:
  if (!yytrial)
{ yyval.int32Val = FUN_value; }
break;
case 189:
  if (!yytrial)
{ yyval.int32Val = FUN_descriptor; }
break;
case 190:
  if (!yytrial)
{ yyval.int32Val = -1 * FUN_reference; }
break;
case 191:
  if (!yytrial)
{ yyval.int32Val = -1 * FUN_descriptor; }
break;
case 192:
  if (!yytrial)
{
				CreateFilterNode* node = newNode<CreateFilterNode>(*yyvsp[-8].metaNamePtr);
				node->inputFilter = yyvsp[-6].filterNameNumber;
				node->outputFilter = yyvsp[-4].filterNameNumber;
				node->entryPoint = *yyvsp[-2].stringPtr;
				node->moduleName = *yyvsp[0].stringPtr;
				yyval.ddlNode = node;
			}
break;
case 193:
  if (!yytrial)
{ yyval.filterNameNumber = newNode<CreateFilterNode::NameNumber>(*yyvsp[0].metaNamePtr); }
break;
case 194:
  if (!yytrial)
{ yyval.filterNameNumber = newNode<CreateFilterNode::NameNumber>(yyvsp[0].int32Val); }
break;
case 195:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 196:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterExceptionNode; }
break;
case 197:
  if (!yytrial)
{
				CreateIndexNode* node = newNode<CreateIndexNode>(*yyvsp[-2].metaNamePtr);
				node->unique = yyvsp[-5].boolVal;
				node->descending = yyvsp[-4].boolVal;
				node->relation = yyvsp[0].relSourceNode;
				yyval.ddlNode = node;
			}
break;
case 198:
yyval.createIndexNode = static_cast<CreateIndexNode*>(yyvsp[0].ddlNode);
break;
case 199:
  if (!yytrial)
{
				yyval.ddlNode = yyvsp[-2].ddlNode;
			}
break;
case 200:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 201:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterProcedureNode; }
break;
case 202:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createRelationNode; }
break;
case 203:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createRelationNode; }
break;
case 204:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterTriggerNode; }
break;
case 205:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterViewNode; }
break;
case 206:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 207:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 208:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].alterDatabaseNode; }
break;
case 209:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createDomainNode; }
break;
case 210:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createShadowNode; }
break;
case 211:
  if (!yytrial)
{ yyvsp[0].createAlterRoleNode->createFlag = true; yyval.ddlNode = yyvsp[0].createAlterRoleNode; }
break;
case 212:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createCollationNode; }
break;
case 213:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 214:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterPackageNode; }
break;
case 215:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createPackageBodyNode; }
break;
case 216:
yyval.boolVal = false;
break;
case 217:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 218:
yyval.boolVal = true;
break;
case 219:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 220:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 221:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateProcedureNode>(yyvsp[0].createAlterProcedureNode); }
break;
case 222:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateFunctionNode>(yyvsp[0].createAlterFunctionNode); }
break;
case 223:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateTableNode>(yyvsp[0].createRelationNode); }
break;
case 224:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateTableNode>(yyvsp[0].createRelationNode); }
break;
case 225:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateViewNode>(yyvsp[0].createAlterViewNode); }
break;
case 226:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateTriggerNode>(yyvsp[0].createAlterTriggerNode); }
break;
case 227:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreatePackageNode>(yyvsp[0].createAlterPackageNode); }
break;
case 228:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreatePackageBodyNode>(yyvsp[0].createPackageBodyNode); }
break;
case 229:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateExceptionNode>(yyvsp[0].createAlterExceptionNode); }
break;
case 230:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateSequenceNode>(yyvsp[0].createAlterSequenceNode); }
break;
case 231:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateSequenceNode>(yyvsp[0].createAlterSequenceNode); }
break;
case 232:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateUserNode>(yyvsp[0].createAlterUserNode); }
break;
case 233:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 234:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterProcedureNode; }
break;
case 235:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 236:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterTriggerNode; }
break;
case 237:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterPackageNode; }
break;
case 238:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 239:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterExceptionNode; }
break;
case 240:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 241:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 242:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 243:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 244:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 245:
  if (!yytrial)
{ yyval.createAlterExceptionNode = newNode<CreateAlterExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].intlStringPtr->getString()); }
break;
case 246:
  if (!yytrial)
{
			CreateAlterExceptionNode* node = newNode<CreateAlterExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].intlStringPtr->getString());
			node->alter = true;
			yyval.createAlterExceptionNode = node;
		}
break;
case 247:
  if (!yytrial)
{
			CreateAlterExceptionNode* node = newNode<CreateAlterExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].intlStringPtr->getString());
			node->create = false;
			node->alter = true;
			yyval.createAlterExceptionNode = node;
		}
break;
case 248:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 249:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 250:
yyval.createIndexNode = yyvsp[0].createIndexNode;
break;
case 251:
  if (!yytrial)
{
			yyvsp[-3].createIndexNode->partial = yyvsp[0].boolSourceClause;
		}
break;
case 252:
  if (!yytrial)
{ yyvsp[-1].createIndexNode->columns = yyvsp[0].valueListNode; }
break;
case 253:
  if (!yytrial)
{ yyvsp[-1].createIndexNode->columns = yyvsp[0].valueListNode; }
break;
case 254:
  if (!yytrial)
{
 			yyvsp[-4].createIndexNode->computed = newNode<ValueSourceClause>();
			yyvsp[-4].createIndexNode->computed->value = yyvsp[-1].valueExprNode;
			yyvsp[-4].createIndexNode->computed->source = makeParseStr(YYPOSNARG(2), YYPOSNARG(4));
		}
break;
case 255:
  if (!yytrial)
{ yyval.boolSourceClause = nullptr; }
break;
case 256:
  if (!yytrial)
{
			auto clause = newNode<BoolSourceClause>();
			clause->value = yyvsp[0].boolExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(2));
			yyval.boolSourceClause = clause;
		}
break;
case 257:
  if (!yytrial)
{
	 			yyval.createShadowNode = newNode<CreateShadowNode>(yyvsp[-4].int32Val);
		 		yyval.createShadowNode->manual = yyvsp[-3].boolVal;
		 		yyval.createShadowNode->conditional = yyvsp[-2].boolVal;
		 		yyval.createShadowNode->files.add(newNode<DbFileClause>(*yyvsp[-1].stringPtr));
		 		yyval.createShadowNode->files.front()->length = yyvsp[0].int32Val;
	 		}
break;
case 258:
yyval.dbFilesClause = NOTRIAL(&yyvsp[0].createShadowNode->files);
break;
case 259:
  if (!yytrial)
{ yyval.createShadowNode = yyvsp[-2].createShadowNode; }
break;
case 260:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 261:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 262:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 263:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 264:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 265:
  if (!yytrial)
{ yyval.int32Val = 0; }
break;
case 266:
  if (!yytrial)
{ yyval.int32Val = yyvsp[-1].int32Val; }
break;
case 268:
yyval.dbFilesClause = yyvsp[0].dbFilesClause;
break;
case 270:
  if (!yytrial)
{ yyvsp[-1].dbFilesClause->add(yyvsp[0].dbFileClause); }
break;
case 271:
  if (!yytrial)
{ yyvsp[-2].dbFilesClause->add(yyvsp[0].dbFileClause); }
break;
case 272:
  if (!yytrial)
{
				yyvsp[-1].legacyField->fld_name = *yyvsp[-3].metaNamePtr;
				yyval.createDomainNode = newNode<CreateDomainNode>(
					newNode<ParameterClause>(yyvsp[-1].legacyField, MetaName(), yyvsp[0].valueSourceClause));
			}
break;
case 273:
yyval.createDomainNode = yyvsp[0].createDomainNode;
break;
case 274:
  if (!yytrial)
{
				yyval.createDomainNode = yyvsp[-3].createDomainNode;
				if (yyvsp[0].metaNamePtr)
					yyval.createDomainNode->nameType->type->collate = *yyvsp[0].metaNamePtr;
			}
break;
case 278:
yyval.createDomainNode = yyvsp[-1].createDomainNode;
break;
case 280:
  if (!yytrial)
{ setClause(yyvsp[-1].createDomainNode->notNull, "NOT NULL"); }
break;
case 281:
  if (!yytrial)
{ setClause(yyvsp[-1].createDomainNode->check, "DOMAIN CHECK CONSTRAINT", yyvsp[0].boolSourceClause); }
break;
case 284:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[0].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(2));
			yyval.valueSourceClause = clause;
		}
break;
case 285:
  if (!yytrial)
{ yyval.valueSourceClause = NULL; }
break;
case 288:
  if (!yytrial)
{
			BoolSourceClause* clause = newNode<BoolSourceClause>();
			clause->value = yyvsp[-1].boolExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(4));
			yyval.boolSourceClause = clause;
		}
break;
case 289:
  if (!yytrial)
{ yyval.createAlterSequenceNode = newNode<CreateAlterSequenceNode>(*yyvsp[0].metaNamePtr); }
break;
case 290:
yyval.createAlterSequenceNode = yyvsp[0].createAlterSequenceNode;
break;
case 291:
  if (!yytrial)
{ yyval.createAlterSequenceNode = yyvsp[-2].createAlterSequenceNode; }
break;
case 293:
yyval.createAlterSequenceNode = yyvsp[-2].createAlterSequenceNode;
break;
case 297:
  if (!yytrial)
{
			setClause(yyvsp[-3].createAlterSequenceNode->value, "START WITH", yyvsp[0].int64Val);
			setClause(yyvsp[-3].createAlterSequenceNode->restartSpecified, "RESTART", true);
		}
break;
case 298:
  if (!yytrial)
{ setClause(yyvsp[-3].createAlterSequenceNode->step, "INCREMENT BY", yyvsp[0].int32Val); }
break;
case 301:
  if (!yytrial)
{
			CreateAlterSequenceNode* node = newNode<CreateAlterSequenceNode>(*yyvsp[0].metaNamePtr);
			node->alter = true;
			yyval.createAlterSequenceNode = node;
		}
break;
case 302:
  if (!yytrial)
{
			/* Remove this to implement CORE-5137*/
			if (!yyvsp[-2].createAlterSequenceNode->restartSpecified && !yyvsp[-2].createAlterSequenceNode->step.specified)
				yyerrorIncompleteCmd(YYPOSNARG(3));
			yyval.createAlterSequenceNode = yyvsp[-2].createAlterSequenceNode;
		}
break;
case 305:
  if (!yytrial)
{
			setClause(yyvsp[-1].createAlterSequenceNode->restartSpecified, "RESTART", true);
		}
break;
case 308:
  if (!yytrial)
{
			CreateAlterSequenceNode* node = newNode<CreateAlterSequenceNode>(*yyvsp[0].metaNamePtr);
			node->create = false;
			node->alter = true;
			yyval.createAlterSequenceNode = node;
		}
break;
case 309:
  if (!yytrial)
{
			if (!yyvsp[-2].createAlterSequenceNode->restartSpecified && !yyvsp[-2].createAlterSequenceNode->value.specified && !yyvsp[-2].createAlterSequenceNode->step.specified)
				yyerrorIncompleteCmd(YYPOSNARG(3));
			yyval.createAlterSequenceNode = yyvsp[-2].createAlterSequenceNode;
		}
break;
case 314:
  if (!yytrial)
{
			setClause(yyvsp[-2].createAlterSequenceNode->restartSpecified, "RESTART", true);
			setClause(yyvsp[-2].createAlterSequenceNode->value, "RESTART WITH", yyvsp[0].nullableInt64Val);
		}
break;
case 315:
  if (!yytrial)
{ yyval.nullableInt64Val = BaseNullable<SINT64>::empty(); }
break;
case 316:
  if (!yytrial)
{ yyval.nullableInt64Val = BaseNullable<SINT64>::val(yyvsp[0].int64Val); }
break;
case 317:
  if (!yytrial)
{
			CreateAlterSequenceNode* node = newNode<CreateAlterSequenceNode>(*yyvsp[-2].metaNamePtr);
			node->create = false;
			node->alter = true;
			node->legacy = true;
			node->restartSpecified = true;
			node->value = BaseNullable<SINT64>::val(yyvsp[0].int64Val);
			yyval.createAlterSequenceNode = node;
		}
break;
case 318:
  if (!yytrial)
{ yyval.int64Val = yyvsp[0].int32Val; }
break;
case 319:
  if (!yytrial)
{
			SINT64 signedNumber = (SINT64) yyvsp[0].scaledNumber.number;

			if (!yyvsp[0].scaledNumber.hex && yyvsp[0].scaledNumber.number > MAX_SINT64)
			{
				ERRD_post(
					Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
					Arg::Gds(isc_arith_except) <<
					Arg::Gds(isc_numeric_out_of_range));
			}

			yyval.int64Val = signedNumber;
		}
break;
case 320:
  if (!yytrial)
{
			SINT64 signedNumber = (SINT64) yyvsp[0].scaledNumber.number;

			if (yyvsp[0].scaledNumber.hex && signedNumber == MIN_SINT64)
				ERRD_post(Arg::Gds(isc_exception_integer_overflow));

			yyval.int64Val = -signedNumber;
		}
break;
case 321:
  if (!yytrial)
{
			yyval.int64Val = MIN_SINT64;
		}
break;
case 322:
  if (!yytrial)
{ yyval.createAlterRoleNode = newNode<CreateAlterRoleNode>(*yyvsp[0].metaNamePtr); }
break;
case 323:
yyval.createAlterRoleNode = yyvsp[0].createAlterRoleNode;
break;
case 324:
  if (!yytrial)
{ yyval.createAlterRoleNode = yyvsp[-2].createAlterRoleNode; }
break;
case 328:
yyval.createAlterRoleNode = yyvsp[-4].createAlterRoleNode;
break;
case 330:
  if (!yytrial)
{ yyvsp[-3].createAlterRoleNode->sysPrivDrop = true; }
break;
case 332:
yyval.createAlterRoleNode = yyvsp[-2].createAlterRoleNode;
break;
case 334:
  if (!yytrial)
{ yyvsp[-1].createAlterRoleNode->addPrivilege(yyvsp[0].metaNamePtr); }
break;
case 335:
  if (!yytrial)
{ yyval.createCollationNode = newNode<CreateCollationNode>(*yyvsp[-2].metaNamePtr, *yyvsp[0].metaNamePtr); }
break;
case 336:
yyval.createCollationNode = yyvsp[0].createCollationNode;
break;
case 337:
yyval.createCollationNode = yyvsp[-2].createCollationNode;
break;
case 338:
yyval.createCollationNode = yyvsp[-4].createCollationNode;
break;
case 339:
  if (!yytrial)
{ yyval.createCollationNode = yyvsp[-6].createCollationNode; }
break;
case 341:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->fromName = *yyvsp[0].metaNamePtr; }
break;
case 342:
  if (!yytrial)
{ yyvsp[-5].createCollationNode->fromExternal = *yyvsp[-1].stringPtr; }
break;
case 346:
yyval.createCollationNode = yyvsp[-1].createCollationNode;
break;
case 351:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->unsetAttribute(TEXTTYPE_ATTR_PAD_SPACE); }
break;
case 352:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->setAttribute(TEXTTYPE_ATTR_PAD_SPACE); }
break;
case 353:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->unsetAttribute(TEXTTYPE_ATTR_CASE_INSENSITIVE); }
break;
case 354:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->setAttribute(TEXTTYPE_ATTR_CASE_INSENSITIVE); }
break;
case 355:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->unsetAttribute(TEXTTYPE_ATTR_ACCENT_INSENSITIVE); }
break;
case 356:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->setAttribute(TEXTTYPE_ATTR_ACCENT_INSENSITIVE); }
break;
case 358:
  if (!yytrial)
{
			const string& s = *yyvsp[0].stringPtr;
			yyvsp[-1].createCollationNode->specificAttributes.clear();
			yyvsp[-1].createCollationNode->specificAttributes.add((const UCHAR*) s.begin(), s.length());
		}
break;
case 359:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterCharSetNode>(*yyvsp[-4].metaNamePtr, *yyvsp[0].metaNamePtr); }
break;
case 360:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterEDSPoolSetNode>(AlterEDSPoolSetNode::POOL_SIZE, yyvsp[0].int32Val); }
break;
case 361:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterEDSPoolSetNode>(AlterEDSPoolSetNode::POOL_LIFETIME, yyvsp[-1].int32Val * yyvsp[0].intVal); }
break;
case 362:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterEDSPoolClearNode>(AlterEDSPoolClearNode::POOL_DB, yyvsp[0].intlStringPtr->getString()); }
break;
case 363:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterEDSPoolClearNode>(AlterEDSPoolClearNode::POOL_ALL); }
break;
case 364:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterEDSPoolClearNode>(AlterEDSPoolClearNode::POOL_OLDEST); }
break;
case 365:
  if (!yytrial)
{ yyval.intVal = 3600; }
break;
case 366:
  if (!yytrial)
{ yyval.intVal = 60; }
break;
case 367:
  if (!yytrial)
{ yyval.intVal = 1; }
break;
case 368:
  if (!yytrial)
{
				yyval.alterDatabaseNode = newNode<AlterDatabaseNode>();
				yyval.alterDatabaseNode->create = true;
			}
break;
case 369:
yyval.alterDatabaseNode = yyvsp[0].alterDatabaseNode;
break;
case 370:
yyval.alterDatabaseNode = yyvsp[-2].alterDatabaseNode;
break;
case 371:
  if (!yytrial)
{ yyval.alterDatabaseNode = yyvsp[-4].alterDatabaseNode; }
break;
case 378:
yyval.alterDatabaseNode = yyvsp[-1].alterDatabaseNode;
break;
case 387:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->createLength = yyvsp[-1].int32Val; }
break;
case 392:
  if (!yytrial)
{ yyvsp[-1].alterDatabaseNode->files.add(yyvsp[0].dbFileClause); }
break;
case 393:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->setDefaultCharSet = *yyvsp[0].metaNamePtr; }
break;
case 394:
  if (!yytrial)
{
			yyvsp[-6].alterDatabaseNode->setDefaultCharSet = *yyvsp[-2].metaNamePtr;
			yyvsp[-6].alterDatabaseNode->setDefaultCollation = *yyvsp[0].metaNamePtr;
		}
break;
case 395:
  if (!yytrial)
{ yyvsp[-3].alterDatabaseNode->differenceFile = *yyvsp[0].stringPtr; }
break;
case 396:
  if (!yytrial)
{
				DbFileClause* clause = newNode<DbFileClause>(*yyvsp[0].stringPtr);
				yyval.dbFileClause = clause;
			}
break;
case 397:
yyval.dbFileClause = yyvsp[0].dbFileClause;
break;
case 398:
  if (!yytrial)
{ yyval.dbFileClause = yyvsp[-2].dbFileClause; }
break;
case 402:
yyval.dbFileClause = yyvsp[-1].dbFileClause;
break;
case 404:
  if (!yytrial)
{ yyvsp[-3].dbFileClause->start = yyvsp[0].int32Val; }
break;
case 405:
  if (!yytrial)
{ yyvsp[-4].dbFileClause->length = yyvsp[-1].int32Val; }
break;
case 412:
  if (!yytrial)
{
				yyval.createRelationNode = newNode<CreateRelationNode>(yyvsp[-1].relSourceNode, yyvsp[0].stringPtr);
			}
break;
case 413:
yyval.createRelationNode = yyvsp[-1].createRelationNode;
break;
case 414:
yyval.relationNode = yyvsp[-4].createRelationNode;
break;
case 415:
  if (!yytrial)
{
				yyval.createRelationNode = yyvsp[-6].createRelationNode;
			}
break;
case 417:
yyval.relationNode = yyvsp[0].relationNode;
break;
case 418:
yyval.relationNode = yyvsp[-2].relationNode;
break;
case 420:
  if (!yytrial)
{ setClause(yyvsp[-1].relationNode->ssDefiner, "SQL SECURITY", yyvsp[0].boolVal); }
break;
case 421:
  if (!yytrial)
{ setClause(yyvsp[-1].relationNode->replicationState, "PUBLICATION", yyvsp[0].boolVal); }
break;
case 422:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 423:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 424:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::empty(); }
break;
case 425:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::val(yyvsp[0].boolVal); }
break;
case 426:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 427:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 428:
  if (!yytrial)
{
				yyval.createRelationNode = newNode<CreateRelationNode>(yyvsp[0].relSourceNode);
				yyval.createRelationNode->relationType = Nullable<rel_t>::empty();
			}
break;
case 429:
yyval.createRelationNode = yyvsp[-4].createRelationNode;
break;
case 430:
  if (!yytrial)
{
				yyval.createRelationNode = yyvsp[-6].createRelationNode;
				if (!yyval.createRelationNode->relationType.specified)
					yyval.createRelationNode->relationType = rel_global_temp_delete;
			}
break;
case 431:
yyval.createRelationNode = yyvsp[0].createRelationNode;
break;
case 433:
yyval.createRelationNode = yyvsp[-2].createRelationNode;
break;
case 436:
  if (!yytrial)
{ setClause(yyvsp[-1].createRelationNode->ssDefiner, "SQL SECURITY", yyvsp[0].nullableBoolVal); }
break;
case 437:
  if (!yytrial)
{ setClause(yyvsp[-4].createRelationNode->relationType, "ON COMMIT DELETE ROWS", rel_global_temp_delete); }
break;
case 438:
  if (!yytrial)
{ setClause(yyvsp[-4].createRelationNode->relationType, "ON COMMIT PRESERVE ROWS", rel_global_temp_preserve); }
break;
case 439:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 440:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 441:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 444:
yyval.relationNode = yyvsp[0].createRelationNode;
break;
case 447:
  if (!yytrial)
{
				RelationNode::AddColumnClause* clause = yyval.addColumnClause =
					newNode<RelationNode::AddColumnClause>();
				clause->field = yyvsp[-1].legacyField;
				clause->field->fld_name = *yyvsp[-2].metaNamePtr;
				clause->defaultValue = yyvsp[0].valueSourceClause;
				yyvsp[-3].relationNode->clauses.add(clause);
			}
break;
case 448:
yyval.addColumnClause = NOTRIAL(yyvsp[0].addColumnClause);
break;
case 449:
  if (!yytrial)
{
				if (yyvsp[0].metaNamePtr)
					yyvsp[-3].addColumnClause->collate = *yyvsp[0].metaNamePtr;
			}
break;
case 450:
  if (!yytrial)
{
				RelationNode::AddColumnClause* clause = yyval.addColumnClause =
					newNode<RelationNode::AddColumnClause>();
				clause->field = yyvsp[-1].legacyField;
				clause->field->fld_name = *yyvsp[-2].metaNamePtr;
				clause->identityOptions = yyvsp[0].identityOptions;
				yyvsp[-3].relationNode->clauses.add(clause);
			}
break;
case 451:
  if (!yytrial)
{
				if (yyvsp[0].metaNamePtr)
					yyvsp[-3].addColumnClause->collate = *yyvsp[0].metaNamePtr;
			}
break;
case 452:
  if (!yytrial)
{
			RelationNode::AddColumnClause* clause = newNode<RelationNode::AddColumnClause>();
			clause->field = yyvsp[-1].legacyField;
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-3].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 453:
  if (!yytrial)
{
			RelationNode::AddColumnClause* clause = newNode<RelationNode::AddColumnClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-1].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-2].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 454:
  if (!yytrial)
{ yyval.identityOptions = newNode<RelationNode::IdentityOptions>(yyvsp[-2].identityType); }
break;
case 455:
yyval.identityOptions = yyvsp[0].identityOptions;
break;
case 456:
  if (!yytrial)
{ yyval.identityOptions = yyvsp[-2].identityOptions; }
break;
case 457:
  if (!yytrial)
{ yyval.identityType = IDENT_TYPE_BY_DEFAULT; }
break;
case 458:
  if (!yytrial)
{ yyval.identityType = IDENT_TYPE_ALWAYS; }
break;
case 460:
yyval.identityOptions = yyvsp[-1].identityOptions;
break;
case 464:
  if (!yytrial)
{ setClause(yyvsp[-3].identityOptions->startValue, "START WITH", yyvsp[0].int64Val); }
break;
case 465:
  if (!yytrial)
{ setClause(yyvsp[-3].identityOptions->increment, "INCREMENT BY", yyvsp[0].int32Val); }
break;
case 466:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[-1].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(2), YYPOSNARG(4));
			yyval.valueSourceClause = clause;
		}
break;
case 473:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 474:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 475:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 477:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 478:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfTable = *yyvsp[-2].metaNamePtr;
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 479:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
			yyval.legacyField->fullDomain = true;
		}
break;
case 480:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 481:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 482:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 483:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 484:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 485:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 487:
yyval.addColumnClause = yyvsp[0].addColumnClause;
break;
case 490:
yyval.addColumnClause = yyvsp[-1].addColumnClause;
break;
case 492:
  if (!yytrial)
{
			if (yyvsp[-2].metaNamePtr)
				yyvsp[-3].addColumnClause->constraints.back()->name = *yyvsp[-2].metaNamePtr;
		}
break;
case 493:
  if (!yytrial)
{
			setClause(yyvsp[-1].addColumnClause->notNullSpecified, "NOT NULL");
			RelationNode::AddConstraintClause& constraint = yyvsp[-1].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_NOT_NULL;
		}
break;
case 494:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-1].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_CHECK;
			constraint.check = yyvsp[0].boolSourceClause;
		}
break;
case 495:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-5].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_FK;

			constraint.columns.add(yyvsp[-5].addColumnClause->field->fld_name);
			constraint.refRelation = *yyvsp[-3].metaNamePtr;
			constraint.refAction = yyvsp[-1].refActionClause;

			const ValueListNode* refColumns = yyvsp[-2].valueListNode;
			if (refColumns)
			{
				const NestConst<ValueExprNode>* ptr = refColumns->items.begin();

				for (const NestConst<ValueExprNode>* const end = refColumns->items.end(); ptr != end; ++ptr)
					constraint.refColumns.add(nodeAs<FieldNode>(*ptr)->dsqlName);
			}

			constraint.index = yyvsp[0].indexConstraintClause;
		}
break;
case 496:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-2].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_UNIQUE;
			constraint.index = yyvsp[0].indexConstraintClause;
		}
break;
case 497:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-3].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_PK;
			constraint.index = yyvsp[0].indexConstraintClause;
		}
break;
case 498:
yyval.relationNode = yyvsp[-1].relationNode;
break;
case 499:
  if (!yytrial)
{
			if (yyvsp[-2].metaNamePtr)
			{
				static_cast<RelationNode::AddConstraintClause*>(
					yyvsp[-3].relationNode->clauses.back().getObject())->name = *yyvsp[-2].metaNamePtr;
			}
		}
break;
case 500:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 501:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 502:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = *newNode<RelationNode::AddConstraintClause>();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_UNIQUE;

			const ValueListNode* columns = yyvsp[-1].valueListNode;
			const NestConst<ValueExprNode>* ptr = columns->items.begin();

			for (const NestConst<ValueExprNode>* const end = columns->items.end(); ptr != end; ++ptr)
				constraint.columns.add(nodeAs<FieldNode>(*ptr)->dsqlName);

			constraint.index = yyvsp[0].indexConstraintClause;

			yyvsp[-3].relationNode->clauses.add(&constraint);
		}
break;
case 503:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = *newNode<RelationNode::AddConstraintClause>();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_PK;

			const ValueListNode* columns = yyvsp[-1].valueListNode;
			const NestConst<ValueExprNode>* ptr = columns->items.begin();

			for (const NestConst<ValueExprNode>* const end = columns->items.end(); ptr != end; ++ptr)
				constraint.columns.add(nodeAs<FieldNode>(*ptr)->dsqlName);

			constraint.index = yyvsp[0].indexConstraintClause;

			yyvsp[-4].relationNode->clauses.add(&constraint);
		}
break;
case 504:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = *newNode<RelationNode::AddConstraintClause>();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_FK;

			const ValueListNode* columns = yyvsp[-5].valueListNode;
			const NestConst<ValueExprNode>* ptr = columns->items.begin();

			for (const NestConst<ValueExprNode>* const end = columns->items.end(); ptr != end; ++ptr)
				constraint.columns.add(nodeAs<FieldNode>(*ptr)->dsqlName);

			constraint.refRelation = *yyvsp[-3].metaNamePtr;
			constraint.refAction = yyvsp[-1].refActionClause;

			const ValueListNode* refColumns = yyvsp[-2].valueListNode;
			if (refColumns)
			{
				const NestConst<ValueExprNode>* ptr = refColumns->items.begin();

				for (const NestConst<ValueExprNode>* const end = refColumns->items.end(); ptr != end; ++ptr)
					constraint.refColumns.add(nodeAs<FieldNode>(*ptr)->dsqlName);
			}

			constraint.index = yyvsp[0].indexConstraintClause;

			yyvsp[-8].relationNode->clauses.add(&constraint);
		}
break;
case 505:
  if (!yytrial)
{
			RelationNode::AddConstraintClause* constraint = newNode<RelationNode::AddConstraintClause>();
			constraint->constraintType = RelationNode::AddConstraintClause::CTYPE_CHECK;
			constraint->check = yyvsp[0].boolSourceClause;
			yyvsp[-1].relationNode->clauses.add(constraint);
		}
break;
case 506:
  if (!yytrial)
{ yyval.indexConstraintClause = newNode<RelationNode::IndexConstraintClause>(); }
break;
case 507:
  if (!yytrial)
{
			RelationNode::IndexConstraintClause* clause = yyval.indexConstraintClause =
				newNode<RelationNode::IndexConstraintClause>();
			clause->descending = yyvsp[-2].boolVal;
			clause->name = *yyvsp[0].metaNamePtr;
		}
break;
case 508:
  if (!yytrial)
{ yyval.refActionClause = NULL; }
break;
case 509:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(yyvsp[0].uintVal, 0); }
break;
case 510:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(0, yyvsp[0].uintVal); }
break;
case 511:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(yyvsp[0].uintVal, yyvsp[-1].uintVal); }
break;
case 512:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(yyvsp[-1].uintVal, yyvsp[0].uintVal); }
break;
case 513:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal;}
break;
case 514:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal;}
break;
case 515:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_CASCADE; }
break;
case 516:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_SET_DEFAULT; }
break;
case 517:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_SET_NULL; }
break;
case 518:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_NONE; }
break;
case 521:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[-4].createAlterProcedureNode;
			yyval.createAlterProcedureNode->ssDefiner = yyvsp[-3].nullableBoolVal;
			yyval.createAlterProcedureNode->source = makeParseStr(YYPOSNARG(4), YYPOSNARG(5));
			yyval.createAlterProcedureNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterProcedureNode->body = yyvsp[0].stmtNode;
		}
break;
case 522:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[-2].createAlterProcedureNode;
			yyval.createAlterProcedureNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterProcedureNode->source = *yyvsp[0].stringPtr;
		}
break;
case 523:
  if (!yytrial)
{ yyval.createAlterProcedureNode = newNode<CreateAlterProcedureNode>(*yyvsp[0].metaNamePtr); }
break;
case 524:
yyval.parametersClause = NOTRIAL(&yyvsp[0].createAlterProcedureNode->parameters);
break;
case 525:
yyval.parametersClause = NOTRIAL(&yyvsp[-2].createAlterProcedureNode->returns);
break;
case 526:
  if (!yytrial)
{ yyval.createAlterProcedureNode = yyvsp[-4].createAlterProcedureNode; }
break;
case 527:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[0].createAlterProcedureNode;
			yyval.createAlterProcedureNode->alter = true;
			yyval.createAlterProcedureNode->create = false;
		}
break;
case 528:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[0].createAlterProcedureNode;
			yyval.createAlterProcedureNode->alter = true;
		}
break;
case 536:
  if (!yytrial)
{ yyvsp[-3].parametersClause->add(newNode<ParameterClause>(yyvsp[-2].legacyField, optName(yyvsp[-1].metaNamePtr), yyvsp[0].valueSourceClause)); }
break;
case 539:
  if (!yytrial)
{ yyvsp[-2].parametersClause->add(newNode<ParameterClause>(yyvsp[-1].legacyField, optName(yyvsp[0].metaNamePtr))); }
break;
case 540:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[0].legacyField;
			yyval.legacyField->fld_name = *yyvsp[-1].metaNamePtr;
		}
break;
case 541:
  if (!yytrial)
{ yyval.valueSourceClause = NULL; }
break;
case 542:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[0].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(2));
			yyval.valueSourceClause = clause;
		}
break;
case 543:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[0].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(2));
			yyval.valueSourceClause = clause;
		}
break;
case 546:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[-4].createAlterFunctionNode;
			yyval.createAlterFunctionNode->ssDefiner = yyvsp[-3].nullableBoolVal;
			yyval.createAlterFunctionNode->source = makeParseStr(YYPOSNARG(4), YYPOSNARG(5));
			yyval.createAlterFunctionNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterFunctionNode->body = yyvsp[0].stmtNode;
		}
break;
case 547:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[-2].createAlterFunctionNode;
			yyval.createAlterFunctionNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterFunctionNode->source = *yyvsp[0].stringPtr;
		}
break;
case 548:
  if (!yytrial)
{ yyval.createAlterFunctionNode = newNode<CreateAlterFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 549:
  if (!yytrial)
{
				yyval.createAlterFunctionNode = yyvsp[-6].createAlterFunctionNode;
				yyval.createAlterFunctionNode->returnType = newNode<ParameterClause>(yyvsp[-2].legacyField, optName(yyvsp[-1].metaNamePtr));
				yyval.createAlterFunctionNode->deterministic = yyvsp[0].boolVal;
			}
break;
case 550:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 551:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 552:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 553:
  if (!yytrial)
{
			yyval.externalClause = newNode<ExternalClause>();
			yyval.externalClause->name = *yyvsp[-2].stringPtr;
			yyval.externalClause->engine = *yyvsp[0].metaNamePtr;
		}
break;
case 554:
  if (!yytrial)
{
			yyval.externalClause = newNode<ExternalClause>();
			yyval.externalClause->engine = *yyvsp[0].metaNamePtr;
		}
break;
case 555:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 556:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 557:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[0].createAlterFunctionNode;
			yyval.createAlterFunctionNode->alter = true;
			yyval.createAlterFunctionNode->create = false;
		}
break;
case 558:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[0].createAlterFunctionNode;
			yyval.createAlterFunctionNode->alter = true;
		}
break;
case 559:
  if (!yytrial)
{
			CreateAlterPackageNode* node = newNode<CreateAlterPackageNode>(*yyvsp[-5].metaNamePtr);
			node->ssDefiner = yyvsp[-4].nullableBoolVal;
			node->source = makeParseStr(YYPOSNARG(4), YYPOSNARG(6));
			node->items = yyvsp[-1].packageItems;
			yyval.createAlterPackageNode = node;
		}
break;
case 561:
  if (!yytrial)
{ yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >(); }
break;
case 562:
  if (!yytrial)
{
			yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >();
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 563:
  if (!yytrial)
{
			yyval.packageItems = yyvsp[-1].packageItems;
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 564:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterFunctionNode); }
break;
case 565:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterProcedureNode); }
break;
case 566:
  if (!yytrial)
{
			yyval.createAlterPackageNode = yyvsp[0].createAlterPackageNode;
			yyval.createAlterPackageNode->alter = true;
			yyval.createAlterPackageNode->create = false;
		}
break;
case 567:
  if (!yytrial)
{
			yyval.createAlterPackageNode = yyvsp[0].createAlterPackageNode;
			yyval.createAlterPackageNode->alter = true;
		}
break;
case 568:
  if (!yytrial)
{
			CreatePackageBodyNode* node = newNode<CreatePackageBodyNode>(*yyvsp[-5].metaNamePtr);
			node->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(6));
			node->declaredItems = yyvsp[-2].packageItems;
			node->items = yyvsp[-1].packageItems;
			yyval.createPackageBodyNode = node;
		}
break;
case 569:
  if (!yytrial)
{
			CreatePackageBodyNode* node = newNode<CreatePackageBodyNode>(*yyvsp[-4].metaNamePtr);
			node->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(5));
			node->items = yyvsp[-1].packageItems;
			yyval.createPackageBodyNode = node;
		}
break;
case 570:
  if (!yytrial)
{ yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >(); }
break;
case 572:
  if (!yytrial)
{
			yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >();
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 573:
  if (!yytrial)
{
			yyval.packageItems = yyvsp[-1].packageItems;
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 574:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[0].createAlterFunctionNode); }
break;
case 575:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterFunctionNode); }
break;
case 576:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[0].createAlterProcedureNode); }
break;
case 577:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterProcedureNode); }
break;
case 578:
  if (!yytrial)
{
			CompoundStmtNode* forward = yyvsp[-1].compoundStmtNode;
			CompoundStmtNode* nonForward = yyvsp[0].compoundStmtNode;

			if (!forward)
				yyval.compoundStmtNode = nonForward;
			else
			{
				if (nonForward)
					forward->statements.add(nonForward->statements.begin(), nonForward->statements.getCount());

				yyval.compoundStmtNode = forward;
			}
		}
break;
case 579:
  if (!yytrial)
{ yyval.compoundStmtNode = NULL; }
break;
case 581:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 582:
  if (!yytrial)
{
			yyvsp[-1].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-1].compoundStmtNode;
		}
break;
case 583:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[-1].declareSubProcNode; }
break;
case 584:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[-1].declareSubFuncNode; }
break;
case 585:
  if (!yytrial)
{ yyval.compoundStmtNode = NULL; }
break;
case 587:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 588:
  if (!yytrial)
{
			yyvsp[-1].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-1].compoundStmtNode;
		}
break;
case 589:
  if (!yytrial)
{
			yyval.stmtNode = yyvsp[-1].stmtNode;
			yyval.stmtNode->line = YYPOSNARG(1).firstLine;
			yyval.stmtNode->column = YYPOSNARG(1).firstColumn;
		}
break;
case 590:
  if (!yytrial)
{
			DeclareSubProcNode* node = yyvsp[-3].declareSubProcNode;
			node->dsqlBlock = newNode<ExecBlockNode>();
			node->dsqlBlock->parameters = node->dsqlParameters;
			node->dsqlBlock->returns = node->dsqlReturns;
			node->dsqlBlock->localDeclList = yyvsp[-1].compoundStmtNode;
			node->dsqlBlock->body = yyvsp[0].stmtNode;

			for (FB_SIZE_T i = 0; i < node->dsqlBlock->parameters.getCount(); ++i)
				node->dsqlBlock->parameters[i]->parameterExpr = make_parameter();

			yyval.stmtNode = node;
		}
break;
case 591:
  if (!yytrial)
{
			DeclareSubFuncNode* node = yyvsp[-3].declareSubFuncNode;
			node->dsqlBlock = newNode<ExecBlockNode>();
			node->dsqlBlock->parameters = node->dsqlParameters;
			node->dsqlBlock->returns = node->dsqlReturns;
			node->dsqlBlock->localDeclList = yyvsp[-1].compoundStmtNode;
			node->dsqlBlock->body = yyvsp[0].stmtNode;

			for (FB_SIZE_T i = 0; i < node->dsqlBlock->parameters.getCount(); ++i)
				node->dsqlBlock->parameters[i]->parameterExpr = make_parameter();

			yyval.stmtNode = node;
		}
break;
case 592:
  if (!yytrial)
{ yyval.declareSubProcNode = newNode<DeclareSubProcNode>(NOTRIAL(*yyvsp[0].metaNamePtr)); }
break;
case 593:
yyval.parametersClause = NOTRIAL(&yyvsp[0].declareSubProcNode->dsqlParameters);
break;
case 594:
yyval.parametersClause = NOTRIAL(&yyvsp[-2].declareSubProcNode->dsqlReturns);
break;
case 595:
  if (!yytrial)
{ yyval.declareSubProcNode = yyvsp[-4].declareSubProcNode; }
break;
case 596:
  if (!yytrial)
{ yyval.declareSubFuncNode = newNode<DeclareSubFuncNode>(NOTRIAL(*yyvsp[0].metaNamePtr)); }
break;
case 597:
yyval.parametersClause = NOTRIAL(&yyvsp[0].declareSubFuncNode->dsqlParameters);
break;
case 598:
  if (!yytrial)
{
				yyval.declareSubFuncNode = yyvsp[-6].declareSubFuncNode;
				yyval.declareSubFuncNode->dsqlReturns.add(newNode<ParameterClause>(yyvsp[-2].legacyField, optName(yyvsp[-1].metaNamePtr)));
				yyval.declareSubFuncNode->dsqlDeterministic = yyvsp[0].boolVal;
			}
break;
case 601:
  if (!yytrial)
{
			DeclareVariableNode* node = newNode<DeclareVariableNode>();
			node->dsqlDef = newNode<ParameterClause>(yyvsp[-2].legacyField, optName(yyvsp[-1].metaNamePtr), yyvsp[0].valueSourceClause);
			yyval.stmtNode = node;
		}
break;
case 604:
  if (!yytrial)
{
			DeclareCursorNode* node = newNode<DeclareCursorNode>(*yyvsp[-6].metaNamePtr,
				DeclareCursorNode::CUR_TYPE_EXPLICIT);
			node->dsqlScroll = yyvsp[-5].boolVal;
			node->dsqlSelect = yyvsp[-1].selectNode;
			yyval.stmtNode = node;
		}
break;
case 605:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 606:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 607:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 610:
  if (!yytrial)
{ yyval.stmtNode = newNode<LineColumnNode>(YYPOSNARG(1).firstLine, YYPOSNARG(1).firstColumn, yyvsp[-1].stmtNode); }
break;
case 611:
  if (!yytrial)
{ yyval.stmtNode = newNode<CompoundStmtNode>(); }
break;
case 612:
  if (!yytrial)
{
			BlockNode* node = newNode<BlockNode>();
			node->action = yyvsp[0].compoundStmtNode;
			yyval.stmtNode = node;
		}
break;
case 613:
  if (!yytrial)
{
			BlockNode* node = newNode<BlockNode>();
			node->action = yyvsp[-1].compoundStmtNode;
			node->handlers = yyvsp[0].compoundStmtNode;
			yyval.stmtNode = node;
		}
break;
case 614:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 615:
  if (!yytrial)
{
			yyvsp[-1].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-1].compoundStmtNode;
		}
break;
case 616:
{YYVALID;}
  if (!yytrial)
{ yyval.stmtNode = newNode<LineColumnNode>(YYPOSNARG(1).firstLine, YYPOSNARG(1).firstColumn, yyvsp[-1].stmtNode); }
break;
case 617:
{YYVALID;}
  if (!yytrial)
{ yyval.stmtNode = newNode<LineColumnNode>(YYPOSNARG(1).firstLine, YYPOSNARG(1).firstColumn, yyvsp[0].stmtNode); }
break;
case 619:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].storeNode; }
break;
case 620:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].mergeNode; }
break;
case 622:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].updInsNode; }
break;
case 626:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execStatementNode; }
break;
case 627:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execStatementNode; }
break;
case 629:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 635:
  if (!yytrial)
{ yyval.stmtNode = newNode<SuspendNode>(); }
break;
case 636:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExitNode>(); }
break;
case 637:
  if (!yytrial)
{ yyval.stmtNode = newNode<ReturnNode>(yyvsp[0].valueExprNode); }
break;
case 638:
  if (!yytrial)
{ yyval.stmtNode = newNode<SessionManagementWrapperNode>(yyvsp[0].mngNode, makeParseStr(YYPOSNARG(1), YYPOSNARG(1))); }
break;
case 640:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 644:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].forNode; }
break;
case 645:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execStatementNode; }
break;
case 646:
  if (!yytrial)
{
			InAutonomousTransactionNode* node = newNode<InAutonomousTransactionNode>();
			node->action = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 647:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(*yyvsp[0].metaNamePtr); }
break;
case 648:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].valueExprNode); }
break;
case 649:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(*yyvsp[-4].metaNamePtr, (ValueExprNode*) NULL, yyvsp[-1].valueListNode); }
break;
case 650:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(); }
break;
case 651:
  if (!yytrial)
{
				ForNode* node = newNode<ForNode>();
				node->dsqlLabelName = yyvsp[-2].metaNamePtr;
				node->dsqlSelect = yyvsp[0].selectNode;
				yyval.forNode = node;
			}
break;
case 652:
yyval.forNode = yyvsp[0].forNode;
break;
case 653:
  if (!yytrial)
{
				ForNode* node = yyvsp[-4].forNode;
				node->statement = yyvsp[0].stmtNode;
				yyval.forNode = node;
			}
break;
case 654:
  if (!yytrial)
{
			yyvsp[-2].forNode->dsqlInto = yyvsp[-1].valueListNode;
			yyvsp[-2].forNode->dsqlCursor = yyvsp[0].declCursorNode;
		}
break;
case 655:
  if (!yytrial)
{
			yyvsp[-2].forNode->dsqlInto = yyvsp[-1].valueListNode;
			yyvsp[-2].forNode->dsqlCursor = yyvsp[0].declCursorNode;
		}
break;
case 656:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 658:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 659:
  if (!yytrial)
{ yyval.execStatementNode = newNode<ExecStatementNode>(); }
break;
case 660:
yyval.execStatementNode = yyvsp[0].execStatementNode;
break;
case 661:
yyval.execStatementNode = yyvsp[-2].execStatementNode;
break;
case 662:
  if (!yytrial)
{
			yyval.execStatementNode = yyvsp[-4].execStatementNode;
		}
break;
case 663:
  if (!yytrial)
{
			yyval.execStatementNode = yyvsp[-2].execStatementNode;
			yyval.execStatementNode->outputs = yyvsp[0].valueListNode;
		}
break;
case 664:
  if (!yytrial)
{
			yyval.execStatementNode = yyvsp[-2].execStatementNode;
			yyval.execStatementNode->dsqlLabelName = yyvsp[-4].metaNamePtr;
			yyval.execStatementNode->innerStmt = yyvsp[0].stmtNode;
		}
break;
case 665:
  if (!yytrial)
{ yyvsp[-1].execStatementNode->sql = yyvsp[0].valueExprNode; }
break;
case 666:
yyval.execStatementNode = yyvsp[-4].execStatementNode;
break;
case 667:
  if (!yytrial)
{ yyvsp[-7].execStatementNode->sql = yyvsp[-5].valueExprNode; }
break;
case 668:
  if (!yytrial)
{ yyvsp[-7].execStatementNode->sql = yyvsp[-5].valueExprNode; }
break;
case 671:
  if (!yytrial)
{
			if (!yyvsp[-3].execStatementNode->inputNames)
				yyvsp[-3].execStatementNode->inputNames = FB_NEW_POOL(getPool()) EDS::ParamNames(getPool());

			yyvsp[-3].execStatementNode->inputNames->add(yyvsp[-2].metaNamePtr);

			if (!yyvsp[-3].execStatementNode->inputs)
				yyvsp[-3].execStatementNode->inputs = newNode<ValueListNode>(yyvsp[0].valueExprNode);
			else
				yyvsp[-3].execStatementNode->inputs->add(yyvsp[0].valueExprNode);
		}
break;
case 672:
  if (!yytrial)
{
			if (!yyvsp[-4].execStatementNode->inputNames)
				yyvsp[-4].execStatementNode->inputNames = FB_NEW_POOL(getPool()) EDS::ParamNames(getPool());

			if (!yyvsp[-4].execStatementNode->excessInputs)
				yyvsp[-4].execStatementNode->excessInputs = FB_NEW_POOL(getPool()) EDS::ParamNumbers(getPool());

			yyvsp[-4].execStatementNode->excessInputs->add(yyvsp[-4].execStatementNode->inputNames->getCount());

			yyvsp[-4].execStatementNode->inputNames->add(yyvsp[-2].metaNamePtr);

			if (!yyvsp[-4].execStatementNode->inputs)
				yyvsp[-4].execStatementNode->inputs = newNode<ValueListNode>(yyvsp[0].valueExprNode);
			else
				yyvsp[-4].execStatementNode->inputs->add(yyvsp[0].valueExprNode);
		}
break;
case 675:
  if (!yytrial)
{
			if (!yyvsp[-1].execStatementNode->inputs)
				yyvsp[-1].execStatementNode->inputs = newNode<ValueListNode>(yyvsp[0].valueExprNode);
			else
				yyvsp[-1].execStatementNode->inputs->add(yyvsp[0].valueExprNode);
		}
break;
case 678:
yyval.execStatementNode = yyvsp[-1].execStatementNode;
break;
case 681:
  if (!yytrial)
{ setClause(yyvsp[-5].execStatementNode->dataSource, "EXTERNAL DATA SOURCE", yyvsp[0].valueExprNode); }
break;
case 682:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->dataSource, "EXTERNAL DATA SOURCE", yyvsp[0].valueExprNode); }
break;
case 683:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->userName, "USER", yyvsp[0].valueExprNode); }
break;
case 684:
  if (!yytrial)
{ setClause(yyvsp[-2].execStatementNode->password, "PASSWORD", yyvsp[0].valueExprNode); }
break;
case 685:
  if (!yytrial)
{ setClause(yyvsp[-2].execStatementNode->role, "ROLE", yyvsp[0].valueExprNode); }
break;
case 686:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->traScope, "TRANSACTION", EDS::traAutonomous); }
break;
case 687:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->traScope, "TRANSACTION", EDS::traCommon); }
break;
case 688:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->useCallerPrivs, "CALLER PRIVILEGES"); }
break;
case 689:
  if (!yytrial)
{
			IfNode* node = newNode<IfNode>();
			node->condition = yyvsp[-5].boolExprNode;
			node->trueAction = yyvsp[-2].stmtNode;
			node->falseAction = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 690:
  if (!yytrial)
{
			IfNode* node = newNode<IfNode>();
			node->condition = yyvsp[-3].boolExprNode;
			node->trueAction = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 691:
  if (!yytrial)
{
			PostEventNode* node = newNode<PostEventNode>();
			node->event = yyvsp[-1].valueExprNode;
			node->argument = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 692:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 693:
  if (!yytrial)
{
			ForNode* node = newNode<ForNode>();
			node->dsqlSelect = yyvsp[-2].selectNode;
			node->dsqlInto = yyvsp[0].valueListNode;
			yyval.stmtNode = node;
		}
break;
case 694:
  if (!yytrial)
{
			VariableNode* node = newNode<VariableNode>();
			node->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.valueExprNode = node;
		}
break;
case 695:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 696:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].fieldNode); }
break;
case 697:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].fieldNode); }
break;
case 698:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 699:
  if (!yytrial)
{
			LoopNode* node = newNode<LoopNode>();
			node->dsqlLabelName = yyvsp[-6].metaNamePtr;
			node->dsqlExpr = yyvsp[-3].boolExprNode;
			node->statement = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 700:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 701:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[-1].metaNamePtr; }
break;
case 702:
  if (!yytrial)
{ yyval.stmtNode = newNode<ContinueLeaveNode>(blr_leave); }
break;
case 703:
  if (!yytrial)
{
			ContinueLeaveNode* node = newNode<ContinueLeaveNode>(blr_leave);
			node->dsqlLabelName = yyvsp[0].metaNamePtr;
			yyval.stmtNode = node;
		}
break;
case 704:
  if (!yytrial)
{
			ContinueLeaveNode* node = newNode<ContinueLeaveNode>(blr_continue_loop);
			node->dsqlLabelName = yyvsp[0].metaNamePtr;
			yyval.stmtNode = node;
		}
break;
case 705:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 707:
  if (!yytrial)
{ yyval.declCursorNode = NULL; }
break;
case 709:
  if (!yytrial)
{ yyval.declCursorNode = newNode<DeclareCursorNode>(*yyvsp[0].metaNamePtr, DeclareCursorNode::CUR_TYPE_FOR); }
break;
case 710:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 711:
  if (!yytrial)
{
			yyvsp[-1].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-1].compoundStmtNode;
		}
break;
case 712:
  if (!yytrial)
{ yyval.errorHandlerNode = newNode<ErrorHandlerNode>(); }
break;
case 713:
yyval.exceptionArray = NOTRIAL(&yyvsp[0].errorHandlerNode->conditions);
break;
case 714:
  if (!yytrial)
{
				ErrorHandlerNode* node = yyvsp[-4].errorHandlerNode;
				node->action = yyvsp[0].stmtNode;
				yyval.stmtNode = node;
			}
break;
case 715:
yyval.exceptionArray = yyvsp[0].exceptionArray;
break;
case 717:
yyval.exceptionArray = yyvsp[-2].exceptionArray;
break;
case 719:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::SQL_CODE;
			item.code = yyvsp[0].int32Val;
		}
break;
case 720:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::SQL_STATE;
			item.name = yyvsp[0].intlStringPtr->getString();
		}
break;
case 721:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::GDS_CODE;
			item.name = yyvsp[0].metaNamePtr->c_str();
		}
break;
case 722:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::XCP_CODE;
			item.name = yyvsp[0].metaNamePtr->c_str();
		}
break;
case 723:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-1].exceptionArray->add();
			item.type = ExceptionItem::XCP_DEFAULT;
		}
break;
case 727:
  if (!yytrial)
{ yyval.stmtNode = newNode<CursorStmtNode>(blr_cursor_open, *yyvsp[0].metaNamePtr); }
break;
case 728:
  if (!yytrial)
{ yyval.stmtNode = newNode<CursorStmtNode>(blr_cursor_close, *yyvsp[0].metaNamePtr); }
break;
case 729:
  if (!yytrial)
{ yyval.cursorStmtNode = newNode<CursorStmtNode>(blr_cursor_fetch_scroll); }
break;
case 730:
yyval.cursorStmtNode = yyvsp[0].cursorStmtNode;
break;
case 731:
  if (!yytrial)
{
			CursorStmtNode* cursorStmt = yyvsp[-5].cursorStmtNode;
			cursorStmt->dsqlName = *yyvsp[-1].metaNamePtr;
			cursorStmt->dsqlIntoStmt = yyvsp[0].valueListNode;
			yyval.stmtNode = cursorStmt;
		}
break;
case 732:
  if (!yytrial)
{ yyval.stmtNode = newNode<CursorStmtNode>(blr_cursor_fetch, *yyvsp[-1].metaNamePtr, yyvsp[0].valueListNode); }
break;
case 733:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_bof; }
break;
case 734:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_eof; }
break;
case 735:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_backward; }
break;
case 736:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_forward; }
break;
case 737:
  if (!yytrial)
{
			yyvsp[-2].cursorStmtNode->scrollOp = blr_scroll_absolute;
			yyvsp[-2].cursorStmtNode->scrollExpr = yyvsp[0].valueExprNode;
		}
break;
case 738:
  if (!yytrial)
{
			yyvsp[-2].cursorStmtNode->scrollOp = blr_scroll_relative;
			yyvsp[-2].cursorStmtNode->scrollExpr = yyvsp[0].valueExprNode;
		}
break;
case 739:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExecProcedureNode>(QualifiedName(*yyvsp[-2].metaNamePtr), yyvsp[-1].valueListNode, yyvsp[0].valueListNode); }
break;
case 740:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExecProcedureNode>(QualifiedName(*yyvsp[-2].metaNamePtr, *yyvsp[-4].metaNamePtr), yyvsp[-1].valueListNode, yyvsp[0].valueListNode); }
break;
case 741:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 742:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 743:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 744:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 745:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 746:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 747:
  if (!yytrial)
{ yyval.execBlockNode = newNode<ExecBlockNode>(); }
break;
case 748:
yyval.parametersClause = NOTRIAL(&yyvsp[0].execBlockNode->parameters);
break;
case 749:
yyval.parametersClause = NOTRIAL(&yyvsp[-2].execBlockNode->returns);
break;
case 750:
  if (!yytrial)
{
			ExecBlockNode* node = yyvsp[-7].execBlockNode;
			node->localDeclList = yyvsp[-1].compoundStmtNode;
			node->body = yyvsp[0].stmtNode;
			yyval.execBlockNode = node;
		}
break;
case 755:
  if (!yytrial)
{ yyvsp[-4].parametersClause->add(newNode<ParameterClause>(yyvsp[-3].legacyField, optName(yyvsp[-2].metaNamePtr), (ValueSourceClause*) NULL, yyvsp[0].valueExprNode)); }
break;
case 756:
  if (!yytrial)
{
			CreateAlterViewNode* node = newNode<CreateAlterViewNode>(yyvsp[-4].relSourceNode, yyvsp[-3].valueListNode, yyvsp[-1].selectExprNode);
			node->source = makeParseStr(YYPOSNARG(4), YYPOSNARG(5));
			node->withCheckOption = yyvsp[0].boolVal;
			yyval.createAlterViewNode = node;
		}
break;
case 757:
  if (!yytrial)
{
			yyvsp[0].createAlterViewNode->alter = true;
			yyval.ddlNode = yyvsp[0].createAlterViewNode;
		}
break;
case 758:
  if (!yytrial)
{
			yyvsp[0].createAlterViewNode->alter = true;
			yyvsp[0].createAlterViewNode->create = false;
			yyval.ddlNode = yyvsp[0].createAlterViewNode;
		}
break;
case 759:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 760:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 761:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = yyvsp[-4].createAlterTriggerNode;
			yyval.createAlterTriggerNode->ssDefiner = yyvsp[-3].nullableSqlSecurityVal;
			yyval.createAlterTriggerNode->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(5));
			yyval.createAlterTriggerNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterTriggerNode->body = yyvsp[0].stmtNode;
		}
break;
case 762:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = yyvsp[-2].createAlterTriggerNode;
			yyval.createAlterTriggerNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterTriggerNode->source = *yyvsp[0].stringPtr;
		}
break;
case 763:
  if (!yytrial)
{ yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[0].metaNamePtr); }
break;
case 764:
yyval.createAlterTriggerNode = NOTRIAL(yyvsp[0].createAlterTriggerNode);
break;
case 765:
  if (!yytrial)
{ yyval.createAlterTriggerNode = yyvsp[-2].createAlterTriggerNode; }
break;
case 766:
yyval.createAlterTriggerNode = NOTRIAL(yyvsp[-1].createAlterTriggerNode);
break;
case 767:
  if (!yytrial)
{
			yyvsp[-4].createAlterTriggerNode->active = yyvsp[-3].nullableBoolVal;
			yyvsp[-4].createAlterTriggerNode->type = yyvsp[-1].uint64Val;
			setClause(yyvsp[-4].createAlterTriggerNode->position, "POSITION", yyvsp[0].nullableIntVal);
		}
break;
case 768:
  if (!yytrial)
{
			yyvsp[-5].createAlterTriggerNode->relationName = *yyvsp[-3].metaNamePtr;
			yyvsp[-5].createAlterTriggerNode->active = yyvsp[-2].nullableBoolVal;
			yyvsp[-5].createAlterTriggerNode->type = yyvsp[-1].uint64Val;
			setClause(yyvsp[-5].createAlterTriggerNode->position, "POSITION", yyvsp[0].nullableIntVal);
		}
break;
case 769:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = yyvsp[0].createAlterTriggerNode;
			yyval.createAlterTriggerNode->alter = true;
		}
break;
case 770:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::val(true); }
break;
case 771:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::val(false); }
break;
case 772:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::empty(); }
break;
case 773:
  if (!yytrial)
{
			yyval.uint64Val = yyvsp[-3].uint64Val;
			setClause(yyvsp[-4].createAlterTriggerNode->position, "POSITION", yyvsp[-2].nullableIntVal);
			yyvsp[-4].createAlterTriggerNode->relationName = *yyvsp[0].metaNamePtr;
		}
break;
case 774:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[0].uint64Val; }
break;
case 775:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[-1].uint64Val + yyvsp[0].uint64Val; }
break;
case 776:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[-1].uint64Val + yyvsp[0].uint64Val - 1; }
break;
case 777:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_CONNECT; }
break;
case 778:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_DISCONNECT; }
break;
case 779:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_TRANS_START; }
break;
case 780:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_TRANS_COMMIT; }
break;
case 781:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_TRANS_ROLLBACK; }
break;
case 783:
  if (!yytrial)
{
			yyval.uint64Val = TRIGGER_TYPE_DDL | DDL_TRIGGER_ANY;
		}
break;
case 784:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_TABLE); }
break;
case 785:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_TABLE); }
break;
case 786:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_TABLE); }
break;
case 787:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_PROCEDURE); }
break;
case 788:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_PROCEDURE); }
break;
case 789:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_PROCEDURE); }
break;
case 790:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_FUNCTION); }
break;
case 791:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_FUNCTION); }
break;
case 792:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_FUNCTION); }
break;
case 793:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_TRIGGER); }
break;
case 794:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_TRIGGER); }
break;
case 795:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_TRIGGER); }
break;
case 796:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_EXCEPTION); }
break;
case 797:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_EXCEPTION); }
break;
case 798:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_EXCEPTION); }
break;
case 799:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_VIEW); }
break;
case 800:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_VIEW); }
break;
case 801:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_VIEW); }
break;
case 802:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_DOMAIN); }
break;
case 803:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_DOMAIN); }
break;
case 804:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_DOMAIN); }
break;
case 805:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_ROLE); }
break;
case 806:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_ROLE); }
break;
case 807:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_ROLE); }
break;
case 808:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_SEQUENCE); }
break;
case 809:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_SEQUENCE); }
break;
case 810:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_SEQUENCE); }
break;
case 811:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_USER); }
break;
case 812:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_USER); }
break;
case 813:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_USER); }
break;
case 814:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_INDEX); }
break;
case 815:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_INDEX); }
break;
case 816:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_INDEX); }
break;
case 817:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_COLLATION); }
break;
case 818:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_COLLATION); }
break;
case 819:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_CHARACTER_SET); }
break;
case 820:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_PACKAGE); }
break;
case 821:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_PACKAGE); }
break;
case 822:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_PACKAGE); }
break;
case 823:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_PACKAGE_BODY); }
break;
case 824:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_PACKAGE_BODY); }
break;
case 825:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_MAPPING); }
break;
case 826:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_MAPPING); }
break;
case 827:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_MAPPING); }
break;
case 828:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[-2].uint64Val | yyvsp[0].uint64Val; }
break;
case 829:
  if (!yytrial)
{ yyval.uint64Val = 0; }
break;
case 830:
  if (!yytrial)
{ yyval.uint64Val = 1; }
break;
case 831:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 0, 0); }
break;
case 832:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 0, 0); }
break;
case 833:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 0, 0); }
break;
case 834:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 2, 0); }
break;
case 835:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 3, 0); }
break;
case 836:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 1, 0); }
break;
case 837:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 3, 0); }
break;
case 838:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 1, 0); }
break;
case 839:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 2, 0); }
break;
case 840:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 2, 3); }
break;
case 841:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 3, 2); }
break;
case 842:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 1, 3); }
break;
case 843:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 3, 1); }
break;
case 844:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 1, 2); }
break;
case 845:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 2, 1); }
break;
case 846:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::empty(); }
break;
case 847:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(yyvsp[0].int32Val); }
break;
case 848:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 849:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 850:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterExceptionNode; }
break;
case 851:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterRelationNode>(yyvsp[0].relSourceNode); }
break;
case 852:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[-2].relationNode; }
break;
case 853:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 854:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterTriggerNode; }
break;
case 855:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterProcedureNode; }
break;
case 856:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterPackageNode; }
break;
case 857:
  if (!yytrial)
{ yyval.alterDatabaseNode = newNode<AlterDatabaseNode>(); }
break;
case 858:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[-2].alterDatabaseNode; }
break;
case 859:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].alterDomainNode; }
break;
case 860:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 861:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 862:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 863:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 864:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 865:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 866:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 867:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 868:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 869:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 870:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 871:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 872:
  if (!yytrial)
{ yyval.alterDomainNode = newNode<AlterDomainNode>(*yyvsp[0].metaNamePtr); }
break;
case 873:
yyval.alterDomainNode = yyvsp[0].alterDomainNode;
break;
case 874:
  if (!yytrial)
{ yyval.alterDomainNode = yyvsp[-2].alterDomainNode; }
break;
case 876:
yyval.alterDomainNode = yyvsp[-1].alterDomainNode;
break;
case 878:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->setDefault, "DOMAIN DEFAULT", yyvsp[0].valueSourceClause); }
break;
case 879:
  if (!yytrial)
{ setClause(yyvsp[-3].alterDomainNode->setConstraint, "DOMAIN CONSTRAINT", yyvsp[0].boolSourceClause); }
break;
case 880:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->setConstraint, "DOMAIN CONSTRAINT", yyvsp[0].boolSourceClause); }
break;
case 881:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->dropDefault, "DOMAIN DROP DEFAULT"); }
break;
case 882:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->dropConstraint, "DOMAIN DROP CONSTRAINT"); }
break;
case 883:
  if (!yytrial)
{ setClause(yyvsp[-3].alterDomainNode->notNullFlag, "{SET | DROP} NOT NULL", false); }
break;
case 884:
  if (!yytrial)
{ setClause(yyvsp[-3].alterDomainNode->notNullFlag, "{SET | DROP} NOT NULL", true); }
break;
case 885:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->renameTo, "DOMAIN NAME", *yyvsp[0].metaNamePtr); }
break;
case 886:
  if (!yytrial)
{
			/*// FIXME: ALTER DOMAIN doesn't support collations, and altered domain's*/
			/*// collation is always lost.*/
			dsql_fld* type = yyvsp[0].legacyField;
			type->collate = "";
			setClause(yyvsp[-2].alterDomainNode->type, "DOMAIN TYPE", type);
		}
break;
case 889:
  if (!yytrial)
{
			RelationNode::DropColumnClause* clause = newNode<RelationNode::DropColumnClause>();
			clause->name = *yyvsp[-1].metaNamePtr;
			clause->cascade = yyvsp[0].boolVal;
			yyvsp[-3].relationNode->clauses.add(clause);
		}
break;
case 890:
  if (!yytrial)
{
			RelationNode::DropConstraintClause* clause = newNode<RelationNode::DropConstraintClause>();
			clause->name = *yyvsp[0].metaNamePtr;
			yyvsp[-3].relationNode->clauses.add(clause);
		}
break;
case 893:
  if (!yytrial)
{
			RelationNode::AlterColPosClause* clause = newNode<RelationNode::AlterColPosClause>();
			clause->name = *yyvsp[-2].metaNamePtr;
			clause->newPos = yyvsp[0].int32Val;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 894:
  if (!yytrial)
{
			RelationNode::AlterColNameClause* clause = newNode<RelationNode::AlterColNameClause>();
			clause->fromName = *yyvsp[-2].metaNamePtr;
			clause->toName = *yyvsp[0].metaNamePtr;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 895:
  if (!yytrial)
{
			RelationNode::AlterColNullClause* clause = newNode<RelationNode::AlterColNullClause>();
			clause->name = *yyvsp[-3].metaNamePtr;
			clause->notNullFlag = false;
			yyvsp[-5].relationNode->clauses.add(clause);
		}
break;
case 896:
  if (!yytrial)
{
			RelationNode::AlterColNullClause* clause = newNode<RelationNode::AlterColNullClause>();
			clause->name = *yyvsp[-3].metaNamePtr;
			clause->notNullFlag = true;
			yyvsp[-5].relationNode->clauses.add(clause);
		}
break;
case 897:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = yyvsp[0].legacyField;
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 898:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = yyvsp[-1].legacyField;
			clause->field->fld_name = *yyvsp[-3].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-5].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 899:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-1].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-3].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 900:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			clause->defaultValue = yyvsp[0].valueSourceClause;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 901:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			clause->dropDefault = true;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 902:
  if (!yytrial)
{ yyval.identityOptions = newNode<RelationNode::IdentityOptions>(); }
break;
case 903:
  if (!yytrial)
{
				RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
				clause->field = newNode<dsql_fld>();
				clause->field->fld_name = *yyvsp[-3].metaNamePtr;
				clause->identityOptions = yyvsp[-2].identityOptions;
				yyvsp[-5].relationNode->clauses.add(clause);
			}
break;
case 904:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			clause->dropIdentity = true;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 905:
  if (!yytrial)
{
			setClause(yyvsp[-4].relationNode->ssDefiner, "SQL SECURITY", true);
			RelationNode::Clause* clause =
				newNode<RelationNode::Clause>(RelationNode::Clause::TYPE_ALTER_SQL_SECURITY);
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 906:
  if (!yytrial)
{
			setClause(yyvsp[-4].relationNode->ssDefiner, "SQL SECURITY", false);
			RelationNode::Clause* clause =
				newNode<RelationNode::Clause>(RelationNode::Clause::TYPE_ALTER_SQL_SECURITY);
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 907:
  if (!yytrial)
{
			setClause(yyvsp[-3].relationNode->ssDefiner, "SQL SECURITY", Nullable<bool>::empty());
			RelationNode::Clause* clause =
				newNode<RelationNode::Clause>(RelationNode::Clause::TYPE_ALTER_SQL_SECURITY);
			yyvsp[-3].relationNode->clauses.add(clause);
		}
break;
case 908:
  if (!yytrial)
{
			setClause(yyvsp[-2].relationNode->replicationState, "PUBLICATION", true);
			RelationNode::Clause* clause =
				newNode<RelationNode::Clause>(RelationNode::Clause::TYPE_ALTER_PUBLICATION);
			yyvsp[-2].relationNode->clauses.add(clause);
		}
break;
case 909:
  if (!yytrial)
{
			setClause(yyvsp[-2].relationNode->replicationState, "PUBLICATION", false);
			RelationNode::Clause* clause =
				newNode<RelationNode::Clause>(RelationNode::Clause::TYPE_ALTER_PUBLICATION);
			yyvsp[-2].relationNode->clauses.add(clause);
		}
break;
case 1008:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 1009:
yyval.identityOptions = yyvsp[-2].identityOptions;
break;
case 1012:
  if (!yytrial)
{ yyvsp[-3].identityOptions->type = IDENT_TYPE_ALWAYS; }
break;
case 1013:
  if (!yytrial)
{ yyvsp[-4].identityOptions->type = IDENT_TYPE_BY_DEFAULT; }
break;
case 1018:
  if (!yytrial)
{
			setClause(yyvsp[-2].identityOptions->restart, "RESTART");
			yyvsp[-2].identityOptions->startValue = yyvsp[0].nullableInt64Val;
		}
break;
case 1019:
  if (!yytrial)
{ setClause(yyvsp[-4].identityOptions->increment, "SET INCREMENT BY", yyvsp[0].int32Val); }
break;
case 1020:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1021:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1022:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1023:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterIndexNode>(*yyvsp[-1].metaNamePtr, true); }
break;
case 1024:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterIndexNode>(*yyvsp[-1].metaNamePtr, false); }
break;
case 1025:
  if (!yytrial)
{
			AlterExternalFunctionNode* node = newNode<AlterExternalFunctionNode>(*yyvsp[-2].metaNamePtr);
			if (yyvsp[-1].stringPtr)
				node->clauses.name = *yyvsp[-1].stringPtr;
			if (yyvsp[0].stringPtr)
				node->clauses.udfModule = *yyvsp[0].stringPtr;
			yyval.ddlNode = node;
		}
break;
case 1026:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 1027:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 1028:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 1029:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 1030:
  if (!yytrial)
{
			MappingNode* mn = newNode<MappingNode>(MappingNode::MAP_RPL, "AutoAdminImplementationMapping");
			mn->op = yyvsp[-3].boolVal ? MappingNode::MAP_RPL : MappingNode::MAP_DROP;
			mn->from = newNode<IntlString>(FB_DOMAIN_ANY_RID_ADMINS);
			mn->fromType = newNode<MetaName>(FB_PREDEFINED_GROUP);
			mn->mode = 'P';
			mn->plugin = newNode<MetaName>("Win_Sspi");
			mn->role = true;
			mn->to = yyvsp[-4].metaNamePtr;
			mn->validateAdmin();
			yyval.ddlNode = mn;
		}
break;
case 1031:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterRoleNode; }
break;
case 1032:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 1033:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1034:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1037:
yyval.dbFilesClause = NOTRIAL(&yyvsp[-1].alterDatabaseNode->files);
break;
case 1039:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->differenceFile = *yyvsp[0].stringPtr; }
break;
case 1040:
  if (!yytrial)
{ yyvsp[-3].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_DROP_DIFFERENCE; }
break;
case 1041:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_BEGIN_BACKUP; }
break;
case 1042:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_END_BACKUP; }
break;
case 1043:
  if (!yytrial)
{ yyvsp[-5].alterDatabaseNode->setDefaultCharSet = *yyvsp[0].metaNamePtr; }
break;
case 1044:
yyval.alterDatabaseNode = yyvsp[-3].alterDatabaseNode;
break;
case 1045:
  if (!yytrial)
{
			setClauseFlag(yyvsp[-5].alterDatabaseNode->clauses, AlterDatabaseNode::CLAUSE_CRYPT, "CRYPT");
			yyvsp[-5].alterDatabaseNode->cryptPlugin = *yyvsp[-2].metaNamePtr;
		}
break;
case 1046:
  if (!yytrial)
{ setClauseFlag(yyvsp[-1].alterDatabaseNode->clauses, AlterDatabaseNode::CLAUSE_CRYPT, "CRYPT"); }
break;
case 1047:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->linger = yyvsp[0].int32Val; }
break;
case 1048:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->linger = 0; }
break;
case 1049:
  if (!yytrial)
{ yyvsp[-3].alterDatabaseNode->ssDefiner = yyvsp[0].boolVal; }
break;
case 1050:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_ENABLE_PUB; }
break;
case 1051:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_DISABLE_PUB; }
break;
case 1052:
  if (!yytrial)
{ yyvsp[-5].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_PUB_INCL_TABLE; }
break;
case 1053:
  if (!yytrial)
{ yyvsp[-5].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_PUB_EXCL_TABLE; }
break;
case 1055:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->keyName = *yyvsp[0].metaNamePtr; }
break;
case 1060:
  if (!yytrial)
{ yyvsp[-1].alterDatabaseNode->pubTables.add(*yyvsp[0].metaNamePtr); }
break;
case 1061:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-7].metaNamePtr);
			yyval.createAlterTriggerNode->alter = true;
			yyval.createAlterTriggerNode->create = false;
			yyval.createAlterTriggerNode->active = yyvsp[-6].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-5].nullableUint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-4].nullableIntVal;
			yyval.createAlterTriggerNode->ssDefiner = yyvsp[-3].nullableSqlSecurityVal;
			yyval.createAlterTriggerNode->source = makeParseStr(YYPOSNARG(6), YYPOSNARG(8));
			yyval.createAlterTriggerNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterTriggerNode->body = yyvsp[0].stmtNode;
		}
break;
case 1062:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-5].metaNamePtr);
			yyval.createAlterTriggerNode->alter = true;
			yyval.createAlterTriggerNode->create = false;
			yyval.createAlterTriggerNode->active = yyvsp[-4].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-3].nullableUint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-2].nullableIntVal;
			yyval.createAlterTriggerNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterTriggerNode->source = *yyvsp[0].stringPtr;
		}
break;
case 1063:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-4].metaNamePtr);
			yyval.createAlterTriggerNode->alter = true;
			yyval.createAlterTriggerNode->create = false;
			yyval.createAlterTriggerNode->active = yyvsp[-3].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-2].nullableUint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-1].nullableIntVal;
			yyval.createAlterTriggerNode->ssDefiner = yyvsp[0].nullableSqlSecurityVal;
		}
break;
case 1064:
  if (!yytrial)
{ yyval.nullableUint64Val = Nullable<FB_UINT64>::val(yyvsp[-1].uint64Val + yyvsp[0].uint64Val - 1); }
break;
case 1065:
  if (!yytrial)
{ yyval.nullableUint64Val = Nullable<FB_UINT64>::empty(); }
break;
case 1066:
  if (!yytrial)
{ yyval.nullableSqlSecurityVal = Nullable<TriggerDefinition::SqlSecurity>::empty(); }
break;
case 1067:
  if (!yytrial)
{ yyval.nullableSqlSecurityVal = Nullable<TriggerDefinition::SqlSecurity>::val(TriggerDefinition::SS_DEFINER); }
break;
case 1068:
  if (!yytrial)
{ yyval.nullableSqlSecurityVal = Nullable<TriggerDefinition::SqlSecurity>::val(TriggerDefinition::SS_INVOKER); }
break;
case 1069:
  if (!yytrial)
{ yyval.nullableSqlSecurityVal = Nullable<TriggerDefinition::SqlSecurity>::val(TriggerDefinition::SS_DROP); }
break;
case 1070:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 1071:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropExceptionNode>(*yyvsp[0].metaNamePtr); }
break;
case 1072:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropIndexNode>(*yyvsp[0].metaNamePtr); }
break;
case 1073:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropProcedureNode>(*yyvsp[0].metaNamePtr); }
break;
case 1074:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropRelationNode>(*yyvsp[0].metaNamePtr, false); }
break;
case 1075:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropTriggerNode>(*yyvsp[0].metaNamePtr); }
break;
case 1076:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropRelationNode>(*yyvsp[0].metaNamePtr, true); }
break;
case 1077:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropFilterNode>(*yyvsp[0].metaNamePtr); }
break;
case 1078:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropDomainNode>(*yyvsp[0].metaNamePtr); }
break;
case 1079:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 1080:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 1081:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropShadowNode>(yyvsp[-1].int32Val, yyvsp[0].boolVal); }
break;
case 1082:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropRoleNode>(*yyvsp[0].metaNamePtr); }
break;
case 1083:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropSequenceNode>(*yyvsp[0].metaNamePtr); }
break;
case 1084:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropSequenceNode>(*yyvsp[0].metaNamePtr); }
break;
case 1085:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropCollationNode>(*yyvsp[0].metaNamePtr); }
break;
case 1086:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropUserNode>(*yyvsp[-3].metaNamePtr, yyvsp[0].metaNamePtr); }
break;
case 1087:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropUserNode>(*yyvsp[0].metaNamePtr); }
break;
case 1088:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropPackageNode>(*yyvsp[0].metaNamePtr); }
break;
case 1089:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropPackageBodyNode>(*yyvsp[0].metaNamePtr); }
break;
case 1090:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 1091:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 1092:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1093:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1094:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1098:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[-2].legacyField;
			yyval.legacyField->notNull = true;
		}
break;
case 1101:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 1102:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
			yyval.legacyField->typeOfTable = *yyvsp[-2].metaNamePtr;
		}
break;
case 1103:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
			yyval.legacyField->fullDomain = true;
		}
break;
case 1106:
  if (!yytrial)
{
			yyvsp[-3].legacyField->ranges = yyvsp[-1].valueListNode;
			yyvsp[-3].legacyField->dimensions = yyvsp[-3].legacyField->ranges->items.getCount() / 2;
			yyvsp[-3].legacyField->elementDtype = yyvsp[-3].legacyField->dtype;
			yyval.legacyField = yyvsp[-3].legacyField;
		}
break;
case 1107:
  if (!yytrial)
{
			yyvsp[-4].legacyField->ranges = yyvsp[-2].valueListNode;
			yyvsp[-4].legacyField->dimensions = yyvsp[-4].legacyField->ranges->items.getCount() / 2;
			yyvsp[-4].legacyField->elementDtype = yyvsp[-4].legacyField->dtype;
			if (yyvsp[0].metaNamePtr)
				yyvsp[-4].legacyField->charSet = *yyvsp[0].metaNamePtr;
			yyval.legacyField = yyvsp[-4].legacyField;
		}
break;
case 1109:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueListNode->items[0])->add(yyvsp[0].valueListNode->items[1]); }
break;
case 1110:
  if (!yytrial)
{
			if (yyvsp[0].int32Val < 1)
		 		yyval.valueListNode = newNode<ValueListNode>(MAKE_const_slong(yyvsp[0].int32Val))->add(MAKE_const_slong(1));
			else
		 		yyval.valueListNode = newNode<ValueListNode>(MAKE_const_slong(1))->add(MAKE_const_slong(yyvsp[0].int32Val));
		}
break;
case 1111:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(MAKE_const_slong(yyvsp[-2].int32Val))->add(MAKE_const_slong(yyvsp[0].int32Val)); }
break;
case 1113:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[-1].legacyField;
			if (yyvsp[0].metaNamePtr)
			{
				yyval.legacyField->charSet = *yyvsp[0].metaNamePtr;
				yyval.legacyField->flags |= FLD_has_chset;
			}
		}
break;
case 1119:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post (Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
					Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
																	Arg::Str("BIGINT"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post (Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
					Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
																Arg::Str("BIGINT"));
			}

			yyval.legacyField->dtype = dtype_int64;
			yyval.legacyField->length = sizeof(SINT64);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1120:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_int128;
			yyval.legacyField->length = sizeof(Int128);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1121:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_long;
			yyval.legacyField->length = sizeof(SLONG);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1122:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_short;
			yyval.legacyField->length = sizeof(SSHORT);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1123:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			stmt_ambiguous = true;

			if (client_dialect <= SQL_DIALECT_V5)
			{
				/* Post warning saying that DATE is equivalent to TIMESTAMP*/
				ERRD_post_warning(Arg::Warning(isc_sqlwarn) << Arg::Num(301) <<
								  Arg::Warning(isc_dtype_renamed));
				yyval.legacyField->dtype = dtype_timestamp;
				yyval.legacyField->length = sizeof(GDS_TIMESTAMP);
			}
			else if (client_dialect == SQL_DIALECT_V6_TRANSITION)
				yyabandon(YYPOSNARG(1), -104, isc_transitional_date);
			else
			{
				yyval.legacyField->dtype = dtype_sql_date;
				yyval.legacyField->length = sizeof(ULONG);
			}
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1124:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			checkTimeDialect();
			yyval.legacyField->dtype = dtype_sql_time;
			yyval.legacyField->length = sizeof(SLONG);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1125:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			checkTimeDialect();
			yyval.legacyField->dtype = dtype_sql_time_tz;
			yyval.legacyField->length = sizeof(ISC_TIME_TZ);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1126:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_timestamp;
			yyval.legacyField->length = sizeof(GDS_TIMESTAMP);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1127:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_timestamp_tz;
			yyval.legacyField->length = sizeof(ISC_TIMESTAMP_TZ);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1128:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_boolean;
			yyval.legacyField->length = sizeof(UCHAR);
		}
break;
case 1133:
  if (!yytrial)
{ yyval.legacyField = newNode<dsql_fld>(); }
break;
case 1134:
yyval.legacyField = NOTRIAL(yyvsp[0].legacyField);
break;
case 1135:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[-4].legacyField;
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = yyvsp[-1].uintVal;
			if (yyvsp[0].metaNamePtr)
			{
				yyval.legacyField->charSet = *yyvsp[0].metaNamePtr;
				yyval.legacyField->flags |= FLD_has_chset;
			}
		}
break;
case 1136:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->subType = 0;
		}
break;
case 1137:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = (USHORT) yyvsp[-3].int32Val;
			yyval.legacyField->subType = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= FLD_has_sub;
		}
break;
case 1138:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = 80;
			yyval.legacyField->subType = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= FLD_has_sub;
		}
break;
case 1139:
  if (!yytrial)
{ yyval.uintVal = (USHORT) 80; }
break;
case 1140:
  if (!yytrial)
{ yyval.uintVal = (USHORT) yyvsp[0].int32Val; }
break;
case 1141:
  if (!yytrial)
{ yyvsp[0].legacyField->subType = (USHORT) 0; }
break;
case 1142:
  if (!yytrial)
{ yyvsp[-2].legacyField->subType = (USHORT) yyvsp[0].int32Val; yyvsp[-2].legacyField->flags |= FLD_has_sub; }
break;
case 1143:
  if (!yytrial)
{ yyvsp[-2].legacyField->subTypeName = *yyvsp[0].metaNamePtr; yyvsp[-2].legacyField->flags |= FLD_has_sub; }
break;
case 1144:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 1145:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1146:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= (FLD_national | FLD_has_len);
		}
break;
case 1147:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = 1;
			yyval.legacyField->flags |= FLD_national;
		}
break;
case 1148:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_varying;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= (FLD_national | FLD_has_len);
		}
break;
case 1149:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->length = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->textType = ttype_binary;
			yyval.legacyField->charSetId = CS_BINARY;
			yyval.legacyField->subType = fb_text_subtype_binary;
			yyval.legacyField->flags |= (FLD_has_len | FLD_has_chset);
		}
break;
case 1150:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = 1;
			yyval.legacyField->length = 1;
			yyval.legacyField->textType = ttype_binary;
			yyval.legacyField->charSetId = CS_BINARY;
			yyval.legacyField->subType = fb_text_subtype_binary;
			yyval.legacyField->flags |= FLD_has_chset;
		}
break;
case 1151:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_varying;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->length = (USHORT) yyvsp[-1].int32Val + sizeof(USHORT);
			yyval.legacyField->textType = ttype_binary;
			yyval.legacyField->charSetId = CS_BINARY;
			yyval.legacyField->subType = fb_text_subtype_binary;
			yyval.legacyField->flags |= (FLD_has_len | FLD_has_chset);
		}
break;
case 1152:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= FLD_has_len;
		}
break;
case 1153:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = 1;
		}
break;
case 1154:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_varying;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= FLD_has_len;
		}
break;
case 1166:
  if (!yytrial)
{
			SLONG precision = yyvsp[0].int32Val;

			if (precision != 0 && precision != 16 && precision != 34)
				yyabandon(YYPOSNARG(2), -842, isc_decprecision_err);	/* DecFloat precision must be 16 or 34.*/

			yyval.legacyField = newNode<dsql_fld>();
			if (precision)
				yyval.legacyField->flags |= FLD_has_prec;
			yyval.legacyField->precision = precision == 0 ? 34 : (USHORT) precision;
			yyval.legacyField->dtype = precision == 16 ? dtype_dec64 : dtype_dec128;
			yyval.legacyField->length = precision == 16 ? sizeof(Decimal64) : sizeof(Decimal128);
		}
break;
case 1167:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[0].legacyField;
			yyval.legacyField->subType = dsc_num_type_numeric;
			yyval.legacyField->flags |= FLD_has_sub;
		}
break;
case 1168:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[0].legacyField;
			yyval.legacyField->subType = dsc_num_type_decimal;
			yyval.legacyField->flags |= FLD_has_sub;

			if (yyval.legacyField->dtype == dtype_short)
			{
				yyval.legacyField->dtype = dtype_long;
				yyval.legacyField->length = sizeof(SLONG);
			}
		}
break;
case 1169:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_long;
			yyval.legacyField->length = sizeof(SLONG);
			yyval.legacyField->precision = 9;
		}
break;
case 1170:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->flags |= FLD_has_prec;

			if (yyvsp[-1].int32Val < 1 || yyvsp[-1].int32Val > 38)
				yyabandon(YYPOSNARG(2), -842, Arg::Gds(isc_precision_err2) << Arg::Num(1) << Arg::Num(38));
																/* Precision must be between 1 and 38*/

			if (yyvsp[-1].int32Val > 18)
			{
				yyval.legacyField->dtype = dtype_int128;
				yyval.legacyField->length = sizeof(Int128);
			}
			else if (yyvsp[-1].int32Val > 9)
			{
				if ( ( (client_dialect <= SQL_DIALECT_V5) && (db_dialect > SQL_DIALECT_V5) ) ||
					( (client_dialect > SQL_DIALECT_V5) && (db_dialect <= SQL_DIALECT_V5) ) )
				{
					ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-817) <<
							  Arg::Gds(isc_ddl_not_allowed_by_db_sql_dial) << Arg::Num(db_dialect));
				}

				if (client_dialect <= SQL_DIALECT_V5)
				{
					yyval.legacyField->dtype = dtype_double;
					yyval.legacyField->length = sizeof(double);
				}
				else
				{
					if (client_dialect == SQL_DIALECT_V6_TRANSITION)
					{
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous1));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous2));
					}

					yyval.legacyField->dtype = dtype_int64;
					yyval.legacyField->length = sizeof(SINT64);
				}
			}
			else
			{
				if (yyvsp[-1].int32Val < 5)
				{
					yyval.legacyField->dtype = dtype_short;
					yyval.legacyField->length = sizeof(SSHORT);
				}
				else
				{
					yyval.legacyField->dtype = dtype_long;
					yyval.legacyField->length = sizeof(SLONG);
				}
			}

			yyval.legacyField->precision = (USHORT) yyvsp[-1].int32Val;
		}
break;
case 1171:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->flags |= (FLD_has_prec | FLD_has_scale);

			if (yyvsp[-3].int32Val < 1 || yyvsp[-3].int32Val > 38)
				yyabandon(YYPOSNARG(2), -842, Arg::Gds(isc_precision_err2) << Arg::Num(1) << Arg::Num(38));
																/* Precision must be between 1 and 38*/

			if (yyvsp[-1].int32Val > yyvsp[-3].int32Val || yyvsp[-1].int32Val < 0)
				yyabandon(YYPOSNARG(4), -842, isc_scale_nogt);	/* Scale must be between 0 and precision*/

			if (yyvsp[-3].int32Val > 18)
			{
				yyval.legacyField->dtype = dtype_int128;
				yyval.legacyField->length = sizeof(Int128);
			}
			else if (yyvsp[-3].int32Val > 9)
			{
				if ( ( (client_dialect <= SQL_DIALECT_V5) && (db_dialect > SQL_DIALECT_V5) ) ||
					( (client_dialect > SQL_DIALECT_V5) && (db_dialect <= SQL_DIALECT_V5) ) )
				{
					ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-817) <<
							  Arg::Gds(isc_ddl_not_allowed_by_db_sql_dial) << Arg::Num(db_dialect));
				}

				if (client_dialect <= SQL_DIALECT_V5)
				{
					yyval.legacyField->dtype = dtype_double;
					yyval.legacyField->length = sizeof(double);
				}
				else
				{
					if (client_dialect == SQL_DIALECT_V6_TRANSITION)
					{
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous1));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous2));
					}
					/* client_dialect >= SQL_DIALECT_V6*/
					yyval.legacyField->dtype = dtype_int64;
					yyval.legacyField->length = sizeof(SINT64);
				}
			}
			else
			{
				if (yyvsp[-3].int32Val < 5)
				{
					yyval.legacyField->dtype = dtype_short;
					yyval.legacyField->length = sizeof(SSHORT);
				}
				else
				{
					yyval.legacyField->dtype = dtype_long;
					yyval.legacyField->length = sizeof(SLONG);
				}
			}

			yyval.legacyField->precision = (USHORT) yyvsp[-3].int32Val;
			yyval.legacyField->scale = - (SSHORT) yyvsp[-1].int32Val;
		}
break;
case 1174:
  if (!yytrial)
{
		    /* Precision is binary digits of the significand: 1-24 for 32 bit single precision, 25-53 for 64 bit double precision*/
			/* Precision 0 is the 'no precision specified' case, which defaults to 32 bit single precision*/
			SLONG precision = yyvsp[0].int32Val;

			if (precision != 0 && (precision < 1 || precision > 53))
				yyabandon(YYPOSNARG(2), -842, Arg::Gds(isc_precision_err2) << Arg::Num(1) << Arg::Num(53));
																/* Precision must be between 1 and 53*/

			yyval.legacyField = newNode<dsql_fld>();

			if (precision > 24)
			{
				yyval.legacyField->dtype = dtype_double;
				yyval.legacyField->length = sizeof(double);
			}
			else
			{
				yyval.legacyField->dtype = dtype_real;
				yyval.legacyField->length = sizeof(float);
			}
		}
break;
case 1175:
  if (!yytrial)
{
			/* Precision is binary digits of the significand: 1-53 for 64 bit double precision*/
			/* Precision 0 is the 'no precision specified case', which defaults to 64 bit double precision*/
			SLONG precision = yyvsp[0].int32Val;

			if (precision != 0 && (precision < 1 || precision > 53))
				yyabandon(YYPOSNARG(3), -842, Arg::Gds(isc_precision_err2) << Arg::Num(1) << Arg::Num(53));
																/* Precision must be between 1 and 53*/

			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_double;
			yyval.legacyField->length = sizeof(double);
		}
break;
case 1176:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_real;
			yyval.legacyField->length = sizeof(float);
		}
break;
case 1177:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_double;
			yyval.legacyField->length = sizeof(double);
		}
break;
case 1178:
  if (!yytrial)
{ yyval.int32Val = 0; }
break;
case 1179:
  if (!yytrial)
{ yyval.int32Val = yyvsp[-1].int32Val; }
break;
case 1183:
  if (!yytrial)
{ yyval.traNode = newNode<UserSavepointNode>(UserSavepointNode::CMD_SET, *yyvsp[0].metaNamePtr); }
break;
case 1184:
  if (!yytrial)
{ yyval.traNode = newNode<UserSavepointNode>(UserSavepointNode::CMD_RELEASE, *yyvsp[0].metaNamePtr); }
break;
case 1185:
  if (!yytrial)
{ yyval.traNode = newNode<UserSavepointNode>(UserSavepointNode::CMD_RELEASE_ONLY, *yyvsp[-1].metaNamePtr); }
break;
case 1186:
  if (!yytrial)
{ yyval.traNode = newNode<UserSavepointNode>(UserSavepointNode::CMD_ROLLBACK, *yyvsp[0].metaNamePtr); }
break;
case 1189:
  if (!yytrial)
{ yyval.traNode = newNode<CommitRollbackNode>(CommitRollbackNode::CMD_COMMIT, yyvsp[0].boolVal); }
break;
case 1190:
  if (!yytrial)
{ yyval.traNode = newNode<CommitRollbackNode>(CommitRollbackNode::CMD_ROLLBACK, yyvsp[0].boolVal); }
break;
case 1193:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1194:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1197:
  if (!yytrial)
{ yyval.setTransactionNode = newNode<SetTransactionNode>(); }
break;
case 1198:
yyval.setTransactionNode = yyvsp[0].setTransactionNode;
break;
case 1199:
  if (!yytrial)
{ yyval.setTransactionNode = yyvsp[-2].setTransactionNode; }
break;
case 1200:
  if (!yytrial)
{ yyval.sessionResetNode = newNode<SessionResetNode>(); }
break;
case 1201:
  if (!yytrial)
{ yyval.setRoleNode = newNode<SetRoleNode>(yyvsp[0].metaNamePtr); }
break;
case 1202:
  if (!yytrial)
{ yyval.setRoleNode = newNode<SetRoleNode>(); }
break;
case 1203:
  if (!yytrial)
{ yyval.mngNode = newNode<SetDebugOptionNode>(yyvsp[-2].metaNamePtr, yyvsp[0].valueExprNode); }
break;
case 1204:
  if (!yytrial)
{ yyval.setDecFloatRoundNode = newNode<SetDecFloatRoundNode>(yyvsp[0].metaNamePtr); }
break;
case 1205:
  if (!yytrial)
{ yyval.setDecFloatTrapsNode = newNode<SetDecFloatTrapsNode>(); }
break;
case 1206:
yyval.setDecFloatTrapsNode = yyvsp[0].setDecFloatTrapsNode;
break;
case 1207:
  if (!yytrial)
{ yyval.setDecFloatTrapsNode = yyvsp[-2].setDecFloatTrapsNode; }
break;
case 1208:
  if (!yytrial)
{ yyval.setBindNode = newNode<SetBindNode>(); yyval.setBindNode->from = yyvsp[-2].legacyField; yyval.setBindNode->to = yyvsp[0].legacyField; }
break;
case 1210:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_timestamp_tz;
			yyval.legacyField->length = 0;
		}
break;
case 1212:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_varying;
			yyval.legacyField->charLength = 0;
		}
break;
case 1213:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[0].legacyField;
		}
break;
case 1214:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->flags = FLD_legacy;
		}
break;
case 1215:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->flags = FLD_native;
		}
break;
case 1216:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->flags = FLD_extended;
		}
break;
case 1217:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			checkTimeDialect();
			yyval.legacyField->dtype = dtype_ex_time_tz;
			yyval.legacyField->length = sizeof(ISC_TIME_TZ_EX);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1218:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_ex_timestamp_tz;
			yyval.legacyField->length = sizeof(ISC_TIMESTAMP_TZ_EX);
			yyval.legacyField->flags |= FLD_has_prec;
		}
break;
case 1222:
yyval.setDecFloatTrapsNode = yyvsp[-2].setDecFloatTrapsNode;
break;
case 1224:
  if (!yytrial)
{ yyvsp[-1].setDecFloatTrapsNode->trap(yyvsp[0].metaNamePtr); }
break;
case 1225:
  if (!yytrial)
{ yyval.mngNode = newNode<SetOptimizeNode>(yyvsp[0].boolVal); }
break;
case 1226:
  if (!yytrial)
{ yyval.mngNode = newNode<SetOptimizeNode>(); }
break;
case 1227:
  if (!yytrial)
{ yyval.setSessionNode = newNode<SetSessionNode>(SetSessionNode::TYPE_IDLE_TIMEOUT, yyvsp[-1].int32Val, yyvsp[0].blrOp); }
break;
case 1228:
  if (!yytrial)
{ yyval.setSessionNode = newNode<SetSessionNode>(SetSessionNode::TYPE_STMT_TIMEOUT, yyvsp[-1].int32Val, yyvsp[0].blrOp); }
break;
case 1229:
  if (!yytrial)
{ yyval.blrOp = blr_extract_minute; }
break;
case 1230:
  if (!yytrial)
{ yyval.blrOp = blr_extract_hour; }
break;
case 1231:
  if (!yytrial)
{ yyval.blrOp = blr_extract_minute; }
break;
case 1232:
  if (!yytrial)
{ yyval.blrOp = blr_extract_second; }
break;
case 1233:
  if (!yytrial)
{ yyval.blrOp = blr_extract_second; }
break;
case 1234:
  if (!yytrial)
{ yyval.blrOp = blr_extract_hour; }
break;
case 1235:
  if (!yytrial)
{ yyval.blrOp = blr_extract_minute; }
break;
case 1236:
  if (!yytrial)
{ yyval.blrOp = blr_extract_second; }
break;
case 1237:
  if (!yytrial)
{ yyval.blrOp = blr_extract_millisecond; }
break;
case 1238:
  if (!yytrial)
{ yyval.mngNode = yyvsp[0].mngNode; }
break;
case 1239:
  if (!yytrial)
{ yyval.mngNode = newNode<SetTimeZoneNode>(yyvsp[0].intlStringPtr->getString()); }
break;
case 1240:
  if (!yytrial)
{ yyval.mngNode = newNode<SetTimeZoneNode>(); }
break;
case 1244:
yyval.setTransactionNode = yyvsp[-1].setTransactionNode;
break;
case 1246:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->readOnly, "READ {ONLY | WRITE}", true); }
break;
case 1247:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->readOnly, "READ {ONLY | WRITE}", false); }
break;
case 1248:
  if (!yytrial)
{ setClause(yyvsp[-1].setTransactionNode->wait, "[NO] WAIT", true); }
break;
case 1249:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->wait, "[NO] WAIT", false); }
break;
case 1250:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->isoLevel, "ISOLATION LEVEL", yyvsp[0].uintVal); }
break;
case 1251:
  if (!yytrial)
{ setClause(yyvsp[-3].setTransactionNode->noAutoUndo, "NO AUTO UNDO", true); }
break;
case 1252:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->ignoreLimbo, "IGNORE LIMBO", true); }
break;
case 1253:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->restartRequests, "RESTART REQUESTS", true); }
break;
case 1254:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->autoCommit, "AUTO COMMIT", true); }
break;
case 1255:
  if (!yytrial)
{ setClause(yyvsp[-3].setTransactionNode->lockTimeout, "LOCK TIMEOUT", (USHORT) yyvsp[0].int32Val); }
break;
case 1256:
  if (!yytrial)
{ checkDuplicateClause(yyvsp[-1].setTransactionNode->reserveList, "RESERVING"); }
break;
case 1257:
yyval.setTransactionNode = yyvsp[-2].setTransactionNode;
break;
case 1259:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal;}
break;
case 1261:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal; }
break;
case 1262:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal; }
break;
case 1263:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal; }
break;
case 1264:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_CONCURRENCY; }
break;
case 1265:
  if (!yytrial)
{
			setClause(yyvsp[-4].setTransactionNode->atSnapshotNumber, "SNAPSHOT AT NUMBER", (CommitNumber) yyvsp[0].int64Val);
			yyval.uintVal = SetTransactionNode::ISO_LEVEL_CONCURRENCY;
		}
break;
case 1266:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_CONSISTENCY; }
break;
case 1267:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_CONSISTENCY; }
break;
case 1268:
  if (!yytrial)
{ yyval.int64Val = yyvsp[0].int32Val; }
break;
case 1269:
  if (!yytrial)
{ yyval.int64Val = yyvsp[0].scaledNumber.number; }
break;
case 1270:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_READ_COMMITTED_NO_REC_VERSION; }
break;
case 1271:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_READ_COMMITTED_REC_VERSION; }
break;
case 1272:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_READ_COMMITTED_NO_REC_VERSION; }
break;
case 1273:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_READ_COMMITTED_READ_CONSISTENCY; }
break;
case 1274:
  if (!yytrial)
{ yyval.uintVal = 0; }
break;
case 1275:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_SHARED; }
break;
case 1276:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_PROTECTED; }
break;
case 1277:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_READ; }
break;
case 1278:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_WRITE; }
break;
case 1279:
  if (!yytrial)
{ yyvsp[-1].setTransactionNode->reserveList.add(yyvsp[0].setTransactionRestrictionClause); }
break;
case 1280:
  if (!yytrial)
{ yyvsp[-3].setTransactionNode->reserveList.add(yyvsp[0].setTransactionRestrictionClause); }
break;
case 1281:
  if (!yytrial)
{ yyval.setTransactionRestrictionClause = newNode<SetTransactionNode::RestrictionOption>(yyvsp[-1].metaNameArray, yyvsp[0].uintVal); }
break;
case 1282:
  if (!yytrial)
{ yyval.uintVal = 0; }
break;
case 1283:
  if (!yytrial)
{ yyval.uintVal = yyvsp[-1].uintVal | yyvsp[0].uintVal; }
break;
case 1284:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = newNode<ObjectsArray<MetaName> >();
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1285:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = yyvsp[-2].metaNameArray;
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1286:
  if (!yytrial)
{ yyval.ddlNode = newNode<SetStatisticsNode>(*yyvsp[0].metaNamePtr); }
break;
case 1287:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-2].intVal, QualifiedName(""), "", *yyvsp[0].stringPtr); }
break;
case 1288:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-3].intVal, QualifiedName(*yyvsp[-2].metaNamePtr), "", *yyvsp[0].stringPtr); }
break;
case 1289:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-4].intVal, QualifiedName(*yyvsp[-3].metaNamePtr), *yyvsp[-2].metaNamePtr, *yyvsp[0].stringPtr); }
break;
case 1290:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-4].intVal, *yyvsp[-3].qualifiedNamePtr, *yyvsp[-2].metaNamePtr, *yyvsp[0].stringPtr); }
break;
case 1291:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-3].intVal, *yyvsp[-2].qualifiedNamePtr, "", *yyvsp[0].stringPtr); }
break;
case 1292:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 1293:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 1294:
  if (!yytrial)
{
				yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_MOD, *yyvsp[0].metaNamePtr);
			}
break;
case 1295:
yyval.createAlterUserNode = yyvsp[0].createAlterUserNode;
break;
case 1296:
  if (!yytrial)
{
				CreateAlterUserNode* node = yyval.createAlterUserNode = yyvsp[-4].createAlterUserNode;
				node->comment = yyvsp[0].stringPtr;
			}
break;
case 1299:
  if (!yytrial)
{ yyval.intVal = obj_database; }
break;
case 1300:
  if (!yytrial)
{ yyval.intVal = obj_field; }
break;
case 1301:
  if (!yytrial)
{ yyval.intVal = obj_relation; }
break;
case 1302:
  if (!yytrial)
{ yyval.intVal = obj_view; }
break;
case 1303:
  if (!yytrial)
{ yyval.intVal = obj_trigger; }
break;
case 1304:
  if (!yytrial)
{ yyval.intVal = obj_blob_filter; }
break;
case 1305:
  if (!yytrial)
{ yyval.intVal = obj_exception; }
break;
case 1306:
  if (!yytrial)
{ yyval.intVal = obj_generator; }
break;
case 1307:
  if (!yytrial)
{ yyval.intVal = obj_generator; }
break;
case 1308:
  if (!yytrial)
{ yyval.intVal = obj_index; }
break;
case 1309:
  if (!yytrial)
{ yyval.intVal = obj_sql_role; }
break;
case 1310:
  if (!yytrial)
{ yyval.intVal = obj_charset; }
break;
case 1311:
  if (!yytrial)
{ yyval.intVal = obj_collation; }
break;
case 1312:
  if (!yytrial)
{ yyval.intVal = obj_package_header; }
break;
case 1313:
  if (!yytrial)
{ yyval.intVal = obj_relation; }
break;
case 1314:
  if (!yytrial)
{ yyval.intVal = obj_parameter; }
break;
case 1315:
  if (!yytrial)
{ yyval.intVal = obj_procedure; }
break;
case 1316:
  if (!yytrial)
{ yyval.intVal = obj_udf; }
break;
case 1317:
  if (!yytrial)
{ yyval.intVal = obj_procedure; }
break;
case 1318:
  if (!yytrial)
{ yyval.intVal = obj_udf; }
break;
case 1319:
  if (!yytrial)
{ yyval.intVal = obj_udf; }
break;
case 1320:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1321:
  if (!yytrial)
{ yyval.qualifiedNamePtr = newNode<QualifiedName>(*yyvsp[0].metaNamePtr); }
break;
case 1322:
  if (!yytrial)
{ yyval.qualifiedNamePtr = newNode<QualifiedName>(*yyvsp[0].metaNamePtr, *yyvsp[-2].metaNamePtr); }
break;
case 1323:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 1324:
  if (!yytrial)
{ yyval.stringPtr = newString(""); }
break;
case 1325:
  if (!yytrial)
{
			SelectNode* node = newNode<SelectNode>();
			node->dsqlExpr = yyvsp[-3].selectExprNode;
			node->dsqlForUpdate = yyvsp[-2].boolVal;
			node->dsqlWithLock = yyvsp[-1].boolPair.first;
			node->dsqlSkipLocked = yyvsp[-1].boolPair.second;
			node->dsqlOptimizeForFirstRows = yyvsp[0].nullableBoolVal;
			yyval.selectNode = node;
		}
break;
case 1326:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1327:
  if (!yytrial)
{ yyval.boolVal = true; /* for_update_list is ignored */ }
break;
case 1328:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1329:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1330:
  if (!yytrial)
{ yyval.boolPair = {false, false}; }
break;
case 1331:
  if (!yytrial)
{ yyval.boolPair = {true, yyvsp[0].boolVal}; }
break;
case 1332:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1333:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1334:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::val(yyvsp[0].boolVal); }
break;
case 1335:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::empty(); }
break;
case 1336:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1337:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1338:
  if (!yytrial)
{
			SelectExprNode* node = yyval.selectExprNode = newNode<SelectExprNode>();
			node->querySpec = yyvsp[-2].recSourceNode;
			node->orderClause = yyvsp[-1].valueListNode;
			node->rowsClause = yyvsp[0].rowsClause;
			node->withClause = yyvsp[-3].withClause;
		}
break;
case 1339:
  if (!yytrial)
{
			SelectExprNode* node = yyval.selectExprNode = newNode<SelectExprNode>();
			node->querySpec = yyvsp[-3].recSourceNode;
			node->orderClause = yyvsp[-2].valueListNode;

			if (yyvsp[-1].valueExprNode || yyvsp[0].valueExprNode)
			{
				RowsClause* rowsNode = newNode<RowsClause>();
				rowsNode->skip = yyvsp[-1].valueExprNode;
				rowsNode->length = yyvsp[0].valueExprNode;
				node->rowsClause = rowsNode;
			}

			node->withClause = yyvsp[-4].withClause;
		}
break;
case 1340:
  if (!yytrial)
{ yyval.withClause = NULL; }
break;
case 1341:
  if (!yytrial)
{
			yyval.withClause = yyvsp[0].withClause;
			yyval.withClause->recursive = true;
		}
break;
case 1342:
  if (!yytrial)
{ yyval.withClause = yyvsp[0].withClause; }
break;
case 1343:
  if (!yytrial)
{
			yyval.withClause = newNode<WithClause>();
			yyval.withClause->add(yyvsp[0].selectExprNode);
		}
break;
case 1344:
  if (!yytrial)
{
			yyval.withClause = yyvsp[-2].withClause;
			yyval.withClause->add(yyvsp[0].selectExprNode);
		}
break;
case 1345:
  if (!yytrial)
{
			yyval.selectExprNode = yyvsp[-1].selectExprNode;
			yyval.selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_DERIVED;
			yyval.selectExprNode->alias = yyvsp[-5].metaNamePtr->c_str();
			yyval.selectExprNode->columns = yyvsp[-4].metaNameArray;
		}
break;
case 1346:
  if (!yytrial)
{
			yyval.selectExprNode = yyvsp[0].selectExprNode;
			yyval.selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_VALUE;
		}
break;
case 1347:
  if (!yytrial)
{
			yyvsp[0].selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_SINGLETON;
			yyval.valueExprNode = newNode<SubQueryNode>(blr_via, yyvsp[0].selectExprNode);
		}
break;
case 1348:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[0].recSourceNode; }
break;
case 1349:
  if (!yytrial)
{
			UnionSourceNode* node = nodeAs<UnionSourceNode>(yyvsp[-3].recSourceNode);
			if (node && !node->dsqlAll)
				node->dsqlClauses->add(yyvsp[0].recSourceNode);
			else
			{
				node = newNode<UnionSourceNode>();
				node->dsqlClauses = newNode<RecSourceListNode>(yyvsp[-3].recSourceNode)->add(yyvsp[0].recSourceNode);
			}
			yyval.recSourceNode = node;
		}
break;
case 1350:
  if (!yytrial)
{
			UnionSourceNode* node = nodeAs<UnionSourceNode>(yyvsp[-3].recSourceNode);
			if (node && node->dsqlAll)
				node->dsqlClauses->add(yyvsp[0].recSourceNode);
			else
			{
				node = newNode<UnionSourceNode>();
				node->dsqlAll = true;
				node->dsqlClauses = newNode<RecSourceListNode>(yyvsp[-3].recSourceNode)->add(yyvsp[0].recSourceNode);
			}
			yyval.recSourceNode = node;
		}
break;
case 1352:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[0].rseNode; }
break;
case 1353:
  if (!yytrial)
{
			if (yyvsp[-3].valueListNode || yyvsp[-2].valueExprNode || yyvsp[-1].valueExprNode)
			{
				const auto selectExpr = newNode<SelectExprNode>();
				selectExpr->dsqlFlags |= RecordSourceNode::DFLAG_DERIVED;
				selectExpr->querySpec = yyvsp[-4].recSourceNode;
				selectExpr->orderClause = yyvsp[-3].valueListNode;

				if (yyvsp[-2].valueExprNode || yyvsp[-1].valueExprNode)
				{
					const auto rowsNode = newNode<RowsClause>();
					rowsNode->skip = yyvsp[-2].valueExprNode;
					rowsNode->length = yyvsp[-1].valueExprNode;
					selectExpr->rowsClause = rowsNode;
				}

				const auto rse = newNode<RseNode>();
				rse->dsqlFlags |= RecordSourceNode::DFLAG_BODY_WRAPPER;
				rse->dsqlFrom = newNode<RecSourceListNode>(selectExpr);
				yyval.recSourceNode = rse;
			}
			else
				yyval.recSourceNode = yyvsp[-4].recSourceNode;
		}
break;
case 1354:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlFirst = yyvsp[-8].valueListNode ? yyvsp[-8].valueListNode->items[1] : NULL;
			rse->dsqlSkip = yyvsp[-8].valueListNode ? yyvsp[-8].valueListNode->items[0] : NULL;
			rse->dsqlDistinct = yyvsp[-7].valueListNode;
			rse->dsqlSelectList = yyvsp[-6].valueListNode->items.hasData() ? yyvsp[-6].valueListNode : nullptr;
			rse->dsqlFrom = yyvsp[-5].recSourceListNode;
			rse->dsqlWhere = yyvsp[-4].boolExprNode;
			rse->dsqlGroup = yyvsp[-3].valueListNode;
			rse->dsqlHaving = yyvsp[-2].boolExprNode;
			rse->dsqlNamedWindows = yyvsp[-1].namedWindowsClause;
			rse->rse_plan = yyvsp[0].planNode;
			yyval.rseNode = rse;
		}
break;
case 1355:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1356:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode)->add(yyvsp[-1].valueExprNode); }
break;
case 1357:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(1u)->add(yyvsp[0].valueExprNode); }
break;
case 1358:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode)->add(NULL); }
break;
case 1359:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(yyvsp[0].int32Val); }
break;
case 1360:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1361:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1362:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(yyvsp[0].int32Val); }
break;
case 1363:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1364:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1365:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(0u); }
break;
case 1366:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1367:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1368:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(0u); }
break;
case 1369:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1370:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1372:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			yyval.valueExprNode = fieldNode;
		}
break;
case 1374:
  if (!yytrial)
{ yyval.valueExprNode = newNode<DsqlAliasNode>(*yyvsp[0].metaNamePtr, yyvsp[-2].valueExprNode); }
break;
case 1377:
  if (!yytrial)
{ yyval.recSourceListNode = yyvsp[0].recSourceListNode; }
break;
case 1378:
  if (!yytrial)
{ yyval.recSourceListNode = newNode<RecSourceListNode>(yyvsp[0].recSourceNode); }
break;
case 1379:
  if (!yytrial)
{ yyval.recSourceListNode = yyvsp[-2].recSourceListNode->add(yyvsp[0].recSourceNode); }
break;
case 1383:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[0].selectExprNode; }
break;
case 1384:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[0].selectExprNode; }
break;
case 1385:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[0].recSourceNode; }
break;
case 1386:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[-1].recSourceNode; }
break;
case 1387:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[-1].recSourceNode; }
break;
case 1388:
  if (!yytrial)
{
			yyval.selectExprNode = yyvsp[-3].selectExprNode;
			yyval.selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_DERIVED;
			if (yyvsp[-1].metaNamePtr)
				yyval.selectExprNode->alias = yyvsp[-1].metaNamePtr->c_str();
			yyval.selectExprNode->columns = yyvsp[0].metaNameArray;
		}
break;
case 1389:
  if (!yytrial)
{
			yyval.selectExprNode = yyvsp[0].selectExprNode;
			yyval.selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_LATERAL;
		}
break;
case 1390:
  if (!yytrial)
{ yyval.metaNamePtr = nullptr; }
break;
case 1392:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1393:
  if (!yytrial)
{ yyval.metaNameArray = NULL; }
break;
case 1394:
  if (!yytrial)
{ yyval.metaNameArray = yyvsp[-1].metaNameArray; }
break;
case 1395:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = newNode<ObjectsArray<MetaName> >();
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1396:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = yyvsp[-2].metaNameArray;
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1400:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = blr_inner;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-3].recSourceNode)->add(yyvsp[0].recSourceNode);
			yyval.recSourceNode = rse;
		}
break;
case 1401:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = yyvsp[-2].blrOp;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-4].recSourceNode)->add(yyvsp[0].recSourceNode);
			rse->dsqlJoinUsing = newNode<ValueListNode>(0u);	/* using list with size 0 -> natural*/
			yyval.recSourceNode = rse;
		}
break;
case 1402:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = yyvsp[-3].blrOp;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-4].recSourceNode);
			rse->dsqlFrom->add(yyvsp[-1].recSourceNode);
			rse->dsqlWhere = yyvsp[0].boolExprNode;
			yyval.recSourceNode = rse;
		}
break;
case 1403:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = yyvsp[-3].blrOp;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-4].recSourceNode);
			rse->dsqlFrom->add(yyvsp[-1].recSourceNode);
			rse->dsqlJoinUsing = yyvsp[0].valueListNode;
			yyval.recSourceNode = rse;
		}
break;
case 1404:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[0].boolExprNode; }
break;
case 1405:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 1406:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(QualifiedName(*yyvsp[-3].metaNamePtr));
			node->sourceList = yyvsp[-2].valueListNode;
			node->alias = yyvsp[0].metaNamePtr->c_str();
			yyval.recSourceNode = node;
		}
break;
case 1407:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(QualifiedName(*yyvsp[-1].metaNamePtr));
			node->sourceList = yyvsp[0].valueListNode;
			yyval.recSourceNode = node;
		}
break;
case 1408:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(
				QualifiedName(*yyvsp[-3].metaNamePtr, *yyvsp[-5].metaNamePtr));
			node->sourceList = yyvsp[-2].valueListNode;
			node->alias = yyvsp[0].metaNamePtr->c_str();
			yyval.recSourceNode = node;
		}
break;
case 1409:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(
				QualifiedName(*yyvsp[-1].metaNamePtr, *yyvsp[-3].metaNamePtr));
			node->sourceList = yyvsp[0].valueListNode;
			yyval.recSourceNode = node;
		}
break;
case 1410:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1411:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 1413:
  if (!yytrial)
{
			RelationSourceNode* node = newNode<RelationSourceNode>(*yyvsp[-2].metaNamePtr);
			node->alias = yyvsp[0].metaNamePtr->c_str();
			yyval.relSourceNode = node;
		}
break;
case 1414:
  if (!yytrial)
{ yyval.relSourceNode = newNode<RelationSourceNode>(*yyvsp[0].metaNamePtr); }
break;
case 1415:
  if (!yytrial)
{ yyval.blrOp = blr_inner; }
break;
case 1416:
  if (!yytrial)
{ yyval.blrOp = blr_inner; }
break;
case 1417:
  if (!yytrial)
{ yyval.blrOp = blr_left; }
break;
case 1418:
  if (!yytrial)
{ yyval.blrOp = blr_right; }
break;
case 1419:
  if (!yytrial)
{ yyval.blrOp = blr_full; }
break;
case 1422:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1423:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1424:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1425:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1427:
  if (!yytrial)
{ yyval.boolExprNode = NULL; }
break;
case 1428:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[0].boolExprNode; }
break;
case 1429:
  if (!yytrial)
{ yyval.boolExprNode = NULL; }
break;
case 1430:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[0].boolExprNode; }
break;
case 1431:
  if (!yytrial)
{ yyval.namedWindowsClause = NULL; }
break;
case 1432:
  if (!yytrial)
{ yyval.namedWindowsClause = yyvsp[0].namedWindowsClause; }
break;
case 1433:
  if (!yytrial)
{
			NamedWindowsClause* node = newNode<NamedWindowsClause>();
			node->add(*yyvsp[0].namedWindowClause);
			yyval.namedWindowsClause = node;
		}
break;
case 1434:
  if (!yytrial)
{
			NamedWindowsClause* node = yyvsp[-2].namedWindowsClause;
			node->add(*yyvsp[0].namedWindowClause);
			yyval.namedWindowsClause = node;
		}
break;
case 1435:
  if (!yytrial)
{
			yyval.namedWindowClause = newNode<NamedWindowClause>(*yyvsp[-4].metaNamePtr, yyvsp[-1].windowClause);
		}
break;
case 1436:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 1438:
  if (!yytrial)
{ yyval.planNode = NULL; }
break;
case 1439:
  if (!yytrial)
{ yyval.planNode = yyvsp[0].planNode; }
break;
case 1440:
  if (!yytrial)
{ yyval.planNode = newNode<PlanNode>(PlanNode::TYPE_RETRIEVE); }
break;
case 1441:
yyval.planNode = yyvsp[-1].planNode;
break;
case 1442:
  if (!yytrial)
{ yyval.planNode = yyvsp[-4].planNode; }
break;
case 1449:
  if (!yytrial)
{ yyvsp[-1].planNode->subNodes.add(yyvsp[0].planNode); }
break;
case 1450:
  if (!yytrial)
{ yyvsp[-3].planNode->subNodes.add(yyvsp[0].planNode); }
break;
case 1451:
  if (!yytrial)
{
			yyval.planNode = newNode<PlanNode>(PlanNode::TYPE_RETRIEVE);
			yyval.planNode->dsqlNames = yyvsp[-1].metaNameArray;
			yyval.planNode->accessType = yyvsp[0].accessType;
		}
break;
case 1453:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = newNode<ObjectsArray<MetaName> >();
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1454:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = yyvsp[-1].metaNameArray;
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1455:
  if (!yytrial)
{ yyval.accessType = newNode<PlanNode::AccessType>(PlanNode::AccessType::TYPE_SEQUENTIAL); }
break;
case 1456:
  if (!yytrial)
{ yyval.accessType = newNode<PlanNode::AccessType>(PlanNode::AccessType::TYPE_INDICES); }
break;
case 1457:
yyval.accessType = yyvsp[-1].accessType;
break;
case 1458:
  if (!yytrial)
{ yyval.accessType = yyvsp[-4].accessType; }
break;
case 1459:
  if (!yytrial)
{ yyval.accessType = newNode<PlanNode::AccessType>(PlanNode::AccessType::TYPE_NAVIGATIONAL); }
break;
case 1460:
  if (!yytrial)
{
			yyval.accessType = yyvsp[-3].accessType;
			yyval.accessType->items.insert(0).indexName = *yyvsp[-2].metaNamePtr;
		}
break;
case 1461:
  if (!yytrial)
{
			PlanNode::AccessItem& item = yyvsp[-1].accessType->items.add();
			item.indexName = *yyvsp[0].metaNamePtr;
		}
break;
case 1462:
  if (!yytrial)
{
			PlanNode::AccessItem& item = yyvsp[-3].accessType->items.add();
			item.indexName = *yyvsp[0].metaNamePtr;
		}
break;
case 1464:
yyval.accessType = yyvsp[-2].accessType;
break;
case 1466:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1468:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1469:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1470:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1471:
  if (!yytrial)
{
			OrderNode* node = newNode<OrderNode>(yyvsp[-2].valueExprNode);
			node->descending = yyvsp[-1].boolVal;
			node->nullsPlacement = yyvsp[0].nullsPlacement;
			yyval.valueExprNode = node;
		}
break;
case 1472:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1473:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1474:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1475:
  if (!yytrial)
{ yyval.nullsPlacement = OrderNode::NULLS_DEFAULT; }
break;
case 1476:
  if (!yytrial)
{ yyval.nullsPlacement = yyvsp[0].nullsPlacement; }
break;
case 1477:
  if (!yytrial)
{ yyval.nullsPlacement = OrderNode::NULLS_FIRST; }
break;
case 1478:
  if (!yytrial)
{ yyval.nullsPlacement = OrderNode::NULLS_LAST; }
break;
case 1479:
  if (!yytrial)
{
			yyval.rowsClause = newNode<RowsClause>();
			yyval.rowsClause->length = yyvsp[0].valueExprNode;
		}
break;
case 1480:
  if (!yytrial)
{
			yyval.rowsClause = newNode<RowsClause>();
			yyval.rowsClause->skip = newNode<ArithmeticNode>(blr_subtract, true, yyvsp[-2].valueExprNode, MAKE_const_slong(1));
			yyval.rowsClause->length = newNode<ArithmeticNode>(blr_add, true,
				newNode<ArithmeticNode>(blr_subtract, true, yyvsp[0].valueExprNode, yyvsp[-2].valueExprNode), MAKE_const_slong(1));
		}
break;
case 1481:
  if (!yytrial)
{ yyval.rowsClause = NULL; }
break;
case 1485:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 1486:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1489:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 1490:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-2].valueExprNode; }
break;
case 1491:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(1); }
break;
case 1492:
yyval.nestFieldArray = NOTRIAL(&yyvsp[0].storeNode->dsqlFields);
break;
case 1493:
  if (!yytrial)
{
			StoreNode* node = yyval.storeNode = yyvsp[-8].storeNode;
			node->overrideClause = yyvsp[-5].nullableOverrideClause;
			node->dsqlValues = yyvsp[-2].valueListNode;
			node->dsqlReturning = yyvsp[0].returningClause;
		}
break;
case 1494:
  if (!yytrial)
{
			StoreNode* node = yyval.storeNode = yyvsp[-5].storeNode;
			node->overrideClause = yyvsp[-2].nullableOverrideClause;
			node->dsqlRse = yyvsp[-1].selectExprNode;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.storeNode = node;
		}
break;
case 1495:
  if (!yytrial)
{
			StoreNode* node = yyval.storeNode = yyvsp[-3].storeNode;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.storeNode = node;
		}
break;
case 1496:
  if (!yytrial)
{
			StoreNode* node = newNode<StoreNode>();
			node->target = yyvsp[0].relSourceNode;
			yyval.storeNode = node;
		}
break;
case 1497:
  if (!yytrial)
{ yyval.nullableOverrideClause = Nullable<OverrideClause>::empty(); }
break;
case 1498:
  if (!yytrial)
{ yyval.nullableOverrideClause = Nullable<OverrideClause>::val(OverrideClause::USER_VALUE); }
break;
case 1499:
  if (!yytrial)
{ yyval.nullableOverrideClause = Nullable<OverrideClause>::val(OverrideClause::SYSTEM_VALUE); }
break;
case 1500:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1501:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1503:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 1504:
  if (!yytrial)
{
				MergeNode* node = yyval.mergeNode = newNode<MergeNode>();
				node->relation = yyvsp[-4].relSourceNode;
				node->usingClause = yyvsp[-2].recSourceNode;
				node->condition = yyvsp[0].boolExprNode;
			}
break;
case 1505:
yyval.mergeNode = yyvsp[0].mergeNode;
break;
case 1506:
  if (!yytrial)
{
				MergeNode* node = yyval.mergeNode = yyvsp[-5].mergeNode;
				node->plan = yyvsp[-2].planNode;
				node->order = yyvsp[-1].valueListNode;
				node->returning = yyvsp[0].returningClause;
			}
break;
case 1509:
yyval.mergeNode = yyvsp[-1].mergeNode;
break;
case 1512:
  if (!yytrial)
{ yyval.mergeMatchedClause = &yyvsp[-2].mergeNode->whenMatched.add(); }
break;
case 1513:
yyval.mergeMatchedClause = NOTRIAL(yyvsp[0].mergeMatchedClause);
break;
case 1514:
yyval.metaNamePtr =  NOTRIAL(&yyvsp[-4].mergeNode->relation->dsqlName);
break;
case 1516:
  if (!yytrial)
{ yyval.mergeNotMatchedClause = &yyvsp[-4].mergeNode->whenNotMatchedByTarget.add(); }
break;
case 1517:
yyval.mergeNotMatchedClause = NOTRIAL(yyvsp[0].mergeNotMatchedClause);
break;
case 1519:
  if (!yytrial)
{ yyval.mergeMatchedClause = &yyvsp[-5].mergeNode->whenNotMatchedBySource.add(); }
break;
case 1520:
yyval.metaNamePtr =  NOTRIAL(&yyvsp[-7].mergeNode->relation->dsqlName);
break;
case 1524:
yyval.metaNamePtr = NOTRIAL(yyvsp[-3].metaNamePtr);
break;
case 1525:
  if (!yytrial)
{ yyvsp[-6].mergeMatchedClause->assignments = yyvsp[0].compoundStmtNode; }
break;
case 1526:
yyval.metaNamePtr = NOTRIAL(yyvsp[-5].metaNamePtr);
break;
case 1527:
  if (!yytrial)
{
			yyvsp[-8].mergeMatchedClause->condition = yyvsp[-5].boolExprNode;
			yyvsp[-8].mergeMatchedClause->assignments = yyvsp[0].compoundStmtNode;
		}
break;
case 1529:
  if (!yytrial)
{ yyvsp[-5].mergeMatchedClause->condition = yyvsp[-2].boolExprNode; }
break;
case 1530:
yyval.nestFieldArray = NOTRIAL(&yyvsp[-2].mergeNotMatchedClause->fields);
break;
case 1531:
  if (!yytrial)
{
			yyvsp[-9].mergeNotMatchedClause->overrideClause = yyvsp[-4].nullableOverrideClause;
			yyvsp[-9].mergeNotMatchedClause->values = yyvsp[-1].valueListNode;
		}
break;
case 1532:
yyval.nestFieldArray = NOTRIAL(&yyvsp[-4].mergeNotMatchedClause->fields);
break;
case 1533:
  if (!yytrial)
{
			yyvsp[-11].mergeNotMatchedClause->overrideClause = yyvsp[-4].nullableOverrideClause;
			yyvsp[-11].mergeNotMatchedClause->values = yyvsp[-1].valueListNode;
			yyvsp[-11].mergeNotMatchedClause->condition = yyvsp[-9].boolExprNode;
		}
break;
case 1536:
  if (!yytrial)
{
			const auto node = newNode<EraseNode>();
			node->dsqlRelation = yyvsp[-6].relSourceNode;
			node->dsqlBoolean = yyvsp[-5].boolExprNode;
			node->dsqlPlan = yyvsp[-4].planNode;
			node->dsqlOrder = yyvsp[-3].valueListNode;
			node->dsqlRows = yyvsp[-2].rowsClause;
			node->dsqlSkipLocked = yyvsp[-1].boolVal;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1537:
  if (!yytrial)
{
			EraseNode* node = newNode<EraseNode>();
			node->dsqlRelation = yyvsp[-2].relSourceNode;
			node->dsqlCursorName = *yyvsp[-1].metaNamePtr;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1540:
yyval.metaNamePtr = NOTRIAL(&yyvsp[-1].relSourceNode->dsqlName);
break;
case 1541:
  if (!yytrial)
{
			ModifyNode* node = newNode<ModifyNode>();
			node->dsqlRelation = yyvsp[-9].relSourceNode;
			node->statement = yyvsp[-6].compoundStmtNode;
			node->dsqlBoolean = yyvsp[-5].boolExprNode;
			node->dsqlPlan = yyvsp[-4].planNode;
			node->dsqlOrder = yyvsp[-3].valueListNode;
			node->dsqlRows = yyvsp[-2].rowsClause;
			node->dsqlSkipLocked = yyvsp[-1].boolVal;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1542:
  if (!yytrial)
{
			ModifyNode* node = newNode<ModifyNode>();
			node->dsqlRelation = yyvsp[-5].relSourceNode;
			node->statement = yyvsp[-2].compoundStmtNode;
			node->dsqlCursorName = *yyvsp[-1].metaNamePtr;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1543:
  if (!yytrial)
{
				UpdateOrInsertNode* node = yyval.updInsNode = newNode<UpdateOrInsertNode>();
				node->relation = yyvsp[0].relSourceNode;
			}
break;
case 1544:
yyval.nestFieldArray = NOTRIAL(&yyvsp[0].updInsNode->fields);
break;
case 1545:
yyval.nestFieldArray = NOTRIAL(&yyvsp[-7].updInsNode->matching);
break;
case 1546:
  if (!yytrial)
{
				UpdateOrInsertNode* node = yyval.updInsNode = yyvsp[-13].updInsNode;
				node->overrideClause = yyvsp[-10].nullableOverrideClause;
				node->values = yyvsp[-7].valueListNode;
				node->plan = yyvsp[-3].planNode;
				node->order = yyvsp[-2].valueListNode;
				node->rows = yyvsp[-1].rowsClause;
				node->returning = yyvsp[0].returningClause;
			}
break;
case 1548:
yyval.nestFieldArray = yyvsp[-1].nestFieldArray;
break;
case 1550:
  if (!yytrial)
{ yyval.returningClause = NULL; }
break;
case 1551:
  if (!yytrial)
{
			yyval.returningClause = FB_NEW_POOL(getPool()) ReturningClause(getPool());
			yyval.returningClause->first = yyvsp[0].valueListNode;
		}
break;
case 1552:
  if (!yytrial)
{
			yyval.returningClause = FB_NEW_POOL(getPool()) ReturningClause(getPool());
			yyval.returningClause->first = yyvsp[-2].valueListNode;
			yyval.returningClause->second = yyvsp[0].valueListNode;
		}
break;
case 1553:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>(*yyvsp[0].metaNamePtr); }
break;
case 1554:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = yyvsp[-2].fieldNode;
			node->asgnFrom = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 1555:
yyval.metaNamePtr = yyvsp[0].metaNamePtr;
break;
case 1556:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 1557:
yyval.metaNamePtr = yyvsp[-2].metaNamePtr;
break;
case 1558:
  if (!yytrial)
{
			yyvsp[-3].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-3].compoundStmtNode;
		}
break;
case 1559:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = yyvsp[-2].fieldNode;
			node->asgnFrom = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 1560:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = yyvsp[-2].fieldNode;
			node->asgnFrom = newNode<DefaultNode>(*yyvsp[-3].metaNamePtr, yyvsp[-2].fieldNode->dsqlName);
			yyval.stmtNode = node;
		}
break;
case 1561:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = NullNode::instance();
			node->asgnFrom = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 1562:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = NullNode::instance();
			node->asgnFrom = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 1563:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1565:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 1566:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].fieldNode); }
break;
case 1567:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].fieldNode); }
break;
case 1569:
yyval.nestFieldArray = yyvsp[0].nestFieldArray;
break;
case 1572:
  if (!yytrial)
{ yyvsp[-1].nestFieldArray->add(yyvsp[0].fieldNode); }
break;
case 1573:
  if (!yytrial)
{ yyvsp[-3].nestFieldArray->add(yyvsp[0].fieldNode); }
break;
case 1575:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.fieldNode = fieldNode;
		}
break;
case 1576:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			fieldNode->dsqlCursorField = true;
			yyval.fieldNode = fieldNode;
		}
break;
case 1577:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.fieldNode = fieldNode;
		}
break;
case 1579:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.fieldNode = fieldNode;
		}
break;
case 1580:
  if (!yytrial)
{ yyval.boolExprNode = valueToBool(yyvsp[0].valueExprNode); }
break;
case 1582:
  if (!yytrial)
{ yyval.boolExprNode = newNode<BinaryBoolNode>(blr_or, valueToBool(yyvsp[-2].valueExprNode), valueToBool(yyvsp[0].valueExprNode)); }
break;
case 1583:
  if (!yytrial)
{ yyval.boolExprNode = newNode<BinaryBoolNode>(blr_and, valueToBool(yyvsp[-2].valueExprNode), valueToBool(yyvsp[0].valueExprNode)); }
break;
case 1584:
  if (!yytrial)
{ yyval.boolExprNode = newNode<NotBoolNode>(valueToBool(yyvsp[0].valueExprNode)); }
break;
case 1585:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[-1].boolExprNode; }
break;
case 1586:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_equiv, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			node->dsqlCheckBoolean = true;
			yyval.boolExprNode = node;
		}
break;
case 1587:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_equiv, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode);
			node->dsqlCheckBoolean = true;
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1600:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(yyvsp[-1].blrOp, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1601:
  if (!yytrial)
{ yyval.blrOp = blr_eql; }
break;
case 1602:
  if (!yytrial)
{ yyval.blrOp = blr_lss; }
break;
case 1603:
  if (!yytrial)
{ yyval.blrOp = blr_gtr; }
break;
case 1604:
  if (!yytrial)
{ yyval.blrOp = blr_geq; }
break;
case 1605:
  if (!yytrial)
{ yyval.blrOp = blr_leq; }
break;
case 1606:
  if (!yytrial)
{ yyval.blrOp = blr_leq; }
break;
case 1607:
  if (!yytrial)
{ yyval.blrOp = blr_geq; }
break;
case 1608:
  if (!yytrial)
{ yyval.blrOp = blr_neq; }
break;
case 1609:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(yyvsp[-4].blrOp, yyvsp[-5].valueExprNode, yyvsp[-3].cmpBoolFlag, yyvsp[-1].selectExprNode); }
break;
case 1610:
  if (!yytrial)
{ yyval.cmpBoolFlag = ComparativeBoolNode::DFLAG_ANSI_ALL; }
break;
case 1611:
  if (!yytrial)
{ yyval.cmpBoolFlag = ComparativeBoolNode::DFLAG_ANSI_ANY; }
break;
case 1612:
  if (!yytrial)
{ yyval.cmpBoolFlag = ComparativeBoolNode::DFLAG_ANSI_ANY; }
break;
case 1613:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_equiv, yyvsp[-4].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1614:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_equiv, yyvsp[-5].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1615:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_between, yyvsp[-4].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
		}
break;
case 1616:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_between, yyvsp[-5].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1617:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(yyvsp[-1].blrOp, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1618:
  if (!yytrial)
{
			ComparativeBoolNode* cmpNode = newNode<ComparativeBoolNode>(yyvsp[-1].blrOp, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(cmpNode);
		}
break;
case 1619:
  if (!yytrial)
{ yyval.blrOp = blr_containing; }
break;
case 1620:
  if (!yytrial)
{ yyval.blrOp = blr_starting; }
break;
case 1621:
  if (!yytrial)
{ yyval.blrOp = blr_starting; }
break;
case 1622:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_like, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1623:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_like, yyvsp[-4].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1624:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_like, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1625:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_like, yyvsp[-5].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1626:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1627:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-5].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1628:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-4].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1629:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-6].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1630:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-2].valueExprNode,
				ComparativeBoolNode::DFLAG_ANSI_ANY, yyvsp[0].exprNode);
		}
break;
case 1631:
  if (!yytrial)
{
			const auto node = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-3].valueExprNode,
				ComparativeBoolNode::DFLAG_ANSI_ANY, yyvsp[0].exprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1632:
  if (!yytrial)
{ yyval.boolExprNode = newNode<RseBoolNode>(blr_any, yyvsp[-1].selectExprNode); }
break;
case 1633:
  if (!yytrial)
{ yyval.boolExprNode = newNode<RseBoolNode>(blr_unique, yyvsp[-1].selectExprNode); }
break;
case 1634:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql,
					newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRIGGER_ACTION)),
					MAKE_const_slong(1));
		}
break;
case 1635:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql,
					newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRIGGER_ACTION)),
					MAKE_const_slong(2));
		}
break;
case 1636:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql,
					newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRIGGER_ACTION)),
					MAKE_const_slong(3));
		}
break;
case 1637:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql,
					newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_SESSION_RESETTING)),
					MAKE_const_slong(1));
		}
break;
case 1638:
  if (!yytrial)
{ yyval.boolExprNode = newNode<MissingBoolNode>(yyvsp[-2].valueExprNode); }
break;
case 1639:
  if (!yytrial)
{ yyval.boolExprNode = newNode<MissingBoolNode>(yyvsp[-2].valueExprNode, true); }
break;
case 1640:
  if (!yytrial)
{ yyval.boolExprNode = newNode<NotBoolNode>(newNode<MissingBoolNode>(yyvsp[-3].valueExprNode)); }
break;
case 1641:
  if (!yytrial)
{ yyval.boolExprNode = newNode<NotBoolNode>(newNode<MissingBoolNode>(yyvsp[-3].valueExprNode, true)); }
break;
case 1642:
  if (!yytrial)
{ yyval.exprNode = yyvsp[0].selectExprNode; }
break;
case 1643:
  if (!yytrial)
{ yyval.exprNode = yyvsp[-1].valueListNode; }
break;
case 1644:
  if (!yytrial)
{ yyval.selectExprNode = yyvsp[-1].selectExprNode; }
break;
case 1645:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_ADD, *yyvsp[0].metaNamePtr);
		}
break;
case 1646:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-2].createAlterUserNode;
		}
break;
case 1647:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_MOD, *yyvsp[-1].metaNamePtr);
		}
break;
case 1648:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-2].createAlterUserNode;
		}
break;
case 1649:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_MOD, "");
		}
break;
case 1650:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-2].createAlterUserNode;
		}
break;
case 1651:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_RPL, *yyvsp[-1].metaNamePtr);
		}
break;
case 1652:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-2].createAlterUserNode;
		}
break;
case 1658:
yyval.createAlterUserNode = yyvsp[-1].createAlterUserNode;
break;
case 1660:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->firstName, "FIRSTNAME", yyvsp[0].stringPtr); }
break;
case 1661:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->middleName, "MIDDLENAME", yyvsp[0].stringPtr); }
break;
case 1662:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->lastName, "LASTNAME", yyvsp[0].stringPtr); }
break;
case 1663:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->password, "PASSWORD", yyvsp[0].stringPtr); }
break;
case 1664:
  if (!yytrial)
{ setClause(yyvsp[-3].createAlterUserNode->adminRole, "ADMIN ROLE", true); }
break;
case 1665:
  if (!yytrial)
{ setClause(yyvsp[-3].createAlterUserNode->adminRole, "ADMIN ROLE", false); }
break;
case 1666:
  if (!yytrial)
{ setClause(yyvsp[-1].createAlterUserNode->active, "ACTIVE/INACTIVE", true); }
break;
case 1667:
  if (!yytrial)
{ setClause(yyvsp[-1].createAlterUserNode->active, "ACTIVE/INACTIVE", false); }
break;
case 1669:
yyval.createAlterUserNode = yyvsp[-2].createAlterUserNode;
break;
case 1671:
  if (!yytrial)
{ setClause(yyvsp[-3].createAlterUserNode->plugin, "USING PLUGIN", yyvsp[0].metaNamePtr); }
break;
case 1674:
  if (!yytrial)
{
			yyvsp[-3].createAlterUserNode->addProperty(yyvsp[-2].metaNamePtr, yyvsp[0].stringPtr);
		}
break;
case 1675:
  if (!yytrial)
{
			yyvsp[-2].createAlterUserNode->addProperty(yyvsp[0].metaNamePtr);
		}
break;
case 1676:
yyval.mappingOp = MappingNode::MAP_ADD;
break;
case 1677:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[0].mappingNode;
				yyval.mappingNode->global = yyvsp[-2].boolVal;
			}
break;
case 1678:
yyval.mappingNode = yyvsp[0].mappingNode;
break;
case 1679:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-2].mappingNode;
			}
break;
case 1680:
yyval.mappingOp = MappingNode::MAP_MOD;
break;
case 1681:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[0].mappingNode;
				yyval.mappingNode->global = yyvsp[-2].boolVal;
			}
break;
case 1682:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-2].mappingNode;
			}
break;
case 1683:
yyval.mappingOp = MappingNode::MAP_RPL;
break;
case 1684:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[0].mappingNode;
				yyval.mappingNode->global = yyvsp[-2].boolVal;
			}
break;
case 1685:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-2].mappingNode;
			}
break;
case 1686:
  if (!yytrial)
{
 			MappingNode* node = newNode<MappingNode>(MappingNode::MAP_DROP, *yyvsp[0].metaNamePtr);
			node->global = yyvsp[-1].boolVal;
			yyval.mappingNode = node;
		}
break;
case 1687:
  if (!yytrial)
{
			yyval.mappingNode = yyvsp[0].mappingNode;
		}
break;
case 1688:
  if (!yytrial)
{
			yyval.mappingNode = yyvsp[0].mappingNode;
		}
break;
case 1689:
  if (!yytrial)
{
 			yyval.mappingNode = newNode<MappingNode>(MappingNode::MAP_COMMENT, *yyvsp[-2].metaNamePtr);
			yyval.mappingNode->global = yyvsp[-3].boolVal;
			yyval.mappingNode->comment = yyvsp[0].stringPtr;
		}
break;
case 1690:
  if (!yytrial)
{
				yyval.mappingNode = newNode<MappingNode>(yyvsp[-1].mappingOp, *yyvsp[0].metaNamePtr);
			}
break;
case 1691:
yyval.mappingNode = yyvsp[-1].mappingNode;
break;
case 1692:
yyval.mappingNode = yyvsp[-4].mappingNode;
break;
case 1693:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-6].mappingNode;
			}
break;
case 1694:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1695:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->fromType = yyvsp[-1].metaNamePtr;
			yyvsp[-2].mappingNode->from = yyvsp[0].intlStringPtr;
		}
break;
case 1696:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->fromType = yyvsp[0].metaNamePtr;
			yyvsp[-2].mappingNode->from = newNode<IntlString>("*");
		}
break;
case 1698:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>("USER"); }
break;
case 1699:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>("GROUP"); }
break;
case 1701:
  if (!yytrial)
{ yyval.intlStringPtr = newNode<IntlString>(yyvsp[0].metaNamePtr->c_str()); }
break;
case 1702:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->mode = 'P';
			yyvsp[-3].mappingNode->plugin = yyvsp[-1].metaNamePtr;
			yyvsp[-3].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1703:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->mode = 'P';
			yyvsp[-3].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1704:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->mode = 'S';
		}
break;
case 1705:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->mode = 'M';
			yyvsp[-2].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1706:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->mode = '*';
			yyvsp[-2].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1707:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 1708:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1709:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->role = yyvsp[-1].boolVal;
			yyvsp[-3].mappingNode->to = yyvsp[0].metaNamePtr;
		}
break;
case 1710:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->role = yyvsp[0].boolVal;
		}
break;
case 1711:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1712:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1714:
  if (!yytrial)
{ yyval.valueExprNode = newNode<BoolAsValueNode>(yyvsp[0].boolExprNode); }
break;
case 1716:
  if (!yytrial)
{ yyval.valueExprNode = newNode<BoolAsValueNode>(yyvsp[-1].boolExprNode); }
break;
case 1718:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1722:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].fieldNode; }
break;
case 1724:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1731:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1732:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1733:
  if (!yytrial)
{ yyval.valueExprNode = newNode<NegateNode>(yyvsp[0].valueExprNode); }
break;
case 1734:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1735:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_add, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1736:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ConcatenateNode>(yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1737:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CollateNode>(yyvsp[-2].valueExprNode, *yyvsp[0].metaNamePtr); }
break;
case 1738:
  if (!yytrial)
{ yyval.valueExprNode = newNode<AtNode>(yyvsp[-2].valueExprNode, nullptr); }
break;
case 1739:
  if (!yytrial)
{ yyval.valueExprNode = newNode<AtNode>(yyvsp[-4].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1740:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_subtract, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1741:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_multiply, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1742:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_divide, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1743:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1744:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1745:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1746:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1747:
  if (!yytrial)
{ yyval.valueExprNode = newNode<RecordKeyNode>(yyvsp[0].blrOp); }
break;
case 1748:
  if (!yytrial)
{ yyval.valueExprNode = newNode<RecordKeyNode>(yyvsp[0].blrOp, *yyvsp[-2].metaNamePtr); }
break;
case 1749:
  if (!yytrial)
{ yyval.valueExprNode = newNode<DomainValidationNode>(); }
break;
case 1750:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1751:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1752:
  if (!yytrial)
{ yyval.blrOp = blr_dbkey; }
break;
case 1753:
  if (!yytrial)
{ yyval.blrOp = blr_record_version2; }
break;
case 1754:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("DATE"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("DATE"));
			}

			yyval.valueExprNode = newNode<CurrentDateNode>();
		}
break;
case 1755:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("TIME"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("TIME"));
			}

			yyval.valueExprNode = newNode<LocalTimeNode>(yyvsp[0].uintVal);
		}
break;
case 1756:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("TIME"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("TIME"));
			}

			yyval.valueExprNode = newNode<CurrentTimeNode>(yyvsp[0].uintVal);
		}
break;
case 1757:
  if (!yytrial)
{ yyval.valueExprNode = newNode<LocalTimeStampNode>(yyvsp[0].uintVal); }
break;
case 1758:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentTimeStampNode>(yyvsp[0].uintVal); }
break;
case 1759:
  if (!yytrial)
{ yyval.uintVal = DEFAULT_TIME_PRECISION; }
break;
case 1760:
  if (!yytrial)
{ yyval.uintVal = yyvsp[-1].int32Val; }
break;
case 1761:
  if (!yytrial)
{ yyval.uintVal = DEFAULT_TIMESTAMP_PRECISION; }
break;
case 1762:
  if (!yytrial)
{ yyval.uintVal = yyvsp[-1].int32Val; }
break;
case 1763:
  if (!yytrial)
{
			ArrayNode* node = newNode<ArrayNode>(yyvsp[-3].fieldNode);
			node->field->dsqlIndices = yyvsp[-1].valueListNode;
			yyval.valueExprNode = node;
		}
break;
case 1764:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(0); }
break;
case 1765:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1766:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1767:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1769:
  if (!yytrial)
{ yyval.valueExprNode = newNode<NegateNode>(yyvsp[0].valueExprNode); }
break;
case 1770:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_sint64(MIN_SINT64, 0); }
break;
case 1771:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_sint64(MIN_SINT64, yyvsp[0].lim64ptr->getScale()); }
break;
case 1772:
  if (!yytrial)
{ yyval.valueExprNode = newNode<NegateNode>(yyvsp[0].valueExprNode); }
break;
case 1774:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1775:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].lim64ptr->c_str(), CONSTANT_NUM128, yyvsp[0].lim64ptr->getScale()); }
break;
case 1776:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].lim64ptr->c_str(), CONSTANT_NUM128); }
break;
case 1778:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].lim64ptr->c_str(), CONSTANT_NUM128, yyvsp[0].lim64ptr->getScale()); }
break;
case 1779:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(yyvsp[0].int32Val); }
break;
case 1780:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].stringPtr->c_str(), CONSTANT_DOUBLE); }
break;
case 1781:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].stringPtr->c_str(), CONSTANT_DECIMAL); }
break;
case 1782:
  if (!yytrial)
{
			SINT64 signedNumber = (SINT64) yyvsp[0].scaledNumber.number;

			if (yyvsp[0].scaledNumber.hex && signedNumber < 0)
				yyval.valueExprNode = newNode<NegateNode>(MAKE_const_sint64(-signedNumber, yyvsp[0].scaledNumber.scale));
			else
				yyval.valueExprNode = MAKE_const_sint64(signedNumber, yyvsp[0].scaledNumber.scale);
		}
break;
case 1783:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_sint64((SINT64) yyvsp[0].scaledNumber.number, yyvsp[0].scaledNumber.scale); }
break;
case 1785:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(yyvsp[0].intlStringPtr, lex.att_charset); }
break;
case 1786:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("DATE"));
			}
			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("DATE"));
			}
			yyval.valueExprNode = MAKE_constant(yyvsp[0].intlStringPtr->getString().c_str(), CONSTANT_DATE);
		}
break;
case 1787:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("TIME"));
			}
			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("TIME"));
			}
			yyval.valueExprNode = MAKE_constant(yyvsp[0].intlStringPtr->getString().c_str(), CONSTANT_TIME);
		}
break;
case 1788:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].intlStringPtr->getString().c_str(), CONSTANT_TIMESTAMP); }
break;
case 1789:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant("", CONSTANT_BOOLEAN); }
break;
case 1790:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant("1", CONSTANT_BOOLEAN); }
break;
case 1791:
  if (!yytrial)
{ yyval.valueExprNode = make_parameter(); }
break;
case 1792:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentUserNode>(); }
break;
case 1793:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentUserNode>(); }
break;
case 1794:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentRoleNode>(); }
break;
case 1795:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_CONNECTION_ID)); }
break;
case 1796:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRANSACTION_ID)); }
break;
case 1797:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_GDSCODE)); }
break;
case 1798:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_SQLCODE)); }
break;
case 1799:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_SQLSTATE)); }
break;
case 1800:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_ROWS_AFFECTED)); }
break;
case 1801:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(yyvsp[-1].int32Val)); }
break;
case 1802:
  if (!yytrial)
{ yyval.int32Val = INFO_TYPE_GDSCODE; }
break;
case 1803:
  if (!yytrial)
{ yyval.int32Val = INFO_TYPE_SQLCODE; }
break;
case 1804:
  if (!yytrial)
{ yyval.int32Val = INFO_TYPE_SQLSTATE; }
break;
case 1805:
  if (!yytrial)
{ yyval.int32Val = INFO_TYPE_EXCEPTION; }
break;
case 1806:
  if (!yytrial)
{ yyval.int32Val = INFO_TYPE_ERROR_MSG; }
break;
case 1808:
  if (!yytrial)
{
			yyval.intlStringPtr = yyvsp[0].intlStringPtr;
			yyval.intlStringPtr->setCharSet(*yyvsp[-1].metaNamePtr);

			StrMark* mark = strMarks.get(yyvsp[0].intlStringPtr);

			if (mark)	/* hex string is not in strMarks*/
				mark->introduced = true;
		}
break;
case 1809:
  if (!yytrial)
{ yyval.stringPtr = newString(yyvsp[0].intlStringPtr->toUtf8(scratch->getTransaction())); }
break;
case 1811:
  if (!yytrial)
{ yyval.int32Val = -yyvsp[0].int32Val; }
break;
case 1812:
  if (!yytrial)
{
			if (yyvsp[0].int32Val > SHRT_POS_MAX)
				yyabandon(YYPOSNARG(1), -842, isc_expec_short);	/* Short integer expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1813:
  if (!yytrial)
{
			if (yyvsp[0].int32Val > SHRT_NEG_MAX)
				yyabandon(YYPOSNARG(1), -842, isc_expec_short);	/* Short integer expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1814:
  if (!yytrial)
{
			if (yyvsp[0].int32Val == 0)
				yyabandon(YYPOSNARG(1), -842, isc_expec_positive);	/* Positive number expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1815:
  if (!yytrial)
{
			if (yyvsp[0].int32Val > SHRT_UNSIGNED_MAX)
				yyabandon(YYPOSNARG(1), -842, isc_expec_ushort);	/* Unsigned short integer expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1817:
  if (!yytrial)
{ yyval.int32Val = -yyvsp[0].int32Val; }
break;
case 1818:
  if (!yytrial)
{ yyval.int32Val = yyvsp[0].int32Val;}
break;
case 1819:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].aggNode; }
break;
case 1826:
  if (!yytrial)
{
			yyval.aggNode = yyvsp[-5].aggNode;

			if (yyval.aggNode->aggInfo.blr == blr_agg_count2 && !yyval.aggNode->arg)	/* count(*)*/
				yyval.aggNode->arg = newNode<ValueIfNode>(yyvsp[-1].boolExprNode, MAKE_const_slong(1), NullNode::instance());
			else
			{
				fb_assert(yyval.aggNode->arg);
				yyval.aggNode->arg = newNode<ValueIfNode>(yyvsp[-1].boolExprNode, yyval.aggNode->arg, NullNode::instance());
			}
		}
break;
case 1827:
  if (!yytrial)
{ yyval.aggNode = newNode<CountAggNode>(false, (client_dialect < SQL_DIALECT_V6_TRANSITION)); }
break;
case 1828:
  if (!yytrial)
{ yyval.aggNode = newNode<CountAggNode>(false, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode); }
break;
case 1829:
  if (!yytrial)
{ yyval.aggNode = newNode<CountAggNode>(true, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode); }
break;
case 1830:
  if (!yytrial)
{
			yyval.aggNode = newNode<SumAggNode>(false,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1831:
  if (!yytrial)
{
			yyval.aggNode = newNode<SumAggNode>(true,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1832:
  if (!yytrial)
{
			yyval.aggNode = newNode<AvgAggNode>(false,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1833:
  if (!yytrial)
{
			yyval.aggNode = newNode<AvgAggNode>(true,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1834:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MIN, yyvsp[-1].valueExprNode); }
break;
case 1835:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MIN, yyvsp[-1].valueExprNode); }
break;
case 1836:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MAX, yyvsp[-1].valueExprNode); }
break;
case 1837:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MAX, yyvsp[-1].valueExprNode); }
break;
case 1838:
  if (!yytrial)
{ yyval.aggNode = newNode<ListAggNode>(false, yyvsp[-2].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1839:
  if (!yytrial)
{ yyval.aggNode = newNode<ListAggNode>(true, yyvsp[-2].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1840:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_STDDEV_SAMP, yyvsp[-1].valueExprNode); }
break;
case 1841:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_STDDEV_POP, yyvsp[-1].valueExprNode); }
break;
case 1842:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_VAR_SAMP, yyvsp[-1].valueExprNode); }
break;
case 1843:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_VAR_POP, yyvsp[-1].valueExprNode); }
break;
case 1844:
  if (!yytrial)
{ yyval.aggNode = newNode<CorrAggNode>(CorrAggNode::TYPE_COVAR_SAMP, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1845:
  if (!yytrial)
{ yyval.aggNode = newNode<CorrAggNode>(CorrAggNode::TYPE_COVAR_POP, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1846:
  if (!yytrial)
{ yyval.aggNode = newNode<CorrAggNode>(CorrAggNode::TYPE_CORR, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1847:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_AVGX, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1848:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_AVGY, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1849:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrCountAggNode>(yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1850:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_INTERCEPT, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1851:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_R2, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1852:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SLOPE, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1853:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SXX, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1854:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SXY, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1855:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SYY, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1856:
  if (!yytrial)
{ yyval.aggNode = newNode<DenseRankWinNode>(); }
break;
case 1857:
  if (!yytrial)
{ yyval.aggNode = newNode<RankWinNode>(); }
break;
case 1858:
  if (!yytrial)
{ yyval.aggNode = newNode<PercentRankWinNode>(); }
break;
case 1859:
  if (!yytrial)
{ yyval.aggNode = newNode<CumeDistWinNode>(); }
break;
case 1860:
  if (!yytrial)
{ yyval.aggNode = newNode<RowNumberWinNode>(); }
break;
case 1861:
  if (!yytrial)
{ yyval.aggNode = newNode<FirstValueWinNode>(yyvsp[-1].valueExprNode); }
break;
case 1862:
  if (!yytrial)
{ yyval.aggNode = newNode<LastValueWinNode>(yyvsp[-1].valueExprNode); }
break;
case 1863:
  if (!yytrial)
{ yyval.aggNode = newNode<NthValueWinNode>(yyvsp[-4].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1864:
  if (!yytrial)
{ yyval.aggNode = newNode<LagWinNode>(yyvsp[-5].valueExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1865:
  if (!yytrial)
{ yyval.aggNode = newNode<LagWinNode>(yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode, NullNode::instance()); }
break;
case 1866:
  if (!yytrial)
{ yyval.aggNode = newNode<LagWinNode>(yyvsp[-1].valueExprNode, MAKE_const_slong(1), NullNode::instance()); }
break;
case 1867:
  if (!yytrial)
{ yyval.aggNode = newNode<LeadWinNode>(yyvsp[-5].valueExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1868:
  if (!yytrial)
{ yyval.aggNode = newNode<LeadWinNode>(yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode, NullNode::instance()); }
break;
case 1869:
  if (!yytrial)
{ yyval.aggNode = newNode<LeadWinNode>(yyvsp[-1].valueExprNode, MAKE_const_slong(1), NullNode::instance()); }
break;
case 1870:
  if (!yytrial)
{ yyval.aggNode = newNode<NTileWinNode>(yyvsp[-1].valueExprNode); }
break;
case 1871:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(NthValueWinNode::FROM_FIRST); }
break;
case 1872:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(NthValueWinNode::FROM_FIRST); }
break;
case 1873:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(NthValueWinNode::FROM_LAST); }
break;
case 1879:
  if (!yytrial)
{ yyval.valueExprNode = newNode<OverNode>(yyvsp[-2].aggNode, yyvsp[0].metaNamePtr); }
break;
case 1880:
  if (!yytrial)
{ yyval.valueExprNode = newNode<OverNode>(yyvsp[-4].aggNode, yyvsp[-1].windowClause); }
break;
case 1881:
  if (!yytrial)
{
			yyval.windowClause = newNode<WindowClause>(yyvsp[-4].metaNamePtr, yyvsp[-3].valueListNode, yyvsp[-2].valueListNode, yyvsp[-1].windowClauseFrameExtent, yyvsp[0].windowClauseExclusion);
		}
break;
case 1882:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1883:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1884:
  if (!yytrial)
{ yyval.windowClauseFrameExtent = NULL; }
break;
case 1885:
  if (!yytrial)
{ yyval.windowClauseFrameExtent = newNode<WindowClause::FrameExtent>(WindowClause::FrameExtent::Unit::RANGE); }
break;
case 1886:
yyval.windowClauseFrameExtent = yyvsp[0].windowClauseFrameExtent;
break;
case 1887:
  if (!yytrial)
{ yyval.windowClauseFrameExtent = yyvsp[-2].windowClauseFrameExtent; }
break;
case 1888:
  if (!yytrial)
{ yyval.windowClauseFrameExtent = newNode<WindowClause::FrameExtent>(WindowClause::FrameExtent::Unit::ROWS); }
break;
case 1889:
  if (!yytrial)
{ yyval.windowClauseFrameExtent = yyvsp[-2].windowClauseFrameExtent; }
break;
case 1890:
  if (!yytrial)
{
			yyvsp[-1].windowClauseFrameExtent->frame1 = yyvsp[0].windowClauseFrame;
			yyvsp[-1].windowClauseFrameExtent->frame2 =
				newNode<WindowClause::Frame>(WindowClause::Frame::Bound::CURRENT_ROW);
		}
break;
case 1891:
  if (!yytrial)
{
			yyvsp[-4].windowClauseFrameExtent->frame1 = yyvsp[-2].windowClauseFrame;
			yyvsp[-4].windowClauseFrameExtent->frame2 = yyvsp[0].windowClauseFrame;
		}
break;
case 1892:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::PRECEDING); }
break;
case 1893:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::CURRENT_ROW); }
break;
case 1894:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::PRECEDING, yyvsp[-1].valueExprNode); }
break;
case 1895:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::PRECEDING); }
break;
case 1896:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::CURRENT_ROW); }
break;
case 1897:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::PRECEDING, yyvsp[-1].valueExprNode); }
break;
case 1898:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::FOLLOWING, yyvsp[-1].valueExprNode); }
break;
case 1899:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::FOLLOWING); }
break;
case 1900:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::CURRENT_ROW); }
break;
case 1901:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::PRECEDING, yyvsp[-1].valueExprNode); }
break;
case 1902:
  if (!yytrial)
{ yyval.windowClauseFrame = newNode<WindowClause::Frame>(WindowClause::Frame::Bound::FOLLOWING, yyvsp[-1].valueExprNode); }
break;
case 1903:
  if (!yytrial)
{ yyval.windowClauseExclusion = WindowClause::Exclusion::NO_OTHERS; }
break;
case 1904:
  if (!yytrial)
{ yyval.windowClauseExclusion = WindowClause::Exclusion::NO_OTHERS; }
break;
case 1905:
  if (!yytrial)
{ yyval.windowClauseExclusion = WindowClause::Exclusion::CURRENT_ROW; }
break;
case 1906:
  if (!yytrial)
{ yyval.windowClauseExclusion = WindowClause::Exclusion::GROUP; }
break;
case 1907:
  if (!yytrial)
{ yyval.windowClauseExclusion = WindowClause::Exclusion::TIES; }
break;
case 1908:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(newIntlString(","), lex.att_charset); }
break;
case 1909:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1912:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ExtractNode>(yyvsp[-3].blrOp, yyvsp[-1].valueExprNode); }
break;
case 1916:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_bit, yyvsp[-1].valueExprNode); }
break;
case 1917:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_char, yyvsp[-1].valueExprNode); }
break;
case 1918:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_char, yyvsp[-1].valueExprNode); }
break;
case 1919:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_octet, yyvsp[-1].valueExprNode); }
break;
case 1920:
  if (!yytrial)
{ yyval.valueExprNode = newNode<SysFuncCallNode>(*yyvsp[-3].metaNamePtr, yyvsp[-1].valueListNode); }
break;
case 1921:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].sysFuncCallNode; }
break;
case 1988:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-6].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-4].valueExprNode)->add(MAKE_const_slong(yyvsp[-3].blrOp))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1989:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-3].valueExprNode)->add(MAKE_const_slong(yyvsp[-5].blrOp))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1990:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(MAKE_const_slong(yyvsp[-5].blrOp))->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1991:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(MAKE_const_slong(yyvsp[-5].blrOp))->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1992:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-11].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-9].valueExprNode)->add(MAKE_str_constant(newIntlString(yyvsp[-7].metaNamePtr->c_str()), CS_ASCII))->
					add(MAKE_str_constant(newIntlString(yyvsp[-6].metaNamePtr->c_str()), CS_ASCII))->add(yyvsp[-4].valueExprNode)->add(yyvsp[-3].valueExprNode)->
					add(MAKE_str_constant(newIntlString(yyvsp[-2].metaNamePtr->c_str()), CS_ASCII))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1993:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-5].metaNamePtr,
				newNode<ValueListNode>(MAKE_const_slong(yyvsp[-3].blrOp))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1994:
  if (!yytrial)
{ yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-3].metaNamePtr, newNode<ValueListNode>(yyvsp[-1].valueExprNode)); }
break;
case 1995:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-5].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-3].valueExprNode)->add(MAKE_str_constant(newIntlString(yyvsp[-1].metaNamePtr->c_str()), CS_ASCII)));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1996:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-5].metaNamePtr,
				newNode<ValueListNode>(MAKE_const_slong(yyvsp[-3].blrOp))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1997:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-9].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-7].valueExprNode)->add(yyvsp[-5].valueExprNode)->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1998:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-5].valueExprNode)->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1999:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-5].metaNamePtr, newNode<ValueListNode>(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 2000:
  if (!yytrial)
{ yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-3].metaNamePtr, yyvsp[-1].valueListNode); }
break;
case 2001:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-8].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-6].valueExprNode)->add(yyvsp[-4].valueExprNode)->add(yyvsp[-3].valueExprNode)->
					add(MAKE_str_constant(newIntlString(yyvsp[-2].metaNamePtr->c_str()), CS_ASCII))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 2002:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-8].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-6].valueExprNode)->add(yyvsp[-4].valueExprNode)->
					add(MAKE_str_constant(newIntlString(yyvsp[-3].metaNamePtr->c_str()), CS_ASCII))->add(yyvsp[-2].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 2003:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-10].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-8].valueExprNode)->add(yyvsp[-6].valueExprNode)->add(yyvsp[-4].valueExprNode)->
					add(MAKE_str_constant(newIntlString(yyvsp[-3].metaNamePtr->c_str()), CS_ASCII))->add(yyvsp[-2].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 2004:
  if (!yytrial)
{
			ValueExprNode* v = MAKE_system_privilege(yyvsp[-1].metaNamePtr->c_str());
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-3].metaNamePtr, newNode<ValueListNode>(v));
		}
break;
case 2009:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(newIntlString(""), CS_ASCII); }
break;
case 2010:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 2011:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(0); }
break;
case 2012:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(1); }
break;
case 2013:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>(""); }
break;
case 2014:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 2015:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(newIntlString(""), CS_ASCII); }
break;
case 2016:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 2017:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>(""); }
break;
case 2018:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 2019:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(newIntlString(""), CS_ASCII); }
break;
case 2020:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 2021:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>(""); }
break;
case 2022:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 2025:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(newIntlString(""), CS_ASCII); }
break;
case 2026:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 2031:
  if (!yytrial)
{ yyval.blrOp = blr_extract_year; }
break;
case 2032:
  if (!yytrial)
{ yyval.blrOp = blr_extract_quarter; }
break;
case 2033:
  if (!yytrial)
{ yyval.blrOp = blr_extract_month; }
break;
case 2034:
  if (!yytrial)
{ yyval.blrOp = blr_extract_week; }
break;
case 2037:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrCaseNode>(blr_upcase, yyvsp[-1].valueExprNode); }
break;
case 2038:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrCaseNode>(blr_lowcase, yyvsp[-1].valueExprNode); }
break;
case 2039:
  if (!yytrial)
{
			/* SQL spec requires numbering to start with 1,*/
			/* hence we decrement the first parameter to make it*/
			/* compatible with the engine's implementation*/
			ArithmeticNode* subtractNode = newNode<ArithmeticNode>(
				blr_subtract, true, yyvsp[-2].valueExprNode, MAKE_const_slong(1));

			yyval.valueExprNode = newNode<SubstringNode>(yyvsp[-4].valueExprNode, subtractNode, yyvsp[-1].valueExprNode);
		}
break;
case 2040:
  if (!yytrial)
{ yyval.valueExprNode = newNode<SubstringSimilarNode>(yyvsp[-5].valueExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 2041:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 2042:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 2043:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(yyvsp[-4].blrOp, yyvsp[-1].valueExprNode, yyvsp[-3].valueExprNode); }
break;
case 2044:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(blr_trim_both, yyvsp[-1].valueExprNode, yyvsp[-3].valueExprNode); }
break;
case 2045:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(yyvsp[-3].blrOp, yyvsp[-1].valueExprNode); }
break;
case 2046:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(blr_trim_both, yyvsp[-1].valueExprNode); }
break;
case 2047:
  if (!yytrial)
{ yyval.blrOp = blr_trim_both; }
break;
case 2048:
  if (!yytrial)
{ yyval.blrOp = blr_trim_trailing; }
break;
case 2049:
  if (!yytrial)
{ yyval.blrOp = blr_trim_leading; }
break;
case 2050:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-3].metaNamePtr, ""), yyvsp[-1].valueListNode); }
break;
case 2051:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-2].metaNamePtr, ""), newNode<ValueListNode>(0)); }
break;
case 2052:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-3].metaNamePtr, *yyvsp[-5].metaNamePtr), yyvsp[-1].valueListNode); }
break;
case 2053:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-2].metaNamePtr, *yyvsp[-4].metaNamePtr), newNode<ValueListNode>(0)); }
break;
case 2054:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CastNode>(yyvsp[-3].valueExprNode, yyvsp[-1].legacyField); }
break;
case 2057:
  if (!yytrial)
{
			ComparativeBoolNode* condition = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode);
			yyval.valueExprNode = newNode<ValueIfNode>(condition, NullNode::instance(), yyvsp[-3].valueExprNode);
		}
break;
case 2058:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ValueIfNode>(yyvsp[-5].boolExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 2059:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CoalesceNode>(yyvsp[-1].valueListNode->addFront(yyvsp[-3].valueExprNode)); }
break;
case 2060:
  if (!yytrial)
{
			ValueListNode* list = yyvsp[-1].valueListNode;
			ValueListNode* conditions = newNode<ValueListNode>(list->items.getCount() / 2);
			ValueListNode* values = newNode<ValueListNode>(list->items.getCount() / 2);

			for (FB_SIZE_T i = 0; i < list->items.getCount(); i += 2)
			{
				conditions->items[i / 2] = list->items[i];
				values->items[i / 2] = list->items[i + 1];
			}

			yyval.valueExprNode = newNode<DecodeNode>(yyvsp[-3].valueExprNode, conditions, values);
		}
break;
case 2061:
  if (!yytrial)
{
			ValueListNode* list = yyvsp[-3].valueListNode;
			ValueListNode* conditions = newNode<ValueListNode>(list->items.getCount() / 2);
			ValueListNode* values = newNode<ValueListNode>(list->items.getCount() / 2 + 1);

			for (FB_SIZE_T i = 0; i < list->items.getCount(); i += 2)
			{
				conditions->items[i / 2] = list->items[i];
				values->items[i / 2] = list->items[i + 1];
			}

			values->items[list->items.getCount() / 2] = yyvsp[-1].valueExprNode;

			yyval.valueExprNode = newNode<DecodeNode>(yyvsp[-5].valueExprNode, conditions, values);
		}
break;
case 2062:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].decodeNode; }
break;
case 2063:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 2064:
  if (!yytrial)
{ yyval.decodeNode = newNode<DecodeNode>(yyvsp[0].valueExprNode, newNode<ValueListNode>(0u), newNode<ValueListNode>(0u)); }
break;
case 2065:
yyval.valueListNode = NOTRIAL(yyvsp[0].decodeNode->conditions);
break;
case 2066:
yyval.valueListNode =  NOTRIAL(yyvsp[-1].decodeNode->values);
break;
case 2067:
  if (!yytrial)
{
				DecodeNode* node = yyval.decodeNode = yyvsp[-5].decodeNode;
				node->label = "CASE";
				if (yyvsp[-1].valueExprNode)
					node->values->add(yyvsp[-1].valueExprNode);
			}
break;
case 2068:
  if (!yytrial)
{
			yyvsp[-5].valueListNode->add(yyvsp[-2].valueExprNode);
			yyvsp[-4].valueListNode->add(yyvsp[0].valueExprNode);
		}
break;
case 2069:
  if (!yytrial)
{
			yyvsp[-6].valueListNode->add(yyvsp[-2].valueExprNode);
			yyvsp[-5].valueListNode->add(yyvsp[0].valueExprNode);
		}
break;
case 2070:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 2071:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 2072:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueIfNode; }
break;
case 2073:
  if (!yytrial)
{
			ValueIfNode* last = yyvsp[-3].valueIfNode;
			ValueIfNode* next;

			while ((next = nodeAs<ValueIfNode>(last->falseValue)))
				last = next;

			fb_assert(nodeIs<NullNode>(last->falseValue));

			last->falseValue = yyvsp[-1].valueExprNode;
			yyval.valueExprNode = yyvsp[-3].valueIfNode;
		}
break;
case 2074:
  if (!yytrial)
{ yyval.valueIfNode = newNode<ValueIfNode>(yyvsp[-2].boolExprNode, yyvsp[0].valueExprNode, NullNode::instance()); }
break;
case 2075:
  if (!yytrial)
{
			ValueIfNode* cond = newNode<ValueIfNode>(yyvsp[-2].boolExprNode, yyvsp[0].valueExprNode, NullNode::instance());
			ValueIfNode* last = yyvsp[-4].valueIfNode;
			ValueIfNode* next;

			while ((next = nodeAs<ValueIfNode>(last->falseValue)))
				last = next;

			fb_assert(nodeIs<NullNode>(last->falseValue));

			last->falseValue = cond;
			yyval.valueIfNode = yyvsp[-4].valueIfNode;
		}
break;
case 2079:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(0u)->add(yyvsp[-2].valueExprNode)->add(yyvsp[0].valueExprNode); }
break;
case 2080:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-4].valueListNode->add(yyvsp[-2].valueExprNode)->add(yyvsp[0].valueExprNode); }
break;
case 2081:
  if (!yytrial)
{
			yyval.valueExprNode = newNode<GenIdNode>((client_dialect < SQL_DIALECT_V6_TRANSITION),
				*yyvsp[0].metaNamePtr, ((Jrd::ValueExprNode*) NULL), true, false);
		}
break;
case 2082:
  if (!yytrial)
{
			yyval.valueExprNode = newNode<GenIdNode>((client_dialect < SQL_DIALECT_V6_TRANSITION),
				*yyvsp[-3].metaNamePtr, yyvsp[-1].valueExprNode, false, false);
		}
break;
case 2083:
  if (!yytrial)
{ yyval.blrOp = blr_extract_year; }
break;
case 2084:
  if (!yytrial)
{ yyval.blrOp = blr_extract_quarter; }
break;
case 2085:
  if (!yytrial)
{ yyval.blrOp = blr_extract_month; }
break;
case 2086:
  if (!yytrial)
{ yyval.blrOp = blr_extract_day; }
break;
case 2087:
  if (!yytrial)
{ yyval.blrOp = blr_extract_hour; }
break;
case 2088:
  if (!yytrial)
{ yyval.blrOp = blr_extract_minute; }
break;
case 2089:
  if (!yytrial)
{ yyval.blrOp = blr_extract_second; }
break;
case 2090:
  if (!yytrial)
{ yyval.blrOp = blr_extract_millisecond; }
break;
case 2091:
  if (!yytrial)
{ yyval.blrOp = blr_extract_timezone_hour; }
break;
case 2092:
  if (!yytrial)
{ yyval.blrOp = blr_extract_timezone_minute; }
break;
case 2093:
  if (!yytrial)
{ yyval.blrOp = blr_extract_timezone_name; }
break;
case 2094:
  if (!yytrial)
{ yyval.blrOp = blr_extract_week; }
break;
case 2095:
  if (!yytrial)
{ yyval.blrOp = blr_extract_weekday; }
break;
case 2096:
  if (!yytrial)
{ yyval.blrOp = blr_extract_yearday; }
break;
case 2101:
  if (!yytrial)
{ yyval.valueExprNode = NullNode::instance(); }
break;
case 2102:
  if (!yytrial)
{
			dsql_fld* field = newNode<dsql_fld>();
			field->dtype = dtype_boolean;
			field->length = sizeof(UCHAR);

			CastNode* castNode = newNode<CastNode>(NullNode::instance(), field);
			castNode->dsqlAlias = "CONSTANT";
			yyval.valueExprNode = castNode;
		}
break;

  default:
    break;
  }

#if YYDEBUG && defined(YYDBPR)
  if (yydebug) {
    printf("yydebug[%d]: after reduction, result is ", yytrial);
    YYDBPR(yyps->val);
    printf("\n");
  }
#endif

  // Perform user-defined position reduction
#ifdef YYREDUCEPOSNFUNC
  if (!yytrial) {
    YYCALLREDUCEPOSN(YYREDUCEPOSNFUNCARG);
  }
#endif

  yyps->ssp -= yym;
  yystate = *(yyps->ssp);
  yyps->vsp -= yym;
  yyps->psp -= yym;

  yym = yylhs[yyn];
  if (yystate == 0 && yym == 0) {
#if YYDEBUG
    if (yydebug) {
      printf("yydebug[%d,%p]: after reduction, shifting from state 0 to state %d\n",
	     int(yydepth), yytrial, YYFINAL);
    }
#endif
    yystate = YYFINAL;
    *++(yyps->ssp) = YYFINAL;
    *++(yyps->vsp) = yyps->val;
    yyretlval = yyps->val;	// return value of root non-terminal to yylval
    *++(yyps->psp) = yyps->pos;
    yyretposn = yyps->pos;	// return value of root position to yyposn
    if (yychar < 0) {
      if ((yychar = yylex1()) < 0) {
        yychar = 0;
      }
#if YYDEBUG
      if (yydebug) {
        yys = 0;
        if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
        if (!yys) yys = "illegal-symbol";
        printf("yydebug[%d,%p]: state %d, reading %d (%s)\n",
	       int(yydepth), yytrial, YYFINAL, yychar, yys);
      }
#endif
    }
    if (yychar == 0) goto yyaccept;
    goto yyloop;
  }

  if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
      yyn <= YYTABLESIZE && yycheck[yyn] == yystate) {
    yystate = yytable[yyn];
  } else {
    yystate = yydgoto[yym];
  }
#if YYDEBUG
  if (yydebug)
    printf("yydebug[%d,%p]: after reduction, shifting from state %d to state %d\n",
           int(yydepth), yytrial, *(yyps->ssp), yystate);
#endif
  if (yyps->ssp >= yyps->ss + yyps->stacksize - 1) {
    yyMoreStack(yyps);
  }
  *++(yyps->ssp) = yystate;
  *++(yyps->vsp) = yyps->val;
  *++(yyps->psp) = yyps->pos;
  goto yyloop;


  //
  // Reduction declares that this path is valid.
  // Set yypath and do a full parse
  //
yyvalid:
  if (yypath) {
    goto yyabort;
  }
  while (yyps->save) {
    yyparsestate *save = yyps->save;
    yyps->save = save->save;
    save->save = yypath;
    yypath = save;
  }
#if YYDEBUG
  if (yydebug)
    printf("yydebug[%d,%p]: CONFLICT trial successful, backtracking to state %d, %d tokens\n",
           int(yydepth), yytrial, yypath->state, int(yylvp - yylvals - yypath->lexeme));
#endif
  if (yyerrctx) {
    yyFreeState(yyerrctx);
    yyerrctx = NULL;
  }
  yychar = -1;
  yyps->ssp = yyps->ss + (yypath->ssp - yypath->ss);
  yyps->vsp = yyps->vs + (yypath->vsp - yypath->vs);
  yyps->psp = yyps->ps + (yypath->psp - yypath->ps);
  memcpy (yyps->ss, yypath->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
  yySCopy(yyps->vs, yypath->vs,  yyps->vsp - yyps->vs + 1);
  yyPCopy(yyps->ps, yypath->ps,  yyps->psp - yyps->ps + 1);
  yylvp = yylvals + yypath->lexeme;
  yylpp = yylpsns + yypath->lexeme;
  yylexp = yylexemes + yypath->lexeme;
  yystate = yypath->state;
  goto yyloop;


yyabort:
  if (yyerrctx) {
    yyFreeState(yyerrctx);
    yyerrctx = NULL;
  }

  YYSTYPE *pv;
  for(pv = yyps->vs; pv < yyps->vsp; pv++) {
    YYDELETEVAL(*pv, 2);
  }

  YYPOSN *pp;
  for(pp = yyps->ps; pp < yyps->psp; pp++) {
    YYDELETEPOSN(*pp, 2);
  }

  while (yyps) {
    yyparsestate *save = yyps;
    yyps = save->save;
    yyFreeState(save);
  }
  while (yypath) {
    yyparsestate *save = yypath;
    yypath = save->save;
    yyFreeState(save);
  }
  return (1);


yyaccept:
  if (yyps->save) goto yyvalid;
  if (yyerrctx) {
    yyFreeState(yyerrctx);
    yyerrctx = NULL;
  }
  while (yyps) {
    yyparsestate *save = yyps;
    yyps = save->save;
    yyFreeState(save);
  }
  while (yypath) {
    yyparsestate *save = yypath;
    yypath = save->save;
    yyFreeState(save);
  }
  return (0);
}


int Parser::yylex1()
{
  if (yylvp < yylve) {
    yylval = *yylvp++;
    yyposn = *yylpp++;
    return *yylexp++;
  } else {
    if (yyps->save) {
      if (yylvp == yylvlim) {
	yyexpand();
      }
      *yylexp = yylex();
      *yylvp++ = yylval;
      yylve++;
      *yylpp++ = yyposn;
      yylpe++;
      return *yylexp++;
    } else {
      return yylex();
    }
  }
}


int Parser::yyexpand()
{
  int p = yylvp - yylvals;
  int s = yylvlim - yylvals;
  s += YYSTACKGROWTH;
  { Yshort  *tl = yylexemes;
    YYSTYPE *tv = yylvals;
    YYPOSN  *tp = yylpsns;
    yylvals = FB_NEW_POOL(getPool()) YYSTYPE[s];
    yylpsns = FB_NEW_POOL(getPool()) YYPOSN[s];
    yylexemes = FB_NEW_POOL(getPool()) Yshort[s];
    memcpy(yylexemes, tl, (s - YYSTACKGROWTH) * sizeof(Yshort));
    yySCopy(yylvals, tv, s - YYSTACKGROWTH);
    yyPCopy(yylpsns, tp, s - YYSTACKGROWTH);
    delete[] tl;
    delete[] tv;
    delete[] tp;
  }
  yylvp = yylve = yylvals + p;
  yylvlim = yylvals + s;
  yylpp = yylpe = yylpsns + p;
  yylplim = yylpsns + s;
  yylexp = yylexemes + p;
  return 0;
}

